#ifndef NESTARRAY_H
#define NESTARRAY_H

#ifndef FILEREADER_H
  #include "filereader.h"
#endif
#ifndef CHRSTRING_H
  #include "chrstring.h"
#endif
#ifndef FULLDATE_H
  #include "fulldate.h"
#endif

#define NESTARRAY_DEBUG                 1
#define NESTARRAY_DEBUG1                1
#define NESTARRAY_DEBUG2                0

#define SizeOfVector(dim, index, lenv)      \
   ((index == dim - 1) ?                    \
        (lenv[index] + 1):                  \
        ((index < dim - 1) ?                \
             (index + lenv[index] + 1):0))

#define CountOfVector(dim, index, off, vectlv1, cntlv1, cntlv2)     \
   ((index == dim - 1) ?                                            \
        (vectlv1[0]):                                               \
        ((index < dim - 1) ?                                        \
             ((off == index-1) ? cntlv2[1][0]:cntlv1[1]):0))

#define IncCountOfVector(dim, index, off, vectlv1, cntlv1, cntlv2)  \
   ((index == dim - 1) ?                                            \
        (++vectlv1[0]):                                             \
        ((index < dim - 1) ?                                        \
             ((off == index-1) ? ++cntlv2[1][0]:++cntlv1[1]):0))

#define DecCountOfVector(dim, index, off, vectlv1, cntlv1, cntlv2)  \
   ((index == dim - 1) ?                                            \
        (--vectlv1[0]):                                             \
        ((index < dim - 1) ?                                        \
             ((off == index-1) ? --cntlv2[1][0]:--cntlv1[1]):0))

#define StatusOfVector(dim, index, off, pos, vectlv1, cntlv1)       \
   ((index == dim - 1) ?                                            \
        (vectlv1[0] == 0):                                          \
        ((index < dim - 1) ?                                        \
             (cntlv1[pos]):0))

#define SetStatusOfVector(dim, index, off, pos, stat, cntlv1)       \
   ((index == dim - 1) ?                                            \
        0:((index < dim - 1) ?                                      \
             (cntlv1[pos]=stat):0))

/****************************************************************************/
// Multidimensional matrix helper functions
//
static void RemoveArray(void* Array_, int Dim_, int Off_, int MatrixIndex_, int* LenVect_);
static void* InitMatrix(void* Array_, int Dim_, int Off_, int Index_, int* LenVect_);
static int* SetParentReference(int* Vector_, int Dim_,
                               int ParentIndex_, int MatrixIndex_,
                               int* LenVect_, int* PosVect_, bool& Error_);
static int* GetParentReference(int* ParentRefs_, int Dim_, int Off_, int MatrixIndex_,
                               int* LenVect_, int* PosVect_, void* Array_,
                               bool& Error_);
static void* SetRefArrays(void* Array_, int Dim_, int Off_, int Index_,
                          int* LenVect_, int* PosVect_, bool& Error_);
static void* SetReferences(int* ParentRefs_, int Dim_, int Off_, int MatrixIndex_,
                               int* LenVect_, int* PosVect_, void* Array_,
                               bool& Error_);
static void* RemoveReferences(void* Array_, void* Container_, int Dim_, int Off_,
                              int MatrixIndex_, int* LenVect_, int* PosVect_,
                              void* Replacement_, bool DeAlloc_, int* Shared_,
                              int* Size_, int* Max_, bool& Error_);
static int DeleteVectorHelper(void* Array_, int Dim_, int Index_, int* LenVect_,
                              int* PosVect_, void* Replacement_, bool Replace_,                              
                              bool DeAlloc_, bool& Error_, bool CheckDealloc_,
                              int* Shared_, int* Size_, int* Max_);

/****************************************************************************/
// Main multidimensional matrix utility functions
//
static int* LinearToCoords(int Pos_, int* Coords_, int CoordsLen_,
                           int* OffSets_, int OffSetsLen_);
static int CoordsToLinear(int* Coords_, int CoordsLen_,
                          int* OffSets_, int OffSetsLen_);
static int* CalculateOffsets(int* ArraySizes_, int ArrayLen_, int& OffSetsLen_);

static void* ParentReference(void* Array_, int Dim_, int Index_,
                             int* LenVect_, int* PosVect_, int* ParentRefs_,
                             bool& Error_);
static int RealVectorCount(void* Array_, int Dim_, int Index_,
                           int* LenVect_, int* PosVect_, bool& Error_);
static int RealVectorMax(void* Array_, int Dim_, int Index_,
                         int* LenVect_, int* PosVect_, bool& Error_);
                             
static int DeleteVector(void* Array_, int Dim_, int Index_, int* LenVect_,
                        int* PosVect_, bool& Error_, int* Shared_=NULL,
                        int* Size_=NULL, int* Max_=NULL);
static int IsDeallocated(void* Array_, int Dim_, int Index_, int* LenVect_,
                         int* PosVect_, bool& Error_, int* Shared_=NULL,
                         int* Size_=NULL, int* Max_=NULL);
static int ReplaceVector(void* Array_, int Dim_, int Index_, int* LenVect_,
                         int* PosVect_, void* Replacement_, bool Replace_,
                         bool DeAlloc_, bool& Error_, int* Shared_=NULL,
                         int* Size_=NULL, int* Max_=NULL);
                         
static void* InitArrayData(void* Array_, int Dim_, int Index_,
                           int* LenVect_, int* PosVect_, bool& Error_);
static void* DestroyArrayData(void* Array_, int Dim_, int* LenVect_);

static int ReadFromArray(void* Array_, int Dim_, int Index_,
                         int* LenVect_, int* PosVect_, bool& Error_);
static void* WriteToArray(void* Array_, int Dim_, int Index_, int* LenVect_,
                          int* PosVect_, int Data_, bool& Error_);
                            
/****************************************************************************/
class IntVector
{
  public:
    int _Deallocated;
    int _ShallowCopy;

    // _Array[0] holds vector length
    // Corresponds to 1d constituent element vector within data matrix if
    // used as base class in NestedDataStorage class.
    int* _Array;

    IntVector();
    IntVector(int Length_);

    ~IntVector();
};

/****************************************************************************/
class NestedArray2
{
  public:
    int _Length[2];
    int _Indexes[2];
    int** _Array;

    NestedArray2(int* Length_);    
};

/****************************************************************************/
class NestedArray3
{
  protected:
    enum { SIZE = 3 };

    int*** _Array;
    int _MatrixIndex;
    int* _LenVect;
    int* _PosVect;
    bool _Error;

    int _Shared;
    int _Size;

    int* _OffSets;
    int _OffSetsLen;  

  public:
    NestedArray3(int* Length_);
    ~NestedArray3();

    NestedArray3& SetCoordinates(int* Coords_);
    NestedArray3& SetLengths(int* Coords_);

    int DeleteVector(int Index_);
    int IsDeallocated(int Index_);
    int ReplaceVector(int Index_, void* Replacement_, bool Replace_, bool DeAlloc_);

    int RealVectorSize(int Index_);
    int* ParentReference(int Index_, int* ParentRefs_);

    int* LinearToCoords(int Pos_, int* OffSets_, int OffSetsLen_);
    int CoordsToLinear(int* OffSets_, int OffSetsLen_);
    int* CalculateOffsets();

    void* InitArrayData(int Index_);

    int ReadFromArray(int Index_);
    void* WriteToArray(int Index_, int Data_);
};

/****************************************************************************/
class NestedArray4
{
  public:
    int _Length[4];
    int _Indexes[4];
    int**** _Array;

    NestedArray4(int* Length_);
};

/****************************************************************************/
class NestedArray5
{
  public:
    int _Length[5];
    int _Indexes[5];
    int***** _Array;

    NestedArray5(int* Length_);
};

/****************************************************************************/
class NestedArray6
{
  public:
    int _Length[5];
    int _Indexes[5];
    int***** _Array;

    NestedArray6(int* Length_);
};

/****************************************************************************/
struct NestedArrayData
{
  int _Index;     // Linear index calculated from CoordsToLinear method
  void* _Data;  
};

/****************************************************************************/
class NestedDataStorage : public IntVector
{
  private:
    enum
    {
      VECTOR1DARRAY,
      NESTED2DARRAY,
      NESTED3DARRAY,
      NESTED4DARRAY,
      NESTED5DARRAY,
      NESTED6DARRAY
    };

    int _Type;
    union
    {
      IntVector* _N1Arr;
      NestedArray2* _N2Arr;
      NestedArray3* _N3Arr;
      NestedArray4* _N4Arr;
      NestedArray5* _N5Arr;
      NestedArray6* _N6Arr;
    };

    // Offset format:
    //     Offset[0...even] = length of dimensional array
    //     Offset[1...odd]  = dimensional array offset size in number of
    //                        1d vector base elements
    //
    int* _OffSets;     // Progressive linear position offsets
    int _OffSetsLen;   // Position offsets vector length
    
    int _VectorIndex;  // Index to access active leaf element vector
    int _NestedIndex;  // Hashed index into nested data array given the
                       // linear position index
    
    NestedArrayData** _DataArr;

  public:    
    NestedDataStorage();    
};

/****************************************************************************/
#endif
