#ifndef STRFILTER_CPP
#define STRFILTER_CPP

#ifndef STRFILTER_H
  #include "strfilter.h"
#endif

/****************************************************************************/
StringFilter::StringFilter():
_Frp(NULL),
_GetchFromTemp(FALSE),
_Buffer(char(0), BUFSIZE),
_MainFileName(NULL),
_EndReadMark(NULL),
_NestingLevel(0),
_Next(NULL),

_SavedIndex(0),
_SavedLen(0),
_SavedStream(NULL),
_LineFound(false),
_EndLineDelim(' '),
_EndPt(0),
_StartPt(0),
_StartTagEndPt(0),
_StartTagStartPt(0),
_EndTagEndPt(0),
_EndTagStartPt(0),
_IsHtmlTag(false),
_TempRec((char*)RawAllocateWith(MEMMATRIX, sizeof(char) * FLDSIZE)),
_Tindex(0),
_Tlength(0),
_StartTag(NULL),
_EndTag(NULL),
_TagEsc(0),
_InTag(false),
_TagEscCnt(0),
_NonEscCnt(0),
_BreakonStartTag(false),
_BreakonEndTag(false),
_HaveTag(false),
_HaveTagEsc(false)
{
  _Buffer.SetCountNulls(TRUE);
  _Buffer.SetAutoAppend(TRUE);
  _Buffer.StrFill(0, BUFSIZE);
}

/****************************************************************************/
StringFilter::StringFilter(const StringFilter& Stok_, StringFilter* Next_):
_Frp(NULL),
_GetchFromTemp(Stok_._GetchFromTemp),
_Buffer(char(0), BUFSIZE),
_MainFileName(new_char_string(Stok_._MainFileName)),
_EndReadMark(new_char_string(Stok_._EndReadMark)),
_NestingLevel(Stok_._NestingLevel),
_Next(Next_),

_SavedIndex(Stok_._SavedIndex),
_SavedLen(0),
_SavedStream(Stok_._SavedStream),
_LineFound(Stok_._LineFound),
_EndLineDelim(Stok_._EndLineDelim),
_EndPt(Stok_._EndPt),
_StartPt(Stok_._StartPt),
_StartTagEndPt(Stok_._StartTagEndPt),
_StartTagStartPt(Stok_._StartTagStartPt),
_EndTagEndPt(Stok_._EndTagEndPt),
_EndTagStartPt(Stok_._EndTagStartPt),
_IsHtmlTag(Stok_._IsHtmlTag),
_TempRec((char*)RawAllocateWith(MEMMATRIX, sizeof(char) * FLDSIZE)),
_Tindex(Stok_._Tindex),
_Tlength(Stok_._Tlength),
_StartTag(new_char_string(Stok_._StartTag)),
_EndTag(new_char_string(Stok_._EndTag)),
_TagEsc(Stok_._TagEsc),
_InTag(Stok_._InTag),
_TagEscCnt(Stok_._TagEscCnt),
_NonEscCnt(Stok_._TagEscCnt),
_BreakonStartTag(Stok_._BreakonStartTag),
_BreakonEndTag(Stok_._BreakonEndTag),
_HaveTag(Stok_._HaveTag),
_HaveTagEsc(Stok_._HaveTagEsc)
{
  if (_MainFileName)
  {
    _Frp = new FileReader();  
    _Frp->Open(_MainFileName, ios::in);
  }

  _Buffer.SetCountNulls(TRUE);
  _Buffer.SetAutoAppend(TRUE);  
  _Buffer = Stok_._Buffer;
  strcpy(_TempRec, Stok_._TempRec);
}

/****************************************************************************/
StringFilter::StringFilter(const char* Fname_,
                           int NestingLevel_, StringFilter* Next_):
_Frp(NULL),
_GetchFromTemp(FALSE),
_Buffer(char(0), BUFSIZE),
_MainFileName(NULL),
_EndReadMark(NULL),
_NestingLevel(NestingLevel_),
_Next(Next_),

_SavedIndex(0),
_SavedLen(0),
_SavedStream(NULL),
_LineFound(false),
_EndLineDelim(' '),
_EndPt(0),
_StartPt(0),
_StartTagEndPt(0),
_StartTagStartPt(0),
_EndTagEndPt(0),
_EndTagStartPt(0),
_IsHtmlTag(false),
_TempRec((char*)RawAllocateWith(MEMMATRIX, sizeof(char) * FLDSIZE)),
_Tindex(0),
_Tlength(0),
_StartTag(NULL),
_EndTag(NULL),
_TagEsc(0),
_InTag(false),
_TagEscCnt(0),
_NonEscCnt(0),
_BreakonStartTag(false),
_BreakonEndTag(false),
_HaveTag(false),
_HaveTagEsc(false)
{
  if (Fname_)
  {
    _Frp = new FileReader();  
    _Frp->Open(Fname_, ios::in);    
    _MainFileName = new_char_string(Fname_);
  }

  _Buffer.SetCountNulls(TRUE);
  _Buffer.SetAutoAppend(TRUE);  
  _Buffer.StrFill(0, BUFSIZE);  
}

/****************************************************************************/
StringFilter::~StringFilter()
{
  ::DeleteArray(_TempRec);
  ::DeleteArray(_MainFileName);
  ::DeleteArray(_StartTag);
  ::DeleteArray(_EndTag);
  ::DeleteArray(_EndReadMark);

  StringFilter* Old_ = Pop();
        
  while (Old_)
  {
    delete Old_;
    Old_ = Pop();
  }
  
  if (_Frp)
  {
    _Frp->Close();
    delete _Frp;
  }

  delete _Next;  
}

/****************************************************************************/
void StringFilter::ResetFilter(bool ResetNesting_)
{
  if (ResetNesting_)
  {
    _NestingLevel = 0;
    _Next = NULL;
  }

  _EndPt = 0;
  _StartPt = 0;
  
  _StartTagEndPt = 0;
  _StartTagStartPt = 0;
  
  _EndTagEndPt = 0;
  _EndTagStartPt = 0;
  
  _SavedIndex = 0;
  _SavedStream = NULL;

  _Buffer.StrFill(0, BUFSIZE);
}

/****************************************************************************/
StringFilter& StringFilter::operator = (const StringFilter& Stok_)
{
  if (this != &Stok_)
  {
    if (_MainFileName != Stok_._MainFileName)
    {
      ::DeleteArray(_MainFileName);
      _MainFileName = NULL;
    }

    if (_StartTag != Stok_._StartTag)
    {
      ::DeleteArray(_StartTag);
      _StartTag = NULL;
    }

    if (_EndTag != Stok_._EndTag)
    {
      ::DeleteArray(_EndTag);
      _EndTag = NULL;
    }

    if (_TempRec != Stok_._TempRec)
    {
      ::DeleteArray(_TempRec);
      _TempRec = NULL;
    }

    if (_Buffer.c_str() == Stok_._Buffer.c_str())
      _Buffer.StrFill(0, BUFSIZE);

    const char* Str_ = Stok_._StartTag;
    _StartTag = Str_ ? new_char_string(Str_):NULL;

    Str_ = Stok_._MainFileName;
    _MainFileName = Str_ ? new_char_string(Str_):NULL;    

    Str_ = Stok_._EndTag;
    _EndTag = Str_ ? new_char_string(Str_):NULL;

    Str_ = Stok_._TempRec;
    _TempRec = Str_ ? new_char_string(Str_):NULL;

    _NestingLevel = Stok_._NestingLevel;
    _TagEsc = Stok_._TagEsc;
    _InTag = Stok_._InTag;
    _TagEscCnt = Stok_._TagEscCnt;
    _NonEscCnt = Stok_._NonEscCnt;
    _BreakonStartTag = Stok_._BreakonStartTag;
    _BreakonEndTag = Stok_._BreakonEndTag;
    _HaveTag = Stok_._HaveTag;
    _HaveTagEsc = Stok_._HaveTagEsc;
    
    _EndPt = Stok_._EndPt;
    _StartPt = Stok_._StartPt;
    _StartTagEndPt = Stok_._StartTagEndPt;
    _StartTagStartPt = Stok_._StartTagStartPt;
    _EndTagEndPt = Stok_._EndTagEndPt;
    _EndTagStartPt = Stok_._EndTagStartPt;
    _IsHtmlTag = Stok_._IsHtmlTag;
    
    _Buffer = Stok_._Buffer;
    _Frp->Open(_MainFileName, ios::in);
  }

  return *this;
}

/****************************************************************************/
StringFilter& StringFilter::SetNestingLevel(int Level_)
{
  ClearNesting();
  _NestingLevel = Level_;
  return *this;
}

/****************************************************************************/
StringFilter& StringFilter::SetParseHtmlTag(bool IsTag_)
{
  _IsHtmlTag = IsTag_;
  return *this;
}

/****************************************************************************/
StringFilter& StringFilter::SetStringTags(const char* Start_, const char* End_)
{
  if (_StartTag != Start_)
    ::DeleteArray(_StartTag);

  if (_EndTag != End_)
    ::DeleteArray(_EndTag);

  _StartTag = Start_ ? new_char_string(Start_):NULL;
  _EndTag = End_ ? new_char_string(End_):NULL;
  
  return *this;
}

/****************************************************************************/
StringFilter& StringFilter::SetTagsEscChar(char Ch_)
{
  // The string tag escape char cannot be included within
  // the set of the start tag characters specified
  if (_TagEsc && InCharSet(_TagEsc, _StartTag))
    return *this;

  // The string tag escape char cannot be included within
  // the set of the end tag characters specified
  if (_TagEsc && InCharSet(_TagEsc, _EndTag))
    return *this;    

  _TagEsc = Ch_;
  return *this;
}

/****************************************************************************/
StringFilter& StringFilter::SetTempString(const char* Str_, Boolean Erase_, long len)
{
  if (_TempRec == Str_)
    return *this;

  if (Str_)
  {
    if (Erase_ || !_TempRec || (len && len > FLDSIZE - 2))
    {    
      ::DeleteArray(_TempRec);
      if (!len || len <= FLDSIZE - 2)
        len = FLDSIZE;
      else
        len += 2;

      _TempRec = (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * len);
    }
    else
      len = FLDSIZE;

    strncpy(_TempRec, Str_, len-1);
    _TempRec[len-1] = 0;
    RemovePadding(_TempRec, " \t\n");

    _Tindex = 0;
    _Tlength = ::SafeStrLen(_TempRec);
  }
  else
  {
    ::DeleteArray(_TempRec);
    _TempRec = NULL;
    _Tindex = _Tlength = 0;
  }

  return *this;
}

/****************************************************************************/
int StringFilter::GetchTempStr()
{
  if (_Tindex < _Tlength)
  {
    int ret_ = _TempRec[_Tindex++];
    return ret_;
  }

  return 0;
}

/****************************************************************************/
int StringFilter::PeekchTempStr()
{
  if (_Tindex < _Tlength)
  {
    int ret_ = _TempRec[_Tindex];
    return ret_;
  }

  return 0;
}

/****************************************************************************/
void StringFilter::PutBackTempStr()
{
  if (_Tindex)
    _Tindex--;
}

/****************************************************************************/
int StringFilter::ExtractUntil(char* InputStr_, short InputLen_,
                              const char* CharSet_, Boolean InSet_,
                              Boolean PutBack_)
{
  if (InputStr_ && CharSet_)
  {
    // The tag escape char cannot be included within
    // the set of the tag characters specified
    if (_TagEsc && InCharSet(_TagEsc, CharSet_))
      return false;
  
    short x, len;
    bool StopCond_ = false;
    bool Found_ = false;
    len = InputLen_ - 1;
    
    _TagEscCnt =
    _NonEscCnt = 0;    
    
    for (x = 0; x < len; x++)
    {
      InputStr_[x] = GetchTempStr();
      if (!InputStr_[x])
        break;

      if (_TagEsc)
        if (InputStr_[x] == _TagEsc)
        {
          if (_NonEscCnt)
            _TagEscCnt = 1;
          else
            ++_TagEscCnt;

          _NonEscCnt = 0;
        }
        else
        {
          if (_NonEscCnt >= 1)
            _NonEscCnt =
            _TagEscCnt = 0;
          else
            ++_NonEscCnt;
        }

      Found_ = (InSet_ && InCharSet(InputStr_[x], CharSet_)) ||
               (!InSet_ && !InCharSet(InputStr_[x], CharSet_));

      if (InSet_)
      {
        _HaveTagEsc = _HaveTag = false;
        StopCond_ = (_TagEsc && x > 0 && _TagEscCnt % 2) ?
           !(_HaveTagEsc=(InputStr_[x-1] == _TagEsc)):true;
        _HaveTag = Found_;
        StopCond_ = StopCond_ && _HaveTag;
      }
      else
        StopCond_ = Found_;

      if (StopCond_)
      {
        if (PutBack_)
          PutBackTempStr();
        break;
      }
    }

    InputStr_[x] = 0;
    return strlen(InputStr_);
  }

  return 0;
}

/****************************************************************************/
int StringFilter::ExtractUntil(char* InputStr_, short InputLen_,
                              int(*iscsetfn)(int), Boolean InSet_,
                              Boolean PutBack_)
{
  if (InputStr_ && iscsetfn)
  {
    // The tag escape char cannot be included within
    // the set of the tag characters specified
    if (_TagEsc && (*iscsetfn)(_TagEsc))
      return false;
  
    short x, len;
    bool StopCond_ = false;
    bool Found_ = false;    
    len = InputLen_ - 1;

    _TagEscCnt =
    _NonEscCnt = 0;    
    
    for (x = 0; x < len; x++)
    {
      InputStr_[x] = GetchTempStr();
      if (!InputStr_[x])
        break;

      if (_TagEsc)
        if (InputStr_[x] == _TagEsc)
        {
          if (_NonEscCnt)
            _TagEscCnt = 1;
          else
            ++_TagEscCnt;

          _NonEscCnt = 0;
        }
        else
        {
          if (_NonEscCnt >= 1)
            _NonEscCnt =
            _TagEscCnt = 0;
          else
            ++_NonEscCnt;
        }

      Found_ = (InSet_ && (*iscsetfn)(InputStr_[x])) ||
               (!InSet_ && !(*iscsetfn)(InputStr_[x]));

      if (InSet_)
      {
        _HaveTagEsc = _HaveTag = false;
        StopCond_ = (_TagEsc && x > 0 && _TagEscCnt % 2) ?
           !(_HaveTagEsc=(InputStr_[x-1] == _TagEsc)):true;
        _HaveTag = Found_;
        StopCond_ = StopCond_ && _HaveTag;
      }
      else
        StopCond_ = Found_;
      
      if (StopCond_)
      {
        if (PutBack_)
          PutBackTempStr();
        break;
      }
    }

    InputStr_[x] = 0;
    return strlen(InputStr_);
  }

  return 0;
}

/****************************************************************************/
int StringFilter::GetTxtLine(const char* Delim_, FileReader* FileRp_, char* StrBuf_, int StrBufSz_)
{
  char* Buffer_ = StrBuf_ ? StrBuf_:_Buffer.c_str();
  int BufSz_ = StrBuf_ ? StrBufSz_:(_Buffer.TotalSize() - 1);
  FileReader* Frp_ = FileRp_ ? FileRp_:_Frp;
  
  if (!Frp_)
    return 0;

  if (Frp_->EndOfFile())
    return 0;

  int ret_ = Frp_->ReadLine(Buffer_, BufSz_, Delim_).GetCount() != 0;
  long x, len;

  // we don't care about overruning delimiters as long as it fits
  // into the buffer we can ignore the rest of the input.
  if (!Frp_->EndOfFile())
  {
    Frp_->ClearState();
    Frp_->GetInStream().clear(ios::goodbit);
  }
    
  ret_ = Frp_->GetCount();

  if (!StrBuf_)
  {
    ::RemoveTrailing(Buffer_, " \t");
    len = strlen(Buffer_);
  }
  else
  {
    _Buffer = Buffer_;
    _Buffer.RemoveTrailing(" \t");
    len = _Buffer.strlen();
  }

  // skipping trailing spaces
  for (x = len-1; x >= 0; x--)
    if (!isspace(Buffer_[x]))
      break;

  if (!StrBuf_)
  {
    Fallible<Subscript> Index_ = _Buffer.findprevious(x, _EndLineDelim);
    
    if (Index_.valid())
    {
      _EndLine = _Buffer.subString(Index_ + 1);
      _EndLine.RemoveTrailing(" \t");
    }

    if (_LineToFind.strlen())
      _LineFound = _Buffer.StrniComp(_LineToFind, _LineToFind.strlen()) == 0;
  }

  return ret_;
}

/****************************************************************************/
int StringFilter::ReadUntilFound(const char* Mark_, const char* EndLine_,
                                 const char* Delim_, Boolean Rws_, bool Csense_)
{
  if (_Frp->EndOfFile() || !Mark_)
    return 0;

  int Found_ = 0;
  long len;
  long pos;
  long x, y, slen;
  const char* Ptr_ = NULL;
  const char* Ws_ = " \t\n\r\f\v";

  len = strlen(Mark_);
  ::DeleteArray(_EndReadMark);
  _EndReadMark = (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * (len + 1));
  strcpy(_EndReadMark, Mark_);
  _EndReadMark = ::AllowLenOnlyOfChar(_EndReadMark, 1, " ");
  
  len = SafeStrLen(EndLine_);
  
  while (!_Frp->EndOfFile())
  {
    GetTxtLine(Delim_);
    _Buffer.AllowLenOnlyOfChar(1, " ");

    Ptr_ = strstr(_Buffer.c_str(), _EndReadMark);

    if (!Found_ && Ptr_)
    {
      _StartPt = Ptr_ - _Buffer.c_str();
      _EndPt = _StartPt + strlen(_EndReadMark);

      if (_IsHtmlTag)
      {
        if (_Buffer[_EndPt-1] != '>')
          while (_Buffer[_EndPt] && _Buffer[_EndPt++] != '>');

        _StartTagStartPt = _StartPt;
        _StartTagEndPt = _EndPt;
      }
      
      Found_ = 1;
    }

    if (Found_)
      if (!EndLine_ || (_EndLine.strlen() && strncmp(EndLine_, _EndLine.c_str(), len) == 0))
      {      
        ++Found_;
        
        if (EndLine_)
        {
          Ptr_ = _Buffer.StrStr(_EndLine.c_str());
          _EndPt = pos = Ptr_ - _Buffer.c_str();
          _EndPt += _EndLine.strlen();

          if (_IsHtmlTag)
          {
            _EndTagStartPt = pos;
            _EndTagEndPt = _EndPt;

            if (_Buffer[_EndTagStartPt] != '<')
              while (_EndTagStartPt && _Buffer[--_EndTagStartPt] != '<');
          }
        }
        
        break;
      }
  }

  if (Found_ > 1 && Rws_)
  {
    size_t Skipped_ = 0;
    _Buffer.RemoveTrailing(Ws_);
    for (y = 0; _Buffer[y] && InCharSet(_Buffer[y], Ws_); y++);
    _StartPt -= y;
    _EndPt -= y;

    if (_IsHtmlTag)
    {
      _StartTagStartPt -= y;
      _StartTagEndPt -= y;

      if (EndLine_)
      {
        _EndTagStartPt -= y;
        _EndTagEndPt -= y;
      }
      else
      {
        _EndTagStartPt = _StartTagStartPt;
        _EndTagEndPt = _StartTagEndPt;
      }
    }
  
    for (x = 0; (_Buffer[x] = _Buffer[y]); x++, y++);

    // allow only 1 intervening space for text enclosed within HTML tags
    if (_IsHtmlTag)
    {
      char* sSaved_ = NULL;
      char* eSaved_ = NULL;
      bool Match_;

      if (Mark_ && EndLine_)
      {
        slen = _StartTagEndPt - _StartTagStartPt;
    
        if (slen)
        {
          sSaved_ = (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * (slen+1));
          strncpy(sSaved_, _Buffer.c_str() + _StartTagStartPt, slen);
          sSaved_[slen] = 0;
        }
      }

      if (Mark_)
      {
        slen = _EndTagEndPt - _EndTagStartPt;

        if (slen)
        {
          eSaved_ = (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * (slen+1));
          strncpy(eSaved_, _Buffer.c_str() + _EndTagStartPt, slen);
          eSaved_[slen] = 0;
        }
      }

      _Buffer.AllowLenOnlyOfChar(1, Ws_, &Skipped_);
  
      if (Mark_ && eSaved_)
      {
        _StartPt = _EndTagStartPt;
        _EndPt = _EndTagEndPt;

        _EndPt -= Skipped_;
        _EndTagEndPt -= Skipped_;

        Match_ = Csense_ ? (strncmp(_Buffer.c_str()+_StartPt-Skipped_, eSaved_, slen) == 0):
                           (StrniComp(_Buffer.c_str()+_StartPt-Skipped_, eSaved_, slen) == 0);

        if (Match_)
        {
          _StartPt -= Skipped_;
          Skipped_ = -Skipped_;
        }
        else
        {
          slen = -Skipped_;
          Skipped_ = 0;
          ::AllowLenOnlyOfChar(eSaved_, 1, Ws_, &Skipped_);
          slen += Skipped_ * 2;
          _StartPt += slen;
          Skipped_ = slen;
        }

        _EndTagStartPt += Skipped_;

        if (_IsHtmlTag && _Buffer[_StartPt] != '<')
        {
          _EndTagStartPt = _EndTagEndPt - 1;
          while (_EndTagStartPt && _Buffer[--_EndTagStartPt] != '<');
          _StartPt = _EndTagStartPt;
        }
      }

      /// with start tags: start
      if (Mark_ && EndLine_ && sSaved_)
      {      
        _StartPt = _StartTagStartPt;
        _EndPt = _StartTagEndPt;

        slen = _StartTagEndPt - _StartTagStartPt;
        Match_ = Csense_ ? (strncmp(_Buffer.c_str()+_StartPt, sSaved_, slen) == 0):
                           (StrniComp(_Buffer.c_str()+_StartPt, sSaved_, slen) == 0);

        if (!Match_)
        {
          Skipped_ = 0;
          ::AllowLenOnlyOfChar(sSaved_, 1, Ws_, &Skipped_);
          _EndPt -= Skipped_;
          _StartTagEndPt -= Skipped_;
        }

        if (_IsHtmlTag && _Buffer[_EndPt-1] != '>')
        {
          _StartTagEndPt = _StartTagStartPt + 1;
          while (_Buffer[_StartTagEndPt] && _Buffer[_StartTagEndPt++] != '>');
          _EndPt = _StartTagEndPt;
        }

        _StartPt = _StartTagStartPt;
        _EndPt = EndLine_ ? _EndTagEndPt:_StartTagEndPt;
      }
      /// with start tags: end

      if (Mark_)
      {
        ::DeleteArray(eSaved_);        
        if (EndLine_)
          ::DeleteArray(sSaved_);
      }
    }
  }

  return (Found_ > 1);
}

/****************************************************************************/
int StringFilter::TestRead(Boolean Find_, const char* Mark_, int Spec_,
                          int Len_, Boolean SkipNl_, Boolean SkipWs_)
{
  if (_Frp->EndOfFile() || !(Mark_ || Spec_))
    return 0;

  istream& Is_ = _Frp->GetInStream();

  int Found_;
  short y, z;
  short x = 0;
  short len = Mark_ ? strlen(Mark_):Len_;
  const char* Nl_ = "\n\r\f\v";
  const char* Ws_ = " \t\n\r\f\v";
  const char* Wsxnl_ = " \t";
  const char* Cset_;
  const char* nlptr_;
  long nlpos_;
  long fndpos_;
  long fndpos2_, fndpos3_;

  if (Len_ < len)
    Len_ = len;

  if (SkipNl_ || SkipWs_)
  {
    Cset_ = (SkipWs_ && SkipNl_) ? Ws_:
            (SkipNl_) ? Nl_:Wsxnl_;
    
    while (!_Frp->EndOfFile())
      if (_GetchFromTemp)
      {
        if (!InCharSet(PeekchTempStr(), Cset_))
          break;
        else
          GetchTempStr();
      }
      else
      {
        if (!InCharSet(Is_.peek(), Cset_))
          break;
        else
          Is_.get();
      }
  }

  if (_Frp->EndOfFile())
    return 0;

  if (Mark_ && len)
  {
    y = _GetchFromTemp ?
            toupper(PeekchTempStr()):
            toupper(Is_.peek());
    Found_ = toupper(*Mark_) == y;
    
    if (Find_ && !Found_)
      return 0;
  }
    
  for (x = 0; x < Len_; x++)
  {
    if (_GetchFromTemp)
      _Buffer[x] = GetchTempStr();
    else
      Is_.get(_Buffer[x]);
    
    if (_Frp->EndOfFile())
    {
      for (;x >= 0; x--)
        if (_GetchFromTemp)
          PutBackTempStr();
        else
          Is_.putback(_Buffer[x]);
      
      return 0;
    }
  }

  _Buffer[x] = 0;
  Found_ = 0;
  nlptr_ = (Cset_ == Wsxnl_) ? _Buffer.StrChr('\n'):NULL;
  nlpos_ = nlptr_ ? (nlptr_ - _Buffer.c_str()):CHARNOTFOUND;
  
  if (Mark_)      
    Found_ = StrniComp(_Buffer.c_str(), Mark_, len) == 0;
  else
  {
    if (Spec_ == DATE)
      Found_ = IsDateStr(_Buffer.c_str());
    else if (Spec_ == TENS_DIGIT)
    {
      len = _Buffer.strlen();
      char* NmeSet_ = MakeCharSet(isdigit);
      fndpos_ = StrHasChar(_Buffer.c_str(), NmeSet_, 1, 1);
      Found_ = fndpos_ != CHARNOTFOUND &&
               (nlptr_ ? (fndpos_ < nlpos_):TRUE);

      if (Found_)
        for (y = fndpos_; y < len && !isspace(_Buffer[y]); y++)
          if (!isdigit(_Buffer[y]) && !isspace(_Buffer[y]))
          {
            Found_ = FALSE;
            break;
          }

      ::DeleteArray(NmeSet_);    
    }
    else if (Spec_ == INT_FIELD)
    {
      len = _Buffer.strlen();
      char* NmeSet_ = MakeCharSet(isdigit);
      fndpos_ = StrHasChar(_Buffer.c_str(), NmeSet_, 1, 0);
      Found_ = fndpos_ != CHARNOTFOUND &&
               (nlptr_ ? (fndpos_ < nlpos_):TRUE);

      if (Found_)
        for (y = fndpos_; y < len && !isspace(_Buffer[y]); y++)
          if (!isdigit(_Buffer[y]) && !isspace(_Buffer[y]))
          {
            Found_ = FALSE;
            break;
          }
      
      ::DeleteArray(NmeSet_);        
    }
    else if (Spec_ == FLOAT_FIELD)
    {
      len = _Buffer.strlen();
      char* NmeSet_ = MakeCharSet(isdigit);
      strcat(NmeSet_, ",.");
      
      fndpos_ = StrHasChar(_Buffer.c_str(), NmeSet_, 1, 1);
      Found_ = fndpos_ != CHARNOTFOUND &&
               (nlptr_ ? (fndpos_ < nlpos_):TRUE);

      if (Found_)
        for (y = fndpos_; y < len && !isspace(_Buffer[y]); y++)
          if (!isdigit(_Buffer[y]) && _Buffer[y] != '.' &&
              _Buffer[y] != ',' && !isspace(_Buffer[y]))
          {
            Found_ = FALSE;
            break;
          }
      
      ::DeleteArray(NmeSet_);
    }
  }

  for (--x; x >= 0; x--)
    if (_GetchFromTemp)
      PutBackTempStr();
    else
      Is_.putback(_Buffer[x]);

  if (!_GetchFromTemp && !_Frp->EndOfFile())
  {
    _Frp->ClearState();
    _Frp->GetInStream().clear(ios::goodbit);
  }        

  if (Find_)
    return Found_;

  return (!Found_);
}

/****************************************************************************/
StringFilter* StringFilter::Push()
{
  if (_NestingLevel)
  {
    StringFilter* Old_ = new StringFilter(*this, _Next);    
    --_NestingLevel;
    
    _Next = Old_;
      
    ResetFilter(false);    
    return Old_;
  }

  return NULL;
}

/****************************************************************************/
StringFilter* StringFilter::Pop()
{
  if (_Next)
  {
    StringFilter* Old_ = _Next;
    _Next = Old_->_Next;
    Old_->_Next = NULL;
    
    return Old_;
  }

  return NULL;
}

/****************************************************************************/
void StringFilter::FindTagEscCount(char* BufStr_, long Start_,
                                   long OffSet_, const char* TagStr_)
{
  long z;
  if (!BufStr_)
    BufStr_ = _Buffer.c_str();
  
  for (z = Start_; z < OffSet_; z++)
  {
    if (BufStr_[z] == _TagEsc)
    {
      if (_NonEscCnt)
        _TagEscCnt = 1;
      else
        ++_TagEscCnt;
            
      _NonEscCnt = 0;
    }
    else
    {
      if (_NonEscCnt >= strlen(TagStr_))
        _NonEscCnt = _TagEscCnt = 0;
      else
        ++_NonEscCnt;
    }
  }
}

/****************************************************************************/
bool StringFilter::IsTagFound(const char* Str_, const char* sPtr_,
                             const char* ePtr_, long Index_, int Found_)
{
  bool StopCond_ = false;

  if (sPtr_)
  {
    // The tag escape char cannot be included within
    // the set of the start tag characters specified
    if (_TagEsc && InCharSet(_TagEsc, sPtr_))
      return false;
  
    _HaveTagEsc = _HaveTag = false;
    StopCond_ = (_TagEsc && Index_ > 0 && _TagEscCnt % 2) ?
       !(_HaveTagEsc=(*(Str_-1) == _TagEsc)):true;
    _HaveTag = sPtr_ && Found_;
    StopCond_ = StopCond_ && _HaveTag;
    
    if (StopCond_)
    {
      // break on start tag
      _BreakonStartTag = true;

      // Set within string tag status flag
      _InTag = true;
      return true;
    }
  }
  else if (ePtr_)
  {
    // The tag escape char cannot be included within
    // the set of the end tag characters specified
    if (_TagEsc && InCharSet(_TagEsc, ePtr_))
      return false;
  
    _HaveTagEsc = _HaveTag = false;
    StopCond_ = (_TagEsc && Index_ > 0 && _TagEscCnt % 2) ?
       !(_HaveTagEsc=(*(Str_-1) == _TagEsc)):true;
    _HaveTag = ePtr_ && Found_;
    StopCond_ = StopCond_ && _HaveTag;
    
    if (StopCond_)
    {
      // break on end tag
      _BreakonEndTag = true;

      // Unset within string tag status flag
      _InTag = false;
      return true;
    }
  }

  return false;
}

/****************************************************************************/
int StringFilter::IsStringFound(short y, long& Index_, long Total_, long Added_, long* len,
                                const char* Ptr_, const char* sPtr_, const char* ePtr_,
                                bool Csense_)
{
  int Found_;
  long slen = _Buffer.strlen();
  long plen = ::SafeStrLen(Ptr_);
  char* Test_;
  short z, off;
  long Base_ = Total_ - Added_;

  // NULL search string is not allowed
  if (!Ptr_)
    return 0;

  if (y > 0)
  {
    if (len[1] == len[0] + len[2])
    {
      Index_ = _TagEscCnt = _NonEscCnt = 0;
      Found_ = Csense_ ? (strncmp(_Buffer.c_str()+len[0], Ptr_, len[2]) == 0):
                         (StrniComp(_Buffer.c_str()+len[0], Ptr_, len[2]) == 0);
      
      if (_TagEsc && Found_)
      {
        FindTagEscCount(NULL, 0, len[0], Ptr_);
        Index_ = len[0];
        Found_ = IsTagFound(_Buffer.c_str()+len[0], sPtr_, ePtr_, len[0], Found_);
      }

      if (Found_)
      {
        _TagEscCnt = 0;
        _StartPt = len[0];
        _EndPt = _StartPt + len[2];
        _EndTagStartPt = _StartPt;
        _EndTagEndPt = _EndPt;        
      }
    }
    else
    {
      off = len[1] - (2 * len[2]);
      
      if (off < 0)
        off += len[2];

      if (off < 0)
        off += len[2];        
      
      Found_ = 0;
      Test_ = _Buffer.c_str() + off;
      slen = strlen(Test_);

      if (Base_ < off || Index_ < off)
      {
        Added_ = off - Index_;
        
        if (Added_ > 0 && _TagEsc)
        {
          FindTagEscCount(NULL, Index_, Index_ + Added_, Ptr_);
          Index_ += Added_;
        }
      }
      
      Added_ = slen + off - len[2];
      for (z = 0; !Found_ && slen >= len[2]; z++)
      {
        Found_ = Csense_ ? (strncmp(Test_+z, Ptr_, len[2]) == 0):
                           (StrniComp(Test_+z, Ptr_, len[2]) == 0);      
        
        if (_TagEsc && Found_)
          Found_ = IsTagFound(Test_+z, sPtr_, ePtr_, off+z, Found_);
          
        slen = strlen(Test_+z+1);

        if (Found_)
        {
          _TagEscCnt = 0;        
          _StartPt = off + z;
          _EndPt = _StartPt + len[2];
          _EndTagStartPt = _StartPt;
          _EndTagEndPt = _EndPt;          
        }
        else if (Index_ <= Added_ && _TagEsc)
        {          
          FindTagEscCount(Test_+z, 0, 1, Ptr_);
          ++Index_;
        }
      }
    }
  }
  else
  {  
    if (slen <= len[0])
    {
      if (slen < len[0])
      {
        Found_ = FALSE;
        return Found_;        
      }

      Index_ = _TagEscCnt = _NonEscCnt = 0;
      Found_ = Csense_ ? (strncmp(_Buffer.c_str(), Ptr_, len[0]) == 0):
                         (StrniComp(_Buffer.c_str(), Ptr_, len[0]) == 0);

      if (Found_)
      {
        _StartPt = 0;
        _EndPt = len[0];
        _StartTagStartPt = _StartPt;
        _StartTagEndPt = _EndPt;        
      }
    }
    else
    {
      if (len[3] > 0)
      {
        off = slen - len[0] - len[3];
        
        if (off < 0)
          off += len[3];

        if (off < 0)
          off += len[0];
      }
      else
      {
        off = slen - (2 * len[0]);
        
        if (off < 0)
          off += len[0];

        if (off < 0)
          off += len[0];          
      }
            
      Found_ = 0;
      Test_ = _Buffer.c_str() + off;
      slen = strlen(Test_);

      if (Base_ < off || Index_ < off)
      {
        Added_ = off - Index_;
        
        if (Added_ > 0 && _TagEsc)
        {
          FindTagEscCount(NULL, Index_, Index_ + Added_, Ptr_);
          Index_ += Added_;
        }
      }

      Added_ = slen + off - len[0];
      for (z = 0; !Found_ && slen >= len[0]; z++)
      {
        Found_ = Csense_ ? (strncmp(Test_+z, Ptr_, len[0]) == 0):
                           (StrniComp(Test_+z, Ptr_, len[0]) == 0);
        
        if (_TagEsc && Found_)
          Found_ = IsTagFound(Test_+z, sPtr_, ePtr_, off+z, Found_);
          
        slen = strlen(Test_+z+1);

        if (Found_)
        {
          _TagEscCnt = 0;
          _StartPt = off + z;
          _EndPt = _StartPt + len[0];
          _StartTagStartPt = _StartPt;
          _StartTagEndPt = _EndPt;          
        }
        else if (Index_ <= Added_ && _TagEsc)
        {
          FindTagEscCount(Test_+z, 0, 1, Ptr_);
          ++Index_;
        }
      }
    }    
  }

  return Found_;
}

/****************************************************************************/
// PURPOSE:
//   Method to extract from file or a block or pre-read text a section of text
//   starting with the Start_ string and Ending with the End_ string.
//   The starting and ending tags can also be escaped by a specified tag
//   escape character that is set by the SetTagsEscChar(char) method.
//   String tags prepended with a single or odd number of escape characters
//   will be treated as ordinary non-tag strings.
//
// PRE:
//   const char* StartOrStop_ : The starting tag to read up to and extract
//                              text from if there is no ending tag specified
//                              or is the position of the first tag after
//                              which string extraction is performed.
//   const char* End_ : The ending tag to indicate the position in the file
//                      or text section to stop string extraction.
//   Boolean SkipNl_ : Skip newlines if encountered in string extraction.
//   Boolean SkipWs_ : Skip whitespace if encountered in string extraction.
//   Boolean HasStart_ : Specify that there should be a starting tag
//                       somewhere within the text section being extracted
//                       from rather than the presence of the starting tag
//                       being indeterminate.
//   int TestLen_ : The sample length of text to examine for the presence of
//                  the starting tag in the case of an indeterminate string
//                  before quitting the method and returning *Not Found*
//   const char* Delim_ : In the case of a delimited string, the character
//                        either in the file stream or text section that
//                        splits up the text into separate text items. In the
//                        case of a text file this usually is the newline
//                        character. This is important to indicate if the tail
//                        section of text string or text file should always
//                        end with the specified Start_ or End_ tag strings.
//   bool Csense_ : Should string comparison of speficied tags be case
//                  sensitive.
//
// POST:
//   Extracts a section of text either ending in the specified string Start_
//   or surrounded by the specified strings Start_ and End_.
//
//   If only the Start_ string is given the _StartPt would indicate the
//   starting point in the extracted text section of where the Start_ string
//   begins and _EndPt would indicate where the Start_ string ends.
//   If both the Start_ and End_ string is given then _StartPt would indicate
//   the starting poin in the extracted text section of where the End_ string
//   begins and _EndPt would indicate where teh End_ string ends.
//
//   Note: The starting and ending tags can also be escaped by a specified
//   tag escape character that is set by the SetTagsEscChar(char) method.
//   String tags prepended with a single or odd number of escape characters
//   will be treated as ordinary non-tag strings.
//
int StringFilter::ReadUntilString(const char* StartOrStop_, const char* End_,
                                 Boolean SkipNl_, Boolean SkipWs_, Boolean HasStart_,
                                 int TestLen_, const char* Delim_,
                                 bool Csense_, bool Continued_)
{
  if (_Frp->EndOfFile() || !StartOrStop_)
    return 0;

  istream& Is_ = Continued_ ? *_SavedStream:_Frp->GetInStream();

  int Found_;
  long y = 0;      // index into length array
  long cnt = 0;    // number of additional characters retrieved per read cycle
                   // excluding specified line break character
  long z = 0;      // total characters retrieved per nested tag level
  long x = 0;      // read cycle character count index before set accumulated
                   // string length to read per cycle reached.
  long len[4];     // length array
  long slen;       // string length
  long Index_ = 0; // start point offset into string to use before seach done
                   // for the html tag escape character or escape string.
  
  const char* Ptr_ = StartOrStop_;
  const char* sPtr_ = Ptr_;        // start tag pointer
  const char* ePtr_ = NULL;        // end tag pointer
  const char* Nl_ = "\n\r\f\v";    // newline characters
  const char* Ws_ = " \t\n\r\f\v"; // whitespace characters
  const char* Wsxnl_ = " \t";      // whitespace characters excluding newlines
  const char* Cset_;               // pointer to character set array

  if (!Continued_)
  {
    _InTag = false;                // parse pointer within HTML tag
    _TagEscCnt = 0;                // escaped HTML tag counts
    _BreakonStartTag = false;      // parse process interrupted on start tag
    _BreakonEndTag = false;        // parse process interrupted on end tag
    _HaveTag = false;              // have specific html tag string set
    _HaveTagEsc = false;           // have tag escape characters or string
    _TagEscCnt = _NonEscCnt = 0;   // html tag escape counts
  }

  len[0] = strlen(StartOrStop_);   // string length of start tag
  len[1] = End_ ? strlen(End_):0;  // continuous accumulating string length
                                   // to process per read cycle for finding
                                   // the end tag.
  len[2] = len[1];                 // string length of end tag.
                                   // len[3], addition characters retrieved
                                   // per read cycle before line break
                                   // character encountered.

  if (SkipNl_ || SkipWs_)
  {
    Cset_ = (SkipWs_ && SkipNl_) ? Ws_:
            (SkipNl_) ? Nl_:Wsxnl_;

    while (!_Frp->EndOfFile())
      if (_GetchFromTemp)
      {
        if (!InCharSet(PeekchTempStr(), Cset_))
          break;
        else
          GetchTempStr();
      }
      else
      {
        if (!InCharSet(Is_.peek(), Cset_))
          break;
        else
          Is_.get();
      }
  }

  if (_Frp->EndOfFile())
    return 0;

  Found_ = 0;
  z = x;

  if (Continued_)
  {
    Index_ = _SavedIndex;  
    x = cnt = len[0];
    z = _Buffer.strlen();
    len[3] = (_Buffer[z-1] == *Delim_) ? (cnt+1):0;
    
    if (len[3] == 0)
      --cnt;
   
    goto DoCont;
  }
  
  for (y = 0; Ptr_; y++)
  {
    for (cnt = 0; x < len[y]; x++, cnt++)
    {
      if (_GetchFromTemp)
        _Buffer[z++] = GetchTempStr();
      else
        Is_.get(_Buffer[z++]);

      if (Continued_ && _SavedLen < 0)
        _SavedLen++;

      if (_Buffer[z-1] == *Delim_)
      {
        len[3] = cnt+1;
        break;
      }
      else
        len[3] = 0;
    
      if (_Frp->EndOfFile())
      {
        _Buffer[z] = 0;
        Found_ = IsStringFound(y, Index_, z, cnt+1, len, Ptr_, sPtr_, ePtr_, Csense_);

        if (!Found_)
          for (--z; z >= 0; z--)
            if (_GetchFromTemp)
              PutBackTempStr();
            else
              Is_.putback(_Buffer[z]);

        goto DoBrk;
      }
    }

    _Buffer[z] = 0;    
    if (len[3] == 0)
      --cnt;

    Found_ = IsStringFound(y, Index_, z, cnt+1, len, Ptr_, sPtr_, ePtr_, Csense_);

    if (!Found_)
    {    
      if (y > 0)
      {
        y = 0;
        
        if (len[3] == 0 || len[3] == len[2])
          len[1] += len[2];
      }      
      else
      {
        if (HasStart_)
        {
          x = 0;
          y = -1;
        }
        else
        {
          if (TestLen_ > 0 && z < TestLen_)
          {
            x = 0;
            y = -1;            
          }
          else
          {
            for (--z; z >= 0; z--)
              if (_GetchFromTemp)
                PutBackTempStr();
              else
                Is_.putback(_Buffer[z]);
                
            break;
          }
        }
      }
    }
    else
    {
DoCont:

      if (y == 0 && End_)
      {
        if (_NestingLevel && !Continued_)
        {        
          _SavedIndex = Index_;
          _SavedStream = &Is_;
          Push();

          if (HasStart_)
          {
            z = x = 0;
            y = -1;
          }
          else
          {
            if (TestLen_ > 0 && z < TestLen_)
            {
              z = x = 0;
              y = -1;
            }
            else
            {
              for (--z; z >= 0; z--)
                if (_GetchFromTemp)
                  PutBackTempStr();
                else
                  Is_.putback(_Buffer[z]);
                
              break;
            }
          }
        }
        else
        {
          y = 0;
          len[1] += len[0];
          Ptr_ = End_;
          sPtr_ = NULL;
          ePtr_ = Ptr_;
          slen = ::SafeStrLen(_Buffer(_StartPt));

          if (_Next && !Continued_)
            _Next->AppendToInput(_Buffer.c_str(), _StartPt);
          else if (Continued_ && _SavedLen > 0)
          {
            if (_Next)
              _Next->AppendToInput(_Buffer.c_str(), _StartPt);
              
            while (_SavedLen > 0)
            {
              len[1] += len[2];
              _SavedLen -= len[2];
            }
          }
          
          _Buffer.MemMove(_Buffer.c_str() + _StartPt, slen + 1);
          _StartTagStartPt -= _StartPt;
          _StartTagEndPt -= _StartPt;
          z = slen;
        }
      }
      else
      {
        slen = ::SafeStrLen(_Buffer(_EndPt));
        for (--z; z >= 0 && slen > 0; z--, slen--)
          if (_GetchFromTemp)
            PutBackTempStr();
          else
            Is_.putback(_Buffer[z]);
        
        _Buffer[_EndPt] = 0;
        
        if (_Next)
        {
          _Next->AppendToInput(_Buffer.c_str()+_StartTagStartPt, _EndPt-_StartTagStartPt);
          _Next->ReadUntilString(StartOrStop_, End_, SkipNl_, SkipWs_,
                                 HasStart_, TestLen_, Delim_, Csense_, true);
        }
        
        break;
      }    
    }
  }

DoBrk:

  if (!Found_)
  {
    if (!_GetchFromTemp && !_Frp->EndOfFile())
    {
      _Frp->ClearState();
      _Frp->GetInStream().clear(ios::goodbit);
    }
  
    return Found_;
  }

  // Removing trailing spaces and adjusting start tag found start/end points
  size_t Skipped_ = 0;
  ::RemoveTrailing(_Buffer.c_str(), Ws_);
  for (y = 0; _Buffer[y] && InCharSet(_Buffer[y], Ws_); y++);
  _StartPt -= y;
  _EndPt -= y;

  _StartTagStartPt -= y;
  _StartTagEndPt -= y;

  if (End_)
  {
    _EndTagStartPt -= y;
    _EndTagEndPt -= y;
  }
  else
  {  
    _EndTagStartPt = _StartTagStartPt;
    _EndTagEndPt = _StartTagEndPt;
  }
  
  for (x = 0; (_Buffer[x] = _Buffer[y]); x++, y++);

  // if search specified for html tags then adjusting position of start
  // tag start/end points and end tag start/end points to match with html
  // starting character for tags "<" and html closing character for tags ">"
  // allow only 1 intervening space for text enclosed within HTML tags
  if (_IsHtmlTag)
  {
    char* sSaved_ = NULL;
    char* eSaved_ = NULL;
    bool Match_;

    if (StartOrStop_ && End_)
    {
      slen = _StartTagEndPt - _StartTagStartPt;

      if (slen)
      {
        sSaved_ = (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * (slen+1));
        strncpy(sSaved_, _Buffer.c_str() + _StartTagStartPt, slen);
        sSaved_[slen] = 0;
      }
    }

    if (StartOrStop_)
    {
      slen = _EndTagEndPt - _EndTagStartPt;

      if (slen)
      {
        eSaved_ = (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * (slen+1));
        strncpy(eSaved_, _Buffer.c_str() + _EndTagStartPt, slen);
        eSaved_[slen] = 0;
      }
    }
  
    ::AllowLenOnlyOfChar(_Buffer.c_str(), 1, Ws_, &Skipped_);

    if (StartOrStop_ && eSaved_)
    {
      _StartPt = _EndTagStartPt;
      _EndPt = _EndTagEndPt;
  
      _EndPt -= Skipped_;
      _EndTagEndPt -= Skipped_;

      Match_ = Csense_ ? (strncmp(_Buffer.c_str()+_StartPt-Skipped_, eSaved_, slen) == 0):
                         (StrniComp(_Buffer.c_str()+_StartPt-Skipped_, eSaved_, slen) == 0);

      if (Match_)
      {
        _StartPt -= Skipped_;
        Skipped_ = -Skipped_;
      }
      else
      {
        slen = -Skipped_;
        Skipped_ = 0;
        ::AllowLenOnlyOfChar(eSaved_, 1, Ws_, &Skipped_);
        slen += Skipped_ * 2;
        _StartPt += slen;
        Skipped_ = slen;
      }

      _EndTagStartPt += Skipped_;

      if (_IsHtmlTag && _Buffer[_StartPt] != '<')
      {
        _EndTagStartPt = _EndTagEndPt - 1;
        while (_EndTagStartPt && _Buffer[--_EndTagStartPt] != '<');
        _StartPt = _EndTagStartPt;
      }
    }

    /// with start tags: start
    if (StartOrStop_ && End_ && sSaved_)
    {
      _StartPt = _StartTagStartPt;
      _EndPt = _StartTagEndPt;

      slen = _StartTagEndPt - _StartTagStartPt;
      Match_ = Csense_ ? (strncmp(_Buffer.c_str()+_StartPt, sSaved_, slen) == 0):
                         (StrniComp(_Buffer.c_str()+_StartPt, sSaved_, slen) == 0);

      if (!Match_)
      {
        Skipped_ = 0;
        ::AllowLenOnlyOfChar(sSaved_, 1, Ws_, &Skipped_);
        _EndPt -= Skipped_;
        _StartTagEndPt -= Skipped_;
      }

      if (_IsHtmlTag && _Buffer[_EndPt-1] != '>')
      {
        _StartTagEndPt = _StartTagStartPt + 1;
        while (_Buffer[_StartTagEndPt] && _Buffer[_StartTagEndPt++] != '>');
        _EndPt = _StartTagEndPt;
      }

      _StartPt = _StartTagStartPt;
      _EndPt = End_ ? _EndTagEndPt:_StartTagEndPt;
    }
    /// with start tags: end

    if (StartOrStop_)
    {
      ::DeleteArray(eSaved_);
      if (End_)
        ::DeleteArray(sSaved_);
    }
  }    

  if (!_GetchFromTemp && !_Frp->EndOfFile())
  {
    _Frp->ClearState();
    _Frp->GetInStream().clear(ios::goodbit);
  }

  return Found_;
}

/****************************************************************************/
void StringFilter::AppendToInput(char* Buffer_, long Len_)
{
  long len = _Buffer.strlen();
  _Buffer += ChrString(Buffer_, Len_);
  len += Len_;
  _Buffer[len] = 0;
  _SavedLen += Len_;
}

/****************************************************************************/
StringFilter* StringFilter::Next(int x)
{
  StringFilter* Old_ = _Next;
  StringFilter* Cur_ = this;

  while (Old_ && x)
  {
    --x;
    Cur_ = Old_;
    Old_ = Old_->_Next;
  }

  return Cur_;
}

/****************************************************************************/
StringFilter& StringFilter::ClearNesting()
{
  StringFilter* Old_ = Pop();
        
  while (Old_)
  {
    delete Old_;
    Old_ = Pop();
  }

  return *this;
}

/****************************************************************************/
StringFilter& StringFilter::SetInputFile(const char* Fname_, int NestingLevel_)
{
  if (Fname_)
  {
    _Frp = new FileReader();  
    _Frp->Open(Fname_, ios::in);

    _MainFileName = new_char_string(Fname_);
    _NestingLevel = NestingLevel_;

    if (!_Buffer.strlen())
     _Buffer.StrFill(0, BUFSIZE);
     // (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * BUFSIZE);

    _Buffer.StrFill(0, BUFSIZE);
  }

  return *this;
}

/****************************************************************************/
StringFilter& StringFilter::ResetStreams()
{
  _Frp->ClearState();
  _Frp->GetInStream().clear(ios::goodbit);
  _Frp->ResetStreams();
}

/****************************************************************************/
int StringFilter::ReadUntilString(Boolean SkipNl_, Boolean SkipWs_, Boolean HasStart_,
                                  int TestLen_, const char* Delim_,
                                  bool Csense_, bool Continued_)
{
  if (_StartTag && _EndTag)
    return ReadUntilString(_StartTag, _EndTag, SkipNl_,
                           SkipWs_, HasStart_, TestLen_,
                           Delim_, Csense_, Continued_);

  return 0;
}

/****************************************************************************/
int StringFilter::IsMonth(const char* Str_)
{
  if (!Str_)
    return 0;

  return
  (
    (StrniComp(Str_, "sep", 3) == 0) ||
    (StrniComp(Str_, "oct", 3) == 0) ||
    (StrniComp(Str_, "nov", 3) == 0) ||
    (StrniComp(Str_, "dec", 3) == 0) ||

    (StrniComp(Str_, "jan", 3) == 0) ||
    (StrniComp(Str_, "feb", 3) == 0) ||
    (StrniComp(Str_, "mar", 3) == 0) ||
    (StrniComp(Str_, "apr", 3) == 0) ||

    (StrniComp(Str_, "may", 3) == 0) ||
    (StrniComp(Str_, "jun", 3) == 0) ||
    (StrniComp(Str_, "jul", 3) == 0) ||
    (StrniComp(Str_, "aug", 3) == 0)
  );
}

/****************************************************************************/
int StringFilter::IsDateStr(const char* Str_)
{
  char TestStr_[32];  
  strncpy(TestStr_, Str_, 31);
  TestStr_[31] = 0;
  RemovePadding(TestStr_, " \t\n");

  if (isdigit(TestStr_[0]))
  {
    if (isdigit(TestStr_[1]))
    {
      if (IsMonth(TestStr_ + 2))
        return (strlen(TestStr_+2) >= 5 &&
                isdigit(TestStr_[5]) &&
                isdigit(TestStr_[6]));
      else
        return FALSE;
    }
    else
    {
      if (IsMonth(TestStr_ + 1))
        return (strlen(TestStr_+1) >= 5 &&
                isdigit(TestStr_[4]) &&
                isdigit(TestStr_[5]));
      else
        return FALSE;      
    }
  }

  return FALSE;
}

/****************************************************************************/
#if STRFILTER_DEBUG
char** GrowTokArray(char** TokArray_, int Incr_)
{
  int x, oldsz;
  int Size_ = (int)TokArray_[0][0];
  char** OldArr_ = TokArray_;

  ++Size_;
  oldsz = Size_;
  Size_ += Incr_;
  TokArray_ = (char**)RawAllocateWith(MEMMATRIX, sizeof(char*) * Size_);

  for (x = 0; x < oldsz; x++)
    TokArray_[x] = OldArr_[x];

  for (;x < Size_; x++)
    TokArray_[x] = NULL;

  TokArray_[0][0] = Size_ - 1;
  TokArray_[0][1] = 0;
  ::DeleteArray(OldArr_);
  
  return TokArray_;
}

/****************************************************************************/
char** InitTokArray(char** TokArray_)
{
  int x, max;

  max = 11;
  TokArray_ = (char**)RawAllocateWith(MEMMATRIX, sizeof(char*) * max);
  TokArray_[0] = (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * 2);
  TokArray_[0][0] = max - 1;
  TokArray_[0][1] = 0;

  for (x = 1; x < max; x++)
    TokArray_[x] = NULL;

  return TokArray_;
}

/****************************************************************************/
char** ClearTokArray(char** TokArray_)
{
  int x, max;
  
  max = (int)TokArray_[0][0];
  ++max;
  
  for (x = 0; x < max; x++)
    ::DeleteArray(TokArray_[x]);

  ::DeleteArray(TokArray_);
  TokArray_ = NULL;
  
  return TokArray_;
}

/****************************************************************************/
void ShowTokArray(char** TokArray_, int& st, int sz)
{
  for (;st < sz; st++)
    cout <<TokArray_[st] <<endl;

  cout <<endl;
}

/****************************************************************************/
// -<symbolname:Symbol_Type:symattr:datatype:structtype;>
// -<symbolname:Operator_Type:symattr:datatype:structtype:optype;>
// -<symbolname:Sequence_Type:seqtype:
//    {decltype if seqtype==declaration |
//     defntype if seqtype==definition}
//   structtype:symattr:datatype:Continue=n|End;>
// -<symbolname:BracketedSymbol_Type:LeftBracket:datatype:structtype:Delim=c|NoDelim:pushblk:pushfnc>
// <BracketedSymbol_Type:Data:datatype:Delim=c|NoDelim:Continue=n|End>
// <BracketedSymbol_Type:LeftBracket:datatype:structtype:Delim=c|NoDelim:pushblk:pushfnc>
// <BracketedSymbol_Type:RightBracket:datatype:structtype:Delim=c|NoDelim:pushblk:pushfnc;>
//
char** ReadTextFile(StringFilter* sfilter_, int& sz, char** TokArray_)
{
  int len, max;
  char* RetStr_ = NULL;
  const char* RdStr_;
  ChrString RealStr_;
  char StartTag_[3];
  bool EndRead_ = false;

  max = (int)TokArray_[0][0] + 1;
  strcpy(StartTag_, "-<");

  while (!EndRead_)
  {
    if (sfilter_->ReadUntilString(StartTag_, ">", 1, 1, 1, 0, 0))
    {
      RdStr_ = sfilter_->GetBuffer();
      RealStr_ += RdStr_;
      len = strlen(RdStr_);

      if (strcmp(RdStr_+len-2, ";>") == 0)
      {
        RetStr_ = new_char_string(RealStr_.c_str());
        TokArray_[sz++] = RetStr_;
        EndRead_ = true;
      }
      else
      {
        RetStr_ = new_char_string(RealStr_.c_str());
        TokArray_[sz++] = RetStr_;
        RealStr_ = "";
        strcpy(StartTag_, "<");
      }

      if (sz == max)
      {
        TokArray_ = GrowTokArray(TokArray_, 10);
        max = (int)TokArray_[0][0] + 1;        
      }
    }

    if (sfilter_->GetFileReader()->EndOfFile())
      EndRead_ = true;
  }

  return TokArray_;
}

/****************************************************************************/
int main()
{
#if STRFILTER_DEBUG1
  StringFilter* sfilter_ = new StringFilter("symtestdata.txt");
  system("cls");
  sfilter_->SetTagsEscChar('\\');
  
  int sz = 1;
  int st = 1;
  char** TokArray_;
  TokArray_ = InitTokArray(TokArray_);

  while (!sfilter_->GetFileReader()->EndOfFile())
  {
    TokArray_ = ReadTextFile(sfilter_, sz, TokArray_);
    ShowTokArray(TokArray_, st, sz);
  }
  
  TokArray_ = ClearTokArray(TokArray_);
  
  delete sfilter_;
  return 0;
#endif

#if STRFILTER_DEBUG2
  long stsp;
  long step;
  long etsp;
  long etep;
  const char* Bufp;
  char Buffer[1024];
  int Nest_ = 1;
  int x;

  system("cls");
  StringFilter* sfilter_ = new StringFilter("nestedsymtest.html", Nest_);
  sfilter_->SetParseHtmlTag(true);
  sfilter_->ReadUntilString("<span", "</span>", 1, 1, 1);
  
  for (x = 0; x <= Nest_; x++)
  {
    stsp = sfilter_->Next(x)->GetStartTagStartPt();
    step = sfilter_->Next(x)->GetStartTagEndPt();
    etsp = sfilter_->Next(x)->GetEndTagStartPt();
    etep = sfilter_->Next(x)->GetEndTagEndPt();
    Bufp = sfilter_->Next(x)->GetBuffer();

    cout <<" stsp: " <<stsp <<" step: " <<step
         <<" etsp: " <<etsp <<" etep: " <<etep <<endl;
    strncpy(Buffer, &Bufp[stsp], step-stsp);
    Buffer[step-stsp] = 0;
    cout <<"Start Tag: " <<Buffer <<endl;
    strncpy(Buffer, &Bufp[step], etsp-step);
    Buffer[etsp-step] = 0;
    cout <<"Content: " <<Buffer <<endl;
    strncpy(Buffer, &Bufp[etsp], etep-etsp);
    Buffer[etep-etsp] = 0;
    cout <<"End Tag: " <<Buffer <<endl;
  }

  Nest_ = 2;
  sfilter_->SetNestingLevel(Nest_);
  sfilter_->ReadUntilString("<div", "</div>", 1, 1, 1);
  cout <<endl
       <<"--------------------------------------------------------------------------------"
       <<endl;

  for (x = 0; x <= Nest_; x++)
  {
    stsp = sfilter_->Next(x)->GetStartTagStartPt();
    step = sfilter_->Next(x)->GetStartTagEndPt();
    etsp = sfilter_->Next(x)->GetEndTagStartPt();
    etep = sfilter_->Next(x)->GetEndTagEndPt();
    Bufp = sfilter_->Next(x)->GetBuffer();

    cout <<" stsp: " <<stsp <<" step: " <<step
         <<" etsp: " <<etsp <<" etep: " <<etep <<endl;
    strncpy(Buffer, &Bufp[stsp], step-stsp);    
    Buffer[step-stsp] = 0;
    cout <<"Start Tag: " <<Buffer <<endl;
    strncpy(Buffer, &Bufp[step], etsp-step);
    Buffer[etsp-step] = 0;
    cout <<"Content: " <<Buffer <<endl;
    strncpy(Buffer, &Bufp[etsp], etep-etsp);
    Buffer[etep-etsp] = 0;
    cout <<"End Tag: " <<Buffer <<endl;
  }

  cout <<endl
       <<"--------------------------------------------------------------------------------"
       <<endl;
  sfilter_->SetParseHtmlTag(false);
  sfilter_->ReadUntilFound("<div style=", NULL, "\n", TRUE);
  stsp = sfilter_->GetStartPt();
  step = sfilter_->GetEndPt();
  Bufp = sfilter_->GetBuffer();
  cout <<" stsp: " <<stsp <<" step: " <<step <<endl;  
  cout <<"Buffer: " <<Bufp <<endl;  
  strncpy(Buffer, &Bufp[stsp], step-stsp);
  Buffer[step-stsp] = 0;  
  cout <<"Start Tag: " <<Buffer <<endl;
      
  sfilter_->ReadUntilFound("\"css-gen3\">Weather:", NULL, "\n", TRUE);
  stsp = sfilter_->GetStartTagStartPt();
  step = sfilter_->GetStartTagEndPt();
  Bufp = sfilter_->GetBuffer();
  cout <<" stsp: " <<stsp <<" step: " <<step <<endl;
  cout <<"Buffer: " <<Bufp <<endl;
  strncpy(Buffer, &Bufp[stsp], step-stsp);
  Buffer[step-stsp] = 0;
  cout <<"Start Tag: " <<Buffer <<endl;

  sfilter_->SetParseHtmlTag(true);
  sfilter_->ReadUntilFound("<span class", "</span>", "\n", TRUE);
  stsp = sfilter_->GetStartTagStartPt();
  step = sfilter_->GetStartTagEndPt();
  etsp = sfilter_->GetEndTagStartPt();
  etep = sfilter_->GetEndTagEndPt();
  Bufp = sfilter_->GetBuffer();
  cout <<" stsp: " <<stsp <<" step: " <<step
       <<" etsp: " <<etsp <<" etep: " <<etep <<endl;
  strncpy(Buffer, &Bufp[stsp], step-stsp);
  Buffer[step-stsp] = 0;
  cout <<"Start Tag: " <<Buffer <<endl;
  strncpy(Buffer, &Bufp[step], etsp-step);
  Buffer[etsp-step] = 0;
  cout <<"Content: " <<Buffer <<endl;
  strncpy(Buffer, &Bufp[etsp], etep-etsp);
  Buffer[etep-etsp] = 0;
  cout <<"End Tag: " <<Buffer <<endl;  

  delete sfilter_;
#endif
}

#endif
/****************************************************************************/
#endif
