#ifndef STRFILTER_H
#define STRFILTER_H

#ifndef FILEREADER_H
  #include "filereader.h"
#endif
#ifndef CHRSTRING_H
  #include "chrstring.h"
#endif
#ifndef FULLDATE_H
  #include "fulldate.h"
#endif

#define STRFILTER_DEBUG         0
#define STRFILTER_DEBUG1        0
#define STRFILTER_DEBUG2        0

/****************************************************************************/
class StringFilter
{
  private:
    FileReader* _Frp;
    Boolean _GetchFromTemp;
    ChrString _Buffer;
    ChrString _EndLine;
    ChrString _LineToFind;
    char* _MainFileName;
    char* _EndReadMark;
    int _NestingLevel;
    StringFilter* _Next;
    
    long _SavedIndex;
    long _SavedLen;
    istream* _SavedStream;
    bool _LineFound;
    char _EndLineDelim;
    
    long _EndPt;
    long _StartPt;
    long _StartTagEndPt;
    long _StartTagStartPt;
    long _EndTagEndPt;
    long _EndTagStartPt;
    bool _IsHtmlTag;
    char* _TempRec;
    long _Tindex;
    long _Tlength;    

    char* _StartTag;
    char* _EndTag;
    char _TagEsc;

    // Tag count and in tag status flag
    bool _InTag;

    // contiguous counts of escape and non-escape characters.
    // contiguous even counts indicate cancelling out of escape sequences.
    long _TagEscCnt;
    long _NonEscCnt;

    // Stoping conditions and escape character detection flags and counts
    bool _BreakonStartTag;
    bool _BreakonEndTag;
    bool _HaveTag;
    bool _HaveTagEsc;

    int GetchTempStr();
    int PeekchTempStr();
    void PutBackTempStr();
    StringFilter* Push();
    StringFilter* Pop();

    int IsMonth(const char* Str_);
    int IsDateStr(const char* Str_);
    
    void AppendToInput(char* Buffer_, long Len_);
    void ResetFilter(bool ResetNesting_);
    void FindTagEscCount(char* BufStr_, long Start_,
                         long OffSet_, const char* TagStr_);
    bool IsTagFound(const char* Str_, const char* sPtr_,
                    const char* ePtr_, long Index_, int Found_);
    int IsStringFound(short y, long& Index_, long Total_, long Added_, long* len,
                      const char* Ptr_, const char* sPtr_, const char* ePtr_,
                      bool Csense_);

  public:
    enum
    {
      DATE                  = 19,
      INT_FIELD             = 29,
      FLOAT_FIELD           = 39,
      TENS_DIGIT            = 49,
      FLDSIZE               = 256,
      BUFSIZE               = 256
    };
  
    StringFilter();
    StringFilter(const StringFilter& Stok_, StringFilter* Next_=NULL);
    StringFilter(const char* Fname_,
                 int NestingLevel_=0, StringFilter* Next_=NULL);
    ~StringFilter();

    StringFilter& operator = (const StringFilter& Stok_);

    inline const char* GetBuffer() const    
        { return _Buffer.c_str(); }
    inline const char* GetTempString() const
        { return _TempRec; }
    inline const char* StartTag() const
        { return _StartTag; }
    inline const char* EndTag() const
        { return _EndTag; }
    inline char TagsEscChar() const
        { return _TagEsc; }
        
    inline long GetStartPt() const
        { return _StartPt; }
    inline long GetEndPt() const    
        { return _EndPt; }
    inline long GetStartTagStartPt() const
        { return _IsHtmlTag ? _StartTagStartPt:_StartPt; }
    inline long GetStartTagEndPt() const
        { return _IsHtmlTag ? _StartTagEndPt:_EndPt; }
    inline long GetEndTagStartPt() const
        { return _EndTagStartPt; }
    inline long GetEndTagEndPt() const
        { return _EndTagEndPt; }
        
    FileReader* GetFileReader()
        { return _Frp; }
    inline bool InTag() const
        { return _InTag; }
    inline bool BreakonStartTag() const
        { return _BreakonStartTag; }
    inline bool BreakonEndTag() const
        { return _BreakonEndTag; }
    inline bool HaveTag() const
        { return _HaveTag; }
    inline bool HaveTagsEscChar() const
        { return _HaveTagEsc; }

    StringFilter* Next(int x);
    StringFilter& ClearNesting();
    StringFilter& SetInputFile(const char* Fname_, int NestingLevel_=0);
    StringFilter& SetParseHtmlTag(bool IsTag_);
    StringFilter& SetNestingLevel(int Level_);
    StringFilter& SetStringTags(const char* Start_, const char* End_);
    StringFilter& SetTagsEscChar(char Ch_);
    StringFilter& SetTempString(const char* Str_, Boolean Erase_, long len);
    StringFilter& ResetStreams();

    int GetTxtLine(const char* Delim_, FileReader* FileRp_=NULL,
                   char* StrBuf_=NULL, int StrBufSz_=0);
    int ReadUntilFound(const char* Mark_, const char* EndLine_,
                       const char* Delim_="\n", Boolean Rws_=1,
                       bool Csense_=false);    
    
    int ExtractUntil(char* InputStr_, short InputLen_,
                     const char* CharSet_, Boolean InSet_,
                     Boolean PutBack_);
    int ExtractUntil(char* InputStr_, short InputLen_,
                     int(*iscsetfn)(int), Boolean InSet_,
                     Boolean PutBack_);

    int TestRead(Boolean Find_, const char* Mark_, int Spec_,
                 int Len_, Boolean SkipNl_, Boolean SkipWs_);
    int ReadUntilString(const char* StartOrStop_, const char* End_,
                        Boolean SkipNl_, Boolean SkipWs_, Boolean HasStart_,
                        int TestLen_=0, const char* Delim_="\n",
                        bool Csense_=false, bool Continued_=false);
    int ReadUntilString(Boolean SkipNl_, Boolean SkipWs_, Boolean HasStart_,
                        int TestLen_=0, const char* Delim_="\n",
                        bool Csense_=false, bool Continued_=false);
};

/****************************************************************************/
#endif

