// File reader ADT class
//
#ifndef TEXTFILELINE_CPP
#define TEXTFILELINE_CPP
#ifndef TEXTFILELINE_H
  #include "textfileline.h"
#endif

/****************************************************************************/
//                      File Position Class
/****************************************************************************/
FilePosition::FilePosition(bool TextType_):
_BytesRead(0),
_TextFileType(false),
_RowPos(0),
_ColPos(0)
{}

/****************************************************************************/
FilePosition& FilePosition::SetTextFileType(bool TextType_)
{
  _TextFileType = TextType_;
  return *this;
}

/****************************************************************************/
FilePosition& FilePosition::IncBytesRead(long Bytes_)
{
  _BytesRead += Bytes_;
  return *this;
}

/****************************************************************************/
FilePosition& FilePosition::IncColPos(long ColPos_)
{
  _ColPos += ColPos_;
  return *this;
}

/****************************************************************************/
FilePosition& FilePosition::IncRowPos(long RowPos_)
{
  _RowPos += RowPos_;
  return *this;
}

/****************************************************************************/
FilePosition& FilePosition::DecColPos(long ColPos_)
{
  _ColPos -= ColPos_;
  return *this;
}

/****************************************************************************/
FilePosition& FilePosition::DecRowPos(long RowPos_)
{
  _RowPos -= RowPos_;
  return *this;
}

/****************************************************************************/
FilePosition& FilePosition::SetColPos(long Row_)
{
  _RowPos = Row_;
  return *this;
}

/****************************************************************************/    
FilePosition& FilePosition::SetRowPos(long Col_)
{
  _ColPos = Col_;
  return *this;
}

/****************************************************************************/
FilePosition& FilePosition::ResetColPos()
{
  _ColPos = 0;
  return *this;
}

/****************************************************************************/
FilePosition& FilePosition::ResetRowPos()
{
  _RowPos = 0;
  return *this;
}

/****************************************************************************/
//			Delim Info Class
/****************************************************************************/
DelimInfo::DelimInfo():
_Delims((char**)RawAllocateWith(MEMMATRIX, sizeof(char*) * TextFileLineBuffer::MAXDELIMS))
{
  memset(_Delims, 0, sizeof(char*) * TextFileLineBuffer::MAXDELIMS);
  memset(_DelimFound, 0, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
  memset(_DelimPos, 0, sizeof(long) * TextFileLineBuffer::MAXDELIMS);
}

/****************************************************************************/
DelimInfo::DelimInfo(char** Delims_, bool* DelimFound_, long* DelimPos_):
_Delims((char**)RawAllocateWith(MEMMATRIX, sizeof(char*) * TextFileLineBuffer::MAXDELIMS))
{
  if (Delims_)
  {
    int x;
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      if (Delims_[x])
        _Delims[x] = NewString(Delims_[x]);
      else
        _Delims[x] = NULL;

    memmove(_DelimFound, DelimFound_, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
    memmove(_DelimPos, DelimPos_, sizeof(long) * TextFileLineBuffer::MAXDELIMS);
  }
  else
  {
    memset(_Delims, 0, sizeof(char*) * TextFileLineBuffer::MAXDELIMS);
    memset(_DelimFound, 0, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
    memset(_DelimPos, 0, sizeof(long) * TextFileLineBuffer::MAXDELIMS);
  }
}

/****************************************************************************/
DelimInfo::DelimInfo(const DelimInfo& Obj_):
_Delims((char**)RawAllocateWith(MEMMATRIX, sizeof(char*) * TextFileLineBuffer::MAXDELIMS))
{
  if (Obj_._Delims)
  {
    int x;
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      if (Obj_._Delims[x])
        _Delims[x] = NewString(Obj_._Delims[x]);
      else
        _Delims[x] = NULL;

    memmove(_DelimFound, Obj_._DelimFound, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
    memmove(_DelimPos, Obj_._DelimPos, sizeof(long) * TextFileLineBuffer::MAXDELIMS);
  }
  else
  {
    memset(_Delims, 0, sizeof(char*) * TextFileLineBuffer::MAXDELIMS);
    memset(_DelimFound, 0, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
    memset(_DelimPos, 0, sizeof(long) * TextFileLineBuffer::MAXDELIMS);
  }
}

/****************************************************************************/
DelimInfo::~DelimInfo()
{
  int x;
  for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
  {
    ::RawDeleteArray(_Delims[x]);
    _Delims[x] = NULL;
  }

  ::RawDeleteArray(_Delims);
  _Delims = NULL;
}

/****************************************************************************/
char** DelimInfo::InitDelimsArray(char** ArrPtr_, size_t max)
{
  size_t x;
  if (ArrPtr_)
    for (x = 0; x < max; x++)
      ArrPtr_[x] = NULL;

  return ArrPtr_;
}

/****************************************************************************/
DelimInfo& DelimInfo::AssignTo(char** Delims_, bool* DelimFound_, long* DelimPos_)
{
  if (_Delims)
  {
    int x;
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
    {
      ::RawDeleteArray(Delims_[x]);

      if (_Delims[x])      
        Delims_[x] = NewString(_Delims[x]);
      else
        Delims_[x] = NULL;
    }

    memmove(DelimFound_, _DelimFound, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
    memmove(DelimPos_, _DelimPos, sizeof(long) * TextFileLineBuffer::MAXDELIMS);
  }
  else
  {
    Delims_ = InitDelimsArray(Delims_, TextFileLineBuffer::MAXDELIMS);
    memset(DelimFound_, 0, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
    memset(DelimPos_, 0, sizeof(long) * TextFileLineBuffer::MAXDELIMS);
  }

  return *this;
}

/****************************************************************************/
MEMORYOPS_DEFN(DelimInfo)

/****************************************************************************/
//			Line Data Record Class
/****************************************************************************/
TextFileLineData::TextFileLineData(TextFileLineBuffer& Obj_):
_RowPos(Obj_.GetFilePosition()->RowPos()),
_LineFitInBuffer(Obj_.LineFitInBuffer()),
_SingleDelimSetFound(Obj_.SingleDelimSetFound()),
_TimesLimitReached(Obj_.TimesLimitReached()),
_ReadTimesRequired(Obj_.ReadTimesRequired()),
_DelimSetTypeFound(Obj_.DelimSetTypeFound()),
_EolFound(Obj_.EolFound()),
_EofFound(Obj_.EofFound()),
_EndLinePos(Obj_.EolPosition()),
_EndFilePos(Obj_.EofPosition()),
_FirstDelims(Obj_._FirstDelims),
_LastDelims(Obj_._LastDelims),
_ReadBufQueue(Obj_._ReadBufQueue),
_DelimStk(Obj_._DelimStk)
{
  memmove(_FirstDelimFound, Obj_._FirstDelimFound, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
  memmove(_FirstDelimPos, Obj_._FirstDelimPos, sizeof(long) * TextFileLineBuffer::MAXDELIMS);

  memmove(_LastDelimFound, Obj_._LastDelimFound, sizeof(bool) * TextFileLineBuffer::MAXDELIMS);
  memmove(_LastDelimPos, Obj_._LastDelimPos, sizeof(long) * TextFileLineBuffer::MAXDELIMS);
}

/****************************************************************************/
TextFileLineData::~TextFileLineData()
{}

/****************************************************************************/
void TextFileLineData::DumpData(ostream& Out_, bool ColHead_)
{
  long x, y;
  long len;
  long max = _ReadBufQueue->Size();
  char sep[4] = ",,:";
  
  long* vect;
  DelimInfo* info;

  if (ColHead_)
    Out_ <<"(rpos)"   <<_RowPos <<":"
         <<"(lfit)"   <<_LineFitInBuffer <<":"
         <<"(sdsfnd)" <<_SingleDelimSetFound <<":"
         <<"(dstfnd)" <<_DelimSetTypeFound <<":"
         <<"(tlimr)"  <<_TimesLimitReached <<":"
         <<"(rdtreq)" <<_ReadTimesRequired <<":"
         <<"(eolfnd)" <<_EolFound <<":"
         <<"(eolpos)" <<_EndLinePos <<":"
         <<"(eoffnd)" <<_EofFound <<":"
         <<"(eofpos)" <<_EndFilePos <<";"
         <<endl;
  else
    Out_ <<_RowPos <<":"
         <<_LineFitInBuffer <<":"
         <<_SingleDelimSetFound <<":"
         <<_DelimSetTypeFound <<":"
         <<_TimesLimitReached <<":"
         <<_ReadTimesRequired <<":"
         <<_EolFound <<":"
         <<_EndLinePos <<":"
         <<_EofFound <<":"
         <<_EndFilePos <<";"
         <<endl;

  if (ColHead_)
    Out_ <<"(rbqmax)" <<max <<";"
         <<endl;
  else
    Out_ <<max <<";"
         <<endl;

  for (x = 0; x < max; x++)
  {
    vect = _ReadBufQueue->Dequeue();

    if (vect)
    {
      len = vect[0];

      if (ColHead_)
      {
        Out_ <<"(rbql[" <<x <<"])" <<len;
        for (y = 1; y <= len; y++)
          Out_ <<":(rbqv[" <<x <<"][" <<y <<"])" <<vect[y];
      }
      else
      {
        Out_ <<len;
        for (y = 1; y <= len; y++)
          Out_ <<":" <<vect[y];
      }

      Out_ <<";"
           <<endl;
    }

    ::RawDeleteArray(vect);
    vect = NULL;
  }

  max = _DelimStk->Size();
  if (ColHead_)
    Out_ <<"(dstkmax)" <<max <<";"
         <<endl;
  else
    Out_ <<max <<";"
         <<endl;

  sep[2] = ':';

  if (ColHead_)
  {
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<"(fdfnd[" <<x <<"])" <<(_FirstDelimFound[x] ? 1:0) <<sep[x];

    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<"(fdpos[" <<x <<"])" <<(_FirstDelimFound[x] ? _FirstDelimPos[x]:0) <<sep[x];

    sep[2] = ';';
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<"(fdstr[" <<x <<"])" <<(_FirstDelimFound[x] ? _FirstDelims[x]:"-") <<sep[x];
  }
  else
  {
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<(_FirstDelimFound[x] ? 1:0) <<sep[x];

    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<(_FirstDelimFound[x] ? _FirstDelimPos[x]:0) <<sep[x];

    sep[2] = ';';
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<(_FirstDelimFound[x] ? _FirstDelims[x]:"-") <<sep[x];
  }

  Out_ <<endl;

  for (x = 0; x < max; x++)
  {  
    info = _DelimStk->Dequeue();

    if (info)
    {
      sep[2] = ':';

      if (ColHead_)
      {
        for (y = 0; y < TextFileLineBuffer::MAXDELIMS; y++)
          Out_ <<"(dlmfnd[" <<x <<"][" <<y <<"])" <<(info->_DelimFound[y] ? 1:0) <<sep[y];

        for (y = 0; y < TextFileLineBuffer::MAXDELIMS; y++)
          Out_ <<"(dlmpos[" <<x <<"][" <<y <<"])" <<(info->_DelimFound[y] ? info->_DelimPos[y]:0) <<sep[y];

        sep[2] = ';';
        for (y = 0; y < TextFileLineBuffer::MAXDELIMS; y++)
          Out_ <<"(dlmstr[" <<x <<"][" <<y <<"])" <<(info->_DelimFound[y] ? info->_Delims[y]:"-") <<sep[y];
      }
      else
      {
        for (y = 0; y < TextFileLineBuffer::MAXDELIMS; y++)
          Out_ <<(info->_DelimFound[y] ? 1:0) <<sep[y];

        for (y = 0; y < TextFileLineBuffer::MAXDELIMS; y++)
          Out_ <<(info->_DelimFound[y] ? info->_DelimPos[y]:0) <<sep[y];

        sep[2] = ';';
        for (y = 0; y < TextFileLineBuffer::MAXDELIMS; y++)
          Out_ <<(info->_DelimFound[y] ? info->_Delims[y]:"-") <<sep[y];
      }

      Out_ <<endl;
    }

    delete info;
    info = NULL;
  }

  sep[2] = ':';

  if (ColHead_)
  {  
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<"(ldfnd[" <<x <<"])" <<(_LastDelimFound[x] ? 1:0) <<sep[x];

    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<"(ldpos[" <<x <<"])" <<(_LastDelimFound[x] ? _LastDelimPos[x]:0) <<sep[x];

    sep[2] = ';';
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<"(ldstr[" <<x <<"])" <<(_LastDelimFound[x] ? _LastDelims[x]:"-") <<sep[x];
  }
  else
  {
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<(_LastDelimFound[x] ? 1:0) <<sep[x];

    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<(_LastDelimFound[x] ? _LastDelimPos[x]:0) <<sep[x];

    sep[2] = ';';
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      Out_ <<(_LastDelimFound[x] ? _LastDelims[x]:"-") <<sep[x];
  }
    
  Out_ <<endl;
  if (ColHead_)
    Out_ <<endl;
}

/****************************************************************************/
MEMORYOPS_DEFN(TextFileLineData)

/****************************************************************************/
//			Line Data Record Class
/****************************************************************************/
TextFileSearchData::TextFileSearchData(int ArrayIndex_, int StringIndex_, int DelimType_):
_ArrayIndex(ArrayIndex_),
_StringIndex(StringIndex_),
_DelimType(DelimType_)
{}

/****************************************************************************/
TextFileSearchData::~TextFileSearchData()
{}

/****************************************************************************/
MEMORYOPS_DEFN(TextFileSearchData)

/****************************************************************************/
//			Text File Line Reader Class
/****************************************************************************/
TextFileLineBuffer::TextFileLineBuffer(FileReader* Freader_,
                                       short IoMode_, short BufLen_):
_TextBuffer(BufLen_ ? (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * (BufLen_+1)):
                      (char*)RawAllocateWith(MEMMATRIX, sizeof(char) * (MAXTEXTBUFFER+1))),
_BufferFull(false),
_LineFitInBuffer(true),
_SingleDelimSetFound(false),
_FreaderCreated(Freader_ ? false:true),

_IoMode(IoMode_),
_DefaultDelim(TextFileLineBuffer::LAST_SET),
_LineEmpty(0),
_CharsRead(0),
_BufSize(BufLen_ ? BufLen_:MAXTEXTBUFFER),
_ReadAttemptNumber(0),
_TimesLimitReached(0),
_ReadTimesRequired(0),

_NlChars(NewString("\n\r\f\v")),
_WsChars(NewString(" \t\n\r\f\v")),
_WsxNlChars(NewString(" \t")),

_GivenInputLine(NULL),
_GivenLineSet(false),

_WsFound(false),
_EolFound(false),
_EofFound(false),
_WsLinePos(0),
_EndLinePos(0),
_EndFilePos(0),

_FirstDelims((char**)RawAllocateWith(MEMMATRIX, sizeof(char*) * MAXDELIMS)),
_LastDelims((char**)RawAllocateWith(MEMMATRIX, sizeof(char*) * MAXDELIMS)),

_File(Freader_ ? Freader_:(new FileReader)),
_LineData(NULL),
_SearchData(NULL),
_FilePos(new FilePosition),
_ColStk(new SimpleStack<long>),
_ReadLenQueue(new SimpleQueue<long>),
_ReadBufQueue(new SimpleQueue<long>),
_DelimStk(new SimpleDeque<DelimInfo>)
{
  _LastDelims = DelimInfo::InitDelimsArray(_LastDelims, MAXDELIMS);
  memset(_LastDelimFound, 0, sizeof(bool) * MAXDELIMS);
  memset(_LastDelimOptional, 0, sizeof(bool) * MAXDELIMS);
  memset(_LastDelimPos, 0, sizeof(long) * MAXDELIMS);

  _FirstDelims = DelimInfo::InitDelimsArray(_FirstDelims, MAXDELIMS);
  memset(_FirstDelimFound, 0, sizeof(bool) * MAXDELIMS);
  memset(_FirstDelimOptional, 0, sizeof(bool) * MAXDELIMS);
  memset(_FirstDelimPos, 0, sizeof(long) * MAXDELIMS);
}

/****************************************************************************/
TextFileLineBuffer::~TextFileLineBuffer()
{
  ::RawDeleteArray(_TextBuffer);
  _TextBuffer = NULL;

  if (_GivenLineSet)
  {
    ::RawDeleteArray(_GivenInputLine);
    _GivenInputLine = NULL;
    _GivenLineSet = false;
  }

  int x;
  for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
  {
    ::RawDeleteArray(_FirstDelims[x]);
    ::RawDeleteArray(_LastDelims[x]);

    _FirstDelims[x] = NULL;
    _LastDelims[x] = NULL;
  }

  ::RawDeleteArray(_FirstDelims);
  ::RawDeleteArray(_LastDelims);
  _FirstDelims = NULL;
  _LastDelims = NULL;

  ::RawDeleteArray(_NlChars);
  ::RawDeleteArray(_WsChars);
  ::RawDeleteArray(_WsxNlChars);
  _NlChars = NULL;
  _WsChars = NULL;
  _WsxNlChars = NULL;

  if (_FreaderCreated)
    delete _File;

  _ReadLenQueue->DeleteAllAndData();
  _ReadBufQueue->DeleteAllAndData();
  _DelimStk->DeleteAllAndData();

  delete _LineData;
  delete _SearchData;
  delete _FilePos;
  delete _ColStk;  
  delete _ReadLenQueue;
  delete _ReadBufQueue;
  delete _DelimStk;
}

/****************************************************************************/
const char* TextFileLineBuffer::TagTypeStr(const char* EscStr_)
{
  if (!EscStr_)
    return NULL;

  return
  (
    (StriComp(EscStr_, "&amp;") == 0)          ? "&":
    (StriComp(EscStr_, "&lt;") == 0)           ? "<":
    (StriComp(EscStr_, "&gt;") == 0)           ? ">":
    (StriComp(EscStr_, "&quot;") == 0)         ? "\"":
    (StriComp(EscStr_, "&tagop;") == 0)        ? "<":
    (StriComp(EscStr_, "&tagcl;") == 0)        ? ">":
    (StriComp(EscStr_, "&endtagop;") == 0)     ? "</":
    (StriComp(EscStr_, "&endtagcl;") == 0)     ? ">":
    (StriComp(EscStr_, "&emptytagop;") == 0)   ? "<":
    (StriComp(EscStr_, "&emptytagcl;") == 0)   ? "/>":
    (StriComp(EscStr_, "&scriptop;") == 0)     ? "<?":
    (StriComp(EscStr_, "&scriptcl;") == 0)     ? "?>":
    (StriComp(EscStr_, "&doctypeop;") == 0)    ? "<!":
    (StriComp(EscStr_, "&doctypecl;") == 0)    ? ">":
    (StriComp(EscStr_, "&commentop;") == 0)    ? "<!--":
    (StriComp(EscStr_, "&commentcl;") == 0)    ? "-->":
    (StriComp(EscStr_, "&nestedcmt1op;") == 0) ? "<!--":
    (StriComp(EscStr_, "&nestedcmt1cl;") == 0) ? ">":
    (StriComp(EscStr_, "&nestedcmt2op;") == 0) ? "<!":
    (StriComp(EscStr_, "&nestedcmt2cl;") == 0) ? "-->":
    (StriComp(EscStr_, "&copcmtblkop;") == 0)  ? "/*":
    (StriComp(EscStr_, "&copcmtblkcl;") == 0)  ? "->":
    (StriComp(EscStr_, "&cclcmtblkop;") == 0)  ? "<-":
    (StriComp(EscStr_, "&cclcmtblkcl;") == 0)  ? "*/":
    (StriComp(EscStr_, "&cppcmtlnop;") == 0)   ? "//":
    (StriComp(EscStr_, "&cppcmtlncl;") == 0)   ? "=>":EscStr_
  );  
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::SetFileReader(FileReader* Freader_)
{
  if (Freader_)
  {
    if (_FreaderCreated)
      delete _File;

    _File = Freader_;
    _FreaderCreated = false;
  }
  
  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::SetInputLine(const char* Input_)
{
  if (Input_)
  {
    _GivenInputLine = NewString(Input_);
    _GivenLineSet = true;
  }

  return *this;
}

/****************************************************************************/
bool TextFileLineBuffer::IsEmptyLine(const char* InputStr_, int len)
{
  if (len == 0 || !InputStr_)
  {
    _LineEmpty = COMPLETE;
    return true;
  }

  int x;
  bool spc = false;
  for (x = 0; x < len && InputStr_[x] && (spc=isspace(InputStr_[x])); x++);

  _LineEmpty = spc          ? INCOMPLETE:
               InputStr_[x] ? NONE:COMPLETE;

  return (_LineEmpty != NONE);
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::SetNewLineChars(const char* Sep_)
{
  if (Sep_)
  {
    ::RawDeleteArray(_NlChars);
    _NlChars = NewString(Sep_);
  }
    
  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::SetWhiteSpaceChars(const char* Sep_)
{
  if (Sep_)
  {
    ::RawDeleteArray(_WsChars);
    _WsChars = NewString(Sep_);
  }
    
  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::SetDefaultToDelims(int Set_)
{
  _DefaultDelim = (Set_ == FIRST_SET) ? FIRST_SET:
                  (Set_ == LAST_SET)  ? LAST_SET:0;

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::SetFirstDelims(long Pos_, const char* Str_, bool Optional_)
{
  if (0 <= Pos_ && Pos_ <= 2 && Str_)
  {
    _FirstDelims[Pos_] = NewString(Str_);
    _FirstDelimOptional[Pos_] = Optional_;
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::SetLastDelims(long Pos_, const char* Str_, bool Optional_)
{
  if (0 <= Pos_ && Pos_ <= 2 && Str_)
  {
    _LastDelims[Pos_] = NewString(Str_);
    _LastDelimOptional[Pos_] = Optional_;
  }

  return *this;
}

/****************************************************************************/
int TextFileLineBuffer::FirstDelimFound(long Pos_, bool All_)
{
  int ret = 0;

  if (!All_ && 0 <= Pos_ && Pos_ <= 2)
    ret = _FirstDelimFound[Pos_] ? COMPLETE:NONE;
  else if (All_)
    ret = DelimSetComplete(_FirstDelims);

  return ret;
}

/****************************************************************************/
int TextFileLineBuffer::LastDelimFound(long Pos_, bool All_)
{
  int ret = 0;

  if (!All_ && 0 <= Pos_ && Pos_ <= 2)
    ret = _LastDelimFound[Pos_] ? COMPLETE:NONE;
  else if (All_)
    ret = DelimSetComplete(_LastDelims);

  return ret;
}

/****************************************************************************/
long TextFileLineBuffer::FirstDelimPos(long Pos_)
{
  if (0 <= Pos_ && Pos_ <= 2)
    return _FirstDelimPos[Pos_];
}

/****************************************************************************/
long TextFileLineBuffer::LastDelimPos(long Pos_)
{
  if (0 <= Pos_ && Pos_ <= 2)
    return _LastDelimPos[Pos_];
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ClearFirstDelims(long Pos_, bool All_)
{
  if (!All_ && 0 <= Pos_ && Pos_ <= 2)
  {
    ::RawDeleteArray(_FirstDelims[Pos_]);
    _FirstDelims[Pos_] = NULL;
  }
  else if (All_)
  {
    long x;
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
    {
      ::RawDeleteArray(_FirstDelims[x]);
      _FirstDelims[x] = NULL;
    }
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ClearLastDelims(long Pos_, bool All_)
{
  if (!All_ && 0 <= Pos_ && Pos_ <= 2)
  {
    ::RawDeleteArray(_LastDelims[Pos_]);
    _LastDelims[Pos_] = NULL;
  }
  else if (All_)
  {
    long x;
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
    {
      ::RawDeleteArray(_LastDelims[x]);
      _LastDelims[x] = NULL;
    }
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ClearAllDelims()
{
  ClearFirstDelims(0, true);
  ClearLastDelims(0, true);
  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ResetFirstDelims(long Pos_, bool All_)
{
  if (!All_ && 0 <= Pos_ && Pos_ <= 2)
  {
    _FirstDelimFound[Pos_] = false;
    _FirstDelimPos[Pos_] = 0;
  }
  else if (All_)
  {
    memset(_FirstDelimFound, 0, sizeof(bool) * MAXDELIMS);
    memset(_FirstDelimPos, 0, sizeof(long) * MAXDELIMS);
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ResetLastDelims(long Pos_, bool All_)
{
  if (!All_ && 0 <= Pos_ && Pos_ <= 2)
  {
    _LastDelimFound[Pos_] = false;
    _LastDelimPos[Pos_] = 0;
  }
  else if (All_)
  {
    memset(_LastDelimFound, 0, sizeof(bool) * MAXDELIMS);
    memset(_LastDelimPos, 0, sizeof(long) * MAXDELIMS);
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ResetAllDelims()
{
  ResetFirstDelims(0, true);
  ResetLastDelims(0, true);
  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ResetLineData(bool ForceReset_)
{
  if (_EolFound || ForceReset_)
  {
    ResetAllDelims();

    _WsFound = false;
    _EolFound = false;
    _BufferFull = false;
    _SingleDelimSetFound = false;
    _LineFitInBuffer = true;

    _CharsRead = 0;
    _LineEmpty = 0;
    _ReadAttemptNumber = 0;
    _TimesLimitReached = 0;
    _ReadTimesRequired = 0;
    _DelimSetTypeFound = 0;
  
    _ReadLenQueue->DeleteAllAndData();
    _ReadBufQueue->DeleteAllAndData();
    _DelimStk->DeleteAllAndData();
  }
  
  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ResetBufferStatus(bool ForceReset_)
{
  if (_BufferFull || _EolFound || ForceReset_)
  {
    size_t x = 0;
    size_t sz = _ReadLenQueue->Size();
    long* Vect_ = (long*)RawAllocateWith(MEMMATRIX, sizeof(long) * (sz+1));
    long* Val_;
  
    Vect_[x++] = sz;

    while (!_ReadLenQueue->Empty())
    {
      Val_ = _ReadLenQueue->Dequeue();

      if (Val_)
      {
        Vect_[x++] = *Val_;
        delete Val_;
      }
    }

    _ReadAttemptNumber = 0;
    _ReadLenQueue->DeleteAllAndData();
    _ReadBufQueue->Enqueue(Vect_);
    _BufferFull = false;

    // defaulting to last delims
    if (_EolFound)
      DetermineDelimSet((_LastDelims && _DefaultDelim != FIRST_SET) ? _LastDelims:_FirstDelims);
  }

  return *this;
}

/****************************************************************************/
int TextFileLineBuffer::DetermineDelimSet(char** Delims_)
{
  int DsComp1_ = DelimSetComplete(_FirstDelims);
  int DsComp2_ = DelimSetComplete(_LastDelims);
  
  _SingleDelimSetFound = (DsComp2_ && DsComp2_ > DsComp1_) ||
                         (DsComp1_ && DsComp1_ > DsComp2_);
  _DelimSetTypeFound = _SingleDelimSetFound ?
                          ((DsComp2_ > DsComp1_) ? LAST_SET:FIRST_SET):NONE;

  if (_DelimSetTypeFound == FIRST_SET)
  {
    if (Delims_ == _LastDelims && _DefaultDelim == LAST_SET)
      SetMatchingDelims(_FirstDelims);
    else
    {
      _SingleDelimSetFound = !(DsComp1_ && DsComp2_) &&
                              (DsComp1_ || DsComp2_);
      _DelimSetTypeFound = _SingleDelimSetFound ?
                              (DsComp2_ ? LAST_SET:FIRST_SET):NONE;
    }
  }
  else if (_DelimSetTypeFound == LAST_SET)
  {
    if (Delims_ == _FirstDelims && _DefaultDelim == FIRST_SET)
      SetMatchingDelims(_LastDelims);
    else
    {
      _SingleDelimSetFound = !(DsComp1_ && DsComp2_) &&
                              (DsComp1_ || DsComp2_);
      _DelimSetTypeFound = _SingleDelimSetFound ?
                              (DsComp2_ ? LAST_SET:FIRST_SET):NONE;
    }
  }

  return _DelimSetTypeFound;
}

/****************************************************************************/
int TextFileLineBuffer::NumDelims(char** Delims_)
{
  int x;
  int num = 0;
  
  if (Delims_ == _LastDelims)
  {
    for (x = num = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      if (_LastDelims[x])
        ++num;
  }
  else if (Delims_ == _FirstDelims)
  {
    for (x = num = 0; x < TextFileLineBuffer::MAXDELIMS; x++)
      if (_FirstDelims[x])
        ++num;
  }

  return num;
}

/****************************************************************************/
bool TextFileLineBuffer::SetMatchingDelims(char** Delims_)
{
  int x, y;
  size_t len = 0;
  size_t len1;
  size_t len2;
  bool ret = false;
  bool fnd = false;
  bool match = false;
  bool dlmfnd = true;

  if (Delims_ == _FirstDelims)
  {
    for (x = y = 0; x < TextFileLineBuffer::MAXDELIMS &&
         _FirstDelims[x] && _LastDelims[y] && (x == 0 || ret);)
    {
      len1 = ::SafeStrLen(_FirstDelims[x]);
      len2 = ::SafeStrLen(_LastDelims[y]);
      len = (len1 == len2) ? len1:0;

      dlmfnd = dlmfnd && _FirstDelimFound[x];
      fnd = (x == 0 || ret) && len > 0 &&
            ((_FirstDelimFound[x] && ::strcmp(_FirstDelims[x], _LastDelims[y]) == 0));
      ret = fnd || _FirstDelimOptional[x];

      if (!fnd)
      {
        if (_FirstDelimOptional[x])
          x++;
        else
        {
          x++;
          y++;
        }
      }
      else
      {
        x++;
        y++;
      }
    }

    if (ret && (!dlmfnd || (NumDelims(_FirstDelims) == NumDelims(_LastDelims))))
    {
      for (x = y = 0; x < TextFileLineBuffer::MAXDELIMS &&
           _FirstDelims[x] && _LastDelims[y] && (x == 0 || ret);)
      {
        len1 = ::SafeStrLen(_FirstDelims[x]);
        len2 = ::SafeStrLen(_LastDelims[y]);
        len = (len1 == len2) ? len1:0;

        fnd = (x == 0 || ret) && len > 0 &&
              ((_FirstDelimFound[x] && ::strcmp(_FirstDelims[x], _LastDelims[y]) == 0));
        ret = fnd || _FirstDelimOptional[x];

        if (!fnd)
        {
          if (_FirstDelimOptional[x])
            x++;
          else
          {
            x++;
            y++;
          }
        }
        else if (_FirstDelims[x] && _FirstDelimFound[x])
        {
          _LastDelimPos[y] = _FirstDelimPos[x];
          _FirstDelimPos[x] = 0;
          _LastDelimFound[y++] = _FirstDelimFound[x];
          _FirstDelimFound[x++] = false;
        }
      }

      if (_DelimSetTypeFound == FIRST_SET)
        _DelimSetTypeFound = LAST_SET;
    }
  }
  else if (Delims_ == _LastDelims)
  {
    for (x = y = 0; x < TextFileLineBuffer::MAXDELIMS &&
         _LastDelims[x] && _FirstDelims[y] && (x == 0 || ret);)
    {
      len1 = ::SafeStrLen(_FirstDelims[y]);
      len2 = ::SafeStrLen(_LastDelims[x]);
      len = (len1 == len2) ? len1:0;

      dlmfnd = dlmfnd && _LastDelimFound[x];
      fnd = (x == 0 || ret) && len > 0 &&
            ((_LastDelimFound[x] && ::strcmp(_LastDelims[x], _FirstDelims[y]) == 0));
      ret = fnd || _LastDelimOptional[x];

      if (!fnd)
      {
        if (_LastDelimOptional[x])
          x++;
        else
        {
          x++;
          y++;
        }
      }
      else
      {
        x++;
        y++;
      }
    }

    if (ret && (!dlmfnd || (NumDelims(_FirstDelims) == NumDelims(_LastDelims))))
    {
      for (x = y = 0; x < TextFileLineBuffer::MAXDELIMS &&
           _LastDelims[x] && _FirstDelims[y] && (x == 0 || ret);)
      {
        len1 = ::SafeStrLen(_FirstDelims[y]);
        len2 = ::SafeStrLen(_LastDelims[x]);
        len = (len1 == len2) ? len1:0;

        fnd = (x == 0 || ret) && len > 0 &&
              ((_LastDelimFound[x] && ::strcmp(_LastDelims[x], _FirstDelims[y]) == 0));
        ret = fnd || _LastDelimOptional[x];

        if (!fnd)
        {
          if (_LastDelimOptional[x])
            x++;
          else
          {
            x++;
            y++;
          }
        }
        else if (_LastDelims[x] && _LastDelimFound[x])
        {
          _FirstDelimPos[y] = _LastDelimPos[x];
          _LastDelimPos[x] = 0;
          _FirstDelimFound[y++] = _LastDelimFound[x];
          _LastDelimFound[x++] = false;
        }
      }

      if (_DelimSetTypeFound == LAST_SET)
        _DelimSetTypeFound = FIRST_SET;
    }
  }

  return ret;
}

/****************************************************************************/
void TextFileLineBuffer::CountColsRows(const char* Buf_, long Len_, const char* Delim_)
{
  long x;
  for (x = 0; x < Len_; x++)
    CountColsRows(Buf_[x]);

  if (Delim_)
    CountColsRows(*Delim_);
}

/****************************************************************************/
void TextFileLineBuffer::CountColsRows(const char* Buf_, const char* Delim_)
{
  if (Buf_ && Delim_)
  {
    size_t x;
    size_t len = strlen(Buf_);
  
    if (*Delim_ == '\n')
    {
      if (_FilePos->ColPos() || _FilePos->RowPos())
      {
        _FilePos->IncRowPos();
        _ColStk->Push(new long(_FilePos->ColPos()));
        _FilePos->ResetColPos();
      }
        
      for (_FilePos->ResetColPos(); _FilePos->ColPos() < len; _FilePos->IncColPos())
        CountColsRows(Buf_[_FilePos->ColPos()]);
    }
    else
    {
      for (x = 0; x < len; x++)
        CountColsRows(Buf_[x]);

      CountColsRows(*Delim_);
    }
  }
}

/****************************************************************************/
char TextFileLineBuffer::CountColsRows(char ch)
{
  if (ch != 0)
    if (ch == '\n')
    {
      _FilePos->IncRowPos();
      _ColStk->Push(new long(_FilePos->ColPos()));
      _FilePos->ResetColPos();
    }
    else
      _FilePos->IncColPos();

  return ch;
}

/****************************************************************************/
char TextFileLineBuffer::UnCountColsRows(char ch)
{
  long* valp_ = NULL;
  
  if (ch != 0)
    if (ch == '\n')
    {
      _FilePos->DecRowPos();
      valp_ = _ColStk->Pop();
      
      if (valp_)
      {
        _FilePos->SetColPos(*valp_);
        ::Delete(valp_);
      }
    }
    else    
      _FilePos->DecColPos();

  return ch;
}

/****************************************************************************/
int TextFileLineBuffer::DelimSetComplete(char** Delims_)
{
  int x;
  bool ret = false;
  bool match = false;

  if (Delims_ == _FirstDelims)
  {
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS && _FirstDelims[x] && (x == 0 || ret); x++)
    {
      match = (x == 0 || match) && _FirstDelimFound[x];
      ret = (x == 0 || ret) && (_FirstDelimFound[x] || _FirstDelimOptional[x]);
    }
  }
  else if (Delims_ == _LastDelims)
  {
    for (x = 0; x < TextFileLineBuffer::MAXDELIMS && _LastDelims[x] && (x == 0 || ret); x++)
    {
      match = (x == 0 || match) && _LastDelimFound[x];
      ret = (x == 0 || ret) && (_LastDelimFound[x] || _LastDelimOptional[x]);
    }
  }

  return (match ? COMPLETE:
          ret   ? INCOMPLETE:NONE);
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::DumpLineRecord(bool ColHead_)
{
  if (_LineData)
  {
    _LineData->DumpData((_IoMode & FILE_OUTPUT) ?
                            _File->GetOutStream():cout, ColHead_);

    _ReadBufQueue->DeleteAllAndData();
    _DelimStk->DeleteAllAndData();
    
    delete _LineData;
    _LineData = NULL;
  }

  return *this;
}

/****************************************************************************/
int TextFileLineBuffer::SearchForFirstDelims(char ch, long PrevCol_, int& ArrayIndex_,
                                             int& StringIndex_, int& DelimType_, bool& Found_)
{
  if (!_FirstDelimFound[0] &&
      _FirstDelims[0] && _FirstDelims[0][0] == ch &&
        ((PrevCol_ < _FirstDelimPos[1] || !_FirstDelimFound[1]) &&
         (PrevCol_ < _FirstDelimPos[2] || !_FirstDelimFound[2])))
  {
    if (strlen(_FirstDelims[0]) > 1)
    {
      StringIndex_ = 1;
      ArrayIndex_ = 0;
      DelimType_ = STARTING;
    }
    else
    {
      StringIndex_ = 0;
      _FirstDelimFound[0] = true;
      _FirstDelimPos[0] = PrevCol_;
    }

    Found_ = true;
  }
  else if (_FirstDelimFound[0] &&
           _FirstDelims[0] && _FirstDelims[0][0] == ch &&
             ((!_FirstDelimFound[1] && !_FirstDelimOptional[1]) ||
              (!_FirstDelimFound[2] && !_FirstDelimOptional[2])))
  {
    if (strlen(_FirstDelims[0]) > 1)
    {
      StringIndex_ = 1;
      ArrayIndex_ = 0;
      DelimType_ = STARTING;
    }
    else
      StringIndex_ = 0;

    Found_ = true;
    return FIRST_SET;
  }
  else if (!_FirstDelimFound[1] &&
           _FirstDelims[1] && _FirstDelims[1][0] == ch &&
             ((_FirstDelimPos[0] < PrevCol_ && _FirstDelimFound[0]) ||
              (!_FirstDelimFound[0] && _FirstDelimOptional[0])) &&
             (PrevCol_ < _FirstDelimPos[2] || !_FirstDelimFound[2]))
  {
    if (strlen(_FirstDelims[1]) > 1)
    {
      StringIndex_ = 1;
      ArrayIndex_ = 1;
      DelimType_ = STARTING;
    }
    else
    {
      StringIndex_ = 0;
      _FirstDelimFound[1] = true;
      _FirstDelimPos[1] = PrevCol_;
    }

    Found_ = true;
  }
  else if (!_FirstDelimFound[2] &&
           _FirstDelims[2] && _FirstDelims[2][0] == ch &&
             ((_FirstDelimPos[0] < PrevCol_ && _FirstDelimFound[0]) ||
              (!_FirstDelimFound[0] && _FirstDelimOptional[0])) &&
             ((_FirstDelimPos[1] < PrevCol_ && _FirstDelimFound[1]) ||
              (!_FirstDelimFound[1] && _FirstDelimOptional[1])))
  {
    if (strlen(_FirstDelims[2]) > 1)
    {
      StringIndex_ = 1;
      ArrayIndex_ = 2;
      DelimType_ = STARTING;
    }
    else
    {
      StringIndex_ = 0;
      _FirstDelimFound[2] = true;
      _FirstDelimPos[2] = PrevCol_;
    }

    Found_ = true;
  }
  else
    Found_ = false;

  return NONE;
}

/****************************************************************************/
int TextFileLineBuffer::SearchForLastDelims(char ch, long PrevCol_, int& ArrayIndex_,
                                            int& StringIndex_, int& DelimType_, bool& Found_)
{
  if (!_LastDelimFound[0] &&
      _LastDelims[0] && _LastDelims[0][0] == ch &&
        ((PrevCol_ < _LastDelimPos[1] || !_LastDelimFound[1]) &&
         (PrevCol_ < _LastDelimPos[2] || !_LastDelimFound[2])))
  {
    if (strlen(_LastDelims[0]) > 1)
    {
      StringIndex_ = 1;
      ArrayIndex_ = 0;
      DelimType_ = ENDING;
    }
    else
    {
      StringIndex_ = 0;
      _LastDelimFound[0] = true;
      _LastDelimPos[0] = PrevCol_;
    }

    Found_ = true;
  }
  else if (_LastDelimFound[0] &&
           _LastDelims[0] && _LastDelims[0][0] == ch &&
             ((!_LastDelimFound[1] && !_LastDelimOptional[1]) ||
              (!_LastDelimFound[2] && !_LastDelimOptional[2])))
  {
    if (strlen(_LastDelims[0]) > 1)
    {
      StringIndex_ = 1;
      ArrayIndex_ = 0;
      DelimType_ = ENDING;
    }
    else
      StringIndex_ = 0;

    Found_ = true;
    return LAST_SET;
  }
  else if (!_LastDelimFound[1] &&
           _LastDelims[1] && _LastDelims[1][0] == ch &&
             ((_LastDelimPos[0] < PrevCol_ && _LastDelimFound[0]) ||
              (!_LastDelimFound[0] && _LastDelimOptional[0])) &&
             (PrevCol_ < _LastDelimPos[2] || !_LastDelimFound[2]))
  {
    if (strlen(_LastDelims[1]) > 1)
    {
      StringIndex_ = 1;
      ArrayIndex_ = 1;
      DelimType_ = ENDING;
    }
    else
    {
      StringIndex_ = 0;
      _LastDelimFound[1] = true;
      _LastDelimPos[1] = PrevCol_;
    }

    Found_ = true;
  }
  else if (!_LastDelimFound[2] &&
           _LastDelims[2] && _LastDelims[2][0] == ch &&
             ((_LastDelimPos[0] < PrevCol_ && _LastDelimFound[0]) ||
              (!_LastDelimFound[0] && _LastDelimOptional[0])) &&
             ((_LastDelimPos[1] < PrevCol_ && _LastDelimFound[1]) ||
              (!_LastDelimFound[1] && _LastDelimOptional[1])))
  {
    if (strlen(_LastDelims[2]) > 1)
    {
      StringIndex_ = 1;
      ArrayIndex_ = 2;
      DelimType_ = ENDING;
    }
    else
    {
      StringIndex_ = 0;
      _LastDelimFound[2] = true;
      _LastDelimPos[2] = PrevCol_;
    }

    Found_ = true;
  }
  else
    Found_ = false;

  return NONE;
}

/****************************************************************************/
int TextFileLineBuffer::SearchForDelims(char** Delims_, char ch, long PrevCol_,
                                        int& ArrayIndex_, int& StringIndex_, int& DelimType_)
{
  int x, y;
  int ret = NONE;
  int len1 = ::SafeStrLen(_FirstDelims[0]);
  int len2 = ::SafeStrLen(_LastDelims[0]);
  
  bool Found_ = false;
  bool done = false;  
  bool fnd;

  if (StringIndex_ > 0)
  {
    if (DelimType_ == STARTING)
    {
      if (_FirstDelims[ArrayIndex_] && _FirstDelims[ArrayIndex_][StringIndex_] == ch)
      {
        if (strlen(_FirstDelims[ArrayIndex_]+StringIndex_) > 1)
          ++StringIndex_;
        else
        {
          long DelimStart_ = PrevCol_ - StringIndex_;
          _FirstDelimFound[ArrayIndex_] = true;
          _FirstDelimPos[ArrayIndex_] = (DelimStart_ >= 0) ? DelimStart_:PrevCol_;
          StringIndex_ = ArrayIndex_ = 0;
          DelimType_ = NONE;
        }

        done = true;
      }
      else if (DelimSetComplete(_LastDelims) == 0)
      {
        fnd = false;
        for (x = y = ArrayIndex_; y >= 0 && !_LastDelims[y] ||
             !(fnd=strncmp(_FirstDelims[x], _LastDelims[y], StringIndex_) == 0);)
          if (_FirstDelimOptional[x])
            --y;
          else
            break;

        if (fnd && _LastDelims[y])
        {
          ArrayIndex_ = y;
          DelimType_ == ENDING;

          if (strlen(_LastDelims[y]) > StringIndex_ &&
              _LastDelims[y][StringIndex_] == ch)
          {
            if (strlen(_LastDelims[ArrayIndex_]+StringIndex_) > 1)
              ++StringIndex_;
            else
            {
              long DelimStart_ = PrevCol_ - StringIndex_;
              _LastDelimFound[ArrayIndex_] = true;
              _LastDelimPos[ArrayIndex_] = (DelimStart_ >= 0) ? DelimStart_:PrevCol_;
              StringIndex_ = ArrayIndex_ = 0;
              DelimType_ = NONE;
            }

            done = true;
          }
          else if (strlen(_LastDelims[y]) == StringIndex_)
          {
            long DelimStart_ = PrevCol_ - StringIndex_;
            _LastDelimFound[ArrayIndex_] = true;
            _LastDelimPos[ArrayIndex_] = (DelimStart_ >= 0) ? DelimStart_:PrevCol_;
            StringIndex_ = ArrayIndex_ = 0;
            DelimType_ = NONE;
            done = true;
          }
        }
      }

      if (!done)
      {
        StringIndex_ = ArrayIndex_ = 0;
        DelimType_ = NONE;
      }
    }
    else if (DelimType_ == ENDING)
    {
      if (_LastDelims[ArrayIndex_] && _LastDelims[ArrayIndex_][StringIndex_] == ch)
      {
        if (strlen(_LastDelims[ArrayIndex_]+StringIndex_) > 1)
          ++StringIndex_;
        else
        {
          long DelimStart_ = PrevCol_ - StringIndex_;
          _LastDelimFound[ArrayIndex_] = true;
          _LastDelimPos[ArrayIndex_] = (DelimStart_ >= 0) ? DelimStart_:PrevCol_;
          StringIndex_ = ArrayIndex_ = 0;
          DelimType_ = NONE;
        }

        done = true;
      }
      else if (DelimSetComplete(_FirstDelims) == 0)
      {
        fnd = false;
        for (x = y = ArrayIndex_; y >= 0 && !_FirstDelims[y] ||
             !(fnd=strncmp(_LastDelims[x], _FirstDelims[y], StringIndex_) == 0);)
          if (_LastDelimOptional[x])
            --y;
          else
            break;
        
        if (fnd && _FirstDelims[y])
        {
          ArrayIndex_ = y;
          DelimType_ == STARTING;

          if (strlen(_FirstDelims[y]) > StringIndex_ &&
              _FirstDelims[y][StringIndex_] == ch)
          {
            if (strlen(_FirstDelims[ArrayIndex_]+StringIndex_) > 1)
              ++StringIndex_;
            else
            {
              long DelimStart_ = PrevCol_ - StringIndex_;
              _FirstDelimFound[ArrayIndex_] = true;
              _FirstDelimPos[ArrayIndex_] = (DelimStart_ >= 0) ? DelimStart_:PrevCol_;
              StringIndex_ = ArrayIndex_ = 0;
              DelimType_ = NONE;
            }

            done = true;
          }
          else if (strlen(_FirstDelims[y]) == StringIndex_)
          {
            long DelimStart_ = PrevCol_ - StringIndex_;
            _FirstDelimFound[ArrayIndex_] = true;
            _FirstDelimPos[ArrayIndex_] = (DelimStart_ >= 0) ? DelimStart_:PrevCol_;
            StringIndex_ = ArrayIndex_ = 0;
            DelimType_ = NONE;
            done = true;
          }
        }
      }

      if (!done)
      {
        StringIndex_ = ArrayIndex_ = 0;
        DelimType_ = NONE;
      }
    }
  }
  else if (Delims_ == _LastDelims)
  {
    if (_FirstDelimFound[0] || !(len2 > len1 || strncmp(_LastDelims[0], _FirstDelims[0], len2)))
    {
      ret = SearchForFirstDelims(ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_, Found_);
      if (!Found_)
        ret = SearchForLastDelims(ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_, Found_);
    }
    else
    {
      ret = SearchForLastDelims(ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_, Found_);
      if (!Found_)
        ret = SearchForFirstDelims(ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_, Found_);
    }
  }
  else if (Delims_ == _FirstDelims)
  {
    if (_LastDelimFound[0] || !(len1 > len2 || strncmp(_FirstDelims[0], _LastDelims[0], len1)))
    {
      ret = SearchForLastDelims(ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_, Found_);
      if (!Found_)
        ret = SearchForFirstDelims(ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_, Found_);
    }
    else
    {
      ret = SearchForFirstDelims(ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_, Found_);
      if (!Found_)
        ret = SearchForLastDelims(ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_, Found_);
    }  
  }

  return ret;
}

/****************************************************************************/
//  Text file line record format:
//
//    <RowPos>,<LineFitInBuffer>:<SingleDelimSetFound>,<DelimSetTypeFound>:
//    <TimesLimitReached>:<ReadTimesRequired>:<EolFound>,<EolPos>:
//    <EofFound>,<EofPos>:
//    <LenQueueSize>:<va1_size><va1_len0>:<va1_len1>...:<va1_len...x>:
//                   <va2_size><va2_len0>:<va2_len1>...:<va2_len...x>:
//                   ...
//                   <vax_size><vax_len0>:<vax_len1>...:<vax_len...x>:
//    <DelimQueueSize>:<FirstDelimFound>[0],[1],[2]:
//                     <FirstDelimPos>[0],[1],[2]:
//                     <FirstDelims>[0],[1],[2]:
//    ...
//    <MidDelimFound...>[0],[1],[2]:
//    <MidDelimPos...>[0],[1],[2]:
//    <MidDelims...>[0],[1],[2]:
//    ...
//    <LastDelimFound>[0],[1],[2]:
//    <LastDelimPos>[0],[1],[2]:
//    <LastDelims>[0],[1],[2]
//
bool TextFileLineBuffer::While_Read(char* Input_)
{
  bool InputArg_ = Input_ != NULL;
  bool ReadDone_ = false;
  bool ReadOk_ = false;

  if (_File && Input_ && !_EolFound && !_EofFound)
  {
    while (!_EolFound && !_EofFound)
    {
      if (InputArg_)
        *this >>Input_;
      else
        Read();

      if (!ReadDone_)
      {
        ReadDone_ = _CharsRead != 0;
        ReadOk_ = ReadOk_ || ReadDone_;
      }
        
      if (ReadDone_ && (_BufferFull || _EolFound))
      {
        ResetBufferStatus();
        ReadDone_ = false;
      }
    }

    if (ReadOk_)
      _LineData = new TextFileLineData(*this);
  }

  return (!_EofFound);
}

/****************************************************************************/
bool TextFileLineBuffer::While_ReadUntil(const char* CharSet_, bool InSet_, bool PutBack_)
{
  bool ReadDone_ = false;
  bool ReadOk_ = false;

  if (_File && !_EolFound && !_EofFound)
  {
    while (!_EolFound && !_EofFound)
    {
      ReadUntil(CharSet_, InSet_, PutBack_);

      if (!ReadDone_)
      {
        ReadDone_ = _CharsRead != 0;
        ReadOk_ = ReadOk_ || ReadDone_;
      }

      if (ReadDone_ && (_BufferFull || _EolFound))
      {
        ResetBufferStatus();
        ReadDone_ = false;
      }
    }

    if (ReadOk_)
      _LineData = new TextFileLineData(*this);
  }

  return (!_EofFound);
}

/****************************************************************************/
bool TextFileLineBuffer::While_ReadLine()
{
  bool ReadDone_ = false;
  bool ReadOk_ = false;

  if (_File && !_EolFound && !_EofFound)
  {
    while (!_EolFound && !_EofFound)
    {
      ReadLine();

      if (!ReadDone_)
      {
        ReadDone_ = _CharsRead != 0;
        ReadOk_ = ReadOk_ || ReadDone_;
      }

      if (ReadDone_ && (_BufferFull || _EolFound))
      {
        ResetBufferStatus();
        ReadDone_ = false;
      }
    }

    if (ReadOk_)
      _LineData = new TextFileLineData(*this);
  }

  return (!_EofFound);
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::operator >> (char* InputStr_)
{
  if (_File && InputStr_ && !(_EolFound || _BufferFull) && !_EofFound)
  {
    int ArrayIndex_;
    int StringIndex_;
    int DelimType_;
    
    char ch;
    long PrevCol_;

    int x;
    int Collide_ = NONE;
    bool Dsc_ = false;
    bool WsFound_ = false;
    const char* Cset_ = _WsChars;
    const char* Nlset_ = _NlChars;
    istream& Is_ = (_IoMode & FILE_INPUT) ? _File->GetInStream():cin;

    // defaulting to last delims search
    char** Delims_ = (_LastDelims && _DefaultDelim != FIRST_SET) ?
                         _LastDelims:_FirstDelims;

    if (_SearchData)
    {
      ArrayIndex_ = _SearchData->ArrayIndex();
      StringIndex_ = _SearchData->StringIndex();
      DelimType_ = _SearchData->DelimType();

      delete _SearchData;
      _SearchData = NULL;
    }
    else
      ArrayIndex_ =
      StringIndex_ =
      DelimType_ = 0;

    // End of line not found, so next repeated occurrence
    // of last delim possible. Push latest occurrence onto stack
    // just in case next occurence of last delim is found.
    if (!_EolFound && DelimSetComplete(_LastDelims) > 0)
    {
      _DelimStk->Push(new DelimInfo(_LastDelims, _LastDelimFound, _LastDelimPos));
      ResetLastDelims(0, true);
    }

    for (x = 0; !WsFound_ && !_EofFound && x < _BufSize;)
    {
      ch = Is_.get();
      WsFound_ = InCharSet(ch, Cset_);

      if (!WsFound_)
      {
        InputStr_[x++] = ch;
        PrevCol_ = _FilePos->ColPos();
        CountColsRows(ch);
        Collide_ = SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);

        if (_File->EndOfFile())
        {
          _EofFound = true;
          _EndFilePos = _FilePos->ColPos();
        }
        else if (!_EolFound)
        {
          if ((Dsc_=DelimSetComplete(_LastDelims) > 0) || Collide_ == LAST_SET)
          {
            if (Dsc_)
            {
              _DelimStk->Push(new DelimInfo(_LastDelims, _LastDelimFound, _LastDelimPos));
              ResetLastDelims(0, true);
            }
            else
            {
              ResetLastDelims(0, true);
              SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);
            }
          }
          else if (Collide_ == FIRST_SET && !DelimSetComplete(_FirstDelims))
          {
            ResetFirstDelims(0, true);
            SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);
          }          
        }
      }
      else
      {      
        _WsFound = true;
        _WsLinePos = x;
        CountColsRows(ch);
        
        if (InCharSet(ch, Nlset_))
        {
          _EolFound = true;
          _EndLinePos = _FilePos->ColPos();
        }

        if (_File->EndOfFile())
        {
          _EofFound = true;
          _EndFilePos = _FilePos->ColPos();
        }
      }
    }

    _BufferFull = (x == _BufSize);
    _ReadAttemptNumber++;
    _ReadLenQueue->Enqueue(new long(x));

    if (x == 0)
    {
      _LineEmpty = COMPLETE;
      return *this;
    }
    else
      _CharsRead = x;

    if (_BufferFull && !_EolFound)
    {
      _LineFitInBuffer = false;
      InputStr_[_BufSize] = 0;
      IsEmptyLine(InputStr_, _BufSize);
      _SearchData = new TextFileSearchData(ArrayIndex_, StringIndex_, DelimType_);
      
      _TimesLimitReached++;
      _ReadTimesRequired++;
    }
    else if (_EolFound)
    {
      InputStr_[x] = 0;
      _ReadTimesRequired++;
      _LineFitInBuffer = _TimesLimitReached == 0;

      if (_LineFitInBuffer || x > 0)
        IsEmptyLine(InputStr_, x);

      // If next occurrence of last delim is not found and delim stack
      // is not empty then pop most recent occurrence of last delim before
      // deleting the stack contents
      if (!DelimSetComplete(_LastDelims) && !_DelimStk->Empty())
      {
        DelimInfo* Delimp_ = _DelimStk->Pop();
        
        if (Delimp_)
        {
          Delimp_->AssignTo(_LastDelims, _LastDelimFound, _LastDelimPos);
          delete Delimp_;
        }
      }

      // defaulting to last delims if found
      DetermineDelimSet((_LastDelims && _DefaultDelim != FIRST_SET) ? _LastDelims:_FirstDelims);
    }
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ReadUntil(const char* CharSet_, bool InSet_, bool PutBack_)
{
  if (_File && !(_EolFound || _BufferFull) && !_EofFound)
  {
    int ArrayIndex_;
    int StringIndex_;
    int DelimType_;
    
    char ch;    
    long PrevCol_;

    int x;
    int Collide_ = NONE;
    bool Dsc_ = false;
    bool BrkChFound_ = false;
    const char* Cset_ = CharSet_;
    const char* Wsset_ = _WsChars;
    const char* Nlset_ = _NlChars;
    istream& Is_ = (_IoMode & FILE_INPUT) ? _File->GetInStream():cin;

    // defaulting to last delims search
    char** Delims_ = (_LastDelims && _DefaultDelim != FIRST_SET) ?
                         _LastDelims:_FirstDelims;

    if (_SearchData)
    {
      ArrayIndex_ = _SearchData->ArrayIndex();
      StringIndex_ = _SearchData->StringIndex();
      DelimType_ = _SearchData->DelimType();

      delete _SearchData;
      _SearchData = NULL;
    }
    else
      ArrayIndex_ =
      StringIndex_ =
      DelimType_ = 0;

    // End of line not found, so next repeated occurrence
    // of last delim possible. Push latest occurrence onto stack
    // just in case next occurence of last delim is found.
    if (!_EolFound && DelimSetComplete(_LastDelims) > 0)
    {
      _DelimStk->Push(new DelimInfo(_LastDelims, _LastDelimFound, _LastDelimPos));
      ResetLastDelims(0, true);
    }

    for (x = 0; !BrkChFound_ && !_EofFound && x < _BufSize;)
    {
      ch = Is_.get();

      if ((InSet_ && InCharSet(ch, Cset_)) ||
          (!InSet_ && !InCharSet(ch, Cset_)))
      {
        if (PutBack_)
          Is_.putback(ch);
          
        BrkChFound_ = true;
      }
      else
        BrkChFound_ = false;

      if (!BrkChFound_)
      {
        _TextBuffer[x++] = ch;
        PrevCol_ = _FilePos->ColPos();
        CountColsRows(ch);
        Collide_ = SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);

        if (_File->EndOfFile())
        {
          _EofFound = true;
          _EndFilePos = _FilePos->ColPos();
        }
        else if (!_EolFound)
        {
          if ((Dsc_=DelimSetComplete(_LastDelims) > 0) || Collide_ == LAST_SET)
          {
            if (Dsc_)
            {
              _DelimStk->Push(new DelimInfo(_LastDelims, _LastDelimFound, _LastDelimPos));
              ResetLastDelims(0, true);
            }
            else
            {
              ResetLastDelims(0, true);
              SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);
            }
          }
          else if (Collide_ == FIRST_SET && !DelimSetComplete(_FirstDelims))
          {
            ResetFirstDelims(0, true);
            SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);
          }
        }
      }
      else
      {
        CountColsRows(ch);
      
        if (InCharSet(ch, Wsset_))
        {
          _WsFound = true;
          _WsLinePos = _FilePos->ColPos();
        }
          
        if (InCharSet(ch, Nlset_))
        {
          _EolFound = true;
          _EndLinePos = _FilePos->ColPos();
        }

        if (_File->EndOfFile())
        {
          _EofFound = true;
          _EndFilePos = _FilePos->ColPos();
        }
      }
    }

    _BufferFull = (x == _BufSize);
    _ReadAttemptNumber++;
    _ReadLenQueue->Enqueue(new long(x));

    if (x == 0)
    {
      _LineEmpty = COMPLETE;
      return *this;
    }
    else
      _CharsRead = x;

    if (_BufferFull && !_EolFound)
    {
      _LineFitInBuffer = false;
      _TextBuffer[_BufSize] = 0;
      IsEmptyLine(_TextBuffer, _BufSize);
      _SearchData = new TextFileSearchData(ArrayIndex_, StringIndex_, DelimType_);

      _TimesLimitReached++;
      _ReadTimesRequired++;
    }
    else if (_EolFound)
    {
      _TextBuffer[x] = 0;
      _ReadTimesRequired++;
      _LineFitInBuffer = _TimesLimitReached == 0;

      if (_LineFitInBuffer || x > 0)
        IsEmptyLine(_TextBuffer, x);

      // If next occurrence of last delim is not found and delim stack
      // is not empty then pop most recent occurrence of last delim before
      // deleting the stack contents
      if (!DelimSetComplete(_LastDelims) && !_DelimStk->Empty())
      {
        DelimInfo* Delimp_ = _DelimStk->Pop();
        
        if (Delimp_)
        {
          Delimp_->AssignTo(_LastDelims, _LastDelimFound, _LastDelimPos);
          delete Delimp_;
        }
      }

      // defaulting to last delims if found
      DetermineDelimSet((_LastDelims && _DefaultDelim != FIRST_SET) ? _LastDelims:_FirstDelims);
    }
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::Read()
{
  if (_File && !(_EolFound || _BufferFull) && !_EofFound)
  {
    int ArrayIndex_;
    int StringIndex_;
    int DelimType_;

    char ch;
    long PrevCol_;

    int x;
    int Collide_ = NONE;
    bool Dsc_ = false;
    bool WsFound_ = false;
    const char* Cset_ = _WsChars;
    const char* Nlset_ = _NlChars;
    istream& Is_ = (_IoMode & FILE_INPUT) ? _File->GetInStream():cin;

    // defaulting to last delims search
    char** Delims_ = (_LastDelims && _DefaultDelim != FIRST_SET) ?
                         _LastDelims:_FirstDelims;

    if (_SearchData)
    {
      ArrayIndex_ = _SearchData->ArrayIndex();
      StringIndex_ = _SearchData->StringIndex();
      DelimType_ = _SearchData->DelimType();

      delete _SearchData;
      _SearchData = NULL;
    }
    else
      ArrayIndex_ =
      StringIndex_ =
      DelimType_ = 0;

    // End of line not found, so next repeated occurrence
    // of last delim possible. Push latest occurrence onto stack
    // just in case next occurence of last delim is found.
    if (!_EolFound && DelimSetComplete(_LastDelims) > 0)
    {
      _DelimStk->Push(new DelimInfo(_LastDelims, _LastDelimFound, _LastDelimPos));
      ResetLastDelims(0, true);
    }

    for (x = 0; !WsFound_ && !_EofFound && x < _BufSize;)
    {
      ch = Is_.get();
      WsFound_ = InCharSet(ch, Cset_);

      if (!WsFound_)
      {
        _TextBuffer[x++] = ch;
        PrevCol_ = _FilePos->ColPos();
        CountColsRows(ch);
        Collide_ = SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);

        if (_File->EndOfFile())
        {
          _EofFound = true;
          _EndFilePos = _FilePos->ColPos();
        }
        else if (!_EolFound)
        {
          if ((Dsc_=DelimSetComplete(_LastDelims) > 0) || Collide_ == LAST_SET)
          {
            if (Dsc_)
            {
              _DelimStk->Push(new DelimInfo(_LastDelims, _LastDelimFound, _LastDelimPos));
              ResetLastDelims(0, true);
            }
            else
            {
              ResetLastDelims(0, true);
              SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);
            }
          }
          else if (Collide_ == FIRST_SET && !DelimSetComplete(_FirstDelims))
          {
            ResetFirstDelims(0, true);
            SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);
          }
        }
      }
      else
      {
        _WsFound = true;
        _WsLinePos = _FilePos->ColPos();
        CountColsRows(ch);
        
        if (InCharSet(ch, Nlset_))
        {
          _EolFound = true;
          _EndLinePos = _FilePos->ColPos();
        }

        if (_File->EndOfFile())
        {
          _EofFound = true;
          _EndFilePos = _FilePos->ColPos();
        }
      }
    }

    _BufferFull = (x == _BufSize);
    _ReadAttemptNumber++;
    _ReadLenQueue->Enqueue(new long(x));

    if (x == 0)
    {
      _LineEmpty = COMPLETE;
      return *this;
    }
    else
      _CharsRead = x;

    if (_BufferFull && !_EolFound)
    {
      _LineFitInBuffer = false;
      _TextBuffer[_BufSize] = 0;
      IsEmptyLine(_TextBuffer, _BufSize);
      _SearchData = new TextFileSearchData(ArrayIndex_, StringIndex_, DelimType_);

      _TimesLimitReached++;
      _ReadTimesRequired++;
    }
    else if (_EolFound)
    {
      _TextBuffer[x] = 0;
      _ReadTimesRequired++;
      _LineFitInBuffer = _TimesLimitReached == 0;

      if (_LineFitInBuffer || x > 0)
        IsEmptyLine(_TextBuffer, x);

      // If next occurrence of last delim is not found and delim stack
      // is not empty then pop most recent occurrence of last delim before
      // deleting the stack contents
      if (!DelimSetComplete(_LastDelims) && !_DelimStk->Empty())
      {
        DelimInfo* Delimp_ = _DelimStk->Pop();
        
        if (Delimp_)
        {
          Delimp_->AssignTo(_LastDelims, _LastDelimFound, _LastDelimPos);
          delete Delimp_;
        }
      }

      // defaulting to last delims if found
      DetermineDelimSet((_LastDelims && _DefaultDelim != FIRST_SET) ? _LastDelims:_FirstDelims);
    }
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ReadLine()
{
  if (_File && !(_EolFound || _BufferFull) && !_EofFound)
  {
    int ArrayIndex_;
    int StringIndex_;
    int DelimType_;
    
    char ch;
    long PrevCol_;

    int x;
    int Collide_ = NONE;
    bool Dsc_ = false;
    bool NlFound_ = false;
    const char* Cset_ = _NlChars;
    istream& Is_ = (_IoMode & FILE_INPUT) ? _File->GetInStream():cin;

    // defaulting to last delims search
    char** Delims_ = (_LastDelims && _DefaultDelim != FIRST_SET) ?
                         _LastDelims:_FirstDelims;

    if (_SearchData)
    {
      ArrayIndex_ = _SearchData->ArrayIndex();
      StringIndex_ = _SearchData->StringIndex();
      DelimType_ = _SearchData->DelimType();

      delete _SearchData;
      _SearchData = NULL;
    }
    else
      ArrayIndex_ =
      StringIndex_ =
      DelimType_ = 0;

    // End of line not found, so next repeated occurrence
    // of last delim possible. Push latest occurrence onto stack
    // just in case next occurence of last delim is found.
    if (!_EolFound && DelimSetComplete(_LastDelims) > 0)
    {
      _DelimStk->Push(new DelimInfo(_LastDelims, _LastDelimFound, _LastDelimPos));
      ResetLastDelims(0, true);
    }

    for (x = 0; !NlFound_ && !_EofFound && x < _BufSize;)
    {
      ch = Is_.get();
      NlFound_ = InCharSet(ch, Cset_);

      if (!NlFound_)
      {
        _TextBuffer[x++] = ch;
        PrevCol_ = _FilePos->ColPos();
        CountColsRows(ch);
        Collide_ = SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);

        if (_File->EndOfFile())
        {
          _EofFound = true;
          _EndFilePos = _FilePos->ColPos();
        }
        else if (!_EolFound)
        {
          if ((Dsc_=DelimSetComplete(_LastDelims) > 0) || Collide_ == LAST_SET)
          {
            if (Dsc_)
            {
              _DelimStk->Push(new DelimInfo(_LastDelims, _LastDelimFound, _LastDelimPos));
              ResetLastDelims(0, true);
            }
            else
            {
              ResetLastDelims(0, true);
              SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);
            }
          }
          else if (Collide_ == FIRST_SET && !DelimSetComplete(_FirstDelims))
          {
            ResetFirstDelims(0, true);
            SearchForDelims(Delims_, ch, PrevCol_, ArrayIndex_, StringIndex_, DelimType_);
          }
        }
      }
      else
      {
        _WsFound = true;
        _EolFound = true;
        _WsLinePos =
        _EndLinePos = _FilePos->ColPos();
        CountColsRows(ch);

        if (_File->EndOfFile())
        {
          _EofFound = true;
          _EndFilePos = _FilePos->ColPos();
        }        
      }
    }

    _BufferFull = (x == _BufSize);
    _ReadAttemptNumber++;
    _ReadLenQueue->Enqueue(new long(x));

    if (x == 0)
    {
      _LineEmpty = COMPLETE;
      return *this;
    }
    else
      _CharsRead = x;

    if (_BufferFull && !_EolFound)
    {
      _LineFitInBuffer = false;
      _TextBuffer[_BufSize] = 0;
      IsEmptyLine(_TextBuffer, _BufSize);
      _SearchData = new TextFileSearchData(ArrayIndex_, StringIndex_, DelimType_);

      _TimesLimitReached++;
      _ReadTimesRequired++;
    }
    else if (_EolFound)
    {
      _TextBuffer[x] = 0;
      _ReadTimesRequired++;
      _LineFitInBuffer = _TimesLimitReached == 0;

      if (_LineFitInBuffer || x > 0)
        IsEmptyLine(_TextBuffer, x);

      // If next occurrence of last delim is not found and delim stack
      // is not empty then pop most recent occurrence of last delim before
      // deleting the stack contents
      if (!DelimSetComplete(_LastDelims) && !_DelimStk->Empty())
      {
        DelimInfo* Delimp_ = _DelimStk->Pop();
        
        if (Delimp_)
        {
          Delimp_->AssignTo(_LastDelims, _LastDelimFound, _LastDelimPos);
          delete Delimp_;
        }
      }

      // defaulting to last delims if found
      DetermineDelimSet((_LastDelims && _DefaultDelim != FIRST_SET) ? _LastDelims:_FirstDelims);
    }
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ReadLineOrToEobuf()
{
  if (_File)
  {
    const char* Cset_ = _NlChars;
    _File->ReadLine(_TextBuffer, _BufSize, Cset_);
    int Ret_ = _File->GetIOResult();
    
    _BufferFull = (Ret_ == StreamReader::IO_NODELIM);
    _LineFitInBuffer = (Ret_ == StreamReader::IO_DONE);
    _EolFound = !_BufferFull && _LineFitInBuffer;    
    _EofFound = _File->EndOfFile();

    if (_BufferFull && !_EolFound)
    {    
      _TimesLimitReached++;
      _ReadTimesRequired++;
    }
    else if (_EolFound)
      _ReadTimesRequired++;
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::TruncLine()
{
  if (_File)
  {
    const char* Cset_ = _NlChars;
    _File->TruncLine(_TextBuffer, _BufSize, Cset_);
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::Ignore()
{
  if (_File)
  {
    const char* Cset_ = _NlChars;
    _File->Ignore(Cset_);
  }

  return *this;
}

/****************************************************************************/
TextFileLineBuffer& TextFileLineBuffer::ReadBuffer(streambuf* Sb_)
{
  if (_File)
  {
    const char* Cset_ = _NlChars;
    _File->ReadBuffer(Sb_, Cset_);
  }

  return *this;
}

/****************************************************************************/
bool TextFileLineBuffer::EndOfFile()
{
  return (_File ? _File->EndOfFile():FALSE);
}

/****************************************************************************/
bool TextFileLineBuffer::Good()
{
  return (_File ? _File->Good():FALSE);
}

/****************************************************************************/
bool TextFileLineBuffer::Bad()
{
  return (_File ? _File->Bad():FALSE);
}

/****************************************************************************/
bool TextFileLineBuffer::Fail()
{
  return (_File ? _File->Fail():FALSE);
}

/****************************************************************************/
bool TextFileLineBuffer::Open(const char* Fname_, int Mode_)
{
  return (_File ? _File->Open(Fname_, Mode_):FALSE);
}

/****************************************************************************/
bool TextFileLineBuffer::Close()
{
  return (_File ? _File->Close():FALSE);
}

/****************************************************************************/
long TextFileLineBuffer::FileSize(int Mode_)
{
  return (_File ? _File->FileSize(Mode_):0);
}

/****************************************************************************/
/****************************************************************************/
#if TEXTFILELINE_DEBUG
int main(int argc, char* argv[])
{
  static int IoMode_ = 0;
  static bool ShowColHead_ = true;

  #if TEXTFILELINE_DEBUG1
    int x;
    static char* TESTIN 	= new_char_string("textfiledata.html");
    static char* TESTOUT	= new_char_string("textfile.out1");
    IoMode_ = TextLineEnums::FILE_OUTPUT | TextLineEnums::FILE_INPUT;

    static char* Fd[3];
    static bool Fo[] = {false, true, false};
    Fd[0] = new_char_string("<");
    Fd[1] = new_char_string("=");
    Fd[2] = new_char_string(">");

    static char* Ld[3];
    static bool Lo[] = {false, false, false};
    Ld[0] = new_char_string("<");
    Ld[1] = new_char_string(">");
    Ld[2] = NULL;

  #elif TEXTFILELINE_DEBUG2
    int x;
    static char* TESTIN 	= new_char_string("textfiledata.html");
    static char* TESTOUT	= new_char_string("textfile.out2");
    IoMode_ = TextLineEnums::FILE_OUTPUT | TextLineEnums::FILE_INPUT;

    static char* Fd[3];
    static bool Fo[] = {false, true, false};
    Fd[0] = new_char_string("<");
    Fd[1] = new_char_string("=");
    Fd[2] = new_char_string(">");

    static char* Ld[3];
    static bool Lo[] = {false, false, false};
    Ld[0] = new_char_string("</");
    Ld[1] = new_char_string(">");
    Ld[2] = NULL;

  #elif TEXTFILELINE_DEBUG3
    int x;
    static char* TESTIN 	= new_char_string("textfiledata.html");
    static char* TESTOUT	= new_char_string("textfile.out3");
    IoMode_ = TextLineEnums::FILE_OUTPUT | TextLineEnums::FILE_INPUT;

    static char* Fd[3];
    static bool Fo[] = {false, false, false};
    Fd[0] = NULL;
    Fd[1] = NULL;
    Fd[2] = NULL;

    static char* Ld[3];
    static bool Lo[] = {false, false, false};
    Ld[0] = NULL;
    Ld[1] = NULL;
    Ld[2] = NULL;

  #elif TEXTFILELINE_DEBUG4
    int x;
    static char* TESTIN 	= new_char_string("textfiledata.html");
    static char* TESTOUT	= new_char_string("textfile.dat");
    IoMode_ = TextLineEnums::FILE_OUTPUT | TextLineEnums::FILE_INPUT;
    ShowColHead_ = false;

    static char* Fd[3];
    static bool Fo[] = {false, false, false};
    Fd[0] = NULL;
    Fd[1] = NULL;
    Fd[2] = NULL;

    static char* Ld[3];
    static bool Lo[] = {false, false, false};
    Ld[0] = new_char_string("<");
    Ld[1] = new_char_string(">");
    Ld[2] = NULL;

  #elif TEXTFILELINE_DEBUG5
    int x, i;
    static char* TESTIN = NULL;
    static char* TESTOUT = NULL;

    static char* Fd[3];
    static bool Fo[] = {false, true, false};
    Fd[0] = NULL;
    Fd[1] = NULL;
    Fd[2] = NULL;

    static char* Ld[3];
    static bool Lo[] = {false, false, false};
    Ld[0] = NULL;
    Ld[1] = NULL;
    Ld[2] = NULL;

    if (argc == 1)
    {
      cout <<"Usage: textfileline -i=<infile> -o=<outfile> -fd=n:f1,f2,f3 -ld=n:l1,l2,l3" <<endl;
      cout <<endl
           <<"infile : input text file with delimiters" <<endl
           <<"outfile : output text file with line record data" <<endl;
      cout <<"-fd=n:f1,f2,f3 : first starting delimiters, n in range [1...3]" <<endl
           <<"                 indicating number of delimiters specified in starting set" <<endl
           <<"f1...f3        : delimiters specified in starting delimiter set," <<endl
           <<"                 append f1(1)...f3(1) to indicate optional delimiter" <<endl;
      cout <<"-ld=n:f1,f2,f3 : last ending delimiters, n in range [1...3]" <<endl
           <<"                 indicating number of delimiters specified in ending set" <<endl
           <<"f1...f3        : delimiters specified in ending delimiter set," <<endl
           <<"                 append f1(1)...f3(1) to indicate optional delimiter" <<endl;
      cout <<"delimiter string escape sequences : " <<endl
           <<"\t&amp;          : ampersand" <<endl
           <<"\t&lt;           : less than" <<endl
           <<"\t&gt;           : greater than" <<endl
           <<"\t&quot;         : quotation" <<endl
           <<"\t&tagop;        : tag open" <<endl
           <<"\t&tagcl;        : tag close" <<endl
           <<"\t&endtagop;     : end tag open" <<endl
           <<"\t&endtagcl;     : end tag close" <<endl
           <<"\t&emptytagop;   : empty tag open" <<endl
           <<"\t&emptytagcl;   : empty tag close" <<endl
           <<"\t&scriptop;     : script tag open" <<endl
           <<"\t&scriptcl;     : script tag close" <<endl
           <<"\t&doctypeop;    : document type tag open" <<endl
           <<"\t&doctypecl;    : document type tag close" <<endl
           <<"\t&commentop;    : comment tag open" <<endl
           <<"\t&commentcl;    : comment tag open" <<endl
           <<"\t&nestedcmt1op; : nested comment1 tag open" <<endl
           <<"\t&nestedcmt1cl; : nested comment1 tag close" <<endl
           <<"\t&nestedcmt2op; : nested comment2 tag open" <<endl
           <<"\t&nestedcmt2cl; : nested comment2 tag close" <<endl
           <<"\t&copcmtblkop;  : C open comment block open" <<endl
           <<"\t&copcmtblkcl;  : C open comment block close" <<endl
           <<"\t&cclcmtblkop;  : C close comment block open" <<endl
           <<"\t&cclcmtblkcl;  : C close comment block close" <<endl
           <<"\t&cppcmtlnop;   : Cpp comment line open" <<endl
           <<"\t&cppcmtlncl;   : Cpp comment line close" <<endl;

      return 1;
    }
    else if (argc > 3)
    {
      int ifpos = 0;
      int ofpos = 0;
      int iflen = 0;
      int oflen = 0;
      int ifoffset = 0;
      int ofoffset = 0;
      int fdlm = 0;
      int ldlm = 0;
      int fdpos = 0;
      int ldpos = 0;
      int fdlen = 0;
      int ldlen = 0;
      int fdcnt = 0;
      int ldcnt = 0;

      if (argc == 2 && strcmp(argv[1], "-ver") == 0)
      {
        fprintf(stderr,
                "Text File Line Parser\n"
                "version: 1.0\n");
        return 0;
      }
    
      for (x = 1; x < argc; x++)
      {
        if (strncmp(argv[x], "-fd=", 4) == 0 &&
            strlen(argv[x]) > 4)
        {
          fdlm = 4;
          fdpos = x;
          fdlen = strlen(argv[x]+4);
        }

        if (strncmp(argv[x], "-ld=", 4) == 0 &&
            strlen(argv[x]) > 4)
        {
          ldlm = 4;
          ldpos = x;
          ldlen = strlen(argv[x]+4);
        }
    
        if (strncmp(argv[x], "-i=", 3) == 0 &&
            strlen(argv[x]) > 3)
        {
          ifpos = x;
          iflen = strlen(argv[x]+3);
          ifoffset = 3;
        }

        if (strncmp(argv[x], "-o=", 3) == 0 &&
            strlen(argv[x]) > 3)
        {
          ofpos = x;
          oflen = strlen(argv[x]+3);
          ofoffset = 3;
        }
      }
      
      if (ifpos && iflen)
      {
        char* ofname = NULL;
        char* ifname = NULL;

        if (oflen && ofpos)
          ofname = new_char_string(argv[ofpos]+ofoffset);

        if (iflen && ifpos)
          ifname = new_char_string(argv[ifpos]+ifoffset);

        if (ofname)
        {
          RemovePadding(ofname, " \t");
          TESTOUT = ofname;
          ofname = NULL;
          IoMode_ |= TextLineEnums::FILE_OUTPUT;
        }

        if (ifname)
        {
          RemovePadding(ifname, " \t");
          TESTIN = ifname;
          ifname = NULL;
          IoMode_ |= TextLineEnums::FILE_INPUT;
        }
      }

      if (fdlen || ldlen)
      {
        char* ldstr = NULL;
        char* fdstr = NULL;
        const char* cpos = NULL;
        char* pos;

        if (fdlen && fdpos)
        {
          fdstr = new_char_string(argv[fdpos]+fdlm);
          cpos = strchr(fdstr, ':');
          
          if (cpos)
          {
            x = cpos - fdstr;
            fdstr[x] = 0;
            fdcnt = atoi(fdstr);
            fdstr[x] = ':';
            i = 0;

            for (pos=strtok(fdstr+x+1, ","); pos && i < fdcnt;
                 pos=strtok(NULL, ","), i++)
            {
              cpos = strchr(pos, '(');
              
              if (cpos)
              {
                x = cpos - pos;
                pos[x] = 0;
                Fd[i] = new_char_string(TextFileLineBuffer::TagTypeStr(pos));
                pos[x] = '(';
                
                if (pos[x+2] == ')')
                {
                  pos[x+2] = 0;
                  Fo[i] = (bool)atoi(pos+x+1);
                  pos[x+2] = ')';
                }
              }
              else
                Fd[i] = new_char_string(TextFileLineBuffer::TagTypeStr(pos));
            }
          }
        }

        if (ldlen && ldpos)
        {        
          ldstr = new_char_string(argv[ldpos]+ldlm);
          cpos = strchr(ldstr, ':');

          if (cpos)
          {
            x = cpos - ldstr;
            ldstr[x] = 0;
            ldcnt = atoi(ldstr);
            ldstr[x] = ':';
            i = 0;

            for (pos=strtok(ldstr+x+1, ","); pos && i < ldcnt;
                 pos=strtok(NULL, ","), i++)
            {
              cpos = strchr(pos, '(');
              
              if (cpos)
              {
                x = cpos - pos;
                pos[x] = 0;
                Ld[i] = new_char_string(TextFileLineBuffer::TagTypeStr(pos));
                pos[x] = '(';
                
                if (pos[x+2] == ')')
                {
                  pos[x+2] = 0;
                  Lo[i] = (bool)atoi(pos+x+1);
                  pos[x+2] = ')';
                }
              }
              else
                Ld[i] = new_char_string(TextFileLineBuffer::TagTypeStr(pos));
            }
          }
        }
      }
    }
  #endif

  TextFileLineBuffer* TxtBuf_ = NULL;
  FileReader* Fr_ = new FileReader;
  
  if (Fr_ &&
      (!(IoMode_ & TextLineEnums::FILE_INPUT) ||
         Fr_->Open(TESTIN, ios::in)) &&
      (!(IoMode_ & TextLineEnums::FILE_OUTPUT) ||
         Fr_->Open(TESTOUT, ios::out)))
  {
    #if TEXTFILELINE_DEBUG3
      TxtBuf_ = new TextFileLineBuffer(Fr_, IoMode_, 32);
    #else
      TxtBuf_ = new TextFileLineBuffer(Fr_, IoMode_);
    #endif

    if (TxtBuf_)
    {
      if (Fd[0])
        TxtBuf_->SetFirstDelims(0, Fd[0], Fo[0]);
      if (Fd[1])
        TxtBuf_->SetFirstDelims(1, Fd[1], Fo[1]);
      if (Fd[2])
        TxtBuf_->SetFirstDelims(2, Fd[2], Fo[2]);

      if (Ld[0])
        TxtBuf_->SetLastDelims(0, Ld[0], Lo[0]);
      if (Ld[1])
        TxtBuf_->SetLastDelims(1, Ld[1], Lo[1]);
      if (Ld[2])
        TxtBuf_->SetLastDelims(2, Ld[2], Lo[2]);

      TxtBuf_->SetDefaultToDelims(TextFileLineBuffer::LAST_SET);
      
      while (TxtBuf_->While_ReadLine())
      {
        TxtBuf_->DumpLineRecord(ShowColHead_);
        TxtBuf_->ResetLineData();
      }

      TxtBuf_->Close();
    }
  }

  for (x = 0; x < TextLineEnums::MAXDELIMS; x++)
  {
    ::RawDeleteArray(Fd[x]);
    ::RawDeleteArray(Ld[x]);
  }

  ::RawDeleteArray(TESTIN);
  ::RawDeleteArray(TESTOUT);

  delete Fr_;
  delete TxtBuf_;  
}
#endif
/****************************************************************************/
#endif

