#ifndef CALCLIB_H
#define CALCLIB_H
/****************************************************************************/
/* standard libraries */
#ifndef INCL_STDIO_H
  #include <stdio.h>
  #define INCL_STDIO_H
#endif
#ifndef INCL_STRING_H
  #include <string.h>
  #define INCL_STRING_H
#endif
#ifndef INCL_CTYPE_H
  #include <ctype.h>
  #define INCL_CTYPE_H
#endif
#ifndef INCL_ASSERT_H
  #include <assert.h>
  #define INCL_ASSERT_H
#endif
#ifndef INCL_STDLIB_H
  #include <stdlib.h>
  #define INCL_STDLIB_H
#endif
#ifndef INCL_MATH_H
  #include <math.h>
  #define INCL_MATH_H
#endif
#if defined(__DJGPP__) | defined(__GNUC__)
  #ifndef INCL_UNISTD_H
    #include <unistd.h>
    #define INCL_UNISTD_H
  #endif
#endif
#if (!defined(__DJGPP__) & !defined(__linux__) & !defined(__unix__) & !defined(__GNUC__))
  #include <Windows.h>
#endif

/* Custom libraries */
// ../cppds-4.2/source/
#ifndef TYPEDEFS_H
  #include "typedefs.h"
#endif
#ifndef CHRSTRING_H
  #include "chrstring.h"
#endif
#ifndef LONGNUM_H
  #include "longnum.h"
#endif
#ifndef DISCRETE_CPP
  #include "discrete.cpp"
#endif
#ifndef RANDOMIZER_H
  #include "randomizer.h"
#endif
#ifndef BITSTR_H
  #include "bitstr.h"
#endif
#ifndef CALCHELPMSG_H
  #include "calchelpmsg.h"
#endif
#ifndef SET_CPP
  #include "set.cpp"
#endif
#ifndef STATARRAY_CPP
  #include "statarray.cpp"
#endif
#ifndef GAUSS_H
  #include "gauss.h"
#endif
#ifndef FILEREADER_H
  #include "filereader.h"
#endif

#define CALCLIB_DEBUG1     0
#define CALCLIB_DEBUG2     0
#define CALCLIB_DEBUG3     0

#define CALCLIB_DEBUG5     0
#define CALCLIB_DEBUG6     0

/* Some useful constants.  */
#define M_E_STRING		        "2.7182818284590452354"	/* e */
#define M_LOG2E_STRING	        "1.4426950408889634074"	/* log_2 e */
#define M_LOG10E_STRING	        "0.43429448190325182765"	/* log_10 e */
#define M_LN2_STRING		        "0.69314718055994530942"	/* log_e 2 */
#define M_LN10_STRING		     "2.30258509299404568402"	/* log_e 10 */
#define M_PI_STRING		        "3.14159265358979323846"	/* pi */
#define M_PI_2_STRING		     "1.57079632679489661923"	/* pi/2 */
#define M_PI_4_STRING		     "0.78539816339744830962"	/* pi/4 */
#define M_1_PI_STRING		     "0.31830988618379067154"	/* 1/pi */
#define M_2_PI_STRING		     "0.63661977236758134308"	/* 2/pi */
#define M_2_SQRTPI_STRING	     "1.12837916709551257390"	/* 2/sqrt(pi) */
#define M_SQRT2_STRING	        "1.41421356237309504880"	/* sqrt(2) */
#define M_SQRT1_2_STRING	     "0.70710678118654752440"	/* 1/sqrt(2) */

/* The above constants are not adequate for computation using `long double's.
   Therefore we provide as an extension constants with similar names as a
   GNU extension.  Provide enough digits for the 128-bit IEEE quad.  */
#define M_El_STRING		        "2.7182818284590452353602874713526625"  /* e */
#define M_LOG2El_STRING	        "1.4426950408889634073599246810018922"  /* log_2 e */
#define M_LOG10El_STRING	     "0.4342944819032518276511289189166051"  /* log_10 e */
#define M_LN2l_STRING		     "0.6931471805599453094172321214581766"  /* log_e 2 */
#define M_LN10l_STRING	        "2.3025850929940456840179914546843642"  /* log_e 10 */
#define M_PIl_STRING		        "3.1415926535897932384626433832795029"  /* pi */
#define M_PI_2l_STRING	        "1.5707963267948966192313216916397514"  /* pi/2 */
#define M_PI_4l_STRING	        "0.7853981633974483096156608458198757"  /* pi/4 */
#define M_1_PIl_STRING	        "0.3183098861837906715377675267450287"  /* 1/pi */
#define M_2_PIl_STRING	        "0.6366197723675813430755350534900574"  /* 2/pi */
#define M_2_SQRTPIl_STRING	     "1.1283791670955125738961589031215452"  /* 2/sqrt(pi) */
#define M_SQRT2l_STRING	        "1.4142135623730950488016887242096981"  /* sqrt(2) */
#define M_SQRT1_2l_STRING	     "0.7071067811865475244008443621048490"  /* 1/sqrt(2) */

/* Program Constants */
#define MAXARGS 4       /* maximum number of arguments allowable */
#define MAXFILES 10     /* maximum number of files allowable */
#define MAXOPERANDS 2   /* maximum number of operands allowable */
#define MAXLINE 256     /* maximum number of characters per line */

#define LIMIT_READ_RETRIES            0    // limit file read retry attempts of the I/O state file (5 is good)
#define IMPLEMENT_KEEP_ALIVE          0    // implement client polling for keep alive state
#define IOSTATE_PRE_SET_DELAY         0    // delay before setting I/O state value into IoState file
#define IOSTATE_POST_SET_DELAY        0    // delay after setting I/O state value into IoState file
#define IOSTATE_DELAY_LENGTH          200  // delay length in milliseconds

#define OUTPUTFILE_SWITCH "-o"  /* input file switch for specifying an input file */
#define INPUTFILE_SWITCH "-i"   /* output file switch for specifying an output file */

/* Message Constants */
#define ERRMSG_MULLERS_METHOD_FAILED    "ERROR: Root solving algorithm failed to converge for solution to: %s\n"
#define ERRMSG_ARGLIST_SIZE             "INVALID: Incorrect value for argument list size specifier in function: %s\n"
#define ERRMSG_INVALID_NUMARGS          "INVALID: invalid number of arguments for function: %s\n"
#define ERRMSG_INVALID_TYPEARGS         "INVALID: invalid type of arguments for function: %s\n"
#define ERRMSG_MISSLEFT_BRACKET         "INVALID: missing left bracket: %s\n"
#define ERRMSG_MISSRIGHT_BRACKET        "INVALID: missing right bracket: %s\n"
#define ERRMSG_INVALID_SYMBOLS          "INVALID: unrecognized math symbols: %s\n"
#define ERRMSG_INVALID_ASSIGNMENT       "INVALID: invalid assignment operation: %s\n" \
                                        "use $A to $Z storage variables for assignments.\n" \
                                        "If you want to test for equality use the" \
                                        " \'==\' symbol instead.\n"
#define ERRMSG_INVALID_EXPR             "INVALID: invalid expression: %s\n"
#define ERRMSG_MEMORY_FAIL              "ERROR: memory allocation error\n"
#define ERRMSG_DIVISION_BY_ZERO         "INVALID: division by zero: %s\n"
#define ERRMSG_FILE_ERROR               "ERROR: could not open file \"%s\" for %s\n"
#define ERRMSG_OPENSTATEFILE            "ERROR: Cannot open I/O state file: %s, calculator server\n"
#define ERRMSG_CLIENTNOTFOUND           "ERROR: Client program for calculator not responding\n"
#define ERRMSG_ARRAYBOUNDS              "ERROR: Exceeded array bounds in function: %s\n"
#define ERRMSG_EMPTY_DATASET            "ERROR: Empty data set for statistics operation: %s\n"
#define ERRMSG_RECURSION_LIMIT          "ERROR: Recursion limit reached: %s\n"
#define ERRMSG_UNSOLVABLE_MATRIX        "INVALID: Number of columns in matrix must be equal to the number of rows\n"
#define ERRMSG_INVALID_MATRIX_COLS      "INVALID: Number of columns in matrix must be the same for all matrix rows\n"
#define ERRMSG_EMPTYROW_MATRIX          "INVALID: Empty row in matrix found"
#define ERRMSG_NO_RESULT_COL_MATRIX     "INVALID: Result column missing for this row of the matrix: %s\n"
#define ERRMSG_INVALID_GRAPHVARNAME     "INVALID: Variable name does not match expression to be plotted: %s\n"

#define MSG_FILEREAD                    "reading"
#define MSG_FILEWRITE                   "writing"

// Default precision values
#define DEFAULT_PRECSTR_APPROXZERO      ".1"     // default precision to use for approx. zero solving
#define DEFAULT_PRECSTR_FTEST           ".01"    // default precision to use for min/max solving
#define DEFAULT_PRECSTR_SOLVE           ".00001" // default precision to use for root solving,
                                                 //   set to .00001 as smallest otherwise overflow
#define DEFAULT_PRECSTR_INTEG           ".00001" // default precision to use for integration,
                                                 //   set to .00001 as smallest otherwise overflow
#define DEFAULT_PRECSTR_DERIV           ".001"   // default precision to use for differentiation
#define DEFAULT_PRECSTR_GRAPH           ".002"   // default tolerance to use for graphing

#define DEFAULT_PRECVAL_APPROXZERO      .1       // default precision to use for approx. zero solving
#define DEFAULT_PRECVAL_FTEST           .01      // default precision to use for min/max solving
#define DEFAULT_PRECVAL_SOLVE           .00001   // default precision to use for root solving,
                                                 //   set to .00001 as smallest otherwise overflow
#define DEFAULT_PRECVAL_INTEG           .00001   // default precision to use for integration,
                                                 //   set to .00001 as smallest otherwise overflow
#define DEFAULT_PRECVAL_DERIV           .001     // default precision to use for differentiation
#define DEFAULT_PRECVAL_NEWTCRIT        .2       // default precision to use for finding value for newton's method
#define DEFAULT_PRECVAL_GRAPH           .002     // default tolerance to use for graphing

#define DEFAULT_SIGFIG_SOLVE            -5       // Negative of significant figures for root solving
#define DEFAULT_SIGFIG_INTEG            -5       // Negative of significant figures for integration
#define DEFAULT_SIGFIG_DERIV            -3       // Negative of significant figures for differentiation

// value pair output format strings
#define DEFAULT_LONGNUM_FORMAT          "%s %s\n"
#define DEFAULT_FLOAT_FORMAT            "%.7f %.7f\n"
#define DEFAULT_DOUBLE_FORMAT           "%.14f %.14f\n"
#define DEFAULT_LONGDOUBLE_FORMAT       "%.19Lf %.19Lf\n"
#define DEFAULT_FLOATNAN_FORMAT         "%.7f %s\n"
#define DEFAULT_DOUBLENAN_FORMAT        "%.14f %s\n"
#define DEFAULT_LONGDOUBLENAN_FORMAT    "%.19Lf %s\n"

// default significant digits cutoff for float types
#define DEFAULT_FLOAT_EPSILON           0.0001
#define DEFAULT_DOUBLE_EPSILON          0.00000001
#define DEFAULT_LONGDOUBLE_EPSILON      0.000000000001

// Default file names
#define MCALC_CONFIG_FILE               "mcalc_fileio.cfg"      // configuration file for using file input/output
#define MCALC_SESSION_COUNT_FILE        "mcalc_sescnt.dat"      // main session counter file
#define MCALC_SESSION_FILE              "mcalc_session.dat"     // main session file
#define MCALC_LOG_FILE                  "mcalc_errors.log"      // error log file
#define MCALC_USER_PROMPT_FILE          "mcalc_userprompt.dat"  // user prompt file
#define MCALC_USER_INPUT_FILE           "mcalc_userinput.dat"   // user input file
#define MCALC_PROG_OUTPUT_FILE          "mcalc_programout.dat"  // program output file
#define MCALC_IOSTATE_FILE              "mcalc_iostate.dat"     // program I/O state file
#define MCALC_SERVERALIVE_FILE          "mcalc_serveralive.dat" // server keep alive state file
#define MCALC_CLIENTALIVE_FILE          "mcalc_clientalive.dat" // client keep alive state file
#define MCALC_CURRENTGRAPH_FILE         "mcalc_curgraph.dat"    // current graph suffix file

// namespace declaration
using namespace std;

/************************ Constants, Typedefs, Globals **********************/
// Type Definitions
// enumerated sign type definition
class Mcalc_SignType
{
  public:
  enum
  {
    Plus,
    Minus
  };
};

class Mcalc_SessionStates
{
  public:
  enum
  {
    InUse  = 1,
    UnUsed = 2
  };
};

// input / output enumerated type
class Mcalc_IOType
{
  public:
  enum
  {
    Input          = 0,
    Output         = 1,
    ErrorLog       = 2,
    UserInput      = 3,
    UserPrompt     = 4,
    ProgramOutput  = 5,
    IoState        = 6,
    ServerAlive    = 7,
    ClientAlive    = 8,
    CurrentGraph   = 9
  };
};

class Mcalc_IOState
{
  public:
  enum
  {
    PROCESS        = 1,
    OUTPUT_READY   = 2,
    CALC_HALT      = 4,
    CALC_RESPONSE  = 8,
    INPUT_REQUIRED = 16,
    OUTPUT_FETCHED = 32,
    INPUT_RECEIVED = 64,
    BREAK_PROGRAM  = 128,
    IDLE_STATE     = 256,
    GRAPH_OUTPUT   = 512,
    GRAPH_WAIT     = 1024,
    CALC_ERROR     = 2048
  };
};

class Mcalc_KeepAliveState
{
  public:
  enum
  {
    SERVER_TO_CLIENT    = 1,
    CLIENT_ACKNOWLEDGED = 2,
    CLIENT_TO_SERVER    = 4,
    SERVER_ACKNOWLEDGED = 8,
    IDLE_STATE          = 16
  };
};

struct Mcalc_GraphType
{
  enum
  {
    FUNCTION   = 0,
    EXTREMA    = 1,
    DERIVATIVE = 2,
    INTEGRAL   = 3,
    ROOTSOLVE  = 4,
    INTERSECT1 = 5,
    INTERSECT2 = 6
  };
};

struct Mcalc_Error
{
  enum
  {
    ERRVAL_INVALID_EXPR           = 1,
    ERRVAL_MISSLEFT_BRACKET       = 2,
    ERRVAL_MISSRIGHT_BRACKET      = 3,
    ERRVAL_INVALID_SYMBOLS        = 4,
    ERRVAL_INVALID_ASSIGNMENT     = 5,
    ERRVAL_INVALID_NUMARGS        = 6,
    ERRVAL_MULLERS_METHOD_FAILED  = 7,
    ERRVAL_DIVISION_BY_ZERO       = 8,
    ERRVAL_OPENSTATEFILE          = 9,
    ERRVAL_MEMORY_FAIL            = 10,
    ERRVAL_FILE_ERROR             = 11,
    ERRVAL_CLIENTNOTFOUND         = 12,
    ERRVAL_ARRAYBOUNDS            = 13,
    ERRVAL_INVALID_TYPEARGS       = 14,
    ERRVAL_EMPTY_DATASET          = 15,
    ERRVAL_RECURSION_LIMIT        = 16,
    ERRVAL_ARGLIST_SIZE           = 17,
    ERRVAL_UNSOLVABLE_MATRIX      = 18,
    ERRVAL_INVALID_MATRIX_COLS    = 19,
    ERRVAL_EMPTYROW_MATRIX        = 20,
    ERRVAL_NO_RESULT_COL_MATRIX   = 21,
    ERRVAL_INVALID_GRAPHVARNAME   = 22,
    MAX_ERRORS                    = 23
  };
};

// operand structure type definition
typedef struct
{
  char* value;
  int size;
  int sign;
} Mcalc_LongValue;

typedef Mcalc_LongValue Mcalc_Operand;

// enumerated operator type definition
class Mcalc_OperatorType
{
  public:
  enum
  {
    Assignment,
    Modulus,
    Logical_Not,
    Logical_And,
    Logical_Or,
    Exclusive_Or,
    Multiply,
    Divide,
    Add,
    Subtract,
    Exponent,
    Square,
    SquareRoot,
    Cube,
    CubeRoot,
    Sine,
    Cosine,
    Tangent,
    ArcSine,
    ArcCosine,
    ArcTangent,
    HyperSine,
    HyperCosine,
    HyperTangent,
    NaturalExp,
    NaturalLog,
    Log10,
    Ceiling,
    Floor,
    AbsoluteValue,
    FractionPart,
    IntegerPart,
    Percent,
    LessThan,
    GreaterThan,
    Round,
    Factorial,
    Combination,
    Permutation,
    CombineWithRep,
    PermuteWithRep,
    LessOrEqual,
    GreaterOrEqual,
    Equal,
    NotEqual,
    Minimum,
    Maximum,
    Derivative,
    Derivative2,
    Integral,
    Random,
    Inverse,
    RootSolve,
    Intersect,
    FunctionMinimum,
    FunctionMaximum,
    RoundEx,
    ItemPermutation,
    ItemCombination,
    ItemPermuteWithRep,
    ItemCombineWithRep,
    MakeSet,
    ShowSet,
    SetUnion,
    SetIntersection,
    SetDifference,
    SetExclusion,
    FractionArithmetic,
    PolarToRectX,
    PolarToRectY,
    RectToPolarRadius,
    RectToPolarAngle,
    ScientificNotation,
    FromHex,
    FromOct,
    FromBin,
    FromCmp2Bin,
    ToHex,
    ToOct,
    ToBin,
    ToCmp2Bin,
    Sum,
    Product,
    AvgMean,
    AvgMedian,
    AvgMode,
    StdDev,
    MatrixSolve,
    PlotGraph,
    UNDEF
  };
};

typedef Uchar* Mcalc_StackData;
typedef Uchar* Mcalc_QueueData;

typedef struct _Mcalc_Stack
{
  Mcalc_StackData data;
  struct _Mcalc_Stack* next;
} Mcalc_Stack;

typedef Mcalc_Stack Mcalc_Queue;

// standalone function declarations
int operator == (SimpleList<int>& x, const char* s);
int operator == (const char* s, SimpleList<int>& x);

char* Mcalc_DefaultInFnc(char*, int);
int Mcalc_DefaultOutFnc(const char*);
int Mcalc_DefaultSetElementEqualFnc(const int& ival1, const int& ival2);

class FunctionCall;
class CalculatorBase;

/****************************************************************************/
template <class T>
class PointerStack
{
  public:
    T* _Active;
    PointerStack<T>* _Next;

    void Set(T* Ptr_);
    void Push(PointerStack<T>* Ptr_);
    void Pop();
    void PopDelete();

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
struct AssignmentParsingData
{
  Boolean _FirstAlpha;
  Boolean _LastDollar;
  Boolean _AnsStrConstFound;
  Boolean _FracAnsConstFound;
  char _VarLetter;
  int _FirstQuotePos;
  int _LastQuotePos;

  AssignmentParsingData();
};

/****************************************************************************/
class SubExpression
{
  friend class FunctionCall;

  protected:
    CalculatorBase* _CalcPtr;
    SubExpression* _Parent;
    int _Level;

    char** _FunctionNames;
    char* _Expression;
    char* _AnswerStr;
    char* _OriginalStr;
    FunctionCall** _FncCalls;
    int _NumFncNames;
    int _NumFncCalls;
    int _ArrayLimit;
    int _ShortestName;
    Boolean _HasAnswer;
    Boolean _ShowAnswer;
    Boolean _SingleFncCall;
    Boolean _ShowFunction;
    Boolean _ShowSetFunction;

    void ResizeArray();
    void AddFunctionCall(char* FncCall_, int* DelimArray_);
    char* SubExpressionTag(int FncIndex_, char* SrcExpr_, const char* AnswerStr_);
    void SubstFncAnswer(int FncIndex_);
    void SubstFncOriginal(int FncIndex_);
    static char* AllocateNewString(char* Buffer_, size_t NewLen_);

  public:
    SubExpression();
    SubExpression(SubExpression* Parent_, int Level_, char* Expr_);
    SubExpression(const SubExpression& Obj_);
    ~SubExpression();

    void SetCalculator(CalculatorBase* CalcPtr_);
    CalculatorBase* GetCalculator();

    Boolean ShowAnswer() const
        { return _ShowAnswer; }
    Boolean HasAnswer() const
        { return _HasAnswer; }
    int GetLevel() const
        { return _Level; }
    const char* GetOriginal() const
        { return _OriginalStr; }
    const char* GetExpression() const
        { return _Expression; }
    const char* GetAnswer() const
        { return _AnswerStr; }
    void SetHasAnswer(Boolean HasAnswer_)
        { _HasAnswer = HasAnswer_; }
    void SetShowAnswer(Boolean ShowAns_)
        { _ShowAnswer = ShowAns_; }
    void SetLevel(int Level_)
        { _Level = Level_; }

    Boolean HasCommandOutput() const;
    void AddFunctionName(char* FncName_);
    void SetAnswer(const char* Ans_);
    void SetExpression(const char* Expr_);
    void SetOriginal(const char* Expr_);
    void FindFunctionCalls();
    void ShowResults();
    const char* Evaluate();
    void Clear();

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
struct FncParameter
{
  SubExpression* _Param;
  int _ShouldEval;
  int _ShouldReplace;

  FncParameter();
  FncParameter(const FncParameter& Obj_);
  ~FncParameter();

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
class FunctionCall
{
  protected:
    SubExpression* _Parent;
    char* _FncName;
    int _NumParams;
    int _MaxParams;
    FncParameter** _ParamArray;
    char* _AnswerStr;
    char* _LiteralAnsStr;
    int _OutputMode;
    int _ShouldShow;

  public:
    FunctionCall();
    FunctionCall(SubExpression* Parent_, char* Name_);
    ~FunctionCall();

    const char* GetFncName() const
        { return _FncName; }
    const char* GetAnswer() const
        { return _AnswerStr; }
    const char* GetLiteralAnswer() const
        { return _LiteralAnsStr; }
    int ShowAnswer() const
        { return _ShouldShow; }
    int OutputMode() const
        { return _OutputMode; }

    void ReplaceParameter(int Index_, CalculatorBase* CalcPtr_,
                          int& Error_, int& ReEvaluate_, int ReFormat_, int ChangeModes_=FALSE);

    const char* GetOriginal() const;
    void SetShowAnswer(int Value_);
    void AddParameter(const char* Expr_, int FindCalls_, int ReplExpr_);
    void SetNumParams(int Params_);
    void Evaluate();
    void Clear();

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
struct LoopParams
{
  char* _Variable;
  long _Begin;
  long _End;
  long _Step;

  char* _BeginStr;
  char* _EndStr;
  char* _StepStr;

  LoopParams();
  LoopParams(const char* Variable_, long Beg_, long End_, long Step_);
  ~LoopParams();

  void SetVariable(const char* Var_);
  void SetBegin(long Begin_);
  void SetEnd(long End_);
  void SetStep(long Step_);

  void SetBegin(const char* Begin_);
  void SetEnd(const char* End_);
  void SetStep(const char* Step_);

  char* SetVarToBegin(char* Buffer_);
  char* CheckVarEqualEnd(char* Buffer_);
  char* IncrVarWithStep(char* Buffer_);
  char* CheckVarGreaterThanEnd(char* Buffer_);
  char* CheckVarLessThanEnd(char* Buffer_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
struct MenuItem
{
  char* _Text;
  char* _Label;
  MenuItem* _Next;

  MenuItem();
  MenuItem(const char* Text_, const char* Label_);
  ~MenuItem();

  void SetText(const char* Text_);
  void SetLabel(const char* Label_);
  void SetNext(MenuItem* Next_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
struct Menu
{
  char* _Title;
  MenuItem* _Item;

  Menu();
  Menu(const char* Title_);
  ~Menu();

  void SetTitle(const char* Title_);
  void AddMenuItem(MenuItem* Item_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
struct CommandType
{
  enum
  {
    NO_TYPE      = 0,
    IF_THEN      = 1,
    IF_THEN_ELSE = 2,
    FOR          = 3,
    WHILE        = 4,
    REPEAT       = 5,
    LABEL        = 6,
    GOTO         = 7,
    INCSKIP      = 8,
    DECSKIP      = 9,
    RETURN       = 10,
    STOP         = 11,
    DISPLAY      = 12,
    GRAPH_EXPR   = 13,
    GRAPH_FMIN   = 14,
    GRAPH_FMAX   = 15,
    GRAPH_DERIV  = 16,
    GRAPH_INTEG  = 17,
    GRAPH_ROOT   = 18,
    PROMPT       = 19,
    INPUT        = 20,
    INPUT_PROMPT = 21,
    PAUSE        = 22,
    PAUSE_VALUE  = 23,
    MENU         = 24,
    CONDITION    = 25,
    PROGRAM      = 26,
    COMMAND      = 27,
    END          = 28,
    ELSE         = 29,
    IF           = 30,
    CALCINPUT    = 31,
    NOOP         = 32
  };
};

/****************************************************************************/
struct IoArguments
{
  enum { ARRAYINCR = 20 };

  size_t _MaxArgs;
  size_t _NumArgs;
  char** _Arguments;

  IoArguments();
  ~IoArguments();

  void Resize();
  void AddArgument(const char* Name_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
struct EndToken
{
  int _Type;
  size_t _Startline;
  size_t _Endline;
  size_t _Depth;
  EndToken* _Next;

  EndToken();
  EndToken(EndToken* Token_);

  int Encloses(EndToken* Token_);
  int IsEnclosedBy(EndToken* Token_);
  int Matches(EndToken* Token_);
  void SetData(EndToken* Token_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
struct CtrlExecData
{
  int _Executed;
  size_t _LineNo;
};

/****************************************************************************/
class CalcProgram;

class ProgramLine
{
  protected:
    char* _Line;
    int _Type;
    int _Key;
    size_t _BlockSize;
    size_t _AltBlockSize;
    int _ClearInput;
    FILE* _Fout;
    CalcProgram* _Parent;

    union
    {
      Menu* _Menu;
      EndToken* _End;
      LoopParams* _Loop;
      IoArguments* _IoFncArgs;
      char* _Condition;
      char* _Label;
      char* _Name;
    };

  public:
    ProgramLine();
    ProgramLine(const char* Line_, CalcProgram* Ptr_);
    ~ProgramLine();

    void SetLine(const char* Line_);
    void SetType(int Type_);
    void SetBlockSize(size_t Size_);
    void SetAltBlockSize(size_t Size_);

    void SetMenu(Menu* Menu_);
    void SetEndToken(EndToken* End_);
    void SetLoopParams(LoopParams* Loop_);
    void SetIoArguments(IoArguments* Args_);
    void SetCondition(const char* Cond_);
    void SetLabel(const char* Label_);
    void SetName(const char* Name_);
    void SetParent(CalcProgram* Ptr_);

    void WriteCondition();
    void WriteCommand();
    void SetVarToBegin();
    void CheckVarEqualEnd();
    void IncrVarWithStep();
    void CheckVarGreaterThanEnd();
    void CheckVarLessThanEnd();

    int GetType()
        { return _Type; }
    const char* GetLine()
        { return _Line; }
    const char* GetCondition()
        { return ((_Key == CommandType::CONDITION) ? _Condition:NULL); }
    const char* GetLabel()
        { return ((_Key == CommandType::LABEL) ? _Label:NULL); }
    const char* GetName()
        { return ((_Key == CommandType::PROGRAM) ? _Name:NULL); }
    Menu* GetMenu()
        { return ((_Key == CommandType::MENU) ? _Menu:NULL); }
    EndToken* GetEndToken()
        { return ((_Key == CommandType::END) ? _End:NULL); }
    LoopParams* GetLoopParams()
        { return ((_Key == CommandType::FOR) ? _Loop:NULL); }
    IoArguments* GetIoArguments()
        { return ((_Key == CommandType::PROMPT) ? _IoFncArgs:NULL); }
    size_t GetBlockSize()
        { return _BlockSize; }
    size_t GetAltBlockSize()
        { return _AltBlockSize; }

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/****************************************************************************/
class CalcProgram
{
  friend class ProgramLine;

  protected:
    enum
    { LINEINCR = 20 };

    char* _Name;
    size_t _NumLines;
    size_t _NumSrcLines;
    size_t _MaxLines;
    ProgramLine** _Lines;
    char** _SourceLines;
    size_t _OutputBufferSize;
    char* _OutputBuffer;
    int _DoQuit;
    int _QuitAll;
    CalcProgram* _NextProg;
    int _IostateFileReadError;
    int _ProgramLoaded;
    EndToken* _Stack;
    size_t _NumNodes;
    size_t _Executed;
    CalculatorBase* _CalcPtr;
    int _QuitCalculator;
    int _InputReceived;

    // default file names
    char* LOG_FILE;                             // Error log file name
    char* USER_INPUT_FILE;                      // User input file
    char* USER_PROMPT_FILE;                     // User prompt file
    char* PROG_OUTPUT_FILE;                     // Program output file
    char* PROG_IOSTATE_FILE;                    // Program I/O state file
    char* SERVERALIVE_FILE;                     // Server keep alive file for async. mode
    char* CLIENTALIVE_FILE;                     // Client keep alive file for async. mode
    char* CURRENTGRAPH_FILE;                    // Current graph file

    void ResizeOutputBuffer();

    void Push(int Type_, size_t Start_, size_t End_);
    void Push(EndToken* End_);
    EndToken* Pop();

    int FreadString(char** buffer, size_t& MaxSize_, char terminator, FILE* Finput_);
    void SetFilenames(const char* LogFile_, const char* InputFile_, const char* PromptFile_,
                      const char* ProgOutFile_, const char* IostateFile_, const char* CurrentGraphFile_,
                      const char* ServerAliveFile_, const char* ClientAliveFile_);
    void SetFilenames(CalculatorBase* CalcPtr_);
    long FileSize(FILE* Fptr_);

    void Resize();
    void AddLine(ProgramLine* Line_);
    int FindEnd(size_t Start_, size_t& LineNo_);
    int FindElse(size_t Start_, size_t& LineNo_);
    char* FindArguments(char* Source_, char* Buffer_);
    void Execute_Command(const char* Str_);

    CtrlExecData Execute_If_Then(size_t x);
    CtrlExecData Execute_If_Then_Else(size_t x);
    CtrlExecData Execute_For(size_t x);
    CtrlExecData Execute_While(size_t x);
    CtrlExecData Execute_Repeat(size_t x);
    size_t Execute_If(size_t x);
    size_t Execute_Else(size_t x);
    size_t Execute_Goto(size_t x);
    size_t Execute_IncSkip(size_t x);
    size_t Execute_DecSkip(size_t x);
    size_t Execute_Menu(size_t x);
    void Execute_Prompt(size_t x);
    void Execute_Input(size_t x);
    void Execute_Pause(size_t x);
    void Execute_Prgm(size_t x);
    void Execute_Display(size_t x);
    void Execute_Command(size_t x);
    void Execute_CalcInput(size_t x);
    size_t FindNextBlock(size_t x);
    size_t Execute_Block(size_t Start_, size_t End_);
    size_t CheckTerminateNested(size_t x, CtrlExecData ExecData_, int& DoBreak_);
    int CheckTerminate(size_t x);

    int CheckCondition();
    const char* ReadFromOutput();
    const char* ReadFromUserInputFile();
    int WaitForGuiResponse();

    int EvalExpression(char* argstr);
    void ReceiveResponse(int Response_);
    void CheckDefaultInputIfQuit();

  public:
    CalcProgram();
    CalcProgram(CalculatorBase* CalcPtr_, const char* Name_);
    ~CalcProgram();

    void SetCalculator(CalculatorBase* CalcPtr_);
    CalculatorBase* GetCalculator();

    // File name retrieval
    const char* GetLogFilename() const;
    const char* GetUserInputFilename() const;
    const char* GetUserPromptFilename() const;
    const char* GetProgramOutputFilename() const;
    const char* GetProgramIostateFilename() const;
    const char* GetCurrentGraphFilename() const;

    void UnloadProgram();
    int QuitCalc() const
        { return _QuitCalculator; }
    int QuitAll() const
        { return _QuitAll; }
    int ProgramLoaded() const
        { return _ProgramLoaded; }

    int HasErrors() const;
    int LoadFile(const char* FileName_);
    int LoadBuffer(char* ArrayBuf_);
    void AddSourceLine(const char* Name_);
    void SetName(const char* Name_);
    void RunProgram();
    void Compile();
    void Execute();

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

/******************************* Prototypes *********************************/
class CalculatorBase
{
  friend class SymbolListData;
  friend class SubExpression;
  friend class FunctionCall;
  friend class CalcProgram;

  friend class FloatCalculator;
  friend class DoubleCalculator;
  friend class LongDoubleCalculator;
  friend class LongNumberCalculator;

  friend char* Mcalc_DefaultInFnc(char*, int);
  friend int Mcalc_DefaultOutFnc(const char*);

  public:
    enum { MAX_INT_COORDS = 3 };

  protected:
    /* Program Constants */
    static const int MAXVALUES;      /* Maximum number of integer values including the result of the calculation */
    static const int RESULTINDEX;    /* index position in the operands array which contains the structure for storing the result of the calculation */

    /* Program Options */
    static const int SHOWPOS;        /* flag to show positive sign if value is positive */

    /* Operator symbols */
    static const Uchar POW_OP;
    static const Uchar SIGN_OP;
    static const Uchar SIN_OP;
    static const Uchar COS_OP;
    static const Uchar TAN_OP;
    static const Uchar ASIN_OP;
    static const Uchar ACOS_OP;
    static const Uchar ATAN_OP;
    static const Uchar SINH_OP;
    static const Uchar COSH_OP;
    static const Uchar TANH_OP;
    static const Uchar EXP_OP;
    static const Uchar LOG_OP;
    static const Uchar LOG10_OP;
    static const Uchar CEILING_OP;
    static const Uchar FLOOR_OP;
    static const Uchar ABS_OP;
    static const Uchar FRACPART_OP;
    static const Uchar INTPART_OP;
    static const Uchar SQR_OP;
    static const Uchar SQRT_OP;
    static const Uchar CUBE_OP;
    static const Uchar CUBERT_OP;
    static const Uchar MINUS_OP;
    static const Uchar RND_OP;
    static const Uchar COMB_OP;
    static const Uchar PERM_OP;
    static const Uchar COMBREP_OP;
    static const Uchar PERMREP_OP;
    static const Uchar LSTHANOREQ_OP;
    static const Uchar GRTHANOREQ_OP;
    static const Uchar EQUAL_OP;
    static const Uchar NOTEQUAL_OP;
    static const Uchar RAND_OP;
    static const Uchar XOR_OP;
    static const Uchar NOT_OP;
    static const Uchar AND_OP;
    static const Uchar OR_OP;
    static const Uchar INV_OP;
    static const Uchar SCI_OP;
    static const Uchar FACTORIAL_OP;

    /* Predefined constant symbols */
    static const Uchar PI_CONST;
    static const Uchar E_CONST;
    static const Uchar NAN_CONST;
    static const Uchar INF_CONST;
    static const Uchar ANS_CONST;
    static const Uchar ANSSTR_CONST;
    static const Uchar FRACANS_CONST;

    // Variable and number indexes
    static const Uchar VARIABLE_INDEX;
    static const Uchar NUMBER_INDEX;
    static const Uchar HEXPREFIX_INDEX;
    static const Uchar FUNCTION_INDEX;
    static const Uchar BINARYOPS_INDEX;
    static const Uchar UNARYOPS_INDEX;
    static const Uchar POSTOPS_INDEX;

    enum
    {
      PROCESSED = 0,
      READ = 1
    };

    enum
    {
      FILE_PATH_SIZE = 512,
      DELIM_ARRAY_SIZE = 512,
      INPUT_BUFFER_SIZE = 2048
    };

  protected:
    char* _AnswerStr;                           // Answer string
    char* _OriginalStr;                         // Original expression string
    char* _FractionStr;                         // Fractional answer result
    char* _LiteralStr;                          // Literal string answer result
    char* _ProgramStr;                          // Program response string
    int _DisplayOption;                         // display option
    int _AnswerMode;                            // Calculator answer format mode
    int _ModeFinalized;                         // format mode has been finalized, no changes allowed
    int _InitialMode;                           // Mode in initial state or after reset
    int _ProcessVars;                           // Process variables instead of displaying them
    int _ErrorValue;                            // zero indicates no errors
    int _SkipEvaluation;                        // Flag set to skip evaluation of expression
    int _NumberBase;                            // Number base 8 for octal, 10 for decimal, 16 for hex
    int _RecursionLimit;                        // Recursion limit when evaluating the contents of string variables
    double _GraphPrecision;                     // Graphing interval precision for graphing function
    int _SuppressOutput;                        // Suppress message display for error conditions
    int _HasCommandOut;                         // Has output for the current executed command
    char* _ExecuteProgram;                      // Name of program to execute or NULL if none
    Uchar* _FncAttribs;                         // Function attribute array

    // For file and asynchronous mode operation
    static int filestatus[MAXFILES];                   // array of file status flags
    static FILE** files;                               // array of file pointers
    static FILE* outfile;                              // pointer to the output file
    static FILE* logfile;                              // error log file pointer
    static FILE* userinputfile;                        // user input file for async. mode execution
    static FILE* userpromptfile;                       // user prompt file for async. mode execution
    static FILE* programoutputfile;                    // program output file for async. mode execution
    static FILE* iostatefile;                          // program I/O state file for async. mode execution
    static FILE* curgraphfile;                         // current graph suffix file
    static int _StreamSet;                             // input and output stream is set
    static int _ArgListSize;                           // size of argument list
    static int _UseSessionCnt;                         // Use session counter value
    static int _SessionNum;                            // Session number if specified to be used
    static int _AsyncMode;                             // Indicates in asynchronous mode or not
    static int _IoState;                               // I/O state value set internally
    static int _WaitForResponse;                       // Indicates if program should poll for response from client
    static int _IostateFileReadError;                  // Indicates read or file open error for I/O state file
    static int _RetryReadCount;                        // Counter for retry attempts to read I/O state file
    static int _KeepAlivePollValue;                    // Used for polling the client program with "keep alive" states
    static int _RetryPollCount;                        // Retry attempts counter for client program "keep alive" polling
    static int _KeepAlivePollError;                    // Error indicator for client program "keep alive" polling
    static int _StdInput;                              // input is standard stream input
    static int _StdOutput;                             // output is standard stream output
    static int _PlotGraph;                             // Plot function output values to file
    static int _PlottingGraph;                         // Indicates program in PlotFunction method
    static int _PlotAllPoints;                         // Plot all points regardless if approaching +inf/-inf
    static int _GraphingOp;                            // Indicate graphing operation
    static int _GraphType;                             // Indicate type of graph
    static int _ShouldQuit;                            // Quit calculator flag
    static int(*_OutFnc)(const char*);                 // pointer to default input function
    static char*(*_InFnc)(char*, int);                 // pointer to default output function
    static char** _ErrorMessageArray;                  // Error messages
    static int* _ErrorMessageArgCnt;                   // Error messages argument counts
    static char* _CurrentFunctionName;                 // Current calculator function name

    // Common static members for all calculators
    static double UPPER_LIMIT;                         // Upper limit for convergence test in root solving formulas
    static Uchar* _QLookAhead;                         // Queue look ahead variable for operators
    static int _OddRoot;                               // Indicates special case of odd root for negative numbers
    static char _PlotFileSuffix;                       // Letter suffix for graph data file for storing multiple graphs

    // Object instances and function names
    static int _Instances;                             // Instances of this class
    static int _NumFncNames;                           // Number of function names
    static int _ShortestFncName;                       // Shortest function name

    // default file names
    static char* LOG_FILE;                             // Error log file name
    static char* USER_INPUT_FILE;                      // User input file
    static char* USER_PROMPT_FILE;                     // User prompt file
    static char* PROG_OUTPUT_FILE;                     // Program output file
    static char* DATA_INPUT_FILE;                      // Data input file
    static char* DATA_OUTPUT_FILE;                     // Data output file
    static char* PROG_IOSTATE_FILE;                    // Program I/O state file
    static char* CURRENTGRAPH_FILE;                    // Current graph file
    static char* SERVERALIVE_FILE;                     // Server keep alive state for use in async. mode
    static char* CLIENTALIVE_FILE;                     // Client keep alive state for use in async. mode

    // Function attribute methods
    inline void SetAsUnary(Uchar Index_, int HexOk_=0) const
         { _FncAttribs[Index_] = UNARYFNC_CODE | (HexOk_ ? HEXOK:0); }
    inline void SetAsBinary(Uchar Index_, int HexOk_=0) const
         { _FncAttribs[Index_] = BINARYFNC_CODE | (HexOk_ ? HEXOK:0); }
    inline void SetAsConstant(Uchar Index_, int HexOk_=0) const
         { _FncAttribs[Index_] = CONSTANT_CODE | (HexOk_ ? HEXOK:0); }
    inline void SetAsVariable(Uchar Index_, int HexOk_=0) const
         { _FncAttribs[Index_] = VARIABLE_CODE | (HexOk_ ? HEXOK:0); }
    inline void SetAsNumber(Uchar Index_, int HexOk_=0) const
         { _FncAttribs[Index_] = NUMBER_CODE | (HexOk_ ? HEXOK:0); }
    inline void SetAsPrefix(Uchar Index_, int HexOk_=0) const
         { _FncAttribs[Index_] = PREFIX_CODE | (HexOk_ ? HEXOK:0); }
    inline void SetAsPostfix(Uchar Index_, int HexOk_=0) const
         { _FncAttribs[Index_] = POSTFIXFNC_CODE | (HexOk_ ? HEXOK:0); }

    inline int IsPostfix(Uchar Index_) const
         { return (_FncAttribs[Index_] & POSTFIX); }
    inline int IsBinary(Uchar Index_) const
         { return (_FncAttribs[Index_] & BINARY); }
    inline int IsUnary(Uchar Index_) const
         { return (_FncAttribs[Index_] & UNARY); }
    inline int IsHexOk(Uchar Index_) const
         { return (_FncAttribs[Index_] & HEXOK); }
    inline int IsNum(Uchar Index_) const
         { return (_FncAttribs[Index_] & ISNUM); }
    inline int IsOp(Uchar Index_) const
         { return (_FncAttribs[Index_] & ISOP); }
    inline int IsRval(Uchar Index_) const
         { return (_FncAttribs[Index_] & RVAL); }
    inline int IsLval(Uchar Index_) const
         { return (_FncAttribs[Index_] & LVAL); }

    /* initialization, deallocation procedures */
    void freearglist(char***, int);

    /* argument checking procedures */
    int CheckForVarDisplay(CalculatorBase* CalcPtr_, char** list, char* original,
                           int CheckLetterVars_, int& VarDisplayed_, int& EvalDone_);
    const char* ProgramName(char* Name_);
    int IsValidNum(const char* value);
    int IsValidOperator(Uchar* token);
    int IsAssignmentOp(Uchar* token, int Index_, int Dir_);
    static int HasOperandSymbols(char symbol, int HexOk_);
    int HasOperandSymbols(char symbol);
    static int HasHexPrefix(const char* Str_, int Index_);
    static int IsValidProcessedChar(char* Str_, int Index_, int IsOp_, int IsBinary_,
                                    int IsNum_, int IsPrefix_, int HexOk_);
    static int IsCalledFunctions(const char* Str_);
    static int IsValidInputChar(char* Str_, int Index_, int IsOp_, int IsPost_, int IsNum_, int HexOk_);
    int IsValidInputChar(char* Str_, int Index_, int IsOp_, int IsPost_, int IsNum_);
    static int IsThisChar(const char* Str_, int Index_, char Target_, int Dir_);
    static int IsFirstTrailingChar(char* list, int i, int len, int ch);
    static int IsLastLeadingChar(char* list, int i, int ch);
    static int IsFirstTrailingChar(char* list, int i, int len, int(*iscsetfn)(int));
    static int IsLastLeadingChar(char* list, int i, int(*iscsetfn)(int));
    static int IsVariable(char* Str_, int IsOp_, int IsBinary_, int IsNum_, int IsVar_, int Index_, int Dir_);
    static int IsGraphCmd(char* Str_, int Index_, int IsOp_, int IsNum_, int Lval_, int Rval_);
    static int IsNumberPrefix(char* Str_, int Index_, int IsOp_, int IsNum_, int IsPrefix_, int HexOk_, int Dir_);
    static int IsValidAssignment(char* list, int len, int cmplen, int index, int HexOk_);
    static int IsStringLiteral(const char* list, int& NewStart_, int& LiteralLen_, int Start_, int len, int InSetStr_);
    static int IsGraphableFunction(char* list, int len, int cmplen, int index, Uchar OpCode_);
    static int WholeWord(char* list, int len, int cmplen, int index,
                         int Lval_, int Rval_, int IsOp_, int IsNum_,
                         int HexOk_, int IsUnary_, int IsBinary_, int IsPost_,
                         int IsGraphable_);
    int WholeWord(char* list, int len, int cmplen, int index, Uchar OpCode_);

    int HasValidOpCodes(char symbol, int Base_);
    int HasUnresolvedNames(const char* Str_);
    static int IsConstSymbols(char symbol);
    static int IsFraction(const char* Str_);
    static int IsConstString(const char* Str_, int Index_);
    static int IsBinaryFunctions(const char* Str_, int Index_);
    static int IsBinaryOperators(char symbol);
    static int IsBinaryOpCodes(char ch_);
    static int IsUnaryFunctions(const char* Str_);
    static int IsUnaryOperators(char symbol);
    static int IsPostfixOperators(char symbol);
    static int IsUnaryOpCodes(char ch_);
    static int IsUnprocessedSymbols(char symbol);
    static int IsRightsideValue(const char* Str_, int Index_, int HexOk_);

    /* integer set procedures */
    static void ShowTheList(SimpleList<int>& x);
    static ChrString MakeTheList(SimpleList<int>& x);
    static int InSetNotation(const char* Str_);
    static int InSetNotation(const char* list, int& NewStart_, int& LiteralLen_, int Start_, int len);
    static Set<int>* MakeIntSet(const char* Str_);
    static int IsNullSetStr(const char* Str_);

    /* string manipulation procedures */
    int InQuotes(const char* str, int& Equal_, int& FirstQuote_, int& LastQuote_);
    Uchar* ToUchar(Uchar*, char);
    Uchar* ToUchar(Uchar*, const char*);
    int stringtoken(char*);
    void stripstring(int, char*, char**);
    char* clearleadingspaces(char**);
    void reversestr(char* str, int len);
    void stripws(char*);
    void BlankOut(char* Str_, int Start_, int End_);
    char* ConcatArglist(int argc, char** list, Boolean LeaveSpace_=FALSE);
    const char* GetFncNameFromOpcode(int Opcode_);

    /* file I/O procedures */
    static FILE* FileOpen(const char* Fname_, const char* Mode_);
    static int FileStreamOpen(ifstream& Fin_, const char* Fname_);
    int openfile(FILE** file, const char* name, const char* mode, int Retry_=FALSE);
    int freadstring(char**, char, FILE* Finput_);

    /* stack operation procedures */
    void push(Mcalc_Stack**, Mcalc_StackData);
    void enqueue(Mcalc_Queue**, Mcalc_StackData);
    Mcalc_StackData pop(Mcalc_Stack**);

    /* expression parsing procedures */
    int priority(Uchar* op);
    void pushop(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Mcalc_StackData op);

    // token detection procedures
    int isoperand(char* str);
    static int isrightbracket(Uchar tok);
    static int isleftbracket(Uchar tok);
    static int isvalidsymbol(Uchar symbol, int HexOk_);
    int isvalidsymbol(Uchar symbol);
    int isdelimiter(Uchar symbol);

    // token extraction procedures
    int getleftbracket(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Uchar* token);
    int getrightbracket(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Uchar* token, int& BalancedBrk_);
    int getoperator(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Uchar* token);
    int getoperand(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Uchar* token, char* buffer, int* index);

    // Math expression evaluation procedures
    static int FindFncDelim(char* list, size_t len, int** DelimArrayPtr_, size_t ArraySize_);
    int LookAhead(const char* list);
    int FindAssignments(char* list, CalculatorBase* CalcPtr_, int Length_,
                        int& Error_, int RecLevel_, int& StrAssign_, int& StrLength_,
                        int SetIndexes_, AssignmentParsingData& ParseData_);
    int ReplaceTokens(char** list, CalculatorBase* CalcPtr_, int FindAssign_,
                      int& Error_, int& ReEvaluate_, int ReFormat_, int ChangeModes_, int Level_=0);
    int formatlist(char** list, CalculatorBase* CalcPtr_);

    // memory management procedures
    void freeoperand(Mcalc_Operand**);
    Mcalc_Operand* allocateoperand(Mcalc_Operand**, int);
    Mcalc_Operand* clearvalue(Mcalc_Operand*);
    Mcalc_Operand* allocatevalue(Mcalc_Operand*, int);
    Mcalc_Operand* reallocatevalue(Mcalc_Operand*, int);
    void* allocmem(unsigned, unsigned, void*);
    void* reallocmem(unsigned, unsigned, void*);

    // string manipulation procedures
    static int icasestrcompn(const char*, const char*, int);
    static int icasestrcomp(const char* s1, const char* s2);

    // initialization, deallocation procedures
    void initarrays(Mcalc_Operand***, char***);
    void freememory(Mcalc_Operand***);
    void freeargs(char***);
    void freelist(char***);

    // display procedures
    void DumpVariables(CalculatorBase* CalcPtr_);
    void DisplayToString(Mcalc_Operand**, char*);
    void DisplayAnswerStringHelper(FILE* Fptr_);
    void DisplayAnswerString();
    void DisplayToProgramOutput(const char* Str_);
    void DisplayToUserPrompt(const char* Str_);
    void DisplayMessage(FILE* Fptr_, const char* FormatStr_, const char* Str_);
    void DisplayMessage(FILE* Fptr_, const char* Str_);
    void DisplayErrorMessage(FILE* Fptr_, int ErrVal_, const char* FormatStr_, const char* Str_);
    void DisplayErrorMessage(FILE* Fptr_, int ErrVal_, const char* Str_);
    void DisplayErrorMessageForCode(FILE* Fptr_, int ErrVal_);

    ChrString DisplayVariable(CalculatorBase* CalcPtr_, char* Original_, int Index_, Boolean SetIoState_=TRUE);
    ChrString ShowFracDivZero(const LongNumber& TotalNumer_, const LongNumber& TotalDenom_);

    // help procedure
    void help(int argc, char* argv[]);
    void helpmessage(FILE* fp, const char* msg_);

    // file I/O procedures
    void getfiles(int*, char**, int*);
    int readfile(int*, char**, char***, int);
    void closefiles(void);
    FILE* GetOutputFile();
    FILE* GetLogFile();
    void CloseOutputFile();
    Ldouble* StatFileToArray(ifstream& Fin_, size_t& sz);

    // Error checking methods
    void PollClientIfAlive();
    void ReceiveKeepAlivePoll();

    // stack operation procedures
    void delstack(Mcalc_Stack**);
    void delqueue(Mcalc_Queue**);

    // expression parsing procedures
    void pushresult(Mcalc_Operand** operands, Mcalc_Stack** rpnstack, int optype);

    // Math expression evaluation procedures
    int InfixToRpn(int argc, char** list, Mcalc_Queue** rpnqueue, Mcalc_Stack** opstack, char** original, CalculatorBase* CalcPtr_);
    int evaluateRPN(Mcalc_Queue** rpnq, Mcalc_Stack** rpnstack, char** list, int* optype);
    void PushStringCompareTokens(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, const char* buffer, int Start_, int End_, int Optype_);
    int TestForStringComparison(const char* buffer, int& Start_, int& End_, int& Optype_);
    int FindComparisonEndPts(const char* buffer, int& Start_, int& End_);
    int CompareStrings(const char* String1_, const char* String2_, int Optype_);
    int HasVariables(const char* Buffer_);

    // virtual math methods implemented in derived classes
    virtual ChrString FindIntersections(int NumParams_, FncParameter** ParamArray_) = 0;
    virtual ChrString FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_) = 0;
    virtual ChrString EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_) = 0;
    virtual ChrString FindDerivative(int NumParams_, FncParameter** ParamArray_) = 0;
    virtual ChrString FindDerivative2(int NumParams_, FncParameter** ParamArray_) = 0;
    virtual ChrString FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_) = 0;
    virtual ChrString NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_) = 0;
    virtual ChrString FindRoot(int NumParams_, FncParameter** ParamArray_) = 0;
    virtual ChrString FindMinimum(int NumParams_, FncParameter** ParamArray_) = 0;
    virtual ChrString FindMaximum(int NumParams_, FncParameter** ParamArray_) = 0;
    virtual ChrString RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_) = 0;
    virtual ChrString PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_) = 0;

    // math helper methods
    long FindNumIntervals(int NumParams_, FncParameter** ParamArray_);

    // Adding function names method
    void AddFunctionName(const char* FncName_);

    // Plot all points
    static void SetPlotAllPoints(int Flag_);

  public:
    enum
    {
      NO_CALC = 0,
      FLOAT = 1,
      DOUBLE = 2,
      LONGDOUBLE = 3,
      LONGNUM = 4,
      MAXCALC = 5,
      MAXVARS = 26,
      STACKSIZE = 64,
      MAXFUNCTIONS = 100,
      RETRY_READ_MAX = 5,
      RECURSION_LIMIT = 26
    };

    enum
    {
      NO_ASSIGN         = 0,
      ASSIGN_ONLY       = 1,
      ASSIGN_PLUS       = 2,
      ASSIGN_MINUS      = 3,
      ASSIGN_MULT       = 4,
      ASSIGN_DIV        = 5,
      ASSIGN_MOD        = 6,
      ASSIGN_AND        = 7,
      ASSIGN_OR         = 8,
      STRING_ASSIGN     = 9,
      STRING_CONCAT     = 10,

      SET_ASSIGN        = 11,
      SET_UNION         = 12,
      SET_INTERSECTION  = 13,
      SET_NEGATION      = 14,
      SET_EXCLUSION     = 15
    };

    enum
    {
      DEGREE,
      RADIAN
    };

    enum
    {
      DISPLAY_TO_STRING,
      DISPLAY_TO_STREAM
    };

    enum
    {
      STRING_ASSIGNMENT = 1,
      STRING_COMPARISON = 2,
      SET_ASSIGNMENT    = 3,
      SET_OPERATION     = 4
    };

    enum
    {
      FLOATINGPOINT_MODE = 101,
      LITERAL_MODE       = 102,
      FRACTION_MODE      = 103,
      ALTBASE_MODE       = 104
    };

    enum
    {
      VARIABLE_CODE      = 0x60,
      UNARYFNC_CODE      = 0x32,
      NUMBER_CODE        = 0x2C,
      PREFIX_CODE        = 0x2A,
      CONSTANT_CODE      = 0x20,
      BINARYFNC_CODE     = 0x11,
      POSTFIXFNC_CODE    = 0x94,
      BINARY             = 0x01,
      UNARY              = 0x02,
      HEXOK              = 0x04,
      ISNUM              = 0x08,
      ISOP               = 0x10,
      RVAL               = 0x20,
      LVAL               = 0x40,
      POSTFIX            = 0x80
    };

    typedef int(*InputFncType)(const char*, const char*);
    typedef char*(*OutputFncType)(char*, int);

    static char** FUNCTION_NAMES;
    static int NumFncNames();
    static int ShortestFncName();

    CalculatorBase();
    virtual ~CalculatorBase();

    // File name retrieval
    static const char* GetLogFilename();
    static const char* GetUserInputFilename();
    static const char* GetUserPromptFilename();
    static const char* GetProgramOutputFilename();
    static const char* GetDataInputFilename();
    static const char* GetDataOutputFilename();
    static const char* GetProgramIostateFilename();
    static const char* GetCurrentGraphFilename();
    static const char* GetServerAliveFilename();
    static const char* GetClientAliveFilename();

    // Graph plotting
    static void SetPlotGraph(int Flag_);
    static void SetGraphType(int Type_);

    // Session management methods
    static int UsesSessionNum();
    static int SessionNum();
    static int InAsyncMode();

    void SetUsesSessionNum(int Num_);
    static void SetSessionNum(int Num_);
    void SetAsyncMode(int Flag_);
    void RemoveSessionFiles(int SesNum_);
    int ReadUserInput(int* argc, char*** list);
    void SetIoState(int State_);
    int GetIoState(int Sleep_);
    int CurrentIoState() const;
    long FileSize(FILE* Fptr_);
    void SetWaitForResponse(int Flag_);
    int WaitForResponse() const;
    void ClearOutputFile();
    void ClearUserPromptFile()
        { DisplayToUserPrompt(""); }
    void ResetRetryRead();

    // virtual math methods implemented in derived classes
    static char PlotFileSuffix();
    static void SaveCurrentGraphName(const char Suffix_);
    virtual ChrString PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                   const ChrString& MidPtStr_, ChrString Radius_="5", ChrString* Solution_=NULL) = 0;
    virtual ChrString FindIntegral(int NumParams_, FncParameter** ParamArray_) = 0;
    virtual ChrString MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_) = 0;
    virtual ChrString SolveMatrix(int NumParams_, FncParameter** ParamArray_) = 0;

    // Subexpression methods
    virtual void CreateNewSubExpr() = 0;
    virtual void DestroySubExprStack() = 0;
    virtual void PopSubExprStack() = 0;
    virtual SubExpression* TopSubExprStack() = 0;
    virtual void ResizeExprStack() = 0;
    virtual size_t SubExprStackSize() const = 0;
    virtual ChrString EvalFncWithValue(const ChrString& ExprStr_, const ChrString& VarName_, const ChrString& Value_) = 0;
    virtual ChrString ReEvaluate(const ChrString& ExprStr_) = 0;
    virtual int AtLowestLevel(int Level_) = 0;

    // Error reporting and handling
    virtual int GetErrorVal() const = 0;
    virtual int HasErrors() const = 0;
    virtual void ClearError() = 0;
    virtual void SetError(int ErrVal_) = 0;
    virtual void SetSuppressOutput(int Flag_, int ShowSuppressedErrors_=TRUE) = 0;
    virtual int SuppressOutput() const = 0;

    virtual ChrString ToString(int Index_) = 0;
    virtual size_t GetVariableLength(int Index_) const = 0;
    virtual int GiveType() const = 0;
    virtual void SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_) = 0;
    virtual void ResetIndexes() = 0;
    virtual void SetIndex(int Pos_, int Val_) = 0;
    virtual void GetIndex(int& Pos_, int& Val_) = 0;
    virtual int HasIndex() = 0;
    virtual void PopIndex() = 0;
    virtual int StoreResults() = 0;
    virtual void SetAngleUnit(int Unit_) = 0;
    virtual int GetAngleUnit() const = 0;
    virtual CalculatorBase* SetupCalculators(int CalcType_) = 0;

    virtual int ProcessCmdLine(int argc, char** argv) = 0;
    virtual void EvaluateLine(int argc, CalculatorBase* CalcPtr_, Mcalc_Operand** operands, char** arglist) = 0;
    virtual void FileInput(int argc, char** argv, CalculatorBase* CalcPtr_) = 0;
    virtual void ProgramProc(int argc, char** argv, CalculatorBase* CalcPtr_) = 0;
    virtual int TransferData(char* argstr, CalculatorBase* CalcPtr_) = 0;
    virtual int EvalExpression(char* argstr, CalculatorBase* CalcPtr_) = 0;
    virtual void SetHasCommandOutput(int Flag_) = 0;
    virtual Boolean HasCommandOutput() const = 0;
    virtual void SetCalcIndexes(int Prev_, int New_) = 0;
    virtual int TransferVariable(int DestIndex_, int SrceIndex_) = 0;
    virtual void SwitchCalcType(int CalcType_) = 0;
    virtual void SetCalcType(int CalcType_) = 0;
    virtual void TransferCalcType(int CalcType_) = 0;
    virtual int RestoreCalcType() = 0;
    virtual const char* GetPrecision() = 0;
    virtual void SetParent(CalculatorBase* Ptr_) = 0;
    virtual CalculatorBase* GetCalcType() = 0;
    virtual CalculatorBase* GetPrevCalcType() = 0;
    virtual int GetCalcIndex() const = 0;
    virtual int GetPrevCalcIndex() const = 0;
    virtual void ProcessExpr(Mcalc_Operand**, char**, int, CalculatorBase*) = 0;
    virtual void SetProcessVariables(int Value_) = 0;
    virtual int ProcessVariables() const = 0;
    virtual int InProgramMode() const = 0;
    virtual void SetNumberBase(int Base_) = 0;
    virtual void RestoreNumberBase() = 0;
    virtual int NumberBase() const = 0;
    virtual int RecursionLimit() const = 0;
    virtual void SetRecursionLimit(int Limit_) = 0;
    virtual double GraphPrecision() const = 0;
    virtual void SetGraphPrecision(double Prec_) = 0;
    void ClearGraphs();

    // I/O function pointers
    inline char* UseDefaultInputFnc(char* Buffer_, int Size_)
        { return (*_InFnc)(Buffer_, Size_); }
    inline int UseDefaultOutputFnc(const char* Buffer_)
        { return (*_OutFnc)(Buffer_); }
    inline void SetInputFnc(char*(*FuncPtr_)(char*, int))
        { _InFnc = FuncPtr_; }
    inline void ResetInputFnc()
        { _InFnc = Mcalc_DefaultInFnc; }
    inline void SetOutputFnc(int(*FuncPtr_)(const char*))
        { _OutFnc = FuncPtr_; }
    inline void ResetOutputFnc()
        { _OutFnc = Mcalc_DefaultOutFnc; }
    inline int StdInput() const
        { return _StdInput; }
    inline int StdOutput() const
        { return _StdOutput; }

    // Calculator program termination
    inline void SetShouldQuit(int Flag_)
         { _ShouldQuit = Flag_; }
    inline int ShouldQuit() const
         { return _ShouldQuit; }

    virtual void SetDisplayOption(int Option_) = 0;
    virtual int GetDisplayOption() = 0;
    virtual int GiveRandom(int Num_) = 0;

    virtual CalcProgram* ActiveProgram() = 0;
    virtual void PushActiveExpression(SubExpression* Ptr_) = 0;
    virtual void PopActiveExpression() = 0;
    virtual void PushActiveProgram(CalcProgram* Ptr_) = 0;
    virtual void PopActiveProgram() = 0;
    virtual void SetHasAnswer(Boolean Flag_) = 0;
    virtual void SetShowAnswer(Boolean Flag_) = 0;
    virtual void ResetAnswerMode() = 0;
    virtual void SetAnswerMode(int Mode_, int Final_=FALSE) = 0;
    virtual void SetSkipEval(int Flag_) = 0;
    virtual int ModeFinalized() const = 0;
    virtual int InitialMode() const = 0;
    virtual int AnswerMode() const = 0;
    virtual int SkipEval() const = 0;
    virtual int HasAnswer() const = 0;
    virtual int ShowAnswer() const = 0;

    virtual void SetStringAssignment(int Flag_, const char* Str_=NULL) = 0;
    virtual int StringAssignment() const = 0;
    virtual void SetStringComparison(int Flag_) = 0;
    virtual int StringComparison() const = 0;
    virtual void SetStringVar(int Index_, Boolean Flag_) = 0;
    virtual Boolean IsStringVar(int Index_) const = 0;

    virtual void SetSetAssignment(int Flag_, const char* Str_, int Null_) = 0;
    virtual int SetAssignment() const = 0;
    virtual void SetSetOperation(int Flag_) = 0;
    virtual int SetOperation() const = 0;
    virtual void SetSetVar(int Index_, int Flag_) = 0;
    virtual int IsSetVar(int Index_) const = 0;
    virtual int IsNullSet(int Index_) const = 0;

    virtual void SetEvalStrings(int Flag_) = 0;
    virtual int EvalStrings() const = 0;
    virtual void SetStrVariable(int x, const ChrString& Str_) = 0;
    virtual void SetSetVariable(int x, const ChrString& Str_, int Null_) = 0;
    virtual const ChrString& GetStrVariable(int Index_) const = 0;
    virtual void SetStrResult(const char* Str_) = 0;
    virtual const ChrString& GetStrResult() const = 0;
    virtual const ChrString& GetSetVariable(int Index_) const = 0;
    virtual void SetSetResult(const char* Str_, int Null_) = 0;
    virtual const ChrString& GetSetResult() const = 0;

    virtual size_t GetStrVarLen(int Index_) const = 0;
    virtual char* CopyStrVar(char* DestStr_, int Index_) = 0;
    virtual ChrString& CopyStrVar(ChrString& DestStr_, int Index_) = 0;
    virtual Boolean IsHexVar(int Index_) const = 0;
    virtual Boolean IsOctVar(int Index_) const = 0;

    virtual void ClearAnswerStr() = 0;
    virtual void ClearOriginalStr() = 0;
    virtual const char* AnswerStr() const = 0;
    virtual void SetLiteralStr(const char* Literal_) = 0;
    virtual const char* LiteralStr() const = 0;
    virtual void SetFractionStr(const char* Frac_) = 0;
    virtual const char* FractionStr() const = 0;
    virtual void SetFloatStr(const char* Str_) = 0;
    virtual const char* FloatStr() const = 0;
    virtual void SetOriginalStr(const char* Str_) = 0;
    virtual const char* OriginalStr() const = 0;
    virtual void SetProgramStr(const char* Str_) = 0;
    virtual const char* ProgramStr() const = 0;
    virtual void PromptAsyncUser(const char* Str_, int GetResponse_);

    // statistics methods
    virtual Ldouble MinimumValue(int NumParams_, FncParameter** ParamArray_);
    virtual Ldouble MaximumValue(int NumParams_, FncParameter** ParamArray_);
    virtual Ldouble CumSum(int NumParams_, FncParameter** ParamArray_);
    virtual Ldouble CumProduct(int NumParams_, FncParameter** ParamArray_);
    virtual Ldouble AverageMean(int NumParams_, FncParameter** ParamArray_);
    virtual Ldouble AverageMedian(int NumParams_, FncParameter** ParamArray_);
    virtual Ldouble AverageMode(int NumParams_, FncParameter** ParamArray_);
    virtual Ldouble StandardDeviation(int NumParams_, FncParameter** ParamArray_);
};

// Implement storage of quoted literal strings into letter vars: $a - $z
//
struct StrVarData
{
  StrVarData();
  ~StrVarData();

  void SetSetElements(int Index_, const ChrString& Str_, int Null_);
  void SetString(int Index_, const ChrString& Str_);
  const ChrString& GetVariable(int Index_) const;
  const ChrString& GetResult() const;
  const ChrString& GetSetVariable(int Index_) const;
  const ChrString& GetSetResult() const;
  void SetResult(const char* Str_);
  void SetSetResult(const char* Str_, bool Null_);

  size_t GetVarLen(int Index_) const;
  char* CopyStrVar(char* DestStr_, int Index_);
  ChrString& CopyStrVar(ChrString& DestStr_, int Index_);

  void SetStringVar(int Index_, Boolean Flag_);
  Boolean IsStringVar(int Index_) const;
  void SetSetVar(int Index_, int Flag_);
  int IsSetVar(int Index_) const;
  int IsNullSet(int Index_) const;

  int _SetResult;
  int _IsSetVar[CalculatorBase::MAXVARS];
  Boolean _IsStringVar[CalculatorBase::MAXVARS];
  ChrString* _strvars[CalculatorBase::MAXVARS];  // for string literal assignments into variables
  ChrString _StrResult;
};

typedef PointerStack<SubExpression> ExpressionStack;
typedef PointerStack<CalcProgram> ProgramStack;

template <class T>
struct IntCoords
{
  T X;
  T Y;

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

template <class T>
class LineCalculator : public CalculatorBase
{
  protected:
    StrVarData* _StrVarData;
    int _StringOperation;     // Indicates string literal assignments to variables
    int _SetOperation;        // Indicates set contents assignment to variables
    int _EvalStrings;         // Evaluate contents of string variables if set true

    int _AngleUnit;
    char* _AssignmentOrder;
    int _StackSize;
    int _CalcIndex;
    int _PrevCalcIndex;
    CalculatorBase* _CalcPtrArray[MAXCALC];
    CalculatorBase* _ParentPtr;
    Randomizer<float> _Random;

    SubExpression** _SubExprStack;
    SubExpression* _Expression;
    size_t _ExprStackIndex;
    size_t _ExprStackLimit;
    ExpressionStack* _ActiveExpression;
    ProgramStack* _ActiveProgram;
    LongNumber _TransferValue;

    LineCalculator();

    // math helper methods
    char* ConvertFloatToFraction(char* Float_);
    char* ConvertIntegerToFraction(char* Int_);

    // Set math methods
    ChrString CreateSet(int NumParams_, FncParameter** ParamArray_);
    ChrString FindSetUnion(int NumParams_, FncParameter** ParamArray_);
    ChrString FindSetIntersection(int NumParams_, FncParameter** ParamArray_);
    ChrString FindSetDifference(int NumParams_, FncParameter** ParamArray_);
    ChrString FindSetExclusion(int NumParams_, FncParameter** ParamArray_);

    // Discrete math methods
    ChrString FindPermutations(int NumParams_, FncParameter** ParamArray_);
    ChrString FindCombinations(int NumParams_, FncParameter** ParamArray_);
    ChrString FindPermutationsWithRep(int NumParams_, FncParameter** ParamArray_);
    ChrString FindCombinationsWithRep(int NumParams_, FncParameter** ParamArray_);
    ChrString FindFractionArithmetic(int NumParams_, FncParameter** ParamArray_);

    // Base conversion methods
    ChrString ConvertFromHex(int NumParams_, FncParameter** ParamArray_);
    ChrString ConvertFromOct(int NumParams_, FncParameter** ParamArray_);
    ChrString ConvertFromBin(int NumParams_, FncParameter** ParamArray_);
    ChrString ConvertFromCmp2Bin(int NumParams_, FncParameter** ParamArray_);
    ChrString ConvertToHex(int NumParams_, FncParameter** ParamArray_);
    ChrString ConvertToOct(int NumParams_, FncParameter** ParamArray_);
    ChrString ConvertToBin(int NumParams_, FncParameter** ParamArray_);
    ChrString ConvertToCmp2Bin(int NumParams_, FncParameter** ParamArray_);

    // Bitwise operations
    ChrString BitwiseOps(const LongNumber& value1, const LongNumber& value2, int optype);

  public:
    virtual ~LineCalculator();

    virtual ChrString ToString(int Index_) = 0;
    virtual const T& DirectProcess(Mcalc_Operand**, char**, int) = 0;
    virtual const T& GetResult() const = 0;
    virtual const T& GetVariable(int Index_) const = 0;
    virtual void SetVariable(int Index_, const T& Value_, int Tranfer_=0) = 0;

    // Subexpression methods
    virtual void CreateNewSubExpr();
    virtual void DestroySubExprStack();
    virtual void PopSubExprStack();
    virtual SubExpression* TopSubExprStack();
    virtual void ResizeExprStack();
    virtual size_t SubExprStackSize() const;
    virtual ChrString EvalFncWithValue(const ChrString& ExprStr_, const ChrString& VarName_, const ChrString& Value_);
    virtual ChrString ReEvaluate(const ChrString& ExprStr_);
    virtual int AtLowestLevel(int Level_);

    virtual CalculatorBase* SetupCalculators(int CalcType_);
    virtual void ResetIndexes();
    virtual void SetIndex(int Pos_, int Val_);
    virtual void GetIndex(int& Pos_, int& Val_);
    virtual int HasIndex();
    virtual void PopIndex();
    virtual int StoreResults();
    virtual void SetAngleUnit(int Unit_);
    virtual int GetAngleUnit() const;

    virtual int ProcessCmdLine(int argc, char** argv);
    virtual void EvaluateLine(int argc, CalculatorBase* CalcPtr_, Mcalc_Operand** operands, char** arglist);
    virtual void FileInput(int argc, char** argv, CalculatorBase* CalcPtr_);
    virtual void ProgramProc(int argc, char** argv, CalculatorBase* CalcPtr_);
    virtual int TransferData(char* argstr, CalculatorBase* CalcPtr_);
    virtual int EvalExpression(char* argstr, CalculatorBase* CalcPtr_);
    virtual void SetHasCommandOutput(int Flag_);
    virtual Boolean HasCommandOutput() const;
    virtual void SetCalcIndexes(int Prev_, int New_);
    virtual int TransferVariable(int DestIndex_, int SrceIndex_);
    virtual void SwitchCalcType(int CalcType_);
    virtual void SetCalcType(int CalcType_);
    virtual void TransferCalcType(int CalcType_);
    virtual int RestoreCalcType();
    virtual const char* GetPrecision();
    virtual void SetParent(CalculatorBase* Ptr_);
    virtual CalculatorBase* GetCalcType();
    virtual CalculatorBase* GetPrevCalcType();
    virtual int GetCalcIndex() const;
    virtual int GetPrevCalcIndex() const;
    virtual void ProcessExpr(Mcalc_Operand**, char**, int, CalculatorBase*);
    virtual void SetProcessVariables(int Value_);
    virtual int ProcessVariables() const;
    virtual int InProgramMode() const;
    virtual void SetNumberBase(int Base_);
    virtual void RestoreNumberBase();
    virtual int NumberBase() const;
    virtual int RecursionLimit() const;
    virtual void SetRecursionLimit(int Limit_);
    virtual double GraphPrecision() const;
    virtual void SetGraphPrecision(double Prec_);

    // I/O function pointers
    virtual void SetDisplayOption(int Option_);
    virtual int GetDisplayOption();
    virtual int GiveRandom(int Num_);

    // Error reporting and handling
    virtual int GetErrorVal() const;
    virtual int HasErrors() const;
    virtual void ClearError();
    virtual void SetError(int ErrVal_);
    virtual void SetSuppressOutput(int Flag_, int ShowSuppressedErrors_=TRUE);
    virtual int SuppressOutput() const;

    virtual CalcProgram* ActiveProgram();
    virtual void PushActiveExpression(SubExpression* Ptr_);
    virtual void PopActiveExpression();
    virtual void PushActiveProgram(CalcProgram* Ptr_);
    virtual void PopActiveProgram();
    virtual void SetHasAnswer(Boolean Flag_);
    virtual void SetShowAnswer(Boolean Flag_);
    virtual void ResetAnswerMode();
    virtual void SetAnswerMode(int Mode_, int Final_=FALSE);
    virtual void SetSkipEval(int Flag_);
    virtual int InitialMode() const;
    virtual int ModeFinalized() const;
    virtual int AnswerMode() const;
    virtual int SkipEval() const;
    virtual int HasAnswer() const;
    virtual int ShowAnswer() const;

    virtual void SetStringAssignment(int Flag_, const char* Str_=NULL);
    virtual int StringAssignment() const;
    virtual void SetStringComparison(int Flag_);
    virtual int StringComparison() const;
    virtual void SetStringVar(int Index_, Boolean Flag_);
    virtual Boolean IsStringVar(int Index_) const;

    virtual void SetSetAssignment(int Flag_, const char* Str_, int Null_);
    virtual int SetAssignment() const;
    virtual void SetSetOperation(int Flag_);
    virtual int SetOperation() const;
    virtual void SetSetVar(int Index_, int Flag_);
    virtual int IsSetVar(int Index_) const;
    virtual int IsNullSet(int Index_) const;

    virtual void SetEvalStrings(int Flag_);
    virtual int EvalStrings() const;
    virtual void SetStrVariable(int x, const ChrString& Str_);
    virtual void SetSetVariable(int x, const ChrString& Str_, int Null_);
    virtual const ChrString& GetStrVariable(int Index_) const;
    virtual void SetStrResult(const char* Str_);
    virtual const ChrString& GetStrResult() const;
    virtual const ChrString& GetSetVariable(int Index_) const;
    virtual void SetSetResult(const char* Str_, int Null_);
    virtual const ChrString& GetSetResult() const;

    virtual size_t GetStrVarLen(int Index_) const;
    virtual char* CopyStrVar(char* DestStr_, int Index_);
    virtual ChrString& CopyStrVar(ChrString& DestStr_, int Index_);
    virtual Boolean IsHexVar(int Index_) const;
    virtual Boolean IsOctVar(int Index_) const;

    virtual void ClearAnswerStr();
    virtual void ClearOriginalStr();
    virtual const char* AnswerStr() const;
    virtual void SetLiteralStr(const char* Literal_);
    virtual const char* LiteralStr() const;
    virtual void SetFractionStr(const char* Frac_);
    virtual const char* FractionStr() const;
    virtual void SetFloatStr(const char* Str_);
    virtual const char* FloatStr() const;
    virtual void SetOriginalStr(const char* Str_);
    virtual const char* OriginalStr() const;
    virtual void SetProgramStr(const char* Str_);
    virtual const char* ProgramStr() const;

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

class FloatCalculator : public LineCalculator<float>
{
  protected:
    float resultvalue;
    float _vars[MAXVARS];
    float _Epsilon;

    ChrString _IntExpr;                    // Intersect expression for plot file.
    IntCoords<float>** _IntCoords;         // Intersection coordinates for graph intersection function
    int _IntsFound;                        // Number of points of intersection found, 0 to 3
    int _ExtremaEnd;                       // Indicate whether finding min. or max. extrema

    static bool FindCommonAngles(float value_, float shift_, float Tol_, float Eps_, float& mult_, double& intp_);
    static size_t FindPrecision(const float& Val_, size_t Prec_);
    static void PrintInfNanValues(FILE* Fout_, const Float& Answer_, float fval);
    virtual ChrString FindIntersections(int NumParams, FncParameter** ParamArray_);
    virtual ChrString FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_);
    virtual ChrString EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindDerivative(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindDerivative2(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString FindRoot(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindMinimum(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindMaximum(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_);
    virtual ChrString PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_);
    static float IntPow(float Value_, long Exp_);

  public:
    FloatCalculator();
    virtual ~FloatCalculator();

    // virtual math methods implemented in derived classes
    virtual ChrString PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                   const ChrString& MidPtStr_, ChrString Radius_="5", ChrString* Solution_=NULL);
    virtual ChrString FindIntegral(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString SolveMatrix(int NumParams_, FncParameter** ParamArray_);

    virtual ChrString ToString(int Index_);
    virtual const float& DirectProcess(Mcalc_Operand**, char**, int);
    virtual const float& GetResult() const;
    virtual const float& GetVariable(int Index_) const;
    virtual size_t GetVariableLength(int Index_) const;
    virtual void SetVariable(int Index_, const float& Value1_, int Transfer_=0);
    virtual int GiveType() const;
    virtual void SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_);
    virtual void SetParent(CalculatorBase* Ptr_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

class DoubleCalculator : public LineCalculator<double>
{
  protected:
    double resultvalue;
    double _vars[MAXVARS];
    double _Epsilon;

    ChrString _IntExpr;                    // Intersect expression for plot file.
    IntCoords<double>** _IntCoords;        // Intersection coordinates for graph intersection function
    int _IntsFound;                        // Number of points of intersection found, 0 to 3
    int _ExtremaEnd;                       // Indicate whether finding min. or max. extrema

    static bool FindCommonAngles(double value_, double shift_, double Tol_, double Eps_, double& mult_, double& intp_);
    static size_t FindPrecision(const double& Val_, size_t Prec_);
    static void PrintInfNanValues(FILE* Fout_, const Double& Answer_, double fval);
    virtual ChrString FindIntersections(int NumParams, FncParameter** ParamArray_);
    virtual ChrString FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_);
    virtual ChrString EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindDerivative(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindDerivative2(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString FindRoot(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindMinimum(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindMaximum(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_);
    virtual ChrString PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_);
    static double IntPow(double Value_, long Exp_);

  public:
    DoubleCalculator();
    virtual ~DoubleCalculator();

    // virtual math methods implemented in derived classes
    virtual ChrString PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                   const ChrString& MidPtStr_, ChrString Radius_="5", ChrString* Solution_=NULL);
    virtual ChrString FindIntegral(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString SolveMatrix(int NumParams_, FncParameter** ParamArray_);

    virtual ChrString ToString(int Index_);
    virtual const double& DirectProcess(Mcalc_Operand**, char**, int);
    virtual const double& GetResult() const;
    virtual const double& GetVariable(int Index_) const;
    virtual size_t GetVariableLength(int Index_) const;
    virtual void SetVariable(int Index_, const double& Value1_, int Transfer_=0);
    virtual int GiveType() const;
    virtual void SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_);
    virtual void SetParent(CalculatorBase* Ptr_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

class LongDoubleCalculator : public LineCalculator<Ldouble>
{
  friend class LongNumCalculator;

  protected:
    Ldouble resultvalue;
    Ldouble _vars[MAXVARS];
    Ldouble _Epsilon;

    ChrString _IntExpr;                    // Intersect expression for plot file.
    IntCoords<Ldouble>** _IntCoords;       // Intersection coordinates for graph intersection function
    int _IntsFound;                        // Number of points of intersection found, 0 to 3
    int _ExtremaEnd;                       // Indicate whether finding min. or max. extrema

    #if HAS_LONGDOUBLE_STDFNCS
      static bool FindCommonAngles(Ldouble value_, Ldouble shift_, Ldouble Tol_, Ldouble Eps_, Ldouble& mult_, Ldouble& intp_);
    #else
      static bool FindCommonAngles(Ldouble value_, Ldouble shift_, Ldouble Tol_, Ldouble Eps_, Ldouble& mult_, double& intp_);
    #endif
    static size_t FindPrecision(const Ldouble& Val_, size_t Prec_);
    static void PrintInfNanValues(FILE* Fout_, const LongDouble& Answer_, Ldouble fval);
    virtual ChrString FindIntersections(int NumParams, FncParameter** ParamArray_);
    virtual ChrString FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_);
    virtual ChrString EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindDerivative(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindDerivative2(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString FindRoot(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindMinimum(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindMaximum(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_);
    virtual ChrString PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_);
    static Ldouble IntPow(Ldouble Value_, long Exp_);

  public:
    LongDoubleCalculator();
    virtual ~LongDoubleCalculator();

    // virtual math methods implemented in derived classes
    virtual ChrString PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                   const ChrString& MidPtStr_, ChrString Radius_="5", ChrString* Solution_=NULL);
    virtual ChrString FindIntegral(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString SolveMatrix(int NumParams_, FncParameter** ParamArray_);

    virtual ChrString ToString(int Index_);
    virtual const Ldouble& DirectProcess(Mcalc_Operand**, char**, int);
    virtual const Ldouble& GetResult() const;
    virtual const Ldouble& GetVariable(int Index_) const;
    virtual size_t GetVariableLength(int Index_) const;
    virtual void SetVariable(int Index_, const Ldouble& Value1_, int Transfer_=0);
    virtual int GiveType() const;
    virtual void SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_);
    virtual void SetParent(CalculatorBase* Ptr_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

class LongNumCalculator : public LineCalculator<LongNumber>
{
  protected:
    LongNumber resultvalue;
    LongNumber* _vars[MAXVARS];
    LongNumber _Epsilon;

    ChrString _IntExpr;                    // Intersect expression for plot file.
    IntCoords<LongNumber>** _IntCoords;    // Intersection coordinates for graph intersection function
    int _IntsFound;                        // Number of points of intersection found, 0 to 3
    int _ExtremaEnd;                       // Indicate whether finding min. or max. extrema

    virtual ChrString FindIntersections(int NumParams, FncParameter** ParamArray_);
    virtual ChrString FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_);
    virtual ChrString EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindDerivative(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindDerivative2(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString FindRoot(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindMinimum(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString FindMaximum(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_);
    virtual ChrString PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_);

  public:
    LongNumCalculator();
    virtual ~LongNumCalculator();

    // virtual math methods implemented in derived classes
    virtual ChrString PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                   const ChrString& MidPtStr_, ChrString Radius_="5", ChrString* Solution_=NULL);
    virtual ChrString FindIntegral(int NumParams_, FncParameter** ParamArray_);
    virtual ChrString MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_);
    virtual ChrString SolveMatrix(int NumParams_, FncParameter** ParamArray_);

    virtual ChrString ToString(int Index_);
    virtual const LongNumber& DirectProcess(Mcalc_Operand**, char**, int);
    virtual const LongNumber& GetResult() const;
    virtual const LongNumber& GetVariable(int Index_) const;
    virtual size_t GetVariableLength(int Index_) const;
    virtual void SetVariable(int Index_, const LongNumber& Value1_, int Transfer_=0);
    virtual int GiveType() const;
    virtual void SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_);
    virtual void SetParent(CalculatorBase* Ptr_);

#if OVERLOAD_NEW
    void* operator new (size_t Bytes_);
    void operator delete (void* Space_);

#if HAS_ARRAY_NEW
    void* operator new[] (size_t Bytes_);
    void operator delete[] (void* Space_);
#endif
#endif
};

MEMORYOPS_TEMPLATE_DEFN(LineCalculator)
MEMORYOPS_TEMPLATE_DEFN(PointerStack)

/****************************************************************************/
template <class T>
void PointerStack<T>::Set(T* Ptr_)
{
  _Active = Ptr_;
}

/****************************************************************************/
template <class T>
void PointerStack<T>::Push(PointerStack<T>* Ptr_)
{
  _Next = Ptr_;
}

/****************************************************************************/
template <class T>
void PointerStack<T>::Pop()
{
  _Next = NULL;
  _Active = NULL;
}

/****************************************************************************/
template <class T>
void PointerStack<T>::PopDelete()
{
  _Next = NULL;
  delete _Active;
  _Active = NULL;
}

/****************************************************************************/
/****************************************************************************/
template <class T>
LineCalculator<T>::LineCalculator():
_StrVarData(NULL),
_StringOperation(0),
_EvalStrings(0),
_AngleUnit(RADIAN),
_StackSize(STACKSIZE),
_PrevCalcIndex(0),
_AssignmentOrder(NULL),
_ParentPtr(this),
_Expression(NULL),
_ActiveExpression(NULL),
_ActiveProgram(NULL),
_ExprStackLimit(20),
_ExprStackIndex(0),
_SubExprStack(NULL),
_TransferValue(0)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: LineCalculator constructor\n", logfile);
  #endif

  _Random.RANDOMIZE();

  int i;
  for (i = 0; i < MAXCALC; i++)
    _CalcPtrArray[i] = NULL;

  #if CALCLIB_DEBUG2
    fputs("Leaving: LineCalculator constructor\n", logfile);
  #endif
}

/****************************************************************************/
template <class T>
LineCalculator<T>::~LineCalculator()
{
  if (!_ParentPtr)
  {
    if (_CalcPtrArray[CalculatorBase::FLOAT] == this)
      _CalcPtrArray[CalculatorBase::FLOAT] =  NULL;

    if (_CalcPtrArray[CalculatorBase::DOUBLE] == this)
      _CalcPtrArray[CalculatorBase::DOUBLE] = NULL;

    if (_CalcPtrArray[CalculatorBase::LONGDOUBLE] == this)
      _CalcPtrArray[CalculatorBase::LONGDOUBLE] = NULL;

    if (_CalcPtrArray[CalculatorBase::LONGNUM] == this)
      _CalcPtrArray[CalculatorBase::LONGNUM] = NULL;

    int i;
    for (i = 1; i < MAXCALC; i++)
    {
      delete _CalcPtrArray[i];
      _CalcPtrArray[i] = NULL;
    }

    if (_StrVarData)
      delete _StrVarData;
    _StrVarData = NULL;

    MemMatrix::Matrix().Deallocate(_AssignmentOrder);
    _AssignmentOrder = NULL;
    DestroySubExprStack();

    while (_ActiveProgram)
    {
      ProgramStack* Top_ = _ActiveProgram;
      _ActiveProgram = _ActiveProgram->_Next;
      Top_->Set(NULL);
      Top_->PopDelete();
      delete Top_;
    }
  }
}

/****************************************************************************/
template <class T>
size_t LineCalculator<T>::SubExprStackSize() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->SubExprStackSize();

  return _ExprStackIndex;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::ResizeExprStack()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->ResizeExprStack();
  else
  {
    SubExpression** OldStack_ = _SubExprStack;
    size_t OldLimit_ = _ExprStackLimit;

    _ExprStackLimit += 20;
    _SubExprStack = (SubExpression**)CalculatorBase::allocmem(_ExprStackLimit, sizeof(SubExpression*), _SubExprStack);

    size_t x;
    for (x = 0; x < OldLimit_; x++)
      _SubExprStack[x] = OldStack_[x];

    for (;x < _ExprStackLimit; x++)
      _SubExprStack[x] = NULL;

    MemMatrix::Matrix().Deallocate(OldStack_);
    OldStack_ = NULL;
  }
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::CreateNewSubExpr()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->CreateNewSubExpr();
  else
  {
    if (_ExprStackIndex == _ExprStackLimit)
      ResizeExprStack();

    _Expression = new SubExpression();
    _Expression->SetCalculator(this);
    _SubExprStack[_ExprStackIndex] = _Expression;
    _ExprStackIndex++;
  }
}

/****************************************************************************/
template <class T>
SubExpression* LineCalculator<T>::TopSubExprStack()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->TopSubExprStack();

  if (_ExprStackIndex)
    _Expression = _SubExprStack[_ExprStackIndex - 1];
  else
    _Expression = NULL;

  return _Expression;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::PopSubExprStack()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->PopSubExprStack();
  else
  {
    if (_ExprStackIndex)
    {
      _ExprStackIndex--;
      delete _SubExprStack[_ExprStackIndex];
      _SubExprStack[_ExprStackIndex] = NULL;
      TopSubExprStack();
    }
  }
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::DestroySubExprStack()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->DestroySubExprStack();
  else
  {
    _Expression = NULL;
    while (_ExprStackIndex)
    {
      _ExprStackIndex--;
      _SubExprStack[_ExprStackIndex]->SetCalculator(NULL);
      delete _SubExprStack[_ExprStackIndex];
      _SubExprStack[_ExprStackIndex] = NULL;
    }

    MemMatrix::Matrix().Deallocate(_SubExprStack);
    _SubExprStack = NULL;

    while (_ActiveExpression)
    {
      ExpressionStack* Top_ = _ActiveExpression;
      _ActiveExpression = _ActiveExpression->_Next;
      Top_->Set(NULL);
      Top_->Pop();
      delete Top_;
    }
  }
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::EvalFncWithValue(const ChrString& ExprStr_, const ChrString& VarName_, const ChrString& Value_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->EvalFncWithValue(ExprStr_, VarName_, Value_);

  CalculatorBase* CalcPtr_ = this;
  CalcPtr_ = CalcPtr_->GetCalcType();
  ChrString FncValue_ = Value_;

  if (!IsThisChar(Value_.c_str(), 0, '(', 1) ||
      !IsThisChar(Value_.c_str(), Value_.strlen() - 1, ')', 0))
    FncValue_ = ChrString("(") + Value_ + ChrString(")");

  ChrString NewExprStr_ = ExprStr_;
  NewExprStr_.Replace(0, VarName_, FncValue_);
  _Expression->SetExpression(NewExprStr_.c_str());
  _Expression->SetCalculator(CalcPtr_);
  _Expression->FindFunctionCalls();
  _Expression->Evaluate();
  ChrString Answer_ = _Expression->GetAnswer();
  _Expression->Clear();
  ResetAnswerMode();
  RestoreCalcType();

  return Answer_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ReEvaluate(const ChrString& ExprStr_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->ReEvaluate(ExprStr_);

  CreateNewSubExpr();
  CalculatorBase* CalcPtr_ = this;
  CalcPtr_ = CalcPtr_->GetCalcType();

  _Expression->SetExpression(ExprStr_.c_str());
  _Expression->SetCalculator(CalcPtr_);
  _Expression->FindFunctionCalls();
  _Expression->Evaluate();
  ChrString Answer_ = _Expression->GetAnswer();
  _Expression->Clear();

  PopSubExprStack();
  return Answer_;
}

/*****************************************************************************/
template <class T>
char* LineCalculator<T>::ConvertFloatToFraction(char* Float_)
{
  LongNumber Frac_;
  LongNumber Int_;
  LongNumber FltNum_ = ChrString(Float_);
  Frac_ = FltNum_.modf(Int_);

  LongNumber Tens_(10L);
  Tens_ = LongNumber::IntPow(Tens_, Frac_.TotalDigits());
  FltNum_ *= Tens_;
  ChrString RetStr_ = FltNum_.ToString();
  RetStr_ += "/";
  RetStr_ += Tens_.ToString();

  char* FractStr_ = NULL;
  FractStr_ = (char*)allocmem(RetStr_.strlen() + 1, sizeof(char), FractStr_);
  strcpy(FractStr_, RetStr_.c_str());
  MemMatrix::Matrix().Deallocate(Float_);
  return FractStr_;
}

/*****************************************************************************/
template <class T>
char* LineCalculator<T>::ConvertIntegerToFraction(char* Int_)
{
  char* FractStr_ = NULL;
  FractStr_ = (char*)allocmem(::SafeStrLen(Int_) + 3, sizeof(char), FractStr_);
  strcpy(FractStr_, Int_);
  strcat(FractStr_, "/1");
  MemMatrix::Matrix().Deallocate(Int_);
  return FractStr_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindFractionArithmetic(int NumParams_, FncParameter** ParamArray_)
{
  char* Ptr_;
  char* Temp_;
  char** Data_ = NULL;
  LongNumber** Denom_;
  LongNumber** Numer_;
  LongNumber Lcm_;
  LongNumber Gcd_;
  LongNumber Mult_;
  LongNumber FirstNumer_;
  LongNumber FirstDenom_;
  LongNumber TotalNumer_;
  LongNumber TotalDenom_;
  ChrString Frac_;
  ChrString Str1_;
  ChrString Str2_;
  int Optype_;
  int NumOperands_ = NumParams_ - 1;

  Data_ = (char**)allocmem(NumParams_, sizeof(char*), Data_);
  Denom_ = (LongNumber**)allocmem(NumParams_, sizeof(LongNumber*), Data_);
  Numer_ = (LongNumber**)allocmem(NumParams_, sizeof(LongNumber*), Data_);
  const char* OptypeStr_ = ParamArray_[0]->_Param->GetExpression();
  if (OptypeStr_)
  {
    Optype_ = (icasestrcomp(OptypeStr_, "Add")      || icasestrcomp(OptypeStr_, "+")) ? Mcalc_OperatorType::Add:
              (icasestrcomp(OptypeStr_, "Subtract") || icasestrcomp(OptypeStr_, "-")) ? Mcalc_OperatorType::Subtract:
              (icasestrcomp(OptypeStr_, "Multiply") || icasestrcomp(OptypeStr_, "*")) ? Mcalc_OperatorType::Multiply:
              (icasestrcomp(OptypeStr_, "Divide")   || icasestrcomp(OptypeStr_, "/")) ? Mcalc_OperatorType::Divide:0;
  }
  else
    Optype_ = 0;

  if (!Optype_ || NumParams_ == 1)
    if (strchr(OptypeStr_, ','))
    {
      if (NumParams_ == 1)
      {
        MemMatrix::Matrix().Deallocate(Denom_);
        MemMatrix::Matrix().Deallocate(Numer_);

        ChrString ArgStr_ = ChrString("FRAC(") + ChrString(OptypeStr_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        SetAnswerMode(CalculatorBase::FRACTION_MODE);

        Data_[0] = ::NewString(AnswerStr_.c_str());
        Ptr_ = strchr(Data_[0], '/');

        if (Ptr_)
        {
          Str1_ = &Ptr_[1];
          Str2_ = ChrString(Data_[0], Ptr_ - Data_[0]);
          Str1_.RemovePadding(" \t\n");
          Str2_.RemovePadding(" \t\n");

          int ValidNum1_ = IsNumber(Str1_.c_str(), 1, 0, 0, 10);
          int ValidNum2_ = IsNumber(Str2_.c_str(), 1, 0, 0, 10);

          if (ValidNum1_ && ValidNum2_)
          {
            TotalNumer_ = Str2_;
            TotalDenom_ = Str1_;

            if (!TotalDenom_.AtZero())
            {
              TotalNumer_ /= TotalDenom_;

              if (TotalNumer_.AtZero())
                SetFractionStr("0");
              else
                SetFractionStr(AnswerStr_.c_str());

              AnswerStr_ = TotalNumer_.ToString();
            }
            else
              AnswerStr_ = ShowFracDivZero(TotalNumer_, TotalDenom_);
          }
        }

        MemMatrix::Matrix().Deallocate(Data_[0]);
        MemMatrix::Matrix().Deallocate(Data_);
        return AnswerStr_;
      }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FRAC");
    CloseOutputFile();
    return Frac_;
  }

  int x, y;
  for (x = 1; x < NumParams_; x++)
  {
    y = x - 1;
    Data_[y] = ::NewString(ParamArray_[x]->_Param->GetExpression());
    Denom_[y] = new LongNumber();
    Numer_[y] = new LongNumber();
  }

  for (x = 0; x < NumOperands_; x++)
  {
    Ptr_ = strchr(Data_[x], '/');

    if (!Ptr_)
    {
      Mult_ = Data_[x];
      if (IsFloat(Data_[x]))
      {
        Data_[x] = ConvertFloatToFraction(Data_[x]);
        Ptr_ = strchr(Data_[x], '/');
      }
      else if (LongNumber::IsInSciNote(Mult_))
      {
        Temp_ = Data_[x];
        Mult_.SetSciNotation(FALSE);
        Str1_ = Mult_.ToString();
        Data_[x] = ::NewString(Str1_.c_str());
        MemMatrix::Matrix().Deallocate(Temp_);

        if (IsFloat(Data_[x]))
        {
          Data_[x] = ConvertFloatToFraction(Data_[x]);
          Ptr_ = strchr(Data_[x], '/');
        }
        else if (::IsNumber(Data_[x], 1, 0, 0, 10))
        {
          Data_[x] = ConvertIntegerToFraction(Data_[x]);
          Ptr_ = strchr(Data_[x], '/');
        }
      }
      else if (::IsNumber(Data_[x], 1, 0, 0, 10))
      {
        Data_[x] = ConvertIntegerToFraction(Data_[x]);
        Ptr_ = strchr(Data_[x], '/');
      }
    }

    if (!Ptr_)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "FRAC");
      CloseOutputFile();
      return Frac_;
    }

    Str1_ = &Ptr_[1];
    Str2_ = ChrString(Data_[x], Ptr_ - Data_[x]);
    Str1_.RemovePadding(" \t\n");
    Str2_.RemovePadding(" \t\n");

    int ValidNum1_ = IsNumber(Str1_.c_str(), 1, 0, 0, 10);
    int ValidNum2_ = IsNumber(Str2_.c_str(), 1, 0, 0, 10);

    if (!ValidNum1_ || !ValidNum2_)
    {
      for (;;)
        if (IsFloat(Str1_.c_str()))
        {
          Temp_ = ::NewString(Str1_.c_str());
          Temp_= ConvertFloatToFraction(Temp_);
          Str1_ = Temp_;
          MemMatrix::Matrix().Deallocate(Temp_);
          ValidNum1_ = IsNumber(Str1_.c_str(), 1, 0, 0, 10);
          break;
        }
        else if (LongNumber::IsInSciNote(Str1_.c_str()))
        {
          Mult_ = Str1_;
          Mult_.SetSciNotation(FALSE);
          Str1_ = Mult_.ToString();
          ValidNum1_ = IsNumber(Str1_.c_str(), 1, 0, 0, 10);
          if (ValidNum1_)
            break;
        }
        else
          break;

      for (;;)
        if (IsFloat(Str2_.c_str()))
        {
          Temp_ = ::NewString(Str2_.c_str());
          Temp_= ConvertFloatToFraction(Temp_);
          Str2_ = Temp_;
          MemMatrix::Matrix().Deallocate(Temp_);
          ValidNum2_ = IsNumber(Str2_.c_str(), 1, 0, 0, 10);
          break;
        }
        else if (LongNumber::IsInSciNote(Str2_.c_str()))
        {
          Mult_ = Str2_;
          Mult_.SetSciNotation(FALSE);
          Str2_ = Mult_.ToString();
          ValidNum2_ = IsNumber(Str2_.c_str(), 1, 0, 0, 10);
          if (ValidNum2_)
            break;
        }
        else
          break;

      if (!ValidNum1_ || !ValidNum2_)
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                            ERRMSG_INVALID_TYPEARGS, "FRAC");
        CloseOutputFile();
        return Frac_;
      }
    }

    Str1_.RemovePadding(" \t\n");
    Str2_.RemovePadding(" \t\n");

    *Numer_[x] = Str2_;
    *Denom_[x] = Str1_;

    if (Denom_[x]->AtZero())
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_DIVISION_BY_ZERO,
                          ERRMSG_DIVISION_BY_ZERO, "FRAC");
      CloseOutputFile();
      return Frac_;
    }

    if (Optype_ == Mcalc_OperatorType::Add ||
        Optype_ == Mcalc_OperatorType::Subtract)
    {
      if (x)
        if (x > 1)
          Lcm_ = Discrete<LongNumber>::LCM(Lcm_, *Denom_[x]);
        else
          Lcm_ = Discrete<LongNumber>::LCM(*Denom_[0], *Denom_[1]);
    }
  }

  if (Optype_ == Mcalc_OperatorType::Add ||
      Optype_ == Mcalc_OperatorType::Subtract)
  {
    if (Optype_ == Mcalc_OperatorType::Subtract)
    {
      x = 1;
      Mult_ = Lcm_ / *Denom_[0];
      *Numer_[0] *= Mult_;
      FirstNumer_ = *Numer_[0];
    }
    else
      x = 0;

    for (;x < NumOperands_; x++)
    {
      Mult_ = Lcm_ / *Denom_[x];
      *Numer_[x] *= Mult_;

      if (Optype_ == Mcalc_OperatorType::Add)
        TotalNumer_ += *Numer_[x];
      else
        TotalNumer_ += -(*Numer_[x]);
    }

    if (Optype_ == Mcalc_OperatorType::Subtract)
      TotalNumer_ += FirstNumer_;

    // Reducing
    bool valid_ = true;
    if (TotalNumer_.AtZero())
      Frac_ = "0";
    else
    {
      valid_ = false;
      Gcd_ = Discrete<LongNumber>::GCD(TotalNumer_, Lcm_);

      if (!Gcd_.AtZero())
      {
        TotalNumer_ /= Gcd_;
        Lcm_ /= Gcd_;

        Frac_ = TotalNumer_.ToString();
        Frac_ += "/";
        Frac_ += Lcm_.ToString();

        if (!Lcm_.AtZero())
        {
          TotalNumer_ /= Lcm_;
          valid_ = true;
        }
        else
          TotalDenom_ = Lcm_;
      }
      else
        TotalDenom_ = Gcd_;
    }

    if (!valid_)
      TotalNumer_ = ShowFracDivZero(TotalNumer_, TotalDenom_);
    else
      SetFractionStr(Frac_.c_str());
  }
  else
  {
    if (Optype_ == Mcalc_OperatorType::Divide)
    {
      x = 1;
      FirstNumer_ = *Numer_[0];
      FirstDenom_ = *Denom_[0];
    }
    else
      x = 0;

    TotalNumer_ = 1L;
    TotalDenom_ = 1L;

    for (;x < NumOperands_; x++)
    {
      TotalNumer_ *= *Numer_[x];
      TotalDenom_ *= *Denom_[x];
    }

    if (Optype_ == Mcalc_OperatorType::Divide)
    {
      TotalDenom_ *= FirstNumer_;
      TotalNumer_ *= FirstDenom_;
      Mult_ = TotalNumer_;
      TotalNumer_ = TotalDenom_;
      TotalDenom_ = Mult_;

      if (TotalDenom_.AtZero())
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_DIVISION_BY_ZERO,
                            ERRMSG_DIVISION_BY_ZERO, "FRAC");
        CloseOutputFile();
        return Frac_;
      }
    }

    // Reducing
    bool valid_ = true;
    if (TotalNumer_.AtZero())
      Frac_ = "0";
    else
    {
      valid_ = false;
      Gcd_ = Discrete<LongNumber>::GCD(TotalNumer_, TotalDenom_);

      if (!Gcd_.AtZero())
      {
        TotalNumer_ /= Gcd_;
        TotalDenom_ /= Gcd_;

        Frac_ = TotalNumer_.ToString();
        Frac_ += "/";
        Frac_ += TotalDenom_.ToString();

        if (!TotalDenom_.AtZero())
        {
          TotalNumer_ /= TotalDenom_;
          valid_ = true;
        }
      }
      else
        TotalDenom_ = Gcd_;
    }

    if (!valid_)
      TotalNumer_ = ShowFracDivZero(TotalNumer_, TotalDenom_);
    else
      SetFractionStr(Frac_.c_str());
  }

  for (x = 0; x < NumOperands_; x++)
  {
    MemMatrix::Matrix().Deallocate(Data_[x]);
    delete Denom_[x];
    delete Numer_[x];
  }

  MemMatrix::Matrix().Deallocate(Data_);
  MemMatrix::Matrix().Deallocate(Denom_);
  MemMatrix::Matrix().Deallocate(Numer_);

  return TotalNumer_.ToString();
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::CreateSet(int NumParams_, FncParameter** ParamArray_)
{
  const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();
  ChrString FirstArg_ = Buffer_;
  FirstArg_.RemovePadding(" \t\n");
  SetHasAnswer(FALSE);

  ChrString ArgStr_;
  ChrString Result_;
  ChrString AnswerStr_;
  ChrString AssignStr_('$');
  CalculatorBase* CalcPtr_ = this;
  CalcPtr_ = CalcPtr_->GetCalcType();
  int IsNull_ = 0;

  if (NumParams_ == 1 || !IsNumber(ParamArray_[1]->_Param->GetExpression(), 0, 0, 0, 10))
  {
    if ((NumParams_ == 1 && isalpha(Buffer_[0]) && FirstArg_.strlen() == 1) || strchr(Buffer_, ','))
    {
      if (isalpha(Buffer_[0]) && FirstArg_.strlen() == 1)
      {
        AnswerStr_ = ChrString("{ null }");
        IsNull_ = 1;
      }
      else
      {
        ArgStr_ = ChrString("SET(") + ChrString(Buffer_) + ChrString(")");
        AnswerStr_ = ReEvaluate(ArgStr_);
      }

      AssignStr_ += ChrString(Buffer_[0]);
      AssignStr_ += ChrString('=');
      SetHasAnswer(AnswerStr_.strlen());

      if (AnswerStr_.strlen())
      {
        IsNull_ = IsNullSetStr(AnswerStr_.c_str());
        AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
        Result_ = AssignStr_ + AnswerStr_;
        SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
        ReEvaluate(Result_.c_str());

        SetAnswerMode(LITERAL_MODE);
        SetLiteralStr(AnswerStr_.c_str());
      }

      return AnswerStr_;
    }
    else if (InSetNotation(ParamArray_[1]->_Param->GetExpression()) &&
             isalpha(Buffer_[0]) && FirstArg_.strlen() == 1)
    {
      Set<int>* Set_ = MakeIntSet(ParamArray_[1]->_Param->GetExpression());
      AnswerStr_ = MakeTheList(Set_->GiveSetData());
      delete Set_;

      AssignStr_ += ChrString(Buffer_[0]);
      AssignStr_ += ChrString('=');
      SetHasAnswer(AnswerStr_.strlen());

      if (AnswerStr_.strlen())
      {
        IsNull_ = IsNullSetStr(AnswerStr_.c_str());
        AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
        Result_ = AssignStr_ + AnswerStr_;
        SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
        ReEvaluate(Result_.c_str());

        SetAnswerMode(LITERAL_MODE);
        SetLiteralStr(AnswerStr_.c_str());
      }

      return AnswerStr_;
    }
    else if (isalpha(Buffer_[0]) && FirstArg_.strlen() == 1)
    {
      ChrString Expr_ = ParamArray_[1]->_Param->GetExpression();
      Expr_.RemovePadding(" \t\n");

      if (Expr_.strlen() == 1 && isalpha(Expr_[0]))
      {
        AnswerStr_ = CalcPtr_->GetStrVariable(tolower(Expr_[0]) - 'a');
        AssignStr_ += ChrString(Buffer_[0]);
        AssignStr_ += ChrString('=');
        SetHasAnswer(AnswerStr_.strlen());

        if (AnswerStr_.strlen())
        {
          IsNull_ = IsNullSetStr(AnswerStr_.c_str());
          AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
          Result_ = AssignStr_ + AnswerStr_;
          SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
          ReEvaluate(Result_.c_str());

          SetAnswerMode(LITERAL_MODE);
          SetLiteralStr(AnswerStr_.c_str());
        }

        return AnswerStr_;
      }
      else
      {
        ifstream Fin_;

        if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
        {
          Set<int>* Set_;
          Boolean Error_;

          Set_ = new Set<int>(Fin_, Error_);
          Fin_.close();

          if (Error_)
          {
            Set_->Flush();
            SetAnswerMode(LITERAL_MODE);
            SetLiteralStr("");
            SetHasAnswer(FALSE);
            delete Set_;

            DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                                ERRMSG_EMPTY_DATASET, "SET");
            return AnswerStr_;
          }

          AnswerStr_ = MakeTheList(Set_->GiveSetData());
          delete Set_;

          AssignStr_ += ChrString(Buffer_[0]);
          AssignStr_ += ChrString('=');
          SetHasAnswer(AnswerStr_.strlen());

          if (AnswerStr_.strlen())
          {
            IsNull_ = IsNullSetStr(AnswerStr_.c_str());
            AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
            Result_ = AssignStr_ + AnswerStr_;
            SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
            ReEvaluate(Result_.c_str());

            SetAnswerMode(LITERAL_MODE);
            SetLiteralStr(AnswerStr_.c_str());
          }

          return AnswerStr_;
        }
        else
        {
          SetAnswerMode(LITERAL_MODE);
          SetLiteralStr("");
          SetHasAnswer(FALSE);
        }
      }
    }
  }
  else if (isalpha(Buffer_[0]) && FirstArg_.strlen() == 1)
  {
    int Size_ = atoi(ParamArray_[1]->_Param->GetExpression());

    if (Size_ <= 0 || NumParams_ <= 2 || Size_ != NumParams_ - 2)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "SET");
      CloseOutputFile();

      SetAnswerMode(LITERAL_MODE);
      SetLiteralStr("");
      SetHasAnswer(FALSE);
      return AnswerStr_;
    }

    if (Size_ && NumParams_)
    {
      Set<int>* Set_ = new Set<int>;

      size_t i;
      for (i = 2; (i-1) <= Size_; i++)
      {
        ArgStr_ = ParamArray_[i]->_Param->GetExpression();
        ArgStr_.RemovePadding(" \t\n");

        if (IsInt(ArgStr_.c_str()))
        {
          int Data_ = atoi(ArgStr_.c_str());
          *Set_ |= &Data_;
        }
      }

      AnswerStr_ = MakeTheList(Set_->GiveSetData());
      delete Set_;

      AssignStr_ += ChrString(Buffer_[0]);
      AssignStr_ += ChrString('=');
      SetHasAnswer(AnswerStr_.strlen());

      if (AnswerStr_.strlen())
      {
        IsNull_ = IsNullSetStr(AnswerStr_.c_str());
        AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
        Result_ = AssignStr_ + AnswerStr_;
        SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
        ReEvaluate(Result_.c_str());

        SetAnswerMode(LITERAL_MODE);
        SetLiteralStr(AnswerStr_.c_str());
      }

      return AnswerStr_;
    }
  }

  AnswerStr_[0] = 0;
  return AnswerStr_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindSetUnion(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Result_;
  SetHasAnswer(FALSE);

  if (NumParams_ != 3)
    return Result_;

  ChrString Expr0_ = ParamArray_[0]->_Param->GetExpression();
  ChrString Expr1_ = ParamArray_[1]->_Param->GetExpression();
  ChrString Expr2_ = ParamArray_[2]->_Param->GetExpression();
  ChrString AssignStr_('$');
  ChrString AnswerStr_;

  int IsNull_;
  CalculatorBase* CalcPtr_ = this;
  CalcPtr_ = CalcPtr_->GetCalcType();

  Expr0_.RemovePadding(" \t\n");
  Expr1_.RemovePadding(" \t\n");
  Expr2_.RemovePadding(" \t\n");

  if (Expr0_.strlen() == 1 && isalpha(Expr0_[0]) &&
      Expr1_.strlen() == 1 && isalpha(Expr1_[0]) &&
      Expr2_.strlen() == 1 && isalpha(Expr2_[0]))
  {
    ChrString TrgSetStr_ = CalcPtr_->GetStrVariable(tolower(Expr0_[0]) - 'a');
    ChrString Set1Str_ = CalcPtr_->GetStrVariable(tolower(Expr1_[0]) - 'a');
    ChrString Set2Str_ = CalcPtr_->GetStrVariable(tolower(Expr2_[0]) - 'a');

    if (InSetNotation(Set1Str_.c_str()) &&
        InSetNotation(Set2Str_.c_str()))
    {
      Set<int>* TrgSet_ = new Set<int>;
      Set<int>* Set1_ = MakeIntSet(Set1Str_.c_str());
      Set<int>* Set2_ = MakeIntSet(Set2Str_.c_str());

      *TrgSet_ = *Set1_;
      *TrgSet_ |= *Set2_;

      AnswerStr_ = MakeTheList(TrgSet_->GiveSetData());
      AssignStr_ += ChrString(Expr0_[0]);
      AssignStr_ += ChrString('=');
      SetHasAnswer(AnswerStr_.strlen());

      if (AnswerStr_.strlen())
      {
        IsNull_ = IsNullSetStr(AnswerStr_.c_str());
        AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
        Result_ = AssignStr_ + AnswerStr_;
        SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
        ReEvaluate(Result_.c_str());

        SetAnswerMode(LITERAL_MODE);
        SetLiteralStr(AnswerStr_.c_str());
      }

      delete TrgSet_;
      delete Set1_;
      delete Set2_;
    }
  }

  return AnswerStr_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindSetIntersection(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Result_;
  SetHasAnswer(FALSE);

  if (NumParams_ != 3)
    return Result_;

  ChrString Expr0_ = ParamArray_[0]->_Param->GetExpression();
  ChrString Expr1_ = ParamArray_[1]->_Param->GetExpression();
  ChrString Expr2_ = ParamArray_[2]->_Param->GetExpression();
  ChrString AssignStr_('$');
  ChrString AnswerStr_;

  int IsNull_;
  CalculatorBase* CalcPtr_ = this;
  CalcPtr_ = CalcPtr_->GetCalcType();

  Expr0_.RemovePadding(" \t\n");
  Expr1_.RemovePadding(" \t\n");
  Expr2_.RemovePadding(" \t\n");

  if (Expr0_.strlen() == 1 && isalpha(Expr0_[0]) &&
      Expr1_.strlen() == 1 && isalpha(Expr1_[0]) &&
      Expr2_.strlen() == 1 && isalpha(Expr2_[0]))
  {
    ChrString TrgSetStr_ = CalcPtr_->GetStrVariable(tolower(Expr0_[0]) - 'a');
    ChrString Set1Str_ = CalcPtr_->GetStrVariable(tolower(Expr1_[0]) - 'a');
    ChrString Set2Str_ = CalcPtr_->GetStrVariable(tolower(Expr2_[0]) - 'a');

    if (InSetNotation(Set1Str_.c_str()) &&
        InSetNotation(Set2Str_.c_str()))
    {
      Set<int>* TrgSet_ = new Set<int>;
      Set<int>* Set1_ = MakeIntSet(Set1Str_.c_str());
      Set<int>* Set2_ = MakeIntSet(Set2Str_.c_str());

      *TrgSet_ = *Set1_;
      *TrgSet_ &= *Set2_;

      AnswerStr_ = MakeTheList(TrgSet_->GiveSetData());
      AssignStr_ += ChrString(Expr0_[0]);
      AssignStr_ += ChrString('=');
      SetHasAnswer(AnswerStr_.strlen());

      if (AnswerStr_.strlen())
      {
        IsNull_ = IsNullSetStr(AnswerStr_.c_str());
        AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
        Result_ = AssignStr_ + AnswerStr_;
        SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
        ReEvaluate(Result_.c_str());

        SetAnswerMode(LITERAL_MODE);
        SetLiteralStr(AnswerStr_.c_str());
      }

      delete TrgSet_;
      delete Set1_;
      delete Set2_;
    }
  }

  return AnswerStr_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindSetDifference(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Result_;
  SetHasAnswer(FALSE);

  if (NumParams_ != 3)
    return Result_;

  ChrString Expr0_ = ParamArray_[0]->_Param->GetExpression();
  ChrString Expr1_ = ParamArray_[1]->_Param->GetExpression();
  ChrString Expr2_ = ParamArray_[2]->_Param->GetExpression();
  ChrString AssignStr_('$');
  ChrString AnswerStr_;

  int IsNull_;
  CalculatorBase* CalcPtr_ = this;
  CalcPtr_ = CalcPtr_->GetCalcType();

  Expr0_.RemovePadding(" \t\n");
  Expr1_.RemovePadding(" \t\n");
  Expr2_.RemovePadding(" \t\n");

  if (Expr0_.strlen() == 1 && isalpha(Expr0_[0]) &&
      Expr1_.strlen() == 1 && isalpha(Expr1_[0]) &&
      Expr2_.strlen() == 1 && isalpha(Expr2_[0]))
  {
    ChrString TrgSetStr_ = CalcPtr_->GetStrVariable(tolower(Expr0_[0]) - 'a');
    ChrString Set1Str_ = CalcPtr_->GetStrVariable(tolower(Expr1_[0]) - 'a');
    ChrString Set2Str_ = CalcPtr_->GetStrVariable(tolower(Expr2_[0]) - 'a');

    if (InSetNotation(Set1Str_.c_str()) &&
        InSetNotation(Set2Str_.c_str()))
    {
      Set<int>* TrgSet_ = new Set<int>;
      Set<int>* Set1_ = MakeIntSet(Set1Str_.c_str());
      Set<int>* Set2_ = MakeIntSet(Set2Str_.c_str());

      *TrgSet_ = *Set1_;
      *TrgSet_ -= *Set2_;

      AnswerStr_ = MakeTheList(TrgSet_->GiveSetData());
      AssignStr_ += ChrString(Expr0_[0]);
      AssignStr_ += ChrString('=');
      SetHasAnswer(AnswerStr_.strlen());

      if (AnswerStr_.strlen())
      {
        IsNull_ = IsNullSetStr(AnswerStr_.c_str());
        AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
        Result_ = AssignStr_ + AnswerStr_;
        SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
        ReEvaluate(Result_.c_str());

        SetAnswerMode(LITERAL_MODE);
        SetLiteralStr(AnswerStr_.c_str());
      }

      delete TrgSet_;
      delete Set1_;
      delete Set2_;
    }
  }

  return AnswerStr_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindSetExclusion(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Result_;
  SetHasAnswer(FALSE);

  if (NumParams_ != 3)
    return Result_;

  ChrString Expr0_ = ParamArray_[0]->_Param->GetExpression();
  ChrString Expr1_ = ParamArray_[1]->_Param->GetExpression();
  ChrString Expr2_ = ParamArray_[2]->_Param->GetExpression();
  ChrString AssignStr_('$');
  ChrString AnswerStr_;

  int IsNull_;
  CalculatorBase* CalcPtr_ = this;
  CalcPtr_ = CalcPtr_->GetCalcType();

  Expr0_.RemovePadding(" \t\n");
  Expr1_.RemovePadding(" \t\n");
  Expr2_.RemovePadding(" \t\n");

  if (Expr0_.strlen() == 1 && isalpha(Expr0_[0]) &&
      Expr1_.strlen() == 1 && isalpha(Expr1_[0]) &&
      Expr2_.strlen() == 1 && isalpha(Expr2_[0]))
  {
    ChrString TrgSetStr_ = CalcPtr_->GetStrVariable(tolower(Expr0_[0]) - 'a');
    ChrString Set1Str_ = CalcPtr_->GetStrVariable(tolower(Expr1_[0]) - 'a');
    ChrString Set2Str_ = CalcPtr_->GetStrVariable(tolower(Expr2_[0]) - 'a');

    if (InSetNotation(Set1Str_.c_str()) &&
        InSetNotation(Set2Str_.c_str()))
    {
      Set<int>* TrgSet_ = new Set<int>;
      Set<int>* NewSet_ = new Set<int>;
      Set<int>* Set1_ = MakeIntSet(Set1Str_.c_str());
      Set<int>* Set2_ = MakeIntSet(Set2Str_.c_str());

      *TrgSet_ = *Set1_;
      *TrgSet_ |= *Set2_;
      *NewSet_ = *Set1_;
      *NewSet_ &= *Set2_;
      *TrgSet_ -= *NewSet_;

      AnswerStr_ = MakeTheList(TrgSet_->GiveSetData());
      AssignStr_ += ChrString(Expr0_[0]);
      AssignStr_ += ChrString('=');
      SetHasAnswer(AnswerStr_.strlen());

      if (AnswerStr_.strlen())
      {
        IsNull_ = IsNullSetStr(AnswerStr_.c_str());
        AnswerStr_ = ChrString("\"") + AnswerStr_ + ChrString("\"");
        Result_ = AssignStr_ + AnswerStr_;
        SetSetAssignment(TRUE, AnswerStr_.c_str(), IsNull_);
        ReEvaluate(Result_.c_str());

        SetAnswerMode(LITERAL_MODE);
        SetLiteralStr(AnswerStr_.c_str());
      }

      delete TrgSet_;
      delete NewSet_;
      delete Set1_;
      delete Set2_;
    }
  }

  return AnswerStr_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindPermutations(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Answer_;

  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();
    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("ITEMPERM(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        SetAnswerMode(LITERAL_MODE, TRUE);
        return AnswerStr_;
      }
    }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "ITEMPERM");
    CloseOutputFile();
    return Answer_;
  }

  int ProgramPopped_ = TRUE;
  int Sel_ = atoi(ParamArray_[0]->_Param->GetExpression());
  int Size_ = NumParams_ - 2;
  int HasIndex_ = atoi(ParamArray_[Size_ + 1]->_Param->GetExpression());
  int InAsync_ = InAsyncMode();
  int Interactive_ = 0;
  char Buffer_[8];
  ChrString TempStr_;

  DiscreteSet<int> Disc_(0);
  int* Data_ = NULL;
  int* RetData_;
  Data_ = (int*)allocmem(Size_, sizeof(int), Data_);

  int x;
  for (x = 0; x < Size_; x++)
    Data_[x] = x + 1;

  if (Sel_ <= Size_)
  {
    int IntroDone_ = 0;
    int StdStreams_ = StdInput() && StdOutput();
    Interactive_ = (StdStreams_ || InAsync_) && !HasIndex_;
    CalcProgram* Program_;

    if (Interactive_ && InAsync_)
    {
      Program_ = new CalcProgram;
      Program_->SetCalculator(this);
      PushActiveProgram(Program_);
      ProgramPopped_ = FALSE;
    }

    const char* RetStr_;
    Disc_.Permute(Data_, Size_, Sel_);

    for (;;)
    {
      RetData_ = Disc_.NextPermutation();
      if (RetData_)
      {
        if (Interactive_ && !IntroDone_)
        {
          if (InAsync_)
            PromptAsyncUser("Enter \"q\" to quit", FALSE);
          else
            fprintf(GetOutputFile(), "Enter \"q\" to quit:\n");
          IntroDone_ = 1;
        }

        TempStr_ = "";
        for (x = 0; x < Sel_; x++)
        {
          RetStr_ = ParamArray_[RetData_[x]]->_Param->GetExpression();
          if (Interactive_)
            if (InAsync_)
            {
              TempStr_ += RetStr_;
              if (x == Sel_ - 1)
              {
                TempStr_ += "\t";
                PromptAsyncUser(TempStr_.c_str(), TRUE);
              }
              else
                TempStr_ += ",";
            }
            else
            {
              fprintf(GetOutputFile(), "%s", RetStr_);
              if (x == Sel_ - 1)
                fprintf(GetOutputFile(), "\t");
              else
                fprintf(GetOutputFile(), ",", RetStr_);
            }

          Answer_ += RetStr_;
          if (x == Sel_ - 1)
            Answer_ += "\n";
          else
            Answer_ += ",";
        }

        if (Interactive_)
          if (InAsync_)
          {
            RetStr_ = ProgramStr();
            if (RetStr_ && toupper(RetStr_[0]) == 'Q')
            {
              ProgramPopped_ = TRUE;
              PopActiveProgram();
              break;
            }
          }
          else
          {
            RetStr_ = fgets(Buffer_, 8, stdin);
            if (RetStr_ && toupper(RetStr_[0]) == 'Q')
              break;
          }
      }
      else
        break;
    }
  }

  MemMatrix::Matrix().Deallocate(Data_);
  SetHasAnswer(Answer_.strlen());

  if (Answer_.strlen())
  {
    Answer_ = ChrString("\"") + Answer_ + ChrString("\"");
    SetLiteralStr(Answer_.c_str());
  }

  if (!ProgramPopped_)
    PopActiveProgram();

  if (InAsync_ && Interactive_)
  {
    ClearOutputFile();
    ClearUserPromptFile();
  }

  return Answer_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindCombinations(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Answer_;

  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();
    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("ITEMCOMB(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        SetAnswerMode(LITERAL_MODE, TRUE);
        return AnswerStr_;
      }
    }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "ITEMCOMB");
    CloseOutputFile();
    return Answer_;
  }

  int ProgramPopped_ = TRUE;
  int Sel_ = atoi(ParamArray_[0]->_Param->GetExpression());
  int Size_ = NumParams_ - 2;
  int HasIndex_ = atoi(ParamArray_[Size_ + 1]->_Param->GetExpression());
  int InAsync_ = InAsyncMode();
  int Interactive_ = 0;
  char Buffer_[8];
  ChrString TempStr_;

  DiscreteSet<int> Disc_(0);
  int* Data_ = NULL;
  int* RetData_;
  Data_ = (int*)allocmem(Size_, sizeof(int), Data_);

  int x;
  for (x = 0; x < Size_; x++)
    Data_[x] = x + 1;

  if (Sel_ <= Size_)
  {
    int IntroDone_ = 0;
    int StdStreams_ = StdInput() && StdOutput();
    Interactive_ = (StdStreams_ || InAsync_) && !HasIndex_;
    CalcProgram* Program_;

    if (Interactive_ && InAsync_)
    {
      Program_ = new CalcProgram;
      Program_->SetCalculator(this);
      PushActiveProgram(Program_);
      ProgramPopped_ = FALSE;
    }

    const char* RetStr_;
    Disc_.Combine(Data_, Size_, Sel_);

    for (;;)
    {
      RetData_ = Disc_.NextCombination();
      if (RetData_)
      {
        if (Interactive_ && !IntroDone_)
        {
          if (InAsync_)
            PromptAsyncUser("Enter \"q\" to quit", FALSE);
          else
            fprintf(GetOutputFile(), "Enter \"q\" to quit:\n");
          IntroDone_ = 1;
        }

        TempStr_ = "";
        for (x = 0; x < Sel_; x++)
        {
          RetStr_ = ParamArray_[RetData_[x]]->_Param->GetExpression();
          if (Interactive_)
            if (InAsync_)
            {
              TempStr_ += RetStr_;
              if (x == Sel_ - 1)
              {
                TempStr_ += "\t";
                PromptAsyncUser(TempStr_.c_str(), TRUE);
              }
              else
                TempStr_ += ",";
            }
            else
            {
              fprintf(GetOutputFile(), "%s", RetStr_);
              if (x == Sel_ - 1)
                fprintf(GetOutputFile(), "\t");
              else
                fprintf(GetOutputFile(), ",", RetStr_);
            }

          Answer_ += RetStr_;
          if (x == Sel_ - 1)
            Answer_ += "\n";
          else
            Answer_ += ",";
        }

        if (Interactive_)
          if (InAsync_)
          {
            RetStr_ = ProgramStr();
            if (RetStr_ && toupper(RetStr_[0]) == 'Q')
            {
              ProgramPopped_ = TRUE;
              PopActiveProgram();
              break;
            }
          }
          else
          {
            RetStr_ = fgets(Buffer_, 8, stdin);
            if (RetStr_ && toupper(RetStr_[0]) == 'Q')
              break;
          }
      }
      else
        break;
    }
  }

  MemMatrix::Matrix().Deallocate(Data_);
  SetHasAnswer(Answer_.strlen());

  if (Answer_.strlen())
  {
    Answer_ = ChrString("\"") + Answer_ + ChrString("\"");
    SetLiteralStr(Answer_.c_str());
  }

  if (!ProgramPopped_)
    PopActiveProgram();

  if (InAsync_ && Interactive_)
  {
    ClearOutputFile();
    ClearUserPromptFile();
  }

  return Answer_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindPermutationsWithRep(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Answer_;

  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();
    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("ITEMPERMREP(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        SetAnswerMode(LITERAL_MODE, TRUE);
        return AnswerStr_;
      }
    }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "ITEMPERMREP");
    CloseOutputFile();
    return Answer_;
  }

  int ProgramPopped_ = TRUE;
  int Sel_ = atoi(ParamArray_[0]->_Param->GetExpression());
  int Size_ = NumParams_ - 2;
  int HasIndex_ = atoi(ParamArray_[Size_ + 1]->_Param->GetExpression());
  int InAsync_ = InAsyncMode();
  int Interactive_ = 0;
  char Buffer_[8];
  ChrString TempStr_;

  DiscreteSet<int> Disc_(0);
  int* Data_ = NULL;
  int* RetData_;
  Data_ = (int*)allocmem(Size_, sizeof(int), Data_);

  int x;
  for (x = 0; x < Size_; x++)
    Data_[x] = x + 1;

  if (Sel_ <= Size_)
  {
    int IntroDone_ = 0;
    int StdStreams_ = StdInput() && StdOutput();
    Interactive_ = (StdStreams_ || InAsync_) && !HasIndex_;
    CalcProgram* Program_;

    if (Interactive_ && InAsync_)
    {
      Program_ = new CalcProgram;
      Program_->SetCalculator(this);
      PushActiveProgram(Program_);
      ProgramPopped_ = FALSE;
    }

    const char* RetStr_;
    Disc_.PermuteWithRep(Data_, Size_, Sel_);

    for (;;)
    {
      RetData_ = Disc_.NextPermutationWithRep();
      if (RetData_)
      {
        if (Interactive_ && !IntroDone_)
        {
          if (InAsync_)
            PromptAsyncUser("Enter \"q\" to quit", FALSE);
          else
            fprintf(GetOutputFile(), "Enter \"q\" to quit:\n");
          IntroDone_ = 1;
        }

        TempStr_ = "";
        for (x = 0; x < Sel_; x++)
        {
          RetStr_ = ParamArray_[RetData_[x]]->_Param->GetExpression();
          if (Interactive_)
            if (InAsync_)
            {
              TempStr_ += RetStr_;
              if (x == Sel_ - 1)
              {
                TempStr_ += "\t";
                PromptAsyncUser(TempStr_.c_str(), TRUE);
              }
              else
                TempStr_ += ",";
            }
            else
            {
              fprintf(GetOutputFile(), "%s", RetStr_);
              if (x == Sel_ - 1)
                fprintf(GetOutputFile(), "\t");
              else
                fprintf(GetOutputFile(), ",", RetStr_);
            }

          Answer_ += RetStr_;
          if (x == Sel_ - 1)
            Answer_ += "\n";
          else
            Answer_ += ",";
        }

        if (Interactive_)
          if (InAsync_)
          {
            RetStr_ = ProgramStr();
            if (RetStr_ && toupper(RetStr_[0]) == 'Q')
            {
              ProgramPopped_ = TRUE;
              PopActiveProgram();
              break;
            }
          }
          else
          {
            RetStr_ = fgets(Buffer_, 8, stdin);
            if (RetStr_ && toupper(RetStr_[0]) == 'Q')
              break;
          }
      }
      else
        break;
    }
  }

  MemMatrix::Matrix().Deallocate(Data_);
  SetHasAnswer(Answer_.strlen());

  if (Answer_.strlen())
  {
    Answer_ = ChrString("\"") + Answer_ + ChrString("\"");
    SetLiteralStr(Answer_.c_str());
  }

  if (!ProgramPopped_)
    PopActiveProgram();

  if (InAsync_ && Interactive_)
  {
    ClearOutputFile();
    ClearUserPromptFile();
  }

  return Answer_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::FindCombinationsWithRep(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Answer_;

  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();
    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("ITEMCOMBREP(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        SetAnswerMode(LITERAL_MODE, TRUE);
        return AnswerStr_;
      }
    }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "ITEMCOMBREP");
    CloseOutputFile();
    return Answer_;
  }

  int ProgramPopped_ = TRUE;
  int Sel_ = atoi(ParamArray_[0]->_Param->GetExpression());
  int Size_ = NumParams_ - 2;
  int HasIndex_ = atoi(ParamArray_[Size_ + 1]->_Param->GetExpression());
  int InAsync_ = InAsyncMode();
  int Interactive_ = 0;
  char Buffer_[8];
  ChrString TempStr_;

  DiscreteSet<int> Disc_(0);
  int* Data_ = NULL;
  int* RetData_;
  Data_ = (int*)allocmem(Size_, sizeof(int), Data_);

  int x;
  for (x = 0; x < Size_; x++)
    Data_[x] = x + 1;

  if (Sel_ <= Size_)
  {
    int IntroDone_ = 0;
    int StdStreams_ = StdInput() && StdOutput();
    Interactive_ = (StdStreams_ || InAsync_) && !HasIndex_;
    CalcProgram* Program_;

    if (Interactive_ && InAsync_)
    {
      Program_ = new CalcProgram;
      Program_->SetCalculator(this);
      PushActiveProgram(Program_);
      ProgramPopped_ = FALSE;
    }

    const char* RetStr_;
    Disc_.CombineWithRep(Data_, Size_, Sel_);

    for (;;)
    {
      RetData_ = Disc_.NextCombinationWithRep();
      if (RetData_)
      {
        if (Interactive_ && !IntroDone_)
        {
          if (InAsync_)
            PromptAsyncUser("Enter \"q\" to quit", FALSE);
          else
            fprintf(GetOutputFile(), "Enter \"q\" to quit:\n");
          IntroDone_ = 1;
        }

        TempStr_ = "";
        for (x = 0; x < Sel_; x++)
        {
          RetStr_ = ParamArray_[RetData_[x]]->_Param->GetExpression();
          if (Interactive_)
            if (InAsync_)
            {
              TempStr_ += RetStr_;
              if (x == Sel_ - 1)
              {
                TempStr_ += "\t";
                PromptAsyncUser(TempStr_.c_str(), TRUE);
              }
              else
                TempStr_ += ",";
            }
            else
            {
              fprintf(GetOutputFile(), "%s", RetStr_);
              if (x == Sel_ - 1)
                fprintf(GetOutputFile(), "\t");
              else
                fprintf(GetOutputFile(), ",", RetStr_);
            }

          Answer_ += RetStr_;
          if (x == Sel_ - 1)
            Answer_ += "\n";
          else
            Answer_ += ",";
        }

        if (Interactive_)
          if (InAsync_)
          {
            RetStr_ = ProgramStr();
            if (RetStr_ && toupper(RetStr_[0]) == 'Q')
            {
              ProgramPopped_ = TRUE;
              PopActiveProgram();
              break;
            }
          }
          else
          {
            RetStr_ = fgets(Buffer_, 8, stdin);
            if (RetStr_ && toupper(RetStr_[0]) == 'Q')
              break;
          }
      }
      else
        break;
    }
  }

  MemMatrix::Matrix().Deallocate(Data_);
  SetHasAnswer(Answer_.strlen());

  if (Answer_.strlen())
  {
    Answer_ = ChrString("\"") + Answer_ + ChrString("\"");
    SetLiteralStr(Answer_.c_str());
  }

  if (!ProgramPopped_)
    PopActiveProgram();

  if (InAsync_ && Interactive_)
  {
    ClearOutputFile();
    ClearUserPromptFile();
  }

  return Answer_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::BitwiseOps(const LongNumber& value1, const LongNumber& value2, int optype)
{
  ChrString Binval1_ = value1.ToString(2);
  ChrString Binval2_ = value2.ToString(2);
  long Diff_;
  Subscript len_;

  if (optype == Mcalc_OperatorType::Logical_Not)
  {
    Subscript DigitLen_ =
        (value2.ConvertedFromBase() == 16) ? 4:
        (value2.ConvertedFromBase() == 8)  ? 3:0;

    if (DigitLen_)
    {
      len_ = Binval2_.strlen();
      len_ = len_ % DigitLen_;

      if (len_)
      {
        len_ = 4 - len_;
        ChrString Fill_('0', len_);
        Binval2_ = Fill_ + Binval2_;
      }
    }

    BitString Bitstr_(Binval2_);
    Bitstr_.Toggle();
    Binval2_ = Bitstr_.To_String();
    Binval2_.BinToHex();
    return Binval2_;
  }

  if (Binval1_.strlen() < Binval2_.strlen())
  {
    Diff_ = Binval2_.strlen() - Binval1_.strlen();
    Binval1_.Insert(0, ChrString('0', Diff_));
  }
  else if (Binval2_.strlen() < Binval1_.strlen())
  {
    Diff_ = Binval1_.strlen() - Binval2_.strlen();
    Binval2_.Insert(0, ChrString('0', Diff_));
  }

  BitString Bitstr1_(Binval1_);
  BitString Bitstr2_(Binval2_);

  switch (optype)
  {
    case Mcalc_OperatorType::Exclusive_Or:
      Bitstr1_ ^= Bitstr2_;
      break;

    case Mcalc_OperatorType::Logical_And:
      Bitstr1_ &= Bitstr2_;
      break;

    case Mcalc_OperatorType::Logical_Or:
      Bitstr1_ |= Bitstr2_;
      break;
  }

  Binval1_ = Bitstr1_.To_String();
  Binval1_.BinToHex();
  return Binval1_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ConvertFromHex(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Temp_ = ParamArray_[0]->_Param->GetExpression();
  Temp_.RemovePadding(" \n\t");

  if (!Temp_.IsIntHex())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FROMHEX");
    CloseOutputFile();
    SetHasAnswer(FALSE);
    ChrString Dummy_;
    return Dummy_;
  }

  SetHasAnswer(TRUE);
  LongNumber Num_(Temp_);
  return Num_.ToString();
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ConvertFromOct(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Temp_ = ParamArray_[0]->_Param->GetExpression();
  Temp_.RemovePadding(" \n\t");

  if (!Temp_.IsIntOct())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FROMOCT");
    CloseOutputFile();
    SetHasAnswer(FALSE);
    ChrString Dummy_;
    return Dummy_;
  }

  SetHasAnswer(TRUE);
  LongNumber Num_(Temp_);
  return Num_.ToString();
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ConvertFromBin(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Temp_ = ParamArray_[0]->_Param->GetExpression();
  Temp_.RemovePadding(" \n\t");

  if (!Temp_.IsIntBin() || !Temp_.BinToHex())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FROMBIN");
    CloseOutputFile();
    SetHasAnswer(FALSE);
    ChrString Dummy_;
    return Dummy_;
  }

  SetHasAnswer(TRUE);
  LongNumber Num_ = Temp_;
  return Num_.ToString();
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ConvertFromCmp2Bin(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Temp_ = ParamArray_[0]->_Param->GetExpression();
  Temp_.RemovePadding(" \n\t");

  if (!Temp_.IsIntBin() || !Temp_.Cmp2BinToBin() || !Temp_.BinToHex())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FROMCMP2BIN");
    CloseOutputFile();
    SetHasAnswer(FALSE);
    ChrString Dummy_;
    return Dummy_;
  }

  SetHasAnswer(TRUE);
  LongNumber Num_ = Temp_;
  return Num_.ToString();
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ConvertToHex(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Old_ = ParamArray_[0]->_Param->GetExpression();
  ChrString Temp_ = Old_;
  Temp_.RemovePadding(" \n\t");

  if (!Temp_.IsIntDec())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "TOHEX");
    CloseOutputFile();
    SetHasAnswer(FALSE);
    ChrString Dummy_;
    return Dummy_;
  }

  LongNumber Num_ = Temp_;
  Temp_ = Num_.ToString(16);
  SetLiteralStr(Temp_.c_str());
  return Old_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ConvertToOct(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Old_ = ParamArray_[0]->_Param->GetExpression();
  ChrString Temp_ = Old_;
  Temp_.RemovePadding(" \n\t");

  if (!Temp_.IsIntDec())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "TOOCT");
    CloseOutputFile();
    SetHasAnswer(FALSE);
    ChrString Dummy_;
    return Dummy_;
  }

  LongNumber Num_ = Temp_;
  Temp_ = Num_.ToString(8);
  SetLiteralStr(Temp_.c_str());
  return Old_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ConvertToBin(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Old_ = ParamArray_[0]->_Param->GetExpression();
  ChrString Temp_ = Old_;
  Temp_.RemovePadding(" \n\t");

  if (!Temp_.IsIntDec())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "TOBIN");
    CloseOutputFile();
    SetHasAnswer(FALSE);
    ChrString Dummy_;
    return Dummy_;
  }

  LongNumber Num_ = Temp_;
  Temp_ = Num_.ToString(2);
  SetLiteralStr(Temp_.c_str());
  return Old_;
}

/*****************************************************************************/
template <class T>
ChrString LineCalculator<T>::ConvertToCmp2Bin(int NumParams_, FncParameter** ParamArray_)
{
  ChrString Old_ = ConvertToBin(NumParams_, ParamArray_);
  ChrString Result_ = LiteralStr();

  if (HasErrors())
    return Result_;

  int BytesRequested_ = 0;
  if (NumParams_ >= 2)
    BytesRequested_ = atoi(ParamArray_[1]->_Param->GetAnswer());

  if (!Result_.BinToCmp2Bin(BytesRequested_))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "TOCMP2BIN");
    CloseOutputFile();
    SetHasAnswer(FALSE);
    ChrString Dummy_;
    return Dummy_;
  }

  SetLiteralStr(Result_.c_str());
  return Old_;
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::GiveRandom(int Num_)
{
  return _Random.GiveRandom(Num_);
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::GetCalcIndex() const
{
  return _CalcIndex;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::GetPrevCalcIndex() const
{
  return _PrevCalcIndex;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetHasCommandOutput(int Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    _HasCommandOut = Flag_;
    _ParentPtr->SetHasCommandOutput(Flag_);
  }
  else
    _HasCommandOut = Flag_;
}

/****************************************************************************/
template <class T>
Boolean LineCalculator<T>::HasCommandOutput() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->HasCommandOutput();

  return _HasCommandOut;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetDisplayOption(int Option_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    _DisplayOption = Option_;
    _ParentPtr->SetDisplayOption(Option_);
  }
  else
    _DisplayOption = Option_;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::GetDisplayOption()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetDisplayOption();

  return _DisplayOption;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetProcessVariables(int Value_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    _ProcessVars = Value_;
    _ParentPtr->SetProcessVariables(Value_);
  }
  else
    _ProcessVars = Value_;
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::ProcessVariables() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->ProcessVariables();

  return _ProcessVars;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::InProgramMode() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->InProgramMode();

  return (_ActiveProgram != NULL);
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::RecursionLimit() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->RecursionLimit();

  return _RecursionLimit;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetRecursionLimit(int Limit_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    if (Limit_ >= 0)
      _RecursionLimit = Limit_;
    _ParentPtr->SetRecursionLimit(Limit_);
  }
  else
  {
    if (Limit_ >= 0)
      _RecursionLimit = Limit_;
  }
}

/****************************************************************************/
template <class T>
double LineCalculator<T>::GraphPrecision() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GraphPrecision();

  return _GraphPrecision;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetGraphPrecision(double Prec_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    if (Prec_ >= 0)
      _GraphPrecision = Prec_;
    _ParentPtr->SetGraphPrecision(Prec_);
  }
  else
  {
    if (Prec_ >= 0)
      _GraphPrecision = Prec_;
  }
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetNumberBase(int Base_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    _NumberBase = Base_;
    _ParentPtr->SetNumberBase(Base_);
  }
  else
    _NumberBase = Base_;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetAngleUnit(int Unit_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    _AngleUnit = Unit_;
    _ParentPtr->SetAngleUnit(Unit_);
  }
  else
    _AngleUnit = Unit_;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::GetAngleUnit() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetAngleUnit();

  return _AngleUnit;
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::SkipEval() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->SkipEval();

  return _SkipEvaluation;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetSkipEval(int Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    _SkipEvaluation = Flag_;
    _ParentPtr->SetSkipEval(Flag_);
  }
  else
    _SkipEvaluation = Flag_;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::TransferVariable(int DestIndex_, int SrceIndex_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    if (_PrevCalcIndex && _PrevCalcIndex != _CalcIndex)
      _ParentPtr->TransferVariable(DestIndex_, SrceIndex_);
  }
  else
  {
    bool AltBase_ = StringAssignment() && AnswerMode() == ALTBASE_MODE;
    ChrString SrceStr_ = _CalcPtrArray[_CalcIndex]->ToString(SrceIndex_);
    _TransferValue = SrceStr_;

    if (AltBase_)
      SrceStr_ = _TransferValue.ToString(10);

    if (_PrevCalcIndex && _PrevCalcIndex != _CalcIndex && !IsStringVar(SrceIndex_) || AltBase_)
    {
      if (_PrevCalcIndex == CalculatorBase::FLOAT)
        ((FloatCalculator*)(_CalcPtrArray[_PrevCalcIndex]))->SetVariable(DestIndex_, Float(SrceStr_).GetData(), TRUE);
      else if (_PrevCalcIndex == CalculatorBase::DOUBLE)
        ((DoubleCalculator*)(_CalcPtrArray[_PrevCalcIndex]))->SetVariable(DestIndex_, Double(SrceStr_).GetData(), TRUE);
      else if (_PrevCalcIndex == CalculatorBase::LONGDOUBLE)
        ((LongDoubleCalculator*)(_CalcPtrArray[_PrevCalcIndex]))->SetVariable(DestIndex_, LongDouble(SrceStr_).GetData(), TRUE);
      else if (_PrevCalcIndex == CalculatorBase::LONGNUM)
        ((LongNumCalculator*)(_CalcPtrArray[_PrevCalcIndex]))->SetVariable(DestIndex_, _TransferValue, TRUE);

      return TRUE;
    }
  }

  return FALSE;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::RestoreCalcType()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    if (_PrevCalcIndex)
    {
      _ParentPtr->SetAngleUnit(_AngleUnit);
      return _ParentPtr->RestoreCalcType();
    }
  }
  else
  {
    if (_PrevCalcIndex)
    {
      SetCalcType(_PrevCalcIndex);
      return TRUE;
    }
  }

  return FALSE;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SwitchCalcType(int CalcType_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    if (CalcType_ && _CalcIndex != CalcType_)
    {
      _ParentPtr->SetAngleUnit(_AngleUnit);
      _ParentPtr->SwitchCalcType(CalcType_);
    }
  }
  else
  {
    if (CalcType_ && _CalcIndex != CalcType_)
    {
      int PrevCalcIndex_ = _CalcIndex;
      int Index_ = CalcType_;
      int VarIndex_;

      for (VarIndex_ = -1; VarIndex_ < MAXVARS; VarIndex_++)
        if (!IsStringVar(VarIndex_))
        {
          ChrString SrceStr_ = _CalcPtrArray[PrevCalcIndex_]->ToString(VarIndex_);
          _TransferValue = SrceStr_;

          if (Index_ == CalculatorBase::LONGNUM)
            ((LongNumCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, _TransferValue, TRUE);
          else if (Index_ == CalculatorBase::FLOAT)
            ((FloatCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, Float(SrceStr_).GetData(), TRUE);
          else if (Index_ == CalculatorBase::DOUBLE)
            ((DoubleCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, Double(SrceStr_).GetData(), TRUE);
          else if (Index_ == CalculatorBase::LONGDOUBLE)
            ((LongDoubleCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, LongDouble(SrceStr_).GetData(), TRUE);
        }

      int x;
      for (x = 1; x < MAXCALC; x++)
        _CalcPtrArray[x]->SetCalcIndexes(PrevCalcIndex_, CalcType_);
    }
  }
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetCalcType(int CalcType_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    if (CalcType_ && _CalcIndex != CalcType_)
    {
      _ParentPtr->SetAngleUnit(_AngleUnit);
      _ParentPtr->SetCalcType(CalcType_);
    }
  }
  else
  {
    if (CalcType_ && _CalcIndex != CalcType_)
    {
      int PrevCalcIndex_ = _CalcIndex;
      int Index_ = CalcType_;
      int VarIndex_ = -1;

      ChrString SrceStr_ = _CalcPtrArray[PrevCalcIndex_]->ToString(VarIndex_);
      _TransferValue = SrceStr_;

      if (Index_ == CalculatorBase::LONGNUM)
        ((LongNumCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, _TransferValue, TRUE);
      else if (Index_ == CalculatorBase::FLOAT)
        ((FloatCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, Float(SrceStr_).GetData(), TRUE);
      else if (Index_ == CalculatorBase::DOUBLE)
        ((DoubleCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, Double(SrceStr_).GetData(), TRUE);
      else if (Index_ == CalculatorBase::LONGDOUBLE)
        ((LongDoubleCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, LongDouble(SrceStr_).GetData(), TRUE);

      int x;
      for (x = 1; x < MAXCALC; x++)
        _CalcPtrArray[x]->SetCalcIndexes(0, CalcType_);
    }
  }
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::TransferCalcType(int CalcType_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    if (CalcType_ && _CalcIndex != CalcType_)
    {
      _ParentPtr->SetAngleUnit(_AngleUnit);
      _ParentPtr->TransferCalcType(CalcType_);
    }
  }
  else
  {
    if (CalcType_ && _CalcIndex != CalcType_)
    {
      int PrevCalcIndex_ = _CalcIndex;
      int Index_ = CalcType_;
      int VarIndex_;

      for (VarIndex_ = -1; VarIndex_ < MAXVARS; VarIndex_++)
        if (!IsStringVar(VarIndex_))
        {
          ChrString SrceStr_ = _CalcPtrArray[PrevCalcIndex_]->ToString(VarIndex_);
          _TransferValue = SrceStr_;

          if (Index_ == CalculatorBase::LONGNUM)
            ((LongNumCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, _TransferValue, TRUE);
          else if (Index_ == CalculatorBase::FLOAT)
            ((FloatCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, Float(SrceStr_).GetData(), TRUE);
          else if (Index_ == CalculatorBase::DOUBLE)
            ((DoubleCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, Double(SrceStr_).GetData(), TRUE);
          else if (Index_ == CalculatorBase::LONGDOUBLE)
            ((LongDoubleCalculator*)(_CalcPtrArray[Index_]))->SetVariable(VarIndex_, LongDouble(SrceStr_).GetData(), TRUE);
        }

      int x;
      for (x = 1; x < MAXCALC; x++)
        _CalcPtrArray[x]->SetCalcIndexes(0, CalcType_);
    }
  }
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetCalcIndexes(int Prev_, int New_)
{
   _PrevCalcIndex = Prev_;
   _CalcIndex = New_;
}

/****************************************************************************/
template <class T>
CalculatorBase* LineCalculator<T>::GetCalcType()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetCalcType();

  return _CalcPtrArray[_CalcIndex];
}

/****************************************************************************/
template <class T>
CalculatorBase* LineCalculator<T>::GetPrevCalcType()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetPrevCalcType();

  if (_PrevCalcIndex)
    return _CalcPtrArray[_PrevCalcIndex];

  return NULL;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetSuppressOutput(int Flag_, int ShowSuppressedErrors_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetSuppressOutput(Flag_, ShowSuppressedErrors_);
  else
  {
    _SuppressOutput = Flag_;
    if (!Flag_ && HasErrors() && ShowSuppressedErrors_)
      DisplayErrorMessageForCode(GetOutputFile(), GetErrorVal());
  }
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::SuppressOutput() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->SuppressOutput();

  return (_SuppressOutput != 0);
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::GetErrorVal() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetErrorVal();

  return _ErrorValue;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::HasErrors() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->HasErrors();

  return (_ErrorValue != 0);
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetError(int ErrVal_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetError(ErrVal_);
  else
    _ErrorValue = ErrVal_;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::ClearError()
{
  SetError(0);
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::AtLowestLevel(int Level_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->AtLowestLevel(Level_);

  return
  (
    Level_ == 0 &&
    _ExprStackIndex == 1 &&
    !_ActiveExpression
  );
}

/*****************************************************************************/
template <class T>
CalcProgram* LineCalculator<T>::ActiveProgram()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->ActiveProgram();

  return _ActiveProgram->_Active;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::PushActiveProgram(CalcProgram* Ptr_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->PushActiveProgram(Ptr_);
  else
  {
    ProgramStack* New_ = new ProgramStack;
    New_->Push(_ActiveProgram);
    New_->Set(Ptr_);
    _ActiveProgram = New_;
  }
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::PopActiveProgram()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->PopActiveProgram();
  else if (_ActiveProgram)
  {
    ProgramStack* Top_ = _ActiveProgram;
    _ActiveProgram = _ActiveProgram->_Next;
    Top_->Set(NULL);
    Top_->PopDelete();
    delete Top_;
  }
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::PushActiveExpression(SubExpression* Ptr_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->PushActiveExpression(Ptr_);
  else
  {
    ExpressionStack* New_ = new ExpressionStack;
    New_->Push(_ActiveExpression);
    New_->Set(Ptr_);
    _ActiveExpression = New_;
  }
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::PopActiveExpression()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->PopActiveExpression();
  else if (_ActiveExpression)
  {
    ExpressionStack* Top_ = _ActiveExpression;
    _ActiveExpression = _ActiveExpression->_Next;
    Top_->Set(NULL);
    Top_->Pop();
    delete Top_;
  }
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetHasAnswer(Boolean Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetHasAnswer(Flag_);
  else
  {
    if (_ActiveExpression && _ActiveExpression->_Active)
      _ActiveExpression->_Active->SetHasAnswer(Flag_);
    else if (_Expression)
      _Expression->SetHasAnswer(Flag_);
    else if (_ExprStackIndex)
      _SubExprStack[_ExprStackIndex - 1]->SetHasAnswer(Flag_);
  }
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::HasAnswer() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->HasAnswer();

  if (_ActiveExpression && _ActiveExpression->_Active)
    return _ActiveExpression->_Active->HasAnswer();

  if (_Expression)
    return _Expression->HasAnswer();

  if (_ExprStackIndex)
    return _SubExprStack[_ExprStackIndex - 1]->HasAnswer();

  return FALSE;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetShowAnswer(Boolean Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetShowAnswer(Flag_);
  else
  {
    if (_ActiveExpression && _ActiveExpression->_Active)
      _ActiveExpression->_Active->SetShowAnswer(Flag_);
    else if (_Expression)
      _Expression->SetShowAnswer(Flag_);
    else if (_ExprStackIndex)
      _SubExprStack[_ExprStackIndex - 1]->SetShowAnswer(Flag_);
  }
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::ShowAnswer() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->ShowAnswer();

  if (_ActiveExpression && _ActiveExpression->_Active)
    return _ActiveExpression->_Active->ShowAnswer();

  if (_Expression)
    return _Expression->ShowAnswer();

  if (_ExprStackIndex)
    return _SubExprStack[_ExprStackIndex - 1]->ShowAnswer();

  return FALSE;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::ResetAnswerMode()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->ResetAnswerMode();
  else
  {
    _ModeFinalized = FALSE;
    _InitialMode = TRUE;
  }
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetAnswerMode(int Mode_, int Final_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetAnswerMode(Mode_);
  else
  {
    _InitialMode = FALSE;

    if (!_ModeFinalized)
    {
      _AnswerMode = Mode_;
      _ModeFinalized = Final_;
    }
  }
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::AnswerMode() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->AnswerMode();

  return _AnswerMode;
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::InitialMode() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->InitialMode();

  return _InitialMode;
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::ModeFinalized() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->ModeFinalized();

  return _ModeFinalized;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetEvalStrings(int Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetEvalStrings(Flag_);
  else
    _EvalStrings = Flag_;
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::EvalStrings() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->EvalStrings();

  return _EvalStrings;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetSetAssignment(int Flag_, const char* Str_, int Null_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetSetAssignment(Flag_, Str_, Null_);
  else
  {
    if (Flag_)
    {
      _SetOperation |= SET_ASSIGNMENT;
      _StringOperation = STRING_ASSIGNMENT;
    }
    else
    {
      _SetOperation = 0;
      _StringOperation = 0;
    }

    if (Str_)
      SetSetResult(Str_, Null_);
  }
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::SetAssignment() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->SetAssignment();

  return (_SetOperation & SET_ASSIGNMENT);
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetSetOperation(int Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetSetOperation(Flag_);
  else
  {
    if (Flag_)
      _SetOperation |= SET_OPERATION;
    else
      _SetOperation = 0;
  }
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::SetOperation() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->SetOperation();

  return (_SetOperation & SET_OPERATION);
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::IsNullSet(int Index_) const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->IsNullSet(Index_);

  return _StrVarData->IsNullSet(Index_);
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetStringAssignment(int Flag_, const char* Str_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetStringAssignment(Flag_, Str_);
  else
  {
    if (Flag_)
      _StringOperation = STRING_ASSIGNMENT;
    else
      _StringOperation = 0;

    if (Str_)
      SetStrResult(Str_);
  }
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::StringAssignment() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->StringAssignment();

  return (_StringOperation == STRING_ASSIGNMENT);
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetStringComparison(int Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetStringComparison(Flag_);
  else
  {
    if (Flag_)
      _StringOperation = STRING_COMPARISON;
    else
      _StringOperation = 0;
  }
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::StringComparison() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->StringComparison();

  return (_StringOperation == STRING_COMPARISON);
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetSetVariable(int x, const ChrString& Str_, int Null_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetSetVariable(x, Str_, Null_);
  else
    _StrVarData->SetSetElements(x, Str_, Null_);
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetStrVariable(int x, const ChrString& Str_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetStrVariable(x, Str_);
  else
    _StrVarData->SetString(x, Str_);
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetStrResult(const char* Str_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetStrResult(Str_);
  else
    _StrVarData->SetResult(Str_);
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetSetResult(const char* Str_, int Null_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetSetResult(Str_, Null_);
  else
    _StrVarData->SetSetResult(Str_, Null_);
}

/*****************************************************************************/
template <class T>
const ChrString& LineCalculator<T>::GetStrResult() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetStrResult();

  return _StrVarData->GetResult();
}

/*****************************************************************************/
template <class T>
const ChrString& LineCalculator<T>::GetStrVariable(int Index_) const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetStrVariable(Index_);

  return _StrVarData->GetVariable(Index_);
}

/*****************************************************************************/
template <class T>
const ChrString& LineCalculator<T>::GetSetResult() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetSetResult();

  return _StrVarData->GetSetResult();
}

/*****************************************************************************/
template <class T>
const ChrString& LineCalculator<T>::GetSetVariable(int Index_) const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetSetVariable(Index_);

  return _StrVarData->GetSetVariable(Index_);
}

/*****************************************************************************/
template <class T>
size_t LineCalculator<T>::GetStrVarLen(int Index_) const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->GetStrVarLen(Index_);

  if (Index_ == CalculatorBase::LITERAL_MODE ||
      Index_ == CalculatorBase::ALTBASE_MODE ||
      Index_ == CalculatorBase::FRACTION_MODE)
    if (Index_ == CalculatorBase::FRACTION_MODE)
    {
      const char* Fraction_ = FractionStr();
      return ::SafeStrLen(Fraction_);
    }
    else
    {
      const char* Literal_ = LiteralStr();
      return ::SafeStrLen(Literal_);
    }

  if (Index_ < MAXVARS)
    return _StrVarData->GetVarLen(Index_);

  return 0;
}

/*****************************************************************************/
template <class T>
char* LineCalculator<T>::CopyStrVar(char* DestStr_, int Index_)
{
  static char* Dummy_ = NULL;
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->CopyStrVar(DestStr_, Index_);

  if (Index_ == CalculatorBase::LITERAL_MODE ||
      Index_ == CalculatorBase::ALTBASE_MODE ||
      Index_ == CalculatorBase::FRACTION_MODE)
    if (Index_ == CalculatorBase::FRACTION_MODE)
    {
      const char* Fraction_ = FractionStr();
      if (Fraction_)
        strcpy(DestStr_, Fraction_);
      return DestStr_;
    }
    else
    {
      const char* Literal_ = LiteralStr();
      if (Literal_)
        strcpy(DestStr_, Literal_);
      return DestStr_;
    }

  if (Index_ < MAXVARS)
    return _StrVarData->CopyStrVar(DestStr_, Index_);

  return Dummy_;
}

/*****************************************************************************/
template <class T>
ChrString& LineCalculator<T>::CopyStrVar(ChrString& DestStr_, int Index_)
{
  static ChrString Dummy_;
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->CopyStrVar(DestStr_, Index_);

  if (Index_ == CalculatorBase::LITERAL_MODE ||
      Index_ == CalculatorBase::ALTBASE_MODE ||
      Index_ == CalculatorBase::FRACTION_MODE)
    if (Index_ == CalculatorBase::FRACTION_MODE)
    {
      const char* Fraction_ = FractionStr();
      if (Fraction_)
        DestStr_ = Fraction_;
      return DestStr_;
    }
    else
    {
      const char* Literal_ = LiteralStr();
      if (Literal_)
        DestStr_ = Literal_;
      return DestStr_;
    }

  if (Index_ < MAXVARS)
    return _StrVarData->CopyStrVar(DestStr_, Index_);

  return Dummy_;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetStringVar(int Index_, Boolean Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetStringVar(Index_, Flag_);
  else
    _StrVarData->SetStringVar(Index_, Flag_);
}

/*****************************************************************************/
template <class T>
Boolean LineCalculator<T>::IsStringVar(int Index_) const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->IsStringVar(Index_);

  if (Index_ == CalculatorBase::LITERAL_MODE ||
      Index_ == CalculatorBase::ALTBASE_MODE ||
      Index_ == CalculatorBase::FRACTION_MODE)
    return TRUE;

  if (Index_ < MAXVARS)
    return _StrVarData->IsStringVar(Index_);

  return FALSE;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetSetVar(int Index_, int Flag_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetSetVar(Index_, Flag_);
  else
    _StrVarData->SetSetVar(Index_, Flag_);
}

/*****************************************************************************/
template <class T>
int LineCalculator<T>::IsSetVar(int Index_) const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->IsSetVar(Index_);

  if (Index_ == CalculatorBase::LITERAL_MODE ||
      Index_ == CalculatorBase::ALTBASE_MODE ||
      Index_ == CalculatorBase::FRACTION_MODE)
    return TRUE;

  if (Index_ < MAXVARS)
    return _StrVarData->IsSetVar(Index_);

  return FALSE;
}

/*****************************************************************************/
template <class T>
Boolean LineCalculator<T>::IsHexVar(int Index_) const
{
  if (IsStringVar(Index_))
  {
    ChrString VarStr_ = GetStrVariable(Index_);
    return IsIntHex(VarStr_.c_str());
  }

  return FALSE;
}

/*****************************************************************************/
template <class T>
Boolean LineCalculator<T>::IsOctVar(int Index_) const
{
  if (IsStringVar(Index_))
  {
    ChrString VarStr_ = GetStrVariable(Index_);
    return IsIntOct(VarStr_.c_str());
  }

  return FALSE;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::ClearOriginalStr()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->ClearOriginalStr();
  else
  {
    if (_OriginalStr)
      _OriginalStr[0] = 0;
  }
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::ClearAnswerStr()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->ClearAnswerStr();
  else
  {
    int Mode_ = AnswerMode();
    if (Mode_ == FLOATINGPOINT_MODE && _AnswerStr)
      _AnswerStr[0] = 0;
    else if (Mode_ == LITERAL_MODE && _LiteralStr)
      _LiteralStr[0] = 0;
    else if (Mode_ == FRACTION_MODE && _FractionStr)
      _FractionStr[0] = 0;

    SetHasAnswer(FALSE);
  }
}

/****************************************************************************/
template <class T>
const char* LineCalculator<T>::AnswerStr() const
{
  int Mode_ = AnswerMode();
  return
  (
    (Mode_ == CalculatorBase::FRACTION_MODE) ? FractionStr():
    (Mode_ == CalculatorBase::LITERAL_MODE ||
     Mode_ == CalculatorBase::ALTBASE_MODE) ? LiteralStr():FloatStr()
  );
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetFloatStr(const char* Str_)
{
  if (Str_)
  {
    if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
      _ParentPtr->SetFloatStr(Str_);
    else
    {
      if (_AnswerStr && ::SafeStrLen(Str_) > ::SafeStrLen(_AnswerStr))
      {
        MemMatrix::Matrix().Deallocate(_AnswerStr);
        _AnswerStr = ::NewString(Str_);
      }
      else
      {
        if (!_AnswerStr)
          _AnswerStr = (char*)MemMatrix::Matrix().Allocate(::SafeStrLen(Str_) + 5);

        if (Str_ && Str_[0])
          strcpy(_AnswerStr, Str_);
        else
          _AnswerStr[0] = 0;
      }

      SetHasAnswer(strlen(_AnswerStr));
    }
  }
}

/****************************************************************************/
template <class T>
const char* LineCalculator<T>::FloatStr() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->FloatStr();

  return _AnswerStr;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetOriginalStr(const char* Str_)
{
  if (Str_)
  {
    if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
      _ParentPtr->SetOriginalStr(Str_);
    else
    {
      if (_OriginalStr && ::SafeStrLen(Str_) > ::SafeStrLen(_OriginalStr))
      {
        MemMatrix::Matrix().Deallocate(_OriginalStr);
        _OriginalStr = ::NewString(Str_);
      }
      else
      {
        if (!_OriginalStr)
          _OriginalStr = (char*)MemMatrix::Matrix().Allocate(::SafeStrLen(Str_) + 5);

        if (Str_ && Str_[0])
          strcpy(_OriginalStr, Str_);
        else
          _OriginalStr[0] = 0;
      }
    }
  }
}

/****************************************************************************/
template <class T>
const char* LineCalculator<T>::OriginalStr() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->OriginalStr();

  return _OriginalStr;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetFractionStr(const char* Frac_)
{
  if (Frac_)
  {
    if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
      _ParentPtr->SetFractionStr(Frac_);
    else
    {
      if (_FractionStr && ::SafeStrLen(Frac_) > ::SafeStrLen(_FractionStr))
      {
        MemMatrix::Matrix().Deallocate(_FractionStr);
        _FractionStr = ::NewString(Frac_);
      }
      else
      {
        if (!_FractionStr)
          _FractionStr = (char*)MemMatrix::Matrix().Allocate(::SafeStrLen(Frac_) + 5);

        if (Frac_ && Frac_[0])
          strcpy(_FractionStr, Frac_);
        else
          _FractionStr[0] = 0;
      }

      SetHasAnswer(strlen(_FractionStr));
    }
  }
}

/*****************************************************************************/
template <class T>
const char* LineCalculator<T>::FractionStr() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->FractionStr();

  return _FractionStr;
}

/*****************************************************************************/
template <class T>
void LineCalculator<T>::SetLiteralStr(const char* Literal_)
{
  if (Literal_)
  {
    if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
      _ParentPtr->SetLiteralStr(Literal_);
    else
    {
      if (_LiteralStr && ::SafeStrLen(Literal_) > ::SafeStrLen(_LiteralStr))
      {
        MemMatrix::Matrix().Deallocate(_LiteralStr);
        _LiteralStr = ::NewString(Literal_);
      }
      else
      {
        if (!_LiteralStr)
          _LiteralStr = (char*)MemMatrix::Matrix().Allocate(::SafeStrLen(Literal_) + 5);

        if (Literal_ && Literal_[0])
          strcpy(_LiteralStr, Literal_);
        else
          _LiteralStr[0] = 0;
      }

      SetHasAnswer(strlen(_LiteralStr));
    }
  }
}

/*****************************************************************************/
template <class T>
const char* LineCalculator<T>::LiteralStr() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->LiteralStr();

  return _LiteralStr;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetProgramStr(const char* Str_)
{
  if (Str_)
  {
    if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
      _ParentPtr->SetProgramStr(Str_);
    else
    {
      if (_ProgramStr && ::SafeStrLen(Str_) > ::SafeStrLen(_ProgramStr))
      {
        MemMatrix::Matrix().Deallocate(_ProgramStr);
        _ProgramStr = ::NewString(Str_);
      }
      else
      {
        if (!_ProgramStr)
          _ProgramStr = (char*)MemMatrix::Matrix().Allocate(::SafeStrLen(Str_) + 5);

        if (Str_ && Str_[0])
          strcpy(_ProgramStr, Str_);
        else
          _ProgramStr[0] = 0;
      }
    }
  }
}

/****************************************************************************/
template <class T>
const char* LineCalculator<T>::ProgramStr() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->ProgramStr();

  return _ProgramStr;
}

/****************************************************************************/
template <class T>
const char* LineCalculator<T>::GetPrecision()
{
  return
  (
    (_CalcIndex == CalculatorBase::FLOAT)      ? "Float (7-digit precision)":
    (_CalcIndex == CalculatorBase::DOUBLE)     ? "Double (15-digit precision)":
    (_CalcIndex == CalculatorBase::LONGDOUBLE) ? "Long Double (19-digit precision)":
    (_CalcIndex == CalculatorBase::LONGNUM)    ? "Long Number (arbitrary precision)":
                                                 "Error: Invalid Precision"
  );
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetParent(CalculatorBase* Ptr_)
{
  _ParentPtr = Ptr_;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::ProcessExpr(Mcalc_Operand** operands, char** arglist, int optype, CalculatorBase* CalcPtr_)
{
  int Confirmed_ = _ExprStackIndex == 1 && !_ActiveExpression;
  CalcPtr_ = CalcPtr_->GetCalcType();
  CalcPtr_->SetAnswerMode(FLOATINGPOINT_MODE, Confirmed_);

  if (CalcPtr_->GetCalcIndex() == CalculatorBase::FLOAT)
    ((FloatCalculator*)CalcPtr_)->DirectProcess(operands, arglist, optype);
  else if (CalcPtr_->GetCalcIndex() == CalculatorBase::DOUBLE)
    ((DoubleCalculator*)CalcPtr_)->DirectProcess(operands, arglist, optype);
  else if (CalcPtr_->GetCalcIndex() == CalculatorBase::LONGDOUBLE)
    ((LongDoubleCalculator*)CalcPtr_)->DirectProcess(operands, arglist, optype);
  else if (CalcPtr_->GetCalcIndex() == CalculatorBase::LONGNUM)
    ((LongNumCalculator*)CalcPtr_)->DirectProcess(operands, arglist, optype);

  SetStringComparison(FALSE);
  SetSetOperation(FALSE);
}

/****************************************************************************/
template <class T>
CalculatorBase* LineCalculator<T>::SetupCalculators(int CalcType_)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: SetupCalculators\n", logfile);
  #endif

  if (!_ParentPtr)
    return this;

  _ParentPtr = NULL;
  _StrVarData = new StrVarData();
  _AssignmentOrder = (char*)MemMatrix::Matrix().Allocate(_StackSize + 1);
  _AssignmentOrder[0] = 0;
  ResetIndexes();
  _SubExprStack = (SubExpression**)CalculatorBase::allocmem(_ExprStackLimit, sizeof(SubExpression*), _SubExprStack);

  int i;
  for (i = 0; i < _ExprStackLimit; i++)
    _SubExprStack[i] = NULL;

  CreateNewSubExpr();

  if (CalcType_ == CalculatorBase::FLOAT)
    _CalcPtrArray[CalculatorBase::FLOAT] = this;
  else
  {
    _CalcPtrArray[CalculatorBase::FLOAT] = new FloatCalculator();
    _CalcPtrArray[CalculatorBase::FLOAT]->SetParent(this);
    _CalcPtrArray[CalculatorBase::FLOAT]->ResetIndexes();
  }

  if (CalcType_ == CalculatorBase::DOUBLE)
    _CalcPtrArray[CalculatorBase::DOUBLE] = this;
  else
  {
    _CalcPtrArray[CalculatorBase::DOUBLE] = new DoubleCalculator();
    _CalcPtrArray[CalculatorBase::DOUBLE]->SetParent(this);
    _CalcPtrArray[CalculatorBase::DOUBLE]->ResetIndexes();
  }

  if (CalcType_ == CalculatorBase::LONGDOUBLE)
    _CalcPtrArray[CalculatorBase::LONGDOUBLE] = this;
  else
  {
    _CalcPtrArray[CalculatorBase::LONGDOUBLE] = new LongDoubleCalculator();
    _CalcPtrArray[CalculatorBase::LONGDOUBLE]->SetParent(this);
    _CalcPtrArray[CalculatorBase::LONGDOUBLE]->ResetIndexes();
  }

  if (CalcType_ == CalculatorBase::LONGNUM)
    _CalcPtrArray[CalculatorBase::LONGNUM] = this;
  else
  {
    _CalcPtrArray[CalculatorBase::LONGNUM] = new LongNumCalculator();
    _CalcPtrArray[CalculatorBase::LONGNUM]->SetParent(this);
    _CalcPtrArray[CalculatorBase::LONGNUM]->ResetIndexes();
  }

  _CalcIndex = (CalcType_ == CalculatorBase::FLOAT)      ? CalculatorBase::FLOAT:
               (CalcType_ == CalculatorBase::DOUBLE)     ? CalculatorBase::DOUBLE:
               (CalcType_ == CalculatorBase::LONGDOUBLE) ? CalculatorBase::LONGDOUBLE:
                                                           CalculatorBase::LONGNUM;
  #if CALCLIB_DEBUG2
    fputs("Leaving: SetupCalculators\n", logfile);
  #endif

  return this;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::GetIndex(int& Pos_, int& Val_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->GetIndex(Pos_, Val_);
  else
  {
    char* Ptr_ = _AssignmentOrder;

    if (SafeStrLen(Ptr_))
    {
      Pos_ = *Ptr_ - '0';
      Ptr_++;

      if (SafeStrLen(Ptr_))
        Val_ = *Ptr_ - '0';
      else
        Val_ = 0;
    }
    else
      Pos_ = Val_ = 0;
  }
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::HasIndex()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->HasIndex();

  return ::SafeStrLen(_AssignmentOrder);
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::PopIndex()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->PopIndex();
  else
  {
    int len = ::SafeStrLen(_AssignmentOrder);
    if (len && len - 2)
      _AssignmentOrder = (char*)memmove(_AssignmentOrder, &_AssignmentOrder[2], len - 1);
    else
      _AssignmentOrder[0] = 0;
  }
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::SetIndex(int Pos_, int Val_)
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->SetIndex(Pos_, Val_);
  else
  {
    char Buffer_[3];
    int len = strlen(_AssignmentOrder);
    int Mode_ = AnswerMode();

    if (!IsStringVar(Pos_) && Val_ == STRING_CONCAT)
      Val_ = STRING_ASSIGN;

    if (!IsSetVar(Pos_) && Val_ == SET_ASSIGN)
      Val_ = SET_ASSIGN;

    if (len >= _StackSize)
    {
      char* Temp_ = _AssignmentOrder;
      _StackSize += STACKSIZE;
      _AssignmentOrder = (char*)MemMatrix::Matrix().Allocate(_StackSize + 1);
      strcpy(_AssignmentOrder, Temp_);
      len = strlen(_AssignmentOrder);
      MemMatrix::Matrix().Deallocate(Temp_);
      Temp_ = NULL;
    }

    Buffer_[0] = Pos_ + '0';
    Buffer_[1] = Val_ + '0';
    Buffer_[2] = 0;
    strcpy(&_AssignmentOrder[len], Buffer_);

    SetSetVar(Pos_, Val_ == SET_ASSIGN || Val_ == SET_UNION ||
                    Val_ == SET_INTERSECTION || Val_ == SET_NEGATION ||
                    Val_ == SET_EXCLUSION);
    SetStringVar(Pos_, (Val_ == STRING_ASSIGN || Val_ == STRING_CONCAT ||
                       (Mode_ == ALTBASE_MODE && (Val_ == ASSIGN_AND || Val_ == ASSIGN_OR))));
  }
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::ResetIndexes()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    _ParentPtr->ResetIndexes();
  else
    _AssignmentOrder[0] = 0;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::NumberBase() const
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
    return _ParentPtr->NumberBase();

  return _NumberBase;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::RestoreNumberBase()
{
  if (_ParentPtr && _ParentPtr != ((CalculatorBase*)this))
  {
    _NumberBase = 10;
    _ParentPtr->RestoreNumberBase();
  }
  else
  {
    if (!HasErrors())
      if (AnswerMode() == FLOATINGPOINT_MODE && GetCalcIndex() == CalculatorBase::LONGNUM)
      {
        int Confirmed_ = _ExprStackIndex == 1 && !_ActiveExpression;
        if (_NumberBase == 16)
        {
          LongNumber Result_ = ((LongNumCalculator*)(_CalcPtrArray[_CalcIndex]))->GetResult();
          ChrString ResStr_ = Result_.ToString(16);
          SetAnswerMode(ALTBASE_MODE, Confirmed_);
          SetLiteralStr(ResStr_.c_str());
        }
        else if (_NumberBase == 8)
        {
          LongNumber Result_ = ((LongNumCalculator*)(_CalcPtrArray[_CalcIndex]))->GetResult();
          ChrString ResStr_ = Result_.ToString(8);
          SetAnswerMode(ALTBASE_MODE, Confirmed_);
          SetLiteralStr(ResStr_.c_str());
        }
      }

    _NumberBase = 10;
  }
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::ProcessCmdLine(int argc, char** argv)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: ProcessCmdLine\n", logfile);
  #endif

  Subscript Pos_;
  ChrString TempStr_;
  Integer SesNum_;
  CalculatorBase* CalcPtr_ = this;
  char** argline = NULL;
  char* buffer = NULL;
  char* RetStr_ = NULL;
  const char* ModeStr_;
  const char* CalcStr_;
  int Index_;
  int UsesSesNum_ = argc == 3 && icasestrcomp(argv[1], "-x") && IsUint(argv[2]);
  int NoPrompt_ = argc == 2 && icasestrcomp(argv[1], "-np");

  if (argc == 1 || UsesSesNum_ || NoPrompt_)
  {
    SetUsesSessionNum(UsesSesNum_);

    int HasInputFile_ = 0;
    int HasOutputFile_ = 0;
    int ParamCnt_ = 0;
    int SessionNum_ = 0;

    if (UsesSesNum_)
    {
      argline = (char**)CalculatorBase::allocmem(2, sizeof(char*), argline);
      argline[ParamCnt_++] = NULL;
      argline[ParamCnt_++] = ::NewString("-x");

      SessionNum_ = atoi(argv[2]);
      SetSessionNum(SessionNum_);
      SesNum_ = SessionNum();

    #if (!defined(__DJGPP__) & !defined(__linux__) & !defined(__unix__) & !defined(__GNUC__))
      TempStr_ = MCALC_LOG_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      LOG_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_USER_INPUT_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      USER_INPUT_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_USER_PROMPT_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      USER_PROMPT_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_PROG_OUTPUT_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      PROG_OUTPUT_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_IOSTATE_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      PROG_IOSTATE_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_CURRENTGRAPH_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      CURRENTGRAPH_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_SERVERALIVE_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      SERVERALIVE_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_CLIENTALIVE_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      CLIENTALIVE_FILE = ::NewString(TempStr_.c_str());
    #else
      TempStr_ = MCALC_LOG_FILE;
      Pos_ = TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      LOG_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_USER_INPUT_FILE;
      Pos_ = TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      USER_INPUT_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_USER_PROMPT_FILE;
      Pos_ = TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      USER_PROMPT_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_PROG_OUTPUT_FILE;
      Pos_ = TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      PROG_OUTPUT_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_IOSTATE_FILE;
      Pos_ = TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      PROG_IOSTATE_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_CURRENTGRAPH_FILE;
      Pos_ = (Subscript&)TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      CURRENTGRAPH_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_SERVERALIVE_FILE;
      Pos_ = TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      SERVERALIVE_FILE = ::NewString(TempStr_.c_str());

      TempStr_ = MCALC_CLIENTALIVE_FILE;
      Pos_ = TempStr_.find('.');
      TempStr_.Insert(Pos_, SesNum_.ToString());
      CLIENTALIVE_FILE = ::NewString(TempStr_.c_str());
    #endif

      ProcessCmdLine(ParamCnt_, argline);
      MemMatrix::Matrix().Deallocate(argline[1]);
      MemMatrix::Matrix().Deallocate(argline);
      argline = NULL;
      RemoveSessionFiles(SessionNum_);

      #if CALCLIB_DEBUG2
        fputs("Leaving: ProcessCmdLine\n", logfile);
      #endif

      return 0;
    }
    else
    {
      CURRENTGRAPH_FILE = ::NewString(MCALC_CURRENTGRAPH_FILE);
      buffer = (char*)CalculatorBase::allocmem(INPUT_BUFFER_SIZE, sizeof(char), buffer);
      FILE* Fptr_ = FileOpen(MCALC_CONFIG_FILE, "r");

      if (Fptr_)
      {
        RetStr_ = fgets(buffer, INPUT_BUFFER_SIZE, Fptr_);
        HasInputFile_ = atoi(RetStr_);
        RetStr_ = fgets(buffer, INPUT_BUFFER_SIZE, Fptr_);
        HasOutputFile_ = atoi(RetStr_);

        argline = (char**)CalculatorBase::allocmem(5, sizeof(char*), argline);
        argline[ParamCnt_++] = NULL;

        if (HasInputFile_)
        {
          argline[ParamCnt_++] = ::NewString("-i");
          RetStr_ = fgets(buffer, INPUT_BUFFER_SIZE, Fptr_);
          RetStr_ = RemovePadding(RetStr_, " \t\n");
          DATA_INPUT_FILE = ::NewString(RetStr_);
          argline[ParamCnt_++] = DATA_INPUT_FILE;
        }

        if (HasOutputFile_)
        {
          argline[ParamCnt_++] = ::NewString("-o");
          RetStr_ = fgets(buffer, INPUT_BUFFER_SIZE, Fptr_);
          RetStr_ = RemovePadding(RetStr_, " \t\n");
          DATA_OUTPUT_FILE = ::NewString(RetStr_);
          argline[ParamCnt_++] = DATA_OUTPUT_FILE;
        }

        if (HasInputFile_ || HasOutputFile_)
          ProcessCmdLine(ParamCnt_, argline);

        for (Index_ = 1; Index_ < ParamCnt_; Index_++)
          if (argline[Index_] != DATA_OUTPUT_FILE &&
              argline[Index_] != DATA_INPUT_FILE)
            MemMatrix::Matrix().Deallocate(argline[Index_]);

        MemMatrix::Matrix().Deallocate(argline);
        argline = NULL;
        fclose(Fptr_);

        if (HasInputFile_ || HasOutputFile_)
        {
          MemMatrix::Matrix().Deallocate(buffer);
          buffer = NULL;
          return 0;
        }
      }
      else
      {
        DATA_INPUT_FILE = NULL;
        DATA_OUTPUT_FILE = NULL;
      }
    }

    _StdInput = TRUE;
    _StdOutput = TRUE;

    do
    {
      CalcPtr_ = CalcPtr_->GetCalcType();
      ModeStr_ = (CalcPtr_->GetAngleUnit() == RADIAN) ? "rad":"deg";
      Index_ = CalcPtr_->GetCalcIndex();
      CalcStr_ = (Index_ == CalculatorBase::FLOAT) ? "flt":
                 (Index_ == CalculatorBase::DOUBLE) ? "dbl":
                 (Index_ == CalculatorBase::LONGDOUBLE) ? "ldbl":
                 (Index_ == CalculatorBase::LONGNUM) ? "lnum":"err";

      if (!NoPrompt_)
      {
        sprintf(buffer, "MCALC:%s:%s>> ", CalcStr_, ModeStr_);
        UseDefaultOutputFnc(buffer);
      }

      buffer = UseDefaultInputFnc(buffer, INPUT_BUFFER_SIZE);
    }
    while (buffer && CalcPtr_->EvalExpression(buffer, CalcPtr_));

    MemMatrix::Matrix().Deallocate(buffer);
    buffer = NULL;
  }
  else
  {
    _StdInput = FALSE;
    _StdOutput = FALSE;

    CalcPtr_ = CalcPtr_->GetCalcType();
    FileInput(argc, argv, CalcPtr_);
  }

  return 0;
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::EvaluateLine(int argc, CalculatorBase* CalcPtr_, Mcalc_Operand** operands, char** arglist)
{
  int optype;
  char* original;
  Mcalc_Queue* rpnqueue = NULL;
  Mcalc_Stack* rpnstack = NULL;
  int ConversionOk_ = InfixToRpn(argc,arglist,&rpnqueue,&rpnstack,&original, CalcPtr_);
  CalcPtr_ = CalcPtr_->GetCalcType();

  if (ConversionOk_)
  {
    if (!CalcPtr_->StringAssignment() && !CalcPtr_->SetAssignment() &&
        !SkipEval() && arglist[0])
      while (!HasErrors() && evaluateRPN(&rpnqueue,&rpnstack,arglist,&optype))
      {
        ProcessExpr(operands,arglist,optype, CalcPtr_);
        pushresult(operands,&rpnstack,optype);
        freeargs(&arglist);
      }

    RestoreNumberBase();
    CalcPtr_->StoreResults();
    DisplayToString(operands,original);
  }
  else
  {
    if (!InProgramMode())
      SetHasAnswer(FALSE);
  }

  SetSkipEval(FALSE);
  MemMatrix::Matrix().Deallocate(original);
  original = NULL;
  delstack(&rpnstack);
  delqueue(&rpnqueue);
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::FileInput(int argc, char** argv, CalculatorBase* CalcPtr_)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: FileInput\n", logfile);
  #endif

  Mcalc_Operand** operands;      /* operands array */
  char** arglist = NULL;         /* substitute argument list */
  int cmdline;                   /* command line argument flag */
  int IoState_;                  // I/O state for async. mode operation

  help(argc, argv);
  getfiles(&argc, argv, &cmdline);   /* get files for the program */
  initarrays(&operands, &arglist);   /* initialize variables */
  SetAsyncMode(UsesSessionNum());

  if (InAsyncMode())
  {
    for (;;)
    {
      ResetRetryRead();
      IoState_ = GetIoState(0);
      while (IoState_ != Mcalc_IOState::PROCESS && IoState_ != Mcalc_IOState::CALC_HALT)
        IoState_ = GetIoState(1);

      #if (CALCLIB_DEBUG1 | CALCLIB_DEBUG3)
        printf("opening %s for reading successful, server side\n", PROG_IOSTATE_FILE);
      #endif

      if (IoState_ == Mcalc_IOState::CALC_HALT)
      {
        #if CALCLIB_DEBUG2
          fputs("FileInput: Breaking I/O Loop\n", logfile);
        #endif

        SetIoState(Mcalc_IOState::IDLE_STATE);
        break;
      }

      if (IoState_ == Mcalc_IOState::PROCESS)
        if (ReadUserInput(&argc, &arglist))
        {
          ConcatArglist(argc, arglist, true);
          if (IsThisChar(arglist[0], 0, '#', 1))
            continue;

          ClearOutputFile();
          CalcPtr_ = CalcPtr_->GetCalcType();
          CalcPtr_->ClearError();

          if (!strlen(arglist[0]))
          {
            SetIoState(Mcalc_IOState::CALC_RESPONSE);
            continue;
          }

          _Expression = CalcPtr_->TopSubExprStack();
          _Expression->SetExpression(arglist[0]);
          _Expression->SetCalculator(CalcPtr_);
          _Expression->FindFunctionCalls();
          _Expression->Evaluate();

          if (_IoState == Mcalc_IOState::CALC_HALT ||
              CalcPtr_->ShouldQuit())
          {
            _Expression->Clear();
            SetIoState(Mcalc_IOState::IDLE_STATE);
            break;
          }

          if (!InProgramMode())
          {
            if (_Expression->HasAnswer() && !CalcPtr_->HasErrors())
              _Expression->ShowResults();
            else
            {
              if (CalcPtr_->HasErrors())
                CalcPtr_->SetIoState(Mcalc_IOState::CALC_ERROR);
              else
                CalcPtr_->SetIoState(Mcalc_IOState::CALC_RESPONSE);
            }
          }
          else
            SetHasCommandOutput(!CalcPtr_->HasErrors() && _Expression->HasCommandOutput());

          _Expression->Clear();
        }
    }
  }
  else
  {
    while (readfile(&argc, argv, &arglist, cmdline))
    {
      ConcatArglist(argc, arglist, true);
      if (IsThisChar(arglist[0], 0, '#', 1))
        continue;

      CalcPtr_ = CalcPtr_->GetCalcType();
      CalcPtr_->ClearError();

      if (!strlen(arglist[0]))
        continue;

      _Expression = CalcPtr_->TopSubExprStack();
      _Expression->SetExpression(arglist[0]);
      _Expression->SetCalculator(CalcPtr_);
      _Expression->FindFunctionCalls();
      _Expression->Evaluate();

      if (!InProgramMode())
      {
        if (_Expression->HasAnswer() && !CalcPtr_->HasErrors())
          _Expression->ShowResults();
      }
      else
        SetHasCommandOutput(!CalcPtr_->HasErrors() && _Expression->HasCommandOutput());

      _Expression->Clear();
    }
  }

  freearglist(&arglist, argc);
  freememory(&operands);
  closefiles();

  #if CALCLIB_DEBUG2
    fputs("Leaving: FileInput\n", logfile);
  #endif
}

/****************************************************************************/
template <class T>
void LineCalculator<T>::ProgramProc(int argc, char** argv, CalculatorBase* CalcPtr_)
{
  Mcalc_Operand** operands;      // operands array
  char** arglist = NULL;         // substitute argument list
  int i;

  initarrays(&operands,&arglist);               // initialize variables
  if (!_StreamSet)
    outfile = stdout;

  for (i = 0; i < argc; i++)
    arglist[i] = argv[i];

  for (;i < MAXARGS; i++)
    arglist[i] = NULL;

  CalcPtr_ = CalcPtr_->GetCalcType();
  EvaluateLine(argc, CalcPtr_, operands, arglist);
  freememory(&operands);
  freelist(&arglist);
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::EvalExpression(char* argstr, CalculatorBase* CalcPtr_)
{
  char* temps_ = NULL;
  temps_ = (char*)allocmem(strlen(argstr) + 1, sizeof(char), temps_);
  char* start_ = temps_;

  strcpy(temps_, argstr);
  RemovePadding(temps_, " \t\n");

  if (!strlen(temps_))
  {
    MemMatrix::Matrix().Deallocate(start_);
    return 1;
  }
  else if (icasestrcomp(temps_, "quit"))
  {
    MemMatrix::Matrix().Deallocate(start_);
    return 0;
  }

  strcpy(start_, argstr);

  if (argstr && strlen(argstr))
  {
    if (!_StreamSet)
      outfile = stdout;

    SetPlotGraph(false);
    ClearOutputFile();
    CalcPtr_ = CalcPtr_->GetCalcType();
    CalcPtr_->ClearError();

    _Expression = CalcPtr_->TopSubExprStack();
    _Expression->SetExpression(start_);
    _Expression->SetCalculator(CalcPtr_);
    _Expression->FindFunctionCalls();

    if (_IoState == Mcalc_IOState::CALC_HALT ||
         CalcPtr_->ShouldQuit())
      CalcPtr_->SetShouldQuit(1);
    else
      _Expression->Evaluate();

    if (_IoState == Mcalc_IOState::CALC_HALT ||
         CalcPtr_->ShouldQuit())
      CalcPtr_->SetShouldQuit(1);

    if (!InProgramMode())
    {
      if (_Expression->HasAnswer() && !CalcPtr_->HasErrors())
        _Expression->ShowResults();
    }
    else
      SetHasCommandOutput(!CalcPtr_->HasErrors() && _Expression->HasCommandOutput());

    _Expression->Clear();
  }

  MemMatrix::Matrix().Deallocate(start_);
  return (!CalcPtr_->ShouldQuit());
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::TransferData(char* argstr, CalculatorBase* CalcPtr_)
{
  char* argvector[2];
  char* temps_ = NULL;
  temps_ = (char*)allocmem(strlen(argstr) + 1, sizeof(char), temps_);
  char* start_ = temps_;

  strcpy(temps_, argstr);
  RemovePadding(temps_, " \t\n");

  if (!strlen(temps_))
  {
    MemMatrix::Matrix().Deallocate(start_);
    SetHasAnswer(FALSE);
    return 1;
  }
  else if (icasestrcomp(temps_, "quit"))
  {
    MemMatrix::Matrix().Deallocate(start_);
    SetHasAnswer(FALSE);
    return 0;
  }

  argvector[0] = NULL;
  argvector[1] = start_;
  strcpy(argvector[1], argstr);

  if (argstr && strlen(argstr))
    ProgramProc(2, argvector, CalcPtr_);
  else
    SetHasAnswer(FALSE);

  MemMatrix::Matrix().Deallocate(start_);
  return 1;
}

/****************************************************************************/
template <class T>
int LineCalculator<T>::StoreResults()
{
  int x;
  int AssignType_;
  int Found_ = HasIndex();
  ChrString TempStr_;
  int Errors_ = HasErrors();

  while (HasIndex())
  {
    GetIndex(x, AssignType_);
    Boolean SetOps_ = AssignType_ == SET_ASSIGN ||
                      AssignType_ == SET_UNION ||
                      AssignType_ == SET_INTERSECTION ||
                      AssignType_ == SET_NEGATION ||
                      AssignType_ == SET_EXCLUSION;

    if (AssignType_ && !Errors_)
    {
      if (AssignType_ == STRING_ASSIGN || SetOps_)
      {
        int Mode_ = AnswerMode();
        if (Mode_ == LITERAL_MODE || Mode_ == ALTBASE_MODE)
        {
          TempStr_ = LiteralStr();

          if (AssignType_ == STRING_ASSIGN)
          {
            SetStringAssignment(TRUE, TempStr_.c_str());
            SetStrVariable(x, TempStr_);
          }
          else if (SetOps_)
          {
            int IsNull_ = IsNullSetStr(TempStr_.c_str());
            SetSetAssignment(TRUE, TempStr_.c_str(), IsNull_);
            SetSetVariable(x, TempStr_, IsNull_);
          }

          if (Mode_ == ALTBASE_MODE && StringAssignment())
          {
            SetVariable(x, GetResult());
            TransferVariable(x, x);
          }
        }
        else if (Mode_ == FRACTION_MODE)
        {
          TempStr_ = FractionStr();
          SetStringAssignment(TRUE, TempStr_.c_str());
          SetStrVariable(x, TempStr_);
        }
        else
        {
          if (SetOps_)
          {
            TempStr_ = GetSetResult();
            int IsNull_ = IsNullSetStr(TempStr_.c_str());
            SetSetVariable(x, TempStr_, IsNull_);
          }
          else
            SetStrVariable(x, GetStrResult());
        }
      }
      else if (AssignType_ == STRING_CONCAT)
      {
        int Mode_ = AnswerMode();
        if (Mode_ == LITERAL_MODE || Mode_ == ALTBASE_MODE)
        {
          CopyStrVar(TempStr_, x);
          TempStr_ += LiteralStr();
          SetStringAssignment(TRUE, TempStr_.c_str());
          SetStrVariable(x, TempStr_);
        }
        else if (Mode_ == FRACTION_MODE)
        {
          CopyStrVar(TempStr_, x);
          TempStr_ += FractionStr();
          SetStringAssignment(TRUE, TempStr_.c_str());
          SetStrVariable(x, TempStr_);
        }
        else
        {
          CopyStrVar(TempStr_, x);
          TempStr_ += GetStrResult();
          SetStrVariable(x, TempStr_);
        }
      }
      else
      {
        SetVariable(x, GetResult());
        TransferVariable(x, x);
      }
    }

    PopIndex();
  }

  if (StringAssignment())
    SetStringAssignment(FALSE);

  if (SetAssignment())
    SetSetAssignment(FALSE, NULL, FALSE);

  ResetIndexes();
  SetShowAnswer((!Found_ || (_GraphingOp && InAsyncMode())) && !Errors_);

  if (!Found_ && !Errors_)
    TransferVariable(-1, -1);

  return Found_;
}

/****************************************************************************/
#endif




