#ifndef CALCLIB_CPP
#define CALCLIB_CPP
#ifndef CALCLIB_H
  #include "calclib.h"
#endif

// function names
char** CalculatorBase::FUNCTION_NAMES = NULL;
int CalculatorBase::_NumFncNames = 0;
int CalculatorBase::_ShortestFncName = 0;

// Class instances counter
int CalculatorBase::_Instances = 0;

/* Program Constants */
const int CalculatorBase::MAXVALUES = MAXOPERANDS+2;      /* Maximum number of integer values including the result of the calculation */
const int CalculatorBase::RESULTINDEX = MAXOPERANDS;      /* index position in the operands array which contains the structure for storing the result of the calculation */

/* Program Options */
const int CalculatorBase::SHOWPOS = 0;       /* flag to show positive sign if value is positive */

/* Operator symbols */
const Uchar CalculatorBase::POW_OP         = '^';
const Uchar CalculatorBase::SIGN_OP        = 174;
const Uchar CalculatorBase::SIN_OP         = 175;
const Uchar CalculatorBase::COS_OP         = 176;
const Uchar CalculatorBase::TAN_OP         = 177;
const Uchar CalculatorBase::ASIN_OP        = 178;
const Uchar CalculatorBase::ACOS_OP        = 179;
const Uchar CalculatorBase::ATAN_OP        = 180;
const Uchar CalculatorBase::SINH_OP        = 181;
const Uchar CalculatorBase::COSH_OP        = 182;
const Uchar CalculatorBase::TANH_OP        = 183;
const Uchar CalculatorBase::EXP_OP         = 184;
const Uchar CalculatorBase::LOG_OP         = 185;
const Uchar CalculatorBase::LOG10_OP       = 186;
const Uchar CalculatorBase::CEILING_OP     = 187;
const Uchar CalculatorBase::FLOOR_OP       = 188;
const Uchar CalculatorBase::ABS_OP         = 189;
const Uchar CalculatorBase::FRACPART_OP    = 190;
const Uchar CalculatorBase::INTPART_OP     = 191;
const Uchar CalculatorBase::SQR_OP         = 192;
const Uchar CalculatorBase::SQRT_OP        = 193;
const Uchar CalculatorBase::CUBE_OP        = 194;
const Uchar CalculatorBase::CUBERT_OP      = 195;
const Uchar CalculatorBase::MINUS_OP       = 196;
const Uchar CalculatorBase::AND_OP         = 197;
const Uchar CalculatorBase::RND_OP         = 198;
const Uchar CalculatorBase::NOT_OP         = 199;
const Uchar CalculatorBase::COMB_OP        = 200;
const Uchar CalculatorBase::PERM_OP        = 201;
const Uchar CalculatorBase::COMBREP_OP     = 202;
const Uchar CalculatorBase::LSTHANOREQ_OP  = 205;
const Uchar CalculatorBase::GRTHANOREQ_OP  = 206;
const Uchar CalculatorBase::EQUAL_OP       = 207;
const Uchar CalculatorBase::NOTEQUAL_OP    = 208;
const Uchar CalculatorBase::RAND_OP        = 209;
const Uchar CalculatorBase::XOR_OP         = 210;
const Uchar CalculatorBase::INV_OP         = 211;
const Uchar CalculatorBase::SCI_OP         = 212;
const Uchar CalculatorBase::OR_OP          = 213;
const Uchar CalculatorBase::FACTORIAL_OP   = 214;
const Uchar CalculatorBase::PERMREP_OP     = 215;

/* Predefined constant symbols */
const Uchar CalculatorBase::PI_CONST       = 220;
const Uchar CalculatorBase::E_CONST        = 221;
const Uchar CalculatorBase::ANS_CONST      = 222;
const Uchar CalculatorBase::ANSSTR_CONST   = 223;
const Uchar CalculatorBase::FRACANS_CONST  = 224;
const Uchar CalculatorBase::NAN_CONST      = 225;
const Uchar CalculatorBase::INF_CONST      = 226;

// Indexes for variables, numbers, prefixes and function calls
const Uchar CalculatorBase::VARIABLE_INDEX  = 230;
const Uchar CalculatorBase::NUMBER_INDEX    = 231;
const Uchar CalculatorBase::HEXPREFIX_INDEX = 232;
const Uchar CalculatorBase::FUNCTION_INDEX  = 233;
const Uchar CalculatorBase::BINARYOPS_INDEX = 234;
const Uchar CalculatorBase::UNARYOPS_INDEX  = 235;
const Uchar CalculatorBase::POSTOPS_INDEX   = 236;

// File pointers
FILE** CalculatorBase::files = NULL;
FILE* CalculatorBase::outfile = NULL;
FILE* CalculatorBase::logfile = NULL;
FILE* CalculatorBase::userinputfile = NULL;
FILE* CalculatorBase::userpromptfile = NULL;
FILE* CalculatorBase::programoutputfile = NULL;
FILE* CalculatorBase::iostatefile = NULL;
FILE* CalculatorBase::curgraphfile = NULL;

// Calculator program termination
int CalculatorBase::_ShouldQuit = 0;

// Graph plotting
int CalculatorBase::_PlotGraph = 0;
int CalculatorBase::_GraphingOp = 0;
int CalculatorBase::_GraphType = 0;
int CalculatorBase::_PlottingGraph = 0;
int CalculatorBase::_PlotAllPoints = 0;

// File operation flags
int CalculatorBase::_StreamSet = 0;
int CalculatorBase::_ArgListSize = 0;

// Asynchronous mode flags
int CalculatorBase::filestatus[];
int CalculatorBase::_UseSessionCnt = 0;
int CalculatorBase::_SessionNum = 0;
int CalculatorBase::_AsyncMode = 0;
int CalculatorBase::_IoState = Mcalc_IOState::IDLE_STATE;
int CalculatorBase::_WaitForResponse = 0;
int CalculatorBase::_IostateFileReadError = 0;
int CalculatorBase::_RetryReadCount = 0;
int CalculatorBase::_KeepAlivePollValue = 0;
int CalculatorBase::_RetryPollCount = 0;
int CalculatorBase::_KeepAlivePollError = 0;
int CalculatorBase::_StdInput = 0;
int CalculatorBase::_StdOutput = 0;

// Convergence test upper limit
double CalculatorBase::UPPER_LIMIT = 1.0e+20;

// File names
char* CalculatorBase::LOG_FILE = NULL;
char* CalculatorBase::USER_INPUT_FILE = NULL;
char* CalculatorBase::USER_PROMPT_FILE = NULL;
char* CalculatorBase::PROG_OUTPUT_FILE = NULL;
char* CalculatorBase::DATA_INPUT_FILE = NULL;
char* CalculatorBase::DATA_OUTPUT_FILE = NULL;
char* CalculatorBase::PROG_IOSTATE_FILE = NULL;
char* CalculatorBase::CURRENTGRAPH_FILE = NULL;
char* CalculatorBase::SERVERALIVE_FILE = NULL;
char* CalculatorBase::CLIENTALIVE_FILE = NULL;

// Pointer to default input/output functions
int(*CalculatorBase::_OutFnc)(const char*) = NULL;
char*(*CalculatorBase::_InFnc)(char*, int) = NULL;

// Odd root special case
Uchar* CalculatorBase::_QLookAhead = NULL;
int CalculatorBase::_OddRoot = 0;

// Plotting data file letter suffix
char CalculatorBase::_PlotFileSuffix = 'a';

// Error messages
char** CalculatorBase::_ErrorMessageArray = NULL;
int* CalculatorBase::_ErrorMessageArgCnt = NULL;
char* CalculatorBase::_CurrentFunctionName = NULL;

/****************************************************************************/
/****************************************************************************/
int operator == (SimpleList<int>& x, const char* s)
{
  int i = 0;
  int y = 0;
  const char* ptr = s;

  SimpleListIterator<int> Iter_(&x);
  if (s)
    for (ptr = s; *ptr && !isdigit(*ptr); ptr++);

  for (i = y = 0; y < x.Size(); ++i, ++y)
  {
    while (ptr[i] && (isspace(ptr[i]) || ptr[i] == ','))
      ++i;

    if (ptr[i] == '}')
      break;

    if (*Iter_.IndexNode(y)->_Object != atoi(ptr+i))
      return 0;
  }

  return (y == x.Size());
}

/****************************************************************************/
int operator == (const char* s, SimpleList<int>& x)
{
  int i = 0;
  int y = 0;
  const char* ptr = s;

  SimpleListIterator<int> Iter_(&x);
  if (s)
    for (ptr = s; *ptr && !isdigit(*ptr); ptr++);

  for (i = y = 0; y < x.Size(); ++i, ++y)
  {
    while (ptr[i] && (isspace(ptr[i]) || ptr[i] == ','))
      ++i;

    if (ptr[i] == '}')
      break;

    if (*Iter_.IndexNode(y)->_Object != atoi(ptr+i))
      return 0;
  }

  return (y == x.Size());
}

/****************************************************************************/
/****************************************************************************/
char* Mcalc_DefaultInFnc(char* Buffer_, int Size_)
{
  ChrString TempStr_;
  char* Retstr_ = NULL;
  int Index_ = 0;

  do
  {
    do Retstr_ = fgets(&Buffer_[Index_], Size_, stdin);
    while (Retstr_ && CalculatorBase::IsThisChar(Retstr_, 0, '#', 1));

    TempStr_ = Buffer_;
    TempStr_.RemovePadding(" \t\n");

    if (TempStr_.strlen() && TempStr_[TempStr_.strlen() - 1] == '\\')
    {
      TempStr_[TempStr_.strlen() - 1] = '\n';
      strcpy(Buffer_, TempStr_.c_str());
      Index_ = TempStr_.strlen();
    }
    else
    {
      Retstr_ = Buffer_;
      Index_ = 0;
    }
  }
  while (Index_ != 0);

  return Retstr_;
}

/****************************************************************************/
int Mcalc_DefaultOutFnc(const char* Buffer_)
{
  return fputs(Buffer_, stdout);
}

/****************************************************************************/
// Set element equality function
//   returns 0 if ival1 == ival2
//   returns !0 if ival1 != ival2
//
int Mcalc_DefaultSetElementEqualFnc(const int& ival1, const int& ival2)
{
  return (ival1 - ival2);
}

/****************************************************************************/
AssignmentParsingData::AssignmentParsingData():
_FirstAlpha(FALSE),
_LastDollar(FALSE),
_AnsStrConstFound(FALSE),
_FracAnsConstFound(FALSE),
_VarLetter(0),
_FirstQuotePos(0),
_LastQuotePos(0)
{}

/****************************************************************************/
/****************************************************************************/
StrVarData::StrVarData():
_SetResult(0)
{
  int i;
  for (i = 0; i < CalculatorBase::MAXVARS; i++)
  {
    _IsStringVar[i] = 0;
    _IsSetVar[i] = 0;
    _strvars[i] = NULL;
  }
}

/****************************************************************************/
StrVarData::~StrVarData()
{
  int i;
  for (i = 0; i < CalculatorBase::MAXVARS; i++)
  {
    if (_strvars[i])
      delete _strvars[i];

    _strvars[i] = NULL;
  }
}

/****************************************************************************/
void StrVarData::SetSetElements(int Index_, const ChrString& Str_, int Null_)
{
  if (Index_ >= 0)
  {
    if (!_strvars[Index_])
      _strvars[Index_] = new ChrString();

    *(_strvars[Index_]) = Str_;
    _IsSetVar[Index_] = Null_ ? 2:1;
  }
}

/****************************************************************************/
void StrVarData::SetString(int Index_, const ChrString& Str_)
{
  if (Index_ >= 0)
  {
    if (!_strvars[Index_])
      _strvars[Index_] = new ChrString();

    *(_strvars[Index_]) = Str_;
    _IsSetVar[Index_] = 0;
  }
}

/****************************************************************************/
const ChrString& StrVarData::GetVariable(int Index_) const
{
  static ChrString Dummy_;
  return ((Index_ >= 0 && _strvars[Index_]) ? *(_strvars[Index_]):Dummy_);
}

/****************************************************************************/
const ChrString& StrVarData::GetSetVariable(int Index_) const
{
  static ChrString Dummy_;
  static ChrString NullStr_("null");

  return
  (
    (Index_ >= 0 && _strvars[Index_]) ?
        ((_IsSetVar[Index_] == 1) ? *(_strvars[Index_]):NullStr_):
        Dummy_
  );
}

/****************************************************************************/
const ChrString& StrVarData::GetResult() const
{
  return _StrResult;
}

/****************************************************************************/
const ChrString& StrVarData::GetSetResult() const
{
  static ChrString NullStr_("null");
  return ((_SetResult == 1) ? _StrResult:NullStr_);
}

/****************************************************************************/
void StrVarData::SetResult(const char* Str_)
{
  if (Str_)
    _StrResult = Str_;
}

/****************************************************************************/
void StrVarData::SetSetResult(const char* Str_, bool Null_)
{
  if (Str_)
  {
    _StrResult = Str_;
    _SetResult = Null_ ? 2:1;
  }
}

/****************************************************************************/
size_t StrVarData::GetVarLen(int Index_) const
{
  return ((Index_ >= 0 && _strvars[Index_]) ? _strvars[Index_]->strlen():0);
}

/****************************************************************************/
void StrVarData::SetStringVar(int Index_, Boolean Flag_)
{
  if (Index_ >= 0)
  {
    _IsStringVar[Index_] = Flag_;
    _IsSetVar[Index_] = 0;
  }
}

/****************************************************************************/
Boolean StrVarData::IsStringVar(int Index_) const
{
  return ((Index_ >= 0) ? _IsStringVar[Index_]:FALSE);
}

/****************************************************************************/
void StrVarData::SetSetVar(int Index_, int Flag_)
{
  if (Index_ >= 0)
  {
    _IsStringVar[Index_] = Flag_;
    _IsSetVar[Index_] = Flag_;
  }
}

/****************************************************************************/
int StrVarData::IsSetVar(int Index_) const
{
  return ((Index_ >= 0) ? _IsSetVar[Index_]:FALSE);
}

/****************************************************************************/
int StrVarData::IsNullSet(int Index_) const
{
  return ((Index_ >= 0) ? (_IsSetVar[Index_] == 2):FALSE);
}

/****************************************************************************/
ChrString& StrVarData::CopyStrVar(ChrString& DestStr_, int Index_)
{
  if (Index_ >= 0 && _strvars[Index_] && _strvars[Index_]->strlen())
    DestStr_ = *(_strvars[Index_]);

  return DestStr_;
}

/****************************************************************************/
char* StrVarData::CopyStrVar(char* DestStr_, int Index_)
{
  if (Index_ >= 0 && _strvars[Index_] && _strvars[Index_]->strlen())
    strcpy(DestStr_, _strvars[Index_]->c_str());

  return DestStr_;
}

/****************************************************************************/
/****************************************************************************/
SubExpression::SubExpression():
_CalcPtr(NULL),
_Parent(NULL),
_Level(0),
_FunctionNames(CalculatorBase::FUNCTION_NAMES),
_Expression(NULL),
_AnswerStr(NULL),
_OriginalStr(NULL),
_FncCalls(NULL),
_NumFncNames(CalculatorBase::NumFncNames()),
_NumFncCalls(0),
_ArrayLimit(0),
_ShortestName(CalculatorBase::ShortestFncName()),
_HasAnswer(FALSE),
_SingleFncCall(FALSE),
_ShowFunction(FALSE),
_ShowSetFunction(FALSE)
{
  _ArrayLimit = 10;
  _FncCalls = (FunctionCall**)MemMatrix::Matrix().Callocate(_ArrayLimit * sizeof(FunctionCall*));

  int x;
  for (x = 0; x < _ArrayLimit; x++)
    _FncCalls[x] = NULL;
}

/****************************************************************************/
SubExpression::SubExpression(SubExpression* Parent_, int Level_, char* Expr_):
_CalcPtr(NULL),
_Parent(Parent_),
_Level(Level_),
_FunctionNames(CalculatorBase::FUNCTION_NAMES),
_Expression(NULL),
_AnswerStr(NULL),
_OriginalStr(NULL),
_FncCalls(NULL),
_NumFncNames(CalculatorBase::NumFncNames()),
_NumFncCalls(0),
_ArrayLimit(0),
_ShortestName(CalculatorBase::ShortestFncName()),
_HasAnswer(FALSE),
_SingleFncCall(FALSE),
_ShowFunction(FALSE),
_ShowSetFunction(FALSE)
{
  _Expression = (char*)MemMatrix::Matrix().Callocate(strlen(Expr_) + 1);
  ::SafeStrCpy(_Expression, Expr_);

  if (!_Parent)
  {
    ChrString ExprStr_ = Expr_;
    ExprStr_.RemovePadding(" \n\t");
    _OriginalStr = (char*)MemMatrix::Matrix().Callocate(strlen(ExprStr_.c_str()) + 1);
    ::SafeStrCpy(_OriginalStr, ExprStr_.c_str());
  }

  _ArrayLimit = 10;
  _FncCalls = (FunctionCall**)MemMatrix::Matrix().Callocate(_ArrayLimit * sizeof(FunctionCall*));

  int x;
  for (x = 0; x < _ArrayLimit; x++)
    _FncCalls[x] = NULL;
}

/****************************************************************************/
SubExpression::SubExpression(const SubExpression& Obj_):
_CalcPtr(Obj_._CalcPtr),
_Parent(Obj_._Parent),
_Level(Obj_._Level),
_FunctionNames(CalculatorBase::FUNCTION_NAMES),
_Expression(NULL),
_AnswerStr(NULL),
_OriginalStr(NULL),
_FncCalls(NULL),
_NumFncNames(CalculatorBase::NumFncNames()),
_NumFncCalls(0),
_ArrayLimit(0),
_ShortestName(CalculatorBase::ShortestFncName()),
_HasAnswer(FALSE),
_SingleFncCall(FALSE),
_ShowFunction(FALSE),
_ShowSetFunction(FALSE)
{
  _ArrayLimit = 10;
  _FncCalls = (FunctionCall**)MemMatrix::Matrix().Callocate(_ArrayLimit * sizeof(FunctionCall*));

  SetExpression(Obj_._Expression);
  SetAnswer(Obj_._AnswerStr);

  int x;
  for (x = 0; x < _ArrayLimit; x++)
    _FncCalls[x] = NULL;
}

/****************************************************************************/
SubExpression::~SubExpression()
{
  if (_CalcPtr)
  {
    if (_CalcPtr->SubExprStackSize() <= 1)
    {
      _CalcPtr->ResetAnswerMode();
      _CalcPtr->RestoreCalcType();
    }
  }

  MemMatrix::Matrix().Deallocate(_Expression);
  MemMatrix::Matrix().Deallocate(_AnswerStr);
  MemMatrix::Matrix().Deallocate(_OriginalStr);
  _Expression = _AnswerStr = _OriginalStr = NULL;

  int x;
  for (x = 0; x < _NumFncCalls; x++)
  {
    delete _FncCalls[x];
    _FncCalls[x] = NULL;
  }

  MemMatrix::Matrix().Deallocate(_FncCalls);
  _FncCalls = NULL;
  _FunctionNames = NULL;
}

/****************************************************************************/
void SubExpression::Clear()
{
  if (_CalcPtr)
  {
    if (_CalcPtr->SubExprStackSize() <= 1)
    {
      _CalcPtr->ResetAnswerMode();
      _CalcPtr->RestoreCalcType();
    }
  }

  MemMatrix::Matrix().Deallocate(_Expression);
  MemMatrix::Matrix().Deallocate(_AnswerStr);
  MemMatrix::Matrix().Deallocate(_OriginalStr);
  _Expression = _AnswerStr = _OriginalStr = NULL;

  int x;
  for (x = 0; x < _NumFncCalls; x++)
  {
    delete _FncCalls[x];
    _FncCalls[x] = NULL;
  }

  _NumFncCalls = 0;
}

/****************************************************************************/
void SubExpression::ResizeArray()
{
  int x;
  int OldLim_ = _ArrayLimit;
  FunctionCall** OldArray_ = _FncCalls;

  _ArrayLimit += 10;
  _FncCalls = (FunctionCall**)MemMatrix::Matrix().Callocate(_ArrayLimit * sizeof(FunctionCall*));

  for (x = 0; x < OldLim_; x++)
    _FncCalls[x] = OldArray_[x];

  for (;x < _ArrayLimit; x++)
    _FncCalls[x] = NULL;

  MemMatrix::Matrix().Deallocate(OldArray_);
  OldArray_ = NULL;
}

/****************************************************************************/
void SubExpression::SetCalculator(CalculatorBase* CalcPtr_)
{
  _CalcPtr = CalcPtr_;
  if (_CalcPtr)
    _CalcPtr->SetDisplayOption(CalculatorBase::DISPLAY_TO_STRING);
}

/****************************************************************************/
CalculatorBase* SubExpression::GetCalculator()
{
  return _CalcPtr;
}

/****************************************************************************/
void SubExpression::SetOriginal(const char* Expr_)
{
  ChrString ExprStr_ = Expr_;
  ExprStr_.RemovePadding(" \n\t");
  _OriginalStr =  AllocateNewString(_OriginalStr, ::SafeStrLen(Expr_));
  ::SafeStrCpy(_OriginalStr, ExprStr_.c_str());
}

/****************************************************************************/
void SubExpression::SetAnswer(const char* Ans_)
{
  ChrString AnsStr_ = Ans_;
  AnsStr_.RemovePadding(" \n\t");
  _AnswerStr =  AllocateNewString(_AnswerStr, ::SafeStrLen(Ans_));
  ::SafeStrCpy(_AnswerStr, AnsStr_.c_str());
}

/****************************************************************************/
void SubExpression::SetExpression(const char* Expr_)
{
  ChrString ExprStr_ = Expr_;
  ExprStr_.RemovePadding(" \n\t");
  _Expression =  AllocateNewString(_Expression, ::SafeStrLen(Expr_));
  ::SafeStrCpy(_Expression, ExprStr_.c_str());

  if (!_Parent)
  {
    _OriginalStr =  AllocateNewString(_OriginalStr, ::SafeStrLen(Expr_));
    ::SafeStrCpy(_OriginalStr, ExprStr_.c_str());
  }
}

/****************************************************************************/
char* SubExpression::AllocateNewString(char* Buffer_, size_t NewLen_)
{
  if (!Buffer_ || strlen(Buffer_) < NewLen_)
  {
    MemMatrix::Matrix().Deallocate(Buffer_);
    Buffer_ = (char*)MemMatrix::Matrix().Callocate(NewLen_ + 1);
  }
  return Buffer_;
}

/****************************************************************************/
void SubExpression::AddFunctionCall(char* FncCall_, int* DelimArray_)
{
  size_t len = DelimArray_[DelimArray_[0]] + 1;
  char* ExprStart_ = NULL;
  char* OldExpr_ = NULL;
  char* Buffer_ = NULL;
  int EvalFnc_ = 0;
  int ShowFnc_ = 0;
  int IntFnc0_ = 0;
  int IntFnc2_ = 0;
  int SingleArgFncs_ = 0;
  int QuadIntFnc_ = 0;
  int PentIntFnc_ = 0;
  int ShowSetFnc_ = 0;
  int x;

  int NumParams_ = DelimArray_[0] - 1;
  Buffer_ = (char*)MemMatrix::Matrix().Callocate(len + 1);
  strncpy(Buffer_, FncCall_, len);
  Buffer_[len] = 0;
  SingleArgFncs_ = (NumParams_ == 1);
  QuadIntFnc_ = (NumParams_ == 4) || (NumParams_ == 2);
  PentIntFnc_ = (NumParams_ == 5) || (NumParams_ == 3);
  EvalFnc_ = _CalcPtr->icasestrcompn(Buffer_, "EVAL", 4) && SingleArgFncs_;
  ShowFnc_ = _CalcPtr->icasestrcompn(Buffer_, "SHOW", 4) && SingleArgFncs_;
  ShowSetFnc_ = _CalcPtr->icasestrcompn(Buffer_, "SHOWSET", 7) && SingleArgFncs_;
  IntFnc0_ = _CalcPtr->icasestrcompn(Buffer_, "INTERSECT", 9) && QuadIntFnc_;
  IntFnc2_ = _CalcPtr->icasestrcompn(Buffer_, "INTERSECT", 9) && PentIntFnc_;

  ExprStart_ = strstr(_Expression, Buffer_);
  ChrString NumStr_;
  NumStr_.IntToStr(_NumFncCalls);
  ChrString Token_("#");
  Token_ += NumStr_;
  Token_ += ChrString("s");
  ChrString SavedToken_ = Token_;
  len = strlen(_Expression);
  size_t newlen;
  newlen = len - strlen(Buffer_) + Token_.strlen();
  OldExpr_ = _Expression;
  *ExprStart_ = 0;

  if (len < newlen)
  {
    len = newlen;
    _Expression = (char*)MemMatrix::Matrix().Callocate(len + 1);
    strcpy(_Expression, OldExpr_);
  }
  else
    OldExpr_ = NULL;

  strcat(_Expression, Token_.c_str());
  Token_ = &ExprStart_[strlen(Buffer_)];
  Token_.RemovePadding(" \t\n");

  if (Token_.strlen())
    strcat(_Expression, Token_.c_str());

  // Setting original string to have a substitution tag since
  // evaluation would mean the display string will be different
  if (EvalFnc_ && !_Parent)
  {
    MemMatrix::Matrix().Deallocate(OldExpr_);
    OldExpr_ = _OriginalStr;
    len = strlen(_OriginalStr);
    ExprStart_ = strstr(_OriginalStr, Buffer_);

    Token_ = SavedToken_;
    SavedToken_ = Buffer_;
    char* ExprPtr_ = ExprStart_ + DelimArray_[DelimArray_[0]];
    *ExprPtr_ = 0;
    ExprPtr_ = ExprStart_ + DelimArray_[1] + 1;
    strcpy(Buffer_, ExprPtr_);
    newlen = len - strlen(Buffer_) + Token_.strlen();
    ExprPtr_ = ExprStart_ + DelimArray_[1] + 1;
    *ExprPtr_ = 0;
    strcpy(Buffer_, SavedToken_.c_str());

    if (len < newlen)
    {
      len = newlen;
      _OriginalStr = (char*)MemMatrix::Matrix().Callocate(len + 1);
      strcpy(_OriginalStr, OldExpr_);
    }
    else
      OldExpr_ = NULL;

    strcat(_OriginalStr, Token_.c_str());
    ExprPtr_ = ExprStart_ + DelimArray_[DelimArray_[0]];
    *ExprPtr_ = ')';
    Token_ = ExprPtr_;
    Token_.RemovePadding(" \t\n");

    if (Token_.strlen())
      strcat(_OriginalStr, Token_.c_str());
  }

  for (x = 1; x <= DelimArray_[0]; x++)
    Buffer_[DelimArray_[x]] = 0;

  if (_NumFncCalls == _ArrayLimit)
    ResizeArray();

  int ItemFncs_ = 0;
  int CalcFncs_ = 0;
  int FracFnc_ = 0;
  int MakeSetFnc_ = 0;
  int SetOpFncs_ = 0;
  int FuncExtrFncs_ = 0;
  int EndEval_ = 0;
  int OptionFncs_ = 0;
  int MatSolveFnc_ = 0;
  int GraphFnc_ = 0;
  AssignmentParsingData ParseData_;

  GraphFnc_ = _CalcPtr->icasestrcompn(Buffer_, "GRAPH", 5);
  ItemFncs_ =
      _CalcPtr->icasestrcompn(Buffer_, "ITEMPERM", 8)         ||
      _CalcPtr->icasestrcompn(Buffer_, "ITEMCOMB", 8)         ||
      _CalcPtr->icasestrcompn(Buffer_, "ITEMPERMREP", 11)     ||
      _CalcPtr->icasestrcompn(Buffer_, "ITEMCOMBREP", 11);
  FuncExtrFncs_ =
      _CalcPtr->icasestrcompn(Buffer_, "FMIN", 4)       ||
      _CalcPtr->icasestrcompn(Buffer_, "FMAX", 4);
  CalcFncs_ =
      _CalcPtr->icasestrcompn(Buffer_, "FMIN", 4)         ||
      _CalcPtr->icasestrcompn(Buffer_, "FMAX", 4)         ||
      _CalcPtr->icasestrcompn(Buffer_, "DERIV", 5)        ||
      _CalcPtr->icasestrcompn(Buffer_, "INTEG", 5)        ||
      _CalcPtr->icasestrcompn(Buffer_, "SOLVE", 5)        ||
      _CalcPtr->icasestrcompn(Buffer_, "DERIV2", 6);
  SetOpFncs_ = _CalcPtr->icasestrcompn(Buffer_, "SETUNI", 6) ||
               _CalcPtr->icasestrcompn(Buffer_, "SETINT", 6) ||
               _CalcPtr->icasestrcompn(Buffer_, "SETDIF", 6) ||
               _CalcPtr->icasestrcompn(Buffer_, "SETXDIF", 7);
  MakeSetFnc_ = _CalcPtr->icasestrcompn(Buffer_, "SET", 3) && !SetOpFncs_;
  FracFnc_ = _CalcPtr->icasestrcompn(Buffer_, "FRAC", 4);
  MatSolveFnc_ = _CalcPtr->icasestrcompn(Buffer_, "MATSOLVE", 8);
  OptionFncs_ = SingleArgFncs_ &&
                   (_CalcPtr->icasestrcompn(Buffer_, "AVGMEAN", 7)        ||
                    _CalcPtr->icasestrcompn(Buffer_, "AVGMED", 6)         ||
                    _CalcPtr->icasestrcompn(Buffer_, "AVGMODE", 7)        ||
                    _CalcPtr->icasestrcompn(Buffer_, "SUM", 3)            ||
                    _CalcPtr->icasestrcompn(Buffer_, "PROD", 4)           ||
                    _CalcPtr->icasestrcompn(Buffer_, "STDDEV", 4)         ||
                    _CalcPtr->icasestrcompn(Buffer_, "MIN", 3)            ||
                    _CalcPtr->icasestrcompn(Buffer_, "MAX", 3)            ||
                    (_CalcPtr->icasestrcompn(Buffer_, "SET", 3) &&
                     !SetOpFncs_)                                         ||
                    _CalcPtr->icasestrcompn(Buffer_, "FRAC", 4)           ||
                    _CalcPtr->icasestrcompn(Buffer_, "ITEMPERM", 8)       ||
                    _CalcPtr->icasestrcompn(Buffer_, "ITEMCOMB", 8)       ||
                    _CalcPtr->icasestrcompn(Buffer_, "ITEMPERMREP", 11)   ||
                    _CalcPtr->icasestrcompn(Buffer_, "ITEMCOMBREP", 11));

  _FncCalls[_NumFncCalls] = new FunctionCall(this, Buffer_);

  if (ItemFncs_)
    _FncCalls[_NumFncCalls]->SetNumParams(NumParams_ + 1);
  else
    _FncCalls[_NumFncCalls]->SetNumParams(NumParams_);

  for (x = 1; x < DelimArray_[0]; x++)
  {
    EndEval_ =
    (
      (x < 3 && IntFnc0_) ||
      (x < 3 && CalcFncs_) ||
      (x < 3 && SetOpFncs_) ||
      (x < 3 && GraphFnc_) ||
      (x < 4 && IntFnc2_) ||
      (x == 1 && FracFnc_) ||
      (x == 5 && FuncExtrFncs_) ||
      (x != 1 && ItemFncs_) ||
      (x != 1 && MakeSetFnc_) ||
      EvalFnc_ || ShowFnc_ || ShowSetFnc_ ||
      OptionFncs_ || MatSolveFnc_
    );

    _FncCalls[_NumFncCalls]->AddParameter(&Buffer_[DelimArray_[x]+1], !EndEval_, EndEval_);
  }

  if (ItemFncs_)
  {
    len = strlen(_Expression);
    int StrAssign_, StrLength_, Error_;
    int EqFound_ = _CalcPtr->FindAssignments(_Expression, _CalcPtr, len - 1, Error_, 0, StrAssign_, StrLength_, FALSE, ParseData_);
    if ((_CalcPtr->InAsyncMode() || _CalcPtr->StdInput() && _CalcPtr->StdOutput()) && !EqFound_)
      _FncCalls[_NumFncCalls]->SetShowAnswer(FALSE);

    ChrString HasIndex_ = EqFound_ ? "1":"0";
    _FncCalls[_NumFncCalls]->AddParameter(HasIndex_.c_str(), !EndEval_, EndEval_);
  }

  _NumFncCalls++;
  MemMatrix::Matrix().Deallocate(OldExpr_);
  MemMatrix::Matrix().Deallocate(Buffer_);
  OldExpr_ = NULL;
  Buffer_ = NULL;
}

/*****************************************************************************/
void SubExpression::FindFunctionCalls()
{
  int i, x, y, z;
  char* list = NULL;
  char* sublist = NULL;
  int len = 0;
  int fnclen = 0;
  int Valid_ = 1;
  int InQuote_ = 0;
  int NumFncs_ = 0;
  int* DelimArray_ = (int*)MemMatrix::Matrix().Callocate(sizeof(int) * CalculatorBase::DELIM_ARRAY_SIZE);
  char* Original_ = ::NewString(_Expression);
  bool GraphFound_ = false;
  bool BreakAll_ = false;

  list = _Expression;
  len = strlen(list);

  for (i = 0; i < len && !BreakAll_; i++)
  {
    if ((len - i) < _ShortestName)
      break;

    if (list[i] == '\"')
      InQuote_ = !InQuote_;

    if (!InQuote_ && isalpha(list[i]) && InRange('A', list[i], 'z'))
    {
      for (x = 0; x < _NumFncNames; x++)
      {
        fnclen = strlen(_FunctionNames[x]);

        if ((len - i) < fnclen || toupper(_FunctionNames[x][0]) != toupper(list[i]))
          continue;

        if (CalculatorBase::icasestrcompn(list+i, "GRAPH", 5) && isspace(list[i+5]))
          GraphFound_ = true;
        else if (CalculatorBase::icasestrcompn(list+i, _FunctionNames[x], fnclen) &&
                 _CalcPtr->WholeWord(list, len, fnclen, i, CalculatorBase::FUNCTION_INDEX) &&
                 CalculatorBase::FindFncDelim(list+i, strlen(list+i), &DelimArray_, CalculatorBase::DELIM_ARRAY_SIZE))
        {
          int SetOpFncs_ =
            (CalculatorBase::icasestrcompn(_FunctionNames[x], "SETUNI", fnclen) && fnclen == 6) ||
            (CalculatorBase::icasestrcompn(_FunctionNames[x], "SETINT", fnclen) && fnclen == 6) ||
            (CalculatorBase::icasestrcompn(_FunctionNames[x], "SETDIF", fnclen) && fnclen == 6) ||
            (CalculatorBase::icasestrcompn(_FunctionNames[x], "SETXDIF", fnclen) && fnclen == 7);

          int OptionFnc_ = ((CalculatorBase::icasestrcompn(_FunctionNames[x], "AVGMEAN", fnclen)     ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "AVGMED", fnclen)      ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "AVGMODE", fnclen)     ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "STDDEV", fnclen)      ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "SUM", fnclen)         ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "PROD", fnclen)        ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "MIN", fnclen)         ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "MAX", fnclen)         ||
                             (CalculatorBase::icasestrcompn(_FunctionNames[x], "SET", fnclen) &&
                              fnclen == 3 && !SetOpFncs_)                                            ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "FRAC", fnclen)        ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "ITEMPERM", fnclen)    ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "ITEMCOMB", fnclen)    ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "ITEMPERMREP", fnclen) ||
                             CalculatorBase::icasestrcompn(_FunctionNames[x], "ITEMCOMBREP", fnclen)) &&
                            DelimArray_[0] >= 3) ||
                           (CalculatorBase::icasestrcompn(_FunctionNames[x], "TOCMP2BIN", fnclen) &&
                            DelimArray_[0] == 3);

          int SingleArgFncs_ =
              (CalculatorBase::icasestrcompn(_FunctionNames[x], "SET", fnclen) && !SetOpFncs_) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "FRAC", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "SHOWSET", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "ITEMPERM", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "ITEMCOMB", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "ITEMPERMREP", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "ITEMCOMBREP", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "EVAL", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "SHOW", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "FROMHEX", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "FROMOCT", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "FROMBIN", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "FROMCMP2BIN", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "TOHEX", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "TOOCT", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "TOBIN", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "TOCMP2BIN", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "AVGMEAN", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "AVGMED", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "AVGMODE", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "STDDEV", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "SUM", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "MIN", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "MAX", fnclen) ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "PROD", fnclen);

          int GraphFnc_ =
              CalculatorBase::icasestrcompn(_FunctionNames[x], "GRAPH", fnclen);

          int ExtrFncs_ =
              CalculatorBase::icasestrcompn(_FunctionNames[x], "FMIN", fnclen)         ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "FMAX", fnclen);

          int DerivFncs_ =
              CalculatorBase::icasestrcompn(_FunctionNames[x], "DERIV", fnclen)        ||
              CalculatorBase::icasestrcompn(_FunctionNames[x], "DERIV2", fnclen);

          int IntegFnc_ =
              CalculatorBase::icasestrcompn(_FunctionNames[x], "INTEG", fnclen);

          int SolveFnc_ =
              CalculatorBase::icasestrcompn(_FunctionNames[x], "SOLVE", fnclen);

          int IntersectFnc_ =
              CalculatorBase::icasestrcompn(_FunctionNames[x], "INTERSECT", fnclen);

          int CalcFncs_ = ExtrFncs_ || DerivFncs_ ||
                          IntegFnc_ || SolveFnc_ || IntersectFnc_;
          int SingleArgValid_ = SingleArgFncs_ && DelimArray_[0] == 2;

          if (GraphFound_ && CalcFncs_)
          {
            CalculatorBase::SetPlotGraph(1);
            if (_CalcPtr->InAsyncMode() && _CalcPtr->InProgramMode())
            {
              _CalcPtr->SetIoState(Mcalc_IOState::GRAPH_WAIT);
              _CalcPtr->ResetRetryRead();
              int IoState_ = _CalcPtr->GetIoState(0);
              while (IoState_ != Mcalc_IOState::PROCESS &&
                     IoState_ != Mcalc_IOState::CALC_HALT &&
                     IoState_ != Mcalc_IOState::OUTPUT_FETCHED &&
                     IoState_ != Mcalc_IOState::INPUT_RECEIVED &&
                     IoState_ != Mcalc_IOState::BREAK_PROGRAM)
                IoState_ = _CalcPtr->GetIoState(1);

              BreakAll_ = IoState_ == Mcalc_IOState::CALC_HALT ||
                          IoState_ == Mcalc_IOState::BREAK_PROGRAM;

              if (BreakAll_)
                break;
            }

            if (ExtrFncs_)
              CalculatorBase::SetGraphType(Mcalc_GraphType::EXTREMA);
            else if (DerivFncs_)
              CalculatorBase::SetGraphType(Mcalc_GraphType::DERIVATIVE);
            else if (IntegFnc_)
              CalculatorBase::SetGraphType(Mcalc_GraphType::INTEGRAL);
            else if (SolveFnc_)
              CalculatorBase::SetGraphType(Mcalc_GraphType::ROOTSOLVE);
            else if (IntersectFnc_)
              CalculatorBase::SetGraphType(Mcalc_GraphType::INTERSECT1);
          }
          else if (GraphFnc_)
            CalculatorBase::SetGraphType(Mcalc_GraphType::FUNCTION);

          GraphFound_ = false;

          if (SingleArgFncs_ && !SingleArgValid_ && !OptionFnc_)
          {
            _CalcPtr->DisplayErrorMessage(_CalcPtr->GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_NUMARGS,
                                          ERRMSG_INVALID_NUMARGS, _FunctionNames[x]);
            _CalcPtr->CloseOutputFile();
            MemMatrix::Matrix().Deallocate(DelimArray_);
            return;
          }
          else if (((!SingleArgValid_ && DelimArray_[0] >= 3) ||
                    (SetOpFncs_ && DelimArray_[0] == 3) ||
                    (OptionFnc_ && SingleArgFncs_)) || SingleArgValid_)
          {
            NumFncs_++;
            sublist = list + i;
            AddFunctionCall(sublist, DelimArray_);
            list = _Expression;
            len = strlen(list);
            break;
          }
        }
      }
    }
  }

  if (BreakAll_)
  {
    MemMatrix::Matrix().Deallocate(Original_);
    MemMatrix::Matrix().Deallocate(DelimArray_);
    return;
  }

  if (DelimArray_[0] && NumFncs_ == 1 && Original_)
  {
    len = strlen(Original_);
    for (z = strlen(Original_) - 1; z && isspace(Original_[z]); z--);
    for (i = z; i && Original_[i] != '='; i--);
    for (;i < len && isspace(Original_[i]); i++);
    for (y = i; y < len && !CalculatorBase::isleftbracket(Original_[y]); y++);
    len = y - i;
    x = DelimArray_[0];

    if (DelimArray_[x] == z &&
        DelimArray_[1] == y &&
        DelimArray_[1] - len == i)
    {
      _SingleFncCall = TRUE;
      for (;y && isspace(Original_[y-1]); y--, len--);
      _ShowFunction = CalculatorBase::icasestrcompn(&Original_[i], "SHOW", len);
      _ShowSetFunction = CalculatorBase::icasestrcompn(&Original_[i], "SHOWSET", len);
    }
    else
    {
      _ShowFunction =
      _ShowSetFunction =
      _SingleFncCall = FALSE;
    }
  }
  else
  {
    _ShowFunction =
    _ShowSetFunction =
    _SingleFncCall = FALSE;
  }

  MemMatrix::Matrix().Deallocate(Original_);
  MemMatrix::Matrix().Deallocate(DelimArray_);
}

/****************************************************************************/
char* SubExpression::SubExpressionTag(int FncIndex_, char* SrcExpr_, const char* AnswerStr_)
{
  ChrString NumStr_;
  NumStr_.IntToStr(FncIndex_);
  ChrString Token_("#");
  Token_ += NumStr_;
  Token_ += ChrString("s");

  const char* cstr_ = Token_.c_str();
  char* start_ = strstr(SrcExpr_, cstr_);
  *start_ = 0;
  char* OldExpr_ = SrcExpr_;
  size_t len = strlen(SrcExpr_) + strlen(AnswerStr_) + strlen(&start_[strlen(cstr_)]);
  SrcExpr_ = (char*)MemMatrix::Matrix().Callocate(len + 1);

  strcpy(SrcExpr_, OldExpr_);
  strcat(SrcExpr_, AnswerStr_);
  strcat(SrcExpr_, &start_[strlen(cstr_)]);
  MemMatrix::Matrix().Deallocate(OldExpr_);

  return SrcExpr_;
}

/****************************************************************************/
void SubExpression::SubstFncAnswer(int FncIndex_)
{
  if (_FncCalls[FncIndex_]->OutputMode() != CalculatorBase::FLOATINGPOINT_MODE)
  {
    if (_ShowFunction && _Level == 0)
    {
      ChrString TempStr_ = ChrString("\"") +
                           ChrString(_FncCalls[FncIndex_]->GetLiteralAnswer()) +
                           ChrString("\"");
      _Expression = SubExpressionTag(FncIndex_, _Expression, TempStr_.c_str());
    }
    else if (_ShowSetFunction && _Level == 0)
    {
      int IsSet_ = 0;
      ChrString NewStr_ = ChrString(_FncCalls[FncIndex_]->GetLiteralAnswer());
      NewStr_.RemoveChar(" \t\n");
      Subscript x = NewStr_.strlen();
      ChrString TempStr_ = (NewStr_[0] == '{') ? NewStr_(1):NewStr_(0);

      if (NewStr_[0] == '{' && x > 2 && TempStr_[x-2] == '}')
      {
        TempStr_[x-2] = 0;
        TempStr_ = ChrString("{ ") + TempStr_ + ChrString(" }");
        IsSet_ = NewStr_[0] == '{';
      }

      if (!CalculatorBase::InSetNotation(TempStr_.c_str()))
      {
        TempStr_ = ChrString("{ ") +
                   ChrString(_FncCalls[FncIndex_]->GetLiteralAnswer()) +
                   ChrString(" }");

        if (CalculatorBase::InSetNotation(TempStr_.c_str()))
        {
          TempStr_ = ChrString("\"") + TempStr_ + ChrString("\"");
          _Expression = SubExpressionTag(FncIndex_, _Expression, TempStr_.c_str());
        }
        else
        {
          TempStr_ = ChrString("\"*** Not A Set ***\"");
          _Expression = SubExpressionTag(FncIndex_, _Expression, TempStr_.c_str());
        }
      }
      else if (IsSet_)
      {
        TempStr_ = ChrString("\"") + TempStr_ + ChrString("\"");
        _Expression = SubExpressionTag(FncIndex_, _Expression, TempStr_.c_str());
      }
      else
      {
        TempStr_ = ChrString("\"*** Not A Set ***\"");
        _Expression = SubExpressionTag(FncIndex_, _Expression, TempStr_.c_str());
      }
    }
    else
      _Expression = SubExpressionTag(FncIndex_, _Expression, _FncCalls[FncIndex_]->GetLiteralAnswer());
  }
  else
    _Expression = SubExpressionTag(FncIndex_, _Expression, _FncCalls[FncIndex_]->GetAnswer());
}

/****************************************************************************/
void SubExpression::SubstFncOriginal(int FncIndex_)
{
  _OriginalStr = SubExpressionTag(FncIndex_, _OriginalStr, _FncCalls[FncIndex_]->GetOriginal());
}

/****************************************************************************/
const char* SubExpression::Evaluate()
{
  if (_CalcPtr->HasErrors() || !_Expression)
    return NULL;

  int i, Error_;
  int ReEval_ = 0;
  int ExprValid_ = _NumFncCalls ? 0:(_Parent && _CalcPtr->IsValidNum(_Expression));
  int EvalFnc_ = 0;
  int ShowFnc_ = 0;

  if (_NumFncCalls)
  {
    _CalcPtr->ResetAnswerMode();
    ShowFnc_ = CalculatorBase::icasestrcompn(_FncCalls[0]->GetFncName(), "SHOW", 4);
    EvalFnc_ = CalculatorBase::icasestrcompn(_FncCalls[0]->GetFncName(), "EVAL", 4);

    for (i = 0; i < _NumFncCalls && !_CalcPtr->HasErrors(); i++)
    {
      _FncCalls[i]->Evaluate();

      if (!_CalcPtr->HasErrors())
      {
        SubstFncAnswer(i);
        if (!_Parent && CalculatorBase::icasestrcompn(_FncCalls[i]->GetFncName(), "EVAL", 4))
          SubstFncOriginal(i);

        ExprValid_ = _Parent && _CalcPtr->IsValidNum(_Expression);

        if (!ExprValid_ && i >= _NumFncCalls - 1 && ((!_ShowFunction && !_ShowSetFunction) || _Level == 0))
        {
          _CalcPtr->SetProcessVariables(_Parent != NULL);
          if (_Level)
            _CalcPtr->PushActiveExpression(this);
          _CalcPtr->TransferData(_Expression, _CalcPtr);
          if (_Level)
            _CalcPtr->PopActiveExpression();
        }
      }
    }
  }

  if (_CalcPtr->HasErrors())
    return NULL;

  if (ExprValid_)
  {
    _AnswerStr =  AllocateNewString(_AnswerStr, ::SafeStrLen(_Expression));
    ::SafeStrCpy(_AnswerStr, _Expression);

    if (!_Parent)
    {
      _OriginalStr =  AllocateNewString(_OriginalStr, ::SafeStrLen(_Expression));
      ::SafeStrCpy(_OriginalStr, _Expression);
      _CalcPtr->ReplaceTokens(&_OriginalStr, _CalcPtr, FALSE, Error_, ReEval_, FALSE, FALSE);
    }

    return _AnswerStr;
  }
  else
  {
    if (!_NumFncCalls)
    {
      _CalcPtr->ResetAnswerMode();
      _CalcPtr->SetProcessVariables(_Parent != NULL);
      if (_Level)
        _CalcPtr->PushActiveExpression(this);
      _CalcPtr->TransferData(_Expression, _CalcPtr);
      if (_Level)
        _CalcPtr->PopActiveExpression();

      if (_CalcPtr->HasErrors())
        return NULL;
    }

    if (HasAnswer())
    {
      int Mode_ = _CalcPtr->AnswerMode();
      const char* Answer_ = _CalcPtr->AnswerStr();

      if (Answer_)
      {
        _AnswerStr =  AllocateNewString(_AnswerStr, ::SafeStrLen(Answer_));
        ::SafeStrCpy(_AnswerStr, Answer_);

        if (_Parent)
        {
          _Expression = AllocateNewString(_Expression, ::SafeStrLen(_AnswerStr));
          ::SafeStrCpy(_Expression, _AnswerStr);
        }
        else
        {
          int SkipEval_ = (ShowFnc_ || EvalFnc_) && _SingleFncCall;

          if (!SkipEval_)
            _CalcPtr->ReplaceTokens(&_OriginalStr, _CalcPtr, FALSE, Error_, ReEval_, FALSE, FALSE);
          _CalcPtr->SetOriginalStr(_OriginalStr);
        }

        return _AnswerStr;
      }
      else
      {
        _AnswerStr =  AllocateNewString(_AnswerStr, 5);
        ::SafeStrCpy(_AnswerStr, "");
      }
    }
    else
    {
      _AnswerStr =  AllocateNewString(_AnswerStr, 5);
      ::SafeStrCpy(_AnswerStr, "");
    }
  }

  return NULL;
}

/****************************************************************************/
Boolean SubExpression::HasCommandOutput() const
{
  int Show_ = HasAnswer() && ShowAnswer();
  int i;

  if (Show_)
    for (i = 0; i < _NumFncCalls; i++)
      if (!_FncCalls[i]->ShowAnswer())
      {
        Show_ = FALSE;
        break;
      }

  return (Show_ && !_CalcPtr->StoreResults());
}

/****************************************************************************/
void SubExpression::ShowResults()
{
  int Show_ = ShowAnswer();
  int i;

  if (Show_)
    for (i = 0; i < _NumFncCalls; i++)
      if (!_FncCalls[i]->ShowAnswer())
      {
        Show_ = FALSE;
        break;
      }

  if (Show_ && !_CalcPtr->StoreResults())
  {
    _CalcPtr->SetDisplayOption(CalculatorBase::DISPLAY_TO_STREAM);
    _CalcPtr->DisplayAnswerString();
    _CalcPtr->SetDisplayOption(CalculatorBase::DISPLAY_TO_STRING);
  }
  else if (_CalcPtr->InAsyncMode())
    _CalcPtr->SetIoState(Mcalc_IOState::CALC_RESPONSE);
}

/****************************************************************************/
/****************************************************************************/
FunctionCall::FunctionCall():
_Parent(NULL),
_FncName(NULL),
_NumParams(0),
_MaxParams(0),
_ParamArray(NULL),
_AnswerStr(NULL),
_LiteralAnsStr(NULL),
_OutputMode(CalculatorBase::FLOATINGPOINT_MODE),
_ShouldShow(TRUE)
{}

/****************************************************************************/
FunctionCall::FunctionCall(SubExpression* Parent_, char* Name_):
_Parent(Parent_),
_FncName(NULL),
_NumParams(0),
_MaxParams(0),
_ParamArray(NULL),
_AnswerStr(NULL),
_LiteralAnsStr(NULL),
_OutputMode(CalculatorBase::FLOATINGPOINT_MODE),
_ShouldShow(TRUE)
{
  ChrString NameStr_ = Name_;
  NameStr_.RemovePadding(" \t\n");

  _FncName = (char*)MemMatrix::Matrix().Callocate(NameStr_.strlen() + 1);
  ::SafeStrCpy(_FncName, NameStr_.c_str());
}

/****************************************************************************/
FunctionCall::~FunctionCall()
{
  Clear();
}

/****************************************************************************/
void FunctionCall::SetShowAnswer(int Value_)
{
  _ShouldShow = Value_;
}

/****************************************************************************/
void FunctionCall::SetNumParams(int Params_)
{
  MemMatrix::Matrix().Deallocate(_ParamArray);
  _ParamArray = (FncParameter**)MemMatrix::Matrix().Callocate(Params_ * sizeof(FncParameter*));

  int x;
  for (x = 0; x < Params_; x++)
    _ParamArray[x] = NULL;

  _MaxParams = Params_;
}

/****************************************************************************/
void FunctionCall::AddParameter(const char* Expr_, int FindCalls_, int ReplExpr_)
{
  if (_NumParams >= _MaxParams)
  {
    _Parent->GetCalculator()->DisplayErrorMessage(
        _Parent->GetCalculator()->GetOutputFile(),
        Mcalc_Error::ERRVAL_ARRAYBOUNDS, "%s", ERRMSG_ARRAYBOUNDS);
    exit(1);
  }
  else
  {
    ChrString ExprStr_ = Expr_;
    ExprStr_.RemovePadding(" \t\n");

    _ParamArray[_NumParams] = new FncParameter;
    _ParamArray[_NumParams]->_Param = new SubExpression(_Parent, _Parent->GetLevel() + 1, ExprStr_.c_str());
    _ParamArray[_NumParams]->_Param->SetCalculator(_Parent->GetCalculator());
    _ParamArray[_NumParams]->_ShouldEval = FindCalls_;
    _ParamArray[_NumParams]->_ShouldReplace = ReplExpr_;

    ++_NumParams;
    if (!_Parent->GetCalculator()->IsValidNum(ExprStr_.c_str()))
      _ParamArray[_NumParams - 1]->_Param->FindFunctionCalls();
  }
}

/****************************************************************************/
void FunctionCall::ReplaceParameter(int Index_, CalculatorBase* CalcPtr_,
                                    int& Error_, int& ReEvaluate_, int ReFormat_, int ChangeModes_)
{
  if (_ParamArray[Index_]->_Param->_ShowFunction ||
      _ParamArray[Index_]->_Param->_ShowSetFunction)
    _ParamArray[Index_]->_Param->Evaluate() != NULL;
  else
    CalcPtr_->ReplaceTokens(&(_ParamArray[Index_]->_Param->_Expression), CalcPtr_, TRUE, Error_, ReEvaluate_, ReFormat_, FALSE);
}

/****************************************************************************/
void FunctionCall::Evaluate()
{
  int i;
  int Error_ = 0;
  int EvalOk_ = TRUE;
  int SetVar_ = 0;
  int SetCnt_ = 0;
  int SetName_ = 0;
  int SetFile_ = 0;
  int SetQuote_ = 0;
  int ReEvaluate_ = 0;
  int SingleArgFncs_ = _NumParams == 1;
  int ConvFncs_ =
      CalculatorBase::icasestrcompn(_FncName, "TOHEX", 5) ||
      CalculatorBase::icasestrcompn(_FncName, "TOOCT", 5) ||
      CalculatorBase::icasestrcompn(_FncName, "TOBIN", 5) ||
      CalculatorBase::icasestrcompn(_FncName, "TOCMP2BIN", 9);
  int SetOpFncs_ =
      !SingleArgFncs_ &&
      (CalculatorBase::icasestrcompn(_FncName, "SETUNI", 6)     ||
       CalculatorBase::icasestrcompn(_FncName, "SETINT", 6)     ||
       CalculatorBase::icasestrcompn(_FncName, "SETDIF", 6)     ||
       CalculatorBase::icasestrcompn(_FncName, "SETXDIF", 7));
  int MakeSetFnc_ =
      CalculatorBase::icasestrcompn(_FncName, "SET", 3) && !SetOpFncs_;
  int ItemFncs_ =
      !SingleArgFncs_ &&
      (CalculatorBase::icasestrcompn(_FncName, "ITEMPERM", 8)         ||
       CalculatorBase::icasestrcompn(_FncName, "ITEMCOMB", 8)         ||
       CalculatorBase::icasestrcompn(_FncName, "ITEMPERMREP", 11)     ||
       CalculatorBase::icasestrcompn(_FncName, "ITEMCOMBREP", 11));
  int FracFnc_ = !SingleArgFncs_ && CalculatorBase::icasestrcompn(_FncName, "FRAC", 4);
  int EvalFnc_ = CalculatorBase::icasestrcompn(_FncName, "EVAL", 4);
  int ShowFnc_ = CalculatorBase::icasestrcompn(_FncName, "SHOW", 4);
  int ShowSetFnc_ = CalculatorBase::icasestrcompn(_FncName, "SHOWSET", 7);
  int ReFormat_ = FALSE;
  int PreCheckInvalid_ = TRUE;
  char* NewOriginal_ = NULL;
  CalculatorBase* CalcPtr_ = _Parent->GetCalculator();

  if (ShowFnc_ || ShowSetFnc_)
    CalcPtr_->SetEvalStrings(TRUE);
  else if (EvalFnc_)
  {
    CalcPtr_->SetEvalStrings(TRUE);
    NewOriginal_ = ::NewString(_ParamArray[0]->_Param->_Expression);

    CalcPtr_->ReplaceTokens(&NewOriginal_, CalcPtr_, FALSE, Error_, ReEvaluate_, FALSE, FALSE);
    CalcPtr_ = CalcPtr_->GetCalcType();
    CalcPtr_->SetEvalStrings(TRUE);
    ReFormat_ = TRUE;
  }
  else if (MakeSetFnc_)
  {
    char* VarStr_ = SingleArgFncs_ ? NULL:_ParamArray[1]->_Param->_Expression;
    SetCnt_ = SingleArgFncs_ ?
                0:(_NumParams > 2 && IsNumber(VarStr_, 0, 0, 0, 10));
    SetVar_ = SingleArgFncs_ ?
                0:(_NumParams == 2 && VarStr_ && strlen(VarStr_) == 1 && isalpha(VarStr_[0]));
    SetName_ = _ParamArray[0]->_Param->_Expression &&
               ::isalpha(_ParamArray[0]->_Param->_Expression[0]);
    SetFile_ = SingleArgFncs_ ?
                 0:(_NumParams == 2 && VarStr_ && strlen(VarStr_) > 1 && isalpha(VarStr_[0]));
    SetQuote_ = SingleArgFncs_ ?
                  0:(_NumParams == 2 && VarStr_ && strlen(VarStr_) > 1 && CalculatorBase::InSetNotation(VarStr_));

    PreCheckInvalid_ = (!SingleArgFncs_ &&
                        !(SetCnt_ || SetVar_ || SetFile_ || SetQuote_)) || !SetName_;

    if (PreCheckInvalid_)
    {
      if ((_NumParams > 2 && VarStr_ && !SetCnt_) ||
          (_NumParams == 2 && VarStr_ && strlen(VarStr_) == 1 && isalpha(VarStr_[0]) && !SetVar_) ||
          (_NumParams == 2 && VarStr_ && strlen(VarStr_) > 1 && isalpha(VarStr_[0]) && !SetFile_) ||
          (_NumParams == 2 && VarStr_ && strlen(VarStr_) > 1 && !isalpha(VarStr_[0]) && !SetQuote_))
        ReplaceParameter(1, CalcPtr_, Error_, ReEvaluate_, TRUE);

      if (!SetName_)
        ReplaceParameter(0, CalcPtr_, Error_, ReEvaluate_, TRUE);

      VarStr_ = SingleArgFncs_ ? NULL:_ParamArray[1]->_Param->_Expression;
      SetCnt_ = SingleArgFncs_ ?
                  0:(_NumParams > 2 && IsNumber(VarStr_, 0, 0, 0, 10));
      SetVar_ = SingleArgFncs_ ?
                  0:(_NumParams == 2 && VarStr_ && strlen(VarStr_) == 1 && isalpha(VarStr_[0]));
      SetName_ = _ParamArray[0]->_Param->_Expression &&
                 ::isalpha(_ParamArray[0]->_Param->_Expression[0]);
      SetFile_ = SingleArgFncs_ ?
                   0:(_NumParams == 2 && VarStr_ && strlen(VarStr_) > 1 && isalpha(VarStr_[0]));
      SetQuote_ = SingleArgFncs_ ?
                    0:(_NumParams == 2 && VarStr_ && strlen(VarStr_) > 1 && CalculatorBase::InSetNotation(VarStr_));

      PreCheckInvalid_ = (!SingleArgFncs_ &&
                          !(SetCnt_ || SetVar_ || SetFile_ || SetQuote_)) || !SetName_;
    }
  }
  else if (ItemFncs_)
  {
    PreCheckInvalid_ = !IsNumber(_ParamArray[0]->_Param->_Expression, 0, 0, 0, 10);

    if (PreCheckInvalid_)
    {
      ReplaceParameter(0, CalcPtr_, Error_, ReEvaluate_, TRUE);
      PreCheckInvalid_ = !IsNumber(_ParamArray[0]->_Param->_Expression, 0, 0, 0, 10);
    }
  }
  else if (SetOpFncs_)
  {
    SetName_ = (_NumParams == 3) &&
               _ParamArray[0]->_Param->_Expression &&
               ::isalpha(_ParamArray[0]->_Param->_Expression[0]) &&
               _ParamArray[1]->_Param->_Expression &&
               ::isalpha(_ParamArray[1]->_Param->_Expression[0]) &&
               _ParamArray[2]->_Param->_Expression &&
               ::isalpha(_ParamArray[2]->_Param->_Expression[0]);
    PreCheckInvalid_ = !SetName_;

    if (PreCheckInvalid_ && _NumParams == 3)
    {
      ReplaceParameter(0, CalcPtr_, Error_, ReEvaluate_, TRUE);
      ReplaceParameter(1, CalcPtr_, Error_, ReEvaluate_, TRUE);
      ReplaceParameter(2, CalcPtr_, Error_, ReEvaluate_, TRUE);

      SetName_ = (_NumParams == 3) &&
                 _ParamArray[0]->_Param->_Expression &&
                 ::isalpha(_ParamArray[0]->_Param->_Expression[0]) &&
                 _ParamArray[1]->_Param->_Expression &&
                 ::isalpha(_ParamArray[1]->_Param->_Expression[0]) &&
                 _ParamArray[2]->_Param->_Expression &&
                 ::isalpha(_ParamArray[2]->_Param->_Expression[0]);
      PreCheckInvalid_ = !SetName_;
    }
  }

  if (FracFnc_)
  {
    for (i = 0; i < _NumParams; i++)
    {
      if (_ParamArray[i]->_ShouldEval)
      {
        if (CalcPtr_->IsFraction(_ParamArray[i]->_Param->_Expression) ||
            ::IsNumber(_ParamArray[i]->_Param->_Expression, 1, 1, 1, 10))
          EvalOk_ = TRUE;
        else
        {
          ReplaceParameter(i, CalcPtr_, Error_, ReEvaluate_, TRUE);

          if (CalcPtr_->IsFraction(_ParamArray[i]->_Param->_Expression) ||
              ::IsNumber(_ParamArray[i]->_Param->_Expression, 1, 1, 1, 10))
            EvalOk_ = TRUE;
          else
          {
            EvalOk_ = _ParamArray[i]->_Param->Evaluate() != NULL;
            if (!EvalOk_)
              break;
          }
        }
      }
      else if (_ParamArray[i]->_ShouldReplace)
        ReplaceParameter(i, CalcPtr_, Error_, ReEvaluate_, FALSE);
    }
  }
  else
  {
    for (i = 0; i < _NumParams; i++)
    {
      if (_ParamArray[i]->_ShouldEval && PreCheckInvalid_)
      {
        EvalOk_ = _ParamArray[i]->_Param->Evaluate() != NULL;
        if (!EvalOk_)
          break;
      }
      else if (_ParamArray[i]->_ShouldReplace)
        ReplaceParameter(i, CalcPtr_, Error_, ReEvaluate_, ReFormat_, ReFormat_);
    }
  }

  ChrString FncResult_;
  ChrString AnsString_;

  if (EvalFnc_)
  {
    CalcPtr_ = CalcPtr_->GetCalcType();
    CalcPtr_->SetEvalStrings(FALSE);

    _ParamArray[0]->_Param->SetOriginal(NewOriginal_);
    MemMatrix::Matrix().Deallocate(NewOriginal_);

    if (ReEvaluate_)
    {
      ChrString ExprStr_ = _ParamArray[0]->_Param->_Expression;
      AnsString_ = CalcPtr_->ReEvaluate(ExprStr_);
    }
    else
    {
      const char* Ans_ = _ParamArray[0]->_Param->Evaluate();
      if (Ans_)
        AnsString_ = Ans_;
    }

    if (AnsString_.strlen())
      FncResult_ = AnsString_;
    _OutputMode = CalcPtr_->AnswerMode();
  }
  else if (ShowFnc_)
  {
    CalcPtr_ = CalcPtr_->GetCalcType();
    CalcPtr_->SetEvalStrings(FALSE);
    AnsString_ = _ParamArray[0]->_Param->_Expression;

    if (AnsString_.strlen())
    {
      FncResult_ = AnsString_;
      CalcPtr_->SetAnswerMode(CalculatorBase::LITERAL_MODE);
      CalcPtr_->SetLiteralStr(FncResult_.c_str());
    }
    _OutputMode = CalcPtr_->AnswerMode();
  }
  else if (ShowSetFnc_)
  {
    CalcPtr_ = CalcPtr_->GetCalcType();
    CalcPtr_->SetEvalStrings(FALSE);
    AnsString_ = _ParamArray[0]->_Param->_Expression;

    if (AnsString_.strlen())
    {
      FncResult_ = AnsString_;
      CalcPtr_->SetAnswerMode(CalculatorBase::LITERAL_MODE);
      CalcPtr_->SetLiteralStr(FncResult_.c_str());
    }
    _OutputMode = CalcPtr_->AnswerMode();
  }
  else if (EvalOk_)
  {
    FncResult_ = CalcPtr_->EvaluateFunction(_FncName, _NumParams, _ParamArray);
    _OutputMode = CalcPtr_->AnswerMode();
  }

  if (_OutputMode != CalculatorBase::FLOATINGPOINT_MODE)
  {
    const char* LiteralStr_ = (_OutputMode == CalculatorBase::LITERAL_MODE ||
                               _OutputMode == CalculatorBase::ALTBASE_MODE) ?
          CalcPtr_->LiteralStr():
          CalcPtr_->FractionStr();

    _LiteralAnsStr = SubExpression::AllocateNewString(_LiteralAnsStr, ::SafeStrLen(LiteralStr_));
    ::SafeStrCpy(_LiteralAnsStr, LiteralStr_);
  }

  _AnswerStr = SubExpression::AllocateNewString(_AnswerStr, FncResult_.strlen());
  ::SafeStrCpy(_AnswerStr, FncResult_.c_str());
}

/****************************************************************************/
const char* FunctionCall::GetOriginal() const
{
  return _ParamArray[0]->_Param->_OriginalStr;
}

/****************************************************************************/
void FunctionCall::Clear()
{
  int x;
  for (x = 0; x < _NumParams; x++)
    delete _ParamArray[x];

  _NumParams = 0;
  MemMatrix::Matrix().Deallocate(_FncName);
  MemMatrix::Matrix().Deallocate(_AnswerStr);
  MemMatrix::Matrix().Deallocate(_LiteralAnsStr);
  MemMatrix::Matrix().Deallocate(_ParamArray);

  _FncName = _AnswerStr = NULL;
  _ParamArray = NULL;
}

/****************************************************************************/
/****************************************************************************/
FncParameter::FncParameter():
_Param(NULL),
_ShouldEval(0)
{}

/****************************************************************************/
FncParameter::FncParameter(const FncParameter& Obj_):
_Param(Obj_._Param ? (new SubExpression(*Obj_._Param)):NULL),
_ShouldEval(Obj_._ShouldEval)
{}

/****************************************************************************/
FncParameter::~FncParameter()
{
  delete _Param;
  _Param = NULL;
}

/****************************************************************************/
MEMORYOPS_DEFN(SubExpression)
MEMORYOPS_DEFN(FunctionCall)
MEMORYOPS_DEFN(FncParameter)

/****************************************************************************/
/****************************************************************************/
// CalculatorBase base class constructor
//
CalculatorBase::CalculatorBase():
_AnswerStr(NULL),
_OriginalStr(NULL),
_FractionStr(NULL),
_LiteralStr(NULL),
_DisplayOption(DISPLAY_TO_STREAM),
_AnswerMode(FLOATINGPOINT_MODE),
_ModeFinalized(0),
_InitialMode(TRUE),
_ProcessVars(0),
_ErrorValue(0),
_SkipEvaluation(0),
_NumberBase(10),
_RecursionLimit(RECURSION_LIMIT),
_GraphPrecision(DEFAULT_PRECVAL_INTEG),
_SuppressOutput(0),
_ExecuteProgram(NULL),
_FncAttribs(NULL)
{
  #if CALCLIB_DEBUG2
    logfile = FileOpen(MCALC_LOG_FILE, "w");
    fputs("Entering: CalculatorBase constructor\n", logfile);
  #endif

  ++_Instances;

  if (_Instances == 1)
  {
    ResetInputFnc();
    ResetOutputFnc();

    _CurrentFunctionName = (char*)allocmem(256, sizeof(char), _CurrentFunctionName);
    _CurrentFunctionName[0] = 0;
    _ErrorMessageArray = (char**)allocmem(Mcalc_Error::MAX_ERRORS, sizeof(char*), _ErrorMessageArray);
    _ErrorMessageArray[0] = NULL;
    _ErrorMessageArray[Mcalc_Error::ERRVAL_INVALID_EXPR] = ::NewString(ERRMSG_INVALID_EXPR);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_MISSLEFT_BRACKET] = ::NewString(ERRMSG_MISSLEFT_BRACKET);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_MISSRIGHT_BRACKET] = ::NewString(ERRMSG_MISSRIGHT_BRACKET);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_INVALID_SYMBOLS] = ::NewString(ERRMSG_INVALID_SYMBOLS);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_INVALID_ASSIGNMENT] = ::NewString(ERRMSG_INVALID_ASSIGNMENT);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_INVALID_NUMARGS] = ::NewString(ERRMSG_INVALID_NUMARGS);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED] = ::NewString(ERRMSG_MULLERS_METHOD_FAILED);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_DIVISION_BY_ZERO] = ::NewString(ERRMSG_DIVISION_BY_ZERO);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_OPENSTATEFILE] = ::NewString(ERRMSG_OPENSTATEFILE);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_MEMORY_FAIL] = ::NewString(ERRMSG_MEMORY_FAIL);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_FILE_ERROR] = ::NewString(ERRMSG_FILE_ERROR);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_CLIENTNOTFOUND] = ::NewString(ERRMSG_CLIENTNOTFOUND);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_ARRAYBOUNDS] = ::NewString(ERRMSG_ARRAYBOUNDS);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_INVALID_TYPEARGS] = ::NewString(ERRMSG_INVALID_TYPEARGS);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_EMPTY_DATASET] = ::NewString(ERRMSG_EMPTY_DATASET);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_RECURSION_LIMIT] = ::NewString(ERRMSG_RECURSION_LIMIT);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_ARGLIST_SIZE] = ::NewString(ERRMSG_ARGLIST_SIZE);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_UNSOLVABLE_MATRIX] = ::NewString(ERRMSG_UNSOLVABLE_MATRIX);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_INVALID_MATRIX_COLS] = ::NewString(ERRMSG_INVALID_MATRIX_COLS);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_EMPTYROW_MATRIX] = ::NewString(ERRMSG_EMPTYROW_MATRIX);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_NO_RESULT_COL_MATRIX] = ::NewString(ERRMSG_NO_RESULT_COL_MATRIX);
    _ErrorMessageArray[Mcalc_Error::ERRVAL_INVALID_GRAPHVARNAME] = ::NewString(ERRMSG_INVALID_GRAPHVARNAME);

    _ErrorMessageArgCnt = (int*)allocmem(Mcalc_Error::MAX_ERRORS, sizeof(int), _ErrorMessageArgCnt);
    _ErrorMessageArgCnt[0] = 0;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_INVALID_EXPR] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_MISSLEFT_BRACKET] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_MISSRIGHT_BRACKET] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_INVALID_SYMBOLS] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_INVALID_ASSIGNMENT] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_INVALID_NUMARGS] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_DIVISION_BY_ZERO] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_OPENSTATEFILE] = 0;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_MEMORY_FAIL] = 0;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_FILE_ERROR] = 2;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_CLIENTNOTFOUND] = 0;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_ARRAYBOUNDS] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_INVALID_TYPEARGS] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_EMPTY_DATASET] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_RECURSION_LIMIT] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_ARGLIST_SIZE] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_UNSOLVABLE_MATRIX] = 0;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_INVALID_MATRIX_COLS] = 0;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_EMPTYROW_MATRIX] = 0;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_NO_RESULT_COL_MATRIX] = 1;
    _ErrorMessageArgCnt[Mcalc_Error::ERRVAL_INVALID_GRAPHVARNAME] = 1;

    if (!FUNCTION_NAMES)
    {
      FUNCTION_NAMES = (char**)allocmem(MAXFUNCTIONS, sizeof(char*), FUNCTION_NAMES);
      AddFunctionName("MIN");
      AddFunctionName("MAX");
      AddFunctionName("DERIV");
      AddFunctionName("DERIV2");
      AddFunctionName("INTEG");
      AddFunctionName("FMIN");
      AddFunctionName("FMAX");
      AddFunctionName("ROUND");
      AddFunctionName("SOLVE");
      AddFunctionName("INTERSECT");
      AddFunctionName("PERMREP");
      AddFunctionName("COMBREP");
      AddFunctionName("COMB");
      AddFunctionName("PERM");
      AddFunctionName("ITEMPERMREP");
      AddFunctionName("ITEMCOMBREP");
      AddFunctionName("ITEMCOMB");
      AddFunctionName("ITEMPERM");
      AddFunctionName("SETUNI");
      AddFunctionName("SETINT");
      AddFunctionName("SETDIF");
      AddFunctionName("SETXDIF");
      AddFunctionName("FRAC");
      AddFunctionName("SET");
      AddFunctionName("SUM");
      AddFunctionName("PROD");
      AddFunctionName("STDDEV");
      AddFunctionName("AVGMODE");
      AddFunctionName("AVGMEAN");
      AddFunctionName("AVGMED");
      AddFunctionName("RECTOPOLR");
      AddFunctionName("RECTOPOLA");
      AddFunctionName("POLTORECX");
      AddFunctionName("POLTORECY");
      AddFunctionName("EVAL");
      AddFunctionName("SHOW");
      AddFunctionName("SHOWSET");
      AddFunctionName("FROMHEX");
      AddFunctionName("FROMOCT");
      AddFunctionName("FROMBIN");
      AddFunctionName("FROMCMP2BIN");
      AddFunctionName("TOHEX");
      AddFunctionName("TOOCT");
      AddFunctionName("TOBIN");
      AddFunctionName("TOCMP2BIN");
      AddFunctionName("MATSOLVE");
      AddFunctionName("GRAPH");
    }

    int x;
    if (!files)
    {
      files = (FILE**)allocmem(MAXFILES, sizeof(FILE*), files);
      for (x = 0; x < MAXFILES; x++)
        files[x] = NULL;
    }

    for (x = 0; x < MAXFILES; x++)
      filestatus[x] = 0;
  }

  int x;
  _FncAttribs = (Uchar*)allocmem(256, sizeof(Uchar), _FncAttribs);
  for (x = 0; x < 256; x++)
    _FncAttribs[x] = 0;

  SetAsUnary(RND_OP);
  SetAsUnary(SIN_OP);
  SetAsUnary(COS_OP);
  SetAsUnary(TAN_OP);
  SetAsUnary(ASIN_OP);
  SetAsUnary(ACOS_OP);
  SetAsUnary(ATAN_OP);
  SetAsUnary(SINH_OP);
  SetAsUnary(COSH_OP);
  SetAsUnary(TANH_OP);
  SetAsUnary(EXP_OP);
  SetAsUnary(LOG_OP);
  SetAsUnary(LOG10_OP);
  SetAsUnary(CEILING_OP);
  SetAsUnary(FLOOR_OP);
  SetAsUnary(ABS_OP);
  SetAsUnary(FRACPART_OP);
  SetAsUnary(INTPART_OP);
  SetAsUnary(SQR_OP);
  SetAsUnary(SQRT_OP);
  SetAsUnary(CUBE_OP);
  SetAsUnary(CUBERT_OP);
  SetAsUnary(RAND_OP);
  SetAsUnary(INV_OP);
  SetAsUnary(NOT_OP, 1);

  SetAsBinary(COMB_OP);
  SetAsBinary(PERM_OP);
  SetAsBinary(COMBREP_OP);
  SetAsBinary(PERMREP_OP);
  SetAsBinary(SCI_OP);
  SetAsBinary(LSTHANOREQ_OP, 1);
  SetAsBinary(GRTHANOREQ_OP, 1);
  SetAsBinary(EQUAL_OP, 1);
  SetAsBinary(NOTEQUAL_OP, 1);
  SetAsBinary(XOR_OP, 1);
  SetAsBinary(AND_OP, 1);
  SetAsBinary(OR_OP, 1);

  SetAsConstant(FRACANS_CONST);
  SetAsConstant(ANS_CONST, 1);
  SetAsConstant(ANSSTR_CONST, 1);
  SetAsConstant(PI_CONST);
  SetAsConstant(E_CONST);
  SetAsConstant(NAN_CONST);
  SetAsConstant(INF_CONST);

  SetAsVariable(VARIABLE_INDEX);
  SetAsPrefix(HEXPREFIX_INDEX, 1);
  SetAsConstant(FUNCTION_INDEX, 1);
  SetAsBinary(BINARYOPS_INDEX, 1);
  SetAsUnary(UNARYOPS_INDEX, 1);
  SetAsPostfix(POSTOPS_INDEX, 1);
  Set<int>::SetComparisonFunction(Mcalc_DefaultSetElementEqualFnc);

  #if CALCLIB_DEBUG2
    fputs("Leaving: CalculatorBase constructor\n", logfile);
  #endif
}

/****************************************************************************/
CalculatorBase::~CalculatorBase()
{
  MemMatrix::Matrix().Deallocate(_LiteralStr);
  MemMatrix::Matrix().Deallocate(_FractionStr);
  MemMatrix::Matrix().Deallocate(_AnswerStr);
  MemMatrix::Matrix().Deallocate(_OriginalStr);
  MemMatrix::Matrix().Deallocate(_ExecuteProgram);

  _LiteralStr =
  _FractionStr =
  _AnswerStr =
  _OriginalStr =
  _ExecuteProgram = NULL;

  --_Instances;

  if (!_Instances)
  {
    GetLogFile();
    if (filestatus[Mcalc_IOType::ErrorLog] && logfile)
    {
      long Size_ = FileSize(logfile);
      fclose(logfile);
      logfile = NULL;

      if (!Size_)
        if (LOG_FILE)
          unlink(LOG_FILE);
        else
          unlink(MCALC_LOG_FILE);
    }

    // session graph files
    char Fname_[32];
    char suf;

    for (suf = 'a'; suf <= 'z'; suf++)
    {
      sprintf(Fname_, "mcalc_graph%d%c.out", _SessionNum, suf);
      unlink(Fname_);
    }

    if (CURRENTGRAPH_FILE)
    {
      unlink(CURRENTGRAPH_FILE);
      MemMatrix::Matrix().Deallocate(CURRENTGRAPH_FILE);
      CURRENTGRAPH_FILE = NULL;
    }

    int x;
    if (FUNCTION_NAMES)
    {
      for (x = 0; x < MAXFUNCTIONS; x++)
        MemMatrix::Matrix().Deallocate(FUNCTION_NAMES[x]);

      MemMatrix::Matrix().Deallocate(FUNCTION_NAMES);
      FUNCTION_NAMES = NULL;
      _ShortestFncName =
      _NumFncNames = 0;
    }

    if (_CurrentFunctionName)
    {
      MemMatrix::Matrix().Deallocate(_CurrentFunctionName);
      _CurrentFunctionName = NULL;
    }

    if (_ErrorMessageArray)
    {
      for (x = 0; x < Mcalc_Error::MAX_ERRORS; x++)
        MemMatrix::Matrix().Deallocate(_ErrorMessageArray[x]);

      MemMatrix::Matrix().Deallocate(_ErrorMessageArray);
      _ErrorMessageArray = NULL;
    }

    if (_ErrorMessageArgCnt)
    {
      MemMatrix::Matrix().Deallocate(_ErrorMessageArgCnt);
      _ErrorMessageArgCnt = NULL;
    }

    MemMatrix::Matrix().Deallocate(LOG_FILE);
    MemMatrix::Matrix().Deallocate(USER_INPUT_FILE);
    MemMatrix::Matrix().Deallocate(USER_PROMPT_FILE);
    MemMatrix::Matrix().Deallocate(PROG_OUTPUT_FILE);
    MemMatrix::Matrix().Deallocate(DATA_INPUT_FILE);
    MemMatrix::Matrix().Deallocate(DATA_OUTPUT_FILE);
    MemMatrix::Matrix().Deallocate(PROG_IOSTATE_FILE);
    MemMatrix::Matrix().Deallocate(SERVERALIVE_FILE);
    MemMatrix::Matrix().Deallocate(CLIENTALIVE_FILE);

    LOG_FILE =
    USER_INPUT_FILE =
    USER_PROMPT_FILE =
    PROG_OUTPUT_FILE =
    DATA_INPUT_FILE =
    DATA_OUTPUT_FILE =
    PROG_IOSTATE_FILE =
    SERVERALIVE_FILE =
    CLIENTALIVE_FILE = NULL;
  }
}

/****************************************************************************/
void CalculatorBase::ShowTheList(SimpleList<int>& x)
{
  if (!x.Size())
  {
    cout <<"{ null }" <<endl;
    return;
  }

  SimpleListIterator<int> Iter_(&x);
  cout <<"{ ";

  for (int i = 0; i < x.Size(); ++i)
    if (Iter_.IndexNode(i) && Iter_.IndexNode(i)->_Object)
    {
      cout <<(*Iter_.IndexNode(i)->_Object);
      if (i < x.Size() - 1)
        cout <<",";
    }

  cout <<" }";
  cout <<endl;
}

/****************************************************************************/
ChrString CalculatorBase::MakeTheList(SimpleList<int>& x)
{
  ChrString Result_;

  if (!x.Size())
  {
    Result_ = ChrString("{ null }");
    return Result_;
  }

  char Buffer_[32];
  SimpleListIterator<int> Iter_(&x);
  Result_ = "{ ";

  for (int i = 0; i < x.Size(); ++i)
    if (Iter_.IndexNode(i) && Iter_.IndexNode(i)->_Object)
    {
      IntToStr(*Iter_.IndexNode(i)->_Object, Buffer_);
      Result_ += ChrString(Buffer_);
      if (i < x.Size() - 1)
        Result_ += ChrString(",");
    }

  Result_ += ChrString(" }");
  return Result_;
}

/****************************************************************************/
int CalculatorBase::InSetNotation(const char* Str_)
{
  char* Res_ = ::NewString(Str_);
  char* ptr;
  char* tok;
  char* nptr;

  RemovePadding(Res_, " \t\n");
  size_t len = ::SafeStrLen(Res_);
  size_t plen;
  int i = 0;
  int ok = 0;
  int nextnum = 0;

  if (!len)
  {
    ::RawDeleteArray(Res_);
    Res_ = NULL;
    return 0;
  }

  if (Res_[i] == '{' && Res_[i+1] == ' ')
  {
    nextnum = 1;
    nptr = Res_ + 2;
    ptr = nptr ? strchr(nptr, ','):NULL;

    if (ptr)
    {
      *ptr = 0;
      tok = nptr;
      nptr = ptr+1;
    }
    else
    {
      tok = nptr;
      nptr = NULL;
    }

    plen = ::SafeStrLen(tok);

    if (!plen)
    {
      ::RawDeleteArray(Res_);
      Res_ = NULL;
      return ok;
    }
    else
      i = tok - Res_;

    while (i < len)
    {
      if (nextnum)
      {
        if (tok[plen-1] == '}')
        {
          tok[plen-2] = 0;
          i = len;
        }

        if (IsInt(tok))
          nextnum = 0;
        else
        {
          ok = strcmp(tok, "null") == 0;
          ::RawDeleteArray(Res_);
          Res_ = NULL;
          return ok;
        }
      }
      else if (i < len)
      {
        nextnum = 1;
        ptr = nptr ? strchr(nptr, ','):NULL;

        if (ptr)
        {
          *ptr = 0;
          tok = nptr;
          nptr = ptr+1;
        }
        else
        {
          tok = nptr;
          nptr = NULL;
        }

        plen = ::SafeStrLen(tok);

        if (!plen)
        {
          ::RawDeleteArray(Res_);
          Res_ = NULL;
          return ok;
        }
        else
          i = tok - Res_;
      }
    }
  }

  ::RawDeleteArray(Res_);
  Res_ = NULL;

  return (i == len);
}

/****************************************************************************/
Set<int>* CalculatorBase::MakeIntSet(const char* Str_)
{
  Set<int>* Set_ = new Set<int>;

  char* Res_ = ::NewString(Str_);
  char* ptr;
  char* tok;
  char* nptr;

  RemovePadding(Res_, " \t\n");
  size_t len = ::SafeStrLen(Res_);
  size_t plen;
  int i = 0;
  int ok = 0;
  int val;
  int nextnum = 0;

  if (!len)
  {
    ::RawDeleteArray(Res_);
    Res_ = NULL;
    return Set_;
  }

  if (Res_[i] == '{' && Res_[i+1] == ' ')
  {
    nextnum = 1;
    nptr = Res_ + 2;
    ptr = nptr ? strchr(nptr, ','):NULL;

    if (ptr)
    {
      *ptr = 0;
      tok = nptr;
      nptr = ptr+1;
    }
    else
    {
      tok = nptr;
      nptr = NULL;
    }

    plen = ::SafeStrLen(tok);

    if (!plen)
    {
      ::RawDeleteArray(Res_);
      Res_ = NULL;
      return Set_;
    }
    else
      i = tok - Res_;

    while (i < len)
    {
      if (nextnum)
      {
        if (tok[plen-1] == '}')
        {
          tok[plen-2] = 0;
          i = len;
        }

        if (strcmp(tok, "null") == 0)
        {
          ::RawDeleteArray(Res_);
          Res_ = NULL;
          return Set_;
        }
        else
        {
          val = atoi(tok);
          *Set_ |= &val;
          nextnum = 0;
        }
      }
      else if (i < len)
      {
        nextnum = 1;
        ptr = nptr ? strchr(nptr, ','):NULL;

        if (ptr)
        {
          *ptr = 0;
          tok = nptr;
          nptr = ptr+1;
        }
        else
        {
          tok = nptr;
          nptr = NULL;
        }

        plen = ::SafeStrLen(tok);

        if (!plen)
        {
          ::RawDeleteArray(Res_);
          Res_ = NULL;
          return Set_;
        }
        else
          i = tok - Res_;
      }
    }
  }

  ::RawDeleteArray(Res_);
  Res_ = NULL;
  return Set_;
}

/*****************************************************************************/
int CalculatorBase::InSetNotation(const char* list, int& NewStart_, int& LiteralLen_, int Start_, int len)
{
  int InSet_ = 0;
  int z1, z2, x;

  for (z1 = Start_; isspace(list[z1]) && z1 < len; z1++);
  if (list[z1] == '{')
  {
    for (z2 = len - 1; isspace(list[z2]) && z2 >= 0; z2--);
    if (list[z2] == '}')
    {
      z1++;
      x = z2 - z1;
      NewStart_ = z1;
      LiteralLen_ = x;
      InSet_ = (x >= 0) ? InSetNotation(list+z1):0;
      return InSet_;
    }
  }

  LiteralLen_ = 0;
  return FALSE;
}

/*****************************************************************************/
int CalculatorBase::IsNullSetStr(const char* Str_)
{
  if (!Str_)
    return 0;

  ChrString Res_ = Str_;
  Res_.RemovePadding(" \t\n");
  int Match_ = strcmp(Res_.c_str(), "{ null }") == 0;
  return Match_;
}

/*****************************************************************************/
void CalculatorBase::SetPlotGraph(int Flag_)
{
  _PlotGraph = Flag_;
  _GraphingOp = Flag_;
}

/*****************************************************************************/
void CalculatorBase::SetPlotAllPoints(int Flag_)
{
  _PlotAllPoints = Flag_;
}

/*****************************************************************************/
void CalculatorBase::SetGraphType(int Type_)
{
  _GraphType = Type_;
}

/*****************************************************************************/
int CalculatorBase::IsFraction(const char* Str_)
{
    const char* Ptr_ = strchr(Str_, '/');

    if (!Ptr_)
      return FALSE;

    ChrString Str1_ = &Ptr_[1];
    ChrString Str2_ = ChrString(Str_, Ptr_ - Str_);
    Str1_.RemovePadding(" \t\n");
    Str2_.RemovePadding(" \t\n");

    int ValidNum1_ = IsNumber(Str1_.c_str(), 1, 0, 0, 10) ||
                     IsFloat(Str1_.c_str()) ||
                     LongNumber::IsInSciNote(Str1_);
    int ValidNum2_ = IsNumber(Str2_.c_str(), 1, 0, 0, 10) ||
                     IsFloat(Str2_.c_str()) ||
                     LongNumber::IsInSciNote(Str2_);

    if (!ValidNum1_ || !ValidNum2_)
      return FALSE;

    return TRUE;
}

/*****************************************************************************
Procedure to check if symbol in expression is valid
*/
int CalculatorBase::HasOperandSymbols(char symbol, int HexOk_)
{
  return
  (
    symbol=='.' ||
    ((Uchar)symbol) == NAN_CONST ||
    ((Uchar)symbol) == INF_CONST ||
    ((Uchar)symbol) == PI_CONST ||
    ((Uchar)symbol) == E_CONST ||
    (HexOk_ ? IsHexc(symbol):IsDecc(symbol))
  );
}

/*****************************************************************************/
int CalculatorBase::HasOperandSymbols(char symbol)
{
  return HasOperandSymbols(symbol, GetCalcIndex() == CalculatorBase::LONGNUM);
}

/*****************************************************************************/
int CalculatorBase::IsConstSymbols(char symbol)
{
  return
  (
    ((Uchar)symbol) == E_CONST       ||
    ((Uchar)symbol) == PI_CONST      ||
    ((Uchar)symbol) == ANS_CONST     ||
    ((Uchar)symbol) == ANSSTR_CONST  ||
    ((Uchar)symbol) == NAN_CONST     ||
    ((Uchar)symbol) == INF_CONST     ||
    ((Uchar)symbol) == FRACANS_CONST
  );
}

/*****************************************************************************/
int CalculatorBase::IsConstString(const char* Str_, int Index_)
{
  const char* Ptr_ = &Str_[Index_];
  return
  (
    (icasestrcompn(Ptr_, "FRACANS", 7)  ||
     icasestrcompn(Ptr_, "ANSSTR", 6)   ||
     icasestrcompn(Ptr_, "ANS", 3)      ||
     icasestrcompn(Ptr_, "INF", 3)      ||
     icasestrcompn(Ptr_, "NAN", 3)      ||
     icasestrcompn(Ptr_, "PI", 2))      ||
        (icasestrcompn(Ptr_, "E", 1)    &&
         !HasHexPrefix(Str_, Index_))
  );
}

/*****************************************************************************/
int CalculatorBase::HasValidOpCodes(char symbol, int Base_)
{
  Uchar uch_ = (Uchar)symbol;

  if (Base_ != 10)
  {
    int ValidCodes_ =
    (
      uch_ == NOT_OP        ||
      uch_ == XOR_OP        ||
      uch_ == AND_OP        ||
      uch_ == LSTHANOREQ_OP ||
      uch_ == GRTHANOREQ_OP ||
      uch_ == EQUAL_OP      ||
      uch_ == NOTEQUAL_OP   ||
      uch_ == OR_OP         ||
      uch_ == ANS_CONST     ||
      uch_ == ANSSTR_CONST
    );

    if (ValidCodes_)
      return TRUE;
  }

  int AllCodes_ =
  (
    IsConstSymbols(symbol) ||
    IsBinaryOpCodes(symbol) ||
    IsUnaryOpCodes(symbol)
  );

  return ((Base_ != 10) ? !AllCodes_:AllCodes_);
}

/*****************************************************************************/
int CalculatorBase::IsBinaryOpCodes(char ch_)
{
  Uchar uch_ = (Uchar)ch_;
  return
  (
    uch_ == PERMREP_OP    ||
    uch_ == COMBREP_OP    ||
    uch_ == COMB_OP       ||
    uch_ == PERM_OP       ||
    uch_ == XOR_OP        ||
    uch_ == AND_OP        ||
    uch_ == LSTHANOREQ_OP ||
    uch_ == GRTHANOREQ_OP ||
    uch_ == EQUAL_OP      ||
    uch_ == NOTEQUAL_OP   ||
    uch_ == OR_OP         ||
    uch_ == SCI_OP
  );
}

/*****************************************************************************/
int CalculatorBase::IsBinaryFunctions(const char* Str_, int Index_)
{
  const char* Ptr_ = &Str_[Index_];
  return
  (
    (icasestrcompn(Ptr_, "COMBREP", 7)  ||
     icasestrcompn(Ptr_, "PERMREP", 7)  ||
     icasestrcompn(Ptr_, "COMB", 4)     ||
     icasestrcompn(Ptr_, "PERM", 4)     ||
     icasestrcompn(Ptr_, "XOR", 3)      ||
     icasestrcompn(Ptr_, "AND", 3)      ||
     icasestrcompn(Ptr_, "<=", 2)       ||
     icasestrcompn(Ptr_, ">=", 2)       ||
     icasestrcompn(Ptr_, "==", 2)       ||
     icasestrcompn(Ptr_, "!=", 2)       ||
     icasestrcompn(Ptr_, "OR", 2))      ||
        (icasestrcompn(Ptr_, "E", 1)    &&
         !HasHexPrefix(Str_, Index_))
  );
}

/*****************************************************************************/
int CalculatorBase::IsUnaryOpCodes(char ch_)
{
  Uchar uch_ = (Uchar)ch_;
  return
  (
    uch_ == FRACPART_OP ||
    uch_ == INTPART_OP  ||
    uch_ == CUBERT_OP   ||
    uch_ == FLOOR_OP    ||
    uch_ == LOG10_OP    ||
    uch_ == ASIN_OP     ||
    uch_ == ACOS_OP     ||
    uch_ == ATAN_OP     ||
    uch_ == COSH_OP     ||
    uch_ == TANH_OP     ||
    uch_ == CEILING_OP  ||
    uch_ == SQRT_OP     ||
    uch_ == CUBE_OP     ||
    uch_ == RAND_OP     ||
    uch_ == SIN_OP      ||
    uch_ == COS_OP      ||
    uch_ == TAN_OP      ||
    uch_ == RND_OP      ||
    uch_ == EXP_OP      ||
    uch_ == LOG_OP      ||
    uch_ == ABS_OP      ||
    uch_ == SQR_OP      ||
    uch_ == INV_OP      ||
    uch_ == NOT_OP      ||
    uch_ == FACTORIAL_OP
  );
}

/*****************************************************************************/
int CalculatorBase::IsUnaryFunctions(const char* Str_)
{
  return
  (
    icasestrcompn(Str_, "FRACPART", 8) ||
    icasestrcompn(Str_, "INTPART", 7)  ||
    icasestrcompn(Str_, "CUBERT", 6)   ||
    icasestrcompn(Str_, "FLOOR", 5)    ||
    icasestrcompn(Str_, "LOG10", 5)    ||
    icasestrcompn(Str_, "ASIN", 4)     ||
    icasestrcompn(Str_, "ACOS", 4)     ||
    icasestrcompn(Str_, "ATAN", 4)     ||
    icasestrcompn(Str_, "SINH", 4)     ||
    icasestrcompn(Str_, "COSH", 4)     ||
    icasestrcompn(Str_, "TANH", 4)     ||
    icasestrcompn(Str_, "CEIL", 4)     ||
    icasestrcompn(Str_, "SQRT", 4)     ||
    icasestrcompn(Str_, "CUBE", 4)     ||
    icasestrcompn(Str_, "RAND", 4)     ||
    icasestrcompn(Str_, "SIN", 3)      ||
    icasestrcompn(Str_, "COS", 3)      ||
    icasestrcompn(Str_, "TAN", 3)      ||
    icasestrcompn(Str_, "RND", 3)      ||
    icasestrcompn(Str_, "EXP", 3)      ||
    icasestrcompn(Str_, "LOG", 3)      ||
    icasestrcompn(Str_, "ABS", 3)      ||
    icasestrcompn(Str_, "SQR", 3)      ||
    icasestrcompn(Str_, "INV", 3)      ||
    icasestrcompn(Str_, "NOT", 3)
  );
}

/*****************************************************************************/
int CalculatorBase::IsCalledFunctions(const char* Str_)
{
  int SetOpFncs_ =
    icasestrcompn(Str_, "SETUNI", 6)         ||
    icasestrcompn(Str_, "SETINT", 6)         ||
    icasestrcompn(Str_, "SETDIF", 6)         ||
    icasestrcompn(Str_, "SETXDIF", 7);

  return
  (
    icasestrcompn(Str_, "MIN", 3)            ||
    icasestrcompn(Str_, "MAX", 3)            ||
    icasestrcompn(Str_, "DERIV", 5)          ||
    icasestrcompn(Str_, "DERIV2", 6)         ||
    icasestrcompn(Str_, "INTEG", 5)          ||
    icasestrcompn(Str_, "FMIN", 4)           ||
    icasestrcompn(Str_, "FMAX", 4)           ||
    icasestrcompn(Str_, "ROUND", 5)          ||
    icasestrcompn(Str_, "SOLVE", 5)          ||
    icasestrcompn(Str_, "INTERSECT", 9)      ||
    icasestrcompn(Str_, "COMBREP", 7)        ||
    icasestrcompn(Str_, "PERMREP", 7)        ||
    icasestrcompn(Str_, "COMB", 4)           ||
    icasestrcompn(Str_, "PERM", 4)           ||
    icasestrcompn(Str_, "ITEMCOMBREP", 11)   ||
    icasestrcompn(Str_, "ITEMPERMREP", 11)   ||
    icasestrcompn(Str_, "ITEMCOMB", 8)       ||
    icasestrcompn(Str_, "ITEMPERM", 8)       ||
    icasestrcompn(Str_, "FRAC", 4)           ||
    SetOpFncs_                               ||
    (icasestrcompn(Str_, "SET", 3) &&
     !SetOpFncs_)                            ||
    icasestrcompn(Str_, "SUM", 3)            ||
    icasestrcompn(Str_, "PROD", 4)           ||
    icasestrcompn(Str_, "STDDEV", 6)         ||
    icasestrcompn(Str_, "AVGMODE", 7)        ||
    icasestrcompn(Str_, "AVGMEAN", 7)        ||
    icasestrcompn(Str_, "AVGMED", 6)         ||
    icasestrcompn(Str_, "RECTOPOLR", 9)      ||
    icasestrcompn(Str_, "RECTOPOLA", 9)      ||
    icasestrcompn(Str_, "POLTORECX", 9)      ||
    icasestrcompn(Str_, "POLTORECY", 9)      ||
    icasestrcompn(Str_, "EVAL", 4)           ||
    icasestrcompn(Str_, "SHOW", 4)           ||
    icasestrcompn(Str_, "SHOWSET", 7)        ||
    icasestrcompn(Str_, "FROMHEX", 7)        ||
    icasestrcompn(Str_, "FROMOCT", 7)        ||
    icasestrcompn(Str_, "FROMBIN", 7)        ||
    icasestrcompn(Str_, "FROMCMP2BIN", 11)   ||
    icasestrcompn(Str_, "TOHEX", 5)          ||
    icasestrcompn(Str_, "TOOCT", 5)          ||
    icasestrcompn(Str_, "TOBIN", 5)          ||
    icasestrcompn(Str_, "TOCMP2BIN", 9)
  );
}

/*****************************************************************************/
int CalculatorBase::IsBinaryOperators(char symbol)
{
  return
  (
    symbol=='-' ||
    symbol=='+' ||
    symbol=='*' ||
    symbol=='/' ||
    symbol=='@' ||
    symbol=='^' ||
    symbol=='<' ||
    symbol=='>'
  );
}

/*****************************************************************************/
int CalculatorBase::IsUnaryOperators(char symbol)
{
  return
  (
    symbol=='-' ||
    symbol=='+'
  );
}

/*****************************************************************************/
int CalculatorBase::IsPostfixOperators(char symbol)
{
  return (symbol=='%');
}

/*****************************************************************************/
int CalculatorBase::IsUnprocessedSymbols(char symbol)
{
  return
  (
    symbol=='(' ||
    symbol==')' ||
    symbol=='[' ||
    symbol==']' ||
    symbol=='{' ||
    symbol=='}' ||
    symbol==',' ||
    isspace(symbol)
  );
}

/****************************************************************************/
const char* CalculatorBase::GetLogFilename()
{
  return LOG_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetUserInputFilename()
{
  return USER_INPUT_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetUserPromptFilename()
{
  return USER_PROMPT_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetProgramOutputFilename()
{
  return PROG_OUTPUT_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetDataInputFilename()
{
  return DATA_INPUT_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetDataOutputFilename()
{
  return DATA_OUTPUT_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetProgramIostateFilename()
{
  return PROG_IOSTATE_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetCurrentGraphFilename()
{
  return CURRENTGRAPH_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetClientAliveFilename()
{
  return CLIENTALIVE_FILE;
}

/****************************************************************************/
const char* CalculatorBase::GetServerAliveFilename()
{
  return SERVERALIVE_FILE;
}

/****************************************************************************/
void CalculatorBase::ResetRetryRead()
{
  _RetryReadCount = 0;
}

/****************************************************************************/
void CalculatorBase::SetIoState(int State_)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: SetIoState\n", logfile);
  #endif

  #if IOSTATE_PRE_SET_DELAY
    ::usleep(IOSTATE_DELAY_LENGTH * 1000);
  #endif

  #if CALCLIB_DEBUG1
    printf("opening %s for writing 1st attempt, server side\n", PROG_IOSTATE_FILE);
  #endif

  if (!iostatefile || !filestatus[Mcalc_IOType::IoState])
  {
    filestatus[Mcalc_IOType::IoState] = openfile(&files[Mcalc_IOType::IoState],PROG_IOSTATE_FILE,"w",TRUE);
    while (!filestatus[Mcalc_IOType::IoState])
    {
	#if (!defined(__DJGPP__) & !defined(__linux__) & !defined(__unix__) & !defined(__GNUC__))
	  Sleep(IOSTATE_DELAY_LENGTH * 1000);
	#else
      ::usleep(IOSTATE_DELAY_LENGTH * 1000);
	#endif

      #if CALCLIB_DEBUG1
        printf("Error: opening %s for writing, server side\n", PROG_IOSTATE_FILE);
      #endif

      filestatus[Mcalc_IOType::IoState] = openfile(&files[Mcalc_IOType::IoState],PROG_IOSTATE_FILE,"w",TRUE);
    }

    iostatefile = files[Mcalc_IOType::IoState];
  }

  #if (CALCLIB_DEBUG1 | CALCLIB_DEBUG3)
    printf("opening %s for writing successful, server side\n", PROG_IOSTATE_FILE);
  #endif

  fprintf(iostatefile, "%d", State_);
  fclose(iostatefile);
  iostatefile = NULL;
  filestatus[Mcalc_IOType::IoState] = 0;
  _IoState = State_;

  #if IOSTATE_POST_SET_DELAY
    ::usleep(IOSTATE_DELAY_LENGTH * 1000);
  #endif

  #if CALCLIB_DEBUG2
    fputs("Leaving: SetIoState\n", logfile);
  #endif
}

/****************************************************************************/
void CalculatorBase::PollClientIfAlive()
{
    int State_;
    char buffer[20];

    // poll to check if calculator client is alive here
    if (_IostateFileReadError || _KeepAlivePollError)
    {
      if (_KeepAlivePollValue == Mcalc_KeepAliveState::IDLE_STATE)
      {
        filestatus[Mcalc_IOType::ClientAlive] = openfile(&files[Mcalc_IOType::ClientAlive],CLIENTALIVE_FILE,"w");
        if (filestatus[Mcalc_IOType::ClientAlive])
        {
          fprintf(files[Mcalc_IOType::ClientAlive], "%d", Mcalc_KeepAliveState::SERVER_TO_CLIENT);
          fclose(files[Mcalc_IOType::ClientAlive]);
          filestatus[Mcalc_IOType::ClientAlive] = 0;
          _KeepAlivePollValue = Mcalc_KeepAliveState::SERVER_TO_CLIENT;
        }
      }
      else if (_KeepAlivePollValue == Mcalc_KeepAliveState::SERVER_TO_CLIENT)
      {
        filestatus[Mcalc_IOType::ClientAlive] = openfile(&files[Mcalc_IOType::ClientAlive],CLIENTALIVE_FILE,"r");
        if (filestatus[Mcalc_IOType::ClientAlive])
        {
          State_ = atoi(fgets(buffer, 20, files[Mcalc_IOType::ClientAlive]));
          fclose(files[Mcalc_IOType::ClientAlive]);
          filestatus[Mcalc_IOType::ClientAlive] = 0;

          if (State_ != Mcalc_KeepAliveState::CLIENT_ACKNOWLEDGED)
          {
            _RetryPollCount++;
            _KeepAlivePollError = TRUE;

            if (_RetryPollCount == RETRY_READ_MAX)
            {
              DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_CLIENTNOTFOUND, ERRMSG_CLIENTNOTFOUND);
              exit(1);
            }
          }
          else
          {
            _RetryPollCount = 0;
            _KeepAlivePollError = FALSE;
            _KeepAlivePollValue = Mcalc_KeepAliveState::IDLE_STATE;
          }
        }
      }
    }
}

/****************************************************************************/
void CalculatorBase::ReceiveKeepAlivePoll()
{
  int x;

  if (!filestatus[Mcalc_IOType::ServerAlive] || !files[Mcalc_IOType::ServerAlive])
  {
    filestatus[Mcalc_IOType::ServerAlive] = openfile(&files[Mcalc_IOType::ServerAlive],SERVERALIVE_FILE,"r",TRUE);
    for (x = 0; x < RETRY_READ_MAX && !filestatus[Mcalc_IOType::ServerAlive]; x++)
    {
	#if (!defined(__DJGPP__) & !defined(__linux__) & !defined(__unix__) & !defined(__GNUC__))
	  Sleep(IOSTATE_DELAY_LENGTH * 1000);
	#else
      ::usleep(IOSTATE_DELAY_LENGTH * 1000);
	#endif
      filestatus[Mcalc_IOType::ServerAlive] = openfile(&files[Mcalc_IOType::ServerAlive],SERVERALIVE_FILE,"r",TRUE);
    }

    if (files[Mcalc_IOType::ServerAlive])
    {
      fprintf(files[Mcalc_IOType::ServerAlive], "%d", Mcalc_KeepAliveState::SERVER_ACKNOWLEDGED);
      fclose(files[Mcalc_IOType::ServerAlive]);
      filestatus[Mcalc_IOType::ClientAlive] = 0;
    }
  }
}

/****************************************************************************/
int CalculatorBase::GetIoState(int Sleep_)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: GetIoState\n", logfile);
  #endif

  #if IMPLEMENT_KEEP_ALIVE
    ReceiveKeepAlivePoll();
  #endif

  if (Sleep_)
  {
    #if IMPLEMENT_KEEP_ALIVE
      PollClientIfAlive();
    #endif
	#if (!defined(__DJGPP__) & !defined(__linux__) & !defined(__unix__) & !defined(__GNUC__))
	  Sleep(IOSTATE_DELAY_LENGTH * 1000);
	#else
      ::usleep(IOSTATE_DELAY_LENGTH * 1000);
	#endif

    #if CALCLIB_DEBUG1
      printf("Error: opening %s for reading, server side\n", PROG_IOSTATE_FILE);
    #endif
  }
  else
  {
    #if CALCLIB_DEBUG1
      printf("opening %s for reading 1st attempt, server side\n", PROG_IOSTATE_FILE);
    #endif
  }

  int Eof_;
  char buffer[20];

  int State_ = Mcalc_IOState::IDLE_STATE;
  filestatus[Mcalc_IOType::IoState] = 0;
  iostatefile = NULL;

  files[Mcalc_IOType::IoState] = FileOpen(PROG_IOSTATE_FILE, "r");
  int ErrCond_ = files[Mcalc_IOType::IoState] == NULL ||
                 (Eof_ = feof(files[Mcalc_IOType::IoState])) ||
                 ferror(files[Mcalc_IOType::IoState]);

  if (ErrCond_ || !FileSize(files[Mcalc_IOType::IoState]))
  {
    if (ErrCond_)
    {
      if (files[Mcalc_IOType::IoState])
      {
        clearerr(files[Mcalc_IOType::IoState]);
        if (Eof_)
          fclose(files[Mcalc_IOType::IoState]);

        files[Mcalc_IOType::IoState] = NULL;
      }
    }
    else
      fclose(files[Mcalc_IOType::IoState]);

    #if LIMIT_READ_RETRIES
      if (_IostateFileReadError)
      {
        _RetryReadCount++;
        if (_RetryReadCount == RETRY_READ_MAX)
        {
          DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_OPENSTATEFILE, ERRMSG_OPENSTATEFILE);
          exit(1);
        }
      }
    #endif

    State_ = Mcalc_IOState::IDLE_STATE;
    _IostateFileReadError = TRUE;
  }
  else
  {
    State_ = atoi(fgets(buffer, 20, files[Mcalc_IOType::IoState]));
    fclose(files[Mcalc_IOType::IoState]);

    _IostateFileReadError = FALSE;
    ResetRetryRead();
  }

  #if CALCLIB_DEBUG2
    fputs("Leaving: GetIoState\n", logfile);
  #endif

  _IoState = State_;
  return State_;
}

/****************************************************************************/
int CalculatorBase::CurrentIoState() const
{
  return _IoState;
}

/****************************************************************************/
long CalculatorBase::FileSize(FILE* Fptr_)
{
  if (Fptr_)
  {
    fseek(Fptr_, 0, SEEK_END);
    long Pos_ = ftell(Fptr_);
    rewind(Fptr_);
    return Pos_;
  }

  return 0;
}

/****************************************************************************/
void CalculatorBase::RemoveSessionFiles(int SesNum_)
{
  // Async. mode data files
  if (USER_INPUT_FILE)
    unlink(USER_INPUT_FILE);

  if (USER_PROMPT_FILE)
    unlink(USER_PROMPT_FILE);

  if (PROG_OUTPUT_FILE)
    unlink(PROG_OUTPUT_FILE);

  if (PROG_IOSTATE_FILE)
    unlink(PROG_IOSTATE_FILE);

  // session graph files
  char Fname_[32];
  char suf;

  for (suf = 'a'; suf <= 'z'; suf++)
  {
    sprintf(Fname_, "mcalc_graph%d%c.out", SesNum_, suf);
    unlink(Fname_);
  }

  if (CURRENTGRAPH_FILE)
  {
    unlink(CURRENTGRAPH_FILE);
    MemMatrix::Matrix().Deallocate(CURRENTGRAPH_FILE);
    CURRENTGRAPH_FILE = NULL;
  }

  // server/client polling files
  if (SERVERALIVE_FILE)
    unlink(SERVERALIVE_FILE);

  if (CLIENTALIVE_FILE)
    unlink(CLIENTALIVE_FILE);
}

/****************************************************************************/
void CalculatorBase::AddFunctionName(const char* FncName_)
{
  if (_NumFncNames >= MAXFUNCTIONS)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARRAYBOUNDS, "%s", ERRMSG_ARRAYBOUNDS);
    exit(1);
  }
  else
  {
    FUNCTION_NAMES[_NumFncNames] = ::NewString(FncName_);

    if (_ShortestFncName < 0)
      _ShortestFncName = strlen(FUNCTION_NAMES[_NumFncNames]);
    else
    {
      int len = strlen(FUNCTION_NAMES[_NumFncNames]);
      if (len < _ShortestFncName)
        _ShortestFncName = len;
    }

    _NumFncNames++;
  }
}

/****************************************************************************/
int CalculatorBase::UsesSessionNum()
{
  return _UseSessionCnt;
}

/****************************************************************************/
int CalculatorBase::SessionNum()
{
  return _SessionNum;
}

/****************************************************************************/
int CalculatorBase::InAsyncMode()
{
  return _AsyncMode;
}

/****************************************************************************/
int CalculatorBase::WaitForResponse() const
{
  return _WaitForResponse;
}

/****************************************************************************/
void CalculatorBase::SetWaitForResponse(int Flag_)
{
  _WaitForResponse = Flag_;
}

/****************************************************************************/
void CalculatorBase::SetUsesSessionNum(int Num_)
{
  if (files && _UseSessionCnt && !Num_ && _AsyncMode)
    SetAsyncMode(0);

  _UseSessionCnt = Num_;
}

/****************************************************************************/
void CalculatorBase::SetSessionNum(int Num_)
{
  _SessionNum = Num_;
}

/****************************************************************************/
void CalculatorBase::SetAsyncMode(int Flag_)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: SetAsyncMode\n", logfile);
  #endif

  _AsyncMode = Flag_;

  if (UsesSessionNum())
  {
    if (!Flag_)
    {
      if (files)
      {
        if (filestatus[Mcalc_IOType::UserInput] || userinputfile)
        {
          fclose(files[Mcalc_IOType::UserInput]);
          files[Mcalc_IOType::UserInput] = NULL;
          userinputfile = NULL;
          filestatus[Mcalc_IOType::UserInput] = 0;
        }

        if (files[Mcalc_IOType::UserPrompt] || userpromptfile)
        {
          fclose(files[Mcalc_IOType::UserPrompt]);
          files[Mcalc_IOType::UserPrompt] = NULL;
          userpromptfile = NULL;
          filestatus[Mcalc_IOType::UserPrompt] = 0;
        }

        if (filestatus[Mcalc_IOType::ProgramOutput] || programoutputfile)
        {
          fclose(files[Mcalc_IOType::ProgramOutput]);
          files[Mcalc_IOType::ProgramOutput] = NULL;
          programoutputfile = NULL;
          filestatus[Mcalc_IOType::ProgramOutput] = 0;
        }

        if (filestatus[Mcalc_IOType::IoState] || iostatefile)
        {
          fclose(files[Mcalc_IOType::IoState]);
          files[Mcalc_IOType::IoState] = NULL;
          iostatefile = NULL;
          filestatus[Mcalc_IOType::IoState] = 0;
        }

        outfile = GetOutputFile();
      }
    }
    else
    {
      _StdInput = FALSE;
      _StdOutput = FALSE;
    }
  }

  #if CALCLIB_DEBUG2
    fputs("Leaving: SetAsyncMode\n", logfile);
  #endif
}

/****************************************************************************/
int CalculatorBase::NumFncNames()
{
  return _NumFncNames;
}

/****************************************************************************/
int CalculatorBase::ShortestFncName()
{
  return _ShortestFncName;
}

/**************************** Memory Procedures *****************************/
/*****************************************************************************
Procedure to free an operand structure
PASSED:
  optr : the pointer to the operand
USES:
  uses clearvalue to free memory from the value array and assign it to NULL
       the array size is set to 0
CHANGED:
  memory for the value array within the operand is released
  memory for the operand is released
*/
void CalculatorBase::freeoperand(Mcalc_Operand** optr)
{
  clearvalue(*optr);
  MemMatrix::Matrix().Deallocate(*optr);
  *optr = NULL;
}

/*****************************************************************************
Procedure to allocate memory to an operand type variable
PASSED:
  optr : the pointer to the operand variable
  items : the number of items of operand type size to allocate
USES:
  uses allocmem to allocate memory
CHANGED:
  optr changed to point to memory in multiples of Mcalc_Operand size
  the field of the all newly allocated operands are set to 0
RETURNED:
  returns the newly allocated operand variable
*/
Mcalc_Operand* CalculatorBase::allocateoperand(Mcalc_Operand** optr, int items)
{
  int i;

  *optr = (Mcalc_Operand*)allocmem(items,sizeof(Mcalc_Operand),*optr);
  for (i=0;i<items;i++)
  {
    (*optr)[i].size = 0;
    (*optr)[i].sign = Mcalc_SignType::Plus;
    (*optr)[i].value = NULL;
  }

  return *optr;
}

/*****************************************************************************
Procedure to free the value array of the operand variable
PASSED:
  ptr : pointer to the operand variable
CHANGED:
  the memory of the value array of the pointer is freed and assigned to NULL
  the size is assigned to 0
RETURNED:
  returns the pointer to the variable with the freed value array
*/
Mcalc_Operand* CalculatorBase::clearvalue(Mcalc_Operand* ptr)
{
  MemMatrix::Matrix().Deallocate(ptr->value);
  ptr->value = NULL;
  ptr->size = 0;
  return ptr;
}

/*****************************************************************************
Procedure to allocate memory to the value array in an operand structure
PASSED:
  ptr : pointer to an operand structure
  size : size to be allocated to the value array
USES:
  uses allocmem to allocate memory
  uses clearvalue to free memory from the value array and assign it to NULL
       the array size is set to 0
READ:
  size is read for the amount of memory to be allocated to the value array
CHANGED:
  ptr is allocated with the amount of memory specified by size
RETURNED:
  returns the pointer to the variable with the newly allocated value array
*/
Mcalc_Operand* CalculatorBase::allocatevalue(Mcalc_Operand* ptr, int size)
{
  ptr = clearvalue(ptr);
  ptr->size = size;
  ptr->value = (char*)allocmem(size+1,sizeof(char),ptr->value);
  return ptr;
}

/*****************************************************************************
Procedure to reallocate memory to the value array in an operand structure
PASSED:
  ptr : pointer to an operand structure
  size : size to be allocated to the value array
USES:
  uses reallocmem to reallocate memory
READ:
  size is read for the amount of memory to be allocated to the value array
CHANGED:
  ptr is allocated with the amount of memory specified by size
RETURNED:
  returns the pointer to the variable with the newly allocated value array
*/
Mcalc_Operand* CalculatorBase::reallocatevalue(Mcalc_Operand* ptr, int size)
{
  ptr->value = (char*)reallocmem(size+1,sizeof(char),ptr->value);
  for (;ptr->size<size;ptr->size++)
    ptr->value[ptr->size] = 0;

  ptr->value[size] = 0;
  ptr->size = size;
  return ptr;
}

/************************ Argument Checking Procedures **********************/
/*****************************************************************************
Procedure to test if string value passed is a valid value for the calculator
program to handle
PARAMETERS:
  char* value : the string value passed
RETURNED:
  returns true if the string represents a valid numeric value for the program
  , otherwise returns false
*/
int CalculatorBase::IsValidNum(const char* value)
{
  return (IsNumber(value,1,1,1,10) ||
         (GetCalcIndex() == CalculatorBase::LONGNUM &&
           (IsNumber(value,1,0,0,8) || IsNumber(value,1,0,0,16))));
}

/********************** String Manipulation Procedures **********************/
/*****************************************************************************
Procedure to separate a string of indefinite length into individual word
tokens by replacing whitespace characters with NULL characters
PASSED:
  buffer : the string buffer
CHANGED:
  the whitespace characters within the string buffer is replaced with NULL
  characters
RETURNED:
  returns the number of words within the string buffer
*/
int CalculatorBase::stringtoken(char* buffer)
{
  int wordcount; /* counter for words within the string */

  for (wordcount=0;*buffer;buffer++)
    if (isspace(*buffer))
    {
      ++wordcount;
      *buffer = '\0';
      for (;buffer[1]&&isspace(buffer[1]);buffer++);
      wordcount -= !buffer[1];
    }

  return wordcount+1;
}

/*****************************************************************************
Procedure to strip individual words of indefinite length from a string
which has been separated with NULL characters.
PASSED:
  count : the word count
  buffer : the string buffer
  list : array of strings
USES
  uses allocmem to allocate memory to the strings in the list string array
READ:
  buffer is read for the words separated by NULL characters
CHANGED:
  list is changed to contain the individual words that are separated from the
  string buffer
*/
void CalculatorBase::stripstring(int count, char* buffer, char** list)
{
  int i;
  int len;     /* length of the word */

  for (i=0;i<count;buffer++,i++)
  {
    len = strlen(buffer);

    if (len)
    {
      list[i] = (char*)allocmem(len+1,sizeof(char),list[i]);
      strcpy(list[i],buffer);
      buffer += len; /* move buffer to NULL character */

      if (i<count-1)
        for (;buffer[1]&&isspace(buffer[1]);buffer++); /* skip leading spaces */
    }
  }
}

/*****************************************************************************
Procedure to clear leading zeros from a string and return the new string
PASSED:
  buffer : the string buffer
USES:
  uses allocmem to allocate memory for the new string buffer
CHANGED:
  buffer has all leading zeros from it cleared.
RETURNED:
  returns the new string with leading zeros cleared from it
*/
char* CalculatorBase::clearleadingspaces(char** buffer)
{
  int index; /* string index */
  char* ptr = NULL;       /* temporary string used for copying the modified string */

  for (index=0;(*buffer)[index]&&isspace((*buffer)[index]);index++);
  ptr = (char*)allocmem(strlen(*buffer+index)+1,sizeof(char),ptr);
  strcpy(ptr,*buffer+index);
  MemMatrix::Matrix().Deallocate(*buffer);
  *buffer = ptr;
  ptr = NULL;
  return *buffer;
}

/****************************************************************************/
int CalculatorBase::InQuotes(const char* str, int& Equal_, int& FirstQuote_, int& LastQuote_)
{
  int Found_ = 0;
  int x, len;

  len = strlen(str);

  if (!len)
    return 0;

  for (x = len - 1; x; x--)
    if (Found_ = str[x] == '\"')
    {
      LastQuote_ = x;
      break;
    }
    else if (!isspace(str[x]))
    {
      LastQuote_ = -1;
      break;
    }

  if (Found_)
  {
    for (--x; x >= 0; x--)
      if (Found_ = str[x] == '\"')
      {
        FirstQuote_ = x;
        break;
      }
  }
  else
    LastQuote_ = -1;

  if (Found_)
  {
    for (--x; x >= 0; x--)
      if (Found_ = str[x] == '=')
      {
        Equal_ = x;
        break;
      }
      else if (!isspace(str[x]))
      {
        if (x >= 4 && toupper(str[x]) == 'H')
        {
          if (icasestrcompn(&str[x-4], "GRAPH", 5))
            x -= 4;
          else
          {
            FirstQuote_ = -1;
            break;
          }
        }
        else
        {
          FirstQuote_ = -1;
          break;
        }
      }
  }
  else
    FirstQuote_ = -1;

  if (!Found_)
    Equal_ = -1;

  return (FirstQuote_ >= 0 && LastQuote_ > 0);
}

/*****************************************************************************
Procedure to strip white space from a string
PARAMETERS:
  char* str : the string which has white space characters to be stripped
*/
void CalculatorBase::stripws(char* str)
{
  int i, x;
  int FirstQuote_ = 0;
  int LastQuote_ = 0;
  int Equal_ = 0;
  int len = strlen(str);

  if (InQuotes(str, Equal_, FirstQuote_, LastQuote_))
  {
    for (x = len - 1; x > LastQuote_; x--)
      str[x] = 0;

    if (Equal_ < 0)
    {
      len = strlen(str);
      i = FirstQuote_;

      for (x = 0; i <= len; i++)
        str[x++] = str[i];
    }
    else
    {
      for (i = x = 0; i < FirstQuote_; i++)
        if (!isspace(str[i]))
          str[x++] = str[i];

      while (str[i])
        str[x++] = str[i++];
      str[x] = 0;
    }
  }
  else
  {
    for (i = 0, x = 0; i < strlen(str); i++)
    {
      for (;i < strlen(str) && isspace(str[i]); i++);
      str[x++] = str[i];
    }
    str[x] = '\0';
  }
}

/*****************************************************************************
Procedure to reverse the characters in a string
PARAMETERS:
  char* str : the string to reverse
  int len : the length of the string
*/
void CalculatorBase::reversestr(char* str, int len)
{
  int end = len - 1,
      sta = 0;
  char temp;

  for (; sta < end; sta++, end--)
  {
    temp = str[sta];
    str[sta] = str[end];
    str[end] = temp;
  }
}

/****************************************************************************/
Uchar* CalculatorBase::ToUchar(Uchar* str, char ch)
{
  str[0] = ch;
  str[1] = 0;
  return str;
}

/****************************************************************************/
Uchar* CalculatorBase::ToUchar(Uchar* Dest_, const char* Src_)
{
  strcpy((char*)Dest_, Src_);
  return Dest_;
}

/*************************** File I/O Procedures ****************************/
/****************************************************************************/
int CalculatorBase::FileStreamOpen(ifstream& Fin_, const char* Fname_)
{
  PATHTOKENIZER(Pathtok_, "PATH");
  size_t Max_ = Pathtok_.Max();
  size_t i;

  char* AbsPath_ = new_char_array(FILE_PATH_SIZE + strlen(Fname_), Fname_);
  Fin_.open(AbsPath_, ios::in);

  for (i = 0; !Fin_.good() && i < Max_; i++)
  {
    strcpy(AbsPath_, Pathtok_[i]);
    strcat(AbsPath_, Pathtok_.DirSeparator());
    strcat(AbsPath_, Fname_);
    Fin_.clear();
    Fin_.open(AbsPath_, ios::in);
  }

  ::DeleteArray(AbsPath_);
  return Fin_.good();
}

/****************************************************************************/
FILE* CalculatorBase::FileOpen(const char* Fname_, const char* Mode_)
{
  PATHTOKENIZER(Pathtok_, "PATH");
  size_t Max_ = Pathtok_.Max();
  size_t i;

  char* AbsPath_ = new_char_array(FILE_PATH_SIZE + strlen(Fname_), Fname_);
  FILE* Fptr_ = fopen(AbsPath_, Mode_);

  for (i = 0; (!Fptr_ || ferror(Fptr_)) && i < Max_; i++)
  {
    strcpy(AbsPath_, Pathtok_[i]);
    strcat(AbsPath_, Pathtok_.DirSeparator());
    strcat(AbsPath_, Fname_);
    if (Fptr_)
      clearerr(Fptr_);
    Fptr_ = fopen(AbsPath_, Mode_);
  }

  ::DeleteArray(AbsPath_);
  return Fptr_;
}

/*****************************************************************************
Procedure to open files for reading or writing
PASSED:
  file : the address of the file pointer
  name : the name of the file
  mode : the mode to open the file to: read, write
READ:
  name is read for the name of the file
  mode is read for the mode of the file
CHANGED:
  file is changed to hold a valid file location
RETURNED:
  returns 1 on success, otherwise program is terminated
*/
int CalculatorBase::openfile(FILE** file, const char* name, const char* mode, int Retry_)
{
  const char* modemessage = (strcmp(mode, "r") == 0) ? MSG_FILEREAD:MSG_FILEWRITE;

  if ((*file = FileOpen(name,mode))==NULL)
  {
    if (Retry_)
      return 0;

    char Buffer_[256];
    sprintf(Buffer_, ERRMSG_FILE_ERROR, name, modemessage);
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_FILE_ERROR, Buffer_);
    exit(1);
  }

  return 1;
}

/*****************************************************************************
Procedure to read a string from a file increasing the size of the string
buffer as necessary
PASSED:
  buffer : the string buffer
  terminator : termination character to end the string
USES:
  uses allocmem to allocate memory to the string buffer
  uses reallocmem to reallocate memory to the string buffer
CHANGED:
  the string buffer is allocated to the memory size necessary to store the
  string read in from the input file. the string buffer is then stored with
  the string read from the input file.
RETURNED:
  returns end of file status : 1 if end of file, 0 otherwise.
*/
int CalculatorBase::freadstring(char** buffer, char terminator, FILE* Finput_)
{
  int eol,           /* end of line flag */
      eof;           /* end of file flag */
  int i;
  int length = MAXLINE; /* length of string buffer */
  int InQuote_ = 0;

  *buffer = (char*)allocmem(length+1,sizeof(char),*buffer);
  if (!(*buffer))
    return 0;

  /* copy characters from the file while not newline or EOF */
  /* increase buffer by increments of 1 line */
  for (i = eol = eof = 0; !eol; i++)
  {
    if (i==length)
    {
      length += MAXLINE;
      *buffer = (char*)reallocmem(length+1,sizeof(char),*buffer);
    }

    (*buffer)[i] = fgetc(Finput_);
    if (isspace((*buffer)[i]))
      (*buffer)[i] = ' ';
    else
    {
      if ((*buffer)[i] == '\"')
        if (InQuote_)
          --InQuote_;
        else
          ++InQuote_;
    }

    eof = (*buffer)[i]==EOF;
    eol = ((*buffer)[i]==terminator && !InQuote_) || eof;
  }

  length = i;
  (*buffer)[length-1] = '\0';

  return eof;
}

/*********************** Stack Operations Procedures ************************/
/*****************************************************************************
Procedure to push characters on to the stack
*/
void CalculatorBase::push(Mcalc_Stack** stk, Mcalc_StackData str)
{
  Mcalc_Stack* srec = NULL;

  srec = (Mcalc_Stack*)allocmem(1,sizeof(Mcalc_Stack),srec);
  srec->data = (Uchar*)allocmem(strlen((char*)str)+1,sizeof(char),srec->data);
  strcpy((char*)srec->data,(char*)str);
  srec->next = *stk;
  *stk = srec;
  srec = NULL;
}

/*****************************************************************************
Procedure to enqueue characters on to the queue
*/
void CalculatorBase::enqueue(Mcalc_Queue** stk, Mcalc_QueueData str)
{
  Mcalc_Stack* srec = NULL;
  Mcalc_Stack* endp;

  srec = (Mcalc_Stack*)allocmem(1,sizeof(Mcalc_Stack),srec);
  srec->data = (Uchar*)allocmem(strlen((char*)str)+1,sizeof(char),srec->data);
  strcpy((char*)srec->data, (char*)str);

  if (*stk)
  {
    endp = *stk;
    while (endp->next)
      endp = endp->next;
    endp->next = srec;
  }
  else
    *stk = srec;

  srec->next = NULL;
  endp = srec = NULL;
}

/*****************************************************************************
Procedure to pop characters from the stack
*/
Mcalc_StackData CalculatorBase::pop(Mcalc_Stack** stk)
{
  Mcalc_Stack* srec;
  Uchar* str;

  if (*stk)
  {
    str = (*stk)->data;
    srec = *stk;
    *stk = (*stk)->next;
    srec->next = NULL;
    MemMatrix::Matrix().Deallocate(srec);
    srec = NULL;
  }
  return str;
}

/********************** Expression Parsing Procedures ***********************/
/*****************************************************************************
Procedure to compare the priority of the math operators
*/
int CalculatorBase::priority(Uchar* op)
{
  if (op == NULL)
    return 1;

  Uchar opcode_ = (Uchar)(op[0]);

  switch (opcode_)
  {
    case '%':
    case FACTORIAL_OP:
    case NOT_OP:
    case SIN_OP:
    case COS_OP:
    case TAN_OP:
    case SINH_OP:
    case COSH_OP:
    case TANH_OP:
    case ASIN_OP:
    case ACOS_OP:
    case ATAN_OP:
    case EXP_OP:
    case LOG_OP:
    case LOG10_OP:
    case CEILING_OP:
    case FLOOR_OP:
    case ABS_OP:
    case FRACPART_OP:
    case INTPART_OP:
    case SQR_OP:
    case SQRT_OP:
    case CUBE_OP:
    case RND_OP:
    case CUBERT_OP:
    case RAND_OP:
    case INV_OP       : return 14;

    case COMB_OP:
    case PERM_OP:
    case PERMREP_OP:
    case COMBREP_OP   : return 13;

    case SIGN_OP      : return 12;     // needed to be here so 2^-4 will work, instead of 2^(-4)
    case SCI_OP       : return 11;
    case '^'          : return 10;

    case '/'          :
    case '*'          :
    case '@'          : return 9;

    case '+'          :
    case MINUS_OP     : return 8;

    case '<':
    case '>':
    case LSTHANOREQ_OP:
    case GRTHANOREQ_OP: return 7;

    case EQUAL_OP:
    case NOTEQUAL_OP  : return 6;

    case AND_OP       : return 5;
    case XOR_OP       : return 4;
    case OR_OP        : return 3;
    case '('          : return 2;
  }

  return 0;
}

/*****************************************************************************
Procedure to push an math operator onto the stack
*/
void CalculatorBase::pushop(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Mcalc_StackData op)
{
  Mcalc_StackData topop;
  int done = FALSE;

  while (!done)
  {
    topop = (*opstk) ? pop(opstk):NULL;
    if (priority(op) > priority(topop))
    {
      if (topop)
        push(opstk,topop);
      done = TRUE;
    }
    else if (topop && *topop!='(')
      enqueue(rpnq,topop);

    MemMatrix::Matrix().Deallocate(topop);
    topop = NULL;
  }

  push(opstk,op);
}

/*****************************************************************************/
// Procedure to check if symbol in expression is valid
// 0: Search backward
// 1: Search forward
//
int CalculatorBase::IsVariable(char* Str_, int IsOp_, int IsBinary_, int IsNum_, int IsVar_, int Index_, int Dir_)
{
  if ((IsOp_ && !IsNum_) || IsVar_)
    return
    (
      Dir_ ? (Str_[Index_ + 1] ?
                 (Str_[Index_] == ' ' ?
                     IsVariable(Str_, IsOp_, IsBinary_, IsNum_, IsVar_, Index_ + 1, Dir_):
                     (Str_[Index_] == '$' && isalpha(Str_[Index_ + 1]) &&
                     InRange('A', Str_[Index_ + 1], 'z'))):0):

             (IsBinary_ ?
                 (Index_ ?
                     (Str_[Index_] == ' ' ?
                         IsVariable(Str_, IsOp_, IsBinary_, IsNum_, IsVar_, Index_ - 1, Dir_):
                         (isalpha(Str_[Index_]) && InRange('A', Str_[Index_], 'z') &&
                         Str_[Index_ - 1] == '$')):0):0)
    );

  return 0;
}

/*****************************************************************************/
int CalculatorBase::IsGraphCmd(char* Str_, int Index_, int IsOp_, int IsNum_, int Lval_, int Rval_)
{
  while (Index_ && !(isalpha(Str_[Index_ - 1]) &&
                     InRange('A', Str_[Index_ - 1], 'z')))
    --Index_;

  if (Index_ >= 5)
    return (!Lval_ && Rval_ && !IsOp_ && !IsNum_ && icasestrcompn(&Str_[Index_ - 5], "graph", 5));
}

/*****************************************************************************
Procedure to check if symbol in expression is valid
*/
int CalculatorBase::IsValidInputChar(char* Str_, int Index_, int IsOp_, int IsPost_, int IsNum_, int HexOk_)
{
  char symbol = Str_[Index_];
  if (symbol && symbol == ' ')
    return IsValidInputChar(Str_, Index_ + 1, IsOp_, IsPost_, IsNum_, HexOk_);

  return
  (
    ((IsNum_ || !IsOp_ || IsPost_) &&
      (symbol=='&' || symbol=='|' ||
       symbol==0 ||
       IsBinaryOpCodes(symbol) ||
       IsBinaryOperators(symbol) ||
       (!IsPost_ && IsPostfixOperators(symbol)) ||
       IsBinaryFunctions(Str_, Index_))) ||

    (IsOp_ && !IsPost_ &&
      (symbol=='$' ||
       IsUnaryOpCodes(symbol) ||
       IsUnaryOperators(symbol) ||
       IsUnaryFunctions(&Str_[Index_]) ||
       IsConstSymbols(symbol) ||
       IsConstString(Str_, Index_))) ||

    IsUnprocessedSymbols(symbol) ||

    (((IsOp_ && !IsPost_) || IsNum_) ?
          HasOperandSymbols(symbol, HexOk_) &&
          !(IsNum_ && IsNumberPrefix(&Str_[Index_], 0, IsOp_, IsNum_, IsNum_, HexOk_, 1)):0)
  );
}

/******************************************************************************/
int CalculatorBase::IsValidInputChar(char* Str_, int Index_, int IsOp_, int IsPost_, int IsNum_)
{
  return IsValidInputChar(Str_, Index_, IsOp_, IsPost_, IsNum_, GetCalcIndex() == CalculatorBase::LONGNUM);
}

/******************************************************************************/
int CalculatorBase::IsValidProcessedChar(char* Str_, int Index_, int IsOp_, int IsBinary_,
                                         int IsNum_, int IsPrefix_, int HexOk_)
{
  char symbol = Str_[Index_];
  if (symbol == ' ' && Index_)
    return IsValidProcessedChar(Str_, Index_ - 1, IsOp_, IsBinary_, IsNum_, IsPrefix_, HexOk_);

  return
  (
    isvalidsymbol((Uchar)symbol, HexOk_) &&
      (IsBinary_ ? (!IsBinaryOperators(symbol) &&
                    !IsBinaryOpCodes(symbol) &&
                    !IsUnaryOperators(symbol) &&
                    !IsUnaryOpCodes(symbol) &&
                    !isleftbracket((Uchar)symbol)):1) ||
    IsNumberPrefix(Str_, Index_, IsOp_, IsNum_, IsPrefix_, HexOk_, 0) ||
    (IsBinary_ ? IsConstSymbols(symbol):0)
  );
}

/*****************************************************************************/
int CalculatorBase::IsThisChar(const char* Str_, int Index_, char Target_, int Dir_)
{
  char symbol = Str_[Index_];
  while (symbol == ' ')
    if (Dir_)
    {
      if (!symbol)
        return 0;

      ++Index_;
      symbol = Str_[Index_];
    }
    else
    {
      if (!Index_)
        return 0;

      --Index_;
      symbol = Str_[Index_];
    }

  return (symbol == Target_);
}

/****************************************************************************/
int CalculatorBase::IsFirstTrailingChar(char* list, int i, int len, int ch)
{
  if (i > len)
    return FALSE;

  for (;i < len; i++)
    if (list[i] == ch)
      return TRUE;

  return (list[i] == ch);
}

/****************************************************************************/
int CalculatorBase::IsLastLeadingChar(char* list, int i, int ch)
{
  if (i < 0)
    return FALSE;

  for (;i;i--)
    if (list[i] == ch)
      return TRUE;

  return (list[i] == ch);
}

/****************************************************************************/
int CalculatorBase::IsFirstTrailingChar(char* list, int i, int len, int(*iscsetfn)(int))
{
  if (i > len)
    return FALSE;

  for (;i < len; i++)
    if ((*iscsetfn)(list[i]))
      return TRUE;

  return (*iscsetfn)(list[i]);
}

/****************************************************************************/
int CalculatorBase::IsLastLeadingChar(char* list, int i, int(*iscsetfn)(int))
{
  if (i < 0)
    return FALSE;

  for (;i;i--)
    if ((*iscsetfn)(list[i]))
      return TRUE;

  return (*iscsetfn)(list[i]);
}

/*****************************************************************************
Procedure to check if symbol in expression is valid
*/
int CalculatorBase::isvalidsymbol(Uchar Usymbol, int HexOk_)
{
  return
  (
    Usymbol=='-' ||
    Usymbol=='+' ||
    Usymbol=='*' ||
    Usymbol=='/' ||
    Usymbol==POW_OP ||
    Usymbol==RAND_OP ||
    Usymbol==XOR_OP ||
    Usymbol==INV_OP ||
    Usymbol==SCI_OP ||
    Usymbol==SIGN_OP ||
    Usymbol==NOT_OP ||
    Usymbol=='%' ||
    Usymbol==AND_OP ||
    Usymbol==OR_OP ||
    Usymbol=='@' ||
    Usymbol==SQR_OP ||
    Usymbol==SQRT_OP ||
    Usymbol==CUBE_OP ||
    Usymbol==CUBERT_OP ||
    Usymbol==SIN_OP ||
    Usymbol==COS_OP ||
    Usymbol==TAN_OP ||
    Usymbol==MINUS_OP ||
    Usymbol==SINH_OP ||
    Usymbol==COSH_OP ||
    Usymbol==TANH_OP ||
    Usymbol==COMB_OP ||
    Usymbol==ASIN_OP ||
    Usymbol==ACOS_OP ||
    Usymbol==ATAN_OP ||
    Usymbol==PERM_OP ||
    Usymbol==EXP_OP ||
    Usymbol==LOG_OP ||
    Usymbol==LOG10_OP ||
    Usymbol==PERMREP_OP ||
    Usymbol==COMBREP_OP ||
    Usymbol==CEILING_OP ||
    Usymbol==FLOOR_OP ||
    Usymbol==ABS_OP ||
    Usymbol==LSTHANOREQ_OP ||
    Usymbol==GRTHANOREQ_OP ||
    Usymbol==FRACPART_OP ||
    Usymbol==INTPART_OP ||
    Usymbol==RND_OP ||
    HasOperandSymbols(Usymbol, HexOk_) ||
    IsUnprocessedSymbols(Usymbol) ||
    Usymbol==FACTORIAL_OP ||
    Usymbol=='^' ||
    Usymbol=='<' ||
    Usymbol=='>' ||
    Usymbol==EQUAL_OP ||
    Usymbol==NOTEQUAL_OP
  );
}

/******************************************************************************/
int CalculatorBase::isvalidsymbol(Uchar Usymbol)
{
  return isvalidsymbol(Usymbol, GetCalcIndex() == CalculatorBase::LONGNUM);
}

/*****************************************************************************
Procedure to check if symbol in expression is valid
*/
int CalculatorBase::isdelimiter(Uchar symbol)
{
  return (symbol==',');
}

/****************************************************************************/
void CalculatorBase::BlankOut(char* Str_, int Start_, int End_)
{
  while (Start_ < End_)
    Str_[Start_++] = ' ';
}

/*****************************************************************************/
// Dir_ == 0: parsing from left to right
// Dir_ == 1: parsing from right to left
//
int CalculatorBase::IsAssignmentOp(Uchar* list, int Index_, int Dir_)
{
  list += Index_;
  char* List_ = (char*)list;
  int PrevChar_ = 1;

  if (Dir_)
  {
    if (List_[1])
      PrevChar_ = List_[1] != '=';
  }
  else
  {
    if (Index_)
      PrevChar_ = *(List_-1) != '=';
  }

  if (Dir_ == 0)
  {
    return
    (
        (strlen(List_) > 1) ?
            ((PrevChar_ && List_[0] == '=' && List_[1] != '=') ||
                (List_[1] == '=' &&
                    (List_[0] == '+' || List_[0] == '-' ||
                     List_[0] == '*' || List_[0] == '/' ||
                     List_[0] == '@' || List_[0] == '|' ||
                     List_[0] == '&'))):
            (PrevChar_ && List_[0] == '=')
    );
  }

  return
  (
    Index_ ?
      (PrevChar_ && List_[0] == '=' && *(List_-1) != '=' && *(List_-1) != '!' && *(List_-1) != '<' && *(List_-1) != '>'):
      (PrevChar_ && List_[0] == '=')
  );
}

/*****************************************************************************/
const char* CalculatorBase::ProgramName(char* Name_)
{
  size_t x;
  size_t len = ::SafeStrLen(Name_);

  if (len)
  {
    for (x = strlen(Name_); x && isspace(Name_[x - 1]); x--);
    Name_[x] = 0;
    for (x = 0; isspace(Name_[x]); x++);
    if (x) memmove(Name_, &Name_[x], strlen(&Name_[x]) + 1);

    if (strlen(Name_) > 0)
      return Name_;
  }

  return NULL;
}

/*****************************************************************************
Procedure to detect a valid symbol representing a left bracket
*/
int CalculatorBase::isleftbracket(Uchar tok)
{
  return (tok=='(' || tok=='[' || tok=='{');
}

/*****************************************************************************
Procedure to detect a valid symbol representing a left bracket
*/
int CalculatorBase::isrightbracket(Uchar tok)
{
  return (tok==')' || tok==']' || tok=='}');
}

/*****************************************************************************
Procedure to check for a left bracket in the expression
*/
int CalculatorBase::getleftbracket(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Uchar* token)
{
  Uchar OpStr_[2];

  if (isleftbracket(token[READ]))
  {
    if (token[PROCESSED]=='1' || token[PROCESSED]=='%' || isrightbracket(token[PROCESSED]))
      pushop(rpnq,opstk, ToUchar(OpStr_, '*'));

    push(opstk, ToUchar(OpStr_, '('));
    token[PROCESSED] = '(';
    return TRUE;
  }

  return FALSE;
}

/*****************************************************************************
Procedure to check for a right bracket in the expression
*/
int CalculatorBase::getrightbracket(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Uchar* token, int& BalancedBrk_)
{
  Mcalc_StackData str;

  if (isrightbracket(token[READ]))
  {
    if (isrightbracket(token[PROCESSED]) || token[PROCESSED]=='1' || token[PROCESSED]=='%')
    {
      while (*opstk)
      {
        if (*(str = pop(opstk)) == '(')
          break;
        else
        {
          enqueue(rpnq,str);
          if (*opstk)
          {
            MemMatrix::Matrix().Deallocate(str);
            str = NULL;
          }
        }
      }

      token[PROCESSED] = ')';
      BalancedBrk_ = str && (*str=='(');

      if (str)
        MemMatrix::Matrix().Deallocate(str);

      return BalancedBrk_;
    }
  }
  return FALSE;
}

/*****************************************************************************/
int CalculatorBase::IsValidOperator(Uchar* token)
{
  Uchar processed_ = token[PROCESSED];
  Uchar read_ = token[READ];

  return
  (
      (read_=='+' || read_=='-' || read_==NOT_OP || read_=='%' || read_==RAND_OP ||
       read_==INV_OP || read_==SQR_OP || read_==SQRT_OP || read_==CUBE_OP ||
       read_==CUBERT_OP || read_==SIN_OP || read_==COS_OP || read_==TAN_OP ||
       read_==SIGN_OP || read_==SINH_OP || read_==COSH_OP || read_==TANH_OP ||
       read_==ASIN_OP || read_==ACOS_OP || read_==ATAN_OP || read_==EXP_OP ||
       read_==LOG_OP || read_==LOG10_OP || read_==CEILING_OP || read_==FLOOR_OP ||
       read_==ABS_OP || read_==FRACPART_OP || read_==INTPART_OP || read_==RND_OP ||
       read_==FACTORIAL_OP) ||
               ((read_==MINUS_OP || read_=='*' || read_=='/' || read_=='^' ||
                 read_=='@' || read_==AND_OP || read_==OR_OP || read_=='<' ||
                 read_=='>' || read_==COMB_OP || read_==PERM_OP || read_==COMBREP_OP||
                 read_==PERMREP_OP || read_==LSTHANOREQ_OP || read_==GRTHANOREQ_OP ||
                 read_==EQUAL_OP || read_==NOTEQUAL_OP || read_==XOR_OP || read_==SCI_OP) &&
                (processed_=='1' || processed_=='%' || isrightbracket(processed_)))
  );
}

/******************************************************************************/
// Procedure to check for a valid operator in the expression
//
int CalculatorBase::getoperator(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Uchar* token)
{
  Uchar Symbol_[5];
  Uchar Buf_[2];
  Buf_[1] = 0;

  if (IsValidOperator(token))
  {
    if (token[READ]=='-')
    {
      if (token[PROCESSED]=='1' || token[PROCESSED]=='%' || isrightbracket(token[PROCESSED]))
      {
        Buf_[0] = MINUS_OP;
        pushop(rpnq,opstk,Buf_);
        token[PROCESSED] = Buf_[0];
        return TRUE;
      }
      else
      {
        Buf_[0] = SIGN_OP;
        enqueue(rpnq, ToUchar(Symbol_, "-1"));
        pushop(rpnq,opstk,Buf_);
        token[PROCESSED] = Buf_[0];
        return TRUE;
      }
    }
    else if (token[READ]==NOT_OP)
    {
      Buf_[0] = NOT_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==SIN_OP)
    {
      Buf_[0] = SIN_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==COS_OP)
    {
      Buf_[0] = COS_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==TAN_OP)
    {
      Buf_[0] = TAN_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==ASIN_OP)
    {
      Buf_[0] = ASIN_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==ACOS_OP)
    {
      Buf_[0] = ACOS_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==ATAN_OP)
    {
      Buf_[0] = ATAN_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==SINH_OP)
    {
      Buf_[0] = SINH_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==COSH_OP)
    {
      Buf_[0] = COSH_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==TANH_OP)
    {
      Buf_[0] = TANH_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==EXP_OP)
    {
      Buf_[0] = EXP_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==LOG_OP)
    {
      Buf_[0] = LOG_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==LOG10_OP)
    {
      Buf_[0] = LOG10_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==CEILING_OP)
    {
      Buf_[0] = CEILING_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==FLOOR_OP)
    {
      Buf_[0] = FLOOR_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==ABS_OP)
    {
      Buf_[0] = ABS_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==INTPART_OP)
    {
      Buf_[0] = INTPART_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==FRACPART_OP)
    {
      Buf_[0] = FRACPART_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==RAND_OP)
    {
      Buf_[0] = RAND_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==INV_OP)
    {
      Buf_[0] = INV_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]=='+')
    {
      if (token[PROCESSED]=='1' || token[PROCESSED]=='%' || isrightbracket(token[PROCESSED]))
      {
        pushop(rpnq,opstk,ToUchar(Symbol_, '+'));
        token[PROCESSED] = '+';
        return TRUE;
      }
      else
      {
        Buf_[0] = SIGN_OP;
        enqueue(rpnq, ToUchar(Symbol_, "1"));
        pushop(rpnq,opstk,Buf_);
        token[PROCESSED] = Buf_[0];
        return TRUE;
      }
    }
    else if (token[READ]=='*')
    {
      pushop(rpnq,opstk, ToUchar(Symbol_, '*'));
      token[PROCESSED] = '*';
      return TRUE;
    }
    else if (token[READ]=='/')
    {
      pushop(rpnq,opstk, ToUchar(Symbol_, '/'));
      token[PROCESSED] = '/';
      return TRUE;
    }
    else if (token[READ]=='@')
    {
      pushop(rpnq,opstk, ToUchar(Symbol_, '@'));
      token[PROCESSED] = '@';
      return TRUE;
    }
    else if (token[READ]=='^')
    {
      pushop(rpnq,opstk, ToUchar(Symbol_, '^'));
      token[PROCESSED] = '^';
      return TRUE;
    }
    else if (token[READ]==AND_OP)
    {
      Buf_[0] = AND_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==OR_OP)
    {
      Buf_[0] = OR_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==SQR_OP)
    {
      Buf_[0] = SQR_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==SQRT_OP)
    {
      Buf_[0] = SQRT_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==CUBE_OP)
    {
      Buf_[0] = CUBE_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==CUBERT_OP)
    {
      Buf_[0] = CUBERT_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]=='%')
    {
      Buf_[0] = '%';
      pushop(rpnq,opstk,Buf_);
      enqueue(rpnq, ToUchar(Symbol_, '1'));
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==FACTORIAL_OP)
    {
      Buf_[0] = FACTORIAL_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==RND_OP)
    {
      Buf_[0] = RND_OP;
      enqueue(rpnq, ToUchar(Symbol_, '0'));
      pushop(rpnq,opstk,Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==COMB_OP)
    {
      Buf_[0] = COMB_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==PERM_OP)
    {
      Buf_[0] = PERM_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==PERMREP_OP)
    {
      Buf_[0] = PERMREP_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==COMBREP_OP)
    {
      Buf_[0] = COMBREP_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==LSTHANOREQ_OP)
    {
      Buf_[0] = LSTHANOREQ_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==GRTHANOREQ_OP)
    {
      Buf_[0] = GRTHANOREQ_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]=='<')
    {
      Buf_[0] = '<';
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]=='>')
    {
      Buf_[0] = '>';
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==EQUAL_OP)
    {
      Buf_[0] = EQUAL_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==NOTEQUAL_OP)
    {
      Buf_[0] = NOTEQUAL_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==XOR_OP)
    {
      Buf_[0] = XOR_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
    else if (token[READ]==SCI_OP)
    {
      Buf_[0] = SCI_OP;
      pushop(rpnq,opstk, Buf_);
      token[PROCESSED] = Buf_[0];
      return TRUE;
    }
  }

  return FALSE;
}

/*****************************************************************************
Procedure to check for an operand in the expression
*/
int CalculatorBase::getoperand(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, Uchar* token,
                               char* buffer, int* index)
{
  int start, valid, diff,
      c = *index;
  char* valstr = NULL;
  Uchar Symbol_[3];

  int Cond1_ = HasOperandSymbols(token[READ]);
  int Cond2_ = (!isdigit(token[PROCESSED]) && token[PROCESSED]!='%') || token[PROCESSED]=='0';

  if (Cond1_ && Cond2_)
  {
    if (isrightbracket(token[PROCESSED]))
      pushop(rpnq,opstk, ToUchar(Symbol_, '*'));

    start = c;
    if (buffer[c])
    {
      if (((Uchar)buffer[c]) == NAN_CONST || ((Uchar)buffer[c]) == INF_CONST)
        diff = 1;
      else if (GetCalcIndex() == CalculatorBase::LONGNUM)
        ::ExtractAnyNum(&buffer[c],1,diff,0,16);    // note: no mem allocation specified
      else
        ::ExtractAnyNum(&buffer[c],1,diff,0,10);

      c += diff;
    }

    valstr = (char*)allocmem(c-start+1,sizeof(char),valstr);
    strncpy(valstr,buffer+start,c-start);
    valstr[c-start] = '\0';
    valid = IsValidNum(valstr) || (((Uchar)buffer[start]) == NAN_CONST || ((Uchar)buffer[start]) == INF_CONST);
    enqueue(rpnq, (Uchar*)valstr);
    *index = c-1;
    token[PROCESSED] = '1';
    MemMatrix::Matrix().Deallocate(valstr);
    valstr = NULL;

    return valid;
  }

  return FALSE;
}

/*****************************************************************************/
int CalculatorBase::IsNumberPrefix(char* Str_, int Index_, int IsOp_, int IsNum_,
                                   int IsPrefix_, int HexOk_, int Dir_)
{
  return (Dir_ ? (strlen(Str_) > 1 && icasestrcompn(Str_, "0x", 2)):
                 (!IsOp_ && !IsPrefix_ && Index_ &&
                  IsNum_ && HexOk_ && icasestrcompn(&Str_[Index_ - 1], "0x", 2)));
}

/*****************************************************************************/
int CalculatorBase::HasUnresolvedNames(const char* Str_)
{
  int x;
  int len = ::SafeStrLen(Str_);
  int Found_;

  for (x = 0; x < len; x++)
  {
    Found_ =
    IsConstString(Str_, x) ||
    IsBinaryFunctions(Str_, x) ||
    IsUnaryFunctions(Str_) ||
    IsCalledFunctions(Str_);

    if (Found_)
      return TRUE;
  }

  return FALSE;
}

/*****************************************************************************/
int CalculatorBase::HasHexPrefix(const char* Str_, int Index_)
{
  const char* Ptr_ = &Str_[Index_];
  if ((*Ptr_ == ' ' || IsHexc(*Ptr_)) && Index_)
    return HasHexPrefix(Str_, Index_ - 1);

  return (toupper(*Ptr_) == 'X' && Index_ && Ptr_[-1] == '0');
}

/*****************************************************************************/
int CalculatorBase::IsRightsideValue(const char* list, int Index_, int HexOk_)
{
  while (list[Index_] && IsUnprocessedSymbols(list[Index_]))
    Index_++;

  if (list[Index_])
  {
    const char* Ptr_ = &list[Index_];
    return
    (
      IsUnaryFunctions(Ptr_) ||
      IsUnaryOperators(*Ptr_) ||
      IsConstString(list, Index_) ||
      HasOperandSymbols(*Ptr_, HexOk_)
    );
  }
}

/*****************************************************************************/
// Tests if there are leading and trailing spaces needed for a "whole" word
//
int CalculatorBase::WholeWord(char* list, int len, int cmplen, int index,
                              int Lval_, int Rval_, int IsOp_, int IsNum_,
                              int HexOk_, int IsUnary_, int IsBinary_, int IsPost_,
                              int IsGraphable_)
{
  int LeftsideChecked_ = 0;
  int RightsideChecked_ = 0;
  int LeftsideValid_ = 1;
  int RightsideValid_ = 1;
  int Start_ = index;

  if (index > 0)
  {
    while (index && list[index - 1] == ' ')
      --index;

    if (index > 0)
    {
      LeftsideChecked_ = 1;
      LeftsideValid_ =
          (IsGraphable_ && IsGraphCmd(list, index, IsOp_, IsNum_, Lval_, Rval_)) ||
          IsVariable(list, IsOp_, (IsBinary_ || IsPost_), IsNum_, (Lval_ && Rval_), index - 1, 0) ||
          IsValidProcessedChar(list, index - 1, IsOp_, (IsBinary_ || IsPost_), IsNum_, (IsNum_ && IsUnary_), HexOk_) ||
          (Rval_ ? IsThisChar(list, index - 1, '=', 0):0);
    }
  }

  if (cmplen < len - index)
  {
    index = Start_;
    while (list[index + cmplen] == ' ')
      ++index;

    if (cmplen < len - index)
    {
      RightsideChecked_ = 1;
      RightsideValid_ =
          IsVariable(list, IsOp_, (IsBinary_ || IsPost_), IsNum_, (Lval_ && Rval_), index + cmplen, 1) ||
          IsValidInputChar(list, index + cmplen, IsOp_, IsPost_, IsNum_, HexOk_) ||
          (Lval_ ? IsThisChar(list, index + cmplen, '=', 1):0);
    }
  }

  if (IsOp_)
    if (IsPost_)
      LeftsideValid_ = LeftsideValid_ && LeftsideChecked_;
    else if (IsUnary_)
      RightsideValid_ = RightsideValid_ && RightsideChecked_;
    else
    {
      LeftsideValid_ = LeftsideValid_ && LeftsideChecked_;
      RightsideValid_ = RightsideValid_ && RightsideChecked_;
    }

  return (LeftsideValid_ && RightsideValid_);
}

/*****************************************************************************/
int CalculatorBase::IsGraphableFunction(char* list, int len, int cmplen, int index, Uchar OpCode_)
{
  if (cmplen < len - index && OpCode_ == CalculatorBase::FUNCTION_INDEX)
    return
    (
      icasestrcompn(&list[index], "FMIN", cmplen)         ||
      icasestrcompn(&list[index], "FMAX", cmplen)         ||
      icasestrcompn(&list[index], "DERIV", cmplen)        ||
      icasestrcompn(&list[index], "INTEG", cmplen)        ||
      icasestrcompn(&list[index], "SOLVE", cmplen)        ||
      icasestrcompn(&list[index], "DERIV2", cmplen)       ||
      icasestrcompn(&list[index], "INTERSECT", cmplen)
    );

  return 0;
}

/*****************************************************************************/
int CalculatorBase::WholeWord(char* list, int len, int cmplen, int index, Uchar OpCode_)
{
  return WholeWord(list, len, cmplen, index,
    IsLval(OpCode_), IsRval(OpCode_), IsOp(OpCode_), IsNum(OpCode_),
    IsHexOk(OpCode_), IsUnary(OpCode_), IsBinary(OpCode_), IsPostfix(OpCode_),
    IsGraphableFunction(list, len, cmplen, index, OpCode_));
}

/*****************************************************************************/
int CalculatorBase::IsValidAssignment(char* list, int len, int cmplen, int index, int HexOk_)
{
  if (index >= len)
    return 0;

  int pre_ = 0;
  if (index > 0)
    while ((index - pre_ > 0) && IsUnprocessedSymbols(list[index - pre_ - 1]))
      pre_++;

  int ext_ = 0;
  if (cmplen < len - index)
    while ((cmplen < len - index - ext_) && IsUnprocessedSymbols(list[index + cmplen + ext_]))
      ext_++;

  return
  (
    ((index - pre_ > 0) ? IsVariable(list, TRUE, TRUE, FALSE, FALSE, index - pre_ - 1, 0):0) &&
    ((cmplen < len - index - ext_) ? (IsThisChar(list, index + cmplen + ext_, '\"', 1) ||
                                      IsVariable(list, TRUE, TRUE, FALSE, FALSE, index + cmplen + ext_, 1) ||
                                      IsRightsideValue(list, index + cmplen + ext_, HexOk_)):0)
  );
}

/*****************************************************************************/
int CalculatorBase::FindFncDelim(char* list, size_t len, int** DelimArrayPtr_, size_t ArraySize_)
{
  if (!list || !DelimArrayPtr_)
    return 0;

  size_t i, x, y;
  size_t bal = 0;
  size_t OldSize_ = 0;
  int* DelimArray_ = *DelimArrayPtr_;
  int* OldArray_ = NULL;
  int InQuote_ = 0;

  x = 1;
  DelimArray_[0] = 0;

  for (i = 0; i < len; i++)
  {
    if (list[i] == '\"')
      InQuote_ = !InQuote_;

    if (InQuote_)
      continue;

    if (isleftbracket((Uchar)list[i]))
    {
      ++bal;
      if (bal == 1)
      {
        DelimArray_[x++] = i;
        ++DelimArray_[0];
      }
    }
    else if (isrightbracket((Uchar)list[i]))
    {
      --bal;
      if (bal == 0)
      {
        DelimArray_[x++] = i;
        ++DelimArray_[0];
        break;
      }
    }
    else if (list[i] == ',' && bal == 1)
    {
      DelimArray_[x++] = i;
      ++DelimArray_[0];
    }

    if (x >= ArraySize_)
    {
      OldSize_ = ArraySize_;
      OldArray_ = DelimArray_;
      ArraySize_ += 20;
      DelimArray_ = (int*)MemMatrix::Matrix().Callocate(ArraySize_ * sizeof(int));

      for (y = 0; y < OldSize_; y++)
        DelimArray_[y] = OldArray_[y];

      for (;y < ArraySize_; y++)
        DelimArray_[y] = 0;

      MemMatrix::Matrix().Deallocate(OldArray_);
    }
  }

  *DelimArrayPtr_ = DelimArray_;
  return DelimArray_[0];
}

/*****************************************************************************
Procedure to format the operators in the argument list to work with the logic
of the program
*/
int CalculatorBase::formatlist(char** arglist, CalculatorBase* CalcPtr_)
{
  int i, x;
  char *list = NULL;
  int len = 0;
  int Valid_ = 1;
  int Operand_ = 0;
  int Number_ = 0;
  int OpSymbols_ = 0;
  int HexValid_ = 0;
  int PrevNumber_ = 0;
  int HexPrefix_ = 0;
  int OctPrefix_ = 0;
  int SciOp_ = 0;
  int Sign_ = 0;
  int OptSign_ = 0;
  int HexNumber_ = 0;
  int OctNumber_ = 0;
  int HexLen_ = 0;
  int OctLen_ = 0;
  int IsConst_ = 0;
  int BinOp_ = 0;
  int UnOp_ = 0;
  int PostOp_ = 0;
  int UnProc_ = 0;
  int ValidOp_ = 0;

  list = *arglist;
  len = strlen(list);

  if (_ExecuteProgram)
  {
    MemMatrix::Matrix().Deallocate(_ExecuteProgram);
    _ExecuteProgram = NULL;
  }

  bool Blanked_ = false;
  for (i = 0; i < len; i++)
    if (isalpha(list[i]) && InRange('A', list[i], 'z') &&
        icasestrcompn(list+i, "GRAPH", 5))
    {
      x = i;
      i += 5;
      BlankOut(list, x, i);
      i--;
      Blanked_ = true;
    }

  if (Blanked_)
  {
    stripws(list);
    len = strlen(list);
  }

  for (i=0, x=0; i < len;)
    if (isalpha(list[i]) && InRange('A', list[i], 'z'))
    {
      if (icasestrcompn(list+i, "SQUAREROOT", 10) && WholeWord(list, len, 10, i, SQRT_OP))
      {
        list[x++] = SQRT_OP;
        i += 10;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "CUBEROOT", 8) && WholeWord(list, len, 8, i, CUBERT_OP))
      {
        list[x++] = CUBERT_OP;
        i += 8;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "FRACPART", 8) && WholeWord(list, len, 8, i, FRACPART_OP))
      {
        list[x++] = FRACPART_OP;
        i += 8;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "COMBREP", 7) && WholeWord(list, len, 7, i, COMBREP_OP))
      {
        list[x++] = COMBREP_OP;
        i += 7;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "PERMREP", 7) && WholeWord(list, len, 7, i, PERMREP_OP))
      {
        list[x++] = PERMREP_OP;
        i += 7;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "CEILING", 7) && WholeWord(list, len, 7, i, CEILING_OP))
      {
        list[x++] = CEILING_OP;
        i += 7;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "INTPART", 7) && WholeWord(list, len, 7, i, INTPART_OP))
      {
        list[x++] = INTPART_OP;
        i += 7;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "FRACANS", 7) && WholeWord(list, len, 7, i, FRACANS_CONST))
      {
        list[x++] = FRACANS_CONST;
        i += 7;
        BlankOut(list, x, i);

        Operand_ = TRUE;
        IsConst_ = TRUE;

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
        PrevNumber_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ARCSIN", 6) && WholeWord(list, len, 6, i, ASIN_OP))
      {
        list[x++] = ASIN_OP;
        i += 6;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ARCCOS", 6) && WholeWord(list, len, 6, i, ACOS_OP))
      {
        list[x++] = ACOS_OP;
        i += 6;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ARCTAN", 6) && WholeWord(list, len, 6, i, ATAN_OP))
      {
        list[x++] = ATAN_OP;
        i += 6;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "CUBERT", 6) && WholeWord(list, len, 6, i, CUBERT_OP))
      {
        list[x++] = CUBERT_OP;
        i += 6;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "SQUARE", 6) && WholeWord(list, len, 6, i, SQR_OP))
      {
        list[x++] = SQR_OP;
        i += 6;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "SQROOT", 6) && WholeWord(list, len, 6, i, SQRT_OP))
      {
        list[x++] = SQRT_OP;
        i += 6;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ANSSTR", 6) && WholeWord(list, len, 6, i, ANSSTR_CONST))
      {
        list[x++] = ANSSTR_CONST;
        i += 6;
        BlankOut(list, x, i);

        Operand_ = TRUE;
        IsConst_ = TRUE;

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
        PrevNumber_ = FALSE;
      }
      else if (icasestrcompn(list+i, "FLOOR", 5) && WholeWord(list, len, 5, i, FLOOR_OP))
      {
        list[x++] = FLOOR_OP;
        i += 5;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "LOG10", 5) && WholeWord(list, len, 5, i, LOG10_OP))
      {
        list[x++] = LOG10_OP;
        i += 5;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "GRAPH", 5))
      {
        i += 5;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "FACT", 4) && WholeWord(list, len, 4, i, FACTORIAL_OP))
      {
        list[x++] = FACTORIAL_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "CEIL", 4) && WholeWord(list, len, 4, i, CEILING_OP))
      {
        list[x++] = CEILING_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "CUBE", 4) && WholeWord(list, len, 4, i, CUBE_OP))
      {
        list[x++] = CUBE_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ASIN", 4) && WholeWord(list, len, 4, i, ASIN_OP))
      {
        list[x++] = ASIN_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ACOS", 4) && WholeWord(list, len, 4, i, ACOS_OP))
      {
        list[x++] = ACOS_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ATAN", 4) && WholeWord(list, len, 4, i, ATAN_OP))
      {
        list[x++] = ATAN_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "SINH", 4) && WholeWord(list, len, 4, i, SINH_OP))
      {
        list[x++] = SINH_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "COSH", 4) && WholeWord(list, len, 4, i, COSH_OP))
      {
        list[x++] = COSH_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "TANH", 4) && WholeWord(list, len, 4, i, TANH_OP))
      {
        list[x++] = TANH_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "SQRT", 4) && WholeWord(list, len, 4, i, SQRT_OP))
      {
        list[x++] = SQRT_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "COMB", 4) && WholeWord(list, len, 4, i, COMB_OP))
      {
        list[x++] = COMB_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "PERM", 4) && WholeWord(list, len, 4, i, PERM_OP))
      {
        list[x++] = PERM_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "RAND", 4) && WholeWord(list, len, 4, i, RAND_OP))
      {
        list[x++] = RAND_OP;
        i += 4;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (i == 0 && icasestrcompn(list, "EXEC", 4) && ProgramName(list+4))
      {
        _ExecuteProgram = ::NewString(ProgramName(list+4));
        break;
      }
      else if (icasestrcompn(list+i, "XOR", 3) && WholeWord(list, len, 3, i, XOR_OP))
      {
        list[x++] = XOR_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "AND", 3) && WholeWord(list, len, 3, i, AND_OP))
      {
        list[x++] = AND_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "NOT", 3) && WholeWord(list, len, 3, i, NOT_OP))
      {
        list[x++] = NOT_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "INV", 3) && WholeWord(list, len, 3, i, INV_OP))
      {
        list[x++] = INV_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "SQR", 3) && WholeWord(list, len, 3, i, SQR_OP))
      {
        list[x++] = SQR_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "SIN", 3) && WholeWord(list, len, 3, i, SIN_OP))
      {
        list[x++] = SIN_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "COS", 3) && WholeWord(list, len, 3, i, COS_OP))
      {
        list[x++] = COS_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "TAN", 3) && WholeWord(list, len, 3, i, TAN_OP))
      {
        list[x++] = TAN_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "EXP", 3) && WholeWord(list, len, 3, i, EXP_OP))
      {
        list[x++] = EXP_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "LOG", 3) && WholeWord(list, len, 3, i, LOG_OP))
      {
        list[x++] = LOG_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ABS", 3) && WholeWord(list, len, 3, i, ABS_OP))
      {
        list[x++] = ABS_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "RND", 3) && WholeWord(list, len, 3, i, RND_OP))
      {
        list[x++] = RND_OP;
        i += 3;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "ANS", 3) && WholeWord(list, len, 3, i, ANS_CONST))
      {
        list[x++] = ANS_CONST;
        i += 3;
        BlankOut(list, x, i);

        Operand_ = TRUE;
        IsConst_ = TRUE;

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
        PrevNumber_ = FALSE;
      }
      else if (icasestrcompn(list+i, "NAN", 3) && WholeWord(list, len, 3, i, NAN_CONST))
      {
        list[x++] = NAN_CONST;
        i += 3;
        BlankOut(list, x, i);

        Operand_ = TRUE;
        IsConst_ = TRUE;

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
        PrevNumber_ = FALSE;
      }
      else if (icasestrcompn(list+i, "INF", 3) && WholeWord(list, len, 3, i, INF_CONST))
      {
        list[x++] = INF_CONST;
        i += 3;
        BlankOut(list, x, i);

        Operand_ = TRUE;
        IsConst_ = TRUE;

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
        PrevNumber_ = FALSE;
      }
      else if (icasestrcompn(list+i, "OR", 2) && WholeWord(list, len, 2, i, OR_OP))
      {
        list[x++] = OR_OP;
        i += 2;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
      }
      else if (icasestrcompn(list+i, "PI", 2) && WholeWord(list, len, 2, i, PI_CONST))
      {
        list[x++] = PI_CONST;
        i += 2;
        BlankOut(list, x, i);

        Operand_ = TRUE;
        IsConst_ = TRUE;

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
        PrevNumber_ = FALSE;
      }
      else if (!HexNumber_ && !OctNumber_ &&
               icasestrcompn(list+i, "E+", 2) &&
               WholeWord(list, len, 2, i, SCI_OP) &&
               (i > 0) && IsLastLeadingChar(list, i-1, isdigit))
      {
        list[x++] = SCI_OP;
        list[x++] = '+';
        i += 2;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = FALSE;
        SciOp_ = TRUE;
      }
      else if (!HexNumber_ && !OctNumber_ &&
               icasestrcompn(list+i, "E-", 2) &&
               WholeWord(list, len, 2, i, SCI_OP) &&
               (i > 0) && IsLastLeadingChar(list, i-1, isdigit))
      {
        list[x++] = SCI_OP;
        list[x++] = '-';
        i += 2;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = FALSE;
        SciOp_ = TRUE;
      }
      else if (!HexNumber_ && !OctNumber_ &&
               icasestrcompn(list+i, "E", 1) &&
               WholeWord(list, len, 1, i, SCI_OP) &&
               (i > 0) && IsLastLeadingChar(list, i-1, isdigit))
      {
        list[x++] = SCI_OP;
        i++;
        BlankOut(list, x, i);
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = FALSE;
        SciOp_ = TRUE;
      }
      else if (!HexNumber_ && !OctNumber_ &&
               icasestrcompn(list+i, "E", 1) &&
               WholeWord(list, len, 1, i, E_CONST))
      {
        list[x++] = E_CONST;
        i++;
        BlankOut(list, x, i);

        Operand_ = TRUE;
        IsConst_ = TRUE;

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
        PrevNumber_ = FALSE;
        // check for binop and no digits later
      }
      else
      {
        HexPrefix_ = (i > 1) && HexPrefix_;
        HexValid_ = CalcPtr_->GetCalcIndex() == CalculatorBase::LONGNUM;
        OpSymbols_ = HasOperandSymbols(list[i], HexValid_);
        Number_ = OpSymbols_ && (!Operand_ || (i == 0) || (i > 0 && (PrevNumber_ || (HexPrefix_ && HexValid_))));
        OctPrefix_ = !PrevNumber_ && !HexPrefix_ && !OctPrefix_ &&
                     !OctNumber_ && !HexNumber_ && list[i] == '0' && OpSymbols_;

        if (!WholeWord(list, len, 1, i, 0, 1, !OpSymbols_, Number_, HexValid_, 0, 0, 0, 0) ||
            (SciOp_ && !isdigit(list[i])))
        {
          Valid_ = 0;
          break;
        }

        if (!OctPrefix_ || !SciOp_)
        {
          list[x++] = list[i++];

          if (HexNumber_)
          {
            --HexLen_;
            if (HexLen_ == 0)
              HexNumber_ = FALSE;
          }
          else if (OctNumber_)
          {
            --OctLen_;
            if (OctLen_ == 0)
              OctNumber_ = FALSE;
          }
        }
        else
        {
          for (;list[i] == '0' && i < len; i++);
          if (i < len)
            list[x++] = list[i++];
          else
          {
            Valid_ = 0;
            break;
          }
        }

        Operand_ = PrevNumber_ = OpSymbols_;
        OctPrefix_ = OctPrefix_ && !SciOp_;
        BlankOut(list, x, i);

        if (OctPrefix_)
        {
          int y;
          int pref_ = i - 1;
          for (y = i; y < len && IsOctc(list[y]); y++);

          if ((y < len && (!IsDecc(list[y]) || list[y]=='0') && list[y] != '.') || y == len)
          {
            ChrString TestStr_(&list[pref_], y - pref_);
            if (IsIntOct(TestStr_.c_str()))
            {
              CalcPtr_->SetNumberBase(8);
              OctNumber_ = TRUE;
              OctLen_ = y - pref_ - 1;

              if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
              {
                CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
                CalcPtr_ = CalcPtr_->GetCalcType();
                HexValid_ = TRUE;
              }
            }
          }
        }

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
      }
    }
    else if (icasestrcompn(list+i, "$", 1))
    {
      if (isalpha(list[i + 1]) && InRange('A', list[i + 1], 'z') &&
          WholeWord(list, len, 2, i, VARIABLE_INDEX))
      {
        int VarIndex_ = toupper(list[i + 1]) - 'A';
        int y, z, start;
        int Assigned_ = 0;

        start = i;
        list[x++] = '$';

        for (y = start + 2; list[y]; y++)
          if (IsAssignmentOp((Uchar*)list, y, 0))
          {
            int PrevOk_ = 0;
            for (z = start - 1; z >= 0; z--)
              if (IsAssignmentOp((Uchar*)list, z, 1))
              {
                PrevOk_ = 1;
                break;
              }
              else if (!isspace(list[z]))
                break;

            if (z < 0)
              PrevOk_ = 1;

            if (PrevOk_)
            {
              list[x++] = tolower(list[i + 1]);
              Assigned_ = 1;
              break;
            }
            else
              break;
          }
          else if (!isspace(list[y]))
            break;

        if (!Assigned_)
          list[x++] = toupper(list[i + 1]);

        if (IsHexVar(VarIndex_))
        {
          CalcPtr_->SetNumberBase(16);
          if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
          {
            CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
            CalcPtr_ = CalcPtr_->GetCalcType();
            HexValid_ = TRUE;
          }
        }
        else if (IsOctVar(VarIndex_))
        {
          CalcPtr_->SetNumberBase(8);
          if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
          {
            CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
            CalcPtr_ = CalcPtr_->GetCalcType();
            HexValid_ = TRUE;
          }
        }

        i += 2;
        Operand_ = TRUE;

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
        PrevNumber_ = FALSE;
        BlankOut(list, x, i);
      }
      else
      {
        Valid_ = 0;
        break;
      }
    }
    else if (icasestrcompn(list+i, "<=", 2) && WholeWord(list, len, 2, i, LSTHANOREQ_OP))
    {
      list[x++] = LSTHANOREQ_OP;
      i += 2;
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (icasestrcompn(list+i, ">=", 2) && WholeWord(list, len, 2, i, GRTHANOREQ_OP))
    {
      list[x++] = GRTHANOREQ_OP;
      i += 2;
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (icasestrcompn(list+i, "==", 2) && WholeWord(list, len, 2, i, EQUAL_OP))
    {
      list[x++] = EQUAL_OP;
      i += 2;
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (icasestrcompn(list+i, "!=", 2) && WholeWord(list, len, 2, i, NOTEQUAL_OP))
    {
      list[x++] = NOTEQUAL_OP;
      i += 2;
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (icasestrcompn(list+i, "+=", 2) && IsValidAssignment(list, len, 2, i, TRUE))
    {
      list[x++] = list[i++];
      list[x++] = list[i++];
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (icasestrcompn(list+i, "-=", 2) && IsValidAssignment(list, len, 2, i, TRUE))
    {
      list[x++] = list[i++];
      list[x++] = list[i++];
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (icasestrcompn(list+i, "*=", 2) && IsValidAssignment(list, len, 2, i, TRUE))
    {
      list[x++] = list[i++];
      list[x++] = list[i++];
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (icasestrcompn(list+i, "/=", 2) && IsValidAssignment(list, len, 2, i, TRUE))
    {
      list[x++] = list[i++];
      list[x++] = list[i++];
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (icasestrcompn(list+i, "@=", 2) && IsValidAssignment(list, len, 2, i, TRUE))
    {
      list[x++] = list[i++];
      list[x++] = list[i++];
      BlankOut(list, x, i);
      HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
    }
    else if (!PrevNumber_ && !HexPrefix_ && !OctPrefix_ &&
             !HexNumber_ && !OctNumber_ &&
             icasestrcompn(list+i, "0x", 2) &&
             WholeWord(list, len, 2, i, HEXPREFIX_INDEX))
    {
      int y;
      int Start_ = i + 2;

      for (y = Start_; y < len; y++)
        if (!IsHexc(list[y]))
          break;

      if (y != Start_ && (y == len || (list[y] != '.' && (toupper(list[y]) != 'X' || !IsHexc(list[y+1])))))
      {
        if (toupper(list[y]) == 'N')
        {
          if (toupper(list[y-1]) == 'A' &&
              toupper(list[y+1]) == 'D')
          {
            --y;

            if (y != Start_ && (y == len || (y > 0 && list[y-1] != '.' && toupper(list[y-1]) != 'X')))
            {
              ChrString TestStr_(&list[i], y - i);
              if (IsIntHex(TestStr_.c_str()))
              {
                list[x++] = list[i++];
                list[x++] = list[i++];
                BlankOut(list, x, i);
                HexPrefix_ = Operand_ = TRUE;
                OctPrefix_ = PrevNumber_ = SciOp_ = FALSE;
                HexNumber_ = TRUE;
                HexLen_ = y - i;
              }
              else
              {
                Valid_ = 0;
                break;
              }
            }
            else if (!icasestrcompn(list+y, "XOR", 3))
            {
              Valid_ = 0;
              break;
            }
          }
          else
          {
            Valid_ = 0;
            break;
          }
        }
        else
        {
          ChrString TestStr_(&list[i], y - i);
          if (IsIntHex(TestStr_.c_str()))
          {
            list[x++] = list[i++];
            list[x++] = list[i++];
            BlankOut(list, x, i);
            HexPrefix_ = Operand_ = TRUE;
            OctPrefix_ = PrevNumber_ = SciOp_ = FALSE;
            HexNumber_ = TRUE;
            HexLen_ = y - i;
          }
          else
          {
            Valid_ = 0;
            break;
          }
        }

        CalcPtr_->SetNumberBase(16);
        if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
        {
          CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
          CalcPtr_ = CalcPtr_->GetCalcType();
          HexValid_ = TRUE;
        }
      }
      else if (icasestrcompn(list+y, "XOR", 3))
      {
        if (y != Start_ && (y == len || (y > 0 && list[y-1] != '.' && toupper(list[y-1]) != 'X')))
        {
          ChrString TestStr_(&list[i], y - i);
          if (IsIntHex(TestStr_.c_str()))
          {
            list[x++] = list[i++];
            list[x++] = list[i++];
            BlankOut(list, x, i);
            HexPrefix_ = Operand_ = TRUE;
            OctPrefix_ = PrevNumber_ = SciOp_ = FALSE;
            HexNumber_ = TRUE;
            HexLen_ = y - i;
          }
          else
          {
            Valid_ = 0;
            break;
          }
        }
        else
        {
          Valid_ = 0;
          break;
        }

        CalcPtr_->SetNumberBase(16);
        if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
        {
          CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
          CalcPtr_ = CalcPtr_->GetCalcType();
          HexValid_ = TRUE;
        }
      }
      else
      {
        Valid_ = 0;
        break;
      }
    }
    else if (list[i] == '\"')
    {
      int y, end_, assign_;
      int EndPt_ = 0;
      int StartPt_ = i;

      end_ = SciOp_ = FALSE;
      for (y = strlen(list) - 1; y > i; y--)
        if (!isspace(list[y]))
        {
          end_ = list[y] == '\"';
          EndPt_ = y;
          break;
        }

      assign_ = SciOp_ = FALSE;
      if (i)
        for (y = i - 1; y; y--)
          if (!isspace(list[y]))
          {
            assign_ = list[y] == '=';
            break;
          }

      if ((assign_ || ((!i || !y) && (list[0] == '\"' || isspace(list[0])))) &&
          end_ && EndPt_ && StartPt_ != EndPt_)
      {
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
        while (i <= EndPt_)
          list[x++] = list[i++];
        BlankOut(list, x, i);
      }
      else
      {
        Valid_ = 0;
        break;
      }
    }
    else if (icasestrcompn(list+i, "&", 1))
    {
      if (list[i+1] != '=' || !IsValidAssignment(list, len, 2, i, TRUE))
      {
        Valid_ = 0;
        break;
      }
      else
      {
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
        list[x++] = '&';
        list[x++] = '=';
        i += 2;
        BlankOut(list, x, i);
      }
    }
    else if (icasestrcompn(list+i, "|", 1))
    {
      if (list[i+1] != '=' || !IsValidAssignment(list, len, 2, i, TRUE))
      {
        Valid_ = 0;
        break;
      }
      else
      {
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
        list[x++] = '|';
        list[x++] = '=';
        i += 2;
        BlankOut(list, x, i);
      }
    }
    else if (icasestrcompn(list+i, "=", 1))
    {
      if (!IsValidAssignment(list, len, 1, i, TRUE))
      {
        Valid_ = 0;
        break;
      }
      else
      {
        HexPrefix_ = OctPrefix_ = PrevNumber_ = Operand_ = SciOp_ = FALSE;
        list[x++] = list[i++];
        BlankOut(list, x, i);
      }
    }
    else
    {
      UnOp_ = BinOp_ = PostOp_ =
      UnProc_ = ValidOp_ = FALSE;
      Boolean Condition_ =
      (UnProc_=IsUnprocessedSymbols(list[i])) ||
      (ValidOp_=HasValidOpCodes(list[i], ((i > 1) && HexPrefix_) ? 16:10)) ||
      (PostOp_=IsPostfixOperators(list[i]) && WholeWord(list, len, 1, i, POSTOPS_INDEX)) ||
      (BinOp_=IsBinaryOperators(list[i]) && WholeWord(list, len, 1, i, BINARYOPS_INDEX)) ||
      (UnOp_=IsUnaryOperators(list[i]) && WholeWord(list, len, 1, i, UNARYOPS_INDEX));
      PrevNumber_ = OpSymbols_ = HasOperandSymbols(list[i], HexValid_);

      if (Condition_ || OpSymbols_)
      {
        if (Uchar(list[i]) == NAN_CONST || Uchar(list[i]) == INF_CONST)
          PrevNumber_ = FALSE;

        if ((!OpSymbols_ && (PostOp_ || BinOp_ || UnOp_ || ValidOp_)) ||
            !(UnProc_ || OpSymbols_ || IsConstSymbols(list[i])))
          Operand_ = FALSE;

        if (IsConst_)
        {
          if ((OpSymbols_ || UnOp_ || IsConstSymbols(list[i])) ||
              !(BinOp_ || PostOp_ || ValidOp_ || UnProc_))
          {
            Valid_ = 0;
            break;
          }

          IsConst_ = FALSE;
        }
        else if (Operand_)
        {
          if (HexNumber_ && IsHexc(list[i]))
          {
            --HexLen_;
            if (HexLen_ == 0)
              HexNumber_ = FALSE;
          }
          else if (OctNumber_ && IsOctc(list[i]))
          {
            --OctLen_;
            if (OctLen_ == 0)
              OctNumber_ = FALSE;
          }
        }

        list[x++] = list[i++];
        BlankOut(list, x, i);

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
      }
      else
      {
        HexPrefix_ = (i > 1) && HexPrefix_;
        HexValid_ = CalcPtr_->GetCalcIndex() == CalculatorBase::LONGNUM;
        OpSymbols_ = HasOperandSymbols(list[i], HexValid_);
        Number_ = OpSymbols_ && (!Operand_ || (i == 0) || (i > 0 && (PrevNumber_ || (HexPrefix_ && HexValid_))));
        OctPrefix_ = !PrevNumber_ && !HexPrefix_ && !OctPrefix_ &&
                     !OctNumber_ && !HexNumber_ && list[i] == '0' && OpSymbols_;


        if (!WholeWord(list, len, 1, i, 0, 1, !OpSymbols_, Number_, HexValid_, 0, 0, 0, 0) ||
            (SciOp_ && !isdigit(list[i])))
        {
          Valid_ = 0;
          break;
        }

        if (!OctPrefix_ || !SciOp_)
        {
          list[x++] = list[i++];

          if (HexNumber_)
          {
            --HexLen_;
            if (HexLen_ == 0)
              HexNumber_ = FALSE;
          }
          else if (OctNumber_)
          {
            --OctLen_;
            if (OctLen_ == 0)
              OctNumber_ = FALSE;
          }
        }
        else
        {
          for (;list[i] == '0' && i < len; i++);
          if (i < len)
            list[x++] = list[i++];
          else
          {
            Valid_ = 0;
            break;
          }
        }

        Operand_ = PrevNumber_ = OpSymbols_;
        OctPrefix_ = OctPrefix_ && !SciOp_;
        BlankOut(list, x, i);

        if (OctPrefix_)
        {
          int y;
          int pref_ = i - 1;
          for (y = i; y < len && IsOctc(list[y]); y++);

          if ((y < len && (!IsDecc(list[y]) || list[y]=='0') && list[y] != '.') || y == len)
          {
            ChrString TestStr_(&list[pref_], y - pref_);
            if (IsIntOct(TestStr_.c_str()))
            {
              CalcPtr_->SetNumberBase(8);
              OctNumber_ = TRUE;
              OctLen_ = y - pref_;

              if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
              {
                CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
                CalcPtr_ = CalcPtr_->GetCalcType();
                HexValid_ = TRUE;
              }
            }
          }
        }

        SciOp_ =
        HexPrefix_ =
        OctPrefix_ = FALSE;
      }
    }

  if (Valid_)
    list[x] = '\0';
  else
    list[len] = 0;

  *arglist = list;
  return Valid_;
}

/*****************************************************************************/
int CalculatorBase::LookAhead(const char* list)
{
  if (IsIntHex(list) || IsIntOct(list))
    return ALTBASE_MODE;

  int Mode_ = AnswerMode();
  if (Mode_ == FRACTION_MODE && IsFraction(list))
    return FRACTION_MODE;

  ChrString Str_ = list;
  Str_.RemovePadding(" \t\n");

  if (Uchar(Str_[0]) == ANSSTR_CONST && Str_.strlen() == 1)
  {
    const char* LitStr_ = LiteralStr();
    if (IsIntHex(LitStr_) || IsIntOct(LitStr_))
      return ALTBASE_MODE;
    else
      return LITERAL_MODE;
  }
  else if (Uchar(Str_[0]) == FRACANS_CONST && Str_.strlen() == 1)
    return FRACTION_MODE;

  if (NumberBase() != 10)
    return ALTBASE_MODE;

  return FLOATINGPOINT_MODE;
}

/*****************************************************************************/
int CalculatorBase::FindAssignments(char* list, CalculatorBase* CalcPtr_,
                                    int Length_, int& Error_, int RecLevel_,
                                    int& StrAssign_, int& StrLength_, int SetIndexes_,
                                    AssignmentParsingData& ParseData_)
{
  Boolean FirstAlpha_        = ParseData_._FirstAlpha;
  Boolean LastDollar_        = ParseData_._LastDollar;
  Boolean AnsStrConstFound_  = ParseData_._AnsStrConstFound;
  Boolean FracAnsConstFound_ = ParseData_._FracAnsConstFound;
  char VarLetter_            = ParseData_._VarLetter;
  int FirstQuotePos_         = ParseData_._FirstQuotePos;
  int LastQuotePos_          = ParseData_._LastQuotePos;

  int i, x;
  int EqFound_ = 0;                            // equal sign found
  int AssignType_ = CalculatorBase::NO_ASSIGN; // Assignment type
  int EqPos_;                                  // equal sign position
  int StrVar_ = FALSE;                         // string var ($a...$z) found
  int VarLen_ = 0;                             // string variable length
  int Mode_;                                   // answer mode flag

  Boolean FirstQuote_ = FALSE;  // First from rightside quote char ->" found
  Boolean LastQuote_ = FALSE;   // Last from rightside quote char "<- found
  Boolean MemStoreSet_ = FALSE; // memory index set
  Boolean FirstChar_ = TRUE;    // first non w.s. character found
  Boolean NextChar_ = TRUE;     // next char after
                                // quote " or letter (a...z) found

  for (i = Length_; i > 0; i--)
  {
    if (IsAssignmentOp((Uchar*)list, i, 1) &&
          ((FirstQuote_ && LastQuote_) ||
           (!FirstQuote_ && !LastQuote_) ||
           i <= 3 || EvalStrings()))
    {
      if (RecLevel_ == 0)
        if (StrAssign_ = (FirstQuote_ && LastQuote_))
          StrLength_ = FirstQuotePos_ - (LastQuotePos_ + 1);

      // if assignment '=' found then determining the assignment type
      if (StrAssign_)
      {
        ChrString VarStr_(&list[LastQuotePos_ + 1], StrLength_);
        VarLen_ = VarStr_.strlen();
        Mode_ = AnswerMode();

        if (list[i-1] == '+' && Mode_ != FRACTION_MODE && Mode_ != ALTBASE_MODE)
          AssignType_ = CalculatorBase::STRING_CONCAT;
        else if (Mode_ == LITERAL_MODE ||
                   (!(Mode_ == ALTBASE_MODE && VarLen_ && (IsIntHex(VarStr_.c_str()) || IsIntOct(VarStr_.c_str()))) &&
                    !(Mode_ == FRACTION_MODE && VarLen_ && IsFraction(VarStr_.c_str()))))
          AssignType_ = CalculatorBase::STRING_ASSIGN;
        else
          AssignType_ = (list[i-1] == '+') ? CalculatorBase::ASSIGN_PLUS:
                        (list[i-1] == '-') ? CalculatorBase::ASSIGN_MINUS:
                        (list[i-1] == '*') ? CalculatorBase::ASSIGN_MULT:
                        (list[i-1] == '/') ? CalculatorBase::ASSIGN_DIV:
                        (list[i-1] == '@') ? CalculatorBase::ASSIGN_MOD:
                        (list[i-1] == '&') ? CalculatorBase::ASSIGN_AND:
                        (list[i-1] == '|') ? CalculatorBase::ASSIGN_OR:
                        (Mode_ == ALTBASE_MODE || Mode_ == FRACTION_MODE) ?
                                             CalculatorBase::STRING_ASSIGN:
                                             CalculatorBase::ASSIGN_ONLY;
      }
      else
      {
        StrVar_ = (FirstAlpha_ && LastDollar_) ? CalcPtr_->IsStringVar(toupper(VarLetter_) - 'A'):
                                                 (AnsStrConstFound_ || FracAnsConstFound_);

        if (StrVar_)
        {
          ChrString VarStr_ = CalcPtr_->GetStrVariable(toupper(VarLetter_) - 'A');
          VarLen_ = VarStr_.strlen();
          Mode_ = AnswerMode();

          if (list[i-1] == '+' && Mode_ != FRACTION_MODE && Mode_ != ALTBASE_MODE)
            AssignType_ = CalculatorBase::STRING_CONCAT;
          else if (Mode_ == LITERAL_MODE ||
                     (!(Mode_ == ALTBASE_MODE && VarLen_ && (IsIntHex(VarStr_.c_str()) || IsIntOct(VarStr_.c_str()))) &&
                      !(Mode_ == FRACTION_MODE && VarLen_ && IsFraction(VarStr_.c_str()))))
            AssignType_ = CalculatorBase::STRING_ASSIGN;
          else
            AssignType_ = (list[i-1] == '+') ? CalculatorBase::ASSIGN_PLUS:
                          (list[i-1] == '-') ? CalculatorBase::ASSIGN_MINUS:
                          (list[i-1] == '*') ? CalculatorBase::ASSIGN_MULT:
                          (list[i-1] == '/') ? CalculatorBase::ASSIGN_DIV:
                          (list[i-1] == '@') ? CalculatorBase::ASSIGN_MOD:
                          (list[i-1] == '&') ? CalculatorBase::ASSIGN_AND:
                          (list[i-1] == '|') ? CalculatorBase::ASSIGN_OR:
                          (Mode_ == ALTBASE_MODE || Mode_ == FRACTION_MODE) ?
                                               CalculatorBase::STRING_ASSIGN:
                                               CalculatorBase::ASSIGN_ONLY;
        }
        else
        {
          Mode_ = LookAhead(&list[i+1]);

          if (Mode_ == LITERAL_MODE)
          {
            if (list[i-1] == '+')
              AssignType_ = CalculatorBase::STRING_CONCAT;
            else
              AssignType_ = CalculatorBase::STRING_ASSIGN;
          }
          else
            AssignType_ = (list[i-1] == '+') ? CalculatorBase::ASSIGN_PLUS:
                          (list[i-1] == '-') ? CalculatorBase::ASSIGN_MINUS:
                          (list[i-1] == '*') ? CalculatorBase::ASSIGN_MULT:
                          (list[i-1] == '/') ? CalculatorBase::ASSIGN_DIV:
                          (list[i-1] == '@') ? CalculatorBase::ASSIGN_MOD:
                          (list[i-1] == '&') ? CalculatorBase::ASSIGN_AND:
                          (list[i-1] == '|') ? CalculatorBase::ASSIGN_OR:
                          (Mode_ == ALTBASE_MODE || Mode_ == FRACTION_MODE) ?
                                               CalculatorBase::STRING_ASSIGN:
                                               CalculatorBase::ASSIGN_ONLY;
        }
      }

      break;
    }
    else if (RecLevel_ == 0 && !isspace(list[i]))
    {
      if (FirstChar_)
      {
        FirstChar_ = FALSE;
        FirstQuote_ = list[i] == '\"';
        FirstAlpha_ = isalpha(list[i]) && InRange('A', list[i], 'z');

        if (FirstAlpha_)
          VarLetter_ = list[i];

        if (FirstQuote_)
          FirstQuotePos_ = i;
      }
      else if (FirstQuote_ || FirstAlpha_)
      {
        if (NextChar_)
          LastDollar_ = list[i] == '$';
        else
          LastDollar_ = FALSE;

        LastQuote_ = list[i] == '\"';
        NextChar_ = FALSE;

        if (LastQuote_)
          LastQuotePos_ = i;
      }
      else
      {
        if (Uchar(list[i]) == ANSSTR_CONST)
          AnsStrConstFound_ = TRUE;
        else if (Uchar(list[i]) == FRACANS_CONST)
          FracAnsConstFound_ = TRUE;
      }
    }
  }

  // if index is > 0 then assignment operator found
  if (i > 0)
  {
    // setting the index to the memory storage location
    // determined from the left side of the equation,
    // up to the next assignment operator or start of string.
    for (x = i - 1; x > 0; x--)
      if (IsAssignmentOp((Uchar*)list, x, 1))
      {
        EqPos_ = x;
        EqFound_ = 1;
        x++;
        break;
      }

    if (SetIndexes_)
      for (;x < i; x++)
        if (list[x] == '$' && islower(list[x + 1]))
        {
          CalcPtr_->SetIndex(list[x + 1] - 'a', AssignType_);
          MemStoreSet_ = TRUE;
        }

    Error_ = !MemStoreSet_;
  }
  else
    i = 0;

  if (EqFound_ && !Error_)
  {
    StrVar_                       = StrVar_ || StrAssign_;
    ParseData_._FirstAlpha        = FirstAlpha_;
    ParseData_._LastDollar        = LastDollar_;
    ParseData_._AnsStrConstFound  = AnsStrConstFound_;
    ParseData_._FracAnsConstFound = FracAnsConstFound_;
    ParseData_._VarLetter         = VarLetter_;
    ParseData_._FirstQuotePos     = FirstQuotePos_;
    ParseData_._LastQuotePos      = LastQuotePos_;

    FindAssignments(list, CalcPtr_, EqPos_, Error_,
                    RecLevel_ + 1, StrVar_, StrLength_,
                    SetIndexes_, ParseData_);
  }

  return i;
}

/*****************************************************************************/
int CalculatorBase::IsStringLiteral(const char* list, int& NewStart_, int& LiteralLen_,
                                    int Start_, int len, int InSetStr_)
{
  int z1, z2, x;
  int testset_ = 0;
  int InSet_ = 0;

  for (z1 = Start_; isspace(list[z1]) && z1 < len; z1++);
  if (list[z1] == '\"' || (InSetStr_ && list[z1] == '{'))
  {
    if (list[z1] == '{')
      testset_ = 1;

    for (z2 = len - 1; isspace(list[z2]) && z2 >= 0; z2--);
    if ((!testset_ && list[z2] == '\"') ||
        (testset_ && list[z2] == '}'))
    {
      z1++;
      x = z2 - z1;
      NewStart_ = z1;
      LiteralLen_ = x;

      if (!testset_ && x >= 0)
        return 1;
      else
      {
        InSet_ = (x >= 0) ? InSetNotation(list+z1):0;
        if (InSet_)
          return 2;
      }
    }
  }

  LiteralLen_ = 0;
  return 0;
}

/*****************************************************************************/
int CalculatorBase::ReplaceTokens(char** arglist, CalculatorBase* CalcPtr_, int FindAssign_,
                                  int& Error_, int& ReEvaluate_, int ReFormat_, int ChangeModes_,
                                  int Level_)
{
  int RecLimit_ = RecursionLimit();
  if (Level_ > RecLimit_)
  {
    char Buffer_[32];
    IntToStr(RecLimit_, Buffer_);
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_RECURSION_LIMIT, ERRMSG_RECURSION_LIMIT, Buffer_);
    return 0;
  }

  int i, x, y, EqPos_;
  int Index_;
  int EqFound_ = 0;
  char* list;
  char* oldlist = NULL;
  int len, newlen;
  int CalcType_ = CalcPtr_->GiveType();
  int PiLen_, ELen_, AnsLen_, AnsStrLen_, FracAnsLen_;
  char* Estr_;
  char* Pistr_;
  int StrAssign_ = FALSE;
  int StrConst_ = FALSE;
  int StrLength_;
  int LiteralAssignment_ = TRUE;
  int DoAssign_;
  int xlen;
  int StrVarCount_ = 0;
  int VarCount_ = 0;
  int UnresolvedSearched_ = 0;
  int LenAssigned_ = 0;
  int beg_, slen_;
  int SubStrVar_ = 0;
  int CheckSubStrVar_ = 0;
  AssignmentParsingData ParseData_;

  list = *arglist;
  len = strlen(list);
  newlen = len + 1;

  AnsLen_ = ::SafeStrLen(FloatStr());
  AnsStrLen_ = ::SafeStrLen(LiteralStr());
  FracAnsLen_ = ::SafeStrLen(FractionStr());

  if (FindAssign_)
  {
    if (IsStringLiteral(list, beg_, slen_, 0, len, 1))
      EqPos_ = i = 0;
    else
    {
      EqPos_ = i =
      FindAssignments(list, CalcPtr_, len - 1, Error_, 0, StrAssign_, StrLength_, ChangeModes_, ParseData_);
      if (Error_)
        return FALSE;
    }

    EqFound_ = i > 0;

    if (EqFound_)
      i++;

    if (CalcType_ == CalcPtr_->LONGNUM)
    {
      Pistr_ = NewString(LongNumber::vPI().ToString().c_str());
      Estr_ = NewString(LongNumber::vE().ToString().c_str());
    }
    else if (CalcType_ == CalcPtr_->LONGDOUBLE)
    {
      Pistr_ = NewString(M_PIl_STRING);
      Estr_ = NewString(M_El_STRING);
    }
    else
    {
      Pistr_ = NewString(M_PI_STRING);
      Estr_ = NewString(M_E_STRING);
    }

    PiLen_ = strlen(Pistr_);
    ELen_ = strlen(Estr_);
  }
  else
    i = 0;

  if (FindAssign_ && StrAssign_ && EqFound_)
  {
    x = i;
    newlen += StrLength_;
    LenAssigned_ = TRUE;
  }
  else
    x = 0;

  if (!LenAssigned_)
  {
    newlen += 5;
    for (x = i; i < len; i++)
      if ((Uchar)(list[i]) == E_CONST)
        newlen += ELen_;
      else if ((Uchar)(list[i]) == PI_CONST)
        newlen += PiLen_;
      else if ((Uchar)(list[i]) == ANS_CONST)
        newlen += AnsLen_;
      else if ((Uchar)(list[i]) == ANSSTR_CONST)
        newlen += AnsStrLen_;
      else if ((Uchar)(list[i]) == FRACANS_CONST)
        newlen += FracAnsLen_;
      else if (list[i] == '$')
      {
        Index_ = toupper(list[i + 1]) - 'A';
        newlen += CalcPtr_->GetVariableLength(Index_);
        if (CalcPtr_->IsStringVar(Index_))
          newlen += 2;
      }

    LenAssigned_ = TRUE;
  }

  // substituting values for variables in the right side of
  // the equation
  if (newlen)
  {
    Boolean LitStr_ = FALSE;
    Boolean FracStr_ = FALSE;
    VarCount_ = 0;
    StrVarCount_ = 0;
    oldlist = list;
    int IsNull_ = 0;
    int InStrType_ = 0;
    int oldlen = ::SafeStrLen(oldlist);
    list = (char*)allocmem(newlen+1, sizeof(char), list);

    if (FindAssign_)
    {
      if (StrAssign_ && EqFound_)
      {
        for (i = len - 1; i > x; i--)
          if (oldlist[i] == '\"')
            break;

        for (y = x; y < len; y++)
          if (oldlist[y] == '\"')
            break;

        y++;
        x = i - y;

        if (x > 0)
          strncpy(list, &oldlist[y], x);
        else
          x = 0;

        list[x] = 0;
        IsNull_ = IsNullSetStr(list);

        if (SetAssignment())
          SetSetAssignment(TRUE, list, IsNull_);
        else
          SetStringAssignment(TRUE, list);

        if (ChangeModes_ && !EvalStrings())
          SetAnswerMode(LITERAL_MODE, AtLowestLevel(Level_));

        if (EvalStrings())
          SetFloatStr(list);
        SetLiteralStr(list);
      }
      else
      {
        if (SetAssignment())
          SetSetAssignment(FALSE, NULL, FALSE);
        else
          SetStringAssignment(FALSE);

        oldlen = ::SafeStrLen(&oldlist[x]);
        InStrType_ = !EqFound_ ? IsStringLiteral(oldlist, beg_, slen_, x, len, 1):0;

        if (InStrType_)
        {
          if (slen_ > 0)
            strncpy(list, &oldlist[beg_], slen_);

          list[slen_] = 0;
          if (ChangeModes_ && (!EvalStrings() || InStrType_ == 2))
            SetAnswerMode(LITERAL_MODE, AtLowestLevel(Level_));

          if (EvalStrings() && InStrType_ == 1)
            SetFloatStr(list);
          SetLiteralStr(list);
          SetSkipEval(TRUE);
          StrConst_ = TRUE;
          x = slen_;
        }

        if (!StrConst_)
        for (i = x, x = 0, CheckSubStrVar_ = 1; i < len; i++)
        {
          if ((Uchar)(oldlist[i]) == E_CONST)
          {
            strcpy(&list[x], Estr_);
            x += strlen(Estr_);
            ++VarCount_;
          }
          else if ((Uchar)(oldlist[i]) == PI_CONST)
          {
            strcpy(&list[x], Pistr_);
            x += strlen(Pistr_);
            ++VarCount_;
          }
          else if ((Uchar)(oldlist[i]) == ANS_CONST)
          {
            int SetToFloat_ = (AnswerMode() != LITERAL_MODE || InitialMode() ||
                              (SubStrVar_ && EvalStrings()));

            if (ChangeModes_ && SetToFloat_)
              CalcPtr_->SetAnswerMode(FLOATINGPOINT_MODE);

            const char* Answer_ = FloatStr();
            xlen = ::SafeStrLen(Answer_);

            if (xlen)
            {
              ::SafeStrCpy(&list[x], Answer_);
              x += xlen;
            }
            else
            {
              strcpy(&list[x], "0");
              x += 1;
            }

            ++VarCount_;
          }
          else if ((Uchar)(oldlist[i]) == ANSSTR_CONST)
          {
            const char* Literal_ = LiteralStr();
            xlen = ::SafeStrLen(Literal_);

            if (xlen)
            {
              LitStr_ = TRUE;
              int IsHexVal_ = IsIntHex(Literal_);
              int IsOctVal_ = IsIntOct(Literal_);
              int SetToAltBase_ = (IsHexVal_ || IsOctVal_) &&
                                  (AnswerMode() != LITERAL_MODE || InitialMode());

              ++StrVarCount_;
              ::SafeStrCpy(&list[x], Literal_);
              x += xlen;

              if (SetToAltBase_ || (SubStrVar_ && EvalStrings()))
              {
                CalcPtr_->SetNumberBase(IsHexVal_ ? 16:8);
                if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
                {
                  CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
                  CalcPtr_ = CalcPtr_->GetCalcType();
                }
              }

              if (!IsHexVal_ && !IsOctVal_ && CheckSubStrVar_)
              {
                SubStrVar_ = Literal_[0] == '$' &&
                             isalpha(Literal_[1]) &&
                             InRange('A', Literal_[1], 'z');
                if (!SubStrVar_)
                  CheckSubStrVar_ = 0;
              }

              if (oldlen == 1)    // only ansstr code left
              {
                if (ChangeModes_)
                  if (SetToAltBase_)
                    SetAnswerMode(ALTBASE_MODE, AtLowestLevel(Level_));
                  else if (!EvalStrings())
                    SetAnswerMode(LITERAL_MODE, AtLowestLevel(Level_));

                if (EvalStrings())
                  SetFloatStr(Literal_);
                SetLiteralStr(Literal_);
              }
            }
            else
            {
              strcpy(&list[x], "0");
              x += 1;
            }

            ++VarCount_;
          }
          else if ((Uchar)(oldlist[i]) == FRACANS_CONST)
          {
            const char* Fraction_ = FractionStr();
            xlen = ::SafeStrLen(Fraction_);

            if (xlen)
            {
              ++StrVarCount_;
              ::SafeStrCpy(&list[x], Fraction_);
              x += xlen;

              if (oldlen == 1)       // only fracans code left
              {
                FracStr_ = TRUE;
                int SetToFrac_ = AnswerMode() != LITERAL_MODE || InitialMode();

                if (ChangeModes_ && SetToFrac_)
                  SetAnswerMode(FRACTION_MODE, AtLowestLevel(Level_));

                if (EvalStrings())
                  SetFloatStr(Fraction_);
                SetFractionStr(Fraction_);
              }
            }
            else
            {
              strcpy(&list[x], "0");
              x += 1;
            }

            ++VarCount_;
          }
          else if (oldlist[i] == '$')
          {
            Index_ = toupper(oldlist[i + 1]) - 'A';

            if (CalcPtr_->IsStringVar(Index_))
            {
              ChrString VarStr_ = CalcPtr_->GetStrVariable(Index_);
              int IsOctVal_ = IsIntOct(VarStr_.c_str());
              int IsHexVal_ = IsIntHex(VarStr_.c_str());
              int SetToAltBase_ = (IsHexVal_ || IsOctVal_) &&
                                  (AnswerMode() != LITERAL_MODE || InitialMode());

              ++StrVarCount_;
              if (SetToAltBase_ || (SubStrVar_ && EvalStrings()))
              {
                CalcPtr_->SetNumberBase(IsHexVal_ ? 16:8);
                if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
                {
                  CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
                  CalcPtr_ = CalcPtr_->GetCalcType();
                }
              }

              if (!IsHexVal_ && !IsOctVal_ && CheckSubStrVar_)
              {
                SubStrVar_ = VarStr_[0] == '$' &&
                             isalpha(VarStr_[1]) &&
                             InRange('A', VarStr_[1], 'z');
                if (!SubStrVar_)
                  CheckSubStrVar_ = 0;
              }

              if (oldlen == 2)    // only variable left
              {
                if (ChangeModes_)
                  if (SetToAltBase_)
                    SetAnswerMode(ALTBASE_MODE, AtLowestLevel(Level_));
                  else if (!EvalStrings())
                    SetAnswerMode(LITERAL_MODE, AtLowestLevel(Level_));

                if (EvalStrings())
                  SetFloatStr(VarStr_.c_str());
                SetLiteralStr(VarStr_.c_str());
              }
            }

            ++VarCount_;
            CalcPtr_->SubstToken(list, x, Index_);
            i++;
          }
          else
          {
            list[x++] = oldlist[i];
            if (!isspace(oldlist[i]))
              LiteralAssignment_ = FALSE;
          }
        } // var. sub loop
      }
    }
    else
    {
      i = x;
      InStrType_ = IsStringLiteral(oldlist, beg_, slen_, x, len, 1);

      if (InStrType_)
      {
        if (slen_ > 0)
          strncpy(list, &oldlist[beg_], slen_);

        list[slen_] = 0;
        if (ChangeModes_ && (!EvalStrings() || InStrType_ == 2))
          SetAnswerMode(LITERAL_MODE, AtLowestLevel(Level_));

        if (EvalStrings() && InStrType_ == 1)
          SetFloatStr(list);
        SetLiteralStr(list);

        SetSkipEval(TRUE);
        StrConst_ = TRUE;
        x = slen_;
      }

      if (!StrConst_)
        for (x = 0; i < len; i++)
        {
          if (oldlist[i] == '$')
          {
            Index_ = toupper(oldlist[i + 1]) - 'A';
            CalcPtr_->SubstToken(list, x, Index_);
            ++VarCount_;
            i++;
          }
          else
            list[x++] = oldlist[i];
        }
    }

    list[x] = 0;
    if (!Error_ && EvalStrings() && ReFormat_)
    {
      formatlist(&list, CalcPtr_);
      CalcPtr_ = CalcPtr_->GetCalcType();
      ReEvaluate_ = HasUnresolvedNames(list);
      UnresolvedSearched_ = TRUE;
    }

    DoAssign_ = LiteralAssignment_ && EqFound_ && !Error_ && VarCount_ &&
                StrVarCount_ == VarCount_ && !ReEvaluate_ && ChangeModes_ &&
                !(EvalStrings() && CalcPtr_->HasVariables(list));

    if (DoAssign_)
    {
      strcpy(list, oldlist);
      x = EqPos_ + 1;
      list[x++] = '\"';
      if (LitStr_)
        CalcPtr_->SubstToken(list, x, CalculatorBase::LITERAL_MODE);
      else if (FracStr_)
        CalcPtr_->SubstToken(list, x, CalculatorBase::FRACTION_MODE);
      else
        CalcPtr_->SubstToken(list, x, Index_);
      list[x++] = '\"';
      list[x] = 0;
      ResetIndexes();
      ReplaceTokens(&list, CalcPtr_, TRUE, Error_, ReEvaluate_, ReFormat_, ChangeModes_, Level_ + 1);
      x = strlen(list);
    }
  }

  if (FindAssign_)
  {
    // Deallocate Pi and E constant strings
    MemMatrix::Matrix().Deallocate(Pistr_);
    MemMatrix::Matrix().Deallocate(Estr_);
    Pistr_ = Estr_ = NULL;
  }

  list[x] = 0;
  *arglist = list;

  if (!Error_ && EvalStrings() && CalcPtr_->HasVariables(list))
  {
    if (!UnresolvedSearched_ && ReFormat_)
    {
      formatlist(&list, CalcPtr_);
      CalcPtr_ = CalcPtr_->GetCalcType();
      ReEvaluate_ = HasUnresolvedNames(list);
      UnresolvedSearched_ = TRUE;
    }

    ReplaceTokens(&list, CalcPtr_, FindAssign_, Error_, ReEvaluate_, ReFormat_, ChangeModes_, Level_ + 1);

    if (ReEvaluate_ && ReFormat_)
    {
      formatlist(&list, CalcPtr_);
      CalcPtr_ = CalcPtr_->GetCalcType();
      *arglist = list;
    }

    ReEvaluate_ = HasUnresolvedNames(list);
    DoAssign_ = LiteralAssignment_ && EqFound_ && !Error_ && VarCount_ &&
                StrVarCount_ == VarCount_ && !ReEvaluate_ && ChangeModes_;

    if (DoAssign_)
    {
      SetFloatStr(list);
      SetLiteralStr(list);
      strcpy(list, oldlist);
      x = EqPos_ + 1;
      list[x++] = '\"';
      CalcPtr_->SubstToken(list, x, CalculatorBase::LITERAL_MODE);
      list[x++] = '\"';
      list[x] = 0;
      ResetIndexes();
      ReplaceTokens(&list, CalcPtr_, TRUE, Error_, ReEvaluate_, ReFormat_, ChangeModes_, Level_ + 1);
      *arglist = list;
    }
  }

  if (!StrAssign_)
  {
    int Mode_ = AnswerMode();
    int SingleValue_ = FALSE;
    int IsHexVal_ = IsIntHex(list);
    int IsOctVal_ = IsIntOct(list);

    if (IsHexVal_ || IsOctVal_)
    {
      if (Mode_ != ALTBASE_MODE)
        Mode_ = ALTBASE_MODE;
      SingleValue_ = TRUE;
    }
    else if (Mode_ == ALTBASE_MODE)
    {
      IsHexVal_ = IsIntHex(list);
      IsOctVal_ = IsIntOct(list);

      if (IsHexVal_ || IsOctVal_)
        SingleValue_ = TRUE;
      else if (ChangeModes_)
        SetAnswerMode(FLOATINGPOINT_MODE);
    }
    else if (Mode_ == FRACTION_MODE)
    {
      if (IsFraction(list) && !InitialMode())
        SingleValue_ = TRUE;
      else if (ChangeModes_)
        SetAnswerMode(FLOATINGPOINT_MODE);
    }
    else if (ChangeModes_)
    {
      len = strlen(list);
      int Base_ = NumberBase();

      if (Base_ != 10)
      {
        for (y = 0; y < len; y++)
          if (!HasValidOpCodes(list[y], Base_))
          {
            SetAnswerMode(FLOATINGPOINT_MODE);
            break;
          }
      }
      else if (Mode_ == LITERAL_MODE)
      {
        for (y = 0; y < len; y++)
          if (HasValidOpCodes(list[y], 10))
          {
            SetAnswerMode(FLOATINGPOINT_MODE);
            break;
          }
      }
    }

    if (SingleValue_)
    {
      if (EqFound_)
      {
        if (!DoAssign_ && oldlist)
        {
          if (Mode_ == LITERAL_MODE || Mode_ == ALTBASE_MODE)
          {
            if (EvalStrings())
              SetFloatStr(list);
            SetLiteralStr(list);
          }
          else
          {
            if (EvalStrings())
              SetFloatStr(list);
            SetFractionStr(list);
          }

          if (ChangeModes_)
          {
            strcpy(list, oldlist);
            x = EqPos_ + 1;
            list[x++] = '\"';
            CalcPtr_->SubstToken(list, x, Mode_);
            list[x++] = '\"';
            list[x] = 0;
            ResetIndexes();
            ReplaceTokens(&list, CalcPtr_, TRUE, Error_, ReEvaluate_, ReFormat_, ChangeModes_, Level_ + 1);
            *arglist = list;
          }
        }
      }
      else
      {
        if (Mode_ == LITERAL_MODE || Mode_ == ALTBASE_MODE)
        {
          if (::SafeStrCmp(list, LiteralStr()) != 0)
          {
            if (EvalStrings())
              SetFloatStr(list);
            SetLiteralStr(list);
          }
        }
        else
        {
          if (::SafeStrCmp(list, FractionStr()) != 0)
          {
            if (EvalStrings())
              SetFloatStr(list);
            SetFractionStr(list);
          }
        }
      }

      if (Mode_ == ALTBASE_MODE)
      {
        CalcPtr_->SetNumberBase(IsHexVal_ ? 16:8);
        if (CalcPtr_->GetCalcIndex() != CalculatorBase::LONGNUM)
        {
          CalcPtr_->SwitchCalcType(CalculatorBase::LONGNUM);
          CalcPtr_ = CalcPtr_->GetCalcType();
        }
      }

      if (ChangeModes_ && !(Mode_ == LITERAL_MODE && EvalStrings()))
        SetAnswerMode(Mode_, AtLowestLevel(Level_));
    }
  }

  if (newlen)
  {
    MemMatrix::Matrix().Deallocate(oldlist);
    oldlist = NULL;
  }

  *arglist = list;
  return EqFound_;
}

/******************** RPN notation evaluation procedures ********************/
/*****************************************************************************
Procedure to test if a string represents a valid operand
*/
int CalculatorBase::isoperand(char* str)
{
  if (strlen(str) > 1)
    return TRUE;

  return HasOperandSymbols(*str);
}

/*****************************************************************************
Procedure to allocate memory to the operands array
PASSED:
  block : the operands array
USES:
  uses allocateoperand to allocate memory to each operand array element
  uses allocmem to allocate memory
CHANGED:
  the operands array is allocated with the memory to run the program
*/
void CalculatorBase::initarrays(Mcalc_Operand*** block, char*** list)
{
  int i;

  *block = (Mcalc_Operand**)allocmem(MAXVALUES,sizeof(Mcalc_Operand*),*block);
  for (i=0;i<MAXVALUES;i++)
    allocateoperand(*block+i,1);

 *list = (char**)allocmem(MAXARGS,sizeof(char*),*list);
}

/*****************************************************************************
Procedure to free memory from the operands array and its elements
PASSED:
  block : the operands array
USES:
  uses freeoperand to free the operand element of the array
CHANGED:
  all memory for the operands array and all its elements are freed
*/
void CalculatorBase::freememory(Mcalc_Operand*** block)
{
  int i;
  for (i=0;i<MAXVALUES;i++)
    freeoperand(*block+i);

  MemMatrix::Matrix().Deallocate(*block);
  *block = NULL;
}

/*****************************************************************************
Procedure to free the argument list and its element strings
PASSED:
  list : the address of the argument list
  wordcount : the count of the words in the argument list
CHANGED:
  list and its elements are freed from memory and assigned to NULL
*/
void CalculatorBase::freearglist(char*** list, int wordcount)
{
  int i;
  if (*list)
  {
    for (i=0;i<wordcount;i++)
    {
      MemMatrix::Matrix().Deallocate((*list)[i]);
      (*list)[i] = NULL;
    }

    MemMatrix::Matrix().Deallocate(*list);
    *list = NULL;
  }
}

/*****************************************************************************
Procedure to free the string arguments popped from the RPN stack
*/
void CalculatorBase::freeargs(char*** args)
{
  MemMatrix::Matrix().Deallocate((*args)[2]);
  MemMatrix::Matrix().Deallocate((*args)[3]);
  (*args)[2] = NULL;
  (*args)[3] = NULL;
}

/*****************************************************************************/
void CalculatorBase::freelist(char*** arglist)
{
  freeargs(arglist);
  MemMatrix::Matrix().Deallocate((*arglist)[0]);
  (*arglist)[0] = NULL;
}

/**************************** Display Procedures ****************************/
/*****************************************************************************/
ChrString CalculatorBase::DisplayVariable(CalculatorBase* CalcPtr_, char* Original_, int Index_, Boolean SetIoState_)
{
  ChrString Str_;
  Original_ = RemovePadding(Original_, " \t\n");

  if (Index_ == FLOATINGPOINT_MODE)
  {
    const char* Float_ = FloatStr();
    if (::SafeStrLen(Float_))
      Str_ = Float_;
  }
  else if (Index_ == LITERAL_MODE || Index_ == ALTBASE_MODE)
  {
    const char* Literal_ = LiteralStr();
    if (::SafeStrLen(Literal_))
      Str_ = Literal_;
  }
  else if (Index_ == FRACTION_MODE)
  {
    const char* Frac_ = FractionStr();
    if (::SafeStrLen(Frac_))
      Str_ = Frac_;
  }
  else
    Str_ = CalcPtr_->ToString(Index_);

  if (InAsyncMode() || InProgramMode())
  {
    if (!SetIoState_)
    {
      Str_ += "\n";
      Str_ = ChrString(Original_) + ChrString(" = ") + Str_;
      return Str_;
    }

    char* Buffer_ = Str_.strlen() ? ::NewString(Str_.c_str()):NULL;

    if (InProgramMode())
    {
      if (::SafeStrLen(Buffer_))
        SetFloatStr(Buffer_);
      else
        SetHasAnswer(FALSE);

      if (InAsyncMode())
        _WaitForResponse = Mcalc_IOState::OUTPUT_READY;
    }
    else
      DisplayToProgramOutput(Buffer_);

    MemMatrix::Matrix().Deallocate(Buffer_);
  }
  else
    fprintf(GetOutputFile(), ">> %s = %s\n", Original_, Str_.c_str());

  return Str_;
}

/****************************************************************************/
void CalculatorBase::DisplayToString(Mcalc_Operand** operands, char* original)
{
  int i;
  Mcalc_Operand *result = operands[RESULTINDEX];

  const char* ErrorStr_ = "CALC ERROR";
  size_t len = result->size + strlen(ErrorStr_);
  char* AnswerBuf_ = NULL;
  AnswerBuf_ = (char*)allocmem(len+1, sizeof(char), AnswerBuf_);
  char* Start_ = AnswerBuf_;
  SetOriginalStr(original);

  if (HasErrors())
    sprintf(AnswerBuf_, ErrorStr_);
  else if (result->size)
  {
    if (result->sign==Mcalc_SignType::Minus)
    {
      sprintf(AnswerBuf_, "-");
      ++AnswerBuf_;
    }
    else if (SHOWPOS)
    {
      sprintf(AnswerBuf_, "+");
      ++AnswerBuf_;
    }

    sprintf(AnswerBuf_, "%s", result->value);
    AnswerBuf_ += result->size;
  }
  else
    AnswerBuf_[0] = 0;

  AnswerBuf_ = Start_;

  if (HasErrors())
  {
    SetLiteralStr(AnswerBuf_);
    SetFloatStr(AnswerBuf_);
    SetFractionStr(AnswerBuf_);
    SetAnswerMode(LITERAL_MODE, TRUE);
    MemMatrix::Matrix().Deallocate(AnswerBuf_);
  }
  else
    if (strlen(AnswerBuf_) || (AnswerMode() == FLOATINGPOINT_MODE))
    {
      SetFloatStr(AnswerBuf_);
      MemMatrix::Matrix().Deallocate(AnswerBuf_);
    }
}

/****************************************************************************/
void CalculatorBase::DisplayAnswerStringHelper(FILE* Fptr_)
{
  const char* Answer_ = AnswerStr();

  if (SafeStrLen(Answer_))
  {
    if (!InAsyncMode() && !InProgramMode())
      fprintf(Fptr_, ">> %s = ", OriginalStr());

    fprintf(Fptr_, "%s", Answer_);

    if (!InAsyncMode() && !InProgramMode())
      fprintf(Fptr_, "\n");
  }
}

/****************************************************************************/
void CalculatorBase::DisplayAnswerString()
{
  if (InAsyncMode())
  {
    if (!programoutputfile || !filestatus[Mcalc_IOType::ProgramOutput])
    {
      filestatus[Mcalc_IOType::ProgramOutput] = openfile(&files[Mcalc_IOType::ProgramOutput],PROG_OUTPUT_FILE,"w",TRUE);
      programoutputfile = files[Mcalc_IOType::ProgramOutput];
    }

    if (filestatus[Mcalc_IOType::ProgramOutput])
    {
      DisplayAnswerStringHelper(programoutputfile);
      fclose(programoutputfile);
      programoutputfile = NULL;
      filestatus[Mcalc_IOType::ProgramOutput] = 0;
    }

    if (InProgramMode())
    {
      if (_GraphingOp && InAsyncMode())
        _WaitForResponse = Mcalc_IOState::GRAPH_OUTPUT;
      else
        _WaitForResponse = Mcalc_IOState::OUTPUT_READY;
    }
    else
      SetIoState(Mcalc_IOState::CALC_RESPONSE);
  }
  else
    DisplayAnswerStringHelper(GetOutputFile());
}

/****************************************************************************/
void CalculatorBase::DisplayToUserPrompt(const char* Str_)
{
  if (InAsyncMode())
  {
    if (!userpromptfile || !filestatus[Mcalc_IOType::UserPrompt])
    {
      filestatus[Mcalc_IOType::UserPrompt] = openfile(&files[Mcalc_IOType::UserPrompt], USER_PROMPT_FILE, "w",TRUE);
      userpromptfile = files[Mcalc_IOType::UserPrompt];
    }

    if (filestatus[Mcalc_IOType::UserPrompt])
    {
      if (Str_ && strlen(Str_))
        fputs(Str_, userpromptfile);
      fclose(userpromptfile);
      userpromptfile = NULL;
      filestatus[Mcalc_IOType::UserPrompt] = 0;
    }

    if (InProgramMode())
      _WaitForResponse = Mcalc_IOState::INPUT_REQUIRED;
    else
      SetIoState(Mcalc_IOState::INPUT_REQUIRED);
  }
}

/****************************************************************************/
void CalculatorBase::DisplayToProgramOutput(const char* Str_)
{
  if (InAsyncMode())
  {
    if (!programoutputfile || !filestatus[Mcalc_IOType::ProgramOutput])
    {
      filestatus[Mcalc_IOType::ProgramOutput] = openfile(&files[Mcalc_IOType::ProgramOutput],PROG_OUTPUT_FILE,"w",TRUE);
      programoutputfile = files[Mcalc_IOType::ProgramOutput];
    }

    if (filestatus[Mcalc_IOType::ProgramOutput])
    {
      if (Str_ && strlen(Str_))
        fputs(Str_, programoutputfile);
      fclose(programoutputfile);
      programoutputfile = NULL;
      filestatus[Mcalc_IOType::ProgramOutput] = 0;
    }

    if (InProgramMode())
    {
      if (HasErrors())
        _WaitForResponse = Mcalc_IOState::CALC_ERROR;
      else
        _WaitForResponse = Mcalc_IOState::OUTPUT_READY;
    }
    else
    {
      if (HasErrors())
        SetIoState(Mcalc_IOState::CALC_ERROR);
      else
        SetIoState(Mcalc_IOState::CALC_RESPONSE);
    }
  }
}

/****************************************************************************/
void CalculatorBase::PromptAsyncUser(const char* Str_, int GetResponse_)
{
  ChrString ProgTemp_;
  char* ProgBuf_ = NULL;
  CalcProgram* Program_ = ActiveProgram();
  Program_->SetCalculator(GetCalcType());

  ProgTemp_ = GetResponse_ ? "calc_input ":"disp ";
  ProgTemp_ += ChrString("\"") + ChrString(Str_) + ChrString("\":");
  ProgBuf_ = ::NewString(ProgTemp_.c_str());
  Program_->LoadBuffer(ProgBuf_);
  Program_->RunProgram();
  Program_->UnloadProgram();
  MemMatrix::Matrix().Deallocate(ProgBuf_);
}

/****************************************************************************/
void CalculatorBase::DisplayMessage(FILE* Fptr_, const char* Str_)
{
  if (!SuppressOutput())
    if (InAsyncMode())
      DisplayToProgramOutput(Str_);
    else
      fputs(Str_, Fptr_);
}

/****************************************************************************/
void CalculatorBase::DisplayMessage(FILE* Fptr_, const char* FormatStr_, const char* Str_)
{
  char StrBuffer_[128];

  if (!SuppressOutput())
  {
    if (InAsyncMode())
    {
      sprintf(StrBuffer_, FormatStr_, Str_);
      DisplayToProgramOutput(StrBuffer_);
    }
    else
      fprintf(Fptr_, FormatStr_, Str_);
  }
  else
    strcpy(_CurrentFunctionName, Str_);
}

/****************************************************************************/
void CalculatorBase::DisplayErrorMessage(FILE* Fptr_, int ErrVal_, const char* FormatStr_, const char* Str_)
{
  SetError(ErrVal_);

  if (Fptr_ && Fptr_ == stdout)
  {
    if (!SuppressOutput())
    {
      DisplayMessage(Fptr_, FormatStr_, Str_);
      if (InAsyncMode())
        fprintf(Fptr_, FormatStr_, Str_);
      CloseOutputFile();
    }
    else
      strcpy(_CurrentFunctionName, Str_);
  }
  else
  {
    if (!SuppressOutput())
    {
      printf(FormatStr_, Str_);
      if (Fptr_)
        DisplayMessage(Fptr_, FormatStr_, Str_);
      CloseOutputFile();
    }
    else
      strcpy(_CurrentFunctionName, Str_);
  }

  GetLogFile();
  if (logfile)
  {
    fprintf(logfile, FormatStr_, Str_);
    fflush(logfile);
  }

  SetHasAnswer(false);
}

/****************************************************************************/
void CalculatorBase::DisplayErrorMessage(FILE* Fptr_, int ErrVal_, const char* Str_)
{
  SetError(ErrVal_);

  if (Fptr_ && Fptr_ == stdout)
  {
    if (!SuppressOutput())
    {
      DisplayMessage(Fptr_, Str_);
      if (InAsyncMode())
        fputs(Str_, Fptr_);
      CloseOutputFile();
    }
  }
  else
  {
    if (!SuppressOutput())
    {
      fputs(Str_, stdout);
      if (Fptr_)
        DisplayMessage(Fptr_, Str_);
      CloseOutputFile();
    }
  }

  GetLogFile();
  if (logfile)
  {
    fputs(Str_, logfile);
    fflush(logfile);
  }

  SetHasAnswer(false);
}

/****************************************************************************/
void CalculatorBase::DisplayErrorMessageForCode(FILE* Fptr_, int ErrVal_)
{
  if (ErrVal_ >= Mcalc_Error::MAX_ERRORS)
    return;

  if (Fptr_ && Fptr_ == stdout)
  {
    if (!SuppressOutput() && _ErrorMessageArray[ErrVal_])
      if (_ErrorMessageArgCnt[ErrVal_])
      {
        DisplayMessage(Fptr_, _ErrorMessageArray[ErrVal_], _CurrentFunctionName);
        if (InAsyncMode())
          fprintf(Fptr_, _ErrorMessageArray[ErrVal_], _CurrentFunctionName);
        CloseOutputFile();
      }
      else
      {
        DisplayMessage(Fptr_, _ErrorMessageArray[ErrVal_]);
        if (InAsyncMode())
          fputs(_ErrorMessageArray[ErrVal_], Fptr_);
        CloseOutputFile();
      }
  }
  else
  {
    if (!SuppressOutput() && _ErrorMessageArray[ErrVal_])
      if (_ErrorMessageArgCnt[ErrVal_])
      {
        printf(_ErrorMessageArray[ErrVal_], _CurrentFunctionName);
        if (Fptr_)
          DisplayMessage(Fptr_, _ErrorMessageArray[ErrVal_], _CurrentFunctionName);
        CloseOutputFile();
      }
      else
      {
        fputs(_ErrorMessageArray[ErrVal_], stdout);
        if (Fptr_)
          DisplayMessage(Fptr_, _ErrorMessageArray[ErrVal_]);
        CloseOutputFile();
      }
  }
}

/****************************** Help Procedure ******************************/
/****************************************************************************/
void CalculatorBase::helpmessage(FILE* fp, const char* msg_)
{
  int dum_;
  int HelpArg1_ = ::SafeStrLen(msg_) == 4 && icasestrcomp(msg_, "help");
  int HelpArg2_ = ::SafeStrLen(msg_) == 6 && icasestrcomp(msg_, "--help");
  int HelpArg3a_ = ::SafeStrLen(msg_) == 7 && icasestrcomp(msg_, "version");
  int HelpArg3b_ = ::SafeStrLen(msg_) == 9 && icasestrcomp(msg_, "--version");

  if (!msg_ || HelpArg1_ || HelpArg2_ || HelpArg3a_ || HelpArg3b_)
  {
    if (HelpArg3a_ || HelpArg3b_)
    {
      fputs(HELPMSG_VERSION, fp);
      return;
    }

    fputs(HELPMSG1, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG2, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG3a, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG3b, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG3c, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG3d, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG3e, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG3f, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG3g, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG3h, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);

    fputs(HELPMSG4a, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG4b, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);

    fputs(HELPMSG5, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG6, fp);
    if (fp == stdout)
      dum_ = fgetc(stdin);
    fputs(HELPMSG7, fp);
  }
  else
  {
    HelpArg1_ = ::SafeStrLen(msg_) > 4 && icasestrcompn(msg_, "help", 4);
    HelpArg2_ = ::SafeStrLen(msg_) > 6 && icasestrcompn(msg_, "--help", 6);
    const char* FncName_ = HelpArg1_ ? &msg_[4]:
                           HelpArg2_ ? &msg_[6]:NULL;

    if (!FncName_)
      return;

    if (icasestrcomp(FncName_, "VERSION"))
      fputs(HELPMSG_VERSION, fp);
    else if (icasestrcomp(FncName_, "ANS"))
      fputs(HELPMSG_FNC_ANS, fp);
    else if (icasestrcomp(FncName_, "ANSSTR"))
      fputs(HELPMSG_FNC_ANSSTR, fp);
    else if (icasestrcomp(FncName_, "FRACANS"))
      fputs(HELPMSG_FNC_FRACANS, fp);
    else if (icasestrcomp(FncName_, "DUMPVARS"))
      fputs(HELPMSG_FNC_DUMPVARS, fp);
    else if (icasestrcomp(FncName_, "GRAPHALLPTS"))
      fputs(HELPMSG_FNC_GRAPHALLPTS, fp);
    else if (icasestrcomp(FncName_, "PI"))
      fputs(HELPMSG_CONST_PI, fp);
    else if (icasestrcomp(FncName_, "MODE"))
      fputs(HELPMSG_FNC_MODE, fp);
    else if (icasestrcomp(FncName_, "GRAPHPREC"))
      fputs(HELPMSG_FNC_GRAPHPREC, fp);
    else if (icasestrcomp(FncName_, "PRECISION"))
      fputs(HELPMSG_FNC_PRECISION, fp);
    else if (icasestrcomp(FncName_, "GETPRECISION"))
      fputs(HELPMSG_FNC_GETPRECISION, fp);
    else if (icasestrcomp(FncName_, "SETRECUR"))
      fputs(HELPMSG_FNC_SETRECUR, fp);
    else if (icasestrcomp(FncName_, "MIN"))
      fputs(HELPMSG_FNC_MIN, fp);
    else if (icasestrcomp(FncName_, "MAX"))
      fputs(HELPMSG_FNC_MAX, fp);
    else if (icasestrcomp(FncName_, "DERIV"))
      fputs(HELPMSG_FNC_DERIV, fp);
    else if (icasestrcomp(FncName_, "DERIV2"))
      fputs(HELPMSG_FNC_DERIV2, fp);
    else if (icasestrcomp(FncName_, "INTEG"))
      fputs(HELPMSG_FNC_INTEG, fp);
    else if (icasestrcomp(FncName_, "FMIN"))
      fputs(HELPMSG_FNC_FMIN, fp);
    else if (icasestrcomp(FncName_, "FMAX"))
      fputs(HELPMSG_FNC_FMAX, fp);
    else if (icasestrcomp(FncName_, "MATSOLVE"))
      fputs(HELPMSG_FNC_MATSOLVE, fp);
    else if (icasestrcomp(FncName_, "SOLVE"))
      fputs(HELPMSG_FNC_SOLVE, fp);
    else if (icasestrcomp(FncName_, "INTERSECT"))
      fputs(HELPMSG_FNC_INTERSECT, fp);
    else if (icasestrcomp(FncName_, "COMBREP"))
      fputs(HELPMSG_FNC_COMBREP, fp);
    else if (icasestrcomp(FncName_, "PERMREP"))
      fputs(HELPMSG_FNC_PERMREP, fp);
    else if (icasestrcomp(FncName_, "COMB"))
      fputs(HELPMSG_FNC_COMB, fp);
    else if (icasestrcomp(FncName_, "PERM"))
      fputs(HELPMSG_FNC_PERM, fp);
    else if (icasestrcomp(FncName_, "ITEMCOMBREP"))
      fputs(HELPMSG_FNC_ITEMCOMBREP, fp);
    else if (icasestrcomp(FncName_, "ITEMPERMREP"))
      fputs(HELPMSG_FNC_ITEMPERMREP, fp);
    else if (icasestrcomp(FncName_, "ITEMCOMB"))
      fputs(HELPMSG_FNC_ITEMCOMB, fp);
    else if (icasestrcomp(FncName_, "ITEMPERM"))
      fputs(HELPMSG_FNC_ITEMPERM, fp);
    else if (icasestrcomp(FncName_, "SETUNI"))
      fputs(HELPMSG_FNC_SETUNI, fp);
    else if (icasestrcomp(FncName_, "SETINT"))
      fputs(HELPMSG_FNC_SETINT, fp);
    else if (icasestrcomp(FncName_, "SETDIF"))
      fputs(HELPMSG_FNC_SETDIF, fp);
    else if (icasestrcomp(FncName_, "SETXDIF"))
      fputs(HELPMSG_FNC_SETXDIF, fp);
    else if (icasestrcomp(FncName_, "SET"))
      fputs(HELPMSG_FNC_SET, fp);
    else if (icasestrcomp(FncName_, "FRAC"))
      fputs(HELPMSG_FNC_FRAC, fp);
    else if (icasestrcomp(FncName_, "RECTOPOLR"))
      fputs(HELPMSG_FNC_RECTOPOLR, fp);
    else if (icasestrcomp(FncName_, "RECTOPOLA"))
      fputs(HELPMSG_FNC_RECTOPOLA, fp);
    else if (icasestrcomp(FncName_, "POLTORECX"))
      fputs(HELPMSG_FNC_POLTORECX, fp);
    else if (icasestrcomp(FncName_, "POLTORECY"))
      fputs(HELPMSG_FNC_POLTORECY, fp);
    else if (icasestrcomp(FncName_, "RAND"))
      fputs(HELPMSG_FNC_RAND, fp);
    else if (icasestrcomp(FncName_, "INV"))
      fputs(HELPMSG_FNC_INV, fp);
    else if (icasestrcomp(FncName_, "ROUND"))
      fputs(HELPMSG_FNC_ROUND, fp);
    else if (icasestrcomp(FncName_, "CEIL"))
      fputs(HELPMSG_FNC_CEIL, fp);
    else if (icasestrcomp(FncName_, "FLOOR"))
      fputs(HELPMSG_FNC_FLOOR, fp);
    else if (icasestrcomp(FncName_, "E"))
    {
      fputs(HELPMSG_FNC_SCINOTE, fp);
      if (fp == stdout)
        dum_ = fgetc(stdin);
      fputs(HELPMSG_CONST_E, fp);
    }
    else if (icasestrcomp(FncName_, "TOHEX"))
      fputs(HELPMSG_FNC_TOHEX, fp);
    else if (icasestrcomp(FncName_, "TOOCT"))
      fputs(HELPMSG_FNC_TOOCT, fp);
    else if (icasestrcomp(FncName_, "TOBIN"))
      fputs(HELPMSG_FNC_TOBIN, fp);
    else if (icasestrcomp(FncName_, "TOCMP2BIN"))
      fputs(HELPMSG_FNC_TOCMP2BIN, fp);
    else if (icasestrcomp(FncName_, "FROMHEX"))
      fputs(HELPMSG_FNC_FROMHEX, fp);
    else if (icasestrcomp(FncName_, "FROMOCT"))
      fputs(HELPMSG_FNC_FROMOCT, fp);
    else if (icasestrcomp(FncName_, "FROMBIN"))
      fputs(HELPMSG_FNC_FROMBIN, fp);
    else if (icasestrcomp(FncName_, "FROMCMP2BIN"))
      fputs(HELPMSG_FNC_FROMCMP2BIN, fp);
    else if (icasestrcomp(FncName_, "SUM"))
      fputs(HELPMSG_FNC_SUM, fp);
    else if (icasestrcomp(FncName_, "PROD"))
      fputs(HELPMSG_FNC_PRODUCT, fp);
    else if (icasestrcomp(FncName_, "STDDEV"))
      fputs(HELPMSG_FNC_STDDEV, fp);
    else if (icasestrcomp(FncName_, "AVGMEAN"))
      fputs(HELPMSG_FNC_AVGMEAN, fp);
    else if (icasestrcomp(FncName_, "AVGMODE"))
      fputs(HELPMSG_FNC_AVGMODE, fp);
    else if (icasestrcomp(FncName_, "AVGMED"))
      fputs(HELPMSG_FNC_AVGMEDIAN, fp);
    else if (icasestrcomp(FncName_, "GRAPH"))
      fputs(HELPMSG_FNC_GRAPH, fp);
    else if (icasestrcomp(FncName_, "EVAL"))
      fputs(HELPMSG_FNC_EVAL, fp);
    else if (icasestrcomp(FncName_, "SHOW"))
      fputs(HELPMSG_FNC_SHOW, fp);
    else if (icasestrcomp(FncName_, "SHOWSET"))
      fputs(HELPMSG_FNC_SHOWSET, fp);
    else if (icasestrcomp(FncName_, "EXEC"))
    {
      fputs(HELPMSG_FNC_EXEC, fp);
      if (fp == stdout)
        dum_ = fgetc(stdin);
      fputs(HELPMSG_PROGRAM_COMMAND_LIST1, fp);
      if (fp == stdout)
        dum_ = fgetc(stdin);
      fputs(HELPMSG_PROGRAM_COMMAND_LIST2, fp);
    }
    else if (icasestrcomp(FncName_, "IF"))
      fputs(HELPMSG_PRGFNC_IF, fp);
    else if (icasestrcomp(FncName_, "IF-THEN"))
      fputs(HELPMSG_PRGFNC_IF_THEN, fp);
    else if (icasestrcomp(FncName_, "IF-THEN-ELSE"))
      fputs(HELPMSG_PRGFNC_IF_THEN_ELSE, fp);
    else if (icasestrcomp(FncName_, "FOR"))
      fputs(HELPMSG_PRGFNC_FOR, fp);
    else if (icasestrcomp(FncName_, "WHILE"))
      fputs(HELPMSG_PRGFNC_WHILE, fp);
    else if (icasestrcomp(FncName_, "REPEAT"))
      fputs(HELPMSG_PRGFNC_REPEAT, fp);
    else if (icasestrcomp(FncName_, "END"))
      fputs(HELPMSG_PRGFNC_END, fp);
    else if (icasestrcomp(FncName_, "PAUSE"))
      fputs(HELPMSG_PRGFNC_PAUSE, fp);
    else if (icasestrcomp(FncName_, "LBL"))
      fputs(HELPMSG_PRGFNC_LBL, fp);
    else if (icasestrcomp(FncName_, "GOTO"))
      fputs(HELPMSG_PRGFNC_GOTO, fp);
    else if (icasestrcomp(FncName_, "IS>"))
      fputs(HELPMSG_PRGFNC_INCSKIP, fp);
    else if (icasestrcomp(FncName_, "DS<"))
      fputs(HELPMSG_PRGFNC_DECSKIP, fp);
    else if (icasestrcomp(FncName_, "MENU"))
      fputs(HELPMSG_PRGFNC_MENU, fp);
    else if (icasestrcomp(FncName_, "PRGM"))
      fputs(HELPMSG_PRGFNC_PRGM, fp);
    else if (icasestrcomp(FncName_, "RETURN"))
      fputs(HELPMSG_PRGFNC_RETURN, fp);
    else if (icasestrcomp(FncName_, "STOP"))
      fputs(HELPMSG_PRGFNC_STOP, fp);
    else if (icasestrcomp(FncName_, "PROMPT"))
      fputs(HELPMSG_PRGFNC_PROMPT, fp);
    else if (icasestrcomp(FncName_, "INPUT"))
      fputs(HELPMSG_PRGFNC_INPUT, fp);
    else if (icasestrcomp(FncName_, "DISP"))
      fputs(HELPMSG_PRGFNC_DISP, fp);
    else if (icasestrcomp(FncName_, "GRAPH"))
      fputs(HELPMSG_PRGFNC_GRAPH, fp);
  }
}

/*****************************************************************************
Procedure to print the help message to the screen if the program is run
without any arguments
PASSED:
  argc : the argument count
*/
void CalculatorBase::help(int argc, char* argv[])
{
  if (argc == 2 && (icasestrcompn(argv[1], "--help", 6) ||
                    icasestrcompn(argv[1], "--version", 9)))
  {
    helpmessage(stdout, argv[1]);
    exit(2);
  }
}


/*****************************************************************************/
FILE* CalculatorBase::GetLogFile()
{
  filestatus[Mcalc_IOType::ErrorLog] = 0;
  if (files)
    files[Mcalc_IOType::ErrorLog] = NULL;

  if (logfile)
  {
    fflush(logfile);
    fclose(logfile);
    logfile = NULL;
  }

  if (files)
  {
    if (LOG_FILE)
      filestatus[Mcalc_IOType::ErrorLog] = openfile(&files[Mcalc_IOType::ErrorLog], LOG_FILE, "a");
    else
      filestatus[Mcalc_IOType::ErrorLog] = openfile(&files[Mcalc_IOType::ErrorLog], MCALC_LOG_FILE, "a");

    if (filestatus[Mcalc_IOType::ErrorLog])
      logfile = files[Mcalc_IOType::ErrorLog];
    else
      logfile = NULL;
  }
  else
  {
    if (LOG_FILE)
      filestatus[Mcalc_IOType::ErrorLog] = openfile(&logfile, LOG_FILE, "a");
    else
      filestatus[Mcalc_IOType::ErrorLog] = openfile(&logfile, MCALC_LOG_FILE, "a");

    if (!filestatus[Mcalc_IOType::ErrorLog])
      logfile = NULL;
  }

  return logfile;
}

/*****************************************************************************/
FILE* CalculatorBase::GetOutputFile()
{
  if (InAsyncMode())
  {
    if (!filestatus[Mcalc_IOType::ProgramOutput] || !outfile)
    {
      filestatus[Mcalc_IOType::ProgramOutput] = openfile(&files[Mcalc_IOType::ProgramOutput],PROG_OUTPUT_FILE,"w");
      outfile = programoutputfile = files[Mcalc_IOType::ProgramOutput];
    }
  }
  else
  {
    if (files[Mcalc_IOType::Output])
      outfile = files[Mcalc_IOType::Output];
    else
      outfile = stdout;
  }

  if (outfile)
    fflush(outfile);

  return outfile;
}

/*****************************************************************************/
void CalculatorBase::CloseOutputFile()
{
  if (InAsyncMode())
  {
    if (programoutputfile)
      fclose(programoutputfile);

    outfile = programoutputfile = NULL;
    filestatus[Mcalc_IOType::ProgramOutput] = 0;
  }
}

/*****************************************************************************/
void CalculatorBase::ClearOutputFile()
{
  GetOutputFile();
  CloseOutputFile();
}

/*****************************************************************************
Procedure to get the files needed for the program to run if passed file
arguments
PASSED:
  argc : the argument count
  argv : the argument vector
  cmdline : the command line flag
USES
  uses openfile to open a file for operations
  uses allocmem to allocate memory for the file pointers array
READ:
  argc is read for the number of arguments passed from the command line
  argv is read for the string arguments passed
CHANGED:
  cmdline is changed to 1 if the number of arguments fit the number needed
  for the command line version of the program, otherwise global file pointers
  are changed to valid file locations
*/
void CalculatorBase::getfiles(int* argc, char* argv[], int* cmdline)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: getfiles\n", logfile);
  #endif

  int inputexists = 0,
      outputexists = 0;

  int i,		// loop index
      ifindex,	        // input file string array index
      ofindex,	        // output file string array index
      numfiles,	        // number of files
      argcount = *argc;

  for (i=1;i<argcount&&!outputexists;i++)
    outputexists = strcmp(argv[i],OUTPUTFILE_SWITCH) ? 0:1;
  ofindex = i;
  for (i=1;i<argcount&&!inputexists;i++)
    inputexists = strcmp(argv[i],INPUTFILE_SWITCH) ? 0:1;
  ifindex = i;
  numfiles = inputexists + outputexists;
  *cmdline = !inputexists;
  filestatus[Mcalc_IOType::Input] =
  filestatus[Mcalc_IOType::Output] = 0;

  if (!files)
  {
    files = (FILE**)allocmem(MAXFILES, sizeof(FILE*), files);
    for (i = 0; i < MAXFILES; i++)
      files[i] = NULL;
  }

  #if CALCLIB_DEBUG2
    filestatus[Mcalc_IOType::ErrorLog] = 0;
    files[Mcalc_IOType::ErrorLog] = NULL;

    if (logfile)
    {
      fclose(logfile);
      logfile = NULL;
    }
  #endif

  if (LOG_FILE)
    filestatus[Mcalc_IOType::ErrorLog] = openfile(&files[Mcalc_IOType::ErrorLog], LOG_FILE, "w");
  else
    filestatus[Mcalc_IOType::ErrorLog] = openfile(&files[Mcalc_IOType::ErrorLog], MCALC_LOG_FILE, "w");

  if (filestatus[Mcalc_IOType::ErrorLog])
    logfile = files[Mcalc_IOType::ErrorLog];
  else
    logfile = NULL;

  #if CALCLIB_DEBUG2
    fputs("Entering: getfiles(1)\n", logfile);
  #endif

  if (numfiles)
  {
    if (inputexists)
      filestatus[Mcalc_IOType::Input] = openfile(&files[Mcalc_IOType::Input],argv[ifindex],"r");
    else
    {
      files[Mcalc_IOType::Input] = NULL;
      _StdInput = TRUE;
    }

    if (outputexists)
      filestatus[Mcalc_IOType::Output] = openfile(&files[Mcalc_IOType::Output],argv[ofindex],"w");
    else
    {
      files[Mcalc_IOType::Output] = NULL;
      _StdOutput = TRUE;
    }
  }
  else
  {
    _StdInput = TRUE;
    _StdOutput = TRUE;
  }

  #if CALCLIB_DEBUG2
    fputs("Entering: getfiles(2)\n", logfile);
  #endif

  if (outputexists)
  {
    outfile = files[Mcalc_IOType::Output];
    argcount = ofindex-1;
  }
  else
    outfile = stdout;

  *argc = argcount;
  _StreamSet = 1;

  #if CALCLIB_DEBUG2
    fputs("Leaving: getfiles\n", logfile);
  #endif
}

/*****************************************************************************
Procedure for reading input from the input file and assigning this to the
list string array, or assigning the list string array to the command line
argument vector depending on the command line argument flag.
PASSED:
  argc : the argument count
  argv : the argument vector
  list : the substitute argument list
  cmdline : command line argument flag
USES:
  uses freadstring to read a string of indefinite length from a file.
  uses stringtoken to separate a string into individual words by replacing
       whitespace with NULL characters.
  uses stripstring to get individual words from a string which has been
       separated with NULL characters.
  uses allocmem to allocate memory to the list string array
  uses clearleadingspaces to clear leading spaces from the string buffer
  uses freearglist to free memory from the argument list and its string
       elements
READ:
  the input file is read if file I/O is specified
  the cmdline flag specifies command line input - no file I/O
CHANGED
  list is changed to contain the arguments read from the input file if
  file I/O is used
  argc is changed to the number of arguments read from the input file.
*/
int CalculatorBase::readfile(int* argc, char* argv[], char*** list, int cmdline)
{
  char* buffer = NULL;     	/* string buffer */
  int eof = 1;   	/* end of file flag */
  int wordcount = 0;    	/* count of the number of arguments in the argument list */

  if (cmdline)
  {
    if (_ArgListSize)
      cmdline = _ArgListSize = 0;
    else
    {
      if (*argc > 1)
      {
        cmdline = _ArgListSize = 1;
        freearglist(list, 0);
        *list = argv;
      }
      else
        cmdline = _ArgListSize = 0;
    }

    return cmdline;
  }
  else
  {
    freearglist(list, _ArgListSize);                                    /* free memory from previous argument list */
    _ArgListSize = wordcount = 0;
    bool Comment_ = false;

    do
    {
      eof = freadstring(&buffer, ';', files[Mcalc_IOType::Input]);        /* read contents of file into buffer */

      if (!buffer)
        break;

      clearleadingspaces(&buffer);                                        /* clear leading spaces from buffer */
      Comment_ = IsThisChar(buffer, 0, '#', 1);

      if (Comment_)
      {
        MemMatrix::Matrix().Deallocate(buffer);
        buffer = NULL;
      }
    }
    while (!eof && Comment_);

    if (buffer && strlen(buffer))
    {
      _ArgListSize = (wordcount = stringtoken(buffer)) + 1;
      *list = (char**)allocmem(_ArgListSize,sizeof(char*),*list);
      ++(*list);
      stripstring(wordcount,buffer,*list); /* strip buffer of word tokens */
      --(*list);
    }
    else
    {
      _ArgListSize = 1;
      *list = (char**)allocmem(_ArgListSize,sizeof(char*),*list);
      (*list)[0] = (char*)allocmem(1,sizeof(char),*list[0]);
      (*list)[0][0] = 0;
    }

    *argc = _ArgListSize;
    MemMatrix::Matrix().Deallocate(buffer);
    buffer = NULL;
  }

  SetPlotGraph(false);
  return !eof;
}

/*****************************************************************************/
int CalculatorBase::ReadUserInput(int* argc, char*** list)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: ReadUserInput\n", logfile);
  #endif

  char* buffer = NULL;     	/* string buffer */
  int wordcount = 0;    	/* count of the number of arguments in the argument list */
  int eof = 0;
  bool Comment_ = false;

  if (!userinputfile || !filestatus[Mcalc_IOType::UserInput])
  {
    filestatus[Mcalc_IOType::UserInput] = openfile(&files[Mcalc_IOType::UserInput],USER_INPUT_FILE,"r",TRUE);
    userinputfile = files[Mcalc_IOType::UserInput];

    if (!filestatus[Mcalc_IOType::UserInput] || !FileSize(userinputfile))
    {
      if (userinputfile)
        fclose(userinputfile);

      userinputfile = NULL;
      filestatus[Mcalc_IOType::UserInput] = 0;
      return 0;
    }
  }

  freearglist(list, _ArgListSize);                     /* free memory from previous argument list */
  _ArgListSize = wordcount = 0;

  do
  {
    eof = freadstring(&buffer, ';', userinputfile);            /* read contents of file into buffer */

    if (!buffer)
      break;

    clearleadingspaces(&buffer);                         /* clear leading spaces from buffer */
    Comment_ = IsThisChar(buffer, 0, '#', 1);

    if (Comment_)
    {
      MemMatrix::Matrix().Deallocate(buffer);
      buffer = NULL;
    }
  }
  while (!eof && Comment_);

  if (buffer && strlen(buffer))
  {
    _ArgListSize = (wordcount = stringtoken(buffer)) + 1;
    *list = (char**)allocmem(_ArgListSize,sizeof(char*),*list);
    ++(*list);
    stripstring(wordcount,buffer,*list); /* strip buffer of word tokens */
    --(*list);
  }
  else
  {
    _ArgListSize = 1;
    *list = (char**)allocmem(_ArgListSize,sizeof(char*),*list);
    (*list)[0] = (char*)allocmem(1,sizeof(char),*list[0]);
    (*list)[0][0] = 0;
  }

  *argc = _ArgListSize;
  MemMatrix::Matrix().Deallocate(buffer);
  buffer = NULL;

  if (userinputfile)
    fclose(userinputfile);

  userinputfile = NULL;
  filestatus[Mcalc_IOType::UserInput] = 0;
  SetPlotGraph(false);

  #if CALCLIB_DEBUG2
    fputs("Leaving: ReadUserInput\n", logfile);
  #endif

  return 1;
}

/*****************************************************************************
Procedure for closing files
*/
void CalculatorBase::closefiles(void)
{
  if (filestatus[Mcalc_IOType::Input])   /* closing input file if opened */
    fclose(files[Mcalc_IOType::Input]);
  if (filestatus[Mcalc_IOType::Output])  /* closing output file if opened */
    fclose(files[Mcalc_IOType::Output]);
  if (filestatus[Mcalc_IOType::ErrorLog])
    fclose(files[Mcalc_IOType::ErrorLog]);

  MemMatrix::Matrix().Deallocate(files);

  files = NULL;
  outfile = NULL;          /* setting output file pointer to NULL */
  logfile = NULL;

  if (InAsyncMode())
    SetAsyncMode(0);
}

/*****************************************************************************
Procedure to delete the stack
*/
void CalculatorBase::delstack(Mcalc_Stack** stk)
{
  Uchar* str = NULL;

  while (*stk)
  {
    str = pop(stk);
    MemMatrix::Matrix().Deallocate(str);
    str = NULL;
  }
}

/*****************************************************************************
Procedure to delete the queue
*/
void CalculatorBase::delqueue(Mcalc_Queue** Q)
{
  delstack(Q);
}

/*****************************************************************************
Procedure to push the result of a calculation back on to the stack
*/
void CalculatorBase::pushresult(Mcalc_Operand** operands, Mcalc_Stack** rpnstack, int optype)
{
  if (optype != Mcalc_OperatorType::Assignment && !HasErrors())
  {
    char* buffer = NULL;
    int i, bindex = 0;
    Mcalc_Operand* resultptr = operands[RESULTINDEX];

    if (resultptr->size && resultptr->value)
    {
      buffer = (char*)allocmem(resultptr->size+2,sizeof(char),buffer);
      if (resultptr->sign)
        buffer[bindex++] = '-';

      strcpy(&buffer[bindex], resultptr->value);
      push(rpnstack, (Uchar*)buffer);
      MemMatrix::Matrix().Deallocate(buffer);
      buffer = NULL;
    }
  }
}

/*****************************************************************************/
char* CalculatorBase::ConcatArglist(int argc, char** list, Boolean LeaveSpace_)
{
  int i;
  int length = 0;
  int total = 0;

  list[0] = (char*)allocmem(MAXLINE, sizeof(char), list[0]);
  list[0][0] = '\0';
  total = MAXLINE-1;

  for (i = 1; i < argc; i++)
  {
    length = strlen(list[0]) + strlen(list[i]);
    if (length > total)
    {
      total += ((length > MAXLINE) ? length:MAXLINE);
      list[0] = (char*)reallocmem(total+2,sizeof(char),list[0]);
    }

    strcpy(list[0] + strlen(list[0]), list[i]);
    if (i < argc - 1 && LeaveSpace_)
      strcat(list[0], " ");
  }

  return list[0];
}

/*****************************************************************************/
int CalculatorBase::CompareStrings(const char* String1_, const char* String2_, int Optype_)
{
  if (Optype_ == Mcalc_OperatorType::Equal)
    return (strcmp(String1_, String2_) == 0);
  else if (Optype_ == Mcalc_OperatorType::NotEqual)
    return (strcmp(String1_, String2_) != 0);
  else if (Optype_ == Mcalc_OperatorType::LessThan)
    return (strcmp(String1_, String2_) < 0);
  else if (Optype_ == Mcalc_OperatorType::GreaterThan)
    return (strcmp(String1_, String2_) > 0);
  else if (Optype_ == Mcalc_OperatorType::LessOrEqual)
    return (strcmp(String1_, String2_) <= 0);
  else if (Optype_ == Mcalc_OperatorType::GreaterOrEqual)
    return (strcmp(String1_, String2_) >= 0);

  return 0;
}

/*****************************************************************************/
void CalculatorBase::PushStringCompareTokens(Mcalc_Queue** rpnq, Mcalc_Stack** opstk, const char* buffer,
                                             int Start_, int End_, int Optype_)
{
  int Index1_ = toupper(buffer[Start_ + 1]) - 'A';
  int Index2_ = toupper(buffer[End_]) - 'A';
  char* Token1_ = NULL;
  char* Token2_ = NULL;
  Uchar Buf_[2];

  Token1_ = (char*)allocmem(GetStrVarLen(Index1_) + 1, sizeof(char), Token1_);
  Token2_ = (char*)allocmem(GetStrVarLen(Index2_) + 1, sizeof(char), Token2_);

  CopyStrVar(Token1_, Index1_);
  CopyStrVar(Token2_, Index2_);

  enqueue(rpnq, (Uchar*)Token1_);
  enqueue(rpnq, (Uchar*)Token2_);
  enqueue(rpnq, ToUchar(Buf_, Optype_));

  MemMatrix::Matrix().Deallocate(Token1_);
  MemMatrix::Matrix().Deallocate(Token2_);
}

/*****************************************************************************/
int CalculatorBase::FindComparisonEndPts(const char* buffer, int& Start_, int& End_)
{
    int x, y, len;
    len = strlen(buffer);

    for (x = 0; isspace(buffer[x]) && x < len; x++);
    for (y = len - 1; isspace(buffer[y]) && y >= 0; y--);

    if (x < y)
    {
      while (isleftbracket((Uchar)buffer[x]) && isrightbracket((Uchar)buffer[y]) && x < len && y >= 0)
      {
        x++;
        y--;
      }

      if ((x + 3) < y && x < (len - 1) &&
          buffer[x] == '$' &&
          isalpha(buffer[x+1]) && InRange('A', buffer[x+1], 'z') &&
          isalpha(buffer[y]) && InRange('A', buffer[y], 'z') &&
          buffer[y-1] == '$')
      {
        int Index1_ = toupper(buffer[x + 1]) - 'A';
        int Index2_ = toupper(buffer[y]) - 'A';

        if (IsStringVar(Index1_) && IsStringVar(Index2_))
        {
          Start_ = x;
          End_ = y;
          return TRUE;
        }
      }
    }

    return FALSE;
}

/*****************************************************************************/
int CalculatorBase::TestForStringComparison(const char* buffer, int& Start_, int& End_, int& Optype_)
{
  const char* PtrStart_ = NULL;
  PtrStart_ = strstr(buffer, "==");

  if (PtrStart_)
  {
    Optype_ = EQUAL_OP;
    return FindComparisonEndPts(buffer, Start_, End_);
  }
  else
  {
    PtrStart_ = strstr(buffer, "!=");

    if (PtrStart_)
    {
      Optype_ = NOTEQUAL_OP;
      return FindComparisonEndPts(buffer, Start_, End_);
    }
    else
    {
      PtrStart_ = strstr(buffer, "<=");

      if (PtrStart_)
      {
        Optype_ = LSTHANOREQ_OP;
        return FindComparisonEndPts(buffer, Start_, End_);
      }
      else
      {
        PtrStart_ = strstr(buffer, ">=");

        if (PtrStart_)
        {
          Optype_ = GRTHANOREQ_OP;
          return FindComparisonEndPts(buffer, Start_, End_);
        }
        else
        {
          PtrStart_ = strstr(buffer, "<");

          if (PtrStart_)
          {
            Optype_ = '<';
            return FindComparisonEndPts(buffer, Start_, End_);
          }
          else
          {
            PtrStart_ = strstr(buffer, ">");

            if (PtrStart_)
            {
              Optype_ = '>';
              return FindComparisonEndPts(buffer, Start_, End_);
            }
          }
        }
      }
    }
  }

  return FALSE;
}

/*****************************************************************************/
int CalculatorBase::HasVariables(const char* Buffer_)
{
  int len = ::SafeStrLen(Buffer_);
  int x;

  if (Buffer_ && len)
    for (x = 0; x < len; x++)
      if (Buffer_[x] == '$' &&
          isalpha(Buffer_[x+1]) && InRange('A', Buffer_[x+1], 'z'))
        return TRUE;

  return FALSE;
}

/*****************************************************************************/
void CalculatorBase::DumpVariables(CalculatorBase* CalcPtr_)
{
  int i;
  char Var_[20];
  ChrString Answer_;

  Var_[0] = '$';
  Var_[2] = 0;

  for (i = 'A'; i <= 'Z'; i++)
  {
    Var_[1] = i;
    Answer_ += DisplayVariable(CalcPtr_, Var_, i - 'A', false);
  }

  strcpy(Var_, "ANS");
  Answer_ += DisplayVariable(CalcPtr_, Var_, FLOATINGPOINT_MODE, false);
  strcpy(Var_, "ANSSTR");
  Answer_ += DisplayVariable(CalcPtr_, Var_, LITERAL_MODE, false);
  strcpy(Var_, "FRACANS");
  Answer_ += DisplayVariable(CalcPtr_, Var_, FRACTION_MODE, false);

  char* Buffer_ = Answer_.strlen() ? ::NewString(Answer_.c_str()):NULL;

  if (InAsyncMode() || InProgramMode())
    if (InProgramMode())
    {
      if (::SafeStrLen(Buffer_))
        SetFloatStr(Buffer_);
      else
        SetHasAnswer(FALSE);

      if (InAsyncMode())
        _WaitForResponse = Mcalc_IOState::OUTPUT_READY;
    }
    else
      DisplayToProgramOutput(Buffer_);

  MemMatrix::Matrix().Deallocate(Buffer_);
}

/*****************************************************************************/
int CalculatorBase::CheckForVarDisplay(CalculatorBase* CalcPtr_, char** list, char* original,
                                       int CheckLetterVars_, int& VarDisplayed_, int& EvalDone_)
{
  int len = strlen(list[0]);
  int StartWithVar_ =
      (CheckLetterVars_ &&
       len == 2 && list[0][0] == '$' &&
       isalpha(list[0][1]) && InRange('A', list[0][1], 'z')) ||
      (len == 1 && Uchar(list[0][0]) == ANS_CONST) ||
      (len == 1 && Uchar(list[0][0]) == ANSSTR_CONST) ||
      (len == 1 && Uchar(list[0][0]) == FRACANS_CONST);

  if (StartWithVar_)
  {
    if (ProcessVariables())
    {
      EvalDone_ = FALSE;
      if (!CheckLetterVars_)
        SetProcessVariables(FALSE);
    }
    else
    {
      if (Uchar(list[0][0]) == ANS_CONST)
        DisplayVariable(CalcPtr_, original, FLOATINGPOINT_MODE);
      else if (Uchar(list[0][0]) == ANSSTR_CONST)
        DisplayVariable(CalcPtr_, original, LITERAL_MODE);
      else if (Uchar(list[0][0]) == FRACANS_CONST)
        DisplayVariable(CalcPtr_, original, FRACTION_MODE);
      else
        DisplayVariable(CalcPtr_, original, toupper(list[0][1]) - 'A');

      EvalDone_ =
      VarDisplayed_ = TRUE;
    }

    return TRUE;
  }

  return FALSE;
}

/*****************************************************************************
Procedure to convert the expression from infix notation to RPN notation
*/
int CalculatorBase::InfixToRpn(int argc, char** list, Mcalc_Queue** rpnqueue, Mcalc_Stack** opstack,
                               char** original, CalculatorBase* CalcPtr_)
{
  int i;
  int ok;
  char* buffer = NULL;
  Uchar token[2];
  int EvalDone_;
  int ErrorType_ = 0;
  int BalancedBrk_ = 1;
  int VarDisplayed_ = FALSE;
  int ValidFormat_;
  int ReEval_ = 0;
  int Start_, End_, Optype_;
  char* ConstOriginal_ = NULL;

  ClearError();
  SetStringComparison(FALSE);
  SetSetOperation(FALSE);
  ConcatArglist(argc, list);
  *original = (char*)allocmem(strlen(list[0])+1, sizeof(char), *original);
  strcpy(*original,list[0]);
  stripws(list[0]);
  EvalDone_ = TRUE;

  ConstOriginal_ = (char*)allocmem(strlen(*original)+1, sizeof(char), ConstOriginal_);
  strcpy(ConstOriginal_, *original);

  if (!CheckForVarDisplay(CalcPtr_, list, *original, TRUE, VarDisplayed_, EvalDone_))
  {
    if (icasestrcompn(list[0], "mode=degree", 11))
      CalcPtr_->SetAngleUnit(CalculatorBase::DEGREE);
    else if (icasestrcompn(list[0], "mode=radian", 11))
      CalcPtr_->SetAngleUnit(CalculatorBase::RADIAN);

    else if (icasestrcompn(list[0], "precision=float", 15))
      CalcPtr_->TransferCalcType(CalculatorBase::FLOAT);
    else if (icasestrcompn(list[0], "precision=double", 16))
      CalcPtr_->TransferCalcType(CalculatorBase::DOUBLE);
    else if (icasestrcompn(list[0], "precision=longdouble", 20))
      CalcPtr_->TransferCalcType(CalculatorBase::LONGDOUBLE);
    else if (icasestrcompn(list[0], "precision=longnum", 17))
      CalcPtr_->TransferCalcType(CalculatorBase::LONGNUM);

    else if (icasestrcompn(list[0], "precision=float,keepvars", 24))
      CalcPtr_->SetCalcType(CalculatorBase::FLOAT);
    else if (icasestrcompn(list[0], "precision=double,keepvars", 25))
      CalcPtr_->SetCalcType(CalculatorBase::DOUBLE);
    else if (icasestrcompn(list[0], "precision=longdouble,keepvars", 29))
      CalcPtr_->SetCalcType(CalculatorBase::LONGDOUBLE);
    else if (icasestrcompn(list[0], "precision=longnum,keepvars", 26))
      CalcPtr_->SetCalcType(CalculatorBase::LONGNUM);

    else if (icasestrcompn(list[0], "graphallpts", 11))
    {
      if (list[0][11])
      {
        if (icasestrcompn(&list[0][11], "=true", 5) ||
            icasestrcompn(&list[0][11], "=yes", 4) ||
            icasestrcompn(&list[0][11], "=yeah", 5) ||
            icasestrcompn(&list[0][11], "=yep", 4))
          CalcPtr_->SetPlotAllPoints(TRUE);
        else
        if (icasestrcompn(&list[0][11], "=false", 6) ||
            icasestrcompn(&list[0][11], "=no", 3) ||
            icasestrcompn(&list[0][11], "=nah", 4) ||
            icasestrcompn(&list[0][11], "=nope", 5))
          CalcPtr_->SetPlotAllPoints(FALSE);
      }
      else
        CalcPtr_->SetPlotAllPoints(TRUE);
    }

    else if (icasestrcompn(list[0], "cleargraph", 10))
      CalcPtr_->ClearGraphs();
    else if (::SafeStrLen(list[0]) > 10 && icasestrcompn(list[0], "graphprec=", 10) &&
             IsNumber(&list[0][10], 0, 1, 0, 10))
      CalcPtr_->SetGraphPrecision(atof(&list[0][10]) / 10);
    else if (::SafeStrLen(list[0]) > 9 && icasestrcompn(list[0], "setrecur=", 9) &&
             IsNumber(&list[0][9], 0, 0, 0, 10))
      CalcPtr_->SetRecursionLimit(atoi(&list[0][9]));
    else if (icasestrcompn(list[0], "getprecision", 12))
    {
      if (InAsyncMode())
        DisplayMessage(GetOutputFile(), "precision = %s\n", CalcPtr_->GetPrecision());
      else
        DisplayMessage(GetOutputFile(), ">> precision = %s\n", CalcPtr_->GetPrecision());

      CloseOutputFile();

      if (InAsyncMode())
        if (InProgramMode())
          _WaitForResponse = Mcalc_IOState::OUTPUT_READY;
        else
          SetIoState(Mcalc_IOState::CALC_RESPONSE);
    }
    else if (icasestrcompn(list[0], "help", 4) ||
             icasestrcompn(list[0], "version", 7))
    {
      helpmessage(GetOutputFile(), list[0]);
      CloseOutputFile();

      if (InAsyncMode())
        if (InProgramMode())
          _WaitForResponse = Mcalc_IOState::OUTPUT_READY;
        else
          SetIoState(Mcalc_IOState::CALC_RESPONSE);
    }
    else if (icasestrcompn(list[0], "dumpvars", 8))
    {
      DumpVariables(CalcPtr_);
      EvalDone_ = VarDisplayed_ = TRUE;

      if (InAsyncMode())
        if (InProgramMode())
          _WaitForResponse = Mcalc_IOState::OUTPUT_READY;
        else
          SetIoState(Mcalc_IOState::CALC_RESPONSE);
    }
    else
      EvalDone_ = FALSE;
  }

  if (!EvalDone_)
  {
    ValidFormat_ = formatlist(&list[0], CalcPtr_);
    CalcPtr_ = CalcPtr_->GetCalcType();

    if (!ValidFormat_)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_SYMBOLS, ERRMSG_INVALID_SYMBOLS, *original);
      EvalDone_ = TRUE;
    }
    else
      CheckForVarDisplay(CalcPtr_, list, *original, FALSE, VarDisplayed_, EvalDone_);
  }

  if (EvalDone_)
  {
    if (!InProgramMode() || !VarDisplayed_)
      SetHasAnswer(FALSE);

    return FALSE;
  }

  if (_ExecuteProgram)
  {
    CalcProgram* Program_ = new CalcProgram(GetCalcType(), _ExecuteProgram);
    MemMatrix::Matrix().Deallocate(_ExecuteProgram);
    _ExecuteProgram = NULL;

    PushActiveProgram(Program_);
    Program_->RunProgram();
    int Quit_ = Program_->QuitCalc();
    PopActiveProgram();

    if (InAsyncMode())
    {
      ClearOutputFile();
      ClearUserPromptFile();
    }

    SetShouldQuit(Quit_);
    return FALSE;
  }

  int Error_ = 0;
  if (!ReplaceTokens(&list[0], CalcPtr_, TRUE, Error_, ReEval_, TRUE, TRUE))
  {
    CalcPtr_ = CalcPtr_->GetCalcType();

    if (Error_)
    {
      CalcPtr_->ResetAnswerMode();
      CalcPtr_->RestoreCalcType();
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_ASSIGNMENT, ERRMSG_INVALID_ASSIGNMENT, *original);
      SetHasAnswer(FALSE);
      return FALSE;
    }
    else if (!CalcPtr_->SkipEval())
      ReplaceTokens(original, CalcPtr_, FALSE, Error_, ReEval_, FALSE, FALSE);
  }

  if (strlen(list[0]) == 0)
  {
    if (SubExprStackSize() <= 1)
      CalcPtr_->RestoreCalcType();
    SetHasAnswer(FALSE);
    return FALSE;
  }
  else if (StringAssignment() || SetAssignment())
  {
    SetShowAnswer(FALSE);
    return TRUE;
  }

  // Skip evaluation
  if (CalcPtr_->SkipEval())
    return TRUE;

  // Converting to RPN starting here
  token[PROCESSED] = '0';
  for (i = 0, ok = 1; i < strlen(list[0]) && ok; i++)
  {
    token[READ] = list[0][i];
    if ((ok = isvalidsymbol(token[READ])) && !isdelimiter(token[READ]))
    {
      ErrorType_ = 0;
      ok = getleftbracket(rpnqueue,opstack,token);

      if (!ok)
      {
        ok = getrightbracket(rpnqueue,opstack,token,BalancedBrk_);

        if (!ok)
        {
          ok = getoperator(rpnqueue,opstack,token);
          if (!ok)
            ok = getoperand(rpnqueue,opstack,token,list[0],&i);
        }
      }
    }
    else if (!ok)
      ErrorType_ = 4;
  }

  // Error checking and detection
  if (ok)
  {
    while (*opstack && ok)
    {
      buffer = (char*)pop(opstack);

      if (*buffer!='(')
        enqueue(rpnqueue,(Uchar*)buffer);
      else
      {
        ok = FALSE;
        ErrorType_ = 3;
      }

      MemMatrix::Matrix().Deallocate(buffer);
      buffer = NULL;
    }
  }
  else
  {
    if (ErrorType_ != 4)
    {
      if (!BalancedBrk_)
        ErrorType_ = 2;
      else
        ErrorType_ = 1;
    }
  }

  if (!ok)
    if (TestForStringComparison(ConstOriginal_, Start_, End_, Optype_))
    {
      SetAnswerMode(FLOATINGPOINT_MODE, TRUE);
      PushStringCompareTokens(rpnqueue, opstack, ConstOriginal_, Start_, End_, Optype_);
      SetStringComparison(TRUE);
      MemMatrix::Matrix().Deallocate(ConstOriginal_);
      ConstOriginal_ = NULL;
      return TRUE;
    }
    else
    {
      MemMatrix::Matrix().Deallocate(ConstOriginal_);
      ConstOriginal_ = NULL;
    }

  // Error message displaying
  if (!ok)
  {
    if (ErrorType_ == 1)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_EXPR, ERRMSG_INVALID_EXPR, *original);
    else if (ErrorType_ == 2)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MISSLEFT_BRACKET, ERRMSG_MISSLEFT_BRACKET, *original);
    else if (ErrorType_ == 3)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MISSRIGHT_BRACKET, ERRMSG_MISSRIGHT_BRACKET, *original);
    else if (ErrorType_ == 4)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_SYMBOLS, ERRMSG_INVALID_SYMBOLS, *original);
  }
  else
    SetAnswerMode(CalculatorBase::FLOATINGPOINT_MODE);

  if (InProgramMode() && !ok)
    SetHasAnswer(FALSE);

  return ok;
}

/*****************************************************************************
Procedure to evaluate an expression in RPN notation
*/
int CalculatorBase::evaluateRPN(Mcalc_Queue** rpnq, Mcalc_Stack** rpnstack, char** list, int* optype)
{
  char* buffer = NULL;
  unsigned char opcode;
  int OpFound_ = FALSE;
  int DoCont_ = FALSE;
  int i = 0;

  while (*rpnq)
  {
    buffer = (char*)pop(rpnq);
    if (*rpnq)
      _QLookAhead = (*rpnq)->data;
    else
      _QLookAhead = NULL;

    if ((StringComparison() && buffer && i < 2) || isoperand(buffer))
    {
      push(rpnstack,(Uchar*)buffer);
      MemMatrix::Matrix().Deallocate(buffer);
      buffer = NULL;
      i++;
    }
    else
    {
      OpFound_ = TRUE;
      opcode = (unsigned char)(*buffer);

      switch (opcode)
      {
        case NOT_OP : *optype = Mcalc_OperatorType::Logical_Not;
                      break;
        case '+' : *optype = Mcalc_OperatorType::Add;
                   break;
        case MINUS_OP : *optype = Mcalc_OperatorType::Subtract;
                        break;
        case SIGN_OP:
        case '*' : *optype = Mcalc_OperatorType::Multiply;
                   break;
        case '/' : *optype = Mcalc_OperatorType::Divide;
                   break;
        case '@' : *optype = Mcalc_OperatorType::Modulus;
                   break;
        case '^' : *optype = Mcalc_OperatorType::Exponent;
                   break;
        case AND_OP : *optype = Mcalc_OperatorType::Logical_And;
                      break;
        case OR_OP : *optype = Mcalc_OperatorType::Logical_Or;
                     break;
        case CUBERT_OP : *optype = Mcalc_OperatorType::CubeRoot;
                         break;
        case CUBE_OP : *optype = Mcalc_OperatorType::Cube;
                      break;
        case SQRT_OP : *optype = Mcalc_OperatorType::SquareRoot;
                      break;
        case SQR_OP : *optype = Mcalc_OperatorType::Square;
                      break;
        case SIN_OP : *optype = Mcalc_OperatorType::Sine;
                      break;
        case COS_OP : *optype = Mcalc_OperatorType::Cosine;
                      break;
        case TAN_OP : *optype = Mcalc_OperatorType::Tangent;
                      break;
        case ASIN_OP : *optype = Mcalc_OperatorType::ArcSine;
                      break;
        case ACOS_OP : *optype = Mcalc_OperatorType::ArcCosine;
                      break;
        case ATAN_OP : *optype = Mcalc_OperatorType::ArcTangent;
                      break;
        case SINH_OP : *optype = Mcalc_OperatorType::HyperSine;
                      break;
        case COSH_OP : *optype = Mcalc_OperatorType::HyperCosine;
                      break;
        case TANH_OP : *optype = Mcalc_OperatorType::HyperTangent;
                       break;
        case EXP_OP : *optype = Mcalc_OperatorType::NaturalExp;
                       break;
        case LOG_OP : *optype = Mcalc_OperatorType::NaturalLog;
                       break;
        case LOG10_OP : *optype = Mcalc_OperatorType::Log10;
                        break;
        case CEILING_OP : *optype = Mcalc_OperatorType::Ceiling;
                          break;
        case FLOOR_OP : *optype = Mcalc_OperatorType::Floor;
                        break;
        case ABS_OP : *optype = Mcalc_OperatorType::AbsoluteValue;
                      break;
        case FRACPART_OP : *optype = Mcalc_OperatorType::FractionPart;
                           break;
        case INTPART_OP : *optype = Mcalc_OperatorType::IntegerPart;
                          break;
        case '%' : *optype = Mcalc_OperatorType::Percent;
                   break;
        case '<' : *optype = Mcalc_OperatorType::LessThan;
                   break;
        case '>' : *optype = Mcalc_OperatorType::GreaterThan;
                   break;
        case RND_OP : *optype = Mcalc_OperatorType::Round;
                      break;
        case FACTORIAL_OP : *optype = Mcalc_OperatorType::Factorial;
                   break;
        case PERM_OP : *optype = Mcalc_OperatorType::Permutation;
                       break;
        case COMB_OP : *optype = Mcalc_OperatorType::Combination;
                       break;
        case COMBREP_OP : *optype = Mcalc_OperatorType::CombineWithRep;
                          break;
        case PERMREP_OP : *optype = Mcalc_OperatorType::PermuteWithRep;
                          break;
        case LSTHANOREQ_OP : *optype = Mcalc_OperatorType::LessOrEqual;
                             break;
        case GRTHANOREQ_OP : *optype = Mcalc_OperatorType::GreaterOrEqual;
                             break;
        case EQUAL_OP : *optype = Mcalc_OperatorType::Equal;
                        break;
        case NOTEQUAL_OP : *optype = Mcalc_OperatorType::NotEqual;
                           break;
        case RAND_OP : *optype = Mcalc_OperatorType::Random;
                       break;
        case XOR_OP : *optype = Mcalc_OperatorType::Exclusive_Or;
                      break;
        case INV_OP : *optype = Mcalc_OperatorType::Inverse;
                      break;
        case SCI_OP : *optype = Mcalc_OperatorType::ScientificNotation;
                      break;

        default : if (!IsUnprocessedSymbols(*buffer))
                    OpFound_ = FALSE;
                  else
                    DoCont_ = TRUE;
                  break;
      }

      if (DoCont_)
      {
        DoCont_ = FALSE;
        continue;
      }

      list[3] = (char*)pop(rpnstack);
      list[2] = (char*)pop(rpnstack);

      MemMatrix::Matrix().Deallocate(buffer);
      buffer = NULL;
      return TRUE;
    }
  }

  if (!OpFound_ && *rpnstack)
  {
    *optype = Mcalc_OperatorType::Assignment;
    list[3] = (char*)pop(rpnstack);
    list[2] = ::NewString(list[3]);
    return TRUE;
  }

  return FALSE;
}

/*****************************************************************************
Procedure for allocating memory using calloc. If memory alloction fails the
procedure terminates the program.
PASSED:
  items : the number of items of size "size" to be allocated
  size : the size of each item
  block : the pointer to the memory to be allocated
CHANGED:
  block is allocated to point to (items * size) memory
RETURNED:
  the pointer to the memory space is returned
*/
void* CalculatorBase::allocmem(unsigned items, unsigned size, void* block)
{
  if ((block=MemMatrix::Matrix().Callocate(items * size))==NULL)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MEMORY_FAIL, ERRMSG_MEMORY_FAIL);
    exit(1);
  }

  return block;
}

/*****************************************************************************
Procedure to reallocate memory using realloc. if memory allocation fails the
procedure terminates the program.
PASSED:
  items : the number of items of size "size" to be allocated
  size : the size of each item
  block : the pointer to the memory to be allocated
CHANGED:
  block is allocated to point to (items * size) memory
RETURNED:
  the pointer to the memory space is returned
*/
void* CalculatorBase::reallocmem(unsigned items, unsigned size, void* block)
{
  if ((block=MemMatrix::Matrix().Reallocate(block, items * size))==NULL)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MEMORY_FAIL, ERRMSG_MEMORY_FAIL);
    exit(1);
  }
  return block;
}

/*****************************************************************************
Procedure to compare two strings for equality. If both strings are NULL
then procedure returns 1, otherwise the strings are compared character
by character up to max characters same as strncmp.
PARAMETERS:
  char* s1 : the first string
  char* s2 : the second string
  int max : the maximum number of characters to compare in each string
RETURNED:
  returns 1 if both string matches in characters this also include the
  the case in which both strings are zero length with an initial NULL char
  ,otherwise returns 0 if the strings have different characters.
*/
int CalculatorBase::icasestrcompn(const char* s1, const char* s2, int max)
{
  int i;

  for (i = 0; i < max; i++)
  {
    if (!(*s1) && !(*s2))
      return 1;
    else if (!(*s1) || !(*s2))
      return 0;

    if (isalpha(*s1) && isalpha(*s2))
    {
      if (tolower(*s1) != tolower(*s2))
        return 0;
    }
    else if ((*s1) != (*s2))
      return 0;
    s1++;
    s2++;
  }

  return 1;
}

/*****************************************************************************
Procedure to compare two strings for equality. If both strings are NULL
then procedure returns 1
PARAMETERS:
  char* s1 : the first string
  char* s2 : the second string
RETURNED:
  returns 1 if both string matches in characters this also include the
  the case in which both strings are zero length with an initial NULL char
  ,otherwise returns 0 if the strings have different characters.
*/
int CalculatorBase::icasestrcomp(const char* s1, const char* s2)
{
  int i;
  int max = strlen(s1);

  if (strlen(s2) > max)
    max = strlen(s2);

  for (i = 0; i < max; i++)
  {
    if (!(*s1) && !(*s2))
      return 1;
    else if (!(*s1) || !(*s2))
      return 0;

    if (isalpha(*s1) && isalpha(*s2))
    {
      if (tolower(*s1) != tolower(*s2))
        return 0;
    }
    else if ((*s1) != (*s2))
      return 0;
    s1++;
    s2++;
  }

  return 1;
}

/*****************************************************************************/
long CalculatorBase::FindNumIntervals(int NumParams_, FncParameter** ParamArray_)
{
  using ::pow;
  using ::sqrt;

  double dprec_ = DEFAULT_PRECVAL_INTEG;
  double gprec_ = GraphPrecision();

  if (0 < gprec_ && gprec_ < DEFAULT_PRECVAL_GRAPH && gprec_ != dprec_)
    dprec_ = gprec_;

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  long Lower_ = atol(ParamArray_[2]->_Param->GetAnswer());
  long Upper_ = atol(ParamArray_[3]->_Param->GetAnswer());
  double Prec_ = (NumParams_ >= 5) ? strtod(ParamArray_[4]->_Param->GetAnswer(), NULL):dprec_;

  Ulong InvPrec_ = Ulong(1 / ::fabs(Prec_));
  Ulong Multiplier_ = 0;
  int ExpFound_;
  long Exp_;

  double Div_;
  size_t x, y;
  Ulong LastVal_;
  Ulong NumInt_ = ::labs(Upper_ - Lower_);

  if (NumInt_ < 1UL)
    NumInt_ = 1UL;
  else
  {
    LastVal_ = NumInt_;

    // Guard against Ulong overflow in NumInt_
    for (x = 1;; x++)
    {
      NumInt_ = (Ulong)pow(NumInt_, 3UL);

      if (NumInt_ < LastVal_)
      {
        NumInt_ = LastVal_;
        Div_ = ::pow(2, x);
        NumInt_ = Ulong(NumInt_ / Div_);
      }
      else
      {
        LastVal_ = NumInt_;
        break;
      }

    }
  }

  for (x = 0; x < ExprStr_.strlen(); x++)
  {
    if (ExprStr_[x] == '^')
    {
      for (y = x + 1; y < ExprStr_.strlen(); y++)
      {
        if (isdigit(ExprStr_[y]))
        {
          Exp_ = atol(ExprStr_(y));
          ExpFound_ = TRUE;
          break;
        }
        else if (!isspace(ExprStr_[y]))
          break;
      }

      if (ExpFound_)
      {
        if (Exp_ > 2)
          Multiplier_ += (Ulong)pow(Upper_, Exp_ - 2) * (Exp_ * (Exp_ - 1));
        else if (Exp_ > 0)
          Multiplier_ += Ulong(Exp_);

        ExpFound_ = FALSE;
      }
    }
  }

  if (!Multiplier_)
    Multiplier_ = 1UL;

  if (ExpFound_ && Exp_ > 2 && Multiplier_ < 10)
    Multiplier_ = 10;

  LastVal_ = NumInt_;

  // Guard against Ulong overflow in NumInt_
  for (;;)
  {
    NumInt_ *= InvPrec_;     // possible overflow with this mult.

    if (NumInt_ < LastVal_ || (NumInt_ == LastVal_ && InvPrec_ > 1UL))
    {
      NumInt_ = LastVal_;
      InvPrec_ /= 2;
    }
    else
    {
      LastVal_ = NumInt_;
      break;
    }
  }

  // Guard against Ulong overflow in NumInt_
  for (;;)
  {
    NumInt_ *= Multiplier_;  // possible overflow with this mult.

    if (NumInt_ < LastVal_ || (NumInt_ == LastVal_ && Multiplier_ > 1UL))
    {
      NumInt_ = LastVal_;
      Multiplier_ /= 2;
    }
    else
    {
      LastVal_ = NumInt_;
      break;
    }
  }

  NumInt_ /= 12L;
  NumInt_ = (Ulong)sqrt(NumInt_);

  return NumInt_;
}

/*****************************************************************************/
const char* CalculatorBase::GetFncNameFromOpcode(int Opcode_)
{
  return
  (
    (Opcode_ == Mcalc_OperatorType::Logical_Not) ? "NOT":
    (Opcode_ == Mcalc_OperatorType::Sine) ? "SIN":
    (Opcode_ == Mcalc_OperatorType::Cosine) ? "COS":
    (Opcode_ == Mcalc_OperatorType::Tangent) ? "TAN":
    (Opcode_ == Mcalc_OperatorType::ArcSine) ? "ARCSIN":
    (Opcode_ == Mcalc_OperatorType::ArcCosine) ? "ARCCOS":
    (Opcode_ == Mcalc_OperatorType::ArcTangent) ? "ARCTAN":
    (Opcode_ == Mcalc_OperatorType::Add) ? "+":
    (Opcode_ == Mcalc_OperatorType::Subtract) ? "-":
    (Opcode_ == Mcalc_OperatorType::Multiply) ? "*":
    (Opcode_ == Mcalc_OperatorType::Divide) ? "/":
    (Opcode_ == Mcalc_OperatorType::Modulus) ? "@":
    (Opcode_ == Mcalc_OperatorType::Logical_And) ? "AND":
    (Opcode_ == Mcalc_OperatorType::Logical_Or) ? "OR":
    (Opcode_ == Mcalc_OperatorType::Exponent) ? "^":
    (Opcode_ == Mcalc_OperatorType::Square) ? "SQR":
    (Opcode_ == Mcalc_OperatorType::SquareRoot) ? "SQRT":
    (Opcode_ == Mcalc_OperatorType::Cube) ? "CUBE":
    (Opcode_ == Mcalc_OperatorType::CubeRoot) ? "CUBERT":
    (Opcode_ == Mcalc_OperatorType::HyperSine) ? "SINH":
    (Opcode_ == Mcalc_OperatorType::HyperCosine) ? "COSH":
    (Opcode_ == Mcalc_OperatorType::HyperTangent) ? "TANH":
    (Opcode_ == Mcalc_OperatorType::NaturalExp) ? "EXP":
    (Opcode_ == Mcalc_OperatorType::NaturalLog) ? "LOG":
    (Opcode_ == Mcalc_OperatorType::Log10) ? "LOG10":
    (Opcode_ == Mcalc_OperatorType::Ceiling) ? "CEIL":
    (Opcode_ == Mcalc_OperatorType::Floor) ? "FLOOR":
    (Opcode_ == Mcalc_OperatorType::AbsoluteValue) ? "ABS":
    (Opcode_ == Mcalc_OperatorType::FractionPart) ? "FRACPART":
    (Opcode_ == Mcalc_OperatorType::IntegerPart) ? "INTPART":
    (Opcode_ == Mcalc_OperatorType::Percent) ? "%":
    (Opcode_ == Mcalc_OperatorType::LessThan) ? "<":
    (Opcode_ == Mcalc_OperatorType::GreaterThan) ? ">":
    (Opcode_ == Mcalc_OperatorType::LessOrEqual) ? "<=":
    (Opcode_ == Mcalc_OperatorType::GreaterOrEqual) ? ">=":
    (Opcode_ == Mcalc_OperatorType::Equal) ? "==":
    (Opcode_ == Mcalc_OperatorType::NotEqual) ? "!=":
    (Opcode_ == Mcalc_OperatorType::Round) ? "ROUND":
    (Opcode_ == Mcalc_OperatorType::Factorial) ? "FACT":
    (Opcode_ == Mcalc_OperatorType::Permutation) ? "PERM":
    (Opcode_ == Mcalc_OperatorType::Combination) ? "COMB":
    (Opcode_ == Mcalc_OperatorType::CombineWithRep) ? "COMBREP":
    (Opcode_ == Mcalc_OperatorType::PermuteWithRep) ? "PERMREP":
    (Opcode_ == Mcalc_OperatorType::Assignment) ? "=":
    (Opcode_ == Mcalc_OperatorType::Random) ? "RAND":
    (Opcode_ == Mcalc_OperatorType::Exclusive_Or) ? "XOR":
    (Opcode_ == Mcalc_OperatorType::Inverse) ? "INV":
    (Opcode_ == Mcalc_OperatorType::ScientificNotation) ? "E":""
  );
}

/*****************************************************************************/
ChrString CalculatorBase::ShowFracDivZero(const LongNumber& TotalNumer_, const LongNumber& TotalDenom_)
{
  ChrString AnswerStr_;

  if (!_PlottingGraph)
  {
    ChrString Numer_, Denom_;
    Numer_ = TotalNumer_.ToString();
    Denom_ = TotalDenom_.ToString();
    Numer_ += ChrString("/") + Denom_;

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_DIVISION_BY_ZERO,
                        ERRMSG_DIVISION_BY_ZERO, Numer_.c_str());
    CloseOutputFile();
  }
  else
  {
    int nsign_ = (TotalNumer_.GetSign() == NumberSign::PLUS) ? 1:-1;
    int dsign_ = (TotalDenom_.GetSign() == NumberSign::PLUS) ? 1:-1;
    nsign_ *= dsign_;

    if (nsign_ > 0)
      AnswerStr_ = "+inf";
    else
      AnswerStr_ = "-inf";
  }

  return AnswerStr_;
}

/*****************************************************************************/
Ldouble* CalculatorBase::StatFileToArray(ifstream& Fin_, size_t& sz)
{
  char Buffer_[256];
  Ldouble* Array_ = NULL;
  Ldouble* Temp_ = NULL;
  size_t size_ = 100;
  size_t x;

  Array_ = (Ldouble*)allocmem(size_, sizeof(Ldouble), Array_);
  sz = 0;

  while (!Fin_.eof())
  {
    Fin_.getline(Buffer_, 256);

    if (strlen(Buffer_))
    {
      Array_[sz++] = strtold(Buffer_, NULL);

      if (sz == size_)
      {
        Temp_ = Array_;
        size_ += 50;
        Array_ = (Ldouble*)allocmem(size_, sizeof(Ldouble), Array_);

        for (x = 0; x < sz; x++)
          Array_[x] = Temp_[x];

        MemMatrix::Matrix().Deallocate(Temp_);
      }
    }
  }

  if (!sz)
  {
    MemMatrix::Matrix().Deallocate(Array_);
    Array_ = NULL;
  }

  Fin_.clear();
  return Array_;
}

/*****************************************************************************/
Ldouble CalculatorBase::MinimumValue(int NumParams_, FncParameter** ParamArray_)
{
  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();

    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("MIN(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        LongDouble Answer_(AnswerStr_);
        SetAnswerMode(FLOATINGPOINT_MODE);
        return Answer_.GetData();
      }
    }
    else
    {
      ifstream Fin_;
      ChrString Expr_ = ParamArray_[0]->_Param->GetExpression();

      if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
      {
        size_t sz = 0;
        Ldouble* RawArray_ = NULL;
        RawArray_ = StatFileToArray(Fin_, sz);
        Fin_.close();

        if (sz && RawArray_ && NumParams_)
        {
          StatArray<Ldouble> Array_(sz, default_size);

          size_t i;
          for (i = 0; i < sz; i++)
            Array_[i] = RawArray_[i];

          if (Array_.ErrorStatus())
            DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                                ERRMSG_EMPTY_DATASET, "MIN");

          MemMatrix::Matrix().Deallocate(RawArray_);
          return Array_.Min();
        }
      }
    }
  }
  else
  {
    int Size_ = atoi(ParamArray_[0]->_Param->GetExpression());
    if (Size_ != NumParams_ - 1)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "MIN");
      CloseOutputFile();
      return 0;
    }

    if (Size_ && NumParams_)
    {
      StatArray<Ldouble> Array_(Size_, default_size);

      size_t i;
      for (i = 1; i <= Size_; i++)
        Array_[i-1] = strtold(ParamArray_[i]->_Param->GetAnswer(), NULL);

      if (Array_.ErrorStatus())
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                            ERRMSG_EMPTY_DATASET, "MIN");

      return Array_.Min();
    }
  }

  return 0;
}

/*****************************************************************************/
Ldouble CalculatorBase::MaximumValue(int NumParams_, FncParameter** ParamArray_)
{
  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();

    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("MAX(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        LongDouble Answer_(AnswerStr_);
        SetAnswerMode(FLOATINGPOINT_MODE);
        return Answer_.GetData();
      }
    }
    else
    {
      ifstream Fin_;
      ChrString Expr_ = ParamArray_[0]->_Param->GetExpression();

      if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
      {
        size_t sz = 0;
        Ldouble* RawArray_ = NULL;
        RawArray_ = StatFileToArray(Fin_, sz);
        Fin_.close();

        if (sz && RawArray_ && NumParams_)
        {
          StatArray<Ldouble> Array_(sz, default_size);

          size_t i;
          for (i = 0; i < sz; i++)
            Array_[i] = RawArray_[i];

          if (Array_.ErrorStatus())
            DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                                ERRMSG_EMPTY_DATASET, "MAX");

          MemMatrix::Matrix().Deallocate(RawArray_);
          return Array_.Max();
        }
      }
    }
  }
  else
  {
    int Size_ = atoi(ParamArray_[0]->_Param->GetExpression());
    if (Size_ != NumParams_ - 1)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "MAX");
      CloseOutputFile();
      return 0;
    }

    if (Size_ && NumParams_)
    {
      StatArray<Ldouble> Array_(Size_, default_size);

      size_t i;
      for (i = 1; i <= Size_; i++)
        Array_[i-1] = strtold(ParamArray_[i]->_Param->GetAnswer(), NULL);

      if (Array_.ErrorStatus())
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                            ERRMSG_EMPTY_DATASET, "MAX");

      return Array_.Max();
    }
  }

  return 0;
}

/*****************************************************************************/
Ldouble CalculatorBase::CumSum(int NumParams_, FncParameter** ParamArray_)
{
  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();

    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("SUM(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        LongDouble Answer_(AnswerStr_);
        SetAnswerMode(FLOATINGPOINT_MODE);
        return Answer_.GetData();
      }
    }
    else
    {
      ifstream Fin_;
      ChrString Expr_ = ParamArray_[0]->_Param->GetExpression();

      if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
      {
        Boolean Error_;
        Ldouble RetVal_ = StatArray<Ldouble>::Sum(Fin_, Error_);
        Fin_.close();
        if (Error_)
          DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                              ERRMSG_EMPTY_DATASET, "SUM");

        return RetVal_;
      }
    }
  }
  else
  {
    int Size_ = atoi(ParamArray_[0]->_Param->GetExpression());
    if (Size_ != NumParams_ - 1)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "SUM");
      CloseOutputFile();
      return 0;
    }

    if (Size_ && NumParams_)
    {
      StatArray<Ldouble> Array_(Size_, default_size);

      size_t i;
      for (i = 1; i <= Size_; i++)
        Array_[i-1] = strtold(ParamArray_[i]->_Param->GetAnswer(), NULL);

      if (Array_.ErrorStatus())
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                            ERRMSG_EMPTY_DATASET, "SUM");

      return Array_.Sum();
    }
  }

  return 0;
}

/*****************************************************************************/
Ldouble CalculatorBase::CumProduct(int NumParams_, FncParameter** ParamArray_)
{
  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();

    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("PROD(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        LongDouble Answer_(AnswerStr_);
        SetAnswerMode(FLOATINGPOINT_MODE);
        return Answer_.GetData();
      }
    }
    else
    {
      ifstream Fin_;
      ChrString Expr_ = ParamArray_[0]->_Param->GetExpression();

      if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
      {
        Boolean Error_;
        Ldouble RetVal_ = StatArray<Ldouble>::Product(Fin_, Error_);
        Fin_.close();
        if (Error_)
          DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                              ERRMSG_EMPTY_DATASET, "PROD");

        return RetVal_;
      }
    }
  }
  else
  {
    int Size_ = atoi(ParamArray_[0]->_Param->GetExpression());
    if (Size_ != NumParams_ - 1)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "PROD");
      CloseOutputFile();
      return 0;
    }

    if (Size_ && NumParams_)
    {
      StatArray<Ldouble> Array_(Size_, default_size);

      size_t i;
      for (i = 1; i <= Size_; i++)
        Array_[i-1] = strtold(ParamArray_[i]->_Param->GetAnswer(), NULL);

      if (Array_.ErrorStatus())
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                            ERRMSG_EMPTY_DATASET, "PROD");

      return Array_.Product();
    }
  }

  return 0;
}

/*****************************************************************************/
Ldouble CalculatorBase::AverageMean(int NumParams_, FncParameter** ParamArray_)
{
  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();

    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("AVGMEAN(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        LongDouble Answer_(AnswerStr_);
        SetAnswerMode(FLOATINGPOINT_MODE);
        return Answer_.GetData();
      }
    }
    else
    {
      ifstream Fin_;
      ChrString Expr_ = ParamArray_[0]->_Param->GetExpression();

      if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
      {
        Boolean Error_;
        Ldouble RetVal_ = StatArray<Ldouble>::AvgMean(Fin_, Error_);
        Fin_.close();
        if (Error_)
          DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                              ERRMSG_EMPTY_DATASET, "AVGMEAN");

        return RetVal_;
      }
    }
  }
  else
  {
    int Size_ = atoi(ParamArray_[0]->_Param->GetExpression());
    if (Size_ != NumParams_ - 1)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "AVGMEAN");
      CloseOutputFile();
      return 0;
    }

    if (Size_ && NumParams_)
    {
      StatArray<Ldouble> Array_(Size_, default_size);

      size_t i;
      for (i = 1; i <= Size_; i++)
        Array_[i-1] = strtold(ParamArray_[i]->_Param->GetAnswer(), NULL);

      if (Array_.ErrorStatus())
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                            ERRMSG_EMPTY_DATASET, "AVGMEAN");

      return Array_.Mean();
    }
  }

  return 0;
}

/*****************************************************************************/
Ldouble CalculatorBase::AverageMedian(int NumParams_, FncParameter** ParamArray_)
{
  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();

    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("AVGMED(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        LongDouble Answer_(AnswerStr_);
        SetAnswerMode(FLOATINGPOINT_MODE);
        return Answer_.GetData();
      }
    }
    else
    {
      ifstream Fin_;
      ChrString Expr_ = ParamArray_[0]->_Param->GetExpression();

      if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
      {
        size_t sz = 0;
        Ldouble* RawArray_ = NULL;
        RawArray_ = StatFileToArray(Fin_, sz);
        Fin_.close();

        if (sz && RawArray_ && NumParams_)
        {
          StatArray<Ldouble> Array_(sz, default_size);

          size_t i;
          for (i = 0; i < sz; i++)
            Array_[i] = RawArray_[i];

          if (Array_.ErrorStatus())
            DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                                ERRMSG_EMPTY_DATASET, "AVGMED");

          MemMatrix::Matrix().Deallocate(RawArray_);
          return Array_.Median();
        }
      }
    }
  }
  else
  {
    int Size_ = atoi(ParamArray_[0]->_Param->GetExpression());
    if (Size_ != NumParams_ - 1)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "AVGMED");
      CloseOutputFile();
      return 0;
    }

    if (Size_ && NumParams_)
    {
      StatArray<Ldouble> Array_(Size_, default_size);

      size_t i;
      for (i = 1; i <= Size_; i++)
        Array_[i-1] = strtold(ParamArray_[i]->_Param->GetAnswer(), NULL);

      if (Array_.ErrorStatus())
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                            ERRMSG_EMPTY_DATASET, "AVGMED");

      return Array_.Median();
    }
  }

  return 0;
}

/*****************************************************************************/
Ldouble CalculatorBase::AverageMode(int NumParams_, FncParameter** ParamArray_)
{
  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();

    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("AVGMODE(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        LongDouble Answer_(AnswerStr_);
        SetAnswerMode(FLOATINGPOINT_MODE);
        return Answer_.GetData();
      }
    }
    else
    {
      ifstream Fin_;
      ChrString Expr_ = ParamArray_[0]->_Param->GetExpression();

      if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
      {
        size_t sz = 0;
        Ldouble* RawArray_ = NULL;
        RawArray_ = StatFileToArray(Fin_, sz);
        Fin_.close();

        if (sz && RawArray_ && NumParams_)
        {
          StatArray<Ldouble> Array_(sz, default_size);

          size_t i;
          for (i = 0; i < sz; i++)
            Array_[i] = RawArray_[i];

          Ldouble** PtrArray_ = Array_.Mode();
          Ldouble RetVal_ = 0;

          if (PtrArray_[0])
            RetVal_ = *PtrArray_[0];

          if (Array_.ErrorStatus())
            DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                                ERRMSG_EMPTY_DATASET, "AVGMODE");

          for (i = 0; PtrArray_[i]; i++)
            delete PtrArray_[i];

          delete[] PtrArray_;
          MemMatrix::Matrix().Deallocate(RawArray_);

          return RetVal_;
        }
      }
    }
  }
  else
  {
    int Size_ = atoi(ParamArray_[0]->_Param->GetExpression());
    if (Size_ != NumParams_ - 1)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "AVGMODE");
      CloseOutputFile();
      return 0;
    }

    if (Size_ && NumParams_)
    {
      StatArray<Ldouble> Array_(Size_, default_size);

      size_t i;
      for (i = 1; i <= Size_; i++)
        Array_[i-1] = strtold(ParamArray_[i]->_Param->GetAnswer(), NULL);

      Ldouble** PtrArray_ = Array_.Mode();
      Ldouble RetVal_ = 0;

      if (PtrArray_[0])
        RetVal_ = *PtrArray_[0];

      if (Array_.ErrorStatus())
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                            ERRMSG_EMPTY_DATASET, "AVGMODE");

      for (i = 0; PtrArray_[i]; i++)
        delete PtrArray_[i];

      delete[] PtrArray_;
      return RetVal_;
    }
  }

  return 0;
}

/*****************************************************************************/
Ldouble CalculatorBase::StandardDeviation(int NumParams_, FncParameter** ParamArray_)
{
  if (!IsNumber(ParamArray_[0]->_Param->GetExpression(), 0, 0, 0, 10) || NumParams_ == 1)
  {
    const char* Buffer_ = ParamArray_[0]->_Param->GetExpression();

    if (strchr(Buffer_, ','))
    {
      if (NumParams_ == 1)
      {
        ChrString ArgStr_ = ChrString("STDDEV(") + ChrString(Buffer_) + ChrString(")");
        ChrString AnswerStr_ = ReEvaluate(ArgStr_);
        LongDouble Answer_(AnswerStr_);
        SetAnswerMode(FLOATINGPOINT_MODE);
        return Answer_.GetData();
      }
    }
    else
    {
      ifstream Fin_;
      ChrString Expr_ = ParamArray_[0]->_Param->GetExpression();

      if (FileReader::Fexists(Expr_.c_str()) && FileStreamOpen(Fin_, Expr_.c_str()))
      {
        Boolean Error_;
        Ldouble RetVal_ = StatArray<Ldouble>::StdDeviation(Fin_, Error_);
        Fin_.close();
        if (Error_)
          DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                              ERRMSG_EMPTY_DATASET, "STDDEV");

        return RetVal_;
      }
    }
  }
  else
  {
    int Size_ = atoi(ParamArray_[0]->_Param->GetExpression());
    if (Size_ != NumParams_ - 1)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_ARGLIST_SIZE,
                          ERRMSG_ARGLIST_SIZE, "STDDEV");
      CloseOutputFile();
      return 0;
    }

    if (Size_ && NumParams_)
    {
      StatArray<Ldouble> Array_(Size_, default_size);

      size_t i;
      for (i = 1; i <= Size_; i++)
        Array_[i-1] = strtold(ParamArray_[i]->_Param->GetAnswer(), NULL);

      if (Array_.ErrorStatus())
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTY_DATASET,
                            ERRMSG_EMPTY_DATASET, "STDDEV");

      return Array_.StdDeviation();
    }
  }

  return 0;
}

/*****************************************************************************/
void CalculatorBase::SaveCurrentGraphName(const char Suffix_)
{
  char CurGraphFname_[32];
  sprintf(CurGraphFname_, "mcalc_graph%d%c.out", SessionNum(), Suffix_);
  FILE* Fout_ = fopen(GetCurrentGraphFilename(), "w");

  if (Fout_)
  {
    fprintf(Fout_, "%s\n", CurGraphFname_);
    fclose(Fout_);
  }
}

/*****************************************************************************/
char CalculatorBase::PlotFileSuffix()
{
  if (_PlotFileSuffix == 'y')
    _PlotFileSuffix = 'a';

  char RetVal_ = _PlotFileSuffix++;
  SaveCurrentGraphName(RetVal_);
  return RetVal_;
}

/*****************************************************************************/
void CalculatorBase::ClearGraphs()
{
  char Fname_[32];
  char ch;

  for (ch = 'a'; ch <= 'z'; ch++)
  {
    sprintf(Fname_, "mcalc_graph%d%c.out", SessionNum(), ch);
    unlink(Fname_);
  }

  _PlotFileSuffix = 'a';
}

/*****************************************************************************/
/*****************************************************************************/
LongNumCalculator::LongNumCalculator():
_Epsilon("0.0000000000000000000000000000000000000001")
{
  int i;
  for (i = 0; i < MAXVARS; i++)
    _vars[i] = NULL;

  _IntCoords =
  (IntCoords<LongNumber>**)MemMatrix::Matrix().Callocate(MAX_INT_COORDS * sizeof(IntCoords<LongNumber>*));
  for (i = 0; i < MAX_INT_COORDS; i++)
    _IntCoords[i] = new IntCoords<LongNumber>();
}

/*****************************************************************************/
LongNumCalculator::~LongNumCalculator()
{
  int i;
  for (i = 0; i < MAXVARS; i++)
    delete _vars[i];

  if (_IntCoords)
  {
    for (i = 0; i < MAX_INT_COORDS; i++)
      delete _IntCoords[i];

    MemMatrix::Matrix().Deallocate(_IntCoords);
    _IntCoords = NULL;
  }
}

/*****************************************************************************/
void LongNumCalculator::SetParent(CalculatorBase* Ptr_)
{
  _ParentPtr = Ptr_;
  _CalcIndex = CalculatorBase::LONGNUM;
}

/*****************************************************************************/
ChrString LongNumCalculator::ToString(int Index_)
{
  ChrString DestStr_;
  LongNumber FpObj_;
  static LongNumber Dummy_;

  if (Index_ >= 0 && IsStringVar(Index_))
  {
    CopyStrVar(DestStr_, Index_);
    return DestStr_;
  }

  if (Index_ < 0)
    FpObj_ = resultvalue;
  else if (_vars[Index_])
    FpObj_ = *(_vars[Index_]);
  else
    return Dummy_.ToString();

  return FpObj_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_)
{
  const char* ParamStr_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongNumber HiTestPt_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongNumber LoTestPt_ = HiTestPt_;
  LongNumber Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  LongNumber Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
  LongNumber Tol_ = ChrString(ParamStr_);
  LongNumber Incr_ = ChrString(DEFAULT_PRECSTR_APPROXZERO);

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  ChrString ValStr_;
  LongNumber LoResult1_;
  LongNumber* LoResult2_ = NULL;
  LongNumber HiResult1_;
  LongNumber* HiResult2_ = NULL;
  LongNumber Low_;
  LongNumber High_;
  LongNumber Testx_;
  LongNumber ZeroPt_;
  int Sign_;
  int ZeroCrossing_;
  ZeroFound_ = FALSE;

  if (HasErrors())
    ClearError();

  while ((HiTestPt_ <= Upper_ || LoTestPt_ >= Lower_) && !HasErrors())
  {
    if (LoTestPt_ >= Lower_)
    {
      ValStr_ = LoTestPt_.ToString();
      FncVal_ = ChrString("(") + ValStr_ + ChrString(")");
      LoResult1_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (LoResult1_.AtZero())
      {
        Testx_ = ValStr_;
        ZeroFound_ = TRUE;
        break;
      }
      else if (!LoResult2_)
        LoResult2_ = new LongNumber(LoResult1_);
      else
      {
        ZeroCrossing_ = LoResult1_.GetSign() == NumberSign::MINUS &&
                        LoResult2_->GetSign() == NumberSign::PLUS ||
                        LoResult1_.GetSign() == NumberSign::PLUS &&
                        LoResult2_->GetSign() == NumberSign::MINUS;

        if (ZeroCrossing_)
        {
          Low_ = LoTestPt_;
          High_ = LoTestPt_ + Incr_;
          ValStr_ = Low_.ToString();
          ParamArray_[3]->_Param->SetAnswer(ValStr_.c_str());
          ValStr_ = High_.ToString();
          ParamArray_[4]->_Param->SetAnswer(ValStr_.c_str());
          Testx_ = (High_ + Low_) / 2L;
          break;
        }
        else
        {
          *LoResult2_ = LoResult1_;
          LoTestPt_ -= Incr_;
        }
      }
    }

    if (HiTestPt_ <= Upper_)
    {
      ValStr_ = HiTestPt_.ToString();
      FncVal_ = ChrString("(") + ValStr_ + ChrString(")");
      HiResult1_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (HiResult1_.AtZero())
      {
        Testx_ = ValStr_;
        ZeroFound_ = TRUE;
        break;
      }
      else if (!HiResult2_)
        HiResult2_ = new LongNumber(HiResult1_);
      else
      {
        ZeroCrossing_ = HiResult1_.GetSign() == NumberSign::MINUS &&
                        HiResult2_->GetSign() == NumberSign::PLUS ||
                        HiResult1_.GetSign() == NumberSign::PLUS &&
                        HiResult2_->GetSign() == NumberSign::MINUS;

        if (ZeroCrossing_)
        {
          Low_ = HiTestPt_ - Incr_;
          High_ = HiTestPt_;
          ValStr_ = Low_.ToString();
          ParamArray_[3]->_Param->SetAnswer(ValStr_.c_str());
          ValStr_ = High_.ToString();
          ParamArray_[4]->_Param->SetAnswer(ValStr_.c_str());
          Testx_ = (High_ + Low_) / 2L;
          break;
        }
        else
        {
          *HiResult2_ = HiResult1_;
          HiTestPt_ += Incr_;
        }
      }
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  LongNumber xTol_ = Testx_;
  AbsoluteValue(xTol_);

  while (Tol_ < 1L)
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < 1L)
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete LoResult2_;
  delete HiResult2_;

  if (HasErrors())
    ClearError();

  Testx_.Round(Exp_);
  return Testx_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::FindDerivative(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      (NumParams_ >= 4 &&
       !LongNumber::IsValue(ParamArray_[3]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "DERIV");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  ChrString NewExprStr_ = ParamArray_[2]->_Param->GetAnswer();
  LongNumber Value_ = NewExprStr_;
  NewExprStr_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Buffer_;
  LongNumber Prec_ = NewExprStr_;
  LongNumber HalfPrec_ = Prec_ / 2L;
  HalfPrec_ *= LongNumber(".8");

  if (_PlotGraph)
  {
    _PlotGraph = false;
    PlotFunction(2, ParamArray_, Value_.ToString());
  }

  LongNumber HighFncRet_;
  LongNumber LowFncRet_;
  LongNumber HighVal_ = Value_ + HalfPrec_;
  LongNumber LowVal_ = Value_ - HalfPrec_;

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + LowVal_.ToString() + ChrString(")");
  LowFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + HighVal_.ToString() + ChrString(")");
  HighFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  HighFncRet_ -= LowFncRet_;
  HighVal_ -= LowVal_;

  LongNumber Slope_ = HighFncRet_;
  Slope_ /= HighVal_;
  PopSubExprStack();

  long Exp_ = 0;
  LongNumber xTol_ = Slope_;
  AbsoluteValue(xTol_);

  while (Prec_ < 1L)
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < 1L)
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_DERIV;

  Slope_.Round(Exp_);
  return Slope_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::FindDerivative2(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      (NumParams_ >= 4 &&
       !LongNumber::IsValue(ParamArray_[3]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "DERIV2");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  ChrString NewExprStr_ = ParamArray_[2]->_Param->GetAnswer();
  LongNumber Value_ = NewExprStr_;
  NewExprStr_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Buffer_;
  LongNumber Prec_ = NewExprStr_;
  LongNumber HalfPrec_ = Prec_ / 2L;
  HalfPrec_ *= LongNumber(".8");

  if (_PlotGraph)
  {
    _PlotGraph = false;
    ChrString TempStr_ = ChrString("DERIV(") + ExprStr_;
    TempStr_ += ChrString(", ") + VarName_;
    TempStr_ += ChrString(", ") + VarName_;
    TempStr_ += ChrString("2)");

    FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(2 * sizeof(FncParameter*));
    Params_[0] = new FncParameter();
    Params_[0]->_Param = new SubExpression();
    Params_[0]->_Param->SetExpression(TempStr_.c_str());
    Params_[0]->_ShouldEval = FALSE;
    TempStr_ = VarName_ + ChrString("2");
    Params_[1] = new FncParameter();
    Params_[1]->_Param = new SubExpression();
    Params_[1]->_Param->SetExpression(TempStr_.c_str());
    Params_[1]->_ShouldEval = FALSE;

    PlotFunction(2, Params_, Value_.ToString());

    int x;
    for (x = 0; x < 2; x++)
      delete Params_[x];

    MemMatrix::Matrix().Deallocate(Params_);
  }

  LongNumber HighFncRet_;
  LongNumber LowFncRet_;
  LongNumber MidTermRet_;
  LongNumber HighVal_ = Value_ + HalfPrec_;
  LongNumber LowVal_ = Value_ - HalfPrec_;

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + LowVal_.ToString() + ChrString(")");
  LowFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + HighVal_.ToString() + ChrString(")");
  HighFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + Value_.ToString() + ChrString(")");
  MidTermRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  MidTermRet_ *= 2L;

  HighFncRet_ -= MidTermRet_;
  HighFncRet_ += LowFncRet_;
  HalfPrec_ *= HalfPrec_;

  LongNumber Slope_ = HighFncRet_;
  Slope_ /= HalfPrec_;
  PopSubExprStack();

  long Exp_ = 0;
  LongNumber xTol_ = Slope_;
  AbsoluteValue(xTol_);

  while (Prec_ < 1L)
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < 1L)
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_DERIV;

  Slope_.Round(Exp_);
  return Slope_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::FindIntersections(int NumParams_, FncParameter** ParamArray_)
{
  int x;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;
  ChrString FirstRt_;
  ChrString RetStr_;
  ChrString FncVal_;
  char CurrentSuffix_ = _PlotFileSuffix;
  LongNumber Yresult_;
  LongNumber Xresult_;
  LongNumber Upper_;
  LongNumber Lower_;
  LongNumber MidPt_;
  LongNumber RadVal_;
  FncParameter** Params_;

  Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_;
  ChrString NewExpr_;

  if (NumParams_ == 5 || NumParams_ == 3)
  {
    NewExpr_ = "(";
    NewExpr_ += ParamArray_[0]->_Param->GetExpression();
    NewExpr_ += ") - (";
    NewExpr_ += ParamArray_[1]->_Param->GetExpression();
    NewExpr_ += ")";
    VarName_ = ParamArray_[2]->_Param->GetExpression();
    CalculatorBase::SetGraphType(Mcalc_GraphType::INTERSECT2);
    _IntExpr = ParamArray_[0]->_Param->GetExpression();
    _IntExpr += " , ";
    _IntExpr += ParamArray_[1]->_Param->GetExpression();
  }
  else if (NumParams_ == 4 || NumParams_ == 2)
  {
    NewExpr_ = ParamArray_[0]->_Param->GetExpression();
    VarName_ = ParamArray_[1]->_Param->GetExpression();
    _IntExpr = NewExpr_;
  }

  Params_[0] = new FncParameter();
  Params_[0]->_Param = new SubExpression();
  Params_[0]->_Param->SetExpression(NewExpr_.c_str());
  Params_[1] = (NumParams_ == 4 || NumParams_ == 2) ?
                   new FncParameter(*ParamArray_[1]):
                   new FncParameter(*ParamArray_[2]);
  Params_[2] = new FncParameter();
  Params_[2]->_Param = new SubExpression();
  Params_[3] = Params_[4] = NULL;

  if (NumParams_ == 2 || NumParams_ == 3)
    Params_[2]->_Param->SetAnswer("0");
  else
  {
    if (NumParams_ == 4)
    {
      Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
      Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;

      if (MidPt_.IsNan() || MidPt_.IsInfinite())
        Params_[2]->_Param->SetAnswer("0");
      else
        Params_[2]->_Param->SetAnswer(MidPt_.ToString().c_str());
    }
    else if (NumParams_ == 5)
    {
      Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;

      if (MidPt_.IsNan() || MidPt_.IsInfinite())
        Params_[2]->_Param->SetAnswer("0");
      else
        Params_[2]->_Param->SetAnswer(MidPt_.ToString().c_str());
    }
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);

  for (_IntsFound = x = 0; x < 3; x++)
  {
    RetStr_ = FindRoot(3, Params_);

    if (!HasErrors())
    {
      FncVal_ = ChrString("(") + RetStr_ + ChrString(")");
      Xresult_ = RetStr_;
      Yresult_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (!_IntsFound ||
          (_IntsFound == 1 && _IntCoords[0]->X != Xresult_) ||
          (_IntsFound == 2 && _IntCoords[1]->X != Xresult_ &&
                              _IntCoords[0]->X != Xresult_))
      {
        if (!_IntsFound)
          FirstRt_ = Xresult_.ToString();

        _IntCoords[_IntsFound]->X = Xresult_;
        _IntCoords[_IntsFound]->Y = Yresult_;
        ++_IntsFound;
      }
    }

    if (x == 0)
    {
      if (NumParams_ == 4 || NumParams_ == 5)
      {
        if (NumParams_ == 4)
          Params_[2]->_Param->SetAnswer(ParamArray_[2]->_Param->GetAnswer());
        else
          Params_[2]->_Param->SetAnswer(ParamArray_[3]->_Param->GetAnswer());
      }
      else
        Params_[2]->_Param->SetAnswer("-5");
    }
    else if (x == 1)
    {
      if (NumParams_ == 4 || NumParams_ == 5)
      {
        if (NumParams_ == 4)
          Params_[2]->_Param->SetAnswer(ParamArray_[3]->_Param->GetAnswer());
        else
          Params_[2]->_Param->SetAnswer(ParamArray_[4]->_Param->GetAnswer());
      }
      else
        Params_[2]->_Param->SetAnswer("5");
    }

    if (HasErrors())
      ClearError();
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();

  if (PlotGraph_)
  {
    int RangeSpecified_ = NumParams_ == 4 || NumParams_ == 5;
    if (_IntsFound)
      RetStr_ = FirstRt_;
    else
      RetStr_ = "0";

    if (RangeSpecified_)
    {
      if (NumParams_ == 5)
      {
        Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
        Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      }
      else if (NumParams_ == 4)
      {
        Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
        Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      }

      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;
      RadVal_ = Upper_ - MidPt_;
      AbsoluteValue(RadVal_);
    }

    if (NumParams_ == 5 || NumParams_ == 3)
    {
      // Plotting First Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[2]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(),
                     RadVal_.ToString(), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);

      // Plotting Second Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[1]);
      Params_[1] = new FncParameter(*ParamArray_[2]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(),
                     RadVal_.ToString(), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);
    }
    else if (NumParams_ == 4 || NumParams_ == 2)
    {
      // Plotting First Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(),
                     RadVal_.ToString(), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);
    }

    SaveCurrentGraphName(CurrentSuffix_);
  }

  for (x = 0; x < 5; x++)
    delete Params_[x];
  MemMatrix::Matrix().Deallocate(Params_);

  char Buffer_[64];
  char Suf_ = 'z';
  sprintf(Buffer_, "mcalc_graph%d%c.out", SessionNum(), Suf_);
  FILE* Fout_ = fopen(Buffer_, "w");
  RetStr_ = "\n";

  if (Fout_)
  {
    if (_IntsFound)
      for (x = 0; x < _IntsFound; x++)
      {
        sprintf(Buffer_, DEFAULT_LONGNUM_FORMAT,
                _IntCoords[x]->X.ToString().c_str(), _IntCoords[x]->Y.ToString().c_str());
        fprintf(Fout_, "%s", Buffer_);
        RetStr_ += Buffer_;
      }

    fclose(Fout_);
  }

  SetHasAnswer(RetStr_.strlen());

  if (RetStr_.strlen())
  {
    RetStr_ = ChrString("\"") + RetStr_ + ChrString("\"");
    SetLiteralStr(RetStr_.c_str());
  }

  return RetStr_;
}

/*****************************************************************************/
ChrString LongNumCalculator::PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                          const ChrString& MidPtStr_, ChrString Radius_, ChrString* Solution_)
{
  // Using longnums in PlotFunction Method takes too long, using long double precision instead
  return _CalcPtrArray[CalculatorBase::LONGDOUBLE]->PlotFunction(NumParams_, ParamArray_,
                                                                 MidPtStr_, Radius_, Solution_);
}

/*****************************************************************************/
ChrString LongNumCalculator::FindIntegral(int NumParams_, FncParameter** ParamArray_)
{
  // Using longnums in FindIntegral Method takes too long, using long double precision instead
  return _CalcPtrArray[CalculatorBase::LONGDOUBLE]->FindIntegral(NumParams_, ParamArray_);
}

/*****************************************************************************/
ChrString LongNumCalculator::FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  Double vUPPER_LIMIT(UPPER_LIMIT);
  LongNumber xUPPER_LIMIT(vUPPER_LIMIT);
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongNumber Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Double Incr_ = DEFAULT_PRECVAL_NEWTCRIT;

  CreateNewSubExpr();
  LongNumber X1 = Guess_;
  X1 -= 10;
  ChrString FncVal_;
  LongNumber Y1;
  LongNumber Ypr;
  LongNumber Ypr2;
  LongNumber CriteriaVal_;
  LongNumber Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  FncVal_ = ChrString("(") + X1.ToString() + ChrString(")");
  Y1 = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  FncVal_ = X1.ToString();
  Params_[2]->_Param->SetAnswer(FncVal_.c_str());
  Ypr = FindDerivative(NumParams_, Params_);
  Ypr2 = FindDerivative2(NumParams_, Params_);

  int i;
  int max = int(20 / Incr_.FloatValue());
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 0; i < max; i++)
  {
    CriteriaVal_ = ((Y1 * Ypr2) / (Ypr * Ypr));
    AbsoluteValue(CriteriaVal_);

    if (CriteriaVal_ < 1L)
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (LongNumber(CriteriaVal_) >= xUPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    X1 += LongNumber(Incr_);

    FncVal_ = ChrString("(") + X1.ToString() + ChrString(")");
    Y1 = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    FncVal_ = X1.ToString();
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Ypr = FindDerivative(NumParams_, Params_);
    Ypr2 = FindDerivative2(NumParams_, Params_);
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  LongNumber xTol_ = X1;
  AbsoluteValue(xTol_);

  while (Tol_ < 1L)
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < 1L)
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  if (HasErrors())
    ClearError();

  LongNumber Answer_ = X1;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  LongDouble vUPPER_LIMIT(UPPER_LIMIT);
  LongNumber xUPPER_LIMIT(vUPPER_LIMIT);
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongNumber Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());

  CreateNewSubExpr();
  LongNumber X1 = Guess_;
  ChrString FncVal_;
  LongNumber Y1;
  LongNumber X2;
  LongNumber Y2;
  LongNumber Ypr;

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  FncVal_ = ChrString("(") + X1.ToString() + ChrString(")");
  Y1 = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  FncVal_ = X1.ToString();
  Params_[2]->_Param->SetAnswer(FncVal_.c_str());
  Ypr = FindDerivative(NumParams_, Params_);

  LongNumber Diff_;
  LongNumber Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);

  int i;
  int max = 200;
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 0; i < max; i++)
  {
    X2 = X1 - Y1 / Ypr;
    FncVal_ = ChrString("(") + X2.ToString() + ChrString(")");
    Y2 = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    Diff_ = X2 - X1;
    AbsoluteValue(Diff_);

    if (Diff_ < Tol_ && Y2 < Tol_)
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (Diff_ >= xUPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    FncVal_ = X2.ToString();
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Ypr = FindDerivative(NumParams_, Params_);

    X1 = X2;
    Y1 = Y2;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  LongNumber xTol_ = X2;
  AbsoluteValue(xTol_);

  while (Tol_ < 1L)
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < 1L)
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  X2.Round(Exp_);

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  if (HasErrors())
    ClearError();

  return X2.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  // Using longnums in Mullers Method takes too long, using long double precision instead
  return _CalcPtrArray[CalculatorBase::LONGDOUBLE]->MullersMethod(NumParams_, ParamArray_, Converge_);
}

/*****************************************************************************/
ChrString LongNumCalculator::FindRoot(int NumParams_, FncParameter** ParamArray_)
{
  int x;
  int Converge_;
  int CriteriaFound_;
  ChrString RetStr_;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;
  bool RangeSpecified_ = false;
  FncParameter** Params_;

  if (NumParams_ > 3)
  {
    RangeSpecified_ = true;
    RetStr_ = MullersMethod(NumParams_, ParamArray_, Converge_);

    if (!Converge_)
    {
      Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      Params_[2] = new FncParameter(*ParamArray_[2]);
      Params_[3] = new FncParameter(*ParamArray_[3]);
      Params_[4] = new FncParameter(*ParamArray_[4]);
      ChrString Answer_ = FindNewtonsCriteria(NumParams_, Params_, CriteriaFound_);

      if (CriteriaFound_)
      {
        Params_[2]->_Param->SetAnswer(Answer_.c_str());
        RetStr_ = NewtonsMethod(NumParams_, Params_, Converge_);

        if (Converge_)
        {
          for (x = 0; x < 5; x++)
            delete Params_[x];

          MemMatrix::Matrix().Deallocate(Params_);
          goto RootSolveDone;
        }
        else
        {
          RetStr_ = MullersMethod(NumParams_, Params_, Converge_);

          if (Converge_)
          {
            for (x = 0; x < 5; x++)
              delete Params_[x];

            MemMatrix::Matrix().Deallocate(Params_);
            goto RootSolveDone;
          }
        }
      }
    }
  }
  else
  {
    RetStr_ = NewtonsMethod(NumParams_, ParamArray_, Converge_);

    if (!Converge_)
    {
      Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      Params_[2] = new FncParameter(*ParamArray_[2]);
      Params_[3] = Params_[4] = NULL;
      ChrString Answer_ = FindNewtonsCriteria(NumParams_, Params_, CriteriaFound_);

      if (CriteriaFound_)
      {
        Params_[2]->_Param->SetAnswer(Answer_.c_str());
        RetStr_ = NewtonsMethod(NumParams_, Params_, Converge_);

        if (Converge_)
        {
          for (x = 0; x < 3; x++)
            delete Params_[x];

          MemMatrix::Matrix().Deallocate(Params_);
          goto RootSolveDone;
        }
      }

      Params_[3] = new FncParameter();
      Params_[4] = new FncParameter();
      Params_[3]->_Param = new SubExpression();
      Params_[4]->_Param = new SubExpression();
      Boolean ZeroFound_ = FALSE;
      LongNumber Lower_ = ChrString(Params_[2]->_Param->GetAnswer());
      LongNumber Upper_ = ChrString(Params_[2]->_Param->GetAnswer());
      NumParams_ += 2;
      Lower_ -= 20L;
      Upper_ += 20L;
      Answer_ = Lower_.ToString();
      Params_[3]->_Param->SetAnswer(Answer_.c_str());
      Params_[3]->_ShouldEval = FALSE;
      Answer_ = Upper_.ToString();
      Params_[4]->_Param->SetAnswer(Answer_.c_str());
      Params_[4]->_ShouldEval = FALSE;
      RetStr_ = MullersMethod(NumParams_, Params_, Converge_);

      if (!Converge_)
      {
        RetStr_ = FindApproxZero(NumParams_, Params_, ZeroFound_);
        if (!ZeroFound_)
          RetStr_ = MullersMethod(NumParams_, Params_, Converge_);
        else
          Converge_ = TRUE;
      }

      if (!Converge_)
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED,
                        ERRMSG_MULLERS_METHOD_FAILED, ParamArray_[0]->_Param->GetExpression());
        CloseOutputFile();
      }

      for (x = 0; x < 5; x++)
        delete Params_[x];

      MemMatrix::Matrix().Deallocate(Params_);
    }
  }

RootSolveDone:
  if (!Converge_)
  {
    if (RangeSpecified_)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED,
                      ERRMSG_MULLERS_METHOD_FAILED, ParamArray_[0]->_Param->GetExpression());
      CloseOutputFile();
    }
  }
  else if (PlotGraph_)
  {
    if (RangeSpecified_)
    {
      LongNumber Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      LongNumber Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      LongNumber MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;
      LongNumber RadVal_ = Upper_ - MidPt_;
      AbsoluteValue(RadVal_);
      PlotFunction(2, ParamArray_, MidPt_.ToString(), RadVal_.ToString(), &RetStr_);
    }
    else
      PlotFunction(2, ParamArray_, RetStr_);
  }

  return RetStr_;
}

/*****************************************************************************/
ChrString LongNumCalculator::FindMinimum(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !LongNumber::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 6 && !LongNumber::IsValue(ParamArray_[5]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMIN");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongNumber Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongNumber Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  ChrString InclOpt_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetExpression():"";
  const char* ParamStr_ = (NumParams_ >= 6) ? ParamArray_[5]->_Param->GetAnswer():Buffer_;
  LongNumber Tol_ = ChrString(ParamStr_);
  LongNumber Incr_ = ChrString(DEFAULT_PRECSTR_FTEST);
  LongNumber xTol_ = Upper_ - Lower_;

  if (xTol_.GetSign() == NumberSign::MINUS)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMIN");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }
  else
  {
    LongNumber Ratio_ = xTol_ / LongNumber(10UL, NumberSign::PLUS);
    xTol_ = Tol_;

    if (Ratio_ < 1L)
      Incr_ *= Ratio_;
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  LongNumber Deriv1_;
  LongNumber Deriv2_;
  LongNumber Lowest_;
  LongNumber Lowestx_;
  LongNumber Testx_;
  LongNumber Low_;
  LongNumber High_;
  LongNumber MidPt_;
  LongNumber RadVal_;

  int ZeroCrossing_;
  int LowestInit_ = 0;
  int PrevInit_ = 0;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;

  if (PlotGraph_)
  {
    MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
  }

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  for (;Lower_ < Upper_; Lower_ += Incr_)
  {
    FncVal_ = Lower_.ToString();
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Deriv2_ = FindDerivative(3, Params_);

    if (!PrevInit_)
    {
      if (!Deriv2_.AtZero())
      {
        Deriv1_ = Deriv2_;
        PrevInit_ = TRUE;
      }
    }
    else
    {
      ZeroCrossing_ = Deriv1_.GetSign() == NumberSign::MINUS &&
                      Deriv2_.GetSign() == NumberSign::PLUS;

      if (ZeroCrossing_)
        for (Testx_ = Lower_ - Incr_; Testx_ < Lower_; Testx_ += xTol_)
        {
          FncVal_ = ChrString("(") + Testx_.ToString() + ChrString(")");
          Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

          if (!LowestInit_)
          {
            Lowest_ = Low_;
            Lowestx_ = Testx_;
            LowestInit_ = TRUE;
          }
          else if (Low_ < Lowest_)
          {
            Lowest_ = Low_;
            Lowestx_ = Testx_;
          }
        }

      if (!Deriv2_.AtZero())
        Deriv1_ = Deriv2_;
    }
  }

  if (InclOpt_.strlen())
    InclOpt_.RemoveChar(" \t\n");

  int ExclEnds_ = (NumParams_ >= 5) ? icasestrcomp(InclOpt_.c_str(), "EXCLUDEENDS"):0;

  if (!ExclEnds_)
  {
    FncVal_ = ChrString("(") + Lower_.ToString() + ChrString(")");
    Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    FncVal_ = ChrString("(") + Upper_.ToString() + ChrString(")");
    High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (High_ < Low_)
    {
      if (High_ < Lowest_)
        Lowestx_ = Upper_;
    }
    else
    {
      if (Low_ < Lowest_)
        Lowestx_ = Lower_;
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  xTol_ = Lowestx_;
  AbsoluteValue(xTol_);

  while (Tol_ < 1L)
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < 1L)
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);
  Lowestx_.Round(Exp_);

  if (PlotGraph_)
  {
    _ExtremaEnd = -1;
    ChrString SolString_ = Lowestx_.ToString();
    PlotFunction(2, ParamArray_, MidPt_.ToString(), RadVal_.ToString(), &SolString_);
    return SolString_;
  }

  return Lowestx_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::FindMaximum(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !LongNumber::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 6 && !LongNumber::IsValue(ParamArray_[5]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMAX");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongNumber Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongNumber Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  ChrString InclOpt_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetExpression():"";
  const char* ParamStr_ = (NumParams_ >= 6) ? ParamArray_[5]->_Param->GetAnswer():Buffer_;
  LongNumber Tol_ = ChrString(ParamStr_);
  LongNumber Incr_ = ChrString(DEFAULT_PRECSTR_FTEST);
  LongNumber xTol_ = Upper_ - Lower_;

  if (xTol_.GetSign() == NumberSign::MINUS)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMAX");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }
  else
  {
    LongNumber Ratio_ = xTol_ / LongNumber(10UL, NumberSign::PLUS);
    xTol_ = Tol_;

    if (Ratio_ < 1L)
      Incr_ *= Ratio_;
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  LongNumber Deriv1_;
  LongNumber Deriv2_;
  LongNumber Highest_;
  LongNumber Highestx_;
  LongNumber Testx_;
  LongNumber High_;
  LongNumber Low_;
  LongNumber MidPt_;
  LongNumber RadVal_;

  int ZeroCrossing_;
  int HighestInit_ = 0;
  int PrevInit_ = 0;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;

  if (PlotGraph_)
  {
    MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
  }

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  for (;Lower_ < Upper_; Lower_ += Incr_)
  {
    FncVal_ = Lower_.ToString();
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Deriv2_ = FindDerivative(3, Params_);

    if (!PrevInit_)
    {
      if (!Deriv2_.AtZero())
      {
        Deriv1_ = Deriv2_;
        PrevInit_ = TRUE;
      }
    }
    else
    {
      ZeroCrossing_ = Deriv1_.GetSign() == NumberSign::PLUS &&
                      Deriv2_.GetSign() == NumberSign::MINUS;

      if (ZeroCrossing_)
        for (Testx_ = Lower_ - Incr_; Testx_ < Lower_; Testx_ += xTol_)
        {
          FncVal_ = ChrString("(") + Testx_.ToString() + ChrString(")");
          High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

          if (!HighestInit_)
          {
            Highest_ = High_;
            Highestx_ = Testx_;
            HighestInit_ = TRUE;
          }
          else if (High_ > Highest_)
          {
            Highest_ = High_;
            Highestx_ = Testx_;
          }
        }

      if (!Deriv2_.AtZero())
        Deriv1_ = Deriv2_;
    }
  }

  if (InclOpt_.strlen())
    InclOpt_.RemoveChar(" \t\n");

  int ExclEnds_ = (NumParams_ >= 5) ? icasestrcomp(InclOpt_.c_str(), "EXCLUDEENDS"):0;

  if (!ExclEnds_)
  {
    FncVal_ = ChrString("(") + Lower_.ToString() + ChrString(")");
    Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    FncVal_ = ChrString("(") + Upper_.ToString() + ChrString(")");
    High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (High_ < Low_)
    {
      if (Low_ > Highest_)
        Highestx_ = Lower_;
    }
    else
    {
      if (High_ > Highest_)
        Highestx_ = Upper_;
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  xTol_ = Highestx_;
  AbsoluteValue(xTol_);

  while (Tol_ < 1L)
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < 1L)
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);
  Highestx_.Round(Exp_);

  if (PlotGraph_)
  {
    _ExtremaEnd = 1;
    ChrString SolString_ = Highestx_.ToString();
    PlotFunction(2, ParamArray_, MidPt_.ToString(), RadVal_.ToString(), &SolString_);
    return SolString_;
  }

  return Highestx_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_)
{
  if (!LongNumber::IsValue(ParamArray_[0]->_Param->GetAnswer()) ||
      !LongNumber::IsValue(ParamArray_[1]->_Param->GetAnswer()))
  {
    if (Retarg_)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "RECTOPOLR");
    else
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "RECTOPOLA");

    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  LongNumber Xpos_ = ChrString(ParamArray_[0]->_Param->GetAnswer());
  LongNumber Ypos_ = ChrString(ParamArray_[1]->_Param->GetAnswer());
  LongNumber Xsqr_;
  LongNumber Ysqr_;
  LongNumber Sin_;
  LongNumber Angle_;
  LongNumber Radius_;
  ChrString Error_ = "0";

  if (Ypos_.fabs() < _Epsilon)
    Ypos_ = 0;

  if (Xpos_.fabs() < _Epsilon)
    Xpos_ = 0;

  if (!Xpos_ && !Ypos_)
    return Error_;

  Ysqr_ = Ypos_;
  Ysqr_ *= Ypos_;
  Xsqr_ = Xpos_;
  Xsqr_ *= Xpos_;
  Radius_ = Xsqr_;
  Radius_ += Ysqr_;
  Radius_ = Radius_.sqrt();

  // Return radius
  if (Retarg_)
    return Radius_.ToString();

  Sin_ = Ypos_ / Radius_;
  Angle_ = Sin_.arcsin();
  if (Xpos_.GetSign() == NumberSign::MINUS)
    if (Ypos_.GetSign() == NumberSign::MINUS)
      Angle_ = -LongNumber::vPI() - Angle_;
    else
      Angle_ = LongNumber::vPI() - Angle_;

  if (GetAngleUnit() == DEGREE)
  {
    Angle_ /= LongNumber::vPI();
    Angle_ *= 180L;
  }

  return Angle_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_)
{
  if (!LongNumber::IsValue(ParamArray_[0]->_Param->GetAnswer()) ||
      !LongNumber::IsValue(ParamArray_[1]->_Param->GetAnswer()))
  {
    if (Retarg_)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "POLTORECX");
    else
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "POLTORECY");

    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  LongNumber Radius_ = ChrString(ParamArray_[0]->_Param->GetAnswer());
  LongNumber Angle_ = ChrString(ParamArray_[1]->_Param->GetAnswer());
  LongNumber Mult_;
  LongNumber Frac_;
  LongNumber Int_;
  LongNumber Odd_;
  LongNumber Xpos_;
  LongNumber Ypos_;
  LongNumber PiDiv2_ = LongNumber::vPI() / 2L;
  LongNumber AbsAngle_;
  ChrString Zero_ = "0";

  if (Radius_.AtZero() || Radius_.GetSign() == NumberSign::MINUS)
    return Zero_;

  if (GetAngleUnit() == DEGREE)
  {
    Angle_ /= 180L;
    Angle_ *= LongNumber::vPI();
  }

  AbsAngle_ = Angle_.fabs();
  AbsAngle_ -= LongNumber::vPI();

  if (AbsAngle_.fabs() > _Epsilon)
  {
    Mult_ = Angle_ / LongNumber::vPI();
    Frac_ = Mult_.modf(Int_);
    Odd_ = Int_ % LongNumber(2L);

    if (Frac_ > _Epsilon)
    {
      Angle_ =  Frac_ * LongNumber::vPI();

      if (Odd_)
        if (Angle_ > 0L)
          Angle_ -= LongNumber::vPI();
        else
          Angle_ += LongNumber::vPI();
    }
    else
    {
      if (Odd_)
        Angle_ = LongNumber::vPI();
      else
        Angle_ = 0;
    }
  }

  if (Angle_.GetSign() == NumberSign::PLUS)
  {
    if (Angle_ > PiDiv2_)
    {
      Angle_ = LongNumber::vPI() - Angle_;
      if (Angle_.fabs() < _Epsilon)
        Angle_ = 0;
      Xpos_ = -(Radius_ * Angle_.cos());
    }
    else
    {
      if (Angle_.fabs() < _Epsilon)
        Angle_ = 0;
      Xpos_ = Radius_ * Angle_.cos();
    }

    Ypos_ = Radius_ * Angle_.sin();
  }
  else
  {
    if (Angle_ < -PiDiv2_)
    {
      Angle_ = -LongNumber::vPI() - Angle_;
      if (Angle_.fabs() < _Epsilon)
        Angle_ = 0;
      Angle_.FlipSign();
      Xpos_ = -(Radius_ * Angle_.cos());
    }
    else
    {
      if (Angle_.fabs() < _Epsilon)
        Angle_ = 0;
      Angle_.FlipSign();
      Xpos_ = Radius_ * Angle_.cos();
    }

    Ypos_ = Radius_ * Angle_.sin();
    Ypos_.FlipSign();
  }

  if (Retarg_)
    return Xpos_.ToString();

  return Ypos_.ToString();
}

/*****************************************************************************/
ChrString LongNumCalculator::SolveMatrix(int NumParams_, FncParameter** ParamArray_)
{
  // Using longnums in SolveMatrix Method takes too long, using long double precision instead
  return _CalcPtrArray[CalculatorBase::LONGDOUBLE]->SolveMatrix(NumParams_, ParamArray_);
}

/*****************************************************************************/
ChrString LongNumCalculator::EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_)
{
  int optype, x;
  int CallFnc_, DiscFnc_, ConvFnc_, OptionFnc_;
  ChrString buffer;
  ChrString Answer_;

  optype = icasestrcomp(FncName_, "MIN")         ? Mcalc_OperatorType::Minimum:
           icasestrcomp(FncName_, "MAX")         ? Mcalc_OperatorType::Maximum:
           icasestrcomp(FncName_, "DERIV")       ? Mcalc_OperatorType::Derivative:
           icasestrcomp(FncName_, "DERIV2")      ? Mcalc_OperatorType::Derivative2:
           icasestrcomp(FncName_, "INTEG")       ? Mcalc_OperatorType::Integral:
           icasestrcomp(FncName_, "SOLVE")       ? Mcalc_OperatorType::RootSolve:
           icasestrcomp(FncName_, "INTERSECT")   ? Mcalc_OperatorType::Intersect:
           icasestrcomp(FncName_, "PERMREP")     ? Mcalc_OperatorType::PermuteWithRep:
           icasestrcomp(FncName_, "COMBREP")     ? Mcalc_OperatorType::CombineWithRep:
           icasestrcomp(FncName_, "COMB")        ? Mcalc_OperatorType::Combination:
           icasestrcomp(FncName_, "PERM")        ? Mcalc_OperatorType::Permutation:
           icasestrcomp(FncName_, "FMIN")        ? Mcalc_OperatorType::FunctionMinimum:
           icasestrcomp(FncName_, "FMAX")        ? Mcalc_OperatorType::FunctionMaximum:
           icasestrcomp(FncName_, "ROUND")       ? Mcalc_OperatorType::RoundEx:
           icasestrcomp(FncName_, "SET")         ? Mcalc_OperatorType::MakeSet:
           icasestrcomp(FncName_, "SETUNI")      ? Mcalc_OperatorType::SetUnion:
           icasestrcomp(FncName_, "SETINT")      ? Mcalc_OperatorType::SetIntersection:
           icasestrcomp(FncName_, "SETDIF")      ? Mcalc_OperatorType::SetDifference:
           icasestrcomp(FncName_, "SETXDIF")     ? Mcalc_OperatorType::SetExclusion:
           icasestrcomp(FncName_, "ITEMPERM")    ? Mcalc_OperatorType::ItemPermutation:
           icasestrcomp(FncName_, "ITEMCOMB")    ? Mcalc_OperatorType::ItemCombination:
           icasestrcomp(FncName_, "ITEMPERMREP") ? Mcalc_OperatorType::ItemPermuteWithRep:
           icasestrcomp(FncName_, "ITEMCOMBREP") ? Mcalc_OperatorType::ItemCombineWithRep:
           icasestrcomp(FncName_, "FRAC")        ? Mcalc_OperatorType::FractionArithmetic:
           icasestrcomp(FncName_, "RECTOPOLR")   ? Mcalc_OperatorType::RectToPolarRadius:
           icasestrcomp(FncName_, "RECTOPOLA")   ? Mcalc_OperatorType::RectToPolarAngle:
           icasestrcomp(FncName_, "POLTORECX")   ? Mcalc_OperatorType::PolarToRectX:
           icasestrcomp(FncName_, "POLTORECY")   ? Mcalc_OperatorType::PolarToRectY:
           icasestrcomp(FncName_, "FROMHEX")     ? Mcalc_OperatorType::FromHex:
           icasestrcomp(FncName_, "FROMOCT")     ? Mcalc_OperatorType::FromOct:
           icasestrcomp(FncName_, "FROMBIN")     ? Mcalc_OperatorType::FromBin:
           icasestrcomp(FncName_, "FROMCMP2BIN") ? Mcalc_OperatorType::FromCmp2Bin:
           icasestrcomp(FncName_, "TOHEX")       ? Mcalc_OperatorType::ToHex:
           icasestrcomp(FncName_, "TOOCT")       ? Mcalc_OperatorType::ToOct:
           icasestrcomp(FncName_, "TOBIN")       ? Mcalc_OperatorType::ToBin:
           icasestrcomp(FncName_, "TOCMP2BIN")   ? Mcalc_OperatorType::ToCmp2Bin:
           icasestrcomp(FncName_, "AVGMEAN")     ? Mcalc_OperatorType::AvgMean:
           icasestrcomp(FncName_, "AVGMED")      ? Mcalc_OperatorType::AvgMedian:
           icasestrcomp(FncName_, "AVGMODE")     ? Mcalc_OperatorType::AvgMode:
           icasestrcomp(FncName_, "STDDEV")      ? Mcalc_OperatorType::StdDev:
           icasestrcomp(FncName_, "SUM")         ? Mcalc_OperatorType::Sum:
           icasestrcomp(FncName_, "PROD")        ? Mcalc_OperatorType::Product:
           icasestrcomp(FncName_, "MATSOLVE")    ? Mcalc_OperatorType::MatrixSolve:
           icasestrcomp(FncName_, "GRAPH")       ? Mcalc_OperatorType::PlotGraph:0;
  DiscFnc_ = optype == Mcalc_OperatorType::ItemPermutation ||
             optype == Mcalc_OperatorType::ItemCombination ||
             optype == Mcalc_OperatorType::ItemCombineWithRep ||
             optype == Mcalc_OperatorType::ItemPermuteWithRep ||
             optype == Mcalc_OperatorType::FractionArithmetic ||
             optype == Mcalc_OperatorType::RectToPolarRadius ||
             optype == Mcalc_OperatorType::RectToPolarAngle ||
             optype == Mcalc_OperatorType::PolarToRectX ||
             optype == Mcalc_OperatorType::PolarToRectY ||
             optype == Mcalc_OperatorType::FromHex ||
             optype == Mcalc_OperatorType::FromOct ||
             optype == Mcalc_OperatorType::FromBin ||
             optype == Mcalc_OperatorType::FromCmp2Bin ||
             optype == Mcalc_OperatorType::ToHex ||
             optype == Mcalc_OperatorType::ToOct ||
             optype == Mcalc_OperatorType::ToBin ||
             optype == Mcalc_OperatorType::ToCmp2Bin ||
             optype == Mcalc_OperatorType::MatrixSolve ||
             optype == Mcalc_OperatorType::Intersect ||
             optype == Mcalc_OperatorType::MakeSet ||
             optype == Mcalc_OperatorType::SetUnion ||
             optype == Mcalc_OperatorType::SetIntersection ||
             optype == Mcalc_OperatorType::SetDifference ||
             optype == Mcalc_OperatorType::SetExclusion;
  CallFnc_ = optype == Mcalc_OperatorType::Derivative ||
             optype == Mcalc_OperatorType::Derivative2 ||
             optype == Mcalc_OperatorType::Integral ||
             optype == Mcalc_OperatorType::RootSolve ||
             optype == Mcalc_OperatorType::FunctionMinimum ||
             optype == Mcalc_OperatorType::FunctionMaximum ||
             optype == Mcalc_OperatorType::PlotGraph ||
             optype == Mcalc_OperatorType::AvgMean ||
             optype == Mcalc_OperatorType::AvgMedian ||
             optype == Mcalc_OperatorType::AvgMode ||
             optype == Mcalc_OperatorType::StdDev ||
             optype == Mcalc_OperatorType::Sum ||
             optype == Mcalc_OperatorType::Product ||
             (NumParams_ != 2 && (optype == Mcalc_OperatorType::Minimum   ||
                                  optype == Mcalc_OperatorType::Maximum)) ||
             DiscFnc_;
  ConvFnc_ = optype == Mcalc_OperatorType::ToHex ||
             optype == Mcalc_OperatorType::ToOct ||
             optype == Mcalc_OperatorType::ToBin ||
             optype == Mcalc_OperatorType::ToCmp2Bin ||
             optype == Mcalc_OperatorType::FromHex ||
             optype == Mcalc_OperatorType::FromOct ||
             optype == Mcalc_OperatorType::FromBin ||
             optype == Mcalc_OperatorType::FromCmp2Bin;
  OptionFnc_ = optype == Mcalc_OperatorType::Sum ||
               optype == Mcalc_OperatorType::Product ||
               optype == Mcalc_OperatorType::AvgMean ||
               optype == Mcalc_OperatorType::AvgMedian ||
               optype == Mcalc_OperatorType::AvgMode ||
               optype == Mcalc_OperatorType::StdDev ||
               (NumParams_ != 2 && (optype == Mcalc_OperatorType::Minimum   ||
                                    optype == Mcalc_OperatorType::Maximum)) ||
               optype == Mcalc_OperatorType::ItemPermutation ||
               optype == Mcalc_OperatorType::ItemCombination ||
               optype == Mcalc_OperatorType::ItemCombineWithRep ||
               optype == Mcalc_OperatorType::ItemPermuteWithRep ||
               optype == Mcalc_OperatorType::FractionArithmetic ||
               optype == Mcalc_OperatorType::MakeSet;

  int ErrorSet_;
  ErrorSet_ = (!optype || NumParams_ < 2) ? (!((ConvFnc_ || OptionFnc_) && NumParams_ == 1) && (NumParams_ < 2)):
              (optype == Mcalc_OperatorType::Derivative) ? (NumParams_ < 3):
              (optype == Mcalc_OperatorType::Integral) ? (NumParams_ < 4):
              (optype == Mcalc_OperatorType::RootSolve) ? (NumParams_ < 3):
              (optype == Mcalc_OperatorType::Intersect) ? (NumParams_ < 2):
              (optype == Mcalc_OperatorType::FunctionMinimum) ? (NumParams_ < 4):
              (optype == Mcalc_OperatorType::FunctionMaximum) ? (NumParams_ < 4):0;

  if (ErrorSet_)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_NUMARGS,
                        ERRMSG_INVALID_NUMARGS, FncName_);
    CloseOutputFile();
    return buffer;
  }

  LongDouble TempResult_;
  LongNumber resultvalue;
  LongNumber** params = NULL;
  long Rexp_;

  if (!CallFnc_)
  {
    params = (LongNumber**)allocmem(NumParams_, sizeof(LongNumber*), params);

    for (x = 0; x < NumParams_; x++)
    {
      buffer = ParamArray_[x]->_Param->GetAnswer();
      if (LongNumber::IsValue(buffer))
        params[x] = new LongNumber(buffer);
      else
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                            ERRMSG_INVALID_TYPEARGS, FncName_);
        CloseOutputFile();
        return buffer;
      }
    }
  }

  // Initialize non-fraction mode for calculations
  ResetAnswerMode();
  SetAnswerMode(FLOATINGPOINT_MODE);

  /* finding the result value and remainder value */
  switch (optype)
  {
    case Mcalc_OperatorType::PlotGraph:
      resultvalue = PlotFunction(NumParams_, ParamArray_, "0");
      break;

    case Mcalc_OperatorType::Permutation:
      resultvalue = Discrete<LongNumber>::Permutation(*params[0], *params[1]);
      break;

    case Mcalc_OperatorType::Combination:
      resultvalue = Discrete<LongNumber>::Combination(*params[0], *params[1]);
      break;

    case Mcalc_OperatorType::PermuteWithRep:
      resultvalue = Discrete<LongNumber>::PermutationWithRep(*params[0], *params[1]);
      break;

    case Mcalc_OperatorType::CombineWithRep:
      resultvalue = Discrete<LongNumber>::CombinationWithRep(*params[0], *params[1]);
      break;

    case Mcalc_OperatorType::Minimum:
      if (NumParams_ == 2)
        resultvalue = (*params[0] < *params[1]) ? *params[0]:*params[1];
      else
      {
        TempResult_ = MinimumValue(NumParams_, ParamArray_);
        resultvalue = TempResult_;
      }
      break;

    case Mcalc_OperatorType::Maximum:
      if (NumParams_ == 2)
        resultvalue = (*params[0] > *params[1]) ? *params[0]:*params[1];
      else
      {
        TempResult_ = MaximumValue(NumParams_, ParamArray_);
        resultvalue = TempResult_;
      }
      break;

    case Mcalc_OperatorType::RoundEx:
      resultvalue = *params[0];
      Rexp_ = atol(ParamArray_[1]->_Param->GetAnswer());
      resultvalue.Round(Rexp_);
      break;

    case Mcalc_OperatorType::Derivative:
      resultvalue = FindDerivative(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Derivative2:
      resultvalue = FindDerivative2(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Integral:
      resultvalue = FindIntegral(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RootSolve:
      resultvalue = FindRoot(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Intersect:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindIntersections(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FunctionMinimum:
      resultvalue = FindMinimum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FunctionMaximum:
      resultvalue = FindMaximum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::MakeSet:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = CreateSet(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetUnion:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetUnion(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetIntersection:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetIntersection(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetDifference:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetDifference(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetExclusion:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetExclusion(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemPermutation:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindPermutations(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemCombination:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindCombinations(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemCombineWithRep:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindCombinationsWithRep(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemPermuteWithRep:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindPermutationsWithRep(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::MatrixSolve:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = SolveMatrix(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FractionArithmetic:
      SetAnswerMode(FRACTION_MODE);
      Answer_ = FindFractionArithmetic(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RectToPolarRadius:
      Answer_ = RectToPolar(NumParams_, ParamArray_, 1);
      break;

    case Mcalc_OperatorType::RectToPolarAngle:
      Answer_ = RectToPolar(NumParams_, ParamArray_, 0);
      break;

    case Mcalc_OperatorType::PolarToRectX:
      Answer_ = PolarToRect(NumParams_, ParamArray_, 1);
      break;

    case Mcalc_OperatorType::PolarToRectY:
      Answer_ = PolarToRect(NumParams_, ParamArray_, 0);
      break;

    case Mcalc_OperatorType::FromHex:
      Answer_ = ConvertFromHex(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromOct:
      Answer_ = ConvertFromOct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromBin:
      Answer_ = ConvertFromBin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromCmp2Bin:
      Answer_ = ConvertFromCmp2Bin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToHex:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToHex(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToOct:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToOct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToBin:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToBin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToCmp2Bin:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToCmp2Bin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Sum:
      TempResult_ = CumSum(NumParams_, ParamArray_);
      resultvalue = TempResult_;
      break;

    case Mcalc_OperatorType::Product:
      TempResult_ = CumProduct(NumParams_, ParamArray_);
      resultvalue = TempResult_;
      break;

    case Mcalc_OperatorType::AvgMean:
      TempResult_ = AverageMean(NumParams_, ParamArray_);
      resultvalue = TempResult_;
      break;

    case Mcalc_OperatorType::AvgMedian:
      TempResult_ = AverageMedian(NumParams_, ParamArray_);
      resultvalue = TempResult_;
      break;

    case Mcalc_OperatorType::AvgMode:
      TempResult_ = AverageMode(NumParams_, ParamArray_);
      resultvalue = TempResult_;
      break;

    case Mcalc_OperatorType::StdDev:
      TempResult_ = StandardDeviation(NumParams_, ParamArray_);
      resultvalue = TempResult_;
      break;

    default:
      buffer = "";
      ErrorSet_ = 1;
  }

  if (!CallFnc_)
  {
    for (x = 0; x < NumParams_; x++)
      delete params[x];

    MemMatrix::Matrix().Deallocate(params);
    params = NULL;
  }

  if (!DiscFnc_)
    Answer_ = (ErrorSet_ || HasErrors()) ? buffer:resultvalue.ToString();
  else if (ErrorSet_ || HasErrors())
    Answer_ = buffer;

  return Answer_;
}

/*****************************************************************************/
const LongNumber& LongNumCalculator::DirectProcess(Mcalc_Operand** operands, char** arglist, int optype)
{
  ChrString buffer;
  int IntVal_;
  int dividebyzero = 0;                 /* divide by zero flag */
  LongNumber value1, value2;            /* values of respective operands */
  LongNumber intval1, intval2;          /* integer values of respective operands */
  int x, i, len;
  Mcalc_Operand *resultptr = operands[RESULTINDEX];
  LongNumber intp_;
  int TrigOps_;
  LongDouble Third_ = (1.0 / 3);
  LongNumber One_(1, NumberSign::PLUS);
  LongNumber Ten_(10, NumberSign::PLUS);
  long ExpVal_;

  Ldouble MaxLdPrec_ = DEFAULT_LONGDOUBLE_EPSILON;
  ChrString PiStr_ = LongNumber::vPI().ToString();
  Ldouble PiVal_ = LongDouble(PiStr_).FloatValue();
  #if HAS_LONGDOUBLE_STDFNCS
    Ldouble ip[8];
  #else
    double ip[8];
  #endif
  Ldouble Tol_;
  Ldouble tval_[8];
  bool Match_;
  bool Match2_;
  bool neg_;
  int cd1, cd2;
  Ldouble Ldval_;
  Ldouble lower_;
  Ldouble upper_;

  if (!StringComparison())
  {
    int FirstArgNotUsed_ = (optype == Mcalc_OperatorType::Square) ||
                           (optype == Mcalc_OperatorType::SquareRoot) ||
                           (optype == Mcalc_OperatorType::Cube) ||
                           (optype == Mcalc_OperatorType::CubeRoot);

    if ((!FirstArgNotUsed_ &&
         !(((Uchar)(*arglist[2])) == NAN_CONST || ((Uchar)(*arglist[2])) == INF_CONST || LongNumber::IsValue(arglist[2]))) ||
         !(((Uchar)(*arglist[3])) == NAN_CONST || ((Uchar)(*arglist[3])) == INF_CONST || LongNumber::IsValue(arglist[3])))
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, GetFncNameFromOpcode(optype));
      CloseOutputFile();
      resultvalue = 0;
      return resultvalue;
    }

    if (((Uchar)(*arglist[2])) == NAN_CONST)
      value1 = LongNumber::vNAN();
    else if (((Uchar)(*arglist[2])) == INF_CONST)
      value1 = LongNumber::vINFINITY();
    else
    {
      buffer = (optype == Mcalc_OperatorType::Square) ? "2":
               (optype == Mcalc_OperatorType::SquareRoot) ? ".5":
               (optype == Mcalc_OperatorType::Cube) ? "3":
               (optype == Mcalc_OperatorType::CubeRoot) ? Third_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION):
               arglist[2];
      value1 = buffer;
    }

    if (((Uchar)(*arglist[3])) == NAN_CONST)
      value2 = LongNumber::vNAN();
    else if (((Uchar)(*arglist[3])) == INF_CONST)
      value2 = LongNumber::vINFINITY();
    else
    {
      buffer = (optype == Mcalc_OperatorType::Percent) ? "100":
               arglist[3];
      value2 = buffer;
    }

    TrigOps_ =
    (optype == Mcalc_OperatorType::Sine)          ||
    (optype == Mcalc_OperatorType::Cosine)        ||
    (optype == Mcalc_OperatorType::Tangent)       ||
    (optype == Mcalc_OperatorType::HyperSine)     ||
    (optype == Mcalc_OperatorType::HyperCosine)   ||
    (optype == Mcalc_OperatorType::HyperTangent);

    if (GetAngleUnit() == DEGREE && TrigOps_)
    {
      value2 /= 180L;
      value2 *= LongNumber::vPI();
    }
  }

  /* finding the result value and remainder value */
  switch (optype)
  {
    case Mcalc_OperatorType::Logical_Not:
      if (value2.ConvertedFromBase() == 16 || value2.ConvertedFromBase() == 8)
        resultvalue = BitwiseOps(value1, value2, optype);
      else
        resultvalue = Integer(!value2);
      break;
    case Mcalc_OperatorType::Sine:
      neg_ = value2.GetSign() == NumberSign::MINUS;
      Ldval_ = LongDouble(value2.ToString()).FloatValue();
      Tol_ = Ldouble(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= PiVal_;

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 0, Tol_, MaxLdPrec_, tval_[0], ip[0]);
      if (value2.AtZero() || Match_)
      {
        resultvalue = 0L;
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 5 * PiVal_ / 6, Tol_, MaxLdPrec_, tval_[1], ip[1]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 6, Tol_, MaxLdPrec_, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[1]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[7]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ChrString(".5");
        else
          resultvalue = ChrString("-.5");
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 3 * PiVal_ / 4, Tol_, MaxLdPrec_, tval_[2], ip[2]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 4, Tol_, MaxLdPrec_, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[2]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[6]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = LongNumber::vROOT2() / 2L;
        else
          resultvalue = -LongNumber::vROOT2() / 2L;
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 2 * PiVal_ / 3, Tol_, MaxLdPrec_, tval_[3], ip[3]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 3, Tol_, MaxLdPrec_, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[3]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[5]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = LongNumber::vROOT3() / 2L;
        else
          resultvalue = -LongNumber::vROOT3() / 2L;
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 2, Tol_, MaxLdPrec_, tval_[4], ip[4]);
      if (Match_)
      {
        cd1 = (long(ip[4]) % 2L) ? 1:-1;
        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1L;
        else
          resultvalue = -1L;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        lower_ = ip[0] - Tol_;
        upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          resultvalue = 0L;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          cd1 = (long(ip[4]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = 1L;
          else
            resultvalue = -1L;
          break;
        }

        lower_ = ip[1] - Tol_;
        upper_ = ip[1] + Tol_;

        if (lower_ < tval_[1] && tval_[1] < upper_)
        {
          cd1 = (long(ip[1]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = ChrString(".5");
          else
            resultvalue = ChrString("-.5");
          break;
        }

        lower_ = ip[7] - Tol_;
        upper_ = ip[7] + Tol_;

        if (lower_ < tval_[7] && tval_[7] < upper_)
        {
          cd1 = (long(ip[7]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = ChrString(".5");
          else
            resultvalue = ChrString("-.5");
          break;
        }
      }

      resultvalue = value2.sin();
      break;
    case Mcalc_OperatorType::Cosine:
      neg_ = value2.GetSign() == NumberSign::MINUS;
      Ldval_ = LongDouble(value2.ToString()).FloatValue();
      Tol_ = Ldouble(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= PiVal_;

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 0, Tol_, MaxLdPrec_, tval_[0], ip[0]);
      if (value2.AtZero() || Match_)
      {
        cd2 = (long(ip[0]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = 1L;
        else
          resultvalue = -1L;
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 5 * PiVal_ / 6, Tol_, MaxLdPrec_, tval_[1], ip[1]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 6, Tol_, MaxLdPrec_, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[1]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[7]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = LongNumber::vROOT3() / 2L;
        else
          resultvalue = -LongNumber::vROOT3() / 2L;
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 3 * PiVal_ / 4, Tol_, MaxLdPrec_, tval_[2], ip[2]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 4, Tol_, MaxLdPrec_, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[2]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[6]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = LongNumber::vROOT2() / 2L;
        else
          resultvalue = -LongNumber::vROOT2() / 2L;
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 2 * PiVal_ / 3, Tol_, MaxLdPrec_, tval_[3], ip[3]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 3, Tol_, MaxLdPrec_, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[3]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[5]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = ChrString(".5");
        else
          resultvalue = ChrString("-.5");
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 2, Tol_, MaxLdPrec_, tval_[4], ip[4]);
      if (Match_)
      {
        resultvalue = 0;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        lower_ = ip[0] - Tol_;
        upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          cd2 = (long(ip[0]) % 2L) ? -1:1;

          if (cd2 > 0)
            resultvalue = 1L;
          else
            resultvalue = -1L;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          resultvalue = 0L;
          break;
        }

        lower_ = ip[3] - Tol_;
        upper_ = ip[3] + Tol_;

        if (lower_ < tval_[3] && tval_[3] < upper_)
        {
          cd2 = (long(ip[3]) % 2L) ? 1:-1;

          if (cd2 > 0)
            resultvalue = ChrString(".5");
          else
            resultvalue = ChrString("-.5");
          break;
        }

        lower_ = ip[5] - Tol_;
        upper_ = ip[5] + Tol_;

        if (lower_ < tval_[5] && tval_[5] < upper_)
        {
          cd2 = (long(ip[5]) % 2L) ? -1:1;

          if (cd2 > 0)
            resultvalue = ChrString(".5");
          else
            resultvalue = ChrString("-.5");
          break;
        }
      }

      resultvalue = value2.cos();
      break;
    case Mcalc_OperatorType::Tangent:
      neg_ = value2.GetSign() == NumberSign::MINUS;
      Ldval_ = LongDouble(value2.ToString()).FloatValue();
      Tol_ = Ldouble(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= PiVal_;

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 0, Tol_, MaxLdPrec_, tval_[0], ip[0]);
      if (value2.AtZero() || Match_)
      {
        resultvalue = 0;
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 5 * PiVal_ / 6, Tol_, MaxLdPrec_, tval_[1], ip[1]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 6, Tol_, MaxLdPrec_, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = LongNumber(1L / LongNumber::vROOT3());
        else
          resultvalue = LongNumber(-(1L / LongNumber::vROOT3()));
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 3 * PiVal_ / 4, Tol_, MaxLdPrec_, tval_[2], ip[2]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 4, Tol_, MaxLdPrec_, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1;
        else
          resultvalue = -1;
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, 2 * PiVal_ / 3, Tol_, MaxLdPrec_, tval_[3], ip[3]);
      Match2_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 3, Tol_, MaxLdPrec_, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = LongNumber::vROOT3();
        else
          resultvalue = -LongNumber::vROOT3();
        break;
      }

      Match_ = LongDoubleCalculator::FindCommonAngles(Ldval_, PiVal_ / 2, Tol_, MaxLdPrec_, tval_[4], ip[4]);
      if (Match_)
      {
        cd1 = (long(ip[4]) % 2L) ? 1:-1;
        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = LongNumber::vINFINITY();
        else
          resultvalue = -LongNumber::vINFINITY();
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        lower_ = ip[0] - Tol_;
        upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          cd1 = (long(ip[4]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = LongNumber::vINFINITY();
          else
            resultvalue = -LongNumber::vINFINITY();
          break;
        }

        lower_ = ip[2] - Tol_;
        upper_ = ip[2] + Tol_;

        if (lower_ < tval_[2] && tval_[2] < upper_)
        {
          cd2 = neg_ ? -1:1;
          if (cd2 > 0)
            resultvalue = 1;
          else
            resultvalue = -1;
          break;
        }

        lower_ = ip[6] - Tol_;
        upper_ = ip[6] + Tol_;

        if (lower_ < tval_[6] && tval_[6] < upper_)
        {
          cd2 = neg_ ? 1:-1;
          if (cd2 > 0)
            resultvalue = 1;
          else
            resultvalue = -1;
          break;
        }
      }

      resultvalue = value2.tan();
      break;
    case Mcalc_OperatorType::ArcSine:
      resultvalue = value2.arcsin();
      break;
    case Mcalc_OperatorType::ArcCosine:
      resultvalue = value2.arccos();
      break;
    case Mcalc_OperatorType::ArcTangent:
      resultvalue = value2.arctan();
      break;
    case Mcalc_OperatorType::Add:
      resultvalue = value1 + value2;
      break;
    case Mcalc_OperatorType::Subtract:
      resultvalue = value1 - value2;
      break;
    case Mcalc_OperatorType::Multiply:
      resultvalue = value1 * value2;
      break;
    case Mcalc_OperatorType::Divide:
      dividebyzero = value2.AtZero();
      if (!dividebyzero)
      {
        if (_PlottingGraph && !_PlotAllPoints)
        {
          Tol_ = Ldouble(DEFAULT_PRECVAL_GRAPH);
          tval_[0] = -Tol_;
          tval_[1] = Tol_;

          if (LongNumber(LongDouble(tval_[0])) < value2 && value2 < LongNumber(LongDouble(tval_[1])))
          {
            int dsign_ = (value2.GetSign() == NumberSign::MINUS) ? -1:1;
            int nsign_ = (value1.GetSign() == NumberSign::MINUS) ? -1:1;
            nsign_ *= dsign_;

            if (nsign_ > 0)
              resultvalue = LongNumber::vINFINITY();
            else
              resultvalue = -LongNumber::vINFINITY();
            break;
          }
        }

        resultvalue = value1 / value2;

        if (!value1.AtZero() &&
            _QLookAhead && *_QLookAhead == '^')
        {
          LongNumber val1a = value1 + _Epsilon;
          LongNumber val2a = value2 + _Epsilon;
          LongNumber val1b = value1 - _Epsilon;
          LongNumber val2b = value2 - _Epsilon;

          if ((value1.IsInteger() ||
               val1a.IsInteger() || val1b.IsInteger()) &&
              (value2.IsInteger() ||
               val2a.IsInteger() || val2b.IsInteger()))
          {
            if (!value1.IsInteger())
              if (val1a.IsInteger())
                value1 = val1a;
              else
                value1 = val1b;

            if (!value2.IsInteger())
              if (val2a.IsInteger())
                value2 = val2a;
              else
                value2 = val2b;

            value1 %= 2L;
            value2 %= 2L;

            if (!value2.AtZero())
              _OddRoot = value1.AtZero() ? 2:1;
            else
              _OddRoot = 0;
          }
        }
      }
      else
      {
        if (_PlottingGraph)
        {
          int dsign_ = (value2.GetSign() == NumberSign::MINUS) ? -1:1;
          int nsign_ = (value1.GetSign() == NumberSign::MINUS) ? -1:1;
          nsign_ *= dsign_;

          if (nsign_ > 0)
            resultvalue = LongNumber::vINFINITY();
          else
            resultvalue = -LongNumber::vINFINITY();
        }
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::Modulus:
      intval1 = value1;
      intval1.Floor();
      intval2 = value2;
      intval2.Floor();
      dividebyzero = intval2.AtZero();
      if (!dividebyzero)
        resultvalue = intval1 % intval2;
      else
      {
        if (_PlottingGraph)
          resultvalue = LongNumber::vNAN();
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::Logical_And:
      if ((value1.ConvertedFromBase() == 16 || value1.ConvertedFromBase() == 8) &&
          (value2.ConvertedFromBase() == 16 || value2.ConvertedFromBase() == 8))
        resultvalue = BitwiseOps(value1, value2, optype);
      else
        resultvalue = Integer(value1 && value2);
      break;
    case Mcalc_OperatorType::Logical_Or:
      if ((value1.ConvertedFromBase() == 16 || value1.ConvertedFromBase() == 8) &&
          (value2.ConvertedFromBase() == 16 || value2.ConvertedFromBase() == 8))
        resultvalue = BitwiseOps(value1, value2, optype);
      else
        resultvalue = Integer(value1 || value2);
      break;
    case Mcalc_OperatorType::Exponent:
      if (value2.AtZero())
        resultvalue = 1L;
      else if (value2.IsInteger())
      {
        ExpVal_ = atol(arglist[3]);
        resultvalue = LongNumber::IntPow(value1, ExpVal_);
      }
      else
      {
        if (value1.GetSign() < 0)
        {
          if (_OddRoot)
          {
            value1.FlipSign();
            resultvalue = value1.pow(value2);

            if (_OddRoot == 1)
              resultvalue.FlipSign();
          }
          else
            resultvalue = value1.pow(value2);
        }
        else
          resultvalue = value1.pow(value2);
      }
      break;
    case Mcalc_OperatorType::Square:
      resultvalue = value2.pow(value1);
      break;
    case Mcalc_OperatorType::SquareRoot:
      resultvalue = value2.pow(value1);
      break;
    case Mcalc_OperatorType::Cube:
      resultvalue = value2.pow(value1);
      break;
    case Mcalc_OperatorType::CubeRoot:
      if (value2.GetSign() < 0)
      {
        value2.FlipSign();
        resultvalue = value2.pow(value1);
        resultvalue.FlipSign();
      }
      else
        resultvalue = value2.pow(value1);
      break;
    case Mcalc_OperatorType::HyperSine:
      resultvalue = value2.sinh();
      break;
    case Mcalc_OperatorType::HyperCosine:
      resultvalue = value2.cosh();
      break;
    case Mcalc_OperatorType::HyperTangent:
      resultvalue = value2.tanh();
      break;
    case Mcalc_OperatorType::NaturalExp:
      resultvalue = value2.exp();
      break;
    case Mcalc_OperatorType::NaturalLog:
      resultvalue = value2.log();
      break;
    case Mcalc_OperatorType::Log10:
      resultvalue = value2.log10();
      break;
    case Mcalc_OperatorType::Ceiling:
      resultvalue = value2.Ceiling();
      break;
    case Mcalc_OperatorType::Floor:
      resultvalue = value2.Floor();
      break;
    case Mcalc_OperatorType::AbsoluteValue:
      resultvalue = value2.fabs();
      break;
    case Mcalc_OperatorType::FractionPart:
      resultvalue = value2.modf(intp_);
      break;
    case Mcalc_OperatorType::IntegerPart:
      value2.modf(intp_);
      resultvalue = intp_;
      break;
    case Mcalc_OperatorType::Percent:
      resultvalue = value1 / value2;
      break;
    case Mcalc_OperatorType::LessThan:
      if (StringComparison())
        resultvalue = CompareStrings(arglist[2], arglist[3], optype);
      else
        resultvalue = Integer(value1 < value2);
      break;
    case Mcalc_OperatorType::GreaterThan:
      if (StringComparison())
        resultvalue = CompareStrings(arglist[2], arglist[3], optype);
      else
        resultvalue = Integer(value1 > value2);
      break;
    case Mcalc_OperatorType::LessOrEqual:
      if (StringComparison())
        resultvalue = CompareStrings(arglist[2], arglist[3], optype);
      else
        resultvalue = Integer(value1 <= value2);
      break;
    case Mcalc_OperatorType::GreaterOrEqual:
      if (StringComparison())
        resultvalue = CompareStrings(arglist[2], arglist[3], optype);
      else
        resultvalue = Integer(value1 >= value2);
      break;
    case Mcalc_OperatorType::Equal:
      if (StringComparison())
        resultvalue = CompareStrings(arglist[2], arglist[3], optype);
      else
        resultvalue = Integer(value1 == value2);
      break;
    case Mcalc_OperatorType::NotEqual:
      if (StringComparison())
        resultvalue = CompareStrings(arglist[2], arglist[3], optype);
      else
        resultvalue = Integer(value1 != value2);
      break;
    case Mcalc_OperatorType::Round:
      resultvalue = value2.Round();
      break;
    case Mcalc_OperatorType::Factorial:
      resultvalue = Discrete<LongNumber>::Factorial(value2);
      break;
    case Mcalc_OperatorType::Permutation:
      resultvalue = Discrete<LongNumber>::Permutation(value1, value2);
      break;
    case Mcalc_OperatorType::Combination:
      resultvalue = Discrete<LongNumber>::Combination(value1, value2);
      break;
    case Mcalc_OperatorType::CombineWithRep:
      resultvalue = Discrete<LongNumber>::CombinationWithRep(value1, value2);
      break;
    case Mcalc_OperatorType::PermuteWithRep:
      resultvalue = Discrete<LongNumber>::PermutationWithRep(value1, value2);
      break;
    case Mcalc_OperatorType::Assignment:
      resultvalue = value2;
      break;
    case Mcalc_OperatorType::Random:
      IntVal_ = atoi(arglist[3]);
      resultvalue = GiveRandom(IntVal_);
      break;
    case Mcalc_OperatorType::Exclusive_Or:
      if ((value1.ConvertedFromBase() == 16 || value1.ConvertedFromBase() == 8) &&
          (value2.ConvertedFromBase() == 16 || value2.ConvertedFromBase() == 8))
        resultvalue = BitwiseOps(value1, value2, optype);
      else
        resultvalue = !(Integer(value1 && value2)) && Integer(value1 || value2);
      break;
    case Mcalc_OperatorType::Inverse:
      dividebyzero = value2.AtZero();
      if (!dividebyzero)
      {
        if (_PlottingGraph && !_PlotAllPoints)
        {
          Tol_ = Ldouble(DEFAULT_PRECVAL_GRAPH);
          tval_[0] = -Tol_;
          tval_[1] = Tol_;

          if (LongNumber(LongDouble(tval_[0])) < value2 && value2 < LongNumber(LongDouble(tval_[1])))
          {
            if (value2.GetSign() == NumberSign::MINUS)
              resultvalue = -LongNumber::vINFINITY();
            else
              resultvalue = LongNumber::vINFINITY();
            break;
          }
        }

        resultvalue = One_ / value2;

        if (_QLookAhead && *_QLookAhead == '^')
        {
          LongNumber val2a = value2 + _Epsilon;
          LongNumber val2b = value2 - _Epsilon;

          if (value2.IsInteger() ||
              val2a.IsInteger() || val2b.IsInteger())
          {
            if (!value2.IsInteger())
              if (val2a.IsInteger())
                value2 = val2a;
              else
                value2 = val2b;

            value2 %= 2L;

            if (!value2.AtZero())
              _OddRoot = 1;
            else
              _OddRoot = 0;
          }
        }
      }
      else
      {
        if (_PlottingGraph)
        {
          if (value2.GetSign() == NumberSign::MINUS)
            resultvalue = -LongNumber::vINFINITY();
          else
            resultvalue = LongNumber::vINFINITY();
        }
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::ScientificNotation:
      if (::IsNumber(arglist[3], TRUE, FALSE, FALSE, 10))
      {
        ExpVal_ = atol(arglist[3]);
        value2 = LongNumber::IntPow(Ten_, ExpVal_);
        resultvalue = value1 * value2;
      }
      break;
  }

  TrigOps_ =
  (optype == Mcalc_OperatorType::ArcSine)          ||
  (optype == Mcalc_OperatorType::ArcCosine)        ||
  (optype == Mcalc_OperatorType::ArcTangent);

  if (GetAngleUnit() == DEGREE && TrigOps_)
  {
    resultvalue /= LongNumber::vPI();
    resultvalue *= 180L;
  }

  if (!dividebyzero)
  {
    if (LongNumber::IsInfinite(resultvalue))
    {
      if (resultvalue.GetSign() < 0)
        SetLiteralStr("-Inf");
      else
        SetLiteralStr("+Inf");
      ResetAnswerMode();
      SetAnswerMode(LITERAL_MODE, TRUE);
      clearvalue(resultptr);
    }
    else if (LongNumber::IsNan(resultvalue))
    {
      SetLiteralStr("Nan");
      ResetAnswerMode();
      SetAnswerMode(LITERAL_MODE, TRUE);
      clearvalue(resultptr);
    }
    else
    {
      /* converting decimal result value to array digit elements */
      LongNumber absresultval = resultvalue;
      AbsoluteValue(absresultval);

      buffer = absresultval.ToString();
      len = buffer.strlen();

      allocatevalue(resultptr, len);
      resultptr->sign = (resultvalue.GetSign() == NumberSign::MINUS) ? Mcalc_SignType::Minus:Mcalc_SignType::Plus;
      strcpy(resultptr->value, buffer.c_str());
    }
  }
  else if (!_PlottingGraph)
  {
    ChrString Numer_, Denom_;

    if (optype == Mcalc_OperatorType::Divide ||
        optype == Mcalc_OperatorType::Inverse)
    {
      if (optype == Mcalc_OperatorType::Inverse)
        Numer_ = "1";
      else
        Numer_ = value1.ToString();

      Denom_ = value2.ToString();
      Numer_ += ChrString("/") + Denom_;
    }
    else if (optype == Mcalc_OperatorType::Modulus)
    {
      Numer_ = intval1.ToString();
      Denom_ = intval2.ToString();
      Numer_ += ChrString("@") + Denom_;
    }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_DIVISION_BY_ZERO,
                        ERRMSG_DIVISION_BY_ZERO, Numer_.c_str());
    CloseOutputFile();
  }

  return resultvalue;
}

/*****************************************************************************/
const LongNumber& LongNumCalculator::GetResult() const
{
  return resultvalue;
}

/*****************************************************************************/
const LongNumber& LongNumCalculator::GetVariable(int Index_) const
{
  static LongNumber Dummy_;
  if (Index_ >= MAXVARS || Index_ < 0)
    return Dummy_;

  return (_vars[Index_] ? *(_vars[Index_]):Dummy_);
}

/*****************************************************************************/
size_t LongNumCalculator::GetVariableLength(int Index_) const
{
  if (Index_ < 0)
    return 0;

  if (IsStringVar(Index_))
    return GetStrVarLen(Index_);
  else if (Index_ >= MAXVARS)
    return 0;

  static LongNumber Dummy_;
  LongNumber* VarPtr_ = (_vars[Index_] ? _vars[Index_]:&Dummy_);

  ChrString ValStr_ = VarPtr_->ToString();
  size_t len = strlen(ValStr_.c_str());
  return len;
}

/*****************************************************************************/
void LongNumCalculator::SetVariable(int Index_, const LongNumber& Value1_, int Transfer_)
{
  int x;
  int AssignType_;
  LongNumber Value_;
  bool AltBase_ = StringAssignment() && AnswerMode() == ALTBASE_MODE;

  if (Transfer_ || !AltBase_)
  {
    Value_ = Value1_;
    AssignType_ = ASSIGN_ONLY;
  }
  else
    Value_ = GetStrResult();

  if (!Transfer_)
    GetIndex(x, AssignType_);

  if (!AssignType_ || Index_ >= MAXVARS)
    return;

  if (Index_ < 0)
  {
    resultvalue = Value_;
    return;
  }

  if (!_vars[Index_])
    _vars[Index_] = new LongNumber;

  if (AssignType_ == ASSIGN_ONLY ||
       (AltBase_ && AssignType_ == STRING_ASSIGN))
    *(_vars[Index_]) = Value_;
  else if (AssignType_ == ASSIGN_PLUS)
    *(_vars[Index_]) += Value_;
  else if (AssignType_ == ASSIGN_MINUS)
    *(_vars[Index_]) -= Value_;
  else if (AssignType_ == ASSIGN_MULT)
    *(_vars[Index_]) *= Value_;
  else if (AssignType_ == ASSIGN_DIV)
    *(_vars[Index_]) /= Value_;
  else if (AssignType_ == ASSIGN_MOD)
    *(_vars[Index_]) %= Value_;
  else if (AssignType_ == ASSIGN_AND && AltBase_)
  {
    char* BaseStr_ = NULL;
    int AssignDone_ = FALSE;

    if (IsStringVar(Index_))
    {
      BaseStr_ = (char*)MemMatrix::Matrix().Allocate(GetStrVarLen(Index_) + 1);
      CopyStrVar(BaseStr_, Index_);

      if ((BaseStr_ && (IsIntHex(BaseStr_) || IsIntOct(BaseStr_))) &&
          (Value_.ConvertedFromBase() == 16 || Value_.ConvertedFromBase() == 8))
      {
        ChrString TempStr_ = BitwiseOps(*_vars[Index_], Value_, Mcalc_OperatorType::Logical_And);
        *(_vars[Index_]) = TempStr_;
        AssignDone_ = TRUE;
      }

      MemMatrix::Matrix().Deallocate(BaseStr_);
    }

    if (!AssignDone_)
      *(_vars[Index_]) = Integer(!_vars[Index_]->AtZero() && !Value_.AtZero());
  }
  else if (AssignType_ == ASSIGN_OR && AltBase_)
  {
    char* BaseStr_ = NULL;
    int AssignDone_ = FALSE;

    if (IsStringVar(Index_))
    {
      BaseStr_ = (char*)MemMatrix::Matrix().Allocate(GetStrVarLen(Index_) + 1);
      CopyStrVar(BaseStr_, Index_);

      if ((BaseStr_ && (IsIntHex(BaseStr_) || IsIntOct(BaseStr_))) &&
          (Value_.ConvertedFromBase() == 16 || Value_.ConvertedFromBase() == 8))
      {
        ChrString TempStr_ = BitwiseOps(*_vars[Index_], Value_, Mcalc_OperatorType::Logical_Or);
        *(_vars[Index_]) = TempStr_;
        AssignDone_ = TRUE;
      }

      MemMatrix::Matrix().Deallocate(BaseStr_);
    }

    if (!AssignDone_)
      *(_vars[Index_]) = Integer(!_vars[Index_]->AtZero() || !Value_.AtZero());
  }

  if (Transfer_ || AltBase_)
    if (Value_.ConvertedFromBase() == 16)
      SetStrVariable(Index_, _vars[Index_]->ToString(16));
    else if (Value_.ConvertedFromBase() == 8)
      SetStrVariable(Index_, _vars[Index_]->ToString(8));

  resultvalue = *(_vars[Index_]);
}

/*****************************************************************************/
int LongNumCalculator::GiveType() const
{
  return LONGNUM;
}

/*****************************************************************************/
void LongNumCalculator::SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_)
{
  if (VarIndex_ < 0)
    return;

  if (IsStringVar(VarIndex_))
  {
    CopyStrVar(&ArgStr_[StrIndex_], VarIndex_);
    StrIndex_ += GetStrVarLen(VarIndex_);
  }
  else if (VarIndex_ < MAXVARS)
  {
    if (_vars[VarIndex_])
    {
      ChrString ValStr_ = _vars[VarIndex_]->ToString();
      strcpy(&ArgStr_[StrIndex_], ValStr_.c_str());
      StrIndex_ += strlen(ValStr_.c_str());
    }
    else
    {
      _vars[VarIndex_] = new LongNumber;
      strcpy(&ArgStr_[StrIndex_], "0");
      StrIndex_++;
    }
  }
}

/*****************************************************************************/
/*****************************************************************************/
FloatCalculator::FloatCalculator():
resultvalue(0),
_Epsilon(DEFAULT_FLOAT_EPSILON)
{
  int i;
  for (i = 0; i < MAXVARS; i++)
    _vars[i] = 0;

  _IntCoords =
  (IntCoords<float>**)MemMatrix::Matrix().Callocate(MAX_INT_COORDS * sizeof(IntCoords<float>*));
  for (i = 0; i < MAX_INT_COORDS; i++)
    _IntCoords[i] = new IntCoords<float>();
}

/*****************************************************************************/
FloatCalculator::~FloatCalculator()
{
  if (_IntCoords)
  {
    int i;
    for (i = 0; i < MAX_INT_COORDS; i++)
      delete _IntCoords[i];

    MemMatrix::Matrix().Deallocate(_IntCoords);
    _IntCoords = NULL;
  }
}

/*****************************************************************************/
void FloatCalculator::SetParent(CalculatorBase* Ptr_)
{
  _ParentPtr = Ptr_;
  _CalcIndex = CalculatorBase::FLOAT;
}

/****************************************************************************/
float FloatCalculator::IntPow(float Value_, long Exp_)
{
  if (!Exp_)
    return float(1);

  if (Exp_ == 1L)
    return Value_;

  float Answer_(Value_);
  long Aexp_ = labs(Exp_);
  long i;

  for (i = 1; i < Aexp_; i++)
    Answer_ *= Value_;

  if (Exp_ < 0)
    Answer_ = 1L / Answer_;

  return Answer_;
}

/*****************************************************************************/
ChrString FloatCalculator::ToString(int Index_)
{
  ChrString DestStr_;
  Float FpObj_;

  if (Index_ >= 0 && IsStringVar(Index_))
  {
    CopyStrVar(DestStr_, Index_);
    return DestStr_;
  }

  if (Index_ < 0)
    FpObj_ = resultvalue;
  else
    FpObj_ = _vars[Index_];

  return FpObj_.ToString(Float::TYPE_FLOAT_PRECISION);
}

/*****************************************************************************/
ChrString FloatCalculator::FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_)
{
  const char* ParamStr_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Float HiTestPt_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Float LoTestPt_ = HiTestPt_;
  Float Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  Float Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
  Float Tol_ = ChrString(ParamStr_);
  Float Incr_ = ChrString(DEFAULT_PRECSTR_APPROXZERO);

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  ChrString ValStr_;
  Float LoResult1_;
  Float* LoResult2_ = NULL;
  Float HiResult1_;
  Float* HiResult2_ = NULL;
  Float Low_;
  Float High_;
  Float Testx_;
  Float ZeroPt_;
  int Sign_;
  int ZeroCrossing_;
  ZeroFound_ = FALSE;

  if (HasErrors())
    ClearError();

  while ((HiTestPt_ <= Upper_ || LoTestPt_ >= Lower_) && !HasErrors())
  {
    if (LoTestPt_ >= Lower_)
    {
      ValStr_ = LoTestPt_.ToString(Float::TYPE_FLOAT_PRECISION);
      FncVal_ = ChrString("(") + ValStr_ + ChrString(")");
      LoResult1_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (LoResult1_.FloatValue() == float(0))
      {
        Testx_ = ValStr_;
        ZeroFound_ = TRUE;
        break;
      }
      else if (!LoResult2_)
        LoResult2_ = new Float(LoResult1_);
      else
      {
        ZeroCrossing_ = LoResult1_.FloatValue() < float(0) &&
                        LoResult2_->FloatValue() > float(0) ||
                        LoResult1_.FloatValue() > float(0) &&
                        LoResult2_->FloatValue() < float(0);

        if (ZeroCrossing_)
        {
          Low_ = LoTestPt_;
          High_ = LoTestPt_ + Incr_;
          ValStr_ = Low_.ToString(Float::TYPE_FLOAT_PRECISION);
          ParamArray_[3]->_Param->SetAnswer(ValStr_.c_str());
          ValStr_ = High_.ToString(Float::TYPE_FLOAT_PRECISION);
          ParamArray_[4]->_Param->SetAnswer(ValStr_.c_str());
          Testx_ = (High_ + Low_) / 2L;
          break;
        }
        else
        {
          *LoResult2_ = LoResult1_;
          LoTestPt_ -= Incr_;
        }
      }
    }

    if (HiTestPt_ <= Upper_)
    {
      ValStr_ = HiTestPt_.ToString(Float::TYPE_FLOAT_PRECISION);
      FncVal_ = ChrString("(") + ValStr_ + ChrString(")");
      HiResult1_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (HiResult1_.FloatValue() == float(0))
      {
        Testx_ = ValStr_;
        ZeroFound_ = TRUE;
        break;
      }
      else if (!HiResult2_)
        HiResult2_ = new Float(HiResult1_);
      else
      {
        ZeroCrossing_ = HiResult1_.FloatValue() < float(0) &&
                        HiResult2_->FloatValue() > float(0) ||
                        HiResult1_.FloatValue() > float(0) &&
                        HiResult2_->FloatValue() < float(0);

        if (ZeroCrossing_)
        {
          Low_ = HiTestPt_ - Incr_;
          High_ = HiTestPt_;
          ValStr_ = Low_.ToString(Float::TYPE_FLOAT_PRECISION);
          ParamArray_[3]->_Param->SetAnswer(ValStr_.c_str());
          ValStr_ = High_.ToString(Float::TYPE_FLOAT_PRECISION);
          ParamArray_[4]->_Param->SetAnswer(ValStr_.c_str());
          Testx_ = (High_ + Low_) / 2L;
          break;
        }
        else
        {
          *HiResult2_ = HiResult1_;
          HiTestPt_ += Incr_;
        }
      }
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  Float xTol_ = Testx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete LoResult2_;
  delete HiResult2_;

  if (HasErrors())
    ClearError();

  LongNumber Answer_ = Testx_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString FloatCalculator::FindDerivative(int NumParams_, FncParameter** ParamArray_)
{
  if (!Float::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      (NumParams_ >= 4 &&
       !Float::IsValue(ParamArray_[3]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "DERIV");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  ChrString NewExprStr_ = ParamArray_[2]->_Param->GetAnswer();
  Float Value_ = NewExprStr_;
  NewExprStr_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Buffer_;
  Float Prec_ = NewExprStr_;
  Float HalfPrec_ = Prec_ / 2L;
  HalfPrec_ *= Float(.8);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    PlotFunction(2, ParamArray_, Value_.ToString(Float::TYPE_FLOAT_PRECISION));
  }

  Float HighFncRet_;
  Float LowFncRet_;
  Float HighVal_ = Value_ + HalfPrec_;
  Float LowVal_ = Value_ - HalfPrec_;

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + LowVal_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  LowFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + HighVal_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  HighFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  HighFncRet_ -= LowFncRet_;
  HighVal_ -= LowVal_;

  Float Slope_ = HighFncRet_;
  Slope_ /= HighVal_;
  PopSubExprStack();

  long Exp_ = 0;
  Float xTol_ = Slope_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_DERIV;

  LongNumber Answer_ = Slope_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString FloatCalculator::FindDerivative2(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      (NumParams_ >= 4 &&
       !LongNumber::IsValue(ParamArray_[3]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "DERIV2");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  ChrString NewExprStr_ = ParamArray_[2]->_Param->GetAnswer();
  Float Value_ = NewExprStr_;
  NewExprStr_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Buffer_;
  Float Prec_ = NewExprStr_;
  Float HalfPrec_ = Prec_ / 2L;
  HalfPrec_ *= Float(.8);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    ChrString TempStr_ = ChrString("DERIV(") + ExprStr_;
    TempStr_ += ChrString(", ") + VarName_;
    TempStr_ += ChrString(", ") + VarName_;
    TempStr_ += ChrString("2)");

    FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(2 * sizeof(FncParameter*));
    Params_[0] = new FncParameter();
    Params_[0]->_Param = new SubExpression();
    Params_[0]->_Param->SetExpression(TempStr_.c_str());
    Params_[0]->_ShouldEval = FALSE;
    TempStr_ = VarName_ + ChrString("2");
    Params_[1] = new FncParameter();
    Params_[1]->_Param = new SubExpression();
    Params_[1]->_Param->SetExpression(TempStr_.c_str());
    Params_[1]->_ShouldEval = FALSE;

    PlotFunction(2, Params_, Value_.ToString(Float::TYPE_FLOAT_PRECISION));

    int x;
    for (x = 0; x < 2; x++)
      delete Params_[x];

    MemMatrix::Matrix().Deallocate(Params_);
  }

  Float HighFncRet_;
  Float LowFncRet_;
  Float MidTermRet_;
  Float HighVal_ = Value_ + HalfPrec_;
  Float LowVal_ = Value_ - HalfPrec_;

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + LowVal_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  LowFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + HighVal_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  HighFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + Value_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  MidTermRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  MidTermRet_ *= 2L;

  HighFncRet_ -= MidTermRet_;
  HighFncRet_ += LowFncRet_;
  HalfPrec_ *= HalfPrec_;

  Float Slope_ = HighFncRet_;
  Slope_ /= HalfPrec_;
  PopSubExprStack();

  long Exp_ = 0;
  Float xTol_ = Slope_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_DERIV;

  LongNumber Answer_ = Slope_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
void FloatCalculator::PrintInfNanValues(FILE* Fout_, const Float& Answer_, float fval)
{
  if (Answer_.IsInfinite())
  {
    if (Answer_.FloatValue() < float(0))
      fprintf(Fout_, DEFAULT_FLOATNAN_FORMAT, fval, "-inf");
    else
      fprintf(Fout_, DEFAULT_FLOATNAN_FORMAT, fval, "+inf");
  }
  else if (Answer_.IsNan()) {
    fprintf(Fout_, DEFAULT_FLOATNAN_FORMAT, fval, "nan"); }
  else
  {
    float Sd = Answer_;
    fprintf(Fout_, DEFAULT_FLOAT_FORMAT, fval, Sd);
  }
}

/*****************************************************************************/
ChrString FloatCalculator::FindIntersections(int NumParams_, FncParameter** ParamArray_)
{
  int x;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;
  ChrString FirstRt_;
  ChrString RetStr_;
  ChrString FncVal_;
  char CurrentSuffix_ = _PlotFileSuffix;
  Float Yresult_;
  Float Xresult_;
  Float Upper_;
  Float Lower_;
  Float MidPt_;
  Float RadVal_;
  FncParameter** Params_;

  Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_;
  ChrString NewExpr_;

  if (NumParams_ == 5 || NumParams_ == 3)
  {
    NewExpr_ = "(";
    NewExpr_ += ParamArray_[0]->_Param->GetExpression();
    NewExpr_ += ") - (";
    NewExpr_ += ParamArray_[1]->_Param->GetExpression();
    NewExpr_ += ")";
    VarName_ = ParamArray_[2]->_Param->GetExpression();
    CalculatorBase::SetGraphType(Mcalc_GraphType::INTERSECT2);
    _IntExpr = ParamArray_[0]->_Param->GetExpression();
    _IntExpr += " , ";
    _IntExpr += ParamArray_[1]->_Param->GetExpression();
  }
  else if (NumParams_ == 4 || NumParams_ == 2)
  {
    NewExpr_ = ParamArray_[0]->_Param->GetExpression();
    VarName_ = ParamArray_[1]->_Param->GetExpression();
    _IntExpr = NewExpr_;
  }

  Params_[0] = new FncParameter();
  Params_[0]->_Param = new SubExpression();
  Params_[0]->_Param->SetExpression(NewExpr_.c_str());
  Params_[1] = (NumParams_ == 4 || NumParams_ == 2) ?
                   new FncParameter(*ParamArray_[1]):
                   new FncParameter(*ParamArray_[2]);
  Params_[2] = new FncParameter();
  Params_[2]->_Param = new SubExpression();
  Params_[3] = Params_[4] = NULL;

  if (NumParams_ == 2 || NumParams_ == 3)
    Params_[2]->_Param->SetAnswer("0");
  else
  {
    if (NumParams_ == 4)
    {
      Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
      Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;

      if (MidPt_.IsNan() || MidPt_.IsInfinite())
        Params_[2]->_Param->SetAnswer("0");
      else
        Params_[2]->_Param->SetAnswer(MidPt_.ToString().c_str());
    }
    else if (NumParams_ == 5)
    {
      Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;

      if (MidPt_.IsNan() || MidPt_.IsInfinite())
        Params_[2]->_Param->SetAnswer("0");
      else
        Params_[2]->_Param->SetAnswer(MidPt_.ToString().c_str());
    }
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);

  for (_IntsFound = x = 0; x < 3; x++)
  {
    RetStr_ = FindRoot(3, Params_);

    if (!HasErrors())
    {
      FncVal_ = ChrString("(") + RetStr_ + ChrString(")");
      Xresult_ = RetStr_;
      Yresult_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (!_IntsFound ||
          (_IntsFound == 1 && _IntCoords[0]->X != Xresult_.FloatValue()) ||
          (_IntsFound == 2 && _IntCoords[1]->X != Xresult_.FloatValue() &&
                              _IntCoords[0]->X != Xresult_.FloatValue()))
      {
        if (!_IntsFound)
          FirstRt_ = Xresult_.ToString(Float::TYPE_FLOAT_PRECISION);

        _IntCoords[_IntsFound]->X = Xresult_.FloatValue();
        _IntCoords[_IntsFound]->Y = Yresult_.FloatValue();
        ++_IntsFound;
      }
    }

    if (x == 0)
    {
      if (NumParams_ == 4 || NumParams_ == 5)
      {
        if (NumParams_ == 4)
          Params_[2]->_Param->SetAnswer(ParamArray_[2]->_Param->GetAnswer());
        else
          Params_[2]->_Param->SetAnswer(ParamArray_[3]->_Param->GetAnswer());
      }
      else
        Params_[2]->_Param->SetAnswer("-5");
    }
    else if (x == 1)
    {
      if (NumParams_ == 4 || NumParams_ == 5)
      {
        if (NumParams_ == 4)
          Params_[2]->_Param->SetAnswer(ParamArray_[3]->_Param->GetAnswer());
        else
          Params_[2]->_Param->SetAnswer(ParamArray_[4]->_Param->GetAnswer());
      }
      else
        Params_[2]->_Param->SetAnswer("5");
    }

    if (HasErrors())
      ClearError();
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();

  if (PlotGraph_)
  {
    int RangeSpecified_ = NumParams_ == 4 || NumParams_ == 5;
    if (_IntsFound)
      RetStr_ = FirstRt_;
    else
      RetStr_ = "0";

    if (RangeSpecified_)
    {
      if (NumParams_ == 5)
      {
        Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
        Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      }
      else if (NumParams_ == 4)
      {
        Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
        Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      }

      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;
      RadVal_ = Upper_ - MidPt_;
      AbsoluteValue(RadVal_);
    }

    if (NumParams_ == 5 || NumParams_ == 3)
    {
      // Plotting First Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[2]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(Float::TYPE_FLOAT_PRECISION),
                     RadVal_.ToString(Float::TYPE_FLOAT_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);

      // Plotting Second Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[1]);
      Params_[1] = new FncParameter(*ParamArray_[2]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(Float::TYPE_FLOAT_PRECISION),
                     RadVal_.ToString(Float::TYPE_FLOAT_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);
    }
    else if (NumParams_ == 4 || NumParams_ == 2)
    {
      // Plotting First Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(Float::TYPE_FLOAT_PRECISION),
                     RadVal_.ToString(Float::TYPE_FLOAT_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);
    }

    SaveCurrentGraphName(CurrentSuffix_);
  }

  for (x = 0; x < 5; x++)
    delete Params_[x];
  MemMatrix::Matrix().Deallocate(Params_);

  char Buffer_[64];
  char Suf_ = 'z';
  sprintf(Buffer_, "mcalc_graph%d%c.out", SessionNum(), Suf_);
  FILE* Fout_ = fopen(Buffer_, "w");
  RetStr_ = "\n";

  if (Fout_)
  {
    if (_IntsFound)
      for (x = 0; x < _IntsFound; x++)
      {
        sprintf(Buffer_, DEFAULT_FLOAT_FORMAT, _IntCoords[x]->X, _IntCoords[x]->Y);
        fprintf(Fout_, "%s", Buffer_);
        RetStr_ += Buffer_;
      }

    fclose(Fout_);
  }

  SetHasAnswer(RetStr_.strlen());

  if (RetStr_.strlen())
  {
    RetStr_ = ChrString("\"") + RetStr_ + ChrString("\"");
    SetLiteralStr(RetStr_.c_str());
  }

  return RetStr_;
}

/*****************************************************************************/
ChrString FloatCalculator::PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                        const ChrString& MidPtStr_, ChrString Radius_, ChrString* Solution_)
{
  if (NumParams_ < 2 || (NumParams_ >= 3 && !LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "GRAPH");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString Caption_;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();

  if (ExprStr_.find(VarName_).failed())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_GRAPHVARNAME,
                        ERRMSG_INVALID_GRAPHVARNAME, "GRAPH");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  char Fname_[32];
  sprintf(Fname_, "mcalc_graph%d%c.out", SessionNum(), PlotFileSuffix());
  FILE* Fout_ = fopen(Fname_, "w");

  Float Mid_ = (NumParams_ >= 3) ? ParamArray_[2]->_Param->GetAnswer():MidPtStr_;
  Float RadiusVal_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Radius_;
  Float Lower_ = Mid_ - RadiusVal_;
  Float Upper_ = Mid_ + RadiusVal_;

  ChrString AnsStr_;
  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(4 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  AnsStr_ = Lower_.ToString(Float::TYPE_FLOAT_PRECISION);
  Params_[2] = new FncParameter();
  Params_[2]->_Param = new SubExpression();
  Params_[2]->_Param->SetAnswer(AnsStr_.c_str());
  Params_[2]->_ShouldEval = FALSE;
  AnsStr_ = Upper_.ToString(Float::TYPE_FLOAT_PRECISION);
  Params_[3] = new FncParameter();
  Params_[3]->_Param = new SubExpression();
  Params_[3]->_Param->SetAnswer(AnsStr_.c_str());
  Params_[3]->_ShouldEval = FALSE;
  long Ints_ = FindNumIntervals(4, Params_);

  Float Ht = (Upper_ - Lower_) / Ints_;
  Float Xi;
  Float Answer_;
  float Sd;
  float Htval_ = Ht;
  float Lowerval_ = Lower_;
  float fval;

  if (Solution_)
    Mid_ = *Solution_;

  if (_GraphType == Mcalc_GraphType::EXTREMA)
  {
    if (_ExtremaEnd < 0)
      Caption_ = ChrString("FMIN( ") + ExprStr_ + ChrString(" )");
    else
      Caption_ = ChrString("FMAX( ") + ExprStr_ + ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::DERIVATIVE)
  {
    ChrString DerivPt_ = ParamArray_[2]->_Param->GetAnswer();
    Caption_ = ChrString("DERIV( ") + ExprStr_;
    Caption_ += ChrString(" , ") + DerivPt_;
    Caption_ += ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::INTEGRAL)
  {
    ChrString LowerIntPt_ = ParamArray_[2]->_Param->GetAnswer();
    ChrString UpperIntPt_ = ParamArray_[3]->_Param->GetAnswer();
    Caption_ = ChrString("INTEG( ") + ExprStr_;
    Caption_ += ChrString(" , ") + LowerIntPt_;
    Caption_ += ChrString(" , ") + UpperIntPt_;
    Caption_ += ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::ROOTSOLVE)
    Caption_ = ChrString("SOLVE( ") + ExprStr_ + ChrString(" )");
  else if (_GraphType == Mcalc_GraphType::INTERSECT1 ||
           _GraphType == Mcalc_GraphType::INTERSECT2)
    Caption_ = ChrString("INTERSECT( ") + _IntExpr + ChrString(" )");
  else
    Caption_ = ExprStr_;

  _PlottingGraph = TRUE;
  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  fprintf(Fout_, "%s\n", Caption_.c_str());
  fprintf(Fout_, "%d\n", _GraphType);
  fprintf(Fout_, "%d\n", (_GraphType == Mcalc_GraphType::INTEGRAL) ? 1:0);

  fval = Mid_;
  ChrString FncVal_ = ChrString("(") + Mid_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    FloatCalculator::PrintInfNanValues(Fout_, Answer_, fval);

  fval = Lower_;
  FncVal_ = ChrString("(") + Lower_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    FloatCalculator::PrintInfNanValues(Fout_, Answer_, fval);

  long x;
  for (x = 1; x < Ints_; x++)
  {
    Xi = Float(Lowerval_ + Htval_ * x);
    fval = Xi;
    FncVal_ = ChrString("(") + Xi.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (HasErrors())
    {
      if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
        ClearError();
      else
        goto HandleError;
    }
    else
      FloatCalculator::PrintInfNanValues(Fout_, Answer_, fval);
  }

  fval = Upper_;
  FncVal_ = ChrString("(") + Upper_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    FloatCalculator::PrintInfNanValues(Fout_, Answer_, fval);

HandleError:
  fclose(Fout_);
  SetSuppressOutput(FALSE);
  PopSubExprStack();
  _PlottingGraph = FALSE;

  for (x = 0; x < 4; x++)
    delete Params_[x];

  MemMatrix::Matrix().Deallocate(Params_);
  return ChrString("0");
}

/*****************************************************************************/
ChrString FloatCalculator::FindIntegral(int NumParams_, FncParameter** ParamArray_)
{
  if (!Float::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !Float::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 5 &&
       !Float::IsValue(ParamArray_[4]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "INTEG");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  long Ints_ = FindNumIntervals(NumParams_, ParamArray_);
  const char* Buffer_ = DEFAULT_PRECSTR_INTEG;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Float Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Float Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  const char* ParamStr_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetAnswer():Buffer_;
  Float Prec_ = strtod(ParamStr_, NULL);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    Float MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    Float RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
    PlotFunction(2, ParamArray_, MidPt_.ToString(Float::TYPE_FLOAT_PRECISION),
                 RadVal_.ToString(Float::TYPE_FLOAT_PRECISION));
  }

  Float Ht = (Upper_ - Lower_) / Ints_;
  Float Xi;
  Float Answer_;
  float Sd;
  float Htval_ = Ht;
  float Lowerval_ = Lower_;

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_ = ChrString("(") + Lower_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  Sd = Answer_;

  long x;
  for (x = 1; x < Ints_; x++)
  {
    Xi = Float(Lowerval_ + Htval_ * x);
    FncVal_ = ChrString("(") + Xi.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Answer_ *= 2L;
    Sd += Answer_;
  }

  FncVal_ = ChrString("(") + Upper_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  Sd += Answer_;
  Answer_ = Float(Htval_ * Sd / 2L);
  SetSuppressOutput(FALSE);
  PopSubExprStack();

  long Exp_ = 0;
  Float xTol_ = Answer_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_INTEG;

  LongNumber StrAnswer_ = Answer_;
  StrAnswer_.Round(Exp_);
  return StrAnswer_.ToString();
}

/*****************************************************************************/
ChrString FloatCalculator::FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!Float::IsValue(ParamArray_[2]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Float Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Float Incr_ = DEFAULT_PRECVAL_NEWTCRIT;

  CreateNewSubExpr();
  float X1 = Guess_;
  X1 -= 10;
  Float Temp_ = X1;
  ChrString FncVal_ = ChrString("(") + Temp_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  float Y1 = Temp_;
  float CriteriaVal_;

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  Temp_ = X1;
  FncVal_ = Temp_.ToString(Float::TYPE_FLOAT_PRECISION);
  Params_[2]->_Param->SetAnswer(FncVal_.c_str());
  Temp_ = FindDerivative(NumParams_, Params_);
  float Ypr = Temp_;
  Temp_ = FindDerivative2(NumParams_, Params_);
  float Ypr2 = Temp_;
  Float Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);

  int i;
  int max = int(20 / Incr_.FloatValue());
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 0; i < max; i++)
  {
    CriteriaVal_ = ::fabs((Y1 * Ypr2) / (Ypr * Ypr));

    if (CriteriaVal_ < 1)
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (CriteriaVal_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Temp_ = X1;
    Temp_ += Incr_;
    X1 = Temp_;
    FncVal_ = ChrString("(") + Temp_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y1 = Temp_;
    Temp_ = X1;
    FncVal_ = Temp_.ToString(Float::TYPE_FLOAT_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Temp_ = FindDerivative(NumParams_, Params_);
    Ypr = Temp_;
    Temp_ = FindDerivative2(NumParams_, Params_);
    Ypr2 = Temp_;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  Float xTol_ = X1;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  if (HasErrors())
    ClearError();

  Temp_ = X1;
  LongNumber Answer_ = Temp_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString FloatCalculator::NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!Float::IsValue(ParamArray_[2]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Float Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());

  CreateNewSubExpr();
  float X1 = Guess_;
  Float Temp_ = X1;
  ChrString FncVal_ = ChrString("(") + Temp_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  float Y1 = Temp_;
  float X2;
  float Y2;

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  Temp_ = X1;
  FncVal_ = Temp_.ToString(Float::TYPE_FLOAT_PRECISION);
  Params_[2]->_Param->SetAnswer(FncVal_.c_str());
  Temp_ = FindDerivative(NumParams_, Params_);
  float Ypr = Temp_;
  float Diff_;
  Float Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);

  int i;
  int max = 100;
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 0; i < max; i++)
  {
    X2 = X1 - Y1 / Ypr;
    Temp_ = X2;
    FncVal_ = ChrString("(") + Temp_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y2 = Temp_;
    Diff_ = X2 - X1;
    Diff_ = ::fabs(Diff_);

    if (Diff_ < Tol_.FloatValue() &&
        Y2 < Tol_.FloatValue())
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (Diff_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Temp_ = X2;
    FncVal_ = Temp_.ToString(Float::TYPE_FLOAT_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Temp_ = FindDerivative(NumParams_, Params_);
    Ypr = Temp_;
    X1 = X2;
    Y1 = Y2;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  Float xTol_ = X2;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  if (HasErrors())
    ClearError();

  Temp_ = X2;
  LongNumber Answer_ = Temp_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString FloatCalculator::MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!Float::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !Float::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      !Float::IsValue(ParamArray_[4]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Float Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Float Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  Float Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + Lower_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Float LowerY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  FncVal_ = ChrString("(") + Upper_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Float UpperY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  FncVal_ = ChrString("(") + Guess_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
  Float GuessY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  float Guessval_ = Guess_;
  float Lowerval_ = Lower_;
  float Upperval_ = Upper_;
  float LowerYval_ = LowerY_;
  float UpperYval_ = UpperY_;
  float GuessYval_ = GuessY_;

  Float Temp_ = (UpperY_ - LowerY_) / (Upper_ - Lower_);
  float C1 = Temp_;
  float C2;
  float D1;
  float S;
  float Det_;
  float X4;
  float Y4;
  float Diff_;
  Float Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);
  long Sign_;

  int i;
  int max = 2000;
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 2; i < max; i++)
  {
    C2 = (GuessYval_ - UpperYval_) / (Guessval_ - Upperval_);
    D1 = (C2 - C1) / (Guessval_ - Lowerval_);
    S = C2 + D1 * (Guessval_ - Upperval_);

    Sign_ = (S >= float(0)) ? 1L:-1L;
    Det_ = ::pow(S, 2) - (4L * GuessYval_ * D1);
    SquareRoot(Det_);
    X4 = Guessval_ - ((2L * GuessYval_) / (S + Sign_ * Det_));

    Temp_ = X4;
    FncVal_ = ChrString("(") + Temp_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y4 = Temp_;
    Diff_ = X4 - Guessval_;
    Diff_ = ::fabs(Diff_);

    if (Diff_ < Tol_.FloatValue() &&
        Y4 < Tol_.FloatValue())
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (Diff_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Lowerval_ = Upperval_;
    Upperval_ = Guessval_;
    UpperYval_ = GuessYval_;
    Guessval_ = X4;
    GuessYval_ = Y4;
    C1 = C2;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  Float xTol_ = X4;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  if (HasErrors())
    ClearError();

  LongNumber Answer_ = Float(X4);
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString FloatCalculator::FindRoot(int NumParams_, FncParameter** ParamArray_)
{
  int x;
  int Converge_;
  int CriteriaFound_;
  ChrString RetStr_;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;
  bool RangeSpecified_ = false;
  FncParameter** Params_;

  if (NumParams_ > 3)
  {
    RangeSpecified_ = true;
    RetStr_ = MullersMethod(NumParams_, ParamArray_, Converge_);

    if (!Converge_)
    {
      Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      Params_[2] = new FncParameter(*ParamArray_[2]);
      Params_[3] = new FncParameter(*ParamArray_[3]);
      Params_[4] = new FncParameter(*ParamArray_[4]);
      ChrString Answer_ = FindNewtonsCriteria(NumParams_, Params_, CriteriaFound_);

      if (CriteriaFound_)
      {
        Params_[2]->_Param->SetAnswer(Answer_.c_str());
        RetStr_ = NewtonsMethod(NumParams_, Params_, Converge_);

        if (Converge_)
        {
          for (x = 0; x < 5; x++)
            delete Params_[x];

          MemMatrix::Matrix().Deallocate(Params_);
          goto RootSolveDone;
        }
        else
        {
          RetStr_ = MullersMethod(NumParams_, Params_, Converge_);

          if (Converge_)
          {
            for (x = 0; x < 5; x++)
              delete Params_[x];

            MemMatrix::Matrix().Deallocate(Params_);
            goto RootSolveDone;
          }
        }
      }
    }
  }
  else
  {
    RetStr_ = NewtonsMethod(NumParams_, ParamArray_, Converge_);

    if (!Converge_)
    {
      Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      Params_[2] = new FncParameter(*ParamArray_[2]);
      Params_[3] = Params_[4] = NULL;
      ChrString Answer_ = FindNewtonsCriteria(NumParams_, Params_, CriteriaFound_);

      if (CriteriaFound_)
      {
        Params_[2]->_Param->SetAnswer(Answer_.c_str());
        RetStr_ = NewtonsMethod(NumParams_, Params_, Converge_);

        if (Converge_)
        {
          for (x = 0; x < 3; x++)
            delete Params_[x];

          MemMatrix::Matrix().Deallocate(Params_);
          goto RootSolveDone;
        }
      }

      Params_[3] = new FncParameter();
      Params_[4] = new FncParameter();
      Params_[3]->_Param = new SubExpression();
      Params_[4]->_Param = new SubExpression();
      Boolean ZeroFound_ = FALSE;
      Float Lower_ = ChrString(Params_[2]->_Param->GetAnswer());
      Float Upper_ = ChrString(Params_[2]->_Param->GetAnswer());
      NumParams_ += 2;
      Lower_ -= 20L;
      Upper_ += 20L;
      Answer_ = Lower_.ToString(Float::TYPE_FLOAT_PRECISION);
      Params_[3]->_Param->SetAnswer(Answer_.c_str());
      Params_[3]->_ShouldEval = FALSE;
      Answer_ = Upper_.ToString(Float::TYPE_FLOAT_PRECISION);
      Params_[4]->_Param->SetAnswer(Answer_.c_str());
      Params_[4]->_ShouldEval = FALSE;
      RetStr_ = MullersMethod(NumParams_, Params_, Converge_);

      if (!Converge_)
      {
        RetStr_ = FindApproxZero(NumParams_, Params_, ZeroFound_);
        if (!ZeroFound_)
          RetStr_ = MullersMethod(NumParams_, Params_, Converge_);
        else
          Converge_ = TRUE;
      }

      if (!Converge_)
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED,
                        ERRMSG_MULLERS_METHOD_FAILED, ParamArray_[0]->_Param->GetExpression());
        CloseOutputFile();
      }

      for (x = 0; x < 5; x++)
        delete Params_[x];

      MemMatrix::Matrix().Deallocate(Params_);
    }
  }

RootSolveDone:
  if (!Converge_)
  {
    if (RangeSpecified_)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED,
                      ERRMSG_MULLERS_METHOD_FAILED, ParamArray_[0]->_Param->GetExpression());
      CloseOutputFile();
    }
  }
  else if (PlotGraph_)
  {
    if (RangeSpecified_)
    {
      Float Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      Float Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      Float MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;
      Float RadVal_ = Upper_ - MidPt_;
      AbsoluteValue(RadVal_);
      PlotFunction(2, ParamArray_, MidPt_.ToString(Float::TYPE_FLOAT_PRECISION),
                   RadVal_.ToString(Float::TYPE_FLOAT_PRECISION), &RetStr_);
    }
    else
      PlotFunction(2, ParamArray_, RetStr_);
  }

  return RetStr_;
}

/*****************************************************************************/
ChrString FloatCalculator::FindMinimum(int NumParams_, FncParameter** ParamArray_)
{
  if (!Float::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !Float::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 6 && !Float::IsValue(ParamArray_[5]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMIN");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Float Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Float Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  ChrString InclOpt_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetExpression():"";
  const char* ParamStr_ = (NumParams_ >= 6) ? ParamArray_[5]->_Param->GetAnswer():Buffer_;
  Float Tol_ = ChrString(ParamStr_);
  Float Incr_ = ChrString(DEFAULT_PRECSTR_FTEST);
  Float xTol_ = Upper_ - Lower_;

  if (xTol_.FloatValue() < Integer(0))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMIN");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }
  else
  {
    Float Ratio_ = xTol_ / 10L;
    xTol_ = Tol_;

    if (Ratio_ < Integer(1))
      Incr_ *= Ratio_;
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  Float Deriv1_;
  Float Deriv2_;
  Float Lowest_;
  Float Lowestx_;
  Float Testx_;
  Float Low_;
  Float High_;
  Float MidPt_;
  Float RadVal_;

  int ZeroCrossing_;
  int LowestInit_ = 0;
  int PrevInit_ = 0;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;

  if (PlotGraph_)
  {
    MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
  }

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  for (;Lower_ < Upper_; Lower_ += Incr_)
  {
    FncVal_ = Lower_.ToString(Float::TYPE_FLOAT_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Deriv2_ = FindDerivative(3, Params_);

    if (!PrevInit_)
    {
      if (Deriv2_.FloatValue() != float(0))
      {
        Deriv1_ = Deriv2_;
        PrevInit_ = TRUE;
      }
    }
    else
    {
      ZeroCrossing_ = Deriv1_.FloatValue() < float(0) &&
                      Deriv2_.FloatValue() > float(0);

      if (ZeroCrossing_)
        for (Testx_ = Lower_ - Incr_; Testx_ < Lower_; Testx_ += xTol_)
        {
          FncVal_ = ChrString("(") + Testx_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
          Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

          if (!LowestInit_)
          {
            Lowest_ = Low_;
            Lowestx_ = Testx_;
            LowestInit_ = TRUE;
          }
          else if (Low_ < Lowest_)
          {
            Lowest_ = Low_;
            Lowestx_ = Testx_;
          }
        }

      if (Deriv2_.FloatValue() != float(0))
        Deriv1_ = Deriv2_;
    }
  }

  if (InclOpt_.strlen())
    InclOpt_.RemoveChar(" \t\n");

  int ExclEnds_ = (NumParams_ >= 5) ? icasestrcomp(InclOpt_.c_str(), "EXCLUDEENDS"):0;

  if (!ExclEnds_)
  {
    FncVal_ = ChrString("(") + Lower_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    FncVal_ = ChrString("(") + Upper_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (High_ < Low_)
    {
      if (High_ < Lowest_)
        Lowestx_ = Upper_;
    }
    else
    {
      if (Low_ < Lowest_)
        Lowestx_ = Lower_;
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  xTol_ = Lowestx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  LongNumber Answer_ = Lowestx_;
  Answer_.Round(Exp_);

  if (PlotGraph_)
  {
    _ExtremaEnd = -1;
    ChrString SolString_ = Answer_.ToString();
    PlotFunction(2, ParamArray_, MidPt_.ToString(Float::TYPE_FLOAT_PRECISION),
                 RadVal_.ToString(Float::TYPE_FLOAT_PRECISION), &SolString_);
    return SolString_;
  }

  return Answer_.ToString();
}

/*****************************************************************************/
ChrString FloatCalculator::FindMaximum(int NumParams_, FncParameter** ParamArray_)
{
  if (!Float::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !Float::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 6 && !Float::IsValue(ParamArray_[5]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMAX");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Float Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Float Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  ChrString InclOpt_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetExpression():"";
  const char* ParamStr_ = (NumParams_ >= 6) ? ParamArray_[5]->_Param->GetAnswer():Buffer_;
  Float Tol_ = ChrString(ParamStr_);
  Float Incr_ = ChrString(DEFAULT_PRECSTR_FTEST);
  Float xTol_ = Upper_ - Lower_;

  if (xTol_.FloatValue() < Integer(0))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMAX");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }
  else
  {
    Float Ratio_ = xTol_ / 10L;
    xTol_ = Tol_;

    if (Ratio_ < Integer(1))
      Incr_ *= Ratio_;
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  Float Deriv1_;
  Float Deriv2_;
  Float Highest_;
  Float Highestx_;
  Float Testx_;
  Float High_;
  Float Low_;
  Float MidPt_;
  Float RadVal_;

  int ZeroCrossing_;
  int HighestInit_ = 0;
  int PrevInit_ = 0;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;

  if (PlotGraph_)
  {
    MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
  }

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  for (;Lower_ < Upper_; Lower_ += Incr_)
  {
    FncVal_ = Lower_.ToString(Float::TYPE_FLOAT_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Deriv2_ = FindDerivative(3, Params_);

    if (!PrevInit_)
    {
      if (Deriv2_.FloatValue() != float(0))
      {
        Deriv1_ = Deriv2_;
        PrevInit_ = TRUE;
      }
    }
    else
    {
      ZeroCrossing_ = Deriv1_.FloatValue() > float(0) &&
                      Deriv2_.FloatValue() < float(0);

      if (ZeroCrossing_)
        for (Testx_ = Lower_ - Incr_; Testx_ < Lower_; Testx_ += xTol_)
        {
          FncVal_ = ChrString("(") + Testx_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
          High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

          if (!HighestInit_)
          {
            Highest_ = High_;
            Highestx_ = Testx_;
            HighestInit_ = TRUE;
          }
          else if (High_ > Highest_)
          {
            Highest_ = High_;
            Highestx_ = Testx_;
          }
        }

      if (Deriv2_.FloatValue() != float(0))
        Deriv1_ = Deriv2_;
    }
  }

  if (InclOpt_.strlen())
    InclOpt_.RemoveChar(" \t\n");

  int ExclEnds_ = (NumParams_ >= 5) ? icasestrcomp(InclOpt_.c_str(), "EXCLUDEENDS"):0;

  if (!ExclEnds_)
  {
    FncVal_ = ChrString("(") + Lower_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    FncVal_ = ChrString("(") + Upper_.ToString(Float::TYPE_FLOAT_PRECISION) + ChrString(")");
    High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (High_ < Low_)
    {
      if (Low_ > Highest_)
        Highestx_ = Lower_;
    }
    else
    {
      if (High_ > Highest_)
        Highestx_ = Upper_;
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  xTol_ = Highestx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  LongNumber Answer_ = Highestx_;
  Answer_.Round(Exp_);

  if (PlotGraph_)
  {
    _ExtremaEnd = 1;
    ChrString SolString_ = Answer_.ToString();
    PlotFunction(2, ParamArray_, MidPt_.ToString(Float::TYPE_FLOAT_PRECISION),
                 RadVal_.ToString(Float::TYPE_FLOAT_PRECISION), &SolString_);
    return SolString_;
  }

  return Answer_.ToString();
}

/*****************************************************************************/
ChrString FloatCalculator::RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_)
{
  if (!Float::IsValue(ParamArray_[0]->_Param->GetAnswer()) ||
      !Float::IsValue(ParamArray_[1]->_Param->GetAnswer()))
  {
    if (Retarg_)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "RECTOPOLR");
    else
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "RECTOPOLA");

    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  Float FltVal_ = ChrString(ParamArray_[0]->_Param->GetAnswer());
  float Xpos_ = FltVal_.FloatValue();
  FltVal_ = ChrString(ParamArray_[1]->_Param->GetAnswer());
  float Ypos_ = FltVal_.FloatValue();
  float Xsqr_;
  float Ysqr_;
  float Sin_;
  Float Angle_;
  Float Radius_;
  ChrString Error_ = "0";

  if (fabs(Ypos_) < _Epsilon)
    Ypos_ = 0;

  if (fabs(Xpos_) < _Epsilon)
    Xpos_ = 0;

  if (!Xpos_ && !Ypos_)
    return Error_;

  Ysqr_ = Ypos_;
  Ysqr_ *= Ypos_;
  Xsqr_ = Xpos_;
  Xsqr_ *= Xpos_;
  Radius_ = Xsqr_;
  Radius_ += Float(Ysqr_);
  Radius_ = sqrt(Radius_.FloatValue());

  // rReturn radius
  if (Retarg_)
    return Radius_.ToString(Float::TYPE_FLOAT_PRECISION);

  Sin_ = Ypos_ / Radius_.FloatValue();
  Angle_ = Float(asin(Sin_));
  if (Xpos_ < float(0))
    if (Ypos_ < float(0))
      Angle_ = Float(-Float::vPI) - Angle_;
    else
      Angle_ = Float(Float::vPI) - Angle_;

  if (GetAngleUnit() == DEGREE)
  {
    Angle_ /= Float(Float::vPI);
    Angle_ *= 180L;
  }

  return Angle_.ToString(Float::TYPE_FLOAT_PRECISION);
}

/*****************************************************************************/
ChrString FloatCalculator::PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_)
{
  if (!Float::IsValue(ParamArray_[0]->_Param->GetAnswer()) ||
      !Float::IsValue(ParamArray_[1]->_Param->GetAnswer()))
  {
    if (Retarg_)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "POLTORECX");
    else
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "POLTORECY");

    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  Float FltVal_ = ChrString(ParamArray_[0]->_Param->GetAnswer());
  float Radius_ = FltVal_.FloatValue();
  FltVal_ = ChrString(ParamArray_[1]->_Param->GetAnswer());
  float Angle_ = FltVal_.FloatValue();
  float Mult_;
  float Frac_;
  double iptr_;
  int Int_;
  int Odd_;
  Float Xpos_;
  Float Ypos_;
  float AbsAngle_;
  float PiDiv2_ = Float::vPI / 2L;
  ChrString Zero_ = "0";

  if (Radius_ < _Epsilon)
    return Zero_;

  if (GetAngleUnit() == DEGREE)
  {
    Angle_ /= 180L;
    Angle_ *= Float::vPI;
  }

  AbsAngle_ = fabs(Angle_);

  if (fabs(AbsAngle_ - Float::vPI) > _Epsilon)
  {
    Mult_ = Angle_ / Float::vPI;
    Frac_ = modf(Mult_, &iptr_);
    Int_ = (int)floor(iptr_);
    Odd_ = Int_ % 2;

    if (Frac_ > _Epsilon)
    {
      Angle_ =  Frac_ * Float::vPI;

      if (Odd_)
        if (Angle_ > float(0))
          Angle_ -= Float::vPI;
        else
          Angle_ += Float::vPI;
    }
    else
    {
      if (Odd_)
        Angle_ = Float::vPI;
      else
        Angle_ = 0;
    }
  }

  if (Angle_ >= float(0))
  {
    if (Angle_ > PiDiv2_)
    {
      Angle_ = Float::vPI - Angle_;
      if (fabs(Angle_) < _Epsilon)
        Angle_ = 0;
      Xpos_ = -(Radius_ * cos(Angle_));
    }
    else
    {
      if (fabs(Angle_) < _Epsilon)
        Angle_ = 0;
      Xpos_ = Radius_ * cos(Angle_);
    }

    Ypos_ = Radius_ * sin(Angle_);
  }
  else
  {
    if (Angle_ < -PiDiv2_)
    {
      Angle_ = -Float::vPI - Angle_;
      if (fabs(Angle_) < _Epsilon)
        Angle_ = 0;
      Angle_ *= -1;
      Xpos_ = -(Radius_ * cos(Angle_));
    }
    else
    {
      if (fabs(Angle_) < _Epsilon)
        Angle_ = 0;
      Angle_ *= -1;
      Xpos_ = Radius_ * cos(Angle_);
    }

    Ypos_ = Radius_ * sin(Angle_);
    Ypos_ *= -1;
  }

  if (Retarg_)
    return Xpos_.ToString(Float::TYPE_FLOAT_PRECISION);

  return Ypos_.ToString(Float::TYPE_FLOAT_PRECISION);
}

/*****************************************************************************/
ChrString FloatCalculator::SolveMatrix(int NumParams_, FncParameter** ParamArray_)
{
  int i, x;
  int NumCols_;
  int Max_ = 0;
  int Error_ = FALSE;
  int MatSize_ = 0;
  ChrString AnsCol_;
  ChrString RowStr_;
  ChrString Answer_;
  int* DelimArray_ = (int*)MemMatrix::Matrix().Callocate(sizeof(int) * DELIM_ARRAY_SIZE);
  GVector<float>** Varr_ = new GVector<float>*[NumParams_ + 1];
  GVector<float>* Vect_ = NULL;
  GVector<float> AnsVect_(NumParams_);
  GMatrix<float> Mat_(NumParams_);

  for (i = 0; i <= NumParams_; i++)
    Varr_[i] = NULL;

  for (i = 0; i < NumParams_; i++)
  {
    RowStr_ = ParamArray_[i]->_Param->GetExpression();
    CalculatorBase::FindFncDelim(RowStr_.c_str(), RowStr_.strlen(), &DelimArray_, DELIM_ARRAY_SIZE);
    NumCols_ = DelimArray_[0] - 1;

    if (NumCols_ > Max_)
    {
      if (!Max_)
      {
        if (NumCols_ != NumParams_)
        {
          Error_ = TRUE;
          DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_UNSOLVABLE_MATRIX, ERRMSG_UNSOLVABLE_MATRIX);
          CloseOutputFile();
          break;
        }
        else
          Max_ = NumCols_;
      }
      else
      {
        Error_ = TRUE;
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_MATRIX_COLS, ERRMSG_INVALID_MATRIX_COLS);
        CloseOutputFile();
        break;
      }
    }
    else if (!Max_)
    {
      Error_ = TRUE;
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTYROW_MATRIX, ERRMSG_EMPTYROW_MATRIX);
      CloseOutputFile();
      break;
    }

    int LastCol_ = DelimArray_[NumCols_] + 1;
    char* Divider_ = strchr(RowStr_(LastCol_), '|');

    if (!Divider_)
    {
      Error_ = TRUE;
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_NO_RESULT_COL_MATRIX, ERRMSG_NO_RESULT_COL_MATRIX, RowStr_.c_str());
      CloseOutputFile();
      break;
    }

    if (AnsCol_.strlen())
      AnsCol_ += ", ";
    AnsCol_ += &Divider_[1];
    AnsCol_.RemoveTrailing(")]} \t\n");
    RowStr_.Left(Divider_ - RowStr_(0));
    RowStr_ += ")";

    Vect_ = new GVector<float>(NumParams_);
    Vect_->SetVector(RowStr_.c_str());
    Varr_[MatSize_++] = Vect_;
  }

  if (!Error_)
  {
    AnsVect_.SetVector(AnsCol_.c_str());
    Mat_.SetMatrix(Varr_);
    GVector<float> ResVect_ = GaussPivot(Mat_, AnsVect_, NumParams_);
    ResVect_.WriteToString(Answer_);
  }

  MemMatrix::Matrix().Deallocate(DelimArray_);
  for (i = 0; i < NumParams_; i++)
    delete Varr_[i];
  delete[] Varr_;

  SetHasAnswer(Answer_.strlen());

  if (Answer_.strlen())
  {
    Answer_ = ChrString("\"") + Answer_ + ChrString("\"");
    SetLiteralStr(Answer_.c_str());
  }

  return Answer_;
}

/*****************************************************************************/
ChrString FloatCalculator::EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_)
{
  int CallFnc_, DiscFnc_, ConvFnc_, OptionFnc_;
  int optype, x;
  ChrString buffer;
  ChrString Answer_;

  optype = icasestrcomp(FncName_, "MIN")         ? Mcalc_OperatorType::Minimum:
           icasestrcomp(FncName_, "MAX")         ? Mcalc_OperatorType::Maximum:
           icasestrcomp(FncName_, "DERIV")       ? Mcalc_OperatorType::Derivative:
           icasestrcomp(FncName_, "DERIV2")      ? Mcalc_OperatorType::Derivative2:
           icasestrcomp(FncName_, "INTEG")       ? Mcalc_OperatorType::Integral:
           icasestrcomp(FncName_, "SOLVE")       ? Mcalc_OperatorType::RootSolve:
           icasestrcomp(FncName_, "INTERSECT")   ? Mcalc_OperatorType::Intersect:
           icasestrcomp(FncName_, "PERMREP")     ? Mcalc_OperatorType::PermuteWithRep:
           icasestrcomp(FncName_, "COMBREP")     ? Mcalc_OperatorType::CombineWithRep:
           icasestrcomp(FncName_, "COMB")        ? Mcalc_OperatorType::Combination:
           icasestrcomp(FncName_, "PERM")        ? Mcalc_OperatorType::Permutation:
           icasestrcomp(FncName_, "FMIN")        ? Mcalc_OperatorType::FunctionMinimum:
           icasestrcomp(FncName_, "FMAX")        ? Mcalc_OperatorType::FunctionMaximum:
           icasestrcomp(FncName_, "ROUND")       ? Mcalc_OperatorType::RoundEx:
           icasestrcomp(FncName_, "SET")         ? Mcalc_OperatorType::MakeSet:
           icasestrcomp(FncName_, "SETUNI")      ? Mcalc_OperatorType::SetUnion:
           icasestrcomp(FncName_, "SETINT")      ? Mcalc_OperatorType::SetIntersection:
           icasestrcomp(FncName_, "SETDIF")      ? Mcalc_OperatorType::SetDifference:
           icasestrcomp(FncName_, "SETXDIF")     ? Mcalc_OperatorType::SetExclusion:
           icasestrcomp(FncName_, "ITEMPERM")    ? Mcalc_OperatorType::ItemPermutation:
           icasestrcomp(FncName_, "ITEMCOMB")    ? Mcalc_OperatorType::ItemCombination:
           icasestrcomp(FncName_, "ITEMPERMREP") ? Mcalc_OperatorType::ItemPermuteWithRep:
           icasestrcomp(FncName_, "ITEMCOMBREP") ? Mcalc_OperatorType::ItemCombineWithRep:
           icasestrcomp(FncName_, "FRAC")        ? Mcalc_OperatorType::FractionArithmetic:
           icasestrcomp(FncName_, "RECTOPOLR")   ? Mcalc_OperatorType::RectToPolarRadius:
           icasestrcomp(FncName_, "RECTOPOLA")   ? Mcalc_OperatorType::RectToPolarAngle:
           icasestrcomp(FncName_, "POLTORECX")   ? Mcalc_OperatorType::PolarToRectX:
           icasestrcomp(FncName_, "POLTORECY")   ? Mcalc_OperatorType::PolarToRectY:
           icasestrcomp(FncName_, "FROMHEX")     ? Mcalc_OperatorType::FromHex:
           icasestrcomp(FncName_, "FROMOCT")     ? Mcalc_OperatorType::FromOct:
           icasestrcomp(FncName_, "FROMBIN")     ? Mcalc_OperatorType::FromBin:
           icasestrcomp(FncName_, "FROMCMP2BIN") ? Mcalc_OperatorType::FromCmp2Bin:
           icasestrcomp(FncName_, "TOHEX")       ? Mcalc_OperatorType::ToHex:
           icasestrcomp(FncName_, "TOOCT")       ? Mcalc_OperatorType::ToOct:
           icasestrcomp(FncName_, "TOBIN")       ? Mcalc_OperatorType::ToBin:
           icasestrcomp(FncName_, "TOCMP2BIN")   ? Mcalc_OperatorType::ToCmp2Bin:
           icasestrcomp(FncName_, "AVGMEAN")     ? Mcalc_OperatorType::AvgMean:
           icasestrcomp(FncName_, "AVGMED")      ? Mcalc_OperatorType::AvgMedian:
           icasestrcomp(FncName_, "AVGMODE")     ? Mcalc_OperatorType::AvgMode:
           icasestrcomp(FncName_, "STDDEV")      ? Mcalc_OperatorType::StdDev:
           icasestrcomp(FncName_, "SUM")         ? Mcalc_OperatorType::Sum:
           icasestrcomp(FncName_, "PROD")        ? Mcalc_OperatorType::Product:
           icasestrcomp(FncName_, "MATSOLVE")    ? Mcalc_OperatorType::MatrixSolve:
           icasestrcomp(FncName_, "GRAPH")       ? Mcalc_OperatorType::PlotGraph:0;
  DiscFnc_ = optype == Mcalc_OperatorType::ItemPermutation ||
             optype == Mcalc_OperatorType::ItemCombination ||
             optype == Mcalc_OperatorType::ItemCombineWithRep ||
             optype == Mcalc_OperatorType::ItemPermuteWithRep ||
             optype == Mcalc_OperatorType::RectToPolarRadius ||
             optype == Mcalc_OperatorType::RectToPolarAngle ||
             optype == Mcalc_OperatorType::PolarToRectX ||
             optype == Mcalc_OperatorType::PolarToRectY ||
             optype == Mcalc_OperatorType::FromHex ||
             optype == Mcalc_OperatorType::FromOct ||
             optype == Mcalc_OperatorType::FromBin ||
             optype == Mcalc_OperatorType::FromCmp2Bin ||
             optype == Mcalc_OperatorType::ToHex ||
             optype == Mcalc_OperatorType::ToOct ||
             optype == Mcalc_OperatorType::ToBin ||
             optype == Mcalc_OperatorType::ToCmp2Bin ||
             optype == Mcalc_OperatorType::MatrixSolve ||
             optype == Mcalc_OperatorType::Intersect ||
             optype == Mcalc_OperatorType::MakeSet ||
             optype == Mcalc_OperatorType::SetUnion ||
             optype == Mcalc_OperatorType::SetIntersection ||
             optype == Mcalc_OperatorType::SetDifference ||
             optype == Mcalc_OperatorType::SetExclusion;
  CallFnc_ = optype == Mcalc_OperatorType::Derivative ||
             optype == Mcalc_OperatorType::Derivative2 ||
             optype == Mcalc_OperatorType::Integral ||
             optype == Mcalc_OperatorType::RootSolve ||
             optype == Mcalc_OperatorType::FunctionMinimum ||
             optype == Mcalc_OperatorType::FunctionMaximum ||
             optype == Mcalc_OperatorType::PlotGraph ||
             optype == Mcalc_OperatorType::FractionArithmetic ||
             optype == Mcalc_OperatorType::AvgMean ||
             optype == Mcalc_OperatorType::AvgMedian ||
             optype == Mcalc_OperatorType::AvgMode ||
             optype == Mcalc_OperatorType::StdDev ||
             optype == Mcalc_OperatorType::Sum ||
             optype == Mcalc_OperatorType::Product ||
             (NumParams_ != 2 && (optype == Mcalc_OperatorType::Minimum   ||
                                  optype == Mcalc_OperatorType::Maximum)) ||
             DiscFnc_;
  ConvFnc_ = optype == Mcalc_OperatorType::ToHex ||
             optype == Mcalc_OperatorType::ToOct ||
             optype == Mcalc_OperatorType::ToBin ||
             optype == Mcalc_OperatorType::ToCmp2Bin ||
             optype == Mcalc_OperatorType::FromHex ||
             optype == Mcalc_OperatorType::FromOct ||
             optype == Mcalc_OperatorType::FromBin ||
             optype == Mcalc_OperatorType::FromCmp2Bin;
  OptionFnc_ = optype == Mcalc_OperatorType::Sum ||
               optype == Mcalc_OperatorType::Product ||
               optype == Mcalc_OperatorType::AvgMean ||
               optype == Mcalc_OperatorType::AvgMedian ||
               optype == Mcalc_OperatorType::AvgMode ||
               optype == Mcalc_OperatorType::StdDev ||
               (NumParams_ != 2 && (optype == Mcalc_OperatorType::Minimum   ||
                                    optype == Mcalc_OperatorType::Maximum)) ||
               optype == Mcalc_OperatorType::ItemPermutation ||
               optype == Mcalc_OperatorType::ItemCombination ||
               optype == Mcalc_OperatorType::ItemCombineWithRep ||
               optype == Mcalc_OperatorType::ItemPermuteWithRep ||
               optype == Mcalc_OperatorType::FractionArithmetic ||
               optype == Mcalc_OperatorType::MakeSet;

  int ErrorSet_;
  ErrorSet_ = (!optype || NumParams_ < 2) ? (!((ConvFnc_ || OptionFnc_) && NumParams_ == 1) && (NumParams_ < 2)):
              (optype == Mcalc_OperatorType::Derivative) ? (NumParams_ < 3):
              (optype == Mcalc_OperatorType::Integral) ? (NumParams_ < 4):
              (optype == Mcalc_OperatorType::RootSolve) ? (NumParams_ < 3):
              (optype == Mcalc_OperatorType::Intersect) ? (NumParams_ < 2):
              (optype == Mcalc_OperatorType::FunctionMinimum) ? (NumParams_ < 4):
              (optype == Mcalc_OperatorType::FunctionMaximum) ? (NumParams_ < 4):0;

  if (ErrorSet_)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_NUMARGS,
                        ERRMSG_INVALID_NUMARGS, FncName_);
    CloseOutputFile();
    return buffer;
  }

  Float resultvalue;
  Float** params = NULL;
  LongNumber LnumResult_;
  LongNumber** lparams = NULL;
  double Tol_ = 0;
  long Exp_ = 0;
  long Rexp_;

  int IntOps_ = (optype == Mcalc_OperatorType::Permutation ||
                 optype == Mcalc_OperatorType::Combination ||
                 optype == Mcalc_OperatorType::CombineWithRep ||
                 optype == Mcalc_OperatorType::PermuteWithRep ||
                 optype == Mcalc_OperatorType::RoundEx);

  if (!CallFnc_)
  {
    if (IntOps_)
      lparams = (LongNumber**)allocmem(NumParams_, sizeof(LongNumber*), params);
    else
      params = (Float**)allocmem(NumParams_, sizeof(Float*), params);

    for (x = 0; x < NumParams_; x++)
    {
      buffer = ParamArray_[x]->_Param->GetAnswer();

      if (Float::IsValue(buffer))
      {
        if (IntOps_)
          lparams[x] = new LongNumber(buffer);
        else
          params[x] = new Float(buffer);
      }
      else
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, FncName_);
        CloseOutputFile();
        return buffer;
      }
    }
  }

  ResetAnswerMode();
  SetAnswerMode(FLOATINGPOINT_MODE);

  /* finding the result value and remainder value */
  switch (optype)
  {
    case Mcalc_OperatorType::PlotGraph:
      resultvalue = PlotFunction(NumParams_, ParamArray_, "0");
      break;

    case Mcalc_OperatorType::Permutation:
      LnumResult_ = Discrete<LongNumber>::Permutation(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Combination:
      LnumResult_ = Discrete<LongNumber>::Combination(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::CombineWithRep:
      LnumResult_ = Discrete<LongNumber>::CombinationWithRep(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::PermuteWithRep:
      LnumResult_ = Discrete<LongNumber>::PermutationWithRep(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Minimum:
      if (NumParams_ == 2)
        resultvalue = (*params[0] < *params[1]) ? *params[0]:*params[1];
      else
        resultvalue = MinimumValue(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Maximum:
      if (NumParams_ == 2)
        resultvalue = (*params[0] > *params[1]) ? *params[0]:*params[1];
      else
        resultvalue = MaximumValue(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RoundEx:
      LnumResult_ = *lparams[0];
      Rexp_ = atol(ParamArray_[1]->_Param->GetAnswer());
      LnumResult_.Round(Rexp_);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Derivative:
      Tol_ = DEFAULT_PRECVAL_DERIV;
      resultvalue = FindDerivative(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Derivative2:
      Tol_ = DEFAULT_PRECVAL_DERIV;
      resultvalue = FindDerivative2(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Integral:
      Tol_ = DEFAULT_PRECVAL_INTEG;
      resultvalue = FindIntegral(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RootSolve:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindRoot(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Intersect:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindIntersections(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FunctionMinimum:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindMinimum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FunctionMaximum:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindMaximum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::MakeSet:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = CreateSet(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetUnion:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetUnion(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetIntersection:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetIntersection(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetDifference:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetDifference(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetExclusion:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetExclusion(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemPermutation:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindPermutations(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemCombination:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindCombinations(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemCombineWithRep:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindCombinationsWithRep(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemPermuteWithRep:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindPermutationsWithRep(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::MatrixSolve:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = SolveMatrix(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FractionArithmetic:
      SetAnswerMode(FRACTION_MODE);
      resultvalue = FindFractionArithmetic(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RectToPolarRadius:
      Answer_ = RectToPolar(NumParams_, ParamArray_, 1);
      break;

    case Mcalc_OperatorType::RectToPolarAngle:
      Answer_ = RectToPolar(NumParams_, ParamArray_, 0);
      break;

    case Mcalc_OperatorType::PolarToRectX:
      Answer_ = PolarToRect(NumParams_, ParamArray_, 1);
      break;

    case Mcalc_OperatorType::PolarToRectY:
      Answer_ = PolarToRect(NumParams_, ParamArray_, 0);
      break;

    case Mcalc_OperatorType::FromHex:
      Answer_ = ConvertFromHex(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromOct:
      Answer_ = ConvertFromOct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromBin:
      Answer_ = ConvertFromBin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromCmp2Bin:
      Answer_ = ConvertFromCmp2Bin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToHex:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToHex(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToOct:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToOct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToBin:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToBin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToCmp2Bin:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToCmp2Bin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Sum:
      resultvalue = CumSum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Product:
      resultvalue = CumProduct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMean:
      resultvalue = AverageMean(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMedian:
      resultvalue = AverageMedian(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMode:
      resultvalue = AverageMode(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::StdDev:
      resultvalue = StandardDeviation(NumParams_, ParamArray_);
      break;

    default:
      buffer = "";
      ErrorSet_ = 1;
  }

  if (!CallFnc_)
    if (IntOps_)
    {
      for (x = 0; x < NumParams_; x++)
       delete lparams[x];
      MemMatrix::Matrix().Deallocate(lparams);
      lparams = NULL;
    }
    else
    {
      for (x = 0; x < NumParams_; x++)
        delete params[x];
      MemMatrix::Matrix().Deallocate(params);
      params = NULL;
    }

  if (Tol_ != double(0))
  {
    while (Tol_ < 1)
    {
      ++Exp_;
      Tol_ *= 10L;
    }

    if (!DiscFnc_ && !(ErrorSet_ || HasErrors()))
    {
      Float xTol_ = resultvalue;
      AbsoluteValue(xTol_);
      if (!xTol_.AtZero())
        while (xTol_ < Integer(1))
        {
          ++Exp_;
          xTol_ *= 10L;
        }
    }

    if (!Exp_)
      Exp_ = Float::TYPE_FLOAT_PRECISION;
  }

  if (!DiscFnc_)
    Answer_ = (ErrorSet_ || HasErrors()) ? buffer:
                  (Exp_ ? resultvalue.ToString(Exp_):
                          resultvalue.ToString(Float::TYPE_FLOAT_PRECISION));
  else if (ErrorSet_ || HasErrors())
    Answer_ = buffer;

  return Answer_;
}

/*****************************************************************************/
size_t FloatCalculator::FindPrecision(const float& Val_, size_t Prec_)
{
  float Cval_ = Val_;
  Ceiling(Cval_);

  size_t x;
  for (x = 0; x < Prec_; x++)
    if (Cval_ > Integer(1))
      Cval_ /= 10L;
    else
      break;

  x = Prec_ - x;
  if (x > 0)
    return x;

  return Prec_;
}

/*****************************************************************************/
bool FloatCalculator::FindCommonAngles(float value_, float shift_, float Tol_,
                                       float Eps_, float& mult_, double& intp_)
{
  float tval1;
  float tval2;
  float Multiplier_;

  tval2 = value_;
  AbsoluteValue(tval2);
  tval2 += shift_;
  tval1 = tval2 / Float::vPI;
  Multiplier_ = ::pow(10, FindPrecision(tval1, size_t(-log10(Eps_))));
  tval2 = tval1;
  tval2 += Tol_;

  tval2 = ::modf(tval2, &intp_);
  tval2 = tval1;
  tval1 *= Multiplier_;
  tval1 += .5;
  tval1 = ::floor(tval1);
  tval1 /= Multiplier_;
  tval2 *= Multiplier_;
  tval2 += .5;
  tval2 = ::floor(tval2);
  tval2 /= Multiplier_;
  tval2 = ::ceil(tval2);
  mult_ = tval1;

  return (tval1 == tval2);
}

/*****************************************************************************/
// Procedure to directly process the values given to the program using built-in
// math operators if the length of the arguments permit
// Procedure executes only if OPTIMIZE flag is TRUE
// PASSED:
//   operands : the operands array
//   arglist : the argument list
//   optype : the math operator
// USES:
//   uses allocmem to allocate memory
//   uses allocatevalue to allocate memory to the value array of a operand
//        structure
// READ:
//   arglist is read for the arguments to the program and if within acceptable
//   length they are converted to decimal values and the result calculated
// CHANGED:
//   the result and remainder structures have their value array changed
//   to the result value and remainder value if argument length conditions are
//   met
// RETURNED:
//   returns 1 if calculation could be done directly, otherwise returns 0
//   returns 0 if OPTIMIZE flag is FALSE
//
const float& FloatCalculator::DirectProcess(Mcalc_Operand** operands, char** arglist, int optype)
{
  using ::pow;

  ChrString buffer;
  int IntVal_;
  int dividebyzero = 0;             /* divide by zero flag */
  float value1, value2;             /* values of respective operands */
  Ulong intval1, intval2;           /* values of respective operands */
  int x, i, len;
  Mcalc_Operand *resultptr = operands[RESULTINDEX];
  int TrigOps_;
  long ExpVal_;
  double ip[8];
  double intp_;
  float Tol_;
  float tval_[8];
  bool neg_;
  bool Match_;
  bool Match2_;
  int cd1, cd2;

  if (!StringComparison())
  {
    int FirstArgNotUsed_ = (optype == Mcalc_OperatorType::Square) ||
                           (optype == Mcalc_OperatorType::SquareRoot) ||
                           (optype == Mcalc_OperatorType::Cube) ||
                           (optype == Mcalc_OperatorType::CubeRoot);

    if ((!FirstArgNotUsed_ &&
         !(((Uchar)(*arglist[2])) == NAN_CONST || ((Uchar)(*arglist[2])) == INF_CONST || Float::IsValue(arglist[2]))) ||
         !(((Uchar)(*arglist[3])) == NAN_CONST || ((Uchar)(*arglist[3])) == INF_CONST || Float::IsValue(arglist[3])))
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, GetFncNameFromOpcode(optype));
      CloseOutputFile();
      resultvalue = 0;
      return resultvalue;
    }

    if (((Uchar)(*arglist[2])) == NAN_CONST)
      value1 = Float::vNAN;
    else if (((Uchar)(*arglist[2])) == INF_CONST)
      value1 = Float::vINFINITY;
    else
    {
      value1 = (optype == Mcalc_OperatorType::Square)       ? 2:
               (optype == Mcalc_OperatorType::SquareRoot)   ? .5:
               (optype == Mcalc_OperatorType::Cube)         ? 3:
               (optype == Mcalc_OperatorType::CubeRoot)     ? (1.0/3):
               atof(arglist[2]);
    }

    if (((Uchar)(*arglist[3])) == NAN_CONST)
      value2 = Float::vNAN;
    else if (((Uchar)(*arglist[3])) == INF_CONST)
      value2 = Float::vINFINITY;
    else
    {
      value2 = (optype == Mcalc_OperatorType::Percent)      ? 100:
               atof(arglist[3]);
    }

    TrigOps_ =
    (optype == Mcalc_OperatorType::Sine)          ||
    (optype == Mcalc_OperatorType::Cosine)        ||
    (optype == Mcalc_OperatorType::Tangent)       ||
    (optype == Mcalc_OperatorType::HyperSine)     ||
    (optype == Mcalc_OperatorType::HyperCosine)   ||
    (optype == Mcalc_OperatorType::HyperTangent);

    if (GetAngleUnit() == DEGREE && TrigOps_)
    {
      value2 /= 180L;
      value2 *= Float::vPI;
    }
  }

  /* finding the result value and remainder value */
  switch (optype)
  {
    case Mcalc_OperatorType::Logical_Not:
      intval2 = Ulong(floor(value2));
      resultvalue = !intval2;
      break;

    case Mcalc_OperatorType::Sine:
      neg_ = value2 < float(0);
      Tol_ = float(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= Float::vPI;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == float(0) || Match_)
      {
        resultvalue = 0;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * Float::vPI / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[1]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[7]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = .5;
        else
          resultvalue = -.5;
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * Float::vPI / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[2]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[6]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(2) / 2;
        else
          resultvalue = -(::sqrt(2) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * Float::vPI / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[3]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[5]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3) / 2;
        else
          resultvalue = -(::sqrt(3) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, Float::vPI / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        cd1 = (long(ip[4]) % 2L) ? 1:-1;
        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1L;
        else
          resultvalue = -1L;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        double lower_ = ip[0] - Tol_;
        double upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          cd1 = (long(ip[4]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = 1L;
          else
            resultvalue = -1L;
          break;
        }

        lower_ = ip[1] - Tol_;
        upper_ = ip[1] + Tol_;

        if (lower_ < tval_[1] && tval_[1] < upper_)
        {
          cd1 = (long(ip[1]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }

        lower_ = ip[7] - Tol_;
        upper_ = ip[7] + Tol_;

        if (lower_ < tval_[7] && tval_[7] < upper_)
        {
          cd1 = (long(ip[7]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }
      }

      resultvalue = sin(value2);
      break;
    case Mcalc_OperatorType::Cosine:
      neg_ = value2 < float(0);
      Tol_ = float(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= Float::vPI;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == float(0) || Match_)
      {
        cd2 = (long(ip[0]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = 1L;
        else
          resultvalue = -1L;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * Float::vPI / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[1]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[7]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3) / 2;
        else
          resultvalue = -(::sqrt(3) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * Float::vPI / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[2]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[6]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = ::sqrt(2) / 2;
        else
          resultvalue = -(::sqrt(2) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * Float::vPI / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[3]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[5]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = .5;
        else
          resultvalue = -.5;
        break;
      }

      Match_ = FindCommonAngles(value2, Float::vPI / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        resultvalue = 0;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        double lower_ = ip[0] - Tol_;
        double upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          cd2 = (long(ip[0]) % 2L) ? -1:1;

          if (cd2 > 0)
            resultvalue = 1L;
          else
            resultvalue = -1L;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[3] - Tol_;
        upper_ = ip[3] + Tol_;

        if (lower_ < tval_[3] && tval_[3] < upper_)
        {
          cd2 = (long(ip[3]) % 2L) ? 1:-1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }

        lower_ = ip[5] - Tol_;
        upper_ = ip[5] + Tol_;

        if (lower_ < tval_[5] && tval_[5] < upper_)
        {
          cd2 = (long(ip[5]) % 2L) ? -1:1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }
      }

      resultvalue = cos(value2);
      break;
    case Mcalc_OperatorType::Tangent:
      neg_ = value2 < float(0);
      Tol_ = float(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= Float::vPI;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == float(0) || Match_)
      {
        resultvalue = 0;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * Float::vPI / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1 / ::sqrt(3);
        else
          resultvalue = -(1 / ::sqrt(3));
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * Float::vPI / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1;
        else
          resultvalue = -1;
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * Float::vPI / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, Float::vPI / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3);
        else
          resultvalue = -(::sqrt(3));
        break;
      }

      Match_ = FindCommonAngles(value2, Float::vPI / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        cd1 = (long(ip[4]) % 2L) ? 1:-1;
        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = Float::vINFINITY;
        else
          resultvalue = -Float::vINFINITY;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        double lower_ = ip[0] - Tol_;
        double upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          cd1 = (long(ip[4]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = Float::vINFINITY;
          else
            resultvalue = -Float::vINFINITY;
          break;
        }

        lower_ = ip[2] - Tol_;
        upper_ = ip[2] + Tol_;

        if (lower_ < tval_[2] && tval_[2] < upper_)
        {
          cd2 = neg_ ? -1:1;
          if (cd2 > 0)
            resultvalue = 1;
          else
            resultvalue = -1;
          break;
        }

        lower_ = ip[6] - Tol_;
        upper_ = ip[6] + Tol_;

        if (lower_ < tval_[6] && tval_[6] < upper_)
        {
          cd2 = neg_ ? 1:-1;
          if (cd2 > 0)
            resultvalue = 1;
          else
            resultvalue = -1;
          break;
        }
      }

      resultvalue = tan(value2);
      break;
    case Mcalc_OperatorType::ArcSine:
      resultvalue = asin(value2);
      break;
    case Mcalc_OperatorType::ArcCosine:
      resultvalue = acos(value2);
      break;
    case Mcalc_OperatorType::ArcTangent:
      resultvalue = atan(value2);
      break;
    case Mcalc_OperatorType::Add:
      resultvalue = value1 + value2;
      break;
    case Mcalc_OperatorType::Subtract:
      resultvalue = value1 - value2;
      break;
    case Mcalc_OperatorType::Multiply:
      resultvalue = value1 * value2;
      break;
    case Mcalc_OperatorType::Divide:
      dividebyzero = value2 == float(0);
      if (!dividebyzero)
      {
        if (_PlottingGraph && !_PlotAllPoints)
        {
          Tol_ = Float(DEFAULT_PRECSTR_GRAPH);
          tval_[0] = -Tol_;
          tval_[1] = Tol_;

          if (tval_[0] < value2 && value2 < tval_[1])
          {
            int dsign_ = (value2 < float(0)) ? -1:1;
            int nsign_ = (value1 < float(0)) ? -1:1;
            nsign_ *= dsign_;

            if (nsign_ > 0)
              resultvalue = Float::vINFINITY;
            else
              resultvalue = -Float::vINFINITY;
            break;
          }
        }

        resultvalue = value1 / value2;

        if (value1 != float(0) &&
            _QLookAhead && *_QLookAhead == '^')
        {
          float val1a = value1 + _Epsilon;
          float val2a = value2 + _Epsilon;
          float val1b = value1 - _Epsilon;
          float val2b = value2 - _Epsilon;

          bool arg2ok = ::IsLong(arglist[2]);
          bool arg3ok = ::IsLong(arglist[3]);
          bool val1aok = !arg2ok && floor(value1 + _Epsilon) == val1a;
          bool val1bok = !arg2ok && floor(value1) == val1b;
          bool val2aok = !arg3ok && floor(value2 + _Epsilon) == val2a;
          bool val2bok = !arg3ok && floor(value2) == val2b;

          if ((arg2ok && arg3ok) ||
              ((val1aok || val1bok) && (val2aok || val2bok)))
          {
            LongNumber val1;
            LongNumber val2;
            Float Tempf_;

            if (arg2ok && arg3ok)
            {
              val1 = ChrString(arglist[2]);
              val2 = ChrString(arglist[3]);
            }
            else if (val1aok)
            {
              Tempf_ = val1a;
              val1 = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);

              if (val2aok)
              {
                Tempf_ = val2a;
                val2 = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);
              }
              else
              {
                Tempf_ = val2b;
                val2 = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);
              }
            }
            else if (val1bok)
            {
              Tempf_ = val1b;
              val1 = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);

              if (val2aok)
              {
                Tempf_ = val2a;
                val2 = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);
              }
              else
              {
                Tempf_ = val2b;
                val2 = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);
              }
            }

            val1 %= 2L;
            val2 %= 2L;

            if (!val2.AtZero())
              _OddRoot = val1.AtZero() ? 2:1;
            else
              _OddRoot = 0;
          }
        }
      }
      else
      {
        if (_PlottingGraph)
        {
          int dsign_ = (value2 < float(0)) ? -1:1;
          int nsign_ = (value1 < float(0)) ? -1:1;
          nsign_ *= dsign_;

          if (nsign_ > 0)
            resultvalue = Float::vINFINITY;
          else
            resultvalue = -Float::vINFINITY;
        }
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::Modulus:
      intval1 = Ulong(floor(value1));
      intval2 = Ulong(floor(value2));
      dividebyzero = !intval2;
      if (!dividebyzero)
        resultvalue = intval1 % intval2;
      else
      {
        if (_PlottingGraph)
          resultvalue = Float::vNAN;
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::Logical_And:
      intval1 = Ulong(floor(value1));
      intval2 = Ulong(floor(value2));
      resultvalue = intval1 && intval2;
      break;
    case Mcalc_OperatorType::Logical_Or:
      intval1 = Ulong(floor(value1));
      intval2 = Ulong(floor(value2));
      resultvalue = intval1 || intval2;
      break;
    case Mcalc_OperatorType::Exponent:
      if (value2 == float(0))
        resultvalue = 1L;
      else if (::IsLong(arglist[3]))
      {
        ExpVal_ = atol(arglist[3]);
        resultvalue = FloatCalculator::IntPow(value1, ExpVal_);
      }
      else
      {
        if (value1 < float(0))
        {
          if (_OddRoot)
          {
            resultvalue = pow(-value1,value2);
            if (_OddRoot == 1)
              resultvalue = -resultvalue;
          }
          else
            resultvalue = pow(value1,value2);
        }
        else
          resultvalue = pow(value1,value2);
      }
      break;
    case Mcalc_OperatorType::Square:
      resultvalue = pow(value2,2);
      break;
    case Mcalc_OperatorType::SquareRoot:
      resultvalue = pow(value2,.5);
      break;
    case Mcalc_OperatorType::Cube:
      resultvalue = pow(value2,3);
      break;
    case Mcalc_OperatorType::CubeRoot:
      if (value2 < float(0))
        resultvalue = -pow(-value2,(1.0/3));
      else
        resultvalue = pow(value2,(1.0/3));
      break;
    case Mcalc_OperatorType::HyperSine:
      resultvalue = sinh(value2);
      break;
    case Mcalc_OperatorType::HyperCosine:
      resultvalue = cosh(value2);
      break;
    case Mcalc_OperatorType::HyperTangent:
      resultvalue = tanh(value2);
      break;
    case Mcalc_OperatorType::NaturalExp:
      resultvalue = exp(value2);
      break;
    case Mcalc_OperatorType::NaturalLog:
      resultvalue = log(value2);
      break;
    case Mcalc_OperatorType::Log10:
      resultvalue = log10(value2);
      break;
    case Mcalc_OperatorType::Ceiling:
      resultvalue = ceil(value2);
      break;
    case Mcalc_OperatorType::Floor:
      resultvalue = floor(value2);
      break;
    case Mcalc_OperatorType::AbsoluteValue:
      resultvalue = fabs(value2);
      break;
    case Mcalc_OperatorType::FractionPart:
      resultvalue = modf(value2, &intp_);
      break;
    case Mcalc_OperatorType::IntegerPart:
      modf(value2, &intp_);
      resultvalue = intp_;
      break;
    case Mcalc_OperatorType::Percent:
      resultvalue = value1 / value2;
      break;
    case Mcalc_OperatorType::LessThan:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = float(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 < value2;
      break;
    case Mcalc_OperatorType::GreaterThan:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = float(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 > value2;
      break;
    case Mcalc_OperatorType::LessOrEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = float(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 <= value2;
      break;
    case Mcalc_OperatorType::GreaterOrEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = float(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 >= value2;
      break;
    case Mcalc_OperatorType::Equal:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = float(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 == value2;
      break;
    case Mcalc_OperatorType::NotEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = float(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 != value2;
      break;
    case Mcalc_OperatorType::Round:
      resultvalue = Float(value2).Round();
      break;
    case Mcalc_OperatorType::Factorial:
      resultvalue = Discrete<Ulong>::Factorial((Ulong)value2);
      break;
    case Mcalc_OperatorType::Permutation:
      resultvalue = Discrete<Ulong>::Permutation((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::Combination:
      resultvalue = Discrete<Ulong>::Combination((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::CombineWithRep:
      resultvalue = Discrete<Ulong>::CombinationWithRep((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::PermuteWithRep:
      resultvalue = Discrete<Ulong>::PermutationWithRep((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::Assignment:
      resultvalue = value2;
      break;
    case Mcalc_OperatorType::Random:
      IntVal_ = atoi(arglist[3]);
      resultvalue = GiveRandom(IntVal_);
      break;
    case Mcalc_OperatorType::Exclusive_Or:
      intval1 = Ulong(floor(value1));
      intval2 = Ulong(floor(value2));
      resultvalue = !(intval1 && intval2) && (intval1 || intval2);
      break;
    case Mcalc_OperatorType::Inverse:
      dividebyzero = value2 == float(0);
      if (!dividebyzero)
      {
        if (_PlottingGraph && !_PlotAllPoints)
        {
          Tol_ = Float(DEFAULT_PRECSTR_GRAPH);
          tval_[0] = -Tol_;
          tval_[1] = Tol_;

          if (tval_[0] < value2 && value2 < tval_[1])
          {
            if (value2 < float(0))
              resultvalue = -Float::vINFINITY;
            else
              resultvalue = Float::vINFINITY;
            break;
          }
        }

        resultvalue = float(1 / value2);

        if (_QLookAhead && *_QLookAhead == '^')
        {
          float val2a = value2 + _Epsilon;
          float val2b = value2 - _Epsilon;

          bool arg3ok = ::IsLong(arglist[3]);
          bool val2aok = !arg3ok && ::floor(value2 + _Epsilon) == val2a;
          bool val2bok = !arg3ok && ::floor(value2) == val2b;

          if (arg3ok || val2aok || val2bok)
          {
            LongNumber val2;
            Float Tempf_;

            if (arg3ok)
              val2 = ChrString(arglist[3]);
            else if (val2aok)
            {
              Tempf_ = val2a;
              val2 = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);
            }
            else
            {
              Tempf_ = val2b;
              val2 = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);
            }

            val2 %= 2L;

            if (!val2.AtZero())
              _OddRoot = 1;
            else
              _OddRoot = 0;
          }
        }
      }
      else
      {
        if (_PlottingGraph)
        {
          if (value2 < float(0))
            resultvalue = -Float::vINFINITY;
          else
            resultvalue = Float::vINFINITY;
        }
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::ScientificNotation:
      if (::IsNumber(arglist[3], TRUE, FALSE, FALSE, 10))
      {
        ExpVal_ = atol(arglist[3]);
        value2 = pow(10, ExpVal_);
        resultvalue = value1 * value2;
      }
      break;
  }

  TrigOps_ =
  (optype == Mcalc_OperatorType::ArcSine)          ||
  (optype == Mcalc_OperatorType::ArcCosine)        ||
  (optype == Mcalc_OperatorType::ArcTangent);

  if (GetAngleUnit() == DEGREE && TrigOps_)
  {
    resultvalue /= Float::vPI;
    resultvalue *= 180L;
  }

  if (!dividebyzero)
  {
    if (Float::IsInfinite(resultvalue))
    {
      if (resultvalue < float(0))
        SetLiteralStr("-Inf");
      else
        SetLiteralStr("+Inf");
      ResetAnswerMode();
      SetAnswerMode(LITERAL_MODE, TRUE);
      clearvalue(resultptr);
    }
    else if (Float::IsNan(resultvalue))
    {
      SetLiteralStr("Nan");
      ResetAnswerMode();
      SetAnswerMode(LITERAL_MODE, TRUE);
      clearvalue(resultptr);
    }
    else
    {
      // converting decimal result value to array digit elements
      float absresultval = resultvalue;
      AbsoluteValue(absresultval);

      Float Tempf_(absresultval);
      buffer = Tempf_.ToString(Float::TYPE_FLOAT_PRECISION);
      len = buffer.strlen();

      allocatevalue(resultptr, len);
      resultptr->sign = (resultvalue < 0) ? Mcalc_SignType::Minus:Mcalc_SignType::Plus;
      strcpy(resultptr->value, buffer.c_str());
    }
  }
  else if (!_PlottingGraph)
  {
    ChrString Numer_, Denom_;
    Float Temp_;

    if (optype == Mcalc_OperatorType::Divide ||
        optype == Mcalc_OperatorType::Inverse)
    {
      if (optype == Mcalc_OperatorType::Inverse)
        Numer_ = "1";
      else
      {
        Temp_ = value1;
        Numer_ = Temp_.ToString(Float::TYPE_FLOAT_PRECISION);
      }

      Temp_ = value2;
      Denom_ = Temp_.ToString(Float::TYPE_FLOAT_PRECISION);
      Numer_ += ChrString("/") + Denom_;
    }
    else if (optype == Mcalc_OperatorType::Modulus)
    {
      Temp_ = intval1;
      Numer_ = Temp_.ToString(Float::TYPE_FLOAT_PRECISION);
      Temp_ = intval2;
      Denom_ = Temp_.ToString(Float::TYPE_FLOAT_PRECISION);
      Numer_ += ChrString("@") + Denom_;
    }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_DIVISION_BY_ZERO,
                        ERRMSG_DIVISION_BY_ZERO, Numer_.c_str());
    CloseOutputFile();
  }

  return resultvalue;
}

/*****************************************************************************/
const float& FloatCalculator::GetResult() const
{
  return resultvalue;
}

/*****************************************************************************/
const float& FloatCalculator::GetVariable(int Index_) const
{
  static float Dummy_ = 0;
  if (Index_ >= MAXVARS || Index_ < 0)
    return Dummy_;

  return _vars[Index_];
}

/*****************************************************************************/
size_t FloatCalculator::GetVariableLength(int Index_) const
{
  if (Index_ < 0)
    return 0;

  if (IsStringVar(Index_))
    return GetStrVarLen(Index_);
  else if (Index_ >= MAXVARS)
    return 0;

  char* ValStr_ = NULL;
  size_t len = strlen(M_E_STRING) / 2;
  size_t prec = len / 2;

  ValStr_ = FloatToStr(_vars[Index_], ValStr_, len, prec);
  len = strlen(ValStr_);
  MemMatrix::Matrix().Deallocate(ValStr_);
  ValStr_ = NULL;
  return len;
}

/*****************************************************************************/
void FloatCalculator::SetVariable(int Index_, const float& Value1_, int Transfer_)
{
  int x;
  int AssignType_;
  float Value_;
  bool AltBase_ = StringAssignment() && AnswerMode() == ALTBASE_MODE;
  LongNumber LongValue_;

  if (Transfer_ || !AltBase_)
  {
    Value_ = Value1_;
    AssignType_ = ASSIGN_ONLY;
  }
  else
  {
    ChrString ValStr_ = GetStrResult();
    LongValue_ = ValStr_;
    Value_ = strtol(ValStr_.c_str(), NULL, 0);
  }

  if (!Transfer_)
    GetIndex(x, AssignType_);

  if (!AssignType_ || Index_ >= MAXVARS)
    return;

  if (Index_ < 0)
  {
    resultvalue = Value_;
    return;
  }

  if (AssignType_ == ASSIGN_ONLY ||
       (AltBase_ && AssignType_ == STRING_ASSIGN))
  {
    _vars[Index_] = Value_;

    if (Transfer_)
      LongValue_ = _TransferValue;
  }
  else if (AssignType_ == ASSIGN_PLUS)
    _vars[Index_] += Value_;
  else if (AssignType_ == ASSIGN_MINUS)
    _vars[Index_] -= Value_;
  else if (AssignType_ == ASSIGN_MULT)
    _vars[Index_] *= Value_;
  else if (AssignType_ == ASSIGN_DIV)
    _vars[Index_] /= Value_;
  else if (AssignType_ == ASSIGN_MOD)
    _vars[Index_] = ((int)_vars[Index_]) % ((int)Value_);
  else if (AssignType_ == ASSIGN_AND && AltBase_)
  {
    char* BaseStr_ = NULL;
    int AssignDone_ = FALSE;

    if (IsStringVar(Index_))
    {
      BaseStr_ = (char*)MemMatrix::Matrix().Allocate(GetStrVarLen(Index_) + 1);
      CopyStrVar(BaseStr_, Index_);

      if ((BaseStr_ && (IsIntHex(BaseStr_) || IsIntOct(BaseStr_))) &&
          (LongValue_.ConvertedFromBase() == 16 || LongValue_.ConvertedFromBase() == 8))
      {
        LongNumber SrceVal_ = Float(_vars[Index_]);
        ChrString TempStr_ = BitwiseOps(SrceVal_, LongValue_, Mcalc_OperatorType::Logical_And);
        float ResultVal_ = strtol(TempStr_.c_str(), NULL, 0);
        _vars[Index_] = ResultVal_;
        AssignDone_ = TRUE;
      }

      MemMatrix::Matrix().Deallocate(BaseStr_);
    }

    if (!AssignDone_)
      _vars[Index_] = Integer(_vars[Index_] != float(0) && !LongValue_.AtZero());
  }
  else if (AssignType_ == ASSIGN_OR && AltBase_)
  {
    char* BaseStr_ = NULL;
    int AssignDone_ = FALSE;

    if (IsStringVar(Index_))
    {
      BaseStr_ = (char*)MemMatrix::Matrix().Allocate(GetStrVarLen(Index_) + 1);
      CopyStrVar(BaseStr_, Index_);

      if ((BaseStr_ && (IsIntHex(BaseStr_) || IsIntOct(BaseStr_))) &&
          (LongValue_.ConvertedFromBase() == 16 || LongValue_.ConvertedFromBase() == 8))
      {
        LongNumber SrceVal_ = Float(_vars[Index_]);
        ChrString TempStr_ = BitwiseOps(SrceVal_, LongValue_, Mcalc_OperatorType::Logical_Or);
        float ResultVal_ = strtol(TempStr_.c_str(), NULL, 0);
        _vars[Index_] = ResultVal_;
        AssignDone_ = TRUE;
      }

      MemMatrix::Matrix().Deallocate(BaseStr_);
    }

    if (!AssignDone_)
      _vars[Index_] = Integer(_vars[Index_] != float(0) && !LongValue_.AtZero());
  }

  if (Transfer_ || AltBase_)
    if (LongValue_.ConvertedFromBase() == 16)
    {
      LongValue_ = Float(_vars[Index_]);
      SetStrVariable(Index_, LongValue_.ToString(16));
    }
    else if (LongValue_.ConvertedFromBase() == 8)
    {
      LongValue_ = Float(_vars[Index_]);
      SetStrVariable(Index_, LongValue_.ToString(8));
    }

  resultvalue = _vars[Index_];
}

/*****************************************************************************/
int FloatCalculator::GiveType() const
{
  return FLOAT;
}

/*****************************************************************************/
void FloatCalculator::SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_)
{
  if (VarIndex_ < 0)
    return;

  if (IsStringVar(VarIndex_))
  {
    CopyStrVar(&ArgStr_[StrIndex_], VarIndex_);
    StrIndex_ += GetStrVarLen(VarIndex_);
  }
  else if (VarIndex_ < MAXVARS)
  {
    char* ValStr_ = NULL;
    int len = strlen(M_E_STRING) / 2;
    int prec = len / 2;

    ValStr_ = FloatToStr(_vars[VarIndex_], ValStr_, len, prec);
    ValStr_ = RemovePadding(ValStr_, " \t\n");
    strcpy(&ArgStr_[StrIndex_], ValStr_);
    StrIndex_ += strlen(ValStr_);
    MemMatrix::Matrix().Deallocate(ValStr_);
    ValStr_ = NULL;
  }
}

/*****************************************************************************/
/*****************************************************************************/
DoubleCalculator::DoubleCalculator():
resultvalue(0),
_Epsilon(DEFAULT_DOUBLE_EPSILON)
{
  #if CALCLIB_DEBUG2
    fputs("Entering: DoubleCalculator constructor\n", logfile);
  #endif

  int i;
  for (i = 0; i < MAXVARS; i++)
    _vars[i] = 0;

  _IntCoords =
  (IntCoords<double>**)MemMatrix::Matrix().Callocate(MAX_INT_COORDS * sizeof(IntCoords<double>*));
  for (i = 0; i < MAX_INT_COORDS; i++)
    _IntCoords[i] = new IntCoords<double>();

  #if CALCLIB_DEBUG2
    fputs("Leaving: DoubleCalculator constructor\n", logfile);
  #endif
}

/*****************************************************************************/
DoubleCalculator::~DoubleCalculator()
{
  if (_IntCoords)
  {
    int i;
    for (i = 0; i < MAX_INT_COORDS; i++)
      delete _IntCoords[i];

    MemMatrix::Matrix().Deallocate(_IntCoords);
    _IntCoords = NULL;
  }
}

/*****************************************************************************/
void DoubleCalculator::SetParent(CalculatorBase* Ptr_)
{
  _ParentPtr = Ptr_;
  _CalcIndex = CalculatorBase::DOUBLE;
}

/****************************************************************************/
double DoubleCalculator::IntPow(double Value_, long Exp_)
{
  if (!Exp_)
    return double(1);

  if (Exp_ == 1L)
    return Value_;

  double Answer_(Value_);
  long Aexp_ = labs(Exp_);
  long i;

  for (i = 1; i < Aexp_; i++)
    Answer_ *= Value_;

  if (Exp_ < 0)
    Answer_ = 1L / Answer_;

  return Answer_;
}

/*****************************************************************************/
ChrString DoubleCalculator::ToString(int Index_)
{
  ChrString DestStr_;
  Double FpObj_;

  if (Index_ >= 0 && IsStringVar(Index_))
  {
    CopyStrVar(DestStr_, Index_);
    return DestStr_;
  }

  if (Index_ < 0)
    FpObj_ = resultvalue;
  else
    FpObj_ = _vars[Index_];

  return FpObj_.ToString(Double::TYPE_DOUBLE_PRECISION);
}

/*****************************************************************************/
ChrString DoubleCalculator::FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_)
{
  const char* ParamStr_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Double HiTestPt_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Double LoTestPt_ = HiTestPt_;
  Double Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  Double Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
  Double Tol_ = ChrString(ParamStr_);
  Double Incr_ = ChrString(DEFAULT_PRECSTR_APPROXZERO);

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  ChrString ValStr_;
  Double LoResult1_;
  Double* LoResult2_ = NULL;
  Double HiResult1_;
  Double* HiResult2_ = NULL;
  Double Low_;
  Double High_;
  Double Testx_;
  Double ZeroPt_;
  int Sign_;
  int ZeroCrossing_;
  ZeroFound_ = FALSE;

  if (HasErrors())
    ClearError();

  while ((HiTestPt_ <= Upper_ || LoTestPt_ >= Lower_) && !HasErrors())
  {
    if (LoTestPt_ >= Lower_)
    {
      ValStr_ = LoTestPt_.ToString(Double::TYPE_DOUBLE_PRECISION);
      FncVal_ = ChrString("(") + ValStr_ + ChrString(")");
      LoResult1_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (LoResult1_.FloatValue() == double(0))
      {
        Testx_ = ValStr_;
        ZeroFound_ = TRUE;
        break;
      }
      else if (!LoResult2_)
        LoResult2_ = new Double(LoResult1_);
      else
      {
        ZeroCrossing_ = LoResult1_.FloatValue() < double(0) &&
                        LoResult2_->FloatValue() > double(0) ||
                        LoResult1_.FloatValue() > double(0) &&
                        LoResult2_->FloatValue() < double(0);

        if (ZeroCrossing_)
        {
          Low_ = LoTestPt_;
          High_ = LoTestPt_ + Incr_;
          ValStr_ = Low_.ToString(Double::TYPE_DOUBLE_PRECISION);
          ParamArray_[3]->_Param->SetAnswer(ValStr_.c_str());
          ValStr_ = High_.ToString(Double::TYPE_DOUBLE_PRECISION);
          ParamArray_[4]->_Param->SetAnswer(ValStr_.c_str());
          Testx_ = (High_ + Low_) / 2L;
          break;
        }
        else
        {
          *LoResult2_ = LoResult1_;
          LoTestPt_ -= Incr_;
        }
      }
    }

    if (HiTestPt_ <= Upper_)
    {
      ValStr_ = HiTestPt_.ToString(Double::TYPE_DOUBLE_PRECISION);
      FncVal_ = ChrString("(") + ValStr_ + ChrString(")");
      HiResult1_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (HiResult1_.FloatValue() == double(0))
      {
        Testx_ = ValStr_;
        ZeroFound_ = TRUE;
        break;
      }
      else if (!HiResult2_)
        HiResult2_ = new Double(HiResult1_);
      else
      {
        ZeroCrossing_ = HiResult1_.FloatValue() < double(0) &&
                        HiResult2_->FloatValue() > double(0) ||
                        HiResult1_.FloatValue() > double(0) &&
                        HiResult2_->FloatValue() < double(0);

        if (ZeroCrossing_)
        {
          Low_ = HiTestPt_ - Incr_;
          High_ = HiTestPt_;
          ValStr_ = Low_.ToString(Double::TYPE_DOUBLE_PRECISION);
          ParamArray_[3]->_Param->SetAnswer(ValStr_.c_str());
          ValStr_ = High_.ToString(Double::TYPE_DOUBLE_PRECISION);
          ParamArray_[4]->_Param->SetAnswer(ValStr_.c_str());
          Testx_ = (High_ + Low_) / 2L;
          break;
        }
        else
        {
          *HiResult2_ = HiResult1_;
          HiTestPt_ += Incr_;
        }
      }
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  Double xTol_ = Testx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete LoResult2_;
  delete HiResult2_;

  if (HasErrors())
    ClearError();

  LongNumber Answer_ = Testx_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString DoubleCalculator::FindDerivative(int NumParams_, FncParameter** ParamArray_)
{
  if (!Double::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      (NumParams_ >= 4 &&
       !Double::IsValue(ParamArray_[3]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "DERIV");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  ChrString NewExprStr_ = ParamArray_[2]->_Param->GetAnswer();
  Double Value_ = NewExprStr_;
  NewExprStr_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Buffer_;
  Double Prec_ = NewExprStr_;
  Double HalfPrec_ = Prec_ / 2L;
  HalfPrec_ *= Double(.8);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    PlotFunction(2, ParamArray_, Value_.ToString(Double::TYPE_DOUBLE_PRECISION));
  }

  Double HighFncRet_;
  Double LowFncRet_;
  Double HighVal_ = Value_ + HalfPrec_;
  Double LowVal_ = Value_ - HalfPrec_;

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + LowVal_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  LowFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + HighVal_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  HighFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  HighFncRet_ -= LowFncRet_;
  HighVal_ -= LowVal_;

  Double Slope_ = HighFncRet_;
  Slope_ /= HighVal_;
  PopSubExprStack();

  long Exp_ = 0;
  Double xTol_ = Slope_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_DERIV;

  LongNumber Answer_ = Slope_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString DoubleCalculator::FindDerivative2(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      (NumParams_ >= 4 &&
       !LongNumber::IsValue(ParamArray_[3]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "DERIV2");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  ChrString NewExprStr_ = ParamArray_[2]->_Param->GetAnswer();
  Double Value_ = NewExprStr_;
  NewExprStr_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Buffer_;
  Double Prec_ = NewExprStr_;
  Double HalfPrec_ = Prec_ / 2L;
  HalfPrec_ *= Double(.8);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    ChrString TempStr_ = ChrString("DERIV(") + ExprStr_;
    TempStr_ += ChrString(", ") + VarName_;
    TempStr_ += ChrString(", ") + VarName_;
    TempStr_ += ChrString("2)");

    FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(2 * sizeof(FncParameter*));
    Params_[0] = new FncParameter();
    Params_[0]->_Param = new SubExpression();
    Params_[0]->_Param->SetExpression(TempStr_.c_str());
    Params_[0]->_ShouldEval = FALSE;
    TempStr_ = VarName_ + ChrString("2");
    Params_[1] = new FncParameter();
    Params_[1]->_Param = new SubExpression();
    Params_[1]->_Param->SetExpression(TempStr_.c_str());
    Params_[1]->_ShouldEval = FALSE;

    PlotFunction(2, Params_, Value_.ToString(Double::TYPE_DOUBLE_PRECISION));

    int x;
    for (x = 0; x < 2; x++)
      delete Params_[x];

    MemMatrix::Matrix().Deallocate(Params_);
  }

  Double HighFncRet_;
  Double LowFncRet_;
  Double MidTermRet_;
  Double HighVal_ = Value_ + HalfPrec_;
  Double LowVal_ = Value_ - HalfPrec_;

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + LowVal_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  LowFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + HighVal_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  HighFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + Value_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  MidTermRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  MidTermRet_ *= 2L;

  HighFncRet_ -= MidTermRet_;
  HighFncRet_ += LowFncRet_;
  HalfPrec_ *= HalfPrec_;

  Double Slope_ = HighFncRet_;
  Slope_ /= HalfPrec_;
  PopSubExprStack();

  long Exp_ = 0;
  Double xTol_ = Slope_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_DERIV;

  LongNumber Answer_ = Slope_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
void DoubleCalculator::PrintInfNanValues(FILE* Fout_, const Double& Answer_, double fval)
{
  if (Answer_.IsInfinite())
  {
    if (Answer_.FloatValue() < double(0))
      fprintf(Fout_, DEFAULT_DOUBLENAN_FORMAT, fval, "-inf");
    else
      fprintf(Fout_, DEFAULT_DOUBLENAN_FORMAT, fval, "+inf");
  }
  else if (Answer_.IsNan()) {
    fprintf(Fout_, DEFAULT_DOUBLENAN_FORMAT, fval, "nan"); }
  else
  {
    double Sd = Answer_;
    fprintf(Fout_, DEFAULT_DOUBLE_FORMAT, fval, Sd);
  }
}

/*****************************************************************************/
ChrString DoubleCalculator::FindIntersections(int NumParams_, FncParameter** ParamArray_)
{
  int x;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;
  ChrString FirstRt_;
  ChrString RetStr_;
  ChrString FncVal_;
  char CurrentSuffix_ = _PlotFileSuffix;
  Double Yresult_;
  Double Xresult_;
  Double Upper_;
  Double Lower_;
  Double MidPt_;
  Double RadVal_;
  FncParameter** Params_;

  Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_;
  ChrString NewExpr_;

  if (NumParams_ == 5 || NumParams_ == 3)
  {
    NewExpr_ = "(";
    NewExpr_ += ParamArray_[0]->_Param->GetExpression();
    NewExpr_ += ") - (";
    NewExpr_ += ParamArray_[1]->_Param->GetExpression();
    NewExpr_ += ")";
    VarName_ = ParamArray_[2]->_Param->GetExpression();
    CalculatorBase::SetGraphType(Mcalc_GraphType::INTERSECT2);
    _IntExpr = ParamArray_[0]->_Param->GetExpression();
    _IntExpr += " , ";
    _IntExpr += ParamArray_[1]->_Param->GetExpression();
  }
  else if (NumParams_ == 4 || NumParams_ == 2)
  {
    NewExpr_ = ParamArray_[0]->_Param->GetExpression();
    VarName_ = ParamArray_[1]->_Param->GetExpression();
    _IntExpr = NewExpr_;
  }

  Params_[0] = new FncParameter();
  Params_[0]->_Param = new SubExpression();
  Params_[0]->_Param->SetExpression(NewExpr_.c_str());
  Params_[1] = (NumParams_ == 4 || NumParams_ == 2) ?
                   new FncParameter(*ParamArray_[1]):
                   new FncParameter(*ParamArray_[2]);
  Params_[2] = new FncParameter();
  Params_[2]->_Param = new SubExpression();
  Params_[3] = Params_[4] = NULL;

  if (NumParams_ == 2 || NumParams_ == 3)
    Params_[2]->_Param->SetAnswer("0");
  else
  {
    if (NumParams_ == 4)
    {
      Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
      Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;

      if (MidPt_.IsNan() || MidPt_.IsInfinite())
        Params_[2]->_Param->SetAnswer("0");
      else
        Params_[2]->_Param->SetAnswer(MidPt_.ToString().c_str());
    }
    else if (NumParams_ == 5)
    {
      Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;

      if (MidPt_.IsNan() || MidPt_.IsInfinite())
        Params_[2]->_Param->SetAnswer("0");
      else
        Params_[2]->_Param->SetAnswer(MidPt_.ToString().c_str());
    }
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);

  for (_IntsFound = x = 0; x < 3; x++)
  {
    RetStr_ = FindRoot(3, Params_);

    if (!HasErrors())
    {
      FncVal_ = ChrString("(") + RetStr_ + ChrString(")");
      Xresult_ = RetStr_;
      Yresult_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (!_IntsFound ||
          (_IntsFound == 1 && _IntCoords[0]->X != Xresult_.FloatValue()) ||
          (_IntsFound == 2 && _IntCoords[1]->X != Xresult_.FloatValue() &&
                              _IntCoords[0]->X != Xresult_.FloatValue()))
      {
        if (!_IntsFound)
          FirstRt_ = Xresult_.ToString(Double::TYPE_DOUBLE_PRECISION);

        _IntCoords[_IntsFound]->X = Xresult_.FloatValue();
        _IntCoords[_IntsFound]->Y = Yresult_.FloatValue();
        ++_IntsFound;
      }
    }

    if (x == 0)
    {
      if (NumParams_ == 4 || NumParams_ == 5)
      {
        if (NumParams_ == 4)
          Params_[2]->_Param->SetAnswer(ParamArray_[2]->_Param->GetAnswer());
        else
          Params_[2]->_Param->SetAnswer(ParamArray_[3]->_Param->GetAnswer());
      }
      else
        Params_[2]->_Param->SetAnswer("-5");
    }
    else if (x == 1)
    {
      if (NumParams_ == 4 || NumParams_ == 5)
      {
        if (NumParams_ == 4)
          Params_[2]->_Param->SetAnswer(ParamArray_[3]->_Param->GetAnswer());
        else
          Params_[2]->_Param->SetAnswer(ParamArray_[4]->_Param->GetAnswer());
      }
      else
        Params_[2]->_Param->SetAnswer("5");
    }

    if (HasErrors())
      ClearError();
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();

  if (PlotGraph_)
  {
    int RangeSpecified_ = NumParams_ == 4 || NumParams_ == 5;
    if (_IntsFound)
      RetStr_ = FirstRt_;
    else
      RetStr_ = "0";

    if (RangeSpecified_)
    {
      if (NumParams_ == 5)
      {
        Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
        Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      }
      else if (NumParams_ == 4)
      {
        Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
        Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      }

      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;
      RadVal_ = Upper_ - MidPt_;
      AbsoluteValue(RadVal_);
    }

    if (NumParams_ == 5 || NumParams_ == 3)
    {
      // Plotting First Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[2]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(Double::TYPE_DOUBLE_PRECISION),
                     RadVal_.ToString(Double::TYPE_DOUBLE_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);

      // Plotting Second Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[1]);
      Params_[1] = new FncParameter(*ParamArray_[2]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(Double::TYPE_DOUBLE_PRECISION),
                     RadVal_.ToString(Double::TYPE_DOUBLE_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);
    }
    else if (NumParams_ == 4 || NumParams_ == 2)
    {
      // Plotting First Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(Double::TYPE_DOUBLE_PRECISION),
                     RadVal_.ToString(Double::TYPE_DOUBLE_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);
    }

    SaveCurrentGraphName(CurrentSuffix_);
  }

  for (x = 0; x < 5; x++)
    delete Params_[x];
  MemMatrix::Matrix().Deallocate(Params_);

  char Buffer_[64];
  char Suf_ = 'z';
  sprintf(Buffer_, "mcalc_graph%d%c.out", SessionNum(), Suf_);
  FILE* Fout_ = fopen(Buffer_, "w");
  RetStr_ = "\n";

  if (Fout_)
  {
    if (_IntsFound)
      for (x = 0; x < _IntsFound; x++)
      {
        sprintf(Buffer_, DEFAULT_DOUBLE_FORMAT, _IntCoords[x]->X, _IntCoords[x]->Y);
        fprintf(Fout_, "%s", Buffer_);
        RetStr_ += Buffer_;
      }

    fclose(Fout_);
  }

  SetHasAnswer(RetStr_.strlen());

  if (RetStr_.strlen())
  {
    RetStr_ = ChrString("\"") + RetStr_ + ChrString("\"");
    SetLiteralStr(RetStr_.c_str());
  }

  return RetStr_;
}

/*****************************************************************************/
ChrString DoubleCalculator::PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                         const ChrString& MidPtStr_, ChrString Radius_, ChrString* Solution_)
{
  if (NumParams_ < 2 || (NumParams_ >= 3 && !LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "GRAPH");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString Caption_;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();

  if (ExprStr_.find(VarName_).failed())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_GRAPHVARNAME,
                        ERRMSG_INVALID_GRAPHVARNAME, "GRAPH");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  char Fname_[32];
  sprintf(Fname_, "mcalc_graph%d%c.out", SessionNum(), PlotFileSuffix());
  FILE* Fout_ = fopen(Fname_, "w");

  Double Mid_ = (NumParams_ >= 3) ? ParamArray_[2]->_Param->GetAnswer():MidPtStr_;
  Double RadiusVal_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Radius_;
  Double Lower_ = Mid_ - RadiusVal_;
  Double Upper_ = Mid_ + RadiusVal_;

  ChrString AnsStr_;
  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(4 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  AnsStr_ = Lower_.ToString(Double::TYPE_DOUBLE_PRECISION);
  Params_[2] = new FncParameter();
  Params_[2]->_Param = new SubExpression();
  Params_[2]->_Param->SetAnswer(AnsStr_.c_str());
  Params_[2]->_ShouldEval = FALSE;
  AnsStr_ = Upper_.ToString(Double::TYPE_DOUBLE_PRECISION);
  Params_[3] = new FncParameter();
  Params_[3]->_Param = new SubExpression();
  Params_[3]->_Param->SetAnswer(AnsStr_.c_str());
  Params_[3]->_ShouldEval = FALSE;
  long Ints_ = FindNumIntervals(4, Params_);

  Double Ht = (Upper_ - Lower_) / Ints_;
  Double Xi;
  Double Answer_;
  double Sd;
  double Htval_ = Ht;
  double Lowerval_ = Lower_;
  double fval;

  if (Solution_)
    Mid_ = *Solution_;

  if (_GraphType == Mcalc_GraphType::EXTREMA)
  {
    if (_ExtremaEnd < 0)
      Caption_ = ChrString("FMIN( ") + ExprStr_ + ChrString(" )");
    else
      Caption_ = ChrString("FMAX( ") + ExprStr_ + ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::DERIVATIVE)
  {
    ChrString DerivPt_ = ParamArray_[2]->_Param->GetAnswer();
    Caption_ = ChrString("DERIV( ") + ExprStr_;
    Caption_ += ChrString(" , ") + DerivPt_;
    Caption_ += ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::INTEGRAL)
  {
    ChrString LowerIntPt_ = ParamArray_[2]->_Param->GetAnswer();
    ChrString UpperIntPt_ = ParamArray_[3]->_Param->GetAnswer();
    Caption_ = ChrString("INTEG( ") + ExprStr_;
    Caption_ += ChrString(" , ") + LowerIntPt_;
    Caption_ += ChrString(" , ") + UpperIntPt_;
    Caption_ += ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::ROOTSOLVE)
    Caption_ = ChrString("SOLVE( ") + ExprStr_ + ChrString(" )");
  else if (_GraphType == Mcalc_GraphType::INTERSECT1 ||
           _GraphType == Mcalc_GraphType::INTERSECT2)
    Caption_ = ChrString("INTERSECT( ") + _IntExpr + ChrString(" )");
  else
    Caption_ = ExprStr_;

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  _PlottingGraph = TRUE;
  fprintf(Fout_, "%s\n", Caption_.c_str());
  fprintf(Fout_, "%d\n", _GraphType);
  fprintf(Fout_, "%d\n", (_GraphType == Mcalc_GraphType::INTEGRAL) ? 1:0);

  fval = Mid_;
  ChrString FncVal_ = ChrString("(") + Mid_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    DoubleCalculator::PrintInfNanValues(Fout_, Answer_, fval);

  fval = Lower_;
  FncVal_ = ChrString("(") + Lower_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    DoubleCalculator::PrintInfNanValues(Fout_, Answer_, fval);

  long x;
  for (x = 1; x < Ints_; x++)
  {
    Xi = Double(Lowerval_ + Htval_ * x);
    fval = Xi;
    FncVal_ = ChrString("(") + Xi.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (HasErrors())
    {
      if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
        ClearError();
      else
        goto HandleError;
    }
    else
      DoubleCalculator::PrintInfNanValues(Fout_, Answer_, fval);
  }

  fval = Upper_;
  FncVal_ = ChrString("(") + Upper_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    DoubleCalculator::PrintInfNanValues(Fout_, Answer_, fval);

HandleError:
  fclose(Fout_);
  SetSuppressOutput(FALSE);
  PopSubExprStack();
  _PlottingGraph = FALSE;

  for (x = 0; x < 4; x++)
    delete Params_[x];

  MemMatrix::Matrix().Deallocate(Params_);
  return ChrString("0");
}

/*****************************************************************************/
ChrString DoubleCalculator::FindIntegral(int NumParams_, FncParameter** ParamArray_)
{
  if (!Double::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !Double::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 5 &&
       !Double::IsValue(ParamArray_[4]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "INTEG");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  long Ints_ = FindNumIntervals(NumParams_, ParamArray_);
  const char* Buffer_ = DEFAULT_PRECSTR_INTEG;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Double Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Double Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  const char* ParamStr_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetAnswer():Buffer_;
  Double Prec_ = strtod(ParamStr_, NULL);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    Double MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    Double RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
    PlotFunction(2, ParamArray_, MidPt_.ToString(Double::TYPE_DOUBLE_PRECISION),
                 RadVal_.ToString(Double::TYPE_DOUBLE_PRECISION));
  }

  Double Ht = (Upper_ - Lower_) / Ints_;
  Double Xi;
  Double Answer_;
  double Sd;
  double Htval_ = Ht;
  double Lowerval_ = Lower_;

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_ = ChrString("(") + Lower_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  Sd = Answer_;

  long x;
  for (x = 1; x < Ints_; x++)
  {
    Xi = Double(Lowerval_ + Htval_ * x);
    FncVal_ = ChrString("(") + Xi.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Answer_ *= 2L;
    Sd += Answer_;
  }

  FncVal_ = ChrString("(") + Upper_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  Sd += Answer_;
  Answer_ = Double(Htval_ * Sd / 2L);
  SetSuppressOutput(FALSE);
  PopSubExprStack();

  long Exp_ = 0;
  Double xTol_ = Answer_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_INTEG;

  LongNumber StrAnswer_ = Answer_;
  StrAnswer_.Round(Exp_);
  return StrAnswer_.ToString();
}

/*****************************************************************************/
ChrString DoubleCalculator::FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!Double::IsValue(ParamArray_[2]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Double Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Double Incr_ = DEFAULT_PRECVAL_NEWTCRIT;

  CreateNewSubExpr();
  double X1 = Guess_;
  X1 -= 10;
  Double Temp_ = X1;
  ChrString FncVal_ = ChrString("(") + Temp_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  double Y1 = Temp_;
  double CriteriaVal_;

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  Temp_ = X1;
  FncVal_ = Temp_.ToString(Double::TYPE_DOUBLE_PRECISION);
  Params_[2]->_Param->SetAnswer(FncVal_.c_str());
  Temp_ = FindDerivative(NumParams_, Params_);
  double Ypr = Temp_;
  Temp_ = FindDerivative2(NumParams_, Params_);
  double Ypr2 = Temp_;
  Double Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);

  int i;
  int max = int(20 / Incr_.FloatValue());
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 0; i < max; i++)
  {
    CriteriaVal_ = ::fabs((Y1 * Ypr2) / (Ypr * Ypr));

    if (CriteriaVal_ < 1)
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (CriteriaVal_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Temp_ = X1;
    Temp_ += Incr_;
    X1 = Temp_;
    FncVal_ = ChrString("(") + Temp_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y1 = Temp_;
    Temp_ = X1;
    FncVal_ = Temp_.ToString(Double::TYPE_DOUBLE_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Temp_ = FindDerivative(NumParams_, Params_);
    Ypr = Temp_;
    Temp_ = FindDerivative2(NumParams_, Params_);
    Ypr2 = Temp_;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  Double xTol_ = X1;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  if (HasErrors())
    ClearError();

  Temp_ = X1;
  LongNumber Answer_ = Temp_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString DoubleCalculator::NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!Double::IsValue(ParamArray_[2]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Double Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());

  CreateNewSubExpr();
  double X1 = Guess_;
  Double Temp_ = X1;
  ChrString FncVal_ = ChrString("(") + Temp_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  double Y1 = Temp_;
  double X2;
  double Y2;

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  Temp_ = X1;
  FncVal_ = Temp_.ToString(Double::TYPE_DOUBLE_PRECISION);
  Params_[2]->_Param->SetAnswer(FncVal_.c_str());
  Temp_ = FindDerivative(NumParams_, Params_);
  double Ypr = Temp_;
  double Diff_;
  Double Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);

  int i;
  int max = 200;
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 0; i < max; i++)
  {
    X2 = X1 - Y1 / Ypr;
    Temp_ = X2;
    FncVal_ = ChrString("(") + Temp_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y2 = Temp_;
    Diff_ = X2 - X1;
    Diff_ = ::fabs(Diff_);

    if (Diff_ < Tol_.FloatValue() &&
        Y2 < Tol_.FloatValue())
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (Diff_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Temp_ = X2;
    FncVal_ = Temp_.ToString(Double::TYPE_DOUBLE_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Temp_ = FindDerivative(NumParams_, Params_);
    Ypr = Temp_;
    X1 = X2;
    Y1 = Y2;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  Double xTol_ = X2;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  if (HasErrors())
    ClearError();

  Temp_ = X2;
  LongNumber Answer_ = Temp_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString DoubleCalculator::MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!Double::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !Double::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      !Double::IsValue(ParamArray_[4]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Double Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Double Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  Double Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + Lower_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Double LowerY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  FncVal_ = ChrString("(") + Upper_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Double UpperY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  FncVal_ = ChrString("(") + Guess_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
  Double GuessY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  double Guessval_ = Guess_;
  double Lowerval_ = Lower_;
  double Upperval_ = Upper_;
  double LowerYval_ = LowerY_;
  double UpperYval_ = UpperY_;
  double GuessYval_ = GuessY_;

  Double Temp_ = (UpperY_ - LowerY_) / (Upper_ - Lower_);
  double C1 = Temp_;
  double C2;
  double D1;
  double S;
  double Det_;
  double X4;
  double Y4;
  double Diff_;
  Double Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);
  long Sign_;

  int i;
  int max = 2000;
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 2; i < max; i++)
  {
    C2 = (GuessYval_ - UpperYval_) / (Guessval_ - Upperval_);
    D1 = (C2 - C1) / (Guessval_ - Lowerval_);
    S = C2 + D1 * (Guessval_ - Upperval_);

    Sign_ = (S >= double(0)) ? 1L:-1L;
    Det_ = ::pow(S, 2) - (4L * GuessYval_ * D1);
    SquareRoot(Det_);
    X4 = Guessval_ - ((2L * GuessYval_) / (S + Sign_ * Det_));

    Temp_ = X4;
    FncVal_ = ChrString("(") + Temp_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y4 = Temp_;
    Diff_ = X4 - Guessval_;
    Diff_ = ::fabs(Diff_);

    if (Diff_ < Tol_.FloatValue() &&
        Y4 < Tol_.FloatValue())
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (Diff_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Lowerval_ = Upperval_;
    Upperval_ = Guessval_;
    UpperYval_ = GuessYval_;
    Guessval_ = X4;
    GuessYval_ = Y4;
    C1 = C2;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  Double xTol_ = X4;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  if (HasErrors())
    ClearError();

  LongNumber Answer_ = Double(X4);
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString DoubleCalculator::FindRoot(int NumParams_, FncParameter** ParamArray_)
{
  int x;
  int Converge_;
  int CriteriaFound_;
  ChrString RetStr_;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;
  bool RangeSpecified_ = false;
  FncParameter** Params_;

  if (NumParams_ > 3)
  {
    RangeSpecified_ = true;
    RetStr_ = MullersMethod(NumParams_, ParamArray_, Converge_);

    if (!Converge_)
    {
      Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      Params_[2] = new FncParameter(*ParamArray_[2]);
      Params_[3] = new FncParameter(*ParamArray_[3]);
      Params_[4] = new FncParameter(*ParamArray_[4]);
      ChrString Answer_ = FindNewtonsCriteria(NumParams_, Params_, CriteriaFound_);

      if (CriteriaFound_)
      {
        Params_[2]->_Param->SetAnswer(Answer_.c_str());
        RetStr_ = NewtonsMethod(NumParams_, Params_, Converge_);

        if (Converge_)
        {
          for (x = 0; x < 5; x++)
            delete Params_[x];

          MemMatrix::Matrix().Deallocate(Params_);
          goto RootSolveDone;
        }
        else
        {
          RetStr_ = MullersMethod(NumParams_, Params_, Converge_);

          if (Converge_)
          {
            for (x = 0; x < 5; x++)
              delete Params_[x];

            MemMatrix::Matrix().Deallocate(Params_);
            goto RootSolveDone;
          }
        }
      }
    }
  }
  else
  {
    RetStr_ = NewtonsMethod(NumParams_, ParamArray_, Converge_);

    if (!Converge_)
    {
      Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      Params_[2] = new FncParameter(*ParamArray_[2]);
      Params_[3] = Params_[4] = NULL;
      ChrString Answer_ = FindNewtonsCriteria(NumParams_, Params_, CriteriaFound_);

      if (CriteriaFound_)
      {
        Params_[2]->_Param->SetAnswer(Answer_.c_str());
        RetStr_ = NewtonsMethod(NumParams_, Params_, Converge_);

        if (Converge_)
        {
          for (x = 0; x < 3; x++)
            delete Params_[x];

          MemMatrix::Matrix().Deallocate(Params_);
          goto RootSolveDone;
        }
      }

      Params_[3] = new FncParameter();
      Params_[4] = new FncParameter();
      Params_[3]->_Param = new SubExpression();
      Params_[4]->_Param = new SubExpression();
      Boolean ZeroFound_ = FALSE;
      Double Lower_ = ChrString(Params_[2]->_Param->GetAnswer());
      Double Upper_ = ChrString(Params_[2]->_Param->GetAnswer());
      NumParams_ += 2;
      Lower_ -= 20L;
      Upper_ += 20L;
      Answer_ = Lower_.ToString(Double::TYPE_DOUBLE_PRECISION);
      Params_[3]->_Param->SetAnswer(Answer_.c_str());
      Params_[3]->_ShouldEval = FALSE;
      Answer_ = Upper_.ToString(Double::TYPE_DOUBLE_PRECISION);
      Params_[4]->_Param->SetAnswer(Answer_.c_str());
      Params_[4]->_ShouldEval = FALSE;
      RetStr_ = MullersMethod(NumParams_, Params_, Converge_);

      if (!Converge_)
      {
        RetStr_ = FindApproxZero(NumParams_, Params_, ZeroFound_);
        if (!ZeroFound_)
          RetStr_ = MullersMethod(NumParams_, Params_, Converge_);
        else
          Converge_ = TRUE;
      }

      if (!Converge_)
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED,
                        ERRMSG_MULLERS_METHOD_FAILED, ParamArray_[0]->_Param->GetExpression());
        CloseOutputFile();
      }

      for (x = 0; x < 5; x++)
        delete Params_[x];

      MemMatrix::Matrix().Deallocate(Params_);
    }
  }

RootSolveDone:
  if (!Converge_)
  {
    if (RangeSpecified_)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED,
                      ERRMSG_MULLERS_METHOD_FAILED, ParamArray_[0]->_Param->GetExpression());
      CloseOutputFile();
    }
  }
  else if (PlotGraph_)
  {
    if (RangeSpecified_)
    {
      Double Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      Double Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      Double MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;
      Double RadVal_ = Upper_ - MidPt_;
      AbsoluteValue(RadVal_);
      PlotFunction(2, ParamArray_, MidPt_.ToString(Double::TYPE_DOUBLE_PRECISION),
                   RadVal_.ToString(Double::TYPE_DOUBLE_PRECISION), &RetStr_);
    }
    else
      PlotFunction(2, ParamArray_, RetStr_);
  }

  return RetStr_;
}

/*****************************************************************************/
ChrString DoubleCalculator::FindMinimum(int NumParams_, FncParameter** ParamArray_)
{
  if (!Double::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !Double::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 6 && !Double::IsValue(ParamArray_[5]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMIN");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Double Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Double Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  ChrString InclOpt_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetExpression():"";
  const char* ParamStr_ = (NumParams_ >= 6) ? ParamArray_[5]->_Param->GetAnswer():Buffer_;
  Double Tol_ = ChrString(ParamStr_);
  Double Incr_ = ChrString(DEFAULT_PRECSTR_FTEST);
  Double xTol_ = Upper_ - Lower_;

  if (xTol_.FloatValue() < Integer(0))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMIN");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }
  else
  {
    Double Ratio_ = xTol_ / 10L;
    xTol_ = Tol_;

    if (Ratio_ < Integer(1))
      Incr_ *= Ratio_;
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  Double Deriv1_;
  Double Deriv2_;
  Double Lowest_;
  Double Lowestx_;
  Double Testx_;
  Double Low_;
  Double High_;
  Double MidPt_;
  Double RadVal_;

  int ZeroCrossing_;
  int LowestInit_ = 0;
  int PrevInit_ = 0;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;

  if (PlotGraph_)
  {
    MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
  }

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  for (;Lower_ < Upper_; Lower_ += Incr_)
  {
    FncVal_ = Lower_.ToString(Double::TYPE_DOUBLE_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Deriv2_ = FindDerivative(3, Params_);

    if (!PrevInit_)
    {
      if (Deriv2_.FloatValue() != double(0))
      {
        Deriv1_ = Deriv2_;
        PrevInit_ = TRUE;
      }
    }
    else
    {
      ZeroCrossing_ = Deriv1_.FloatValue() < double(0) &&
                      Deriv2_.FloatValue() > double(0);

      if (ZeroCrossing_)
        for (Testx_ = Lower_ - Incr_; Testx_ < Lower_; Testx_ += xTol_)
        {
          FncVal_ = ChrString("(") + Testx_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
          Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

          if (!LowestInit_)
          {
            Lowest_ = Low_;
            Lowestx_ = Testx_;
            LowestInit_ = TRUE;
          }
          else if (Low_ < Lowest_)
          {
            Lowest_ = Low_;
            Lowestx_ = Testx_;
          }
        }

      if (Deriv2_.FloatValue() != double(0))
        Deriv1_ = Deriv2_;
    }
  }

  if (InclOpt_.strlen())
    InclOpt_.RemoveChar(" \t\n");

  int ExclEnds_ = (NumParams_ >= 5) ? icasestrcomp(InclOpt_.c_str(), "EXCLUDEENDS"):0;

  if (!ExclEnds_)
  {
    FncVal_ = ChrString("(") + Lower_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    FncVal_ = ChrString("(") + Upper_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (High_ < Low_)
    {
      if (High_ < Lowest_)
        Lowestx_ = Upper_;
    }
    else
    {
      if (Low_ < Lowest_)
        Lowestx_ = Lower_;
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  xTol_ = Lowestx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  LongNumber Answer_ = Lowestx_;
  Answer_.Round(Exp_);

  if (PlotGraph_)
  {
    _ExtremaEnd = -1;
    ChrString SolString_ = Answer_.ToString();
    PlotFunction(2, ParamArray_, MidPt_.ToString(Double::TYPE_DOUBLE_PRECISION),
                 RadVal_.ToString(Double::TYPE_DOUBLE_PRECISION), &SolString_);
    return SolString_;
  }

  return Answer_.ToString();
}

/*****************************************************************************/
ChrString DoubleCalculator::FindMaximum(int NumParams_, FncParameter** ParamArray_)
{
  if (!Double::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !Double::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 6 && !Double::IsValue(ParamArray_[5]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMAX");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  Double Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  Double Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  ChrString InclOpt_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetExpression():"";
  const char* ParamStr_ = (NumParams_ >= 6) ? ParamArray_[5]->_Param->GetAnswer():Buffer_;
  Double Tol_ = ChrString(ParamStr_);
  Double Incr_ = ChrString(DEFAULT_PRECSTR_FTEST);
  Double xTol_ = Upper_ - Lower_;

  if (xTol_.FloatValue() < Integer(0))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMAX");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }
  else
  {
    Double Ratio_ = xTol_ / 10L;
    xTol_ = Tol_;

    if (Ratio_ < Integer(1))
      Incr_ *= Ratio_;
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  Double Deriv1_;
  Double Deriv2_;
  Double Highest_;
  Double Highestx_;
  Double Testx_;
  Double High_;
  Double Low_;
  Double MidPt_;
  Double RadVal_;

  int ZeroCrossing_;
  int HighestInit_ = 0;
  int PrevInit_ = 0;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;

  if (PlotGraph_)
  {
    MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
  }

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  for (;Lower_ < Upper_; Lower_ += Incr_)
  {
    FncVal_ = Lower_.ToString(Double::TYPE_DOUBLE_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Deriv2_ = FindDerivative(3, Params_);

    if (!PrevInit_)
    {
      if (Deriv2_.FloatValue() != double(0))
      {
        Deriv1_ = Deriv2_;
        PrevInit_ = TRUE;
      }
    }
    else
    {
      ZeroCrossing_ = Deriv1_.FloatValue() > double(0) &&
                      Deriv2_.FloatValue() < double(0);

      if (ZeroCrossing_)
        for (Testx_ = Lower_ - Incr_; Testx_ < Lower_; Testx_ += xTol_)
        {
          FncVal_ = ChrString("(") + Testx_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
          High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

          if (!HighestInit_)
          {
            Highest_ = High_;
            Highestx_ = Testx_;
            HighestInit_ = TRUE;
          }
          else if (High_ > Highest_)
          {
            Highest_ = High_;
            Highestx_ = Testx_;
          }
        }

      if (Deriv2_.FloatValue() != double(0))
        Deriv1_ = Deriv2_;
    }
  }

  if (InclOpt_.strlen())
    InclOpt_.RemoveChar(" \t\n");

  int ExclEnds_ = (NumParams_ >= 5) ? icasestrcomp(InclOpt_.c_str(), "EXCLUDEENDS"):0;

  if (!ExclEnds_)
  {
    FncVal_ = ChrString("(") + Lower_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    FncVal_ = ChrString("(") + Upper_.ToString(Double::TYPE_DOUBLE_PRECISION) + ChrString(")");
    High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (High_ < Low_)
    {
      if (Low_ > Highest_)
        Highestx_ = Lower_;
    }
    else
    {
      if (High_ > Highest_)
        Highestx_ = Upper_;
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  xTol_ = Highestx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  LongNumber Answer_ = Highestx_;
  Answer_.Round(Exp_);

  if (PlotGraph_)
  {
    _ExtremaEnd = 1;
    ChrString SolString_ = Answer_.ToString();
    PlotFunction(2, ParamArray_, MidPt_.ToString(Double::TYPE_DOUBLE_PRECISION),
                 RadVal_.ToString(Double::TYPE_DOUBLE_PRECISION), &SolString_);
    return SolString_;
  }

  return Answer_.ToString();
}

/*****************************************************************************/
ChrString DoubleCalculator::RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_)
{
  if (!Double::IsValue(ParamArray_[0]->_Param->GetAnswer()) ||
      !Double::IsValue(ParamArray_[1]->_Param->GetAnswer()))
  {
    if (Retarg_)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "RECTOPOLR");
    else
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "RECTOPOLA");

    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  Double FltVal_ = ChrString(ParamArray_[0]->_Param->GetAnswer());
  double Xpos_ = FltVal_.FloatValue();
  FltVal_ = ChrString(ParamArray_[1]->_Param->GetAnswer());
  double Ypos_ = FltVal_.FloatValue();
  double Xsqr_;
  double Ysqr_;
  double Sin_;
  Double Angle_;
  Double Radius_;
  ChrString Error_ = "0";

  if (fabs(Ypos_) < _Epsilon)
    Ypos_ = 0;

  if (fabs(Xpos_) < _Epsilon)
    Xpos_ = 0;

  if (!Xpos_ && !Ypos_)
    return Error_ = "0";

  Ysqr_ = Ypos_;
  Ysqr_ *= Ypos_;
  Xsqr_ = Xpos_;
  Xsqr_ *= Xpos_;
  Radius_ = Xsqr_;
  Radius_ += Double(Ysqr_);
  Radius_ = sqrt(Radius_.FloatValue());

  // Return radius
  if (Retarg_)
    return Radius_.ToString(Double::TYPE_DOUBLE_PRECISION);

  Sin_ = Ypos_ / Radius_.FloatValue();
  Angle_ = Double(asin(Sin_));
  if (Xpos_ < double(0))
    if (Ypos_ < double(0))
      Angle_ = Double(-Double::vPI) - Angle_;
    else
      Angle_ = Double(Double::vPI) - Angle_;

  if (GetAngleUnit() == DEGREE)
  {
    Angle_ /= Double(Double::vPI);
    Angle_ *= 180L;
  }

  return Angle_.ToString(Double::TYPE_DOUBLE_PRECISION);
}

/*****************************************************************************/
ChrString DoubleCalculator::PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_)
{
  if (!Double::IsValue(ParamArray_[0]->_Param->GetAnswer()) ||
      !Double::IsValue(ParamArray_[1]->_Param->GetAnswer()))
  {
    if (Retarg_)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "POLTORECX");
    else
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "POLTORECY");

    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  Double FltVal_ = ChrString(ParamArray_[0]->_Param->GetAnswer());
  double Radius_ = FltVal_.FloatValue();
  FltVal_ = ChrString(ParamArray_[1]->_Param->GetAnswer());
  double Angle_ = FltVal_.FloatValue();
  double Mult_;
  double Frac_;
  double iptr_;
  int Int_;
  int Odd_;
  Double Xpos_;
  Double Ypos_;
  double AbsAngle_;
  double PiDiv2_ = Double::vPI / 2L;
  ChrString Zero_ = "0";

  if (Radius_ < _Epsilon)
    return Zero_;

  if (GetAngleUnit() == DEGREE)
  {
    Angle_ /= 180L;
    Angle_ *= Double::vPI;
  }

  AbsAngle_ = fabs(Angle_);

  if (fabs(AbsAngle_ - Double::vPI) > _Epsilon)
  {
    Mult_ = Angle_ / Double::vPI;
    Frac_ = modf(Mult_, &iptr_);
    Int_ = (int)floor(iptr_);
    Odd_ = Int_ % 2;

    if (Frac_ > _Epsilon)
    {
      Angle_ =  Frac_ * Double::vPI;

      if (Odd_)
        if (Angle_ > double(0))
          Angle_ -= Double::vPI;
        else
          Angle_ += Double::vPI;
    }
    else
    {
      if (Odd_)
        Angle_ = Double::vPI;
      else
        Angle_ = 0;
    }
  }

  if (Angle_ >= double(0))
  {
    if (Angle_ > PiDiv2_)
    {
      Angle_ = Double::vPI - Angle_;
      if (fabs(Angle_) < _Epsilon)
        Angle_ = 0;
      Xpos_ = -(Radius_ * cos(Angle_));
    }
    else
    {
      if (fabs(Angle_) < _Epsilon)
        Angle_ = 0;
      Xpos_ = Radius_ * cos(Angle_);
    }

    Ypos_ = Radius_ * sin(Angle_);
  }
  else
  {
    if (Angle_ < -PiDiv2_)
    {
      Angle_ = -Double::vPI - Angle_;
      if (fabs(Angle_) < _Epsilon)
        Angle_ = 0;
      Angle_ *= -1;
      Xpos_ = -(Radius_ * cos(Angle_));
    }
    else
    {
      if (fabs(Angle_) < _Epsilon)
        Angle_ = 0;
      Angle_ *= -1;
      Xpos_ = Radius_ * cos(Angle_);
    }

    Ypos_ = Radius_ * sin(Angle_);
    Ypos_ *= -1;
  }

  if (Retarg_)
    return Xpos_.ToString(Double::TYPE_DOUBLE_PRECISION);

  return Ypos_.ToString(Double::TYPE_DOUBLE_PRECISION);
}

/*****************************************************************************/
ChrString DoubleCalculator::SolveMatrix(int NumParams_, FncParameter** ParamArray_)
{
  int i, x;
  int NumCols_;
  int Max_ = 0;
  int Error_ = FALSE;
  int MatSize_ = 0;
  ChrString AnsCol_;
  ChrString RowStr_;
  ChrString Answer_;
  int* DelimArray_ = (int*)MemMatrix::Matrix().Callocate(sizeof(int) * DELIM_ARRAY_SIZE);
  GVector<double>** Varr_ = new GVector<double>*[NumParams_ + 1];
  GVector<double>* Vect_ = NULL;
  GVector<double> AnsVect_(NumParams_);
  GMatrix<double> Mat_(NumParams_);

  for (i = 0; i <= NumParams_; i++)
    Varr_[i] = NULL;

  for (i = 0; i < NumParams_; i++)
  {
    RowStr_ = ParamArray_[i]->_Param->GetExpression();
    CalculatorBase::FindFncDelim(RowStr_.c_str(), RowStr_.strlen(), &DelimArray_, DELIM_ARRAY_SIZE);
    NumCols_ = DelimArray_[0] - 1;

    if (NumCols_ > Max_)
    {
      if (!Max_)
      {
        if (NumCols_ != NumParams_)
        {
          Error_ = TRUE;
          DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_UNSOLVABLE_MATRIX, ERRMSG_UNSOLVABLE_MATRIX);
          CloseOutputFile();
          break;
        }
        else
          Max_ = NumCols_;
      }
      else
      {
        Error_ = TRUE;
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_MATRIX_COLS, ERRMSG_INVALID_MATRIX_COLS);
        CloseOutputFile();
        break;
      }
    }
    else if (!Max_)
    {
      Error_ = TRUE;
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTYROW_MATRIX, ERRMSG_EMPTYROW_MATRIX);
      CloseOutputFile();
      break;
    }

    int LastCol_ = DelimArray_[NumCols_] + 1;
    char* Divider_ = strchr(RowStr_(LastCol_), '|');

    if (!Divider_)
    {
      Error_ = TRUE;
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_NO_RESULT_COL_MATRIX, ERRMSG_NO_RESULT_COL_MATRIX, RowStr_.c_str());
      CloseOutputFile();
      break;
    }

    if (AnsCol_.strlen())
      AnsCol_ += ", ";
    AnsCol_ += &Divider_[1];
    AnsCol_.RemoveTrailing(")]} \t\n");
    RowStr_.Left(Divider_ - RowStr_(0));
    RowStr_ += ")";

    Vect_ = new GVector<double>(NumParams_);
    Vect_->SetVector(RowStr_.c_str());
    Varr_[MatSize_++] = Vect_;
  }

  if (!Error_)
  {
    AnsVect_.SetVector(AnsCol_.c_str());
    Mat_.SetMatrix(Varr_);
    GVector<double> ResVect_ = GaussPivot(Mat_, AnsVect_, NumParams_);
    ResVect_.WriteToString(Answer_);
  }

  MemMatrix::Matrix().Deallocate(DelimArray_);
  for (i = 0; i < NumParams_; i++)
    delete Varr_[i];
  delete[] Varr_;

  SetHasAnswer(Answer_.strlen());

  if (Answer_.strlen())
  {
    Answer_ = ChrString("\"") + Answer_ + ChrString("\"");
    SetLiteralStr(Answer_.c_str());
  }

  return Answer_;
}

/*****************************************************************************/
ChrString DoubleCalculator::EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_)
{
  int CallFnc_, DiscFnc_, ConvFnc_, OptionFnc_;
  int optype, x;
  ChrString buffer;
  ChrString Answer_;

  optype = icasestrcomp(FncName_, "MIN")         ? Mcalc_OperatorType::Minimum:
           icasestrcomp(FncName_, "MAX")         ? Mcalc_OperatorType::Maximum:
           icasestrcomp(FncName_, "DERIV")       ? Mcalc_OperatorType::Derivative:
           icasestrcomp(FncName_, "DERIV2")      ? Mcalc_OperatorType::Derivative2:
           icasestrcomp(FncName_, "INTEG")       ? Mcalc_OperatorType::Integral:
           icasestrcomp(FncName_, "SOLVE")       ? Mcalc_OperatorType::RootSolve:
           icasestrcomp(FncName_, "INTERSECT")   ? Mcalc_OperatorType::Intersect:
           icasestrcomp(FncName_, "PERMREP")     ? Mcalc_OperatorType::PermuteWithRep:
           icasestrcomp(FncName_, "COMBREP")     ? Mcalc_OperatorType::CombineWithRep:
           icasestrcomp(FncName_, "COMB")        ? Mcalc_OperatorType::Combination:
           icasestrcomp(FncName_, "PERM")        ? Mcalc_OperatorType::Permutation:
           icasestrcomp(FncName_, "FMIN")        ? Mcalc_OperatorType::FunctionMinimum:
           icasestrcomp(FncName_, "FMAX")        ? Mcalc_OperatorType::FunctionMaximum:
           icasestrcomp(FncName_, "ROUND")       ? Mcalc_OperatorType::RoundEx:
           icasestrcomp(FncName_, "SET")         ? Mcalc_OperatorType::MakeSet:
           icasestrcomp(FncName_, "SETUNI")      ? Mcalc_OperatorType::SetUnion:
           icasestrcomp(FncName_, "SETINT")      ? Mcalc_OperatorType::SetIntersection:
           icasestrcomp(FncName_, "SETDIF")      ? Mcalc_OperatorType::SetDifference:
           icasestrcomp(FncName_, "SETXDIF")     ? Mcalc_OperatorType::SetExclusion:
           icasestrcomp(FncName_, "ITEMPERM")    ? Mcalc_OperatorType::ItemPermutation:
           icasestrcomp(FncName_, "ITEMCOMB")    ? Mcalc_OperatorType::ItemCombination:
           icasestrcomp(FncName_, "ITEMPERMREP") ? Mcalc_OperatorType::ItemPermuteWithRep:
           icasestrcomp(FncName_, "ITEMCOMBREP") ? Mcalc_OperatorType::ItemCombineWithRep:
           icasestrcomp(FncName_, "FRAC")        ? Mcalc_OperatorType::FractionArithmetic:
           icasestrcomp(FncName_, "RECTOPOLR")   ? Mcalc_OperatorType::RectToPolarRadius:
           icasestrcomp(FncName_, "RECTOPOLA")   ? Mcalc_OperatorType::RectToPolarAngle:
           icasestrcomp(FncName_, "POLTORECX")   ? Mcalc_OperatorType::PolarToRectX:
           icasestrcomp(FncName_, "POLTORECY")   ? Mcalc_OperatorType::PolarToRectY:
           icasestrcomp(FncName_, "FROMHEX")     ? Mcalc_OperatorType::FromHex:
           icasestrcomp(FncName_, "FROMOCT")     ? Mcalc_OperatorType::FromOct:
           icasestrcomp(FncName_, "FROMBIN")     ? Mcalc_OperatorType::FromBin:
           icasestrcomp(FncName_, "FROMCMP2BIN") ? Mcalc_OperatorType::FromCmp2Bin:
           icasestrcomp(FncName_, "TOHEX")       ? Mcalc_OperatorType::ToHex:
           icasestrcomp(FncName_, "TOOCT")       ? Mcalc_OperatorType::ToOct:
           icasestrcomp(FncName_, "TOBIN")       ? Mcalc_OperatorType::ToBin:
           icasestrcomp(FncName_, "TOCMP2BIN")   ? Mcalc_OperatorType::ToCmp2Bin:
           icasestrcomp(FncName_, "AVGMEAN")     ? Mcalc_OperatorType::AvgMean:
           icasestrcomp(FncName_, "AVGMED")      ? Mcalc_OperatorType::AvgMedian:
           icasestrcomp(FncName_, "AVGMODE")     ? Mcalc_OperatorType::AvgMode:
           icasestrcomp(FncName_, "STDDEV")      ? Mcalc_OperatorType::StdDev:
           icasestrcomp(FncName_, "SUM")         ? Mcalc_OperatorType::Sum:
           icasestrcomp(FncName_, "PROD")        ? Mcalc_OperatorType::Product:
           icasestrcomp(FncName_, "MATSOLVE")    ? Mcalc_OperatorType::MatrixSolve:
           icasestrcomp(FncName_, "GRAPH")       ? Mcalc_OperatorType::PlotGraph:0;
  DiscFnc_ = optype == Mcalc_OperatorType::ItemPermutation ||
             optype == Mcalc_OperatorType::ItemCombination ||
             optype == Mcalc_OperatorType::ItemCombineWithRep ||
             optype == Mcalc_OperatorType::ItemPermuteWithRep ||
             optype == Mcalc_OperatorType::RectToPolarRadius ||
             optype == Mcalc_OperatorType::RectToPolarAngle ||
             optype == Mcalc_OperatorType::PolarToRectX ||
             optype == Mcalc_OperatorType::PolarToRectY ||
             optype == Mcalc_OperatorType::FromHex ||
             optype == Mcalc_OperatorType::FromOct ||
             optype == Mcalc_OperatorType::FromBin ||
             optype == Mcalc_OperatorType::FromCmp2Bin ||
             optype == Mcalc_OperatorType::ToHex ||
             optype == Mcalc_OperatorType::ToOct ||
             optype == Mcalc_OperatorType::ToBin ||
             optype == Mcalc_OperatorType::ToCmp2Bin ||
             optype == Mcalc_OperatorType::MatrixSolve ||
             optype == Mcalc_OperatorType::Intersect ||
             optype == Mcalc_OperatorType::MakeSet ||
             optype == Mcalc_OperatorType::SetUnion ||
             optype == Mcalc_OperatorType::SetIntersection ||
             optype == Mcalc_OperatorType::SetDifference ||
             optype == Mcalc_OperatorType::SetExclusion;
  CallFnc_ = optype == Mcalc_OperatorType::Derivative ||
             optype == Mcalc_OperatorType::Derivative2 ||
             optype == Mcalc_OperatorType::Integral ||
             optype == Mcalc_OperatorType::RootSolve ||
             optype == Mcalc_OperatorType::FunctionMinimum ||
             optype == Mcalc_OperatorType::FunctionMaximum ||
             optype == Mcalc_OperatorType::PlotGraph ||
             optype == Mcalc_OperatorType::FractionArithmetic ||
             optype == Mcalc_OperatorType::AvgMean ||
             optype == Mcalc_OperatorType::AvgMedian ||
             optype == Mcalc_OperatorType::AvgMode ||
             optype == Mcalc_OperatorType::StdDev ||
             optype == Mcalc_OperatorType::Sum ||
             optype == Mcalc_OperatorType::Product ||
             (NumParams_ != 2 && (optype == Mcalc_OperatorType::Minimum   ||
                                  optype == Mcalc_OperatorType::Maximum)) ||
             DiscFnc_;
  ConvFnc_ = optype == Mcalc_OperatorType::ToHex ||
             optype == Mcalc_OperatorType::ToOct ||
             optype == Mcalc_OperatorType::ToBin ||
             optype == Mcalc_OperatorType::ToCmp2Bin ||
             optype == Mcalc_OperatorType::FromHex ||
             optype == Mcalc_OperatorType::FromOct ||
             optype == Mcalc_OperatorType::FromBin ||
             optype == Mcalc_OperatorType::FromCmp2Bin;
  OptionFnc_ = optype == Mcalc_OperatorType::Sum ||
               optype == Mcalc_OperatorType::Product ||
               optype == Mcalc_OperatorType::AvgMean ||
               optype == Mcalc_OperatorType::AvgMedian ||
               optype == Mcalc_OperatorType::AvgMode ||
               optype == Mcalc_OperatorType::StdDev ||
               (NumParams_ != 2 && (optype == Mcalc_OperatorType::Minimum   ||
                                    optype == Mcalc_OperatorType::Maximum)) ||
               optype == Mcalc_OperatorType::ItemPermutation ||
               optype == Mcalc_OperatorType::ItemCombination ||
               optype == Mcalc_OperatorType::ItemCombineWithRep ||
               optype == Mcalc_OperatorType::ItemPermuteWithRep ||
               optype == Mcalc_OperatorType::FractionArithmetic ||
               optype == Mcalc_OperatorType::MakeSet;

  int ErrorSet_;
  ErrorSet_ = (!optype || NumParams_ < 2) ? (!((ConvFnc_ || OptionFnc_) && NumParams_ == 1) && (NumParams_ < 2)):
              (optype == Mcalc_OperatorType::Derivative) ? (NumParams_ < 3):
              (optype == Mcalc_OperatorType::Integral) ? (NumParams_ < 4):
              (optype == Mcalc_OperatorType::RootSolve) ? (NumParams_ < 3):
              (optype == Mcalc_OperatorType::Intersect) ? (NumParams_ < 2):
              (optype == Mcalc_OperatorType::FunctionMinimum) ? (NumParams_ < 4):
              (optype == Mcalc_OperatorType::FunctionMaximum) ? (NumParams_ < 4):0;

  if (ErrorSet_)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_NUMARGS,
                        ERRMSG_INVALID_NUMARGS, FncName_);
    CloseOutputFile();
    return buffer;
  }

  Double resultvalue;
  Double** params = NULL;
  LongNumber LnumResult_;
  LongNumber** lparams = NULL;
  double Tol_ = 0;
  long Exp_ = 0;
  long Rexp_;

  int IntOps_ = (optype == Mcalc_OperatorType::Permutation ||
                 optype == Mcalc_OperatorType::Combination ||
                 optype == Mcalc_OperatorType::CombineWithRep ||
                 optype == Mcalc_OperatorType::PermuteWithRep ||
                 optype == Mcalc_OperatorType::RoundEx);

  if (!CallFnc_)
  {
    if (IntOps_)
      lparams = (LongNumber**)allocmem(NumParams_, sizeof(LongNumber*), params);
    else
      params = (Double**)allocmem(NumParams_, sizeof(Double*), params);

    for (x = 0; x < NumParams_; x++)
    {
      buffer = ParamArray_[x]->_Param->GetAnswer();

      if (Double::IsValue(buffer))
      {
        if (IntOps_)
          lparams[x] = new LongNumber(buffer);
        else
          params[x] = new Double(buffer);
      }
      else
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, FncName_);
        CloseOutputFile();
        return buffer;
      }
    }
  }

  ResetAnswerMode();
  SetAnswerMode(FLOATINGPOINT_MODE);

  /* finding the result value and remainder value */
  switch (optype)
  {
    case Mcalc_OperatorType::PlotGraph:
      resultvalue = PlotFunction(NumParams_, ParamArray_, "0");
      break;

    case Mcalc_OperatorType::Permutation:
      LnumResult_ = Discrete<LongNumber>::Permutation(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Combination:
      LnumResult_ = Discrete<LongNumber>::Combination(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::CombineWithRep:
      LnumResult_ = Discrete<LongNumber>::CombinationWithRep(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::PermuteWithRep:
      LnumResult_ = Discrete<LongNumber>::PermutationWithRep(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Minimum:
      if (NumParams_ == 2)
        resultvalue = (*params[0] < *params[1]) ? *params[0]:*params[1];
      else
        resultvalue = MinimumValue(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Maximum:
      if (NumParams_ == 2)
        resultvalue = (*params[0] > *params[1]) ? *params[0]:*params[1];
      else
        resultvalue = MaximumValue(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RoundEx:
      LnumResult_ = *lparams[0];
      Rexp_ = atol(ParamArray_[1]->_Param->GetAnswer());
      LnumResult_.Round(Rexp_);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Derivative:
      Tol_ = DEFAULT_PRECVAL_DERIV;
      resultvalue = FindDerivative(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Derivative2:
      Tol_ = DEFAULT_PRECVAL_DERIV;
      resultvalue = FindDerivative2(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Integral:
      Tol_ = DEFAULT_PRECVAL_INTEG;
      resultvalue = FindIntegral(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RootSolve:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindRoot(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Intersect:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindIntersections(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FunctionMinimum:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindMinimum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FunctionMaximum:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindMaximum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::MakeSet:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = CreateSet(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetUnion:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetUnion(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetIntersection:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetIntersection(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetDifference:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetDifference(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetExclusion:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetExclusion(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemPermutation:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindPermutations(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemCombination:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindCombinations(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemCombineWithRep:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindCombinationsWithRep(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemPermuteWithRep:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindPermutationsWithRep(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::MatrixSolve:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = SolveMatrix(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FractionArithmetic:
      SetAnswerMode(FRACTION_MODE);
      resultvalue = FindFractionArithmetic(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RectToPolarRadius:
      Answer_ = RectToPolar(NumParams_, ParamArray_, 1);
      break;

    case Mcalc_OperatorType::RectToPolarAngle:
      Answer_ = RectToPolar(NumParams_, ParamArray_, 0);
      break;

    case Mcalc_OperatorType::PolarToRectX:
      Answer_ = PolarToRect(NumParams_, ParamArray_, 1);
      break;

    case Mcalc_OperatorType::PolarToRectY:
      Answer_ = PolarToRect(NumParams_, ParamArray_, 0);
      break;

    case Mcalc_OperatorType::FromHex:
      Answer_ = ConvertFromHex(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromOct:
      Answer_ = ConvertFromOct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromBin:
      Answer_ = ConvertFromBin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromCmp2Bin:
      Answer_ = ConvertFromCmp2Bin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToHex:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToHex(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToOct:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToOct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToBin:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToBin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToCmp2Bin:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToCmp2Bin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Sum:
      resultvalue = CumSum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Product:
      resultvalue = CumProduct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMean:
      resultvalue = AverageMean(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMedian:
      resultvalue = AverageMedian(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMode:
      resultvalue = AverageMode(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::StdDev:
      resultvalue = StandardDeviation(NumParams_, ParamArray_);
      break;

    default:
      buffer = "";
      ErrorSet_ = 1;
  }

  if (!CallFnc_)
    if (IntOps_)
    {
      for (x = 0; x < NumParams_; x++)
        delete lparams[x];
      MemMatrix::Matrix().Deallocate(lparams);
      lparams = NULL;
    }
    else
    {
      for (x = 0; x < NumParams_; x++)
        delete params[x];
      MemMatrix::Matrix().Deallocate(params);
      params = NULL;
    }

  if (Tol_ != double(0))
  {
    while (Tol_ < 1)
    {
      ++Exp_;
      Tol_ *= 10L;
    }

    if (!DiscFnc_ && !(ErrorSet_ || HasErrors()))
    {
      Double xTol_ = resultvalue;
      AbsoluteValue(xTol_);
      if (!xTol_.AtZero())
        while (xTol_ < Integer(1))
        {
          ++Exp_;
          xTol_ *= 10L;
        }
    }

    if (!Exp_)
      Exp_ = Double::TYPE_DOUBLE_PRECISION;
  }

  if (!DiscFnc_)
    Answer_ = (ErrorSet_ || HasErrors()) ? buffer:
                  (Exp_ ? resultvalue.ToString(Exp_):
                          resultvalue.ToString(Double::TYPE_DOUBLE_PRECISION));
  else if (ErrorSet_ || HasErrors())
    Answer_ = buffer;

  return Answer_;
}

/*****************************************************************************/
size_t DoubleCalculator::FindPrecision(const double& Val_, size_t Prec_)
{
  double Cval_ = Val_;
  Ceiling(Cval_);

  size_t x;
  for (x = 0; x < Prec_; x++)
    if (Cval_ > Integer(1))
      Cval_ /= 10L;
    else
      break;

  x = Prec_ - x;
  if (x > 0)
    return x;

  return Prec_;
}

/*****************************************************************************/
bool DoubleCalculator::FindCommonAngles(double value_, double shift_, double Tol_,
                                        double Eps_, double& mult_, double& intp_)
{
  double tval1;
  double tval2;
  double Multiplier_;

  tval2 = value_;
  AbsoluteValue(tval2);
  tval2 += shift_;
  tval1 = tval2 / Double::vPI;
  Multiplier_ = ::pow(10, FindPrecision(tval1, size_t(-log10(Eps_))));
  tval2 = tval1;
  tval2 += Tol_;

  tval2 = ::modf(tval2, &intp_);
  tval2 = tval1;
  tval1 *= Multiplier_;
  tval1 += .5;
  tval1 = ::floor(tval1);
  tval1 /= Multiplier_;
  tval2 *= Multiplier_;
  tval2 += .5;
  tval2 = ::floor(tval2);
  tval2 /= Multiplier_;
  tval2 = ::ceil(tval2);
  mult_ = tval1;

  return (tval1 == tval2);
}

/*****************************************************************************/
// Procedure to directly process the values given to the program using built-in
// math operators if the length of the arguments permit
// Procedure executes only if OPTIMIZE flag is TRUE
// PASSED:
//   operands : the operands array
//   arglist : the argument list
//   optype : the math operator
// USES:
//   uses allocmem to allocate memory
//   uses allocatevalue to allocate memory to the value array of a operand
//        structure
// READ:
//   arglist is read for the arguments to the program and if within acceptable
//   length they are converted to decimal values and the result calculated
// CHANGED:
//   the result and remainder structures have their value array changed
//   to the result value and remainder value if argument length conditions are
//   met
// RETURNED:
//   returns 1 if calculation could be done directly, otherwise returns 0
//   returns 0 if OPTIMIZE flag is FALSE
//
const double& DoubleCalculator::DirectProcess(Mcalc_Operand** operands, char** arglist, int optype)
{
  using ::pow;

  ChrString buffer;
  int IntVal_;
  int dividebyzero = 0;                 /* divide by zero flag */
  double value1, value2;                /* values of respective operands */
  Ulong intval1, intval2;               /* integer values of respective operands */
  int x, i, len;
  Mcalc_Operand *resultptr = operands[RESULTINDEX];
  int TrigOps_;
  long ExpVal_;
  double ip[8];
  double intp_;
  double Tol_;
  double tval_[8];
  bool neg_;
  bool Match_;
  bool Match2_;
  int cd1, cd2;

  if (!StringComparison())
  {
    int FirstArgNotUsed_ = (optype == Mcalc_OperatorType::Square) ||
                           (optype == Mcalc_OperatorType::SquareRoot) ||
                           (optype == Mcalc_OperatorType::Cube) ||
                           (optype == Mcalc_OperatorType::CubeRoot);

    if ((!FirstArgNotUsed_ &&
         !(((Uchar)(*arglist[2])) == NAN_CONST || ((Uchar)(*arglist[2])) == INF_CONST || Double::IsValue(arglist[2]))) ||
         !(((Uchar)(*arglist[3])) == NAN_CONST || ((Uchar)(*arglist[3])) == INF_CONST || Double::IsValue(arglist[3])))
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, GetFncNameFromOpcode(optype));
      CloseOutputFile();
      resultvalue = 0;
      return resultvalue;
    }

    if (((Uchar)(*arglist[2])) == NAN_CONST)
      value1 = Double::vNAN;
    else if (((Uchar)(*arglist[2])) == INF_CONST)
      value1 = Double::vINFINITY;
    else
    {
      value1 = (optype == Mcalc_OperatorType::Square)       ? 2:
               (optype == Mcalc_OperatorType::SquareRoot)   ? .5:
               (optype == Mcalc_OperatorType::Cube)         ? 3:
               (optype == Mcalc_OperatorType::CubeRoot)     ? (1.0/3):
               atof(arglist[2]);
    }

    if (((Uchar)(*arglist[3])) == NAN_CONST)
      value2 = Double::vNAN;
    else if (((Uchar)(*arglist[3])) == INF_CONST)
      value2 = Double::vINFINITY;
    else
    {
      value2 = (optype == Mcalc_OperatorType::Percent)      ? 100:
               strtod(arglist[3],NULL);
    }

    TrigOps_ =
    (optype == Mcalc_OperatorType::Sine)          ||
    (optype == Mcalc_OperatorType::Cosine)        ||
    (optype == Mcalc_OperatorType::Tangent)       ||
    (optype == Mcalc_OperatorType::HyperSine)     ||
    (optype == Mcalc_OperatorType::HyperCosine)   ||
    (optype == Mcalc_OperatorType::HyperTangent);

    if (GetAngleUnit() == DEGREE && TrigOps_)
    {
      value2 /= 180L;
      value2 *= Double::vPI;
    }
  }

  /* finding the result value and remainder value */
  switch (optype)
  {
    case Mcalc_OperatorType::Logical_Not:
      intval2 = Ulong(floor(value2));
      resultvalue = !intval2;
      break;
    case Mcalc_OperatorType::Sine:
      neg_ = value2 < double(0);
      Tol_ = double(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= Double::vPI;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == double(0) || Match_)
      {
        resultvalue = 0;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * Double::vPI / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[1]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[7]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = .5;
        else
          resultvalue = -.5;
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * Double::vPI / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[2]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[6]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(2) / 2;
        else
          resultvalue = -(::sqrt(2) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * Double::vPI / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[3]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[5]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3) / 2;
        else
          resultvalue = -(::sqrt(3) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, Double::vPI / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        cd1 = (long(ip[4]) % 2L) ? 1:-1;
        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1L;
        else
          resultvalue = -1L;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        double lower_ = ip[0] - Tol_;
        double upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          cd1 = (long(ip[4]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = 1L;
          else
            resultvalue = -1L;
          break;
        }

        lower_ = ip[1] - Tol_;
        upper_ = ip[1] + Tol_;

        if (lower_ < tval_[1] && tval_[1] < upper_)
        {
          cd1 = (long(ip[1]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }

        lower_ = ip[7] - Tol_;
        upper_ = ip[7] + Tol_;

        if (lower_ < tval_[7] && tval_[7] < upper_)
        {
          cd1 = (long(ip[7]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }
      }

      resultvalue = sin(value2);
      break;
    case Mcalc_OperatorType::Cosine:
      neg_ = value2 < double(0);
      Tol_ = double(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= Double::vPI;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == double(0) || Match_)
      {
        cd2 = (long(ip[0]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = 1L;
        else
          resultvalue = -1L;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * Double::vPI / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[1]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[7]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3) / 2;
        else
          resultvalue = -(::sqrt(3) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * Double::vPI / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[2]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[6]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = ::sqrt(2) / 2;
        else
          resultvalue = -(::sqrt(2) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * Double::vPI / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[3]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[5]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = .5;
        else
          resultvalue = -.5;
        break;
      }

      Match_ = FindCommonAngles(value2, Double::vPI / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        resultvalue = 0;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        double lower_ = ip[0] - Tol_;
        double upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          cd2 = (long(ip[0]) % 2L) ? -1:1;

          if (cd2 > 0)
            resultvalue = 1L;
          else
            resultvalue = -1L;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[3] - Tol_;
        upper_ = ip[3] + Tol_;

        if (lower_ < tval_[3] && tval_[3] < upper_)
        {
          cd2 = (long(ip[3]) % 2L) ? 1:-1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }

        lower_ = ip[5] - Tol_;
        upper_ = ip[5] + Tol_;

        if (lower_ < tval_[5] && tval_[5] < upper_)
        {
          cd2 = (long(ip[5]) % 2L) ? -1:1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }
      }

      resultvalue = cos(value2);
      break;
    case Mcalc_OperatorType::Tangent:
      neg_ = value2 < double(0);
      Tol_ = double(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= Double::vPI;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == double(0) || Match_)
      {
        resultvalue = 0;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * Double::vPI / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1 / ::sqrt(3);
        else
          resultvalue = -(1 / ::sqrt(3));
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * Double::vPI / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1;
        else
          resultvalue = -1;
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * Double::vPI / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, Double::vPI / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3);
        else
          resultvalue = -(::sqrt(3));
        break;
      }

      Match_ = FindCommonAngles(value2, Double::vPI / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        cd1 = (long(ip[4]) % 2L) ? 1:-1;
        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = Double::vINFINITY;
        else
          resultvalue = -Double::vINFINITY;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        double lower_ = ip[0] - Tol_;
        double upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          cd1 = (long(ip[4]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = Double::vINFINITY;
          else
            resultvalue = -Double::vINFINITY;
          break;
        }

        lower_ = ip[2] - Tol_;
        upper_ = ip[2] + Tol_;

        if (lower_ < tval_[2] && tval_[2] < upper_)
        {
          cd2 = neg_ ? -1:1;
          if (cd2 > 0)
            resultvalue = 1;
          else
            resultvalue = -1;
          break;
        }

        lower_ = ip[6] - Tol_;
        upper_ = ip[6] + Tol_;

        if (lower_ < tval_[6] && tval_[6] < upper_)
        {
          cd2 = neg_ ? 1:-1;
          if (cd2 > 0)
            resultvalue = 1;
          else
            resultvalue = -1;
          break;
        }
      }

      resultvalue = tan(value2);
      break;
    case Mcalc_OperatorType::ArcSine:
      resultvalue = asin(value2);
      break;
    case Mcalc_OperatorType::ArcCosine:
      resultvalue = acos(value2);
      break;
    case Mcalc_OperatorType::ArcTangent:
      resultvalue = atan(value2);
      break;
    case Mcalc_OperatorType::Add:
      resultvalue = value1 + value2;
      break;
    case Mcalc_OperatorType::Subtract:
      resultvalue = value1 - value2;
      break;
    case Mcalc_OperatorType::Multiply:
      resultvalue = value1 * value2;
      break;
    case Mcalc_OperatorType::Divide:
      dividebyzero = value2 == double(0);
      if (!dividebyzero)
      {
        if (_PlottingGraph && !_PlotAllPoints)
        {
          Tol_ = Double(DEFAULT_PRECSTR_GRAPH);
          tval_[0] = -Tol_;
          tval_[1] = Tol_;

          if (tval_[0] < value2 && value2 < tval_[1])
          {
            int dsign_ = (value2 < double(0)) ? -1:1;
            int nsign_ = (value1 < double(0)) ? -1:1;
            nsign_ *= dsign_;

            if (nsign_ > 0)
              resultvalue = Double::vINFINITY;
            else
              resultvalue = -Double::vINFINITY;
            break;
          }
        }

        resultvalue = value1 / value2;

        if (value1 != double(0) &&
            _QLookAhead && *_QLookAhead == '^')
        {
          double val1a = value1 + _Epsilon;
          double val2a = value2 + _Epsilon;
          double val1b = value1 - _Epsilon;
          double val2b = value2 - _Epsilon;

          bool arg2ok = ::IsLong(arglist[2]);
          bool arg3ok = ::IsLong(arglist[3]);
          bool val1aok = !arg2ok && floor(value1 + _Epsilon) == val1a;
          bool val1bok = !arg2ok && floor(value1) == val1b;
          bool val2aok = !arg3ok && floor(value2 + _Epsilon) == val2a;
          bool val2bok = !arg3ok && floor(value2) == val2b;

          if ((arg2ok && arg3ok) ||
              ((val1aok || val1bok) && (val2aok || val2bok)))
          {
            LongNumber val1;
            LongNumber val2;
            Double Tempf_;

            if (arg2ok && arg3ok)
            {
              val1 = ChrString(arglist[2]);
              val2 = ChrString(arglist[3]);
            }
            else if (val1aok)
            {
              Tempf_ = val1a;
              val1 = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);

              if (val2aok)
              {
                Tempf_ = val2a;
                val2 = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);
              }
              else
              {
                Tempf_ = val2b;
                val2 = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);
              }
            }
            else if (val1bok)
            {
              Tempf_ = val1b;
              val1 = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);

              if (val2aok)
              {
                Tempf_ = val2a;
                val2 = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);
              }
              else
              {
                Tempf_ = val2b;
                val2 = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);
              }
            }

            val1 %= 2L;
            val2 %= 2L;

            if (!val2.AtZero())
              _OddRoot = val1.AtZero() ? 2:1;
            else
              _OddRoot = 0;
          }
        }
      }
      else
      {
        if (_PlottingGraph)
        {
          int dsign_ = (value2 < double(0)) ? -1:1;
          int nsign_ = (value1 < double(0)) ? -1:1;
          nsign_ *= dsign_;

          if (nsign_ > 0)
            resultvalue = Double::vINFINITY;
          else
            resultvalue = -Double::vINFINITY;
        }
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::Modulus:
      intval1 = Ulong(floor(value1));
      intval2 = Ulong(floor(value2));
      dividebyzero = !intval2;
      if (!dividebyzero)
        resultvalue = intval1 % intval2;
      else
      {
        if (_PlottingGraph)
          resultvalue = Double::vNAN;
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::Logical_And:
      intval1 = Ulong(floor(value1));
      intval2 = Ulong(floor(value2));
      resultvalue = intval1 && intval2;
      break;
    case Mcalc_OperatorType::Logical_Or:
      intval1 = Ulong(floor(value1));
      intval2 = Ulong(floor(value2));
      resultvalue = intval1 || intval2;
      break;
    case Mcalc_OperatorType::Exponent:
      if (value2 == double(0))
        resultvalue = 1L;
      else if (::IsLong(arglist[3]))
      {
        ExpVal_ = atol(arglist[3]);
        resultvalue = DoubleCalculator::IntPow(value1, ExpVal_);
      }
      else
      {
        if (value1 < double(0))
        {
          if (_OddRoot)
          {
            resultvalue = pow(-value1,value2);
            if (_OddRoot == 1)
              resultvalue = -resultvalue;
          }
          else
            resultvalue = pow(value1,value2);
        }
        else
          resultvalue = pow(value1,value2);
      }
      break;
    case Mcalc_OperatorType::Square:
      resultvalue = pow(value2,2);
      break;
    case Mcalc_OperatorType::SquareRoot:
      resultvalue = pow(value2,.5);
      break;
    case Mcalc_OperatorType::Cube:
      resultvalue = pow(value2,3);
      break;
    case Mcalc_OperatorType::CubeRoot:
      if (value2 < double(0))
        resultvalue = -pow(-value2,(1.0/3));
      else
        resultvalue = pow(value2,(1.0/3));
      break;
    case Mcalc_OperatorType::HyperSine:
      resultvalue = sinh(value2);
      break;
    case Mcalc_OperatorType::HyperCosine:
      resultvalue = cosh(value2);
      break;
    case Mcalc_OperatorType::HyperTangent:
      resultvalue = tanh(value2);
      break;
    case Mcalc_OperatorType::NaturalExp:
      resultvalue = exp(value2);
      break;
    case Mcalc_OperatorType::NaturalLog:
      resultvalue = log(value2);
      break;
    case Mcalc_OperatorType::Log10:
      resultvalue = log10(value2);
      break;
    case Mcalc_OperatorType::Ceiling:
      resultvalue = ceil(value2);
      break;
    case Mcalc_OperatorType::Floor:
      resultvalue = floor(value2);
      break;
    case Mcalc_OperatorType::AbsoluteValue:
      resultvalue = fabs(value2);
      break;
    case Mcalc_OperatorType::FractionPart:
      resultvalue = modf(value2, &intp_);
      break;
    case Mcalc_OperatorType::IntegerPart:
      modf(value2, &intp_);
      resultvalue = intp_;
      break;
    case Mcalc_OperatorType::Percent:
      resultvalue = value1 / value2;
      break;
    case Mcalc_OperatorType::LessThan:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = double(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 < value2;
      break;
    case Mcalc_OperatorType::GreaterThan:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = double(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 > value2;
      break;
    case Mcalc_OperatorType::LessOrEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = double(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 <= value2;
      break;
    case Mcalc_OperatorType::GreaterOrEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = double(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 >= value2;
      break;
    case Mcalc_OperatorType::Equal:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = double(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 == value2;
      break;
    case Mcalc_OperatorType::NotEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = double(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 != value2;
      break;
    case Mcalc_OperatorType::Round:
      resultvalue = Double(value2).Round();
      break;
    case Mcalc_OperatorType::Factorial:
      resultvalue = Discrete<Ulong>::Factorial((Ulong)value2);
      break;
    case Mcalc_OperatorType::Permutation:
      resultvalue = Discrete<Ulong>::Permutation((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::Combination:
      resultvalue = Discrete<Ulong>::Combination((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::CombineWithRep:
      resultvalue = Discrete<Ulong>::CombinationWithRep((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::PermuteWithRep:
      resultvalue = Discrete<Ulong>::PermutationWithRep((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::Assignment:
      resultvalue = value2;
      break;
    case Mcalc_OperatorType::Random:
      IntVal_ = atoi(arglist[3]);
      resultvalue = GiveRandom(IntVal_);
      break;
    case Mcalc_OperatorType::Exclusive_Or:
      intval1 = Ulong(floor(value1));
      intval2 = Ulong(floor(value2));
      resultvalue = !(intval1 && intval2) && (intval1 || intval2);
      break;
    case Mcalc_OperatorType::Inverse:
      dividebyzero = value2 == double(0);
      if (!dividebyzero)
      {
        if (_PlottingGraph && !_PlotAllPoints)
        {
          Tol_ = Double(DEFAULT_PRECSTR_GRAPH);
          tval_[0] = -Tol_;
          tval_[1] = Tol_;

          if (tval_[0] < value2 && value2 < tval_[1])
          {
            if (value2 < double(0))
              resultvalue = -Double::vINFINITY;
            else
              resultvalue = Double::vINFINITY;
            break;
          }
        }

        resultvalue = double(1 / value2);

        if (_QLookAhead && *_QLookAhead == '^')
        {
          double val2a = value2 + _Epsilon;
          double val2b = value2 - _Epsilon;

          bool arg3ok = ::IsLong(arglist[3]);
          bool val2aok = !arg3ok && ::floor(value2 + _Epsilon) == val2a;
          bool val2bok = !arg3ok && ::floor(value2) == val2b;

          if (arg3ok || val2aok || val2bok)
          {
            LongNumber val2;
            Double Tempf_;

            if (arg3ok)
              val2 = ChrString(arglist[3]);
            else if (val2aok)
            {
              Tempf_ = val2a;
              val2 = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);
            }
            else
            {
              Tempf_ = val2b;
              val2 = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);
            }

            val2 %= 2L;

            if (!val2.AtZero())
              _OddRoot = 1;
            else
              _OddRoot = 0;
          }
        }
      }
      else
      {
        if (_PlottingGraph)
        {
          if (value2 < double(0))
            resultvalue = -Double::vINFINITY;
          else
            resultvalue = Double::vINFINITY;
        }
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::ScientificNotation:
      if (::IsNumber(arglist[3], TRUE, FALSE, FALSE, 10))
      {
        ExpVal_ = atol(arglist[3]);
        value2 = pow(10, ExpVal_);
        resultvalue = value1 * value2;
      }
      break;
  }

  TrigOps_ =
  (optype == Mcalc_OperatorType::ArcSine)          ||
  (optype == Mcalc_OperatorType::ArcCosine)        ||
  (optype == Mcalc_OperatorType::ArcTangent);

  if (GetAngleUnit() == DEGREE && TrigOps_)
  {
    resultvalue /= Double::vPI;
    resultvalue *= 180L;
  }

  if (!dividebyzero)
  {
    if (Double::IsInfinite(resultvalue))
    {
      if (resultvalue < double(0))
        SetLiteralStr("-Inf");
      else
        SetLiteralStr("+Inf");
      ResetAnswerMode();
      SetAnswerMode(LITERAL_MODE, TRUE);
      clearvalue(resultptr);
    }
    else if (Double::IsNan(resultvalue))
    {
      SetLiteralStr("Nan");
      ResetAnswerMode();
      SetAnswerMode(LITERAL_MODE, TRUE);
      clearvalue(resultptr);
    }
    else
    {
      // converting decimal result value to array digit elements
      double absresultval = resultvalue;
      AbsoluteValue(absresultval);

      Double Tempf_(absresultval);
      buffer = Tempf_.ToString(Double::TYPE_DOUBLE_PRECISION);
      len = buffer.strlen();

      allocatevalue(resultptr, len);
      resultptr->sign = (resultvalue < 0) ? Mcalc_SignType::Minus:Mcalc_SignType::Plus;
      strcpy(resultptr->value, buffer.c_str());
    }
  }
  else if (!_PlottingGraph)
  {
    ChrString Numer_, Denom_;
    Double Temp_;

    if (optype == Mcalc_OperatorType::Divide ||
        optype == Mcalc_OperatorType::Inverse)
    {
      if (optype == Mcalc_OperatorType::Inverse)
        Numer_ = "1";
      else
      {
        Temp_ = value1;
        Numer_ = Temp_.ToString(Double::TYPE_DOUBLE_PRECISION);
      }

      Temp_ = value2;
      Denom_ = Temp_.ToString(Double::TYPE_DOUBLE_PRECISION);
      Numer_ += ChrString("/") + Denom_;
    }
    else if (optype == Mcalc_OperatorType::Modulus)
    {
      Temp_ = intval1;
      Numer_ = Temp_.ToString(Double::TYPE_DOUBLE_PRECISION);
      Temp_ = intval2;
      Denom_ = Temp_.ToString(Double::TYPE_DOUBLE_PRECISION);
      Numer_ += ChrString("@") + Denom_;
    }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_DIVISION_BY_ZERO,
                        ERRMSG_DIVISION_BY_ZERO, Numer_.c_str());
    CloseOutputFile();
  }

  return resultvalue;
}

/*****************************************************************************/
const double& DoubleCalculator::GetResult() const
{
  return resultvalue;
}

/*****************************************************************************/
const double& DoubleCalculator::GetVariable(int Index_) const
{
  static double Dummy_ = 0;
  if (Index_ >= MAXVARS || Index_ < 0)
    return Dummy_;

  return _vars[Index_];
}

/*****************************************************************************/
size_t DoubleCalculator::GetVariableLength(int Index_) const
{
  if (Index_ < 0)
    return 0;

  if (IsStringVar(Index_))
    return GetStrVarLen(Index_);
  else if (Index_ >= MAXVARS)
    return 0;

  char* ValStr_ = NULL;
  size_t len = strlen(M_E_STRING);
  size_t prec = len / 2;

  ValStr_ = FloatToStr(_vars[Index_], ValStr_, len, prec);
  len = strlen(ValStr_);
  MemMatrix::Matrix().Deallocate(ValStr_);
  ValStr_ = NULL;
  return len;
}

/*****************************************************************************/
void DoubleCalculator::SetVariable(int Index_, const double& Value1_, int Transfer_)
{
  int x;
  int AssignType_;
  double Value_;
  bool AltBase_ = StringAssignment() && AnswerMode() == ALTBASE_MODE;
  LongNumber LongValue_;

  if (Transfer_ || !AltBase_)
  {
    Value_ = Value1_;
    AssignType_ = ASSIGN_ONLY;
  }
  else
  {
    ChrString ValStr_ = GetStrResult();
    LongValue_ = ValStr_;
    Value_ = strtol(ValStr_.c_str(), NULL, 0);
  }

  if (!Transfer_)
    GetIndex(x, AssignType_);

  if (!AssignType_ || Index_ >= MAXVARS)
    return;

  if (Index_ < 0)
  {
    resultvalue = Value_;
    return;
  }

  if (AssignType_ == ASSIGN_ONLY ||
       (AltBase_ && AssignType_ == STRING_ASSIGN))
  {
    _vars[Index_] = Value_;

    if (Transfer_)
      LongValue_ = _TransferValue;
  }
  else if (AssignType_ == ASSIGN_PLUS)
    _vars[Index_] += Value_;
  else if (AssignType_ == ASSIGN_MINUS)
    _vars[Index_] -= Value_;
  else if (AssignType_ == ASSIGN_MULT)
    _vars[Index_] *= Value_;
  else if (AssignType_ == ASSIGN_DIV)
    _vars[Index_] /= Value_;
  else if (AssignType_ == ASSIGN_MOD)
    _vars[Index_] = ((int)_vars[Index_]) % ((int)Value_);
  else if (AssignType_ == ASSIGN_AND && AltBase_)
  {
    char* BaseStr_ = NULL;
    int AssignDone_ = FALSE;

    if (IsStringVar(Index_))
    {
      BaseStr_ = (char*)MemMatrix::Matrix().Allocate(GetStrVarLen(Index_) + 1);
      CopyStrVar(BaseStr_, Index_);

      if ((BaseStr_ && (IsIntHex(BaseStr_) || IsIntOct(BaseStr_))) &&
          (LongValue_.ConvertedFromBase() == 16 || LongValue_.ConvertedFromBase() == 8))
      {
        LongNumber SrceVal_ = Double(_vars[Index_]);
        ChrString TempStr_ = BitwiseOps(SrceVal_, LongValue_, Mcalc_OperatorType::Logical_And);
        double ResultVal_ = strtol(TempStr_.c_str(), NULL, 0);
        _vars[Index_] = ResultVal_;
        AssignDone_ = TRUE;
      }

      MemMatrix::Matrix().Deallocate(BaseStr_);
    }

    if (!AssignDone_)
      _vars[Index_] = Integer(_vars[Index_] != double(0) && !LongValue_.AtZero());
  }
  else if (AssignType_ == ASSIGN_OR && AltBase_)
  {
    char* BaseStr_ = NULL;
    int AssignDone_ = FALSE;

    if (IsStringVar(Index_))
    {
      BaseStr_ = (char*)MemMatrix::Matrix().Allocate(GetStrVarLen(Index_) + 1);
      CopyStrVar(BaseStr_, Index_);

      if ((BaseStr_ && (IsIntHex(BaseStr_) || IsIntOct(BaseStr_))) &&
          (LongValue_.ConvertedFromBase() == 16 || LongValue_.ConvertedFromBase() == 8))
      {
        LongNumber SrceVal_ = Double(_vars[Index_]);
        ChrString TempStr_ = BitwiseOps(SrceVal_, LongValue_, Mcalc_OperatorType::Logical_Or);
        double ResultVal_ = strtol(TempStr_.c_str(), NULL, 0);
        _vars[Index_] = ResultVal_;
        AssignDone_ = TRUE;
      }

      MemMatrix::Matrix().Deallocate(BaseStr_);
    }

    if (!AssignDone_)
      _vars[Index_] = Integer(_vars[Index_] != double(0) && !LongValue_.AtZero());
  }

  if (Transfer_ || AltBase_)
    if (LongValue_.ConvertedFromBase() == 16)
    {
      LongValue_ = Double(_vars[Index_]);
      SetStrVariable(Index_, LongValue_.ToString(16));
    }
    else if (LongValue_.ConvertedFromBase() == 8)
    {
      LongValue_ = Double(_vars[Index_]);
      SetStrVariable(Index_, LongValue_.ToString(8));
    }

  resultvalue = _vars[Index_];
}

/*****************************************************************************/
int DoubleCalculator::GiveType() const
{
  return DOUBLE;
}

/*****************************************************************************/
void DoubleCalculator::SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_)
{
  if (VarIndex_ < 0)
    return;

  if (IsStringVar(VarIndex_))
  {
    CopyStrVar(&ArgStr_[StrIndex_], VarIndex_);
    StrIndex_ += GetStrVarLen(VarIndex_);
  }
  else if (VarIndex_ < MAXVARS)
  {
    char* ValStr_ = NULL;
    int len = strlen(M_E_STRING);
    int prec = len / 2;

    ValStr_ = FloatToStr(_vars[VarIndex_], ValStr_, len, prec);
    ValStr_ = RemovePadding(ValStr_, " \t\n");
    strcpy(&ArgStr_[StrIndex_], ValStr_);
    StrIndex_ += strlen(ValStr_);
    MemMatrix::Matrix().Deallocate(ValStr_);
    ValStr_ = NULL;
  }
}

/*****************************************************************************/
/*****************************************************************************/
LongDoubleCalculator::LongDoubleCalculator():
resultvalue(0),
_Epsilon(DEFAULT_LONGDOUBLE_EPSILON)
{
  int i;
  for (i = 0; i < MAXVARS; i++)
    _vars[i] = 0;

  _IntCoords =
  (IntCoords<Ldouble>**)MemMatrix::Matrix().Callocate(MAX_INT_COORDS * sizeof(IntCoords<Ldouble>*));
  for (i = 0; i < MAX_INT_COORDS; i++)
    _IntCoords[i] = new IntCoords<Ldouble>();
}

/*****************************************************************************/
LongDoubleCalculator::~LongDoubleCalculator()
{
  if (_IntCoords)
  {
    int i;
    for (i = 0; i < MAX_INT_COORDS; i++)
      delete _IntCoords[i];

    MemMatrix::Matrix().Deallocate(_IntCoords);
    _IntCoords = NULL;
  }
}

/*****************************************************************************/
void LongDoubleCalculator::SetParent(CalculatorBase* Ptr_)
{
  _ParentPtr = Ptr_;
  _CalcIndex = CalculatorBase::LONGDOUBLE;
}

/****************************************************************************/
Ldouble LongDoubleCalculator::IntPow(Ldouble Value_, long Exp_)
{
  if (!Exp_)
    return Ldouble(1);

  if (Exp_ == 1L)
    return Value_;

  Ldouble Answer_(Value_);
  long Aexp_ = labs(Exp_);
  long i;

  for (i = 1; i < Aexp_; i++)
    Answer_ *= Value_;

  if (Exp_ < 0)
    Answer_ = 1L / Answer_;

  return Answer_;
}

/*****************************************************************************/
ChrString LongDoubleCalculator::ToString(int Index_)
{
  ChrString DestStr_;
  LongDouble FpObj_;

  if (Index_ >= 0 && IsStringVar(Index_))
  {
    CopyStrVar(DestStr_, Index_);
    return DestStr_;
  }

  if (Index_ < 0)
    FpObj_ = resultvalue;
  else
    FpObj_ = _vars[Index_];

  return FpObj_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindApproxZero(int NumParams_, FncParameter** ParamArray_, Boolean& ZeroFound_)
{
  const char* ParamStr_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongDouble HiTestPt_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongDouble LoTestPt_ = HiTestPt_;
  LongDouble Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  LongDouble Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
  LongDouble Tol_ = ChrString(ParamStr_);
  LongDouble Incr_ = ChrString(DEFAULT_PRECSTR_APPROXZERO);

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  ChrString ValStr_;
  LongDouble LoResult1_;
  LongDouble* LoResult2_ = NULL;
  LongDouble HiResult1_;
  LongDouble* HiResult2_ = NULL;
  LongDouble Low_;
  LongDouble High_;
  LongDouble Testx_;
  LongDouble ZeroPt_;
  int Sign_;
  int ZeroCrossing_;
  ZeroFound_ = FALSE;

  if (HasErrors())
    ClearError();

  while ((HiTestPt_ <= Upper_ || LoTestPt_ >= Lower_) && !HasErrors())
  {
    if (LoTestPt_ >= Lower_)
    {
      ValStr_ = LoTestPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      FncVal_ = ChrString("(") + ValStr_ + ChrString(")");
      LoResult1_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (LoResult1_.FloatValue() == Ldouble(0))
      {
        Testx_ = ValStr_;
        ZeroFound_ = TRUE;
        break;
      }
      else if (!LoResult2_)
        LoResult2_ = new LongDouble(LoResult1_);
      else
      {
        ZeroCrossing_ = LoResult1_.FloatValue() < Ldouble(0) &&
                        LoResult2_->FloatValue() > Ldouble(0) ||
                        LoResult1_.FloatValue() > Ldouble(0) &&
                        LoResult2_->FloatValue() < Ldouble(0);

        if (ZeroCrossing_)
        {
          Low_ = LoTestPt_;
          High_ = LoTestPt_ + Incr_;
          ValStr_ = Low_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
          ParamArray_[3]->_Param->SetAnswer(ValStr_.c_str());
          ValStr_ = High_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
          ParamArray_[4]->_Param->SetAnswer(ValStr_.c_str());
          Testx_ = (High_ + Low_) / 2L;
          break;
        }
        else
        {
          *LoResult2_ = LoResult1_;
          LoTestPt_ -= Incr_;
        }
      }
    }

    if (HiTestPt_ <= Upper_)
    {
      ValStr_ = HiTestPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      FncVal_ = ChrString("(") + ValStr_ + ChrString(")");
      HiResult1_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (HiResult1_.FloatValue() == Ldouble(0))
      {
        Testx_ = ValStr_;
        ZeroFound_ = TRUE;
        break;
      }
      else if (!HiResult2_)
        HiResult2_ = new LongDouble(HiResult1_);
      else
      {
        ZeroCrossing_ = HiResult1_.FloatValue() < Ldouble(0) &&
                        HiResult2_->FloatValue() > Ldouble(0) ||
                        HiResult1_.FloatValue() > Ldouble(0) &&
                        HiResult2_->FloatValue() < Ldouble(0);

        if (ZeroCrossing_)
        {
          Low_ = HiTestPt_ - Incr_;
          High_ = HiTestPt_;
          ValStr_ = Low_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
          ParamArray_[3]->_Param->SetAnswer(ValStr_.c_str());
          ValStr_ = High_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
          ParamArray_[4]->_Param->SetAnswer(ValStr_.c_str());
          Testx_ = (High_ + Low_) / 2L;
          break;
        }
        else
        {
          *HiResult2_ = HiResult1_;
          HiTestPt_ += Incr_;
        }
      }
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  LongDouble xTol_ = Testx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete LoResult2_;
  delete HiResult2_;

  if (HasErrors())
    ClearError();

  LongNumber Answer_ = Testx_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindDerivative(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongDouble::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      (NumParams_ >= 4 &&
       !LongDouble::IsValue(ParamArray_[3]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "DERIV");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  ChrString NewExprStr_ = ParamArray_[2]->_Param->GetAnswer();
  LongDouble Value_ = NewExprStr_;
  NewExprStr_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Buffer_;
  LongDouble Prec_ = NewExprStr_;
  LongDouble HalfPrec_ = Prec_ / 2L;
  HalfPrec_ *= LongDouble(.8);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    PlotFunction(2, ParamArray_, Value_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION));
  }

  LongDouble HighFncRet_;
  LongDouble LowFncRet_;
  LongDouble HighVal_ = Value_ + HalfPrec_;
  LongDouble LowVal_ = Value_ - HalfPrec_;

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + LowVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  LowFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + HighVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  HighFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  HighFncRet_ -= LowFncRet_;
  HighVal_ -= LowVal_;

  LongDouble Slope_ = HighFncRet_;
  Slope_ /= HighVal_;
  PopSubExprStack();

  long Exp_ = 0;
  LongDouble xTol_ = Slope_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_DERIV;

  LongNumber Answer_ = Slope_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindDerivative2(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      (NumParams_ >= 4 &&
       !LongNumber::IsValue(ParamArray_[3]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "DERIV2");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  ChrString NewExprStr_ = ParamArray_[2]->_Param->GetAnswer();
  LongDouble Value_ = NewExprStr_;
  NewExprStr_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Buffer_;
  LongDouble Prec_ = NewExprStr_;
  LongDouble HalfPrec_ = Prec_ / 2L;
  HalfPrec_ *= LongDouble(.8);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    ChrString TempStr_ = ChrString("DERIV(") + ExprStr_;
    TempStr_ += ChrString(", ") + VarName_;
    TempStr_ += ChrString(", ") + VarName_;
    TempStr_ += ChrString("2)");

    FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(2 * sizeof(FncParameter*));
    Params_[0] = new FncParameter();
    Params_[0]->_Param = new SubExpression();
    Params_[0]->_Param->SetExpression(TempStr_.c_str());
    Params_[0]->_ShouldEval = FALSE;
    TempStr_ = VarName_ + ChrString("2");
    Params_[1] = new FncParameter();
    Params_[1]->_Param = new SubExpression();
    Params_[1]->_Param->SetExpression(TempStr_.c_str());
    Params_[1]->_ShouldEval = FALSE;

    PlotFunction(2, Params_, Value_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION));

    int x;
    for (x = 0; x < 2; x++)
      delete Params_[x];

    MemMatrix::Matrix().Deallocate(Params_);
  }

  LongDouble HighFncRet_;
  LongDouble LowFncRet_;
  LongDouble MidTermRet_;
  LongDouble HighVal_ = Value_ + HalfPrec_;
  LongDouble LowVal_ = Value_ - HalfPrec_;

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + LowVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  LowFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + HighVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  HighFncRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  FncVal_ = ChrString("(") + Value_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  MidTermRet_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  MidTermRet_ *= 2L;

  HighFncRet_ -= MidTermRet_;
  HighFncRet_ += LowFncRet_;
  HalfPrec_ *= HalfPrec_;

  LongDouble Slope_ = HighFncRet_;
  Slope_ /= HalfPrec_;
  PopSubExprStack();

  long Exp_ = 0;
  LongDouble xTol_ = Slope_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_DERIV;

  LongNumber Answer_ = Slope_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
void LongDoubleCalculator::PrintInfNanValues(FILE* Fout_, const LongDouble& Answer_, Ldouble fval)
{
  if (Answer_.IsInfinite())
  {
    if (Answer_.FloatValue() < Ldouble(0))
      fprintf(Fout_, DEFAULT_LONGDOUBLENAN_FORMAT, fval, "-inf");
    else
      fprintf(Fout_, DEFAULT_LONGDOUBLENAN_FORMAT, fval, "+inf");
  }
  else if (Answer_.IsNan()) {
    fprintf(Fout_, DEFAULT_LONGDOUBLENAN_FORMAT, fval, "nan"); }
  else
  {
    Ldouble Sd = Answer_;
    fprintf(Fout_, DEFAULT_LONGDOUBLE_FORMAT, fval, Sd);
  }
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindIntersections(int NumParams_, FncParameter** ParamArray_)
{
  int x;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;
  ChrString FirstRt_;
  ChrString RetStr_;
  ChrString FncVal_;
  char CurrentSuffix_ = _PlotFileSuffix;
  LongDouble Yresult_;
  LongDouble Xresult_;
  LongDouble Upper_;
  LongDouble Lower_;
  LongDouble MidPt_;
  LongDouble RadVal_;
  FncParameter** Params_;

  Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_;
  ChrString NewExpr_;

  if (NumParams_ == 5 || NumParams_ == 3)
  {
    NewExpr_ = "(";
    NewExpr_ += ParamArray_[0]->_Param->GetExpression();
    NewExpr_ += ") - (";
    NewExpr_ += ParamArray_[1]->_Param->GetExpression();
    NewExpr_ += ")";
    VarName_ = ParamArray_[2]->_Param->GetExpression();
    CalculatorBase::SetGraphType(Mcalc_GraphType::INTERSECT2);
    _IntExpr = ParamArray_[0]->_Param->GetExpression();
    _IntExpr += " , ";
    _IntExpr += ParamArray_[1]->_Param->GetExpression();
  }
  else if (NumParams_ == 4 || NumParams_ == 2)
  {
    NewExpr_ = ParamArray_[0]->_Param->GetExpression();
    VarName_ = ParamArray_[1]->_Param->GetExpression();
    _IntExpr = NewExpr_;
  }

  Params_[0] = new FncParameter();
  Params_[0]->_Param = new SubExpression();
  Params_[0]->_Param->SetExpression(NewExpr_.c_str());
  Params_[1] = (NumParams_ == 4 || NumParams_ == 2) ?
                   new FncParameter(*ParamArray_[1]):
                   new FncParameter(*ParamArray_[2]);
  Params_[2] = new FncParameter();
  Params_[2]->_Param = new SubExpression();
  Params_[3] = Params_[4] = NULL;

  if (NumParams_ == 2 || NumParams_ == 3)
    Params_[2]->_Param->SetAnswer("0");
  else
  {
    if (NumParams_ == 4)
    {
      Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
      Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;

      if (MidPt_.IsNan() || MidPt_.IsInfinite())
        Params_[2]->_Param->SetAnswer("0");
      else
        Params_[2]->_Param->SetAnswer(MidPt_.ToString().c_str());
    }
    else if (NumParams_ == 5)
    {
      Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;

      if (MidPt_.IsNan() || MidPt_.IsInfinite())
        Params_[2]->_Param->SetAnswer("0");
      else
        Params_[2]->_Param->SetAnswer(MidPt_.ToString().c_str());
    }
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);

  for (_IntsFound = x = 0; x < 3; x++)
  {
    RetStr_ = FindRoot(3, Params_);

    if (!HasErrors())
    {
      FncVal_ = ChrString("(") + RetStr_ + ChrString(")");
      Xresult_ = RetStr_;
      Yresult_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

      if (!_IntsFound ||
          (_IntsFound == 1 && _IntCoords[0]->X != Xresult_.FloatValue()) ||
          (_IntsFound == 2 && _IntCoords[1]->X != Xresult_.FloatValue() &&
                              _IntCoords[0]->X != Xresult_.FloatValue()))
      {
        if (!_IntsFound)
          FirstRt_ = Xresult_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);

        _IntCoords[_IntsFound]->X = Xresult_.FloatValue();
        _IntCoords[_IntsFound]->Y = Yresult_.FloatValue();
        ++_IntsFound;
      }
    }

    if (x == 0)
    {
      if (NumParams_ == 4 || NumParams_ == 5)
      {
        if (NumParams_ == 4)
          Params_[2]->_Param->SetAnswer(ParamArray_[2]->_Param->GetAnswer());
        else
          Params_[2]->_Param->SetAnswer(ParamArray_[3]->_Param->GetAnswer());
      }
      else
        Params_[2]->_Param->SetAnswer("-5");
    }
    else if (x == 1)
    {
      if (NumParams_ == 4 || NumParams_ == 5)
      {
        if (NumParams_ == 4)
          Params_[2]->_Param->SetAnswer(ParamArray_[3]->_Param->GetAnswer());
        else
          Params_[2]->_Param->SetAnswer(ParamArray_[4]->_Param->GetAnswer());
      }
      else
        Params_[2]->_Param->SetAnswer("5");
    }

    if (HasErrors())
      ClearError();
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();

  if (PlotGraph_)
  {
    int RangeSpecified_ = NumParams_ == 4 || NumParams_ == 5;
    if (_IntsFound)
      RetStr_ = FirstRt_;
    else
      RetStr_ = "0";

    if (RangeSpecified_)
    {
      if (NumParams_ == 5)
      {
        Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
        Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      }
      else if (NumParams_ == 4)
      {
        Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
        Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      }

      MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;
      RadVal_ = Upper_ - MidPt_;
      AbsoluteValue(RadVal_);
    }

    if (NumParams_ == 5 || NumParams_ == 3)
    {
      // Plotting First Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[2]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION),
                     RadVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);

      // Plotting Second Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[1]);
      Params_[1] = new FncParameter(*ParamArray_[2]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION),
                     RadVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);
    }
    else if (NumParams_ == 4 || NumParams_ == 2)
    {
      // Plotting First Graph
      delete Params_[0];
      delete Params_[1];
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      if (RangeSpecified_)
        PlotFunction(2, Params_, MidPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION),
                     RadVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION), &RetStr_);
      else
        PlotFunction(2, Params_, RetStr_);
    }

    SaveCurrentGraphName(CurrentSuffix_);
  }

  for (x = 0; x < 5; x++)
    delete Params_[x];
  MemMatrix::Matrix().Deallocate(Params_);

  char Buffer_[64];
  char Suf_ = 'z';
  sprintf(Buffer_, "mcalc_graph%d%c.out", SessionNum(), Suf_);
  FILE* Fout_ = fopen(Buffer_, "w");
  RetStr_ = "\n";

  if (Fout_)
  {
    if (_IntsFound)
      for (x = 0; x < _IntsFound; x++)
      {
        sprintf(Buffer_, DEFAULT_LONGDOUBLE_FORMAT, _IntCoords[x]->X, _IntCoords[x]->Y);
        fprintf(Fout_, "%s", Buffer_);
        RetStr_ += Buffer_;
      }

    fclose(Fout_);
  }

  SetHasAnswer(RetStr_.strlen());

  if (RetStr_.strlen())
  {
    RetStr_ = ChrString("\"") + RetStr_ + ChrString("\"");
    SetLiteralStr(RetStr_.c_str());
  }

  return RetStr_;
}

/*****************************************************************************/
ChrString LongDoubleCalculator::PlotFunction(int NumParams_, FncParameter** ParamArray_,
                                             const ChrString& MidPtStr_, ChrString Radius_, ChrString* Solution_)
{
  if (NumParams_ < 2 || (NumParams_ >= 3 && !LongNumber::IsValue(ParamArray_[2]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "GRAPH");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString Caption_;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();

  if (ExprStr_.find(VarName_).failed())
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_GRAPHVARNAME,
                        ERRMSG_INVALID_GRAPHVARNAME, "GRAPH");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  char Fname_[32];
  sprintf(Fname_, "mcalc_graph%d%c.out", SessionNum(), PlotFileSuffix());
  FILE* Fout_ = fopen(Fname_, "w");

  LongDouble Mid_ = (NumParams_ >= 3) ? ParamArray_[2]->_Param->GetAnswer():MidPtStr_;
  LongDouble RadiusVal_ = (NumParams_ >= 4) ? ParamArray_[3]->_Param->GetAnswer():Radius_;
  LongDouble Lower_ = Mid_ - RadiusVal_;
  LongDouble Upper_ = Mid_ + RadiusVal_;

  ChrString AnsStr_;
  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(4 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  AnsStr_ = Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
  Params_[2] = new FncParameter();
  Params_[2]->_Param = new SubExpression();
  Params_[2]->_Param->SetAnswer(AnsStr_.c_str());
  Params_[2]->_ShouldEval = FALSE;
  AnsStr_ = Upper_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
  Params_[3] = new FncParameter();
  Params_[3]->_Param = new SubExpression();
  Params_[3]->_Param->SetAnswer(AnsStr_.c_str());
  Params_[3]->_ShouldEval = FALSE;
  long Ints_ = FindNumIntervals(4, Params_);

  LongDouble Ht = (Upper_ - Lower_) / Ints_;
  LongDouble Xi;
  LongDouble Answer_;
  Ldouble Sd;
  Ldouble Htval_ = Ht;
  Ldouble Lowerval_ = Lower_;
  Ldouble fval;

  if (Solution_)
    Mid_ = *Solution_;

  if (_GraphType == Mcalc_GraphType::EXTREMA)
  {
    if (_ExtremaEnd < 0)
      Caption_ = ChrString("FMIN( ") + ExprStr_ + ChrString(" )");
    else
      Caption_ = ChrString("FMAX( ") + ExprStr_ + ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::DERIVATIVE)
  {
    ChrString DerivPt_ = ParamArray_[2]->_Param->GetAnswer();
    Caption_ = ChrString("DERIV( ") + ExprStr_;
    Caption_ += ChrString(" , ") + DerivPt_;
    Caption_ += ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::INTEGRAL)
  {
    ChrString LowerIntPt_ = ParamArray_[2]->_Param->GetAnswer();
    ChrString UpperIntPt_ = ParamArray_[3]->_Param->GetAnswer();
    Caption_ = ChrString("INTEG( ") + ExprStr_;
    Caption_ += ChrString(" , ") + LowerIntPt_;
    Caption_ += ChrString(" , ") + UpperIntPt_;
    Caption_ += ChrString(" )");
  }
  else if (_GraphType == Mcalc_GraphType::ROOTSOLVE)
    Caption_ = ChrString("SOLVE( ") + ExprStr_ + ChrString(" )");
  else if (_GraphType == Mcalc_GraphType::INTERSECT1 ||
           _GraphType == Mcalc_GraphType::INTERSECT2)
    Caption_ = ChrString("INTERSECT( ") + _IntExpr + ChrString(" )");
  else
    Caption_ = ExprStr_;

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  _PlottingGraph = TRUE;
  fprintf(Fout_, "%s\n", Caption_.c_str());
  fprintf(Fout_, "%d\n", _GraphType);
  fprintf(Fout_, "%d\n", (_GraphType == Mcalc_GraphType::INTEGRAL) ? 1:0);

  fval = Mid_;
  ChrString FncVal_ = ChrString("(") + Mid_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    LongDoubleCalculator::PrintInfNanValues(Fout_, Answer_, fval);

  fval = Lower_;
  FncVal_ = ChrString("(") + Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    LongDoubleCalculator::PrintInfNanValues(Fout_, Answer_, fval);

  long x;
  for (x = 1; x < Ints_; x++)
  {
    Xi = LongDouble(Lowerval_ + Htval_ * x);
    fval = Xi;
    FncVal_ = ChrString("(") + Xi.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (HasErrors())
    {
      if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
        ClearError();
      else
        goto HandleError;
    }
    else
      LongDoubleCalculator::PrintInfNanValues(Fout_, Answer_, fval);
  }

  fval = Upper_;
  FncVal_ = ChrString("(") + Upper_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  if (HasErrors())
  {
    if (GetErrorVal() == Mcalc_Error::ERRVAL_DIVISION_BY_ZERO)
      ClearError();
    else
      goto HandleError;
  }
  else
    LongDoubleCalculator::PrintInfNanValues(Fout_, Answer_, fval);

HandleError:
  fclose(Fout_);
  SetSuppressOutput(FALSE);
  PopSubExprStack();
  _PlottingGraph = FALSE;

  for (x = 0; x < 4; x++)
    delete Params_[x];

  MemMatrix::Matrix().Deallocate(Params_);
  return ChrString("0");
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindIntegral(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongDouble::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !LongDouble::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 5 &&
       !LongDouble::IsValue(ParamArray_[4]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "INTEG");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  long Ints_ = FindNumIntervals(NumParams_, ParamArray_);
  const char* Buffer_ = DEFAULT_PRECSTR_INTEG;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongDouble Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongDouble Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  const char* ParamStr_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetAnswer():Buffer_;
  LongDouble Prec_ = strtod(ParamStr_, NULL);

  if (_PlotGraph)
  {
    _PlotGraph = false;
    LongDouble MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    LongDouble RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
    PlotFunction(2, ParamArray_, MidPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION),
                 RadVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION));
  }

  LongDouble Ht = (Upper_ - Lower_) / Ints_;
  LongDouble Xi;
  LongDouble Answer_;
  Ldouble Sd;
  Ldouble Htval_ = Ht;
  Ldouble Lowerval_ = Lower_;

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_ = ChrString("(") + Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  Sd = Answer_;

  long x;
  for (x = 1; x < Ints_; x++)
  {
    Xi = LongDouble(Lowerval_ + Htval_ * x);
    FncVal_ = ChrString("(") + Xi.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Answer_ *= 2L;
    Sd += Answer_;
  }

  FncVal_ = ChrString("(") + Upper_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  Answer_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  Sd += Answer_;
  Answer_ = LongDouble(Htval_ * Sd / 2L);
  SetSuppressOutput(FALSE);
  PopSubExprStack();

  long Exp_ = 0;
  LongDouble xTol_ = Answer_;
  AbsoluteValue(xTol_);

  while (Prec_ < Integer(1))
  {
    --Exp_;
    Prec_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_INTEG;

  LongNumber StrAnswer_ = Answer_;
  StrAnswer_.Round(Exp_);
  return StrAnswer_.ToString();
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindNewtonsCriteria(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!LongDouble::IsValue(ParamArray_[2]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongDouble Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongDouble Incr_ = DEFAULT_PRECVAL_NEWTCRIT;

  CreateNewSubExpr();
  Ldouble X1 = Guess_;
  X1 -= 10;
  LongDouble Temp_ = X1;
  ChrString FncVal_ = ChrString("(") + Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  Ldouble Y1 = Temp_;
  Ldouble CriteriaVal_;

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  Temp_ = X1;
  FncVal_ = Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
  Params_[2]->_Param->SetAnswer(FncVal_.c_str());
  Temp_ = FindDerivative(NumParams_, Params_);
  Ldouble Ypr = Temp_;
  Temp_ = FindDerivative2(NumParams_, Params_);
  Ldouble Ypr2 = Temp_;
  LongDouble Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);

  int i;
  int max = int(20 / Incr_.FloatValue());
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 0; i < max; i++)
  {
    CriteriaVal_ = ((Y1 * Ypr2) / (Ypr * Ypr));
    AbsoluteValue(CriteriaVal_);

    if (CriteriaVal_ < 1)
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (CriteriaVal_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Temp_ = X1;
    Temp_ += Incr_;
    X1 = Temp_;
    FncVal_ = ChrString("(") + Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y1 = Temp_;
    Temp_ = X1;
    FncVal_ = Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Temp_ = FindDerivative(NumParams_, Params_);
    Ypr = Temp_;
    Temp_ = FindDerivative2(NumParams_, Params_);
    Ypr2 = Temp_;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  LongDouble xTol_ = X1;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  if (HasErrors())
    ClearError();

  Temp_ = X1;
  LongNumber Answer_ = Temp_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString LongDoubleCalculator::NewtonsMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!LongDouble::IsValue(ParamArray_[2]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongDouble Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());

  CreateNewSubExpr();
  Ldouble X1 = Guess_;
  LongDouble Temp_ = X1;
  ChrString FncVal_ = ChrString("(") + Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  Ldouble Y1 = Temp_;
  Ldouble X2;
  Ldouble Y2;

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  Temp_ = X1;
  FncVal_ = Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
  Params_[2]->_Param->SetAnswer(FncVal_.c_str());
  Temp_ = FindDerivative(NumParams_, Params_);
  Ldouble Ypr = Temp_;
  Ldouble Diff_;
  LongDouble Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);

  int i;
  int max = 100;
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 0; i < max; i++)
  {
    X2 = X1 - Y1 / Ypr;
    Temp_ = X2;
    FncVal_ = ChrString("(") + Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y2 = Temp_;
    Diff_ = X2 - X1;
    Diff_ = FABSL(Diff_);

    if (Diff_ < Tol_.FloatValue() &&
        Y2 < Tol_.FloatValue())
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (Diff_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Temp_ = X2;
    FncVal_ = Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Temp_ = FindDerivative(NumParams_, Params_);
    Ypr = Temp_;
    X1 = X2;
    Y1 = Y2;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  LongDouble xTol_ = X2;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  if (HasErrors())
    ClearError();

  Temp_ = X2;
  LongNumber Answer_ = Temp_;
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString LongDoubleCalculator::MullersMethod(int NumParams_, FncParameter** ParamArray_, int& Converge_)
{
  if (!LongDouble::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !LongDouble::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      !LongDouble::IsValue(ParamArray_[4]->_Param->GetAnswer()))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "SOLVE");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongDouble Guess_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongDouble Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  LongDouble Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());

  CreateNewSubExpr();
  ChrString FncVal_ = ChrString("(") + Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  LongDouble LowerY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  FncVal_ = ChrString("(") + Upper_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  LongDouble UpperY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
  FncVal_ = ChrString("(") + Guess_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
  LongDouble GuessY_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

  Ldouble Guessval_ = Guess_;
  Ldouble Lowerval_ = Lower_;
  Ldouble Upperval_ = Upper_;
  Ldouble LowerYval_ = LowerY_;
  Ldouble UpperYval_ = UpperY_;
  Ldouble GuessYval_ = GuessY_;

  LongDouble Temp_ = (UpperY_ - LowerY_) / (Upper_ - Lower_);
  Ldouble C1 = Temp_;
  Ldouble C2;
  Ldouble D1;
  Ldouble S;
  Ldouble Det_;
  Ldouble X4;
  Ldouble Y4;
  Ldouble Diff_;
  LongDouble Tol_ = ChrString(DEFAULT_PRECSTR_SOLVE);
  long Sign_;

  int i;
  int max = 2000;
  Converge_ = FALSE;
  SetSuppressOutput(TRUE);

  for (i = 2; i < max; i++)
  {
    C2 = (GuessYval_ - UpperYval_) / (Guessval_ - Upperval_);
    D1 = (C2 - C1) / (Guessval_ - Lowerval_);
    S = C2 + D1 * (Guessval_ - Upperval_);

    Sign_ = (S >= Ldouble(0)) ? 1L:-1L;
    Det_ = POWL(S, 2) - (4L * GuessYval_ * D1);
    SquareRoot(Det_);
    X4 = Guessval_ - ((2L * GuessYval_) / (S + Sign_ * Det_));

    Temp_ = X4;
    FncVal_ = ChrString("(") + Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    Temp_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);
    Y4 = Temp_;
    Diff_ = X4 - Guessval_;
    AbsoluteValue(Diff_);

    if (Diff_ < Tol_.FloatValue() &&
        Y4 < Tol_.FloatValue())
    {
      if (HasErrors())
        Converge_ = FALSE;
      else
        Converge_ = TRUE;
      break;
    }
    else if (Diff_ >= UPPER_LIMIT)
    {
      Converge_ = FALSE;
      break;
    }

    Lowerval_ = Upperval_;
    Upperval_ = Guessval_;
    UpperYval_ = GuessYval_;
    Guessval_ = X4;
    GuessYval_ = Y4;
    C1 = C2;
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  LongDouble xTol_ = X4;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  if (HasErrors())
    ClearError();

  LongNumber Answer_ = LongDouble(X4);
  Answer_.Round(Exp_);
  return Answer_.ToString();
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindRoot(int NumParams_, FncParameter** ParamArray_)
{
  int x;
  int Converge_;
  int CriteriaFound_;
  ChrString RetStr_;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;
  bool RangeSpecified_ = false;
  FncParameter** Params_;

  if (NumParams_ > 3)
  {
    RangeSpecified_ = true;
    RetStr_ = MullersMethod(NumParams_, ParamArray_, Converge_);

    if (!Converge_)
    {
      Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      Params_[2] = new FncParameter(*ParamArray_[2]);
      Params_[3] = new FncParameter(*ParamArray_[3]);
      Params_[4] = new FncParameter(*ParamArray_[4]);
      ChrString Answer_ = FindNewtonsCriteria(NumParams_, Params_, CriteriaFound_);

      if (CriteriaFound_)
      {
        Params_[2]->_Param->SetAnswer(Answer_.c_str());
        RetStr_ = NewtonsMethod(NumParams_, Params_, Converge_);

        if (Converge_)
        {
          for (x = 0; x < 5; x++)
            delete Params_[x];

          MemMatrix::Matrix().Deallocate(Params_);
          goto RootSolveDone;
        }
        else
        {
          RetStr_ = MullersMethod(NumParams_, Params_, Converge_);

          if (Converge_)
          {
            for (x = 0; x < 5; x++)
              delete Params_[x];

            MemMatrix::Matrix().Deallocate(Params_);
            goto RootSolveDone;
          }
        }
      }
    }
  }
  else
  {
    RetStr_ = NewtonsMethod(NumParams_, ParamArray_, Converge_);

    if (!Converge_)
    {
      Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(5 * sizeof(FncParameter*));
      Params_[0] = new FncParameter(*ParamArray_[0]);
      Params_[1] = new FncParameter(*ParamArray_[1]);
      Params_[2] = new FncParameter(*ParamArray_[2]);
      Params_[3] = Params_[4] = NULL;
      ChrString Answer_ = FindNewtonsCriteria(NumParams_, Params_, CriteriaFound_);

      if (CriteriaFound_)
      {
        Params_[2]->_Param->SetAnswer(Answer_.c_str());
        RetStr_ = NewtonsMethod(NumParams_, Params_, Converge_);

        if (Converge_)
        {
          for (x = 0; x < 3; x++)
            delete Params_[x];

          MemMatrix::Matrix().Deallocate(Params_);
          goto RootSolveDone;
        }
      }

      Params_[3] = new FncParameter();
      Params_[4] = new FncParameter();
      Params_[3]->_Param = new SubExpression();
      Params_[4]->_Param = new SubExpression();
      Boolean ZeroFound_ = FALSE;
      LongDouble Lower_ = ChrString(Params_[2]->_Param->GetAnswer());
      LongDouble Upper_ = ChrString(Params_[2]->_Param->GetAnswer());
      NumParams_ += 2;
      Lower_ -= 20L;
      Upper_ += 20L;
      Answer_ = Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      Params_[3]->_Param->SetAnswer(Answer_.c_str());
      Params_[3]->_ShouldEval = FALSE;
      Answer_ = Upper_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      Params_[4]->_Param->SetAnswer(Answer_.c_str());
      Params_[4]->_ShouldEval = FALSE;
      RetStr_ = MullersMethod(NumParams_, Params_, Converge_);

      if (!Converge_)
      {
        RetStr_ = FindApproxZero(NumParams_, Params_, ZeroFound_);
        if (!ZeroFound_)
          RetStr_ = MullersMethod(NumParams_, Params_, Converge_);
        else
          Converge_ = TRUE;
      }

      if (!Converge_)
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED,
                        ERRMSG_MULLERS_METHOD_FAILED, ParamArray_[0]->_Param->GetExpression());
        CloseOutputFile();
      }

      for (x = 0; x < 5; x++)
        delete Params_[x];

      MemMatrix::Matrix().Deallocate(Params_);
    }
  }

RootSolveDone:
  if (!Converge_)
  {
    if (RangeSpecified_)
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_MULLERS_METHOD_FAILED,
                      ERRMSG_MULLERS_METHOD_FAILED, ParamArray_[0]->_Param->GetExpression());
      CloseOutputFile();
    }
  }
  else if (PlotGraph_)
  {
    if (RangeSpecified_)
    {
      LongDouble Lower_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
      LongDouble Upper_ = ChrString(ParamArray_[4]->_Param->GetAnswer());
      LongDouble MidPt_ = Lower_ + Upper_;
      MidPt_ /= 2L;
      LongDouble RadVal_ = Upper_ - MidPt_;
      AbsoluteValue(RadVal_);
      PlotFunction(2, ParamArray_, MidPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION),
                   RadVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION), &RetStr_);
    }
    else
      PlotFunction(2, ParamArray_, RetStr_);
  }

  return RetStr_;
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindMinimum(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongDouble::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !LongDouble::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 6 && !LongDouble::IsValue(ParamArray_[5]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMIN");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongDouble Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongDouble Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  ChrString InclOpt_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetExpression():"";
  const char* ParamStr_ = (NumParams_ >= 6) ? ParamArray_[5]->_Param->GetAnswer():Buffer_;
  LongDouble Tol_ = ChrString(ParamStr_);
  LongDouble Incr_ = ChrString(DEFAULT_PRECSTR_FTEST);
  LongDouble xTol_ = Upper_ - Lower_;

  if (xTol_.FloatValue() < Integer(0))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMIN");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }
  else
  {
    LongDouble Ratio_ = xTol_ / 10L;
    xTol_ = Tol_;

    if (Ratio_ < Integer(1))
      Incr_ *= Ratio_;
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  LongDouble Deriv1_;
  LongDouble Deriv2_;
  LongDouble Lowest_;
  LongDouble Lowestx_;
  LongDouble Testx_;
  LongDouble Low_;
  LongDouble High_;
  LongDouble MidPt_;
  LongDouble RadVal_;

  int ZeroCrossing_;
  int LowestInit_ = 0;
  int PrevInit_ = 0;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;

  if (PlotGraph_)
  {
    MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
  }

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  for (;Lower_ < Upper_; Lower_ += Incr_)
  {
    FncVal_ = Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Deriv2_ = FindDerivative(3, Params_);

    if (!PrevInit_)
    {
      if (Deriv2_.FloatValue() != Ldouble(0))
      {
        Deriv1_ = Deriv2_;
        PrevInit_ = TRUE;
      }
    }
    else
    {
      ZeroCrossing_ = Deriv1_.FloatValue() < Ldouble(0) &&
                      Deriv2_.FloatValue() > Ldouble(0);

      if (ZeroCrossing_)
        for (Testx_ = Lower_ - Incr_; Testx_ < Lower_; Testx_ += xTol_)
        {
          FncVal_ = ChrString("(") + Testx_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
          Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

          if (!LowestInit_)
          {
            Lowest_ = Low_;
            Lowestx_ = Testx_;
            LowestInit_ = TRUE;
          }
          else if (Low_ < Lowest_)
          {
            Lowest_ = Low_;
            Lowestx_ = Testx_;
          }
        }

      if (Deriv2_.FloatValue() != Ldouble(0))
        Deriv1_ = Deriv2_;
    }
  }

  if (InclOpt_.strlen())
    InclOpt_.RemoveChar(" \t\n");

  int ExclEnds_ = (NumParams_ >= 5) ? icasestrcomp(InclOpt_.c_str(), "EXCLUDEENDS"):0;

  if (!ExclEnds_)
  {
    FncVal_ = ChrString("(") + Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    FncVal_ = ChrString("(") + Upper_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (High_ < Low_)
    {
      if (High_ < Lowest_)
        Lowestx_ = Upper_;
    }
    else
    {
      if (Low_ < Lowest_)
        Lowestx_ = Lower_;
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  xTol_ = Lowestx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  LongNumber Answer_ = Lowestx_;
  Answer_.Round(Exp_);

  if (PlotGraph_)
  {
    _ExtremaEnd = -1;
    ChrString SolString_ = Answer_.ToString();
    PlotFunction(2, ParamArray_, MidPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION),
                 RadVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION), &SolString_);
    return SolString_;
  }

  return Answer_.ToString();
}

/*****************************************************************************/
ChrString LongDoubleCalculator::FindMaximum(int NumParams_, FncParameter** ParamArray_)
{
  if (!LongDouble::IsValue(ParamArray_[2]->_Param->GetAnswer()) ||
      !LongDouble::IsValue(ParamArray_[3]->_Param->GetAnswer()) ||
      (NumParams_ >= 6 && !LongDouble::IsValue(ParamArray_[5]->_Param->GetAnswer())))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMAX");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  const char* Buffer_ = DEFAULT_PRECSTR_DERIV;
  ChrString ExprStr_ = ParamArray_[0]->_Param->GetExpression();
  ChrString VarName_ = ParamArray_[1]->_Param->GetExpression();
  LongDouble Lower_ = ChrString(ParamArray_[2]->_Param->GetAnswer());
  LongDouble Upper_ = ChrString(ParamArray_[3]->_Param->GetAnswer());
  ChrString InclOpt_ = (NumParams_ >= 5) ? ParamArray_[4]->_Param->GetExpression():"";
  const char* ParamStr_ = (NumParams_ >= 6) ? ParamArray_[5]->_Param->GetAnswer():Buffer_;
  LongDouble Tol_ = ChrString(ParamStr_);
  LongDouble Incr_ = ChrString(DEFAULT_PRECSTR_FTEST);
  LongDouble xTol_ = Upper_ - Lower_;

  if (xTol_.FloatValue() < Integer(0))
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                        ERRMSG_INVALID_TYPEARGS, "FMAX");
    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }
  else
  {
    LongDouble Ratio_ = xTol_ / 10L;
    xTol_ = Tol_;

    if (Ratio_ < Integer(1))
      Incr_ *= Ratio_;
  }

  CreateNewSubExpr();
  SetSuppressOutput(TRUE);
  ChrString FncVal_;
  LongDouble Deriv1_;
  LongDouble Deriv2_;
  LongDouble Highest_;
  LongDouble Highestx_;
  LongDouble Testx_;
  LongDouble High_;
  LongDouble Low_;
  LongDouble MidPt_;
  LongDouble RadVal_;

  int ZeroCrossing_;
  int HighestInit_ = 0;
  int PrevInit_ = 0;
  int PlotGraph_ = _PlotGraph;
  _PlotGraph = 0;

  if (PlotGraph_)
  {
    MidPt_ = Lower_ + Upper_;
    MidPt_ /= 2L;
    RadVal_ = Upper_ - MidPt_;
    AbsoluteValue(RadVal_);
  }

  FncParameter** Params_ = (FncParameter**)MemMatrix::Matrix().Callocate(3 * sizeof(FncParameter*));
  Params_[0] = new FncParameter(*ParamArray_[0]);
  Params_[1] = new FncParameter(*ParamArray_[1]);
  Params_[2] = new FncParameter(*ParamArray_[2]);

  for (;Lower_ < Upper_; Lower_ += Incr_)
  {
    FncVal_ = Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
    Params_[2]->_Param->SetAnswer(FncVal_.c_str());
    Deriv2_ = FindDerivative(3, Params_);

    if (!PrevInit_)
    {
      if (Deriv2_.FloatValue() != Ldouble(0))
      {
        Deriv1_ = Deriv2_;
        PrevInit_ = TRUE;
      }
    }
    else
    {
      ZeroCrossing_ = Deriv1_.FloatValue() > Ldouble(0) &&
                      Deriv2_.FloatValue() < Ldouble(0);

      if (ZeroCrossing_)
        for (Testx_ = Lower_ - Incr_; Testx_ < Lower_; Testx_ += xTol_)
        {
          FncVal_ = ChrString("(") + Testx_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
          High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

          if (!HighestInit_)
          {
            Highest_ = High_;
            Highestx_ = Testx_;
            HighestInit_ = TRUE;
          }
          else if (High_ > Highest_)
          {
            Highest_ = High_;
            Highestx_ = Testx_;
          }
        }

      if (Deriv2_.FloatValue() != Ldouble(0))
        Deriv1_ = Deriv2_;
    }
  }

  if (InclOpt_.strlen())
    InclOpt_.RemoveChar(" \t\n");

  int ExclEnds_ = (NumParams_ >= 5) ? icasestrcomp(InclOpt_.c_str(), "EXCLUDEENDS"):0;

  if (!ExclEnds_)
  {
    FncVal_ = ChrString("(") + Lower_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    Low_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    FncVal_ = ChrString("(") + Upper_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION) + ChrString(")");
    High_ = EvalFncWithValue(ExprStr_, VarName_, FncVal_);

    if (High_ < Low_)
    {
      if (Low_ > Highest_)
        Highestx_ = Lower_;
    }
    else
    {
      if (High_ > Highest_)
        Highestx_ = Upper_;
    }
  }

  SetSuppressOutput(FALSE);
  PopSubExprStack();
  long Exp_ = 0;
  xTol_ = Highestx_;
  AbsoluteValue(xTol_);

  while (Tol_ < Integer(1))
  {
    --Exp_;
    Tol_ *= 10L;
  }

  if (!xTol_.AtZero())
    while (xTol_ < Integer(1))
    {
      --Exp_;
      xTol_ *= 10L;
    }

  if (!Exp_)
    Exp_ = DEFAULT_SIGFIG_SOLVE;

  delete Params_[0];
  delete Params_[1];
  delete Params_[2];
  MemMatrix::Matrix().Deallocate(Params_);

  LongNumber Answer_ = Highestx_;
  Answer_.Round(Exp_);

  if (PlotGraph_)
  {
    _ExtremaEnd = 1;
    ChrString SolString_ = Answer_.ToString();
    PlotFunction(2, ParamArray_, MidPt_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION),
                 RadVal_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION), &SolString_);
    return SolString_;
  }

  return Answer_.ToString();
}

/*****************************************************************************/
ChrString LongDoubleCalculator::RectToPolar(int NumParams_, FncParameter** ParamArray_, int Retarg_)
{
  if (!LongDouble::IsValue(ParamArray_[0]->_Param->GetAnswer()) ||
      !LongDouble::IsValue(ParamArray_[1]->_Param->GetAnswer()))
  {
    if (Retarg_)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "RECTOPOLR");
    else
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "RECTOPOLA");

    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  LongDouble FltVal_ = ChrString(ParamArray_[0]->_Param->GetAnswer());
  Ldouble Xpos_ = FltVal_.FloatValue();
  FltVal_ = ChrString(ParamArray_[1]->_Param->GetAnswer());
  Ldouble Ypos_ = FltVal_.FloatValue();
  Ldouble Xsqr_;
  Ldouble Ysqr_;
  Ldouble Sin_;
  LongDouble Angle_;
  LongDouble Radius_;
  ChrString Error_ = "0";

  if (FABSL(Ypos_) < _Epsilon)
    Ypos_ = 0;

  if (FABSL(Xpos_) < _Epsilon)
    Xpos_ = 0;

  if (!Xpos_ && !Ypos_)
    return Error_;

  Ysqr_ = Ypos_;
  Ysqr_ *= Ypos_;
  Xsqr_ = Xpos_;
  Xsqr_ *= Xpos_;
  Radius_ = Xsqr_;
  Radius_ += LongDouble(Ysqr_);
  Radius_ = SQRTL(Radius_.FloatValue());

  // Return radius
  if (Retarg_)
    return Radius_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);

  Sin_ = Ypos_ / Radius_.FloatValue();
  Angle_ = LongDouble(ASINL(Sin_));
  if (Xpos_ < Ldouble(0))
    if (Ypos_ < Ldouble(0))
      Angle_ = LongDouble(-LongDouble::vPI) - Angle_;
    else
      Angle_ = LongDouble(LongDouble::vPI) - Angle_;

  if (GetAngleUnit() == DEGREE)
  {
    Angle_ /= LongDouble(LongDouble::vPI);
    Angle_ *= 180L;
  }

  return Angle_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
}

/*****************************************************************************/
ChrString LongDoubleCalculator::PolarToRect(int NumParams_, FncParameter** ParamArray_, int Retarg_)
{
  if (!LongDouble::IsValue(ParamArray_[0]->_Param->GetAnswer()) ||
      !LongDouble::IsValue(ParamArray_[1]->_Param->GetAnswer()))
  {
    if (Retarg_)
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "POLTORECX");
    else
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, "POLTORECY");

    CloseOutputFile();
    ChrString Dummy_;
    return Dummy_;
  }

  LongDouble FltVal_ = ChrString(ParamArray_[0]->_Param->GetAnswer());
  Ldouble Radius_ = FltVal_.FloatValue();
  FltVal_ = ChrString(ParamArray_[1]->_Param->GetAnswer());
  Ldouble Angle_ = FltVal_.FloatValue();
  Ldouble Mult_;
  Ldouble Frac_;
  #if HAS_LONGDOUBLE_STDFNCS
    Ldouble iptr_;
  #else
    double iptr_;
  #endif
  int Int_;
  int Odd_;
  LongDouble Xpos_;
  LongDouble Ypos_;
  Ldouble AbsAngle_;
  Ldouble PiDiv2_ = LongDouble::vPI / 2L;
  ChrString Zero_ = "0";

  if (Radius_ < _Epsilon)
    return Zero_;

  if (GetAngleUnit() == DEGREE)
  {
    Angle_ /= 180L;
    Angle_ *= LongDouble::vPI;
  }

  AbsAngle_ = FABSL(Angle_);

  if (FABSL(AbsAngle_ - LongDouble::vPI) > _Epsilon)
  {
    Mult_ = Angle_ / LongDouble::vPI;
    Frac_ = MODFL(Mult_, &iptr_);
    Int_ = (int)FLOORL(iptr_);
    Odd_ = Int_ % 2;

    if (Frac_ > _Epsilon)
    {
      Angle_ =  Frac_ * LongDouble::vPI;

      if (Odd_)
        if (Angle_ > Ldouble(0))
          Angle_ -= LongDouble::vPI;
        else
          Angle_ += LongDouble::vPI;
    }
    else
    {
      if (Odd_)
        Angle_ = LongDouble::vPI;
      else
        Angle_ = 0;
    }
  }

  if (Angle_ >= Ldouble(0))
  {
    if (Angle_ > PiDiv2_)
    {
      Angle_ = LongDouble::vPI - Angle_;
      if (FABSL(Angle_) < _Epsilon)
        Angle_ = 0;
      Xpos_ = -(Radius_ * COSL(Angle_));
    }
    else
    {
      if (FABSL(Angle_) < _Epsilon)
        Angle_ = 0;
      Xpos_ = Radius_ * COSL(Angle_);
    }

    Ypos_ = Radius_ * SINL(Angle_);
  }
  else
  {
    if (Angle_ < -PiDiv2_)
    {
      Angle_ = -LongDouble::vPI - Angle_;
      if (FABSL(Angle_) < _Epsilon)
        Angle_ = 0;
      Angle_ *= -1;
      Xpos_ = -(Radius_ * COSL(Angle_));
    }
    else
    {
      if (FABSL(Angle_) < _Epsilon)
        Angle_ = 0;
      Angle_ *= -1;
      Xpos_ = Radius_ * COSL(Angle_);
    }

    Ypos_ = Radius_ * SINL(Angle_);
    Ypos_ *= -1;
  }

  if (Retarg_)
    return Xpos_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);

  return Ypos_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
}

/*****************************************************************************/
ChrString LongDoubleCalculator::SolveMatrix(int NumParams_, FncParameter** ParamArray_)
{
  int i, x;
  int NumCols_;
  int Max_ = 0;
  int Error_ = FALSE;
  int MatSize_ = 0;
  ChrString AnsCol_;
  ChrString RowStr_;
  ChrString Answer_;
  int* DelimArray_ = (int*)MemMatrix::Matrix().Callocate(sizeof(int) * DELIM_ARRAY_SIZE);
  GVector<Ldouble>** Varr_ = new GVector<Ldouble>*[NumParams_ + 1];
  GVector<Ldouble>* Vect_ = NULL;
  GVector<Ldouble> AnsVect_(NumParams_);
  GMatrix<Ldouble> Mat_(NumParams_);

  for (i = 0; i <= NumParams_; i++)
    Varr_[i] = NULL;

  for (i = 0; i < NumParams_; i++)
  {
    RowStr_ = ParamArray_[i]->_Param->GetExpression();
    CalculatorBase::FindFncDelim(RowStr_.c_str(), RowStr_.strlen(), &DelimArray_, DELIM_ARRAY_SIZE);
    NumCols_ = DelimArray_[0] - 1;

    if (NumCols_ > Max_)
    {
      if (!Max_)
      {
        if (NumCols_ != NumParams_)
        {
          Error_ = TRUE;
          DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_UNSOLVABLE_MATRIX, ERRMSG_UNSOLVABLE_MATRIX);
          CloseOutputFile();
          break;
        }
        else
          Max_ = NumCols_;
      }
      else
      {
        Error_ = TRUE;
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_MATRIX_COLS, ERRMSG_INVALID_MATRIX_COLS);
        CloseOutputFile();
        break;
      }
    }
    else if (!Max_)
    {
      Error_ = TRUE;
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_EMPTYROW_MATRIX, ERRMSG_EMPTYROW_MATRIX);
      CloseOutputFile();
      break;
    }

    int LastCol_ = DelimArray_[NumCols_] + 1;
    char* Divider_ = strchr(RowStr_(LastCol_), '|');

    if (!Divider_)
    {
      Error_ = TRUE;
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_NO_RESULT_COL_MATRIX, ERRMSG_NO_RESULT_COL_MATRIX, RowStr_.c_str());
      CloseOutputFile();
      break;
    }

    if (AnsCol_.strlen())
      AnsCol_ += ", ";
    AnsCol_ += &Divider_[1];
    AnsCol_.RemoveTrailing(")]} \t\n");
    RowStr_.Left(Divider_ - RowStr_(0));
    RowStr_ += ")";

    Vect_ = new GVector<Ldouble>(NumParams_);
    Vect_->SetVector(RowStr_.c_str());
    Varr_[MatSize_++] = Vect_;
  }

  if (!Error_)
  {
    AnsVect_.SetVector(AnsCol_.c_str());
    Mat_.SetMatrix(Varr_);
    GVector<Ldouble> ResVect_ = GaussPivot(Mat_, AnsVect_, NumParams_);
    ResVect_.WriteToString(Answer_);
  }

  MemMatrix::Matrix().Deallocate(DelimArray_);
  for (i = 0; i < NumParams_; i++)
    delete Varr_[i];
  delete[] Varr_;

  SetHasAnswer(Answer_.strlen());

  if (Answer_.strlen())
  {
    Answer_ = ChrString("\"") + Answer_ + ChrString("\"");
    SetLiteralStr(Answer_.c_str());
  }

  return Answer_;
}

/*****************************************************************************/
ChrString LongDoubleCalculator::EvaluateFunction(const char* FncName_, int NumParams_, FncParameter** ParamArray_)
{
  int CallFnc_, DiscFnc_, ConvFnc_, OptionFnc_;
  int optype, x;
  ChrString buffer;
  ChrString Answer_;

  optype = icasestrcomp(FncName_, "MIN")         ? Mcalc_OperatorType::Minimum:
           icasestrcomp(FncName_, "MAX")         ? Mcalc_OperatorType::Maximum:
           icasestrcomp(FncName_, "DERIV")       ? Mcalc_OperatorType::Derivative:
           icasestrcomp(FncName_, "DERIV2")      ? Mcalc_OperatorType::Derivative2:
           icasestrcomp(FncName_, "INTEG")       ? Mcalc_OperatorType::Integral:
           icasestrcomp(FncName_, "SOLVE")       ? Mcalc_OperatorType::RootSolve:
           icasestrcomp(FncName_, "INTERSECT")   ? Mcalc_OperatorType::Intersect:
           icasestrcomp(FncName_, "PERMREP")     ? Mcalc_OperatorType::PermuteWithRep:
           icasestrcomp(FncName_, "COMBREP")     ? Mcalc_OperatorType::CombineWithRep:
           icasestrcomp(FncName_, "COMB")        ? Mcalc_OperatorType::Combination:
           icasestrcomp(FncName_, "PERM")        ? Mcalc_OperatorType::Permutation:
           icasestrcomp(FncName_, "FMIN")        ? Mcalc_OperatorType::FunctionMinimum:
           icasestrcomp(FncName_, "FMAX")        ? Mcalc_OperatorType::FunctionMaximum:
           icasestrcomp(FncName_, "ROUND")       ? Mcalc_OperatorType::RoundEx:
           icasestrcomp(FncName_, "SET")         ? Mcalc_OperatorType::MakeSet:
           icasestrcomp(FncName_, "SETUNI")      ? Mcalc_OperatorType::SetUnion:
           icasestrcomp(FncName_, "SETINT")      ? Mcalc_OperatorType::SetIntersection:
           icasestrcomp(FncName_, "SETDIF")      ? Mcalc_OperatorType::SetDifference:
           icasestrcomp(FncName_, "SETXDIF")     ? Mcalc_OperatorType::SetExclusion:
           icasestrcomp(FncName_, "ITEMPERM")    ? Mcalc_OperatorType::ItemPermutation:
           icasestrcomp(FncName_, "ITEMCOMB")    ? Mcalc_OperatorType::ItemCombination:
           icasestrcomp(FncName_, "ITEMPERMREP") ? Mcalc_OperatorType::ItemPermuteWithRep:
           icasestrcomp(FncName_, "ITEMCOMBREP") ? Mcalc_OperatorType::ItemCombineWithRep:
           icasestrcomp(FncName_, "FRAC")        ? Mcalc_OperatorType::FractionArithmetic:
           icasestrcomp(FncName_, "RECTOPOLR")   ? Mcalc_OperatorType::RectToPolarRadius:
           icasestrcomp(FncName_, "RECTOPOLA")   ? Mcalc_OperatorType::RectToPolarAngle:
           icasestrcomp(FncName_, "POLTORECX")   ? Mcalc_OperatorType::PolarToRectX:
           icasestrcomp(FncName_, "POLTORECY")   ? Mcalc_OperatorType::PolarToRectY:
           icasestrcomp(FncName_, "FROMHEX")     ? Mcalc_OperatorType::FromHex:
           icasestrcomp(FncName_, "FROMOCT")     ? Mcalc_OperatorType::FromOct:
           icasestrcomp(FncName_, "FROMBIN")     ? Mcalc_OperatorType::FromBin:
           icasestrcomp(FncName_, "FROMCMP2BIN") ? Mcalc_OperatorType::FromCmp2Bin:
           icasestrcomp(FncName_, "TOHEX")       ? Mcalc_OperatorType::ToHex:
           icasestrcomp(FncName_, "TOOCT")       ? Mcalc_OperatorType::ToOct:
           icasestrcomp(FncName_, "TOBIN")       ? Mcalc_OperatorType::ToBin:
           icasestrcomp(FncName_, "TOCMP2BIN")   ? Mcalc_OperatorType::ToCmp2Bin:
           icasestrcomp(FncName_, "AVGMEAN")     ? Mcalc_OperatorType::AvgMean:
           icasestrcomp(FncName_, "AVGMED")      ? Mcalc_OperatorType::AvgMedian:
           icasestrcomp(FncName_, "AVGMODE")     ? Mcalc_OperatorType::AvgMode:
           icasestrcomp(FncName_, "STDDEV")      ? Mcalc_OperatorType::StdDev:
           icasestrcomp(FncName_, "SUM")         ? Mcalc_OperatorType::Sum:
           icasestrcomp(FncName_, "PROD")        ? Mcalc_OperatorType::Product:
           icasestrcomp(FncName_, "MATSOLVE")    ? Mcalc_OperatorType::MatrixSolve:
           icasestrcomp(FncName_, "GRAPH")       ? Mcalc_OperatorType::PlotGraph:0;
  DiscFnc_ = optype == Mcalc_OperatorType::ItemPermutation ||
             optype == Mcalc_OperatorType::ItemCombination ||
             optype == Mcalc_OperatorType::ItemCombineWithRep ||
             optype == Mcalc_OperatorType::ItemPermuteWithRep ||
             optype == Mcalc_OperatorType::RectToPolarRadius ||
             optype == Mcalc_OperatorType::RectToPolarAngle ||
             optype == Mcalc_OperatorType::PolarToRectX ||
             optype == Mcalc_OperatorType::PolarToRectY ||
             optype == Mcalc_OperatorType::FromHex ||
             optype == Mcalc_OperatorType::FromOct ||
             optype == Mcalc_OperatorType::FromBin ||
             optype == Mcalc_OperatorType::FromCmp2Bin ||
             optype == Mcalc_OperatorType::ToHex ||
             optype == Mcalc_OperatorType::ToOct ||
             optype == Mcalc_OperatorType::ToBin ||
             optype == Mcalc_OperatorType::ToCmp2Bin ||
             optype == Mcalc_OperatorType::MatrixSolve ||
             optype == Mcalc_OperatorType::Intersect ||
             optype == Mcalc_OperatorType::MakeSet ||
             optype == Mcalc_OperatorType::SetUnion ||
             optype == Mcalc_OperatorType::SetIntersection ||
             optype == Mcalc_OperatorType::SetDifference ||
             optype == Mcalc_OperatorType::SetExclusion;
  CallFnc_ = optype == Mcalc_OperatorType::Derivative ||
             optype == Mcalc_OperatorType::Derivative2 ||
             optype == Mcalc_OperatorType::Integral ||
             optype == Mcalc_OperatorType::RootSolve ||
             optype == Mcalc_OperatorType::FunctionMinimum ||
             optype == Mcalc_OperatorType::FunctionMaximum ||
             optype == Mcalc_OperatorType::PlotGraph ||
             optype == Mcalc_OperatorType::FractionArithmetic ||
             optype == Mcalc_OperatorType::AvgMean ||
             optype == Mcalc_OperatorType::AvgMedian ||
             optype == Mcalc_OperatorType::AvgMode ||
             optype == Mcalc_OperatorType::StdDev ||
             optype == Mcalc_OperatorType::Sum ||
             optype == Mcalc_OperatorType::Product ||
             (NumParams_ != 2 && (optype == Mcalc_OperatorType::Minimum   ||
                                  optype == Mcalc_OperatorType::Maximum)) ||
             DiscFnc_;
  ConvFnc_ = optype == Mcalc_OperatorType::ToHex ||
             optype == Mcalc_OperatorType::ToOct ||
             optype == Mcalc_OperatorType::ToBin ||
             optype == Mcalc_OperatorType::ToCmp2Bin ||
             optype == Mcalc_OperatorType::FromHex ||
             optype == Mcalc_OperatorType::FromOct ||
             optype == Mcalc_OperatorType::FromBin ||
             optype == Mcalc_OperatorType::FromCmp2Bin;
  OptionFnc_ = optype == Mcalc_OperatorType::Sum ||
               optype == Mcalc_OperatorType::Product ||
               optype == Mcalc_OperatorType::AvgMean ||
               optype == Mcalc_OperatorType::AvgMedian ||
               optype == Mcalc_OperatorType::AvgMode ||
               optype == Mcalc_OperatorType::StdDev ||
               (NumParams_ != 2 && (optype == Mcalc_OperatorType::Minimum   ||
                                    optype == Mcalc_OperatorType::Maximum)) ||
               optype == Mcalc_OperatorType::ItemPermutation ||
               optype == Mcalc_OperatorType::ItemCombination ||
               optype == Mcalc_OperatorType::ItemCombineWithRep ||
               optype == Mcalc_OperatorType::ItemPermuteWithRep ||
               optype == Mcalc_OperatorType::FractionArithmetic ||
               optype == Mcalc_OperatorType::MakeSet;

  int ErrorSet_;
  ErrorSet_ = (!optype || NumParams_ < 2) ? (!((ConvFnc_ || OptionFnc_) && NumParams_ == 1) && (NumParams_ < 2)):
              (optype == Mcalc_OperatorType::Derivative) ? (NumParams_ < 3):
              (optype == Mcalc_OperatorType::Integral) ? (NumParams_ < 4):
              (optype == Mcalc_OperatorType::RootSolve) ? (NumParams_ < 3):
              (optype == Mcalc_OperatorType::Intersect) ? (NumParams_ < 2):
              (optype == Mcalc_OperatorType::FunctionMinimum) ? (NumParams_ < 4):
              (optype == Mcalc_OperatorType::FunctionMaximum) ? (NumParams_ < 4):0;

  if (ErrorSet_)
  {
    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_NUMARGS,
                        ERRMSG_INVALID_NUMARGS, FncName_);
    CloseOutputFile();
    return buffer;
  }

  LongDouble resultvalue;
  LongDouble** params = NULL;
  LongNumber LnumResult_;
  LongNumber** lparams = NULL;
  double Tol_ = 0;
  long Exp_ = 0;
  long Rexp_;

  int IntOps_ = (optype == Mcalc_OperatorType::Permutation ||
                 optype == Mcalc_OperatorType::Combination ||
                 optype == Mcalc_OperatorType::CombineWithRep ||
                 optype == Mcalc_OperatorType::PermuteWithRep ||
                 optype == Mcalc_OperatorType::RoundEx);

  if (!CallFnc_)
  {
    if (IntOps_)
      lparams = (LongNumber**)allocmem(NumParams_, sizeof(LongNumber*), params);
    else
      params = (LongDouble**)allocmem(NumParams_, sizeof(LongDouble*), params);

    for (x = 0; x < NumParams_; x++)
    {
      buffer = ParamArray_[x]->_Param->GetAnswer();

      if (LongDouble::IsValue(buffer))
      {
        if (IntOps_)
          lparams[x] = new LongNumber(buffer);
        else
          params[x] = new LongDouble(buffer);
      }
      else
      {
        DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, FncName_);
        CloseOutputFile();
        return buffer;
      }
    }
  }

  ResetAnswerMode();
  SetAnswerMode(FLOATINGPOINT_MODE);

  /* finding the result value and remainder value */
  switch (optype)
  {
    case Mcalc_OperatorType::PlotGraph:
      resultvalue = PlotFunction(NumParams_, ParamArray_, "0");
      break;

    case Mcalc_OperatorType::Permutation:
      LnumResult_ = Discrete<LongNumber>::Permutation(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Combination:
      LnumResult_ = Discrete<LongNumber>::Combination(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::CombineWithRep:
      LnumResult_ = Discrete<LongNumber>::CombinationWithRep(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::PermuteWithRep:
      LnumResult_ = Discrete<LongNumber>::PermutationWithRep(*lparams[0], *lparams[1]);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Minimum:
      if (NumParams_ == 2)
        resultvalue = (*params[0] < *params[1]) ? *params[0]:*params[1];
      else
        resultvalue = MinimumValue(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Maximum:
      if (NumParams_ == 2)
        resultvalue = (*params[0] > *params[1]) ? *params[0]:*params[1];
      else
        resultvalue = MaximumValue(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RoundEx:
      LnumResult_ = *lparams[0];
      Rexp_ = atol(ParamArray_[1]->_Param->GetAnswer());
      LnumResult_.Round(Rexp_);
      resultvalue.Assign(LnumResult_);
      break;

    case Mcalc_OperatorType::Derivative:
      Tol_ = DEFAULT_PRECVAL_DERIV;
      resultvalue = FindDerivative(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Derivative2:
      Tol_ = DEFAULT_PRECVAL_DERIV;
      resultvalue = FindDerivative2(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Integral:
      Tol_ = DEFAULT_PRECVAL_INTEG;
      resultvalue = FindIntegral(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RootSolve:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindRoot(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Intersect:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindIntersections(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FunctionMinimum:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindMinimum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FunctionMaximum:
      Tol_ = DEFAULT_PRECVAL_SOLVE;
      resultvalue = FindMaximum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::MakeSet:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = CreateSet(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetUnion:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetUnion(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetIntersection:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetIntersection(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetDifference:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetDifference(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::SetExclusion:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindSetExclusion(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemPermutation:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindPermutations(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemCombination:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindCombinations(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemCombineWithRep:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindCombinationsWithRep(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ItemPermuteWithRep:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = FindPermutationsWithRep(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::MatrixSolve:
      SetAnswerMode(LITERAL_MODE, TRUE);
      Answer_ = SolveMatrix(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FractionArithmetic:
      SetAnswerMode(FRACTION_MODE);
      resultvalue = FindFractionArithmetic(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::RectToPolarRadius:
      Answer_ = RectToPolar(NumParams_, ParamArray_, 1);
      break;

    case Mcalc_OperatorType::RectToPolarAngle:
      Answer_ = RectToPolar(NumParams_, ParamArray_, 0);
      break;

    case Mcalc_OperatorType::PolarToRectX:
      Answer_ = PolarToRect(NumParams_, ParamArray_, 1);
      break;

    case Mcalc_OperatorType::PolarToRectY:
      Answer_ = PolarToRect(NumParams_, ParamArray_, 0);
      break;

    case Mcalc_OperatorType::FromHex:
      Answer_ = ConvertFromHex(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromOct:
      Answer_ = ConvertFromOct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromBin:
      Answer_ = ConvertFromBin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::FromCmp2Bin:
      Answer_ = ConvertFromCmp2Bin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToHex:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToHex(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToOct:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToOct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToBin:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToBin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::ToCmp2Bin:
      SetAnswerMode(ALTBASE_MODE);
      Answer_ = ConvertToCmp2Bin(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Sum:
      resultvalue = CumSum(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::Product:
      resultvalue = CumProduct(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMean:
      resultvalue = AverageMean(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMedian:
      resultvalue = AverageMedian(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::AvgMode:
      resultvalue = AverageMode(NumParams_, ParamArray_);
      break;

    case Mcalc_OperatorType::StdDev:
      resultvalue = StandardDeviation(NumParams_, ParamArray_);
      break;

    default:
      buffer = "";
      ErrorSet_ = 1;
  }

  if (!CallFnc_)
    if (IntOps_)
    {
      for (x = 0; x < NumParams_; x++)
        delete lparams[x];
      MemMatrix::Matrix().Deallocate(lparams);
      lparams = NULL;
    }
    else
    {
      for (x = 0; x < NumParams_; x++)
        delete params[x];
      MemMatrix::Matrix().Deallocate(params);
      params = NULL;
    }

  if (Tol_ != double(0))
  {
    while (Tol_ < 1)
    {
      ++Exp_;
      Tol_ *= 10L;
    }

    if (!DiscFnc_ && !(ErrorSet_ || HasErrors()))
    {
      LongDouble xTol_ = resultvalue;
      AbsoluteValue(xTol_);
      if (!xTol_.AtZero())
        while (xTol_ < Integer(1))
        {
          ++Exp_;
          xTol_ *= 10L;
        }
    }

    if (!Exp_)
      Exp_ = LongDouble::TYPE_LONGDOUBLE_PRECISION;
  }

  if (!DiscFnc_)
    Answer_ = (ErrorSet_ || HasErrors()) ? buffer:
                  (Exp_ ? resultvalue.ToString(Exp_):
                          resultvalue.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION));
  else if (ErrorSet_ || HasErrors())
    Answer_ = buffer;

  return Answer_;
}

/*****************************************************************************/
size_t LongDoubleCalculator::FindPrecision(const Ldouble& Val_, size_t Prec_)
{
  Ldouble Cval_ = Val_;
  Ceiling(Cval_);

  size_t x;
  for (x = 0; x < Prec_; x++)
    if (Cval_ > Integer(1))
      Cval_ /= 10L;
    else
      break;

  x = Prec_ - x;
  if (x > 0)
    return x;

  return Prec_;
}

/*****************************************************************************/
#if HAS_LONGDOUBLE_STDFNCS
bool LongDoubleCalculator::FindCommonAngles(Ldouble value_, Ldouble shift_, Ldouble Tol_,
                                            Ldouble Eps_, Ldouble& mult_, Ldouble& intp_)
#else
bool LongDoubleCalculator::FindCommonAngles(Ldouble value_, Ldouble shift_, Ldouble Tol_,
                                            Ldouble Eps_, Ldouble& mult_, double& intp_)
#endif
{
  Ldouble tval1;
  Ldouble tval2;
  Ldouble Multiplier_;

  tval2 = value_;
  AbsoluteValue(tval2);
  tval2 += shift_;
  tval1 = tval2 / LongDouble::vPI;
  Multiplier_ = POWL(10, FindPrecision(tval1, size_t(-log10(Eps_))));
  tval2 = tval1;
  tval2 += Tol_;

  tval2 = MODFL(tval2, &intp_);
  tval2 = tval1;
  tval1 *= Multiplier_;
  tval1 += .5;
  tval1 = FLOORL(tval1);
  tval1 /= Multiplier_;
  tval2 *= Multiplier_;
  tval2 += .5;
  tval2 = FLOORL(tval2);
  tval2 /= Multiplier_;
  tval2 = CEILL(tval2);
  mult_ = tval1;

  return (tval1 == tval2);
}

/*****************************************************************************/
// Procedure to directly process the values given to the program using built-in
// math operators if the length of the arguments permit
// Procedure executes only if OPTIMIZE flag is TRUE
// PASSED:
//   operands : the operands array
//   arglist : the argument list
//   optype : the math operator
// USES:
//   uses allocmem to allocate memory
//   uses allocatevalue to allocate memory to the value array of a operand
//        structure
// READ:
//   arglist is read for the arguments to the program and if within acceptable
//   length they are converted to decimal values and the result calculated
// CHANGED:
//   the result and remainder structures have their value array changed
//   to the result value and remainder value if argument length conditions are
//   met
// RETURNED:
//   returns 1 if calculation could be done directly, otherwise returns 0
//   returns 0 if OPTIMIZE flag is FALSE
//
const Ldouble& LongDoubleCalculator::DirectProcess(Mcalc_Operand** operands, char** arglist, int optype)
{
  #if defined(__DJGPP__)
    // using std::pow;
    using ::pow;
    using ::sqrt;
  #endif

  ChrString buffer;
  int IntVal_;
  int dividebyzero = 0;           /* divide by zero flag */
  Ldouble value1, value2;         /* values of respective operands */
  Ulong intval1, intval2;         /* integer values of respective operands */
  int x, i, len;
  Mcalc_Operand *resultptr = operands[RESULTINDEX];
  int TrigOps_;
  long ExpVal_;
  #if HAS_LONGDOUBLE_STDFNCS
    Ldouble ip[8];
    Ldouble intp_;
  #else
    double ip[8];
    double intp_;
  #endif

  Ldouble PiVal_ = LongDouble(M_PIl_STRING).FloatValue();
  Ldouble Tol_;
  Ldouble tval_[8];
  bool neg_;
  bool Match_;
  bool Match2_;
  int cd1, cd2;


  if (!StringComparison())
  {
    int FirstArgNotUsed_ = (optype == Mcalc_OperatorType::Square) ||
                           (optype == Mcalc_OperatorType::SquareRoot) ||
                           (optype == Mcalc_OperatorType::Cube) ||
                           (optype == Mcalc_OperatorType::CubeRoot);

    if ((!FirstArgNotUsed_ &&
         !(((Uchar)(*arglist[2])) == NAN_CONST || ((Uchar)(*arglist[2])) == INF_CONST || LongDouble::IsValue(arglist[2]))) ||
         !(((Uchar)(*arglist[3])) == NAN_CONST || ((Uchar)(*arglist[3])) == INF_CONST || LongDouble::IsValue(arglist[3])))
    {
      DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_INVALID_TYPEARGS,
                          ERRMSG_INVALID_TYPEARGS, GetFncNameFromOpcode(optype));
      CloseOutputFile();
      resultvalue = 0;
      return resultvalue;
    }

    if (((Uchar)(*arglist[2])) == NAN_CONST)
      value1 = LongDouble::vNAN;
    else if (((Uchar)(*arglist[2])) == INF_CONST)
      value1 = LongDouble::vINFINITY;
    else
    {
      value1 = (optype == Mcalc_OperatorType::Square)       ? 2:
               (optype == Mcalc_OperatorType::SquareRoot)   ? .5:
               (optype == Mcalc_OperatorType::Cube)         ? 3:
               (optype == Mcalc_OperatorType::CubeRoot)     ? (1.0/3):
               strtold(arglist[2], NULL);
    }

    if (((Uchar)(*arglist[3])) == NAN_CONST)
      value2 = LongDouble::vNAN;
    else if (((Uchar)(*arglist[3])) == INF_CONST)
      value2 = LongDouble::vINFINITY;
    else
    {
      value2 = (optype == Mcalc_OperatorType::Percent)      ? 100:
               strtold(arglist[3],NULL);
    }

    TrigOps_ =
    (optype == Mcalc_OperatorType::Sine)          ||
    (optype == Mcalc_OperatorType::Cosine)        ||
    (optype == Mcalc_OperatorType::Tangent)       ||
    (optype == Mcalc_OperatorType::HyperSine)     ||
    (optype == Mcalc_OperatorType::HyperCosine)   ||
    (optype == Mcalc_OperatorType::HyperTangent);

    if (GetAngleUnit() == DEGREE && TrigOps_)
    {
      value2 /= 180L;
      value2 *= LongDouble::vPI;
    }
  }

  /* finding the result value and remainder value */
  switch (optype)
  {
    case Mcalc_OperatorType::Logical_Not:
      intval2 = Ulong(FLOORL(value2));
      resultvalue = !intval2;
      break;
    case Mcalc_OperatorType::Sine:
      neg_ = value2 < Ldouble(0);
      Tol_ = Ldouble(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= PiVal_;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == Ldouble(0) || Match_)
      {
        resultvalue = 0;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * PiVal_ / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[1]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[7]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = .5;
        else
          resultvalue = -.5;
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * PiVal_ / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[2]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[6]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(2) / 2;
        else
          resultvalue = -(::sqrt(2) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * PiVal_ / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = (long(ip[3]) % 2L) ? 1:-1;
        else
          cd1 = (long(ip[5]) % 2L) ? 1:-1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3) / 2;
        else
          resultvalue = -(::sqrt(3) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, PiVal_ / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        cd1 = (long(ip[4]) % 2L) ? 1:-1;
        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1L;
        else
          resultvalue = -1L;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        Ldouble lower_ = ip[0] - Tol_;
        Ldouble upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          cd1 = (long(ip[4]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = 1L;
          else
            resultvalue = -1L;
          break;
        }

        lower_ = ip[1] - Tol_;
        upper_ = ip[1] + Tol_;

        if (lower_ < tval_[1] && tval_[1] < upper_)
        {
          cd1 = (long(ip[1]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }

        lower_ = ip[7] - Tol_;
        upper_ = ip[7] + Tol_;

        if (lower_ < tval_[7] && tval_[7] < upper_)
        {
          cd1 = (long(ip[7]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }
      }

      resultvalue = sin(value2);
      break;
    case Mcalc_OperatorType::Cosine:
      neg_ = value2 < Ldouble(0);
      Tol_ = Ldouble(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= PiVal_;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == Ldouble(0) || Match_)
      {
        cd2 = (long(ip[0]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = 1L;
        else
          resultvalue = -1L;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * PiVal_ / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[1]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[7]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3) / 2;
        else
          resultvalue = -(::sqrt(3) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * PiVal_ / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[2]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[6]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = ::sqrt(2) / 2;
        else
          resultvalue = -(::sqrt(2) / 2);
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * PiVal_ / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd2 = (long(ip[3]) % 2L) ? 1:-1;
        else
          cd2 = (long(ip[5]) % 2L) ? -1:1;

        if (cd2 > 0)
          resultvalue = .5;
        else
          resultvalue = -.5;
        break;
      }

      Match_ = FindCommonAngles(value2, PiVal_ / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        resultvalue = 0;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        Ldouble lower_ = ip[0] - Tol_;
        Ldouble upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          cd2 = (long(ip[0]) % 2L) ? -1:1;

          if (cd2 > 0)
            resultvalue = 1L;
          else
            resultvalue = -1L;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[3] - Tol_;
        upper_ = ip[3] + Tol_;

        if (lower_ < tval_[3] && tval_[3] < upper_)
        {
          cd2 = (long(ip[3]) % 2L) ? 1:-1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }

        lower_ = ip[5] - Tol_;
        upper_ = ip[5] + Tol_;

        if (lower_ < tval_[5] && tval_[5] < upper_)
        {
          cd2 = (long(ip[5]) % 2L) ? -1:1;

          if (cd2 > 0)
            resultvalue = .5;
          else
            resultvalue = -.5;
          break;
        }
      }

      resultvalue = cos(value2);
      break;
    case Mcalc_OperatorType::Tangent:
      neg_ = value2 < Ldouble(0);
      Tol_ = Ldouble(DEFAULT_PRECVAL_GRAPH);
      Tol_ /= PiVal_;

      Match_ = FindCommonAngles(value2, 0, Tol_, _Epsilon, tval_[0], ip[0]);
      if (value2 == Ldouble(0) || Match_)
      {
        resultvalue = 0;
        break;
      }

      Match_ = FindCommonAngles(value2, 5 * PiVal_ / 6, Tol_, _Epsilon, tval_[1], ip[1]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 6, Tol_, _Epsilon, tval_[7], ip[7]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1 / ::sqrt(3);
        else
          resultvalue = -(1 / ::sqrt(3));
        break;
      }

      Match_ = FindCommonAngles(value2, 3 * PiVal_ / 4, Tol_, _Epsilon, tval_[2], ip[2]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 4, Tol_, _Epsilon, tval_[6], ip[6]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = 1;
        else
          resultvalue = -1;
        break;
      }

      Match_ = FindCommonAngles(value2, 2 * PiVal_ / 3, Tol_, _Epsilon, tval_[3], ip[3]);
      Match2_ = FindCommonAngles(value2, PiVal_ / 3, Tol_, _Epsilon, tval_[5], ip[5]);
      if (Match_ || Match2_)
      {
        if (Match_)
          cd1 = 1;
        else
          cd1 = -1;

        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = ::sqrt(3);
        else
          resultvalue = -(::sqrt(3));
        break;
      }

      Match_ = FindCommonAngles(value2, PiVal_ / 2, Tol_, _Epsilon, tval_[4], ip[4]);
      if (Match_)
      {
        cd1 = (long(ip[4]) % 2L) ? 1:-1;
        cd2 = neg_ ? -1:1;
        cd2 *= cd1;

        if (cd2 > 0)
          resultvalue = LongDouble::vINFINITY;
        else
          resultvalue = -LongDouble::vINFINITY;
        break;
      }

      if (_PlottingGraph && !_PlotAllPoints)
      {
        Ldouble lower_ = ip[0] - Tol_;
        Ldouble upper_ = ip[0] + Tol_;

        if (lower_ < tval_[0] && tval_[0] < upper_)
        {
          resultvalue = 0;
          break;
        }

        lower_ = ip[4] - Tol_;
        upper_ = ip[4] + Tol_;

        if (lower_ < tval_[4] && tval_[4] < upper_)
        {
          cd1 = (long(ip[4]) % 2L) ? 1:-1;
          cd2 = neg_ ? -1:1;
          cd2 *= cd1;

          if (cd2 > 0)
            resultvalue = LongDouble::vINFINITY;
          else
            resultvalue = -LongDouble::vINFINITY;
          break;
        }

        lower_ = ip[2] - Tol_;
        upper_ = ip[2] + Tol_;

        if (lower_ < tval_[2] && tval_[2] < upper_)
        {
          cd2 = neg_ ? -1:1;
          if (cd2 > 0)
            resultvalue = 1;
          else
            resultvalue = -1;
          break;
        }

        lower_ = ip[6] - Tol_;
        upper_ = ip[6] + Tol_;

        if (lower_ < tval_[6] && tval_[6] < upper_)
        {
          cd2 = neg_ ? 1:-1;
          if (cd2 > 0)
            resultvalue = 1;
          else
            resultvalue = -1;
          break;
        }
      }

      resultvalue = tan(value2);
      break;
    case Mcalc_OperatorType::ArcSine:
      resultvalue = ASINL(value2);
      break;
    case Mcalc_OperatorType::ArcCosine:
      resultvalue = ACOSL(value2);
      break;
    case Mcalc_OperatorType::ArcTangent:
      resultvalue = ATANL(value2);
      break;
    case Mcalc_OperatorType::Add:
      resultvalue = value1 + value2;
      break;
    case Mcalc_OperatorType::Subtract:
      resultvalue = value1 - value2;
      break;
    case Mcalc_OperatorType::Multiply:
      resultvalue = value1 * value2;
      break;
    case Mcalc_OperatorType::Divide:
      dividebyzero = value2 == Ldouble(0);
      if (!dividebyzero)
      {
        if (_PlottingGraph && !_PlotAllPoints)
        {
          Tol_ = LongDouble(DEFAULT_PRECSTR_GRAPH);
          tval_[0] = -Tol_;
          tval_[1] = Tol_;

          if (tval_[0] < value2 && value2 < tval_[1])
          {
            int dsign_ = (value2 < Ldouble(0)) ? -1:1;
            int nsign_ = (value1 < Ldouble(0)) ? -1:1;
            nsign_ *= dsign_;

            if (nsign_ > 0)
              resultvalue = LongDouble::vINFINITY;
            else
              resultvalue = -LongDouble::vINFINITY;
            break;
          }
        }

        resultvalue = value1 / value2;

        if (_QLookAhead && *_QLookAhead == '^')
        {
          Ldouble val1a = value1 + _Epsilon;
          Ldouble val2a = value2 + _Epsilon;
          Ldouble val1b = value1 - _Epsilon;
          Ldouble val2b = value2 - _Epsilon;

          bool arg2ok = ::IsLong(arglist[2]);
          bool arg3ok = ::IsLong(arglist[3]);
          bool val1aok = !arg2ok && FLOORL(value1 + _Epsilon) == val1a;
          bool val1bok = !arg2ok && FLOORL(value1) == val1b;
          bool val2aok = !arg3ok && FLOORL(value2 + _Epsilon) == val2a;
          bool val2bok = !arg3ok && FLOORL(value2) == val2b;

          if ((arg2ok && arg3ok) ||
              ((val1aok || val1bok) && (val2aok || val2bok)))
          {
            LongNumber val1;
            LongNumber val2;
            LongDouble Tempf_;

            if (arg2ok && arg3ok)
            {
              val1 = ChrString(arglist[2]);
              val2 = ChrString(arglist[3]);
            }
            else if (val1aok)
            {
              Tempf_ = val1a;
              val1 = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);

              if (val2aok)
              {
                Tempf_ = val2a;
                val2 = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
              }
              else
              {
                Tempf_ = val2b;
                val2 = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
              }
            }
            else if (val1bok)
            {
              Tempf_ = val1b;
              val1 = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);

              if (val2aok)
              {
                Tempf_ = val2a;
                val2 = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
              }
              else
              {
                Tempf_ = val2b;
                val2 = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
              }
            }

            val1 %= 2L;
            val2 %= 2L;

            if (!val2.AtZero())
              _OddRoot = val1.AtZero() ? 2:1;
            else
              _OddRoot = 0;
          }
        }
      }
      else
      {
        if (_PlottingGraph)
        {
          int dsign_ = (value2 < Ldouble(0)) ? -1:1;
          int nsign_ = (value1 < Ldouble(0)) ? -1:1;
          nsign_ *= dsign_;

          if (nsign_ > 0)
            resultvalue = LongDouble::vINFINITY;
          else
            resultvalue = -LongDouble::vINFINITY;
        }
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::Modulus:
      intval1 = Ulong(FLOORL(value1));
      intval2 = Ulong(FLOORL(value2));
      dividebyzero = !intval2;
      if (!dividebyzero)
        resultvalue = intval1 % intval2;
      else
      {
        if (_PlottingGraph)
          resultvalue = LongDouble::vNAN;
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::Logical_And:
      intval1 = Ulong(FLOORL(value1));
      intval2 = Ulong(FLOORL(value2));
      resultvalue = intval1 && intval2;
      break;
    case Mcalc_OperatorType::Logical_Or:
      intval1 = Ulong(FLOORL(value1));
      intval2 = Ulong(FLOORL(value2));
      resultvalue = intval1 || intval2;
      break;
    case Mcalc_OperatorType::Exponent:
      if (value2 == Ldouble(0))
        resultvalue = 1L;
      else if (::IsLong(arglist[3]))
      {
        ExpVal_ = atol(arglist[3]);
        resultvalue = LongDoubleCalculator::IntPow(value1, ExpVal_);
      }
      else
      {
        if (value1 < Ldouble(0))
        {
          if (_OddRoot)
          {
            resultvalue = POWL(-value1,value2);
            if (_OddRoot == 1)
              resultvalue = -resultvalue;
          }
          else
            resultvalue = POWL(value1,value2);
        }
        else
          resultvalue = POWL(value1,value2);
      }
      break;
    case Mcalc_OperatorType::Square:
      resultvalue = POWL(value2,2);
      break;
    case Mcalc_OperatorType::SquareRoot:
      resultvalue = POWL(value2,.5);
      break;
    case Mcalc_OperatorType::Cube:
      resultvalue = POWL(value2,3);
      break;
    case Mcalc_OperatorType::CubeRoot:
      if (value2 < Ldouble(0))
        resultvalue = -POWL(-value2,(1.0/3));
      else
        resultvalue = POWL(value2,(1.0/3));
      break;
    case Mcalc_OperatorType::HyperSine:
      resultvalue = SINHL(value2);
      break;
    case Mcalc_OperatorType::HyperCosine:
      resultvalue = COSHL(value2);
      break;
    case Mcalc_OperatorType::HyperTangent:
      resultvalue = TANHL(value2);
      break;
    case Mcalc_OperatorType::NaturalExp:
      resultvalue = EXPL(value2);
      break;
    case Mcalc_OperatorType::NaturalLog:
      resultvalue = LOGL(value2);
      break;
    case Mcalc_OperatorType::Log10:
      resultvalue = LOG10L(value2);
      break;
    case Mcalc_OperatorType::Ceiling:
      resultvalue = CEILL(value2);
      break;
    case Mcalc_OperatorType::Floor:
      resultvalue = FLOORL(value2);
      break;
    case Mcalc_OperatorType::AbsoluteValue:
      resultvalue = FABSL(value2);
      break;
    case Mcalc_OperatorType::FractionPart:
      resultvalue = MODFL(value2, &intp_);
      break;
    case Mcalc_OperatorType::IntegerPart:
      MODFL(value2, &intp_);
      resultvalue = intp_;
      break;
    case Mcalc_OperatorType::Percent:
      resultvalue = value1 / value2;
      break;
    case Mcalc_OperatorType::LessThan:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = Ldouble(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 < value2;
      break;
    case Mcalc_OperatorType::GreaterThan:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = Ldouble(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 > value2;
      break;
    case Mcalc_OperatorType::LessOrEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = Ldouble(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 <= value2;
      break;
    case Mcalc_OperatorType::GreaterOrEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = Ldouble(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 >= value2;
      break;
    case Mcalc_OperatorType::Equal:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = Ldouble(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 == value2;
      break;
    case Mcalc_OperatorType::NotEqual:
      if (StringComparison())
      {
        IntVal_ = CompareStrings(arglist[2], arglist[3], optype);
        resultvalue = Ldouble(IntVal_ ? 1:0);
      }
      else
        resultvalue = value1 != value2;
      break;
    case Mcalc_OperatorType::Round:
      resultvalue = LongDouble(value2).Round();
      break;
    case Mcalc_OperatorType::Factorial:
      resultvalue = Discrete<Ulong>::Factorial((Ulong)value2);
      break;
    case Mcalc_OperatorType::Permutation:
      resultvalue = Discrete<Ulong>::Permutation((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::Combination:
      resultvalue = Discrete<Ulong>::Combination((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::CombineWithRep:
      resultvalue = Discrete<Ulong>::CombinationWithRep((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::PermuteWithRep:
      resultvalue = Discrete<Ulong>::PermutationWithRep((Ulong)value1, (Ulong)value2);
      break;
    case Mcalc_OperatorType::Assignment:
      resultvalue = value2;
      break;
    case Mcalc_OperatorType::Random:
      IntVal_ = atoi(arglist[3]);
      resultvalue = GiveRandom(IntVal_);
      break;
    case Mcalc_OperatorType::Exclusive_Or:
      intval1 = Ulong(FLOORL(value1));
      intval2 = Ulong(FLOORL(value2));
      resultvalue = !(intval1 && intval2) && (intval1 || intval2);
      break;
    case Mcalc_OperatorType::Inverse:
      dividebyzero = value2 == Ldouble(0);
      if (!dividebyzero)
      {
        if (_PlottingGraph && !_PlotAllPoints)
        {
          Tol_ = LongDouble(DEFAULT_PRECSTR_GRAPH);
          tval_[0] = -Tol_;
          tval_[1] = Tol_;

          if (tval_[0] < value2 && value2 < tval_[1])
          {
            if (value2 < Ldouble(0))
              resultvalue = -LongDouble::vINFINITY;
            else
              resultvalue = LongDouble::vINFINITY;
            break;
          }
        }

        resultvalue = Ldouble(1 / value2);

        if (_QLookAhead && *_QLookAhead == '^')
        {
          Ldouble val2a = value2 + _Epsilon;
          Ldouble val2b = value2 - _Epsilon;

          bool arg3ok = ::IsLong(arglist[3]);
          bool val2aok = !arg3ok && FLOORL(value2 + _Epsilon) == val2a;
          bool val2bok = !arg3ok && FLOORL(value2) == val2b;

          if (arg3ok || val2aok || val2bok)
          {
            LongNumber val2;
            LongDouble Tempf_;

            if (arg3ok)
              val2 = ChrString(arglist[3]);
            else if (val2aok)
            {
              Tempf_ = val2a;
              val2 = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
            }
            else
            {
              Tempf_ = val2b;
              val2 = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
            }

            val2 %= 2L;

            if (!val2.AtZero())
              _OddRoot = 1;
            else
              _OddRoot = 0;
          }
        }
      }
      else
      {
        if (_PlottingGraph)
        {
          if (value2 < Ldouble(0))
            resultvalue = -LongDouble::vINFINITY;
          else
            resultvalue = LongDouble::vINFINITY;
        }
        else
          resultptr->size = 0;
      }
      break;
    case Mcalc_OperatorType::ScientificNotation:
      if (::IsNumber(arglist[3], TRUE, FALSE, FALSE, 10))
      {
        ExpVal_ = atol(arglist[3]);
        value2 = POWL(10, ExpVal_);
        resultvalue = value1 * value2;
      }
      break;
  }

  TrigOps_ =
  (optype == Mcalc_OperatorType::ArcSine)          ||
  (optype == Mcalc_OperatorType::ArcCosine)        ||
  (optype == Mcalc_OperatorType::ArcTangent);

  if (GetAngleUnit() == DEGREE && TrigOps_)
  {
    resultvalue /= LongDouble::vPI;
    resultvalue *= 180L;
  }

  if (!dividebyzero)
  {
    if (LongDouble::IsInfinite(resultvalue))
    {
      if (resultvalue < Ldouble(0))
        SetLiteralStr("-Inf");
      else
        SetLiteralStr("+Inf");
      ResetAnswerMode();
      SetAnswerMode(LITERAL_MODE, TRUE);
      clearvalue(resultptr);
    }
    else if (LongDouble::IsNan(resultvalue))
    {
      SetLiteralStr("Nan");
      ResetAnswerMode();
      SetAnswerMode(LITERAL_MODE, TRUE);
      clearvalue(resultptr);
    }
    else
    {
      // converting decimal result value to array digit elements
      Ldouble absresultval = resultvalue;
      AbsoluteValue(absresultval);

      LongDouble Tempf_(absresultval);
      buffer = Tempf_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      len = buffer.strlen();

      allocatevalue(resultptr, len);
      resultptr->sign = (resultvalue < 0) ? Mcalc_SignType::Minus:Mcalc_SignType::Plus;
      strcpy(resultptr->value, buffer.c_str());
    }
  }
  else if (!_PlottingGraph)
  {
    ChrString Numer_, Denom_;
    LongDouble Temp_;

    if (optype == Mcalc_OperatorType::Divide ||
        optype == Mcalc_OperatorType::Inverse)
    {
      if (optype == Mcalc_OperatorType::Inverse)
        Numer_ = "1";
      else
      {
        Temp_ = value1;
        Numer_ = Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      }

      Temp_ = value2;
      Denom_ = Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      Numer_ += ChrString("/") + Denom_;
    }
    else if (optype == Mcalc_OperatorType::Modulus)
    {
      Temp_ = intval1;
      Numer_ = Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      Temp_ = intval2;
      Denom_ = Temp_.ToString(LongDouble::TYPE_LONGDOUBLE_PRECISION);
      Numer_ += ChrString("@") + Denom_;
    }

    DisplayErrorMessage(GetOutputFile(), Mcalc_Error::ERRVAL_DIVISION_BY_ZERO,
                        ERRMSG_DIVISION_BY_ZERO, Numer_.c_str());
    CloseOutputFile();
  }

  return resultvalue;
}

/*****************************************************************************/
const Ldouble& LongDoubleCalculator::GetResult() const
{
  return resultvalue;
}

/*****************************************************************************/
const Ldouble& LongDoubleCalculator::GetVariable(int Index_) const
{
  static Ldouble Dummy_ = 0;
  if (Index_ >= MAXVARS || Index_ < 0)
    return Dummy_;

  return _vars[Index_];
}

/*****************************************************************************/
size_t LongDoubleCalculator::GetVariableLength(int Index_) const
{
  if (Index_ < 0)
    return 0;

  if (IsStringVar(Index_))
    return GetStrVarLen(Index_);
  else if (Index_ >= MAXVARS)
    return 0;

  char* ValStr_ = (char*)MemMatrix::Matrix().Allocate(32);
  size_t len = strlen(M_El_STRING);
  size_t prec = len / 2;

  sprintf(ValStr_, "%-*.*Lg", len, prec, _vars[Index_]);
  len = strlen(ValStr_);
  MemMatrix::Matrix().Deallocate(ValStr_);
  ValStr_ = NULL;
  return len;
}

/*****************************************************************************/
void LongDoubleCalculator::SetVariable(int Index_, const Ldouble& Value1_, int Transfer_)
{
  int x;
  int AssignType_;
  Ldouble Value_;
  bool AltBase_ = StringAssignment() && AnswerMode() == ALTBASE_MODE;
  LongNumber LongValue_;

  if (Transfer_ || !AltBase_)
  {
    Value_ = Value1_;
    AssignType_ = ASSIGN_ONLY;
  }
  else
  {
    ChrString ValStr_ = GetStrResult();
    LongValue_ = ValStr_;
    Value_ = strtol(ValStr_.c_str(), NULL, 0);
  }

  if (!Transfer_)
    GetIndex(x, AssignType_);

  if (!AssignType_ || Index_ >= MAXVARS)
    return;

  if (Index_ < 0)
  {
    resultvalue = Value_;
    return;
  }

  if (AssignType_ == ASSIGN_ONLY ||
       (AltBase_ && AssignType_ == STRING_ASSIGN))
  {
    _vars[Index_] = Value_;

    if (Transfer_)
      LongValue_ = _TransferValue;
  }
  else if (AssignType_ == ASSIGN_PLUS)
    _vars[Index_] += Value_;
  else if (AssignType_ == ASSIGN_MINUS)
    _vars[Index_] -= Value_;
  else if (AssignType_ == ASSIGN_MULT)
    _vars[Index_] *= Value_;
  else if (AssignType_ == ASSIGN_DIV)
    _vars[Index_] /= Value_;
  else if (AssignType_ == ASSIGN_MOD)
    _vars[Index_] = ((int)_vars[Index_]) % ((int)Value_);
  else if (AssignType_ == ASSIGN_AND && AltBase_)
  {
    char* BaseStr_ = NULL;
    int AssignDone_ = FALSE;

    if (IsStringVar(Index_))
    {
      BaseStr_ = (char*)MemMatrix::Matrix().Allocate(GetStrVarLen(Index_) + 1);
      CopyStrVar(BaseStr_, Index_);

      if ((BaseStr_ && (IsIntHex(BaseStr_) || IsIntOct(BaseStr_))) &&
          (LongValue_.ConvertedFromBase() == 16 || LongValue_.ConvertedFromBase() == 8))
      {
        LongNumber SrceVal_ = LongDouble(_vars[Index_]);
        ChrString TempStr_ = BitwiseOps(SrceVal_, LongValue_, Mcalc_OperatorType::Logical_And);
        Ldouble ResultVal_ = strtol(TempStr_.c_str(), NULL, 0);
        _vars[Index_] = ResultVal_;
        AssignDone_ = TRUE;
      }

      MemMatrix::Matrix().Deallocate(BaseStr_);
    }

    if (!AssignDone_)
      _vars[Index_] = Integer(_vars[Index_] != Ldouble(0) && !LongValue_.AtZero());
  }
  else if (AssignType_ == ASSIGN_OR && AltBase_)
  {
    char* BaseStr_ = NULL;
    int AssignDone_ = FALSE;

    if (IsStringVar(Index_))
    {
      BaseStr_ = (char*)MemMatrix::Matrix().Allocate(GetStrVarLen(Index_) + 1);
      CopyStrVar(BaseStr_, Index_);

      if ((BaseStr_ && (IsIntHex(BaseStr_) || IsIntOct(BaseStr_))) &&
          (LongValue_.ConvertedFromBase() == 16 || LongValue_.ConvertedFromBase() == 8))
      {
        LongNumber SrceVal_ = LongDouble(_vars[Index_]);
        ChrString TempStr_ = BitwiseOps(SrceVal_, LongValue_, Mcalc_OperatorType::Logical_And);
        Ldouble ResultVal_ = strtol(TempStr_.c_str(), NULL, 0);
        _vars[Index_] = ResultVal_;
        AssignDone_ = TRUE;
      }

      MemMatrix::Matrix().Deallocate(BaseStr_);
    }

    if (!AssignDone_)
      _vars[Index_] = Integer(_vars[Index_] != Ldouble(0) && !LongValue_.AtZero());
  }

  if (Transfer_ || AltBase_)
    if (LongValue_.ConvertedFromBase() == 16)
    {
      LongValue_ = LongDouble(_vars[Index_]);
      SetStrVariable(Index_, LongValue_.ToString(16));
    }
    else if (LongValue_.ConvertedFromBase() == 8)
    {
      LongValue_ = LongDouble(_vars[Index_]);
      SetStrVariable(Index_, LongValue_.ToString(8));
    }

  resultvalue = _vars[Index_];
}

/*****************************************************************************/
int LongDoubleCalculator::GiveType() const
{
  return LONGDOUBLE;
}

/*****************************************************************************/
void LongDoubleCalculator::SubstToken(char* ArgStr_, int& StrIndex_, int VarIndex_)
{
  if (VarIndex_ < 0)
    return;

  if (IsStringVar(VarIndex_))
  {
    CopyStrVar(&ArgStr_[StrIndex_], VarIndex_);
    StrIndex_ += GetStrVarLen(VarIndex_);
  }
  else if (VarIndex_ < MAXVARS)
  {
    int len = strlen(M_El_STRING);
    int prec = len / 2;
    char* ValStr_ = (char*)MemMatrix::Matrix().Allocate(len >= 32 ? (len + 1):32);

    sprintf(ValStr_, "%-*.*Lg", len, prec, _vars[VarIndex_]);
    ValStr_ = RemovePadding(ValStr_, " \t\n");
    strcpy(&ArgStr_[StrIndex_], ValStr_);
    StrIndex_ += strlen(ValStr_);
    MemMatrix::Matrix().Deallocate(ValStr_);
    ValStr_ = NULL;
  }
}

/*****************************************************************************/
MEMORYOPS_DEFN(FloatCalculator)
MEMORYOPS_DEFN(DoubleCalculator)
MEMORYOPS_DEFN(LongDoubleCalculator)
MEMORYOPS_DEFN(LongNumCalculator)

/****************************************************************************/
/****************************************************************************/
LoopParams::LoopParams():
_Variable(NULL),
_Begin(0),
_End(0),
_Step(0),
_BeginStr(NULL),
_EndStr(NULL),
_StepStr(NULL)
{}

/****************************************************************************/
LoopParams::LoopParams(const char* Variable_, long Beg_, long End_, long Step_):
_Variable(::NewString(Variable_)),
_Begin(Beg_),
_End(End_),
_Step(Step_),
_BeginStr(NULL),
_EndStr(NULL),
_StepStr(NULL)
{}

/****************************************************************************/
LoopParams::~LoopParams()
{
  MemMatrix::Matrix().Deallocate(_Variable);
  MemMatrix::Matrix().Deallocate(_BeginStr);
  MemMatrix::Matrix().Deallocate(_EndStr);
  MemMatrix::Matrix().Deallocate(_StepStr);

  _BeginStr =
  _EndStr =
  _StepStr =
  _Variable = NULL;
}

/****************************************************************************/
void LoopParams::SetVariable(const char* Var_)
{
  MemMatrix::Matrix().Deallocate(_Variable);
  _Variable = ::NewString(Var_);
  _Variable = ::RemovePadding(_Variable, " \t\n");
}

/****************************************************************************/
void LoopParams::SetBegin(long Begin_)
{
  MemMatrix::Matrix().Deallocate(_BeginStr);
  _BeginStr = NULL;
  _Begin = Begin_;
}

/****************************************************************************/
void LoopParams::SetEnd(long End_)
{
  MemMatrix::Matrix().Deallocate(_EndStr);
  _EndStr = NULL;
  _End = End_;
}

/****************************************************************************/
void LoopParams::SetStep(long Step_)
{
  MemMatrix::Matrix().Deallocate(_StepStr);
  _StepStr = NULL;
  _Step = Step_;
}

/****************************************************************************/
void LoopParams::SetBegin(const char* Begin_)
{
  MemMatrix::Matrix().Deallocate(_BeginStr);
  _BeginStr = ::NewString(Begin_);
  _BeginStr = ::RemovePadding(_BeginStr, " \t\n");
}

/****************************************************************************/
void LoopParams::SetEnd(const char* End_)
{
  MemMatrix::Matrix().Deallocate(_EndStr);
  _EndStr = ::NewString(End_);
  _EndStr = ::RemovePadding(_EndStr, " \t\n");
}

/****************************************************************************/
void LoopParams::SetStep(const char* Step_)
{
  MemMatrix::Matrix().Deallocate(_StepStr);
  _StepStr = ::NewString(Step_);
  _StepStr = ::RemovePadding(_StepStr, " \t\n");
}

/****************************************************************************/
char* LoopParams::SetVarToBegin(char* Buffer_)
{
  if (_BeginStr)
    sprintf(Buffer_, "%s = %s", _Variable, _BeginStr);
  else
    sprintf(Buffer_, "%s = %ld", _Variable, _Begin);

  return Buffer_;
}

/****************************************************************************/
char* LoopParams::CheckVarEqualEnd(char* Buffer_)
{
  if (_EndStr)
    sprintf(Buffer_, "%s == %s", _Variable, _EndStr);
  else
    sprintf(Buffer_, "%s == %ld", _Variable, _End);

  return Buffer_;
}

/****************************************************************************/
char* LoopParams::IncrVarWithStep(char* Buffer_)
{
  if (_StepStr)
    sprintf(Buffer_, "%s += %s", _Variable, _StepStr);
  else
    sprintf(Buffer_, "%s += %ld", _Variable, _Step);

  return Buffer_;
}

/****************************************************************************/
char* LoopParams::CheckVarGreaterThanEnd(char* Buffer_)
{
  if (_EndStr)
    sprintf(Buffer_, "%s > %s", _Variable, _EndStr);
  else
    sprintf(Buffer_, "%s > %ld", _Variable, _End);

  return Buffer_;
}

/****************************************************************************/
char* LoopParams::CheckVarLessThanEnd(char* Buffer_)
{
  if (_EndStr)
    sprintf(Buffer_, "%s < %s", _Variable, _EndStr);
  else
    sprintf(Buffer_, "%s < %ld", _Variable, _End);

  return Buffer_;
}

/****************************************************************************/
/****************************************************************************/
MenuItem::MenuItem():
_Text(NULL),
_Label(NULL),
_Next(NULL)
{}

/****************************************************************************/
MenuItem::MenuItem(const char* Text_, const char* Label_):
_Text(::NewString(Text_)),
_Label(::NewString(Label_)),
_Next(NULL)
{}

/****************************************************************************/
MenuItem::~MenuItem()
{
  MemMatrix::Matrix().Deallocate(_Text);
  MemMatrix::Matrix().Deallocate(_Label);

  _Text = NULL;
  _Label = NULL;

  delete _Next;
  _Next = NULL;
}

/****************************************************************************/
void MenuItem::SetText(const char* Text_)
{
  MemMatrix::Matrix().Deallocate(_Text);
  _Text = ::NewString(Text_);
  _Text = ::RemovePadding(_Text, " \t\n");
}

/****************************************************************************/
void MenuItem::SetLabel(const char* Label_)
{
  MemMatrix::Matrix().Deallocate(_Label);
  _Label = ::NewString(Label_);
  _Label = ::RemovePadding(_Label, " \t\n");
}

/****************************************************************************/
void MenuItem::SetNext(MenuItem* Next_)
{
  _Next = Next_;
}

/****************************************************************************/
/****************************************************************************/
Menu::Menu():
_Title(NULL),
_Item(NULL)
{}

/****************************************************************************/
Menu::Menu(const char* Title_):
_Title(::NewString(Title_)),
_Item(NULL)
{}

/****************************************************************************/
Menu::~Menu()
{
  MemMatrix::Matrix().Deallocate(_Title);
  _Title = NULL;
  delete _Item;
  _Item = NULL;
}

/****************************************************************************/
void Menu::SetTitle(const char* Title_)
{
  MemMatrix::Matrix().Deallocate(_Title);
  _Title = ::NewString(Title_);
  _Title = ::RemovePadding(_Title, " \t\n");
}

/****************************************************************************/
void Menu::AddMenuItem(MenuItem* Item_)
{
  if (!_Item)
    _Item = Item_;
  else
  {
    MenuItem* Ptr_ = _Item;
    while (Ptr_->_Next)
      Ptr_ = Ptr_->_Next;

    Item_->SetNext(NULL);
    Ptr_->SetNext(Item_);
  }
}

/****************************************************************************/
/****************************************************************************/
IoArguments::IoArguments():
_MaxArgs(0),
_NumArgs(0),
_Arguments(NULL)
{}

/****************************************************************************/
IoArguments::~IoArguments()
{
  if (_MaxArgs)
  {
    size_t x;
    for (x = 0; x < _NumArgs; x++)
      MemMatrix::Matrix().Deallocate(_Arguments[x]);

    MemMatrix::Matrix().Deallocate(_Arguments);
    _MaxArgs = 0;
  }
}

/****************************************************************************/
void IoArguments::Resize()
{
  char** OldArgs_ = _Arguments;
  size_t OldSize_ = _MaxArgs;
  size_t x;

  _MaxArgs += ARRAYINCR;
  _Arguments = (char**)MemMatrix::Matrix().Allocate(_MaxArgs * sizeof(char*));

  for (x = 0; x < OldSize_; x++)
    _Arguments[x] = OldArgs_[x];

  for (;x < _MaxArgs; x++)
    _Arguments[x] = NULL;

  MemMatrix::Matrix().Deallocate(OldArgs_);
}

/****************************************************************************/
void IoArguments::AddArgument(const char* Arg_)
{
  if (_NumArgs >= _MaxArgs)
    Resize();

  _Arguments[_NumArgs] = ::NewString(Arg_);
  _Arguments[_NumArgs] = ::RemovePadding(_Arguments[_NumArgs], " \t\n");
  _NumArgs++;
}

/****************************************************************************/
/****************************************************************************/
EndToken::EndToken():
_Type(0),
_Startline(0),
_Endline(0),
_Depth(0)
{
}

/****************************************************************************/
EndToken::EndToken(EndToken* Token_):
_Type(Token_->_Type),
_Startline(Token_->_Startline),
_Endline(Token_->_Endline),
_Depth(Token_->_Depth)
{}

/****************************************************************************/
int EndToken::Encloses(EndToken* Token_)
{
  if (Token_)
    return Token_->IsEnclosedBy(this);

  return FALSE;
}

/****************************************************************************/
int EndToken::IsEnclosedBy(EndToken* Token_)
{
  if (Token_)
    return
    (
      _Depth > Token_->_Depth &&
      _Startline > Token_->_Startline &&
      _Endline < Token_->_Endline
    );

  return FALSE;
}

/****************************************************************************/
int EndToken::Matches(EndToken* Token_)
{
  if (Token_)
    return
    (
      _Type == Token_->_Type &&
      _Depth == Token_->_Depth &&
      _Startline == Token_->_Startline &&
      _Endline == Token_->_Endline
    );

  return FALSE;
}

/****************************************************************************/
void EndToken::SetData(EndToken* Token_)
{
  _Type = Token_->_Type;
  _Startline = Token_->_Startline;
  _Endline = Token_->_Endline;
  _Depth = Token_->_Depth;
}

/****************************************************************************/
/****************************************************************************/
ProgramLine::ProgramLine():
_Line(NULL),
_Type(0),
_Key(0),
_BlockSize(0),
_AltBlockSize(0),
_Fout(NULL),
_Parent(NULL)
{}

/****************************************************************************/
ProgramLine::ProgramLine(const char* Line_, CalcProgram* Ptr_):
_Line(::NewString(Line_)),
_Type(0),
_Key(0),
_BlockSize(0),
_AltBlockSize(0),
_Fout(NULL),
_Parent(Ptr_)
{}

/****************************************************************************/
ProgramLine::~ProgramLine()
{
  switch (_Key)
  {
    case CommandType::MENU:
      delete _Menu;
      _Menu = NULL;
      break;

    case CommandType::END:
      delete _End;
      _End = NULL;
      break;

    case CommandType::FOR:
      delete _Loop;
      _Loop = NULL;
      break;

    case CommandType::PROMPT:
      delete _IoFncArgs;
      _IoFncArgs = NULL;
      break;

    case CommandType::CONDITION:
      MemMatrix::Matrix().Deallocate(_Condition);
      _Condition = NULL;
      break;

    case CommandType::LABEL:
      MemMatrix::Matrix().Deallocate(_Label);
      _Label = NULL;
      break;

    case CommandType::PROGRAM:
      MemMatrix::Matrix().Deallocate(_Name);
      _Name = NULL;
      break;
  }

  MemMatrix::Matrix().Deallocate(_Line);
  _Line = NULL;
}

/****************************************************************************/
void ProgramLine::SetParent(CalcProgram* Ptr_)
{
  _Parent = Ptr_;
}

/****************************************************************************/
void ProgramLine::WriteCondition()
{
  if (_Key == CommandType::CONDITION)
    _Parent->EvalExpression(_Condition);
}

/****************************************************************************/
void ProgramLine::WriteCommand()
{
  _Parent->EvalExpression(_Line);
}

/****************************************************************************/
void ProgramLine::SetVarToBegin()
{
  char Buffer_[64];
  if (_Key == CommandType::FOR)
  {
    _Loop->SetVarToBegin(Buffer_);
    _Parent->EvalExpression(Buffer_);
  }
}

/****************************************************************************/
void ProgramLine::CheckVarEqualEnd()
{
  char Buffer_[64];
  if (_Key == CommandType::FOR)
  {
    _Loop->CheckVarEqualEnd(Buffer_);
    _Parent->EvalExpression(Buffer_);
  }
}

/****************************************************************************/
void ProgramLine::IncrVarWithStep()
{
  char Buffer_[64];
  if (_Key == CommandType::FOR)
  {
    _Loop->IncrVarWithStep(Buffer_);
    _Parent->EvalExpression(Buffer_);
  }
}

/****************************************************************************/
void ProgramLine::CheckVarGreaterThanEnd()
{
  char Buffer_[64];
  if (_Key == CommandType::FOR)
  {
    _Loop->CheckVarGreaterThanEnd(Buffer_);
    _Parent->EvalExpression(Buffer_);
  }
}

/****************************************************************************/
void ProgramLine::CheckVarLessThanEnd()
{
  char Buffer_[64];
  if (_Key == CommandType::FOR)
  {
    _Loop->CheckVarLessThanEnd(Buffer_);
    _Parent->EvalExpression(Buffer_);
  }
}

/****************************************************************************/
void ProgramLine::SetBlockSize(size_t Size_)
{
  _BlockSize = Size_;
}

/****************************************************************************/
void ProgramLine::SetAltBlockSize(size_t Size_)
{
  _AltBlockSize = Size_;
}

/****************************************************************************/
void ProgramLine::SetLine(const char* Line_)
{
  MemMatrix::Matrix().Deallocate(_Line);
  _Line = ::NewString(Line_);
  _Line = ::RemovePadding(_Line, " \t\n");
}

/****************************************************************************/
void ProgramLine::SetType(int Type_)
{
  _Type = Type_;
}

/****************************************************************************/
void ProgramLine::SetLoopParams(LoopParams* Loop_)
{
  if (_Key == CommandType::FOR)
    delete _Loop;

  _Loop = Loop_;
  _Key = CommandType::FOR;
}

/****************************************************************************/
void ProgramLine::SetMenu(Menu* Menu_)
{
  if (_Key == CommandType::MENU)
    delete _Menu;

  _Menu = Menu_;
  _Key = CommandType::MENU;
}

/****************************************************************************/
void ProgramLine::SetEndToken(EndToken* End_)
{
  if (_Key == CommandType::END)
    delete _End;

  _End = End_;
  _Key = CommandType::END;
}

/****************************************************************************/
void ProgramLine::SetIoArguments(IoArguments* Args_)
{
  if (_Key == CommandType::PROMPT)
    delete _IoFncArgs;

  _IoFncArgs = Args_;
  _Key = CommandType::PROMPT;
}

/****************************************************************************/
void ProgramLine::SetCondition(const char* Cond_)
{
  if (_Key == CommandType::CONDITION)
    MemMatrix::Matrix().Deallocate(_Condition);

  _Condition = ::NewString(Cond_);
  _Condition = ::RemovePadding(_Condition, " \t\n");
  _Key = CommandType::CONDITION;
}

/****************************************************************************/
void ProgramLine::SetLabel(const char* Label_)
{
  if (_Key == CommandType::LABEL)
    MemMatrix::Matrix().Deallocate(_Label);

  _Label = ::NewString(Label_);
  _Label = ::RemovePadding(_Label, " \t\n");
  _Key = CommandType::LABEL;
}

/****************************************************************************/
void ProgramLine::SetName(const char* Name_)
{
  if (_Key == CommandType::PROGRAM)
    MemMatrix::Matrix().Deallocate(_Name);

  _Name = ::NewString(Name_);
  _Name = ::RemovePadding(_Name, " \t\n");
  _Key = CommandType::PROGRAM;
}

/****************************************************************************/
/****************************************************************************/
CalcProgram::CalcProgram():
_Name(NULL),
_NumLines(0),
_NumSrcLines(0),
_MaxLines(0),
_Lines(NULL),
_SourceLines(NULL),
_OutputBufferSize(256),
_OutputBuffer((char*)MemMatrix::Matrix().Allocate(256 * sizeof(char))),
_DoQuit(0),
_QuitAll(0),
_NextProg(NULL),
_IostateFileReadError(0),
_ProgramLoaded(0),
_Stack(NULL),
_NumNodes(0),
_Executed(0),
_CalcPtr(NULL),
_QuitCalculator(FALSE),
_InputReceived(0),
LOG_FILE(NULL),
USER_INPUT_FILE(NULL),
USER_PROMPT_FILE(NULL),
PROG_OUTPUT_FILE(NULL),
PROG_IOSTATE_FILE(NULL),
CURRENTGRAPH_FILE(NULL),
SERVERALIVE_FILE(NULL),
CLIENTALIVE_FILE(NULL)
{
  _MaxLines += LINEINCR;
  _Lines = (ProgramLine**)MemMatrix::Matrix().Allocate(_MaxLines * sizeof(ProgramLine*));
  _SourceLines = (char**)MemMatrix::Matrix().Allocate(_MaxLines * sizeof(char*));

  size_t x;
  for (x = 0; x < _MaxLines; x++)
  {
    _Lines[x] = NULL;
    _SourceLines[x] = NULL;
  }
}

/****************************************************************************/
CalcProgram::CalcProgram(CalculatorBase* CalcPtr_, const char* Name_):
_Name(::NewString(Name_)),
_NumLines(0),
_NumSrcLines(0),
_MaxLines(0),
_Lines(NULL),
_SourceLines(NULL),
_OutputBufferSize(256),
_OutputBuffer((char*)MemMatrix::Matrix().Allocate(256 * sizeof(char))),
_DoQuit(0),
_QuitAll(0),
_NextProg(NULL),
_IostateFileReadError(0),
_ProgramLoaded(0),
_Stack(NULL),
_NumNodes(0),
_Executed(0),
_CalcPtr(CalcPtr_),
_QuitCalculator(FALSE),
_InputReceived(0),
LOG_FILE(NULL),
USER_INPUT_FILE(NULL),
USER_PROMPT_FILE(NULL),
PROG_OUTPUT_FILE(NULL),
PROG_IOSTATE_FILE(NULL),
CURRENTGRAPH_FILE(NULL),
SERVERALIVE_FILE(NULL),
CLIENTALIVE_FILE(NULL)
{
  _MaxLines += LINEINCR;
  _Lines = (ProgramLine**)MemMatrix::Matrix().Allocate(_MaxLines * sizeof(ProgramLine*));
  _SourceLines = (char**)MemMatrix::Matrix().Allocate(_MaxLines * sizeof(char*));
  SetFilenames(CalcPtr_);

  size_t x;
  for (x = 0; x < _MaxLines; x++)
  {
    _Lines[x] = NULL;
    _SourceLines[x] = NULL;
  }

  ChrString ProgName_ = Name_;
  ProgName_ += ".prg";
  LoadFile(ProgName_.c_str());
}

/****************************************************************************/
CalcProgram::~CalcProgram()
{
  size_t x;
  for (x = 0; x < _NumSrcLines; x++)
    MemMatrix::Matrix().Deallocate(_SourceLines[x]);

  for (x = 0; x < _NumLines; x++)
    MemMatrix::Matrix().Deallocate(_Lines[x]);

  MemMatrix::Matrix().Deallocate(_SourceLines);
  MemMatrix::Matrix().Deallocate(_Lines);
  MemMatrix::Matrix().Deallocate(_Name);
  MemMatrix::Matrix().Deallocate(_OutputBuffer);
  MemMatrix::Matrix().Deallocate(LOG_FILE);
  MemMatrix::Matrix().Deallocate(USER_INPUT_FILE);
  MemMatrix::Matrix().Deallocate(USER_PROMPT_FILE);
  MemMatrix::Matrix().Deallocate(PROG_OUTPUT_FILE);
  MemMatrix::Matrix().Deallocate(PROG_IOSTATE_FILE);
  MemMatrix::Matrix().Deallocate(CURRENTGRAPH_FILE);
  MemMatrix::Matrix().Deallocate(SERVERALIVE_FILE);
  MemMatrix::Matrix().Deallocate(CLIENTALIVE_FILE);

  EndToken* Token_;
  while (Token_ = Pop())
  {
    delete Token_;
    Token_ = NULL;
  }

  _CalcPtr->ClearOutputFile();
}

/****************************************************************************/
void CalcProgram::ResizeOutputBuffer()
{
  size_t x;
  size_t OldSize_ = _OutputBufferSize;
  char* OldBuffer_ = _OutputBuffer;

  _OutputBufferSize += 256;
  _OutputBuffer = (char*)MemMatrix::Matrix().Allocate(_OutputBufferSize * sizeof(char));

  for (x = 0; x < OldSize_; x++)
    _OutputBuffer[x] = OldBuffer_[x];

  for (;x < _OutputBufferSize; x++)
    _OutputBuffer[x] = 0;

  MemMatrix::Matrix().Deallocate(OldBuffer_);
  OldBuffer_ = NULL;
}

/****************************************************************************/
void CalcProgram::SetCalculator(CalculatorBase* CalcPtr_)
{
  _CalcPtr = CalcPtr_;
  SetFilenames(CalcPtr_);
}

/****************************************************************************/
CalculatorBase* CalcProgram::GetCalculator()
{
  return _CalcPtr;
}

/****************************************************************************/
int CalcProgram::EvalExpression(char* argstr)
{
  if (_CalcPtr)
    return _CalcPtr->EvalExpression(argstr, _CalcPtr);

  return 0;
}

/****************************************************************************/
void CalcProgram::Push(EndToken* End_)
{
  _NumNodes++;
  EndToken* Node_ = new EndToken;
  Node_->_Type = End_->_Type;
  Node_->_Startline = End_->_Startline;
  Node_->_Endline = End_->_Endline;
  Node_->_Depth = End_->_Depth;
  Node_->_Next = _Stack;
  _Stack = Node_;
}

/****************************************************************************/
void CalcProgram::Push(int Type_, size_t Start_, size_t End_)
{
  _NumNodes++;
  EndToken* Node_ = new EndToken;
  Node_->_Type = Type_;
  Node_->_Startline = Start_;
  Node_->_Endline = End_;
  Node_->_Depth = _NumNodes;
  Node_->_Next = _Stack;
  _Stack = Node_;
}

/****************************************************************************/
EndToken* CalcProgram::Pop()
{
  if (_Stack)
  {
    EndToken* Node_ = _Stack;
    _Stack = _Stack->_Next;
    Node_->_Next = NULL;
    _NumNodes--;
    return Node_;
  }

  return NULL;
}

/****************************************************************************/
long CalcProgram::FileSize(FILE* Fptr_)
{
  if (Fptr_)
  {
    fseek(Fptr_, 0, SEEK_END);
    long Pos_ = ftell(Fptr_);
    rewind(Fptr_);
    return Pos_;
  }

  return 0;
}

/****************************************************************************/
void CalcProgram::SetFilenames(const char* LogFile_, const char* InputFile_, const char* PromptFile_,
                               const char* ProgOutFile_, const char* IostateFile_, const char* CurrentGraphFile_,
                               const char* ServerAliveFile_, const char* ClientAliveFile_)
{
    LOG_FILE = ::NewString(LogFile_);
    USER_INPUT_FILE = ::NewString(InputFile_);
    USER_PROMPT_FILE = ::NewString(PromptFile_);
    PROG_OUTPUT_FILE = ::NewString(ProgOutFile_);
    PROG_IOSTATE_FILE = ::NewString(IostateFile_);
    CURRENTGRAPH_FILE = ::NewString(CurrentGraphFile_);
    SERVERALIVE_FILE = ::NewString(ServerAliveFile_);
    CLIENTALIVE_FILE = ::NewString(ClientAliveFile_);
}

/****************************************************************************/
void CalcProgram::SetFilenames(CalculatorBase* CalcPtr_)
{
    SetFilenames
    (
      CalcPtr_->GetLogFilename(),
      CalcPtr_->GetUserInputFilename(),
      CalcPtr_->GetUserPromptFilename(),
      CalcPtr_->GetProgramOutputFilename(),
      CalcPtr_->GetProgramIostateFilename(),
      CalcPtr_->GetServerAliveFilename(),
      CalcPtr_->GetClientAliveFilename(),
      CalcPtr_->GetCurrentGraphFilename()
    );
}

/****************************************************************************/
void CalcProgram::Resize()
{
  size_t OldSize_ = _MaxLines;
  ProgramLine** OldLines_ = _Lines;
  char** OldSrcLines_ = _SourceLines;

  _MaxLines += LINEINCR;
  _Lines = (ProgramLine**)MemMatrix::Matrix().Allocate(_MaxLines * sizeof(ProgramLine*));
  _SourceLines = (char**)MemMatrix::Matrix().Allocate(_MaxLines * sizeof(char*));

  size_t x;
  for (x = 0; x < OldSize_; x++)
  {
    _Lines[x] = OldLines_[x];
    _SourceLines[x] = OldSrcLines_[x];
  }

  for (;x < _MaxLines; x++)
  {
    _Lines[x] = NULL;
    _SourceLines[x] = NULL;
  }

  MemMatrix::Matrix().Deallocate(OldLines_);
  MemMatrix::Matrix().Deallocate(OldSrcLines_);
}

/****************************************************************************/
void CalcProgram::AddLine(ProgramLine* Line_)
{
  if (_NumLines >= _MaxLines)
    Resize();

  _Lines[_NumLines] = Line_;
  _NumLines++;
}

/****************************************************************************/
void CalcProgram::AddSourceLine(const char* Line_)
{
  if (_NumSrcLines >= _MaxLines)
    Resize();

  _SourceLines[_NumSrcLines] = ::NewString(Line_);
  _NumSrcLines++;
}
/****************************************************************************/
int CalcProgram::FreadString(char** buffer, size_t& MaxSize_, char terminator, FILE* Finput_)
{
  int eol,           /* end of line flag */
      eof;           /* end of file flag */
  int i;
  int length = MaxSize_; /* length of string buffer */
  char* OldBuf_;
  int InQuote_ = 0;

  /* copy characters from the file while not newline or EOF */
  /* increase buffer by increments of 1 line */
  for (i = eol = eof = 0; !eol; i++)
  {
    if (i==length)
    {
      length += MaxSize_;
      OldBuf_ = *buffer;
      *buffer = (char*)MemMatrix::Matrix().Reallocate(*buffer, (length + 1) * sizeof(char));
      MemMatrix::Matrix().Deallocate(OldBuf_);
    }

    (*buffer)[i] = fgetc(Finput_);
    if (isspace((*buffer)[i]))
      (*buffer)[i] = ' ';
    else
    {
      if ((*buffer)[i] == '\"')
        if (InQuote_)
          --InQuote_;
        else
          ++InQuote_;
    }

    eof = (*buffer)[i]==EOF;
    eol = ((*buffer)[i]==terminator && !InQuote_) || eof;
  }

  MaxSize_ = length;
  length = i;
  (*buffer)[length-1] = '\0';

  return eof;
}

/****************************************************************************/
void CalcProgram::UnloadProgram()
{
  size_t x;
  for (x = 0; x < _NumSrcLines; x++)
    MemMatrix::Matrix().Deallocate(_SourceLines[x]);

  for (x = 0; x < _NumLines; x++)
    MemMatrix::Matrix().Deallocate(_Lines[x]);

  _NumLines = _NumSrcLines = 0;

  for (x = 0; x < _MaxLines; x++)
  {
    _Lines[x] = NULL;
    _SourceLines[x] = NULL;
  }
}

/****************************************************************************/
int CalcProgram::LoadFile(const char* FileName_)
{
  FILE* Fin_ = NULL;
  char* buffer;
  size_t MaxSize_ = 256;
  ChrString TempStr_;

  buffer = (char*)MemMatrix::Matrix().Allocate(256 * sizeof(char));
  Fin_ = _CalcPtr->FileOpen(FileName_, "r");
  int LoadOk_ = Fin_ && FileSize(Fin_);

  if (LoadOk_)
  {
    LoadOk_ = FALSE;
    while (!FreadString(&buffer, MaxSize_, ':', Fin_))
    {
      if (strlen(buffer) && !_CalcPtr->IsThisChar(buffer, 0, '#', 1))
      {
        TempStr_ = buffer;
        TempStr_.RemovePadding(" \t\n");
        if (TempStr_.strlen())
        {
          LoadOk_ = TRUE;
          AddSourceLine(TempStr_.c_str());
        }
      }

      buffer[0] = 0;
    }

    if (strlen(buffer) && !_CalcPtr->IsThisChar(buffer, 0, '#', 1))
    {
      TempStr_ = buffer;
      TempStr_.RemovePadding(" \t\n");
      if (TempStr_.strlen())
      {
        LoadOk_ = TRUE;
        AddSourceLine(TempStr_.c_str());
      }
    }
  }

  MemMatrix::Matrix().Deallocate(buffer);
  _ProgramLoaded = LoadOk_;

  if (Fin_)
    fclose(Fin_);

  return LoadOk_;
}

/****************************************************************************/
int CalcProgram::LoadBuffer(char* ArrayBuf_)
{
  ChrString TempStr_;
  char* token_ = strtok(ArrayBuf_, ":");
  int LoadOk_ = FALSE;
  int x;

  while (token_)
  {
    for (x = strlen(token_); x && isspace(token_[x - 1]); x--);
    token_[x] = 0;
    for (x = 0; isspace(token_[x]); x++);
    if (x) memmove(token_, &token_[x], strlen(&token_[x]) + 1);

    if (strlen(token_) && !_CalcPtr->IsThisChar(token_, 0, '#', 1))
    {
      TempStr_ = token_;
      TempStr_.RemovePadding(" \t\n");
      if (TempStr_.strlen())
      {
        LoadOk_ = TRUE;
        AddSourceLine(TempStr_.c_str());
      }
    }

    token_ = strtok(NULL, ":");
  }

  MemMatrix::Matrix().Deallocate(_Name);
  _Name = ::NewString("From-Buffer");
  _ProgramLoaded = LoadOk_;

  return LoadOk_;
}

/****************************************************************************/
int CalcProgram::HasErrors() const
{
  return (_Stack && _NumNodes && (!_Executed));
}

/****************************************************************************/
void CalcProgram::SetName(const char* Name_)
{
  MemMatrix::Matrix().Deallocate(_Name);
  _Name = ::NewString(Name_);

  ChrString ProgName_ = Name_;
  ProgName_ += ".prg";
  LoadFile(ProgName_.c_str());
}

/****************************************************************************/
int CalcProgram::FindEnd(size_t Start_, size_t& LineNo_)
{
  size_t x;
  size_t Bal_ = 0;

  for (x = Start_; x < _MaxLines; x++)
  {
    if ((StrniComp(_SourceLines[x], "if", 2) == 0 && strlen(_SourceLines[x]) > 3)    ||
        (StrniComp(_SourceLines[x], "for", 3) == 0 && strlen(_SourceLines[x]) > 4)   ||
        (StrniComp(_SourceLines[x], "while", 5) == 0 && strlen(_SourceLines[x]) > 6) ||
        (StrniComp(_SourceLines[x], "repeat", 6) == 0 && strlen(_SourceLines[x]) > 7))
      Bal_++;
    else if (StrniComp(_SourceLines[x], "end", 3) == 0)
    {
      if (Bal_)
        Bal_--;
      else
      {
        LineNo_ = x;
        return 1;
      }
    }
  }

  return 0;
}

/****************************************************************************/
int CalcProgram::FindElse(size_t Start_, size_t& LineNo_)
{
  size_t x;
  size_t Bal_ = 0;

  for (x = Start_; x < _MaxLines; x++)
  {
    if ((StrniComp(_SourceLines[x], "if", 2) == 0 && strlen(_SourceLines[x]) > 3)    ||
        (StrniComp(_SourceLines[x], "for", 3) == 0 && strlen(_SourceLines[x]) > 4)   ||
        (StrniComp(_SourceLines[x], "while", 5) == 0 && strlen(_SourceLines[x]) > 6) ||
        (StrniComp(_SourceLines[x], "repeat", 6) == 0 && strlen(_SourceLines[x]) > 7))
      Bal_++;
    else if (StrniComp(_SourceLines[x], "end", 3) == 0)
    {
      if (Bal_)
        Bal_--;
      else
        return 0;
    }
    else if (StrniComp(_SourceLines[x], "else", 4) == 0)
    {
      if (Bal_ == 0)
      {
        LineNo_ = x;
        return 1;
      }
    }
  }

  return 0;
}

/****************************************************************************/
char* CalcProgram::FindArguments(char* Source_, char* Buffer_)
{
  if (!Source_)
  {
    Buffer_[0] = 0;
    return Source_;
  }

  char* token_ = strchr(Source_, ',');

  if (!token_)
  {
    if (Source_)
      strcpy(Buffer_, Source_);
    else
      Buffer_[0] = 0;

    return token_;
  }

  size_t x;
  size_t InQuote_ = 0;
  size_t len = strlen(Source_);
  char* next_;

  for (x = 0; x < len;)
  {
    for (;x < len && (&Source_[x] != token_); x++)
      if (Source_[x] == '\"')
      {
        if (InQuote_)
          InQuote_--;
        else
          InQuote_++;
      }

    if (InQuote_)
    {
      token_ = strchr(&token_[1], ',');
      if (!token_)
      {
        if (Source_ && InQuote_)
          strcpy(Buffer_, Source_);
        else
          Buffer_[0] = 0;

        next_ = NULL;
        break;
      }
    }
    else
    {
      len = token_ - Source_;

      if (len)
      {
        strncpy(Buffer_, Source_, len);
        Buffer_[len] = 0;

        if (Source_[len])
        {
          next_ = &Source_[len + 1];
          if (!next_[0])
            next_ = NULL;
        }
        else
          next_ = NULL;

        break;
      }
      else
      {
        token_ = strchr(&token_[1], ',');
        if (!token_)
        {
          if (Source_ && InQuote_)
            strcpy(Buffer_, Source_);
          else
            Buffer_[0] = 0;

          next_ = NULL;
          break;
        }
      }
    }
  }

  return next_;
}

/****************************************************************************/
void CalcProgram::Compile()
{
  size_t x, y, lineno;
  Menu* Menu_;
  MenuItem* Item_;
  LoopParams* Loop_;
  IoArguments* IoArgs_;
  char* FncArgs_;
  char* Label_;
  int Found_;

  int* DelimArray_ = (int*)MemMatrix::Matrix().Callocate(20 * sizeof(int));
  ProgramLine* PrgLine_;
  ChrString TempStr_;
  ChrString Begin_;
  ChrString End_;
  ChrString Step_;

  for (x = 0; x < _NumSrcLines; x++)
  {
    if (StrniComp(_SourceLines[x], "prgm", 4) == 0 && strlen(_SourceLines[x]) > 5)
    {
      TempStr_ = &_SourceLines[x][4];
      TempStr_.RemovePadding(" \t\n");

      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetName(TempStr_.c_str());
      PrgLine_->SetType(CommandType::PROGRAM);

      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "if", 2) == 0 && strlen(_SourceLines[x]) > 3)
    {
      TempStr_ = &_SourceLines[x][2];
      TempStr_.RemovePadding(" \t\n");
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetCondition(TempStr_.c_str());

      if ((x < _NumSrcLines - 1) && StrniComp(_SourceLines[x+1], "then", 4) == 0)
      {
        if (FindElse(x + 1, lineno))
        {
          PrgLine_->SetBlockSize(lineno - x);
          y = lineno;

          if (FindEnd(y + 1, lineno))
          {
            PrgLine_->SetAltBlockSize(lineno - y);
            PrgLine_->SetType(CommandType::IF_THEN_ELSE);
            AddLine(PrgLine_);
            Push(CommandType::IF_THEN_ELSE, x, x + PrgLine_->GetBlockSize() + PrgLine_->GetAltBlockSize());
          }
        }
        else
        {
          if (FindEnd(x + 1, lineno))
          {
            PrgLine_->SetBlockSize(lineno - x);
            PrgLine_->SetType(CommandType::IF_THEN);
            AddLine(PrgLine_);
            Push(CommandType::IF_THEN, x, x + PrgLine_->GetBlockSize());
          }
        }
      }
      else
      {
        PrgLine_->SetType(CommandType::IF);
        AddLine(PrgLine_);
      }
    }
    else if (StrniComp(_SourceLines[x], "for", 3) == 0 && strlen(_SourceLines[x]) > 4)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);

      TempStr_ = &_SourceLines[x][3];
      TempStr_.RemovePadding(" \t\n");
      Found_ = FindFunctionDelimiters(TempStr_.c_str(), TempStr_.strlen(), &DelimArray_, 20, "(", ")");

      if (Found_ && FindEnd(x + 1, lineno))
      {
        FncArgs_ = RetrieveFunctionArguments(TempStr_.c_str(), DelimArray_);
        Begin_ = &FncArgs_[DelimArray_[2]+1];
        End_ = &FncArgs_[DelimArray_[3]+1];
        Step_ = ((DelimArray_[0] - 1) == 4) ? &FncArgs_[DelimArray_[4]+1]:"1";
        Begin_.RemovePadding(" \t\n");
        End_.RemovePadding(" \t\n");
        Step_.RemovePadding(" \t\n");

        int Arg1Num_ = ::IsUlong(Begin_.c_str(), 10);
        int Arg2Num_ = ::IsUlong(End_.c_str(), 10);
        int Arg3Num_ = ::IsUlong(Step_.c_str(), 10);

        if (Arg1Num_ && Arg2Num_ && Arg3Num_)
          Loop_ = new LoopParams(&FncArgs_[DelimArray_[1]+1],
                                 atol(Begin_.c_str()),
                                 atol(End_.c_str()),
                                 atol(Step_.c_str()));
        else
        {
          Loop_ = new LoopParams();
          Loop_->SetVariable(&FncArgs_[DelimArray_[1]+1]);

          if (Arg1Num_)
            Loop_->SetBegin(atol(Begin_.c_str()));
          else
            Loop_->SetBegin(Begin_.c_str());

          if (Arg2Num_)
            Loop_->SetEnd(atol(End_.c_str()));
          else
            Loop_->SetEnd(End_.c_str());

          if (Arg3Num_)
            Loop_->SetStep(atol(Step_.c_str()));
          else
            Loop_->SetStep(Step_.c_str());
        }

        PrgLine_->SetBlockSize(lineno - x);
        PrgLine_->SetLoopParams(Loop_);
        PrgLine_->SetType(CommandType::FOR);
        AddLine(PrgLine_);
        Push(CommandType::FOR, x, x + PrgLine_->GetBlockSize());
      }
    }
    else if (StrniComp(_SourceLines[x], "while", 5) == 0 && strlen(_SourceLines[x]) > 6)
    {
      TempStr_ = &_SourceLines[x][5];
      TempStr_.RemovePadding(" \t\n");

      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetCondition(TempStr_.c_str());

      if (FindEnd(x + 1, lineno))
      {
        PrgLine_->SetBlockSize(lineno - x);
        PrgLine_->SetType(CommandType::WHILE);
        AddLine(PrgLine_);
        Push(CommandType::WHILE, x, x + PrgLine_->GetBlockSize());
      }
    }
    else if (StrniComp(_SourceLines[x], "repeat", 6) == 0 && strlen(_SourceLines[x]) > 7)
    {
      TempStr_ = &_SourceLines[x][6];
      TempStr_.RemovePadding(" \t\n");

      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetCondition(TempStr_.c_str());

      if (FindEnd(x + 1, lineno))
      {
        PrgLine_->SetBlockSize(lineno - x);
        PrgLine_->SetType(CommandType::REPEAT);
        AddLine(PrgLine_);
        Push(CommandType::REPEAT, x, x + PrgLine_->GetBlockSize());
      }
    }
    else if (StrniComp(_SourceLines[x], "lbl", 3) == 0 && strlen(_SourceLines[x]) > 4)
    {
      TempStr_ = &_SourceLines[x][3];
      TempStr_.RemovePadding(" \t\n");

      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetLabel(TempStr_.c_str());
      PrgLine_->SetType(CommandType::LABEL);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "goto", 4) == 0 && strlen(_SourceLines[x]) > 5)
    {
      TempStr_ = &_SourceLines[x][4];
      TempStr_.RemovePadding(" \t\n");

      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetLabel(TempStr_.c_str());
      PrgLine_->SetType(CommandType::GOTO);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "is>", 3) == 0 && strlen(_SourceLines[x]) > 4)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      TempStr_ = &_SourceLines[x][3];
      TempStr_.RemovePadding(" \t\n");
      Found_ = FindFunctionDelimiters(TempStr_.c_str(), TempStr_.strlen(), &DelimArray_, 20, "(", ")");

      if (Found_)
      {
        FncArgs_ = RetrieveFunctionArguments(TempStr_.c_str(), DelimArray_);
        End_ = &FncArgs_[DelimArray_[2]+1];
        End_.RemovePadding(" \t\n");
        int Arg2Num_ = ::IsUlong(End_.c_str(), 10);

        if (Arg2Num_)
          Loop_ = new LoopParams(&FncArgs_[DelimArray_[1]+1], 0,
                                 atol(&FncArgs_[DelimArray_[2]+1]), 1);
        else
        {
          Loop_ = new LoopParams();
          Loop_->SetVariable(&FncArgs_[DelimArray_[1]+1]);
          Loop_->SetBegin(0L);
          Loop_->SetEnd(&FncArgs_[DelimArray_[2]+1]);
          Loop_->SetStep(1L);
        }

        PrgLine_->SetLoopParams(Loop_);
        PrgLine_->SetType(CommandType::INCSKIP);
        AddLine(PrgLine_);
      }
    }
    else if (StrniComp(_SourceLines[x], "ds<", 3) == 0 && strlen(_SourceLines[x]) > 4)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      TempStr_ = &_SourceLines[x][3];
      TempStr_.RemovePadding(" \t\n");
      Found_ = FindFunctionDelimiters(TempStr_.c_str(), TempStr_.strlen(), &DelimArray_, 20, "(", ")");

      if (Found_)
      {
        FncArgs_ = RetrieveFunctionArguments(TempStr_.c_str(), DelimArray_);
        End_ = &FncArgs_[DelimArray_[2]+1];
        End_.RemovePadding(" \t\n");
        int Arg2Num_ = ::IsUlong(End_.c_str(), 10);

        if (Arg2Num_)
          Loop_ = new LoopParams(&FncArgs_[DelimArray_[1]+1], 0,
                                 atol(&FncArgs_[DelimArray_[2]+1]), -1);
        else
        {
          Loop_ = new LoopParams();
          Loop_->SetVariable(&FncArgs_[DelimArray_[1]+1]);
          Loop_->SetBegin(0L);
          Loop_->SetEnd(&FncArgs_[DelimArray_[2]+1]);
          Loop_->SetStep(-1L);
        }

        PrgLine_->SetLoopParams(Loop_);
        PrgLine_->SetType(CommandType::DECSKIP);
        AddLine(PrgLine_);
      }
    }
    else if (StrniComp(_SourceLines[x], "return", 6) == 0)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetType(CommandType::RETURN);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "stop", 4) == 0)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetType(CommandType::STOP);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "disp", 4) == 0 && strlen(_SourceLines[x]) > 5)
    {
      IoArgs_ = new IoArguments();
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      TempStr_ = &_SourceLines[x][4];
      TempStr_.RemovePadding(" \t\n");
      char* Source_ = ::NewString(TempStr_.c_str());
      char* token_ = ::NewString(TempStr_.c_str());
      char* next_ = Source_;
      size_t i;

      next_ = FindArguments(next_, token_);
      while (token_[0])
      {
        for (i = strlen(token_); i && isspace(token_[i - 1]); i--);
        token_[i] = 0;
        for (i = 0; isspace(token_[i]); i++);
        if (i) memmove(token_, &token_[i], strlen(&token_[i]) + 1);
        IoArgs_->AddArgument(token_);

        if (next_)
          next_ = FindArguments(next_, token_);
        else
          break;
      }

      MemMatrix::Matrix().Deallocate(Source_);
      MemMatrix::Matrix().Deallocate(token_);
      PrgLine_->SetIoArguments(IoArgs_);
      PrgLine_->SetType(CommandType::DISPLAY);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "prompt", 6) == 0 && strlen(_SourceLines[x]) > 7)
    {
      IoArgs_ = new IoArguments();
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      TempStr_ = &_SourceLines[x][6];
      TempStr_.RemovePadding(" \t\n");
      char* Source_ = ::NewString(TempStr_.c_str());
      char* token_ = ::NewString(TempStr_.c_str());
      char* next_ = Source_;
      size_t i;

      next_ = FindArguments(next_, token_);
      while (token_[0])
      {
        for (i = strlen(token_); i && isspace(token_[i - 1]); i--);
        token_[i] = 0;
        for (i = 0; isspace(token_[i]); i++);
        if (i) memmove(token_, &token_[i], strlen(&token_[i]) + 1);
        IoArgs_->AddArgument(token_);

        if (next_)
          next_ = FindArguments(next_, token_);
        else
          break;
      }

      MemMatrix::Matrix().Deallocate(Source_);
      MemMatrix::Matrix().Deallocate(token_);
      PrgLine_->SetIoArguments(IoArgs_);
      PrgLine_->SetType(CommandType::PROMPT);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "input", 5) == 0 && strlen(_SourceLines[x]) > 6)
    {
      IoArgs_ = new IoArguments();
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      TempStr_ = &_SourceLines[x][5];
      TempStr_.RemovePadding(" \t\n");
      char* Source_ = ::NewString(TempStr_.c_str());
      char* token_ = ::NewString(TempStr_.c_str());
      char* next_ = Source_;
      size_t i;
      size_t args_;

      args_ = 0;
      next_ = FindArguments(next_, token_);

      while (token_[0])
      {
        ++args_;
        for (i = strlen(token_); i && isspace(token_[i - 1]); i--);
        token_[i] = 0;
        for (i = 0; isspace(token_[i]); i++);
        if (i) memmove(token_, &token_[i], strlen(&token_[i]) + 1);
        IoArgs_->AddArgument(token_);

        if (next_)
          next_ = FindArguments(next_, token_);
        else
          break;
      }

      MemMatrix::Matrix().Deallocate(Source_);
      MemMatrix::Matrix().Deallocate(token_);

      if (args_ > 1)
        PrgLine_->SetType(CommandType::INPUT_PROMPT);
      else
        PrgLine_->SetType(CommandType::INPUT);

      PrgLine_->SetIoArguments(IoArgs_);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "calc_input", 10) == 0 && strlen(_SourceLines[x]) > 11)
    {
      IoArgs_ = new IoArguments();
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      TempStr_ = &_SourceLines[x][10];
      TempStr_.RemovePadding(" \t\n");
      char* Source_ = ::NewString(TempStr_.c_str());
      char* token_ = ::NewString(TempStr_.c_str());
      char* next_ = Source_;
      size_t i;
      size_t args_;

      args_ = 0;
      next_ = FindArguments(next_, token_);

      while (token_[0])
      {
        ++args_;
        for (i = strlen(token_); i && isspace(token_[i - 1]); i--);
        token_[i] = 0;
        for (i = 0; isspace(token_[i]); i++);
        if (i) memmove(token_, &token_[i], strlen(&token_[i]) + 1);
        IoArgs_->AddArgument(token_);

        if (next_)
          next_ = FindArguments(next_, token_);
        else
          break;
      }

      MemMatrix::Matrix().Deallocate(Source_);
      MemMatrix::Matrix().Deallocate(token_);
      PrgLine_->SetType(CommandType::CALCINPUT);

      PrgLine_->SetIoArguments(IoArgs_);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "pause", 5) == 0)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);

      if (strlen(_SourceLines[x]) > 6)
      {
        TempStr_ = &_SourceLines[x][5];
        TempStr_.RemovePadding(" \t\n");

        if (TempStr_.strlen())
        {
          IoArgs_ = new IoArguments();
          IoArgs_->AddArgument(TempStr_.c_str());
          PrgLine_->SetIoArguments(IoArgs_);
          PrgLine_->SetType(CommandType::PAUSE_VALUE);
        }
        else
          PrgLine_->SetType(CommandType::PAUSE);
      }
      else
        PrgLine_->SetType(CommandType::PAUSE);

      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "menu", 4) == 0 && strlen(_SourceLines[x]) > 5)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      TempStr_ = &_SourceLines[x][4];
      TempStr_.RemovePadding(" \t\n");
      Found_ = FindFunctionDelimiters(TempStr_.c_str(), TempStr_.strlen(), &DelimArray_, 20, "(", ")");
      char* Prompt_;
      size_t i;

      if (Found_)
      {
        FncArgs_ = RetrieveFunctionArguments(TempStr_.c_str(), DelimArray_);
        Menu_ = new Menu(&FncArgs_[DelimArray_[1]+1]);

        for (i = 2; i < DelimArray_[0]; i += 2)
        {
          Prompt_ = &FncArgs_[DelimArray_[i]+1];
          y = i + 1;

          if (y < DelimArray_[0])
          {
            Label_ = &FncArgs_[DelimArray_[y]+1];
            Item_ = new MenuItem(Prompt_, Label_);
            Menu_->AddMenuItem(Item_);
          }
        }

        PrgLine_->SetMenu(Menu_);
        PrgLine_->SetType(CommandType::MENU);
        AddLine(PrgLine_);
      }
    }
    else if (StrniComp(_SourceLines[x], "else", 4) == 0)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);

      if (FindEnd(x + 1, lineno))
      {
        PrgLine_->SetBlockSize(lineno - x);
        PrgLine_->SetType(CommandType::ELSE);
        AddLine(PrgLine_);
      }
    }
    else if (StrniComp(_SourceLines[x], "end", 3) == 0)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetType(CommandType::END);
      EndToken* Node_ = Pop();
      PrgLine_->SetEndToken(Node_);
      AddLine(PrgLine_);
    }
    else if (StrniComp(_SourceLines[x], "then", 4) == 0)
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetType(CommandType::NOOP);
      AddLine(PrgLine_);
    }
    else if (_SourceLines[x] && strlen(_SourceLines[x]) &&
             _SourceLines[x][0] != '#')
    {
      PrgLine_ = new ProgramLine(_SourceLines[x], this);
      PrgLine_->SetType(CommandType::COMMAND);
      AddLine(PrgLine_);
    }
  }

  MemMatrix::Matrix().Deallocate(DelimArray_);
}

/****************************************************************************/
int CalcProgram::CheckCondition()
{
  ReadFromOutput();
  char* Ptr_ = _OutputBuffer;

  if (Ptr_)
  {
    for (;isspace(*Ptr_); Ptr_++);
    char* Denom_ = strchr(Ptr_, '/');

    if (Denom_)
    {
      ChrString TempStr1_(Ptr_, Denom_ - Ptr_);
      LongNumber NumerVal_ = TempStr1_;
      ++Denom_;
      ChrString TempStr2_(Denom_);
      LongNumber DenomVal_ = TempStr2_;
      NumerVal_ /= DenomVal_;
      return !NumerVal_.AtZero();
    }
    else
    {
      ChrString TempStr1_(Ptr_);
      LongNumber Value_ = TempStr1_;
      return !Value_.AtZero();
    }
  }

  return 0;
}

/****************************************************************************/
const char* CalcProgram::ReadFromOutput()
{
  const char* AnswerStr_ = _CalcPtr->AnswerStr();

  if (_CalcPtr->HasAnswer() && AnswerStr_ && strlen(AnswerStr_))
  {
    size_t len = strlen(AnswerStr_);
    while (len >= _OutputBufferSize)
      ResizeOutputBuffer();

    strcpy(_OutputBuffer, AnswerStr_);
    return _OutputBuffer;
  }

  return NULL;
}

/****************************************************************************/
const char* CalcProgram::ReadFromUserInputFile()
{
  FILE* Fptr_ = _CalcPtr->FileOpen(USER_INPUT_FILE, "r");
  long Size_ = (Fptr_ && !ferror(Fptr_)) ? FileSize(Fptr_):0;
  int eof = 1;

  if (Size_)
  {
    while (Size_ >= _OutputBufferSize)
      ResizeOutputBuffer();

    do
    {
      eof = _CalcPtr->freadstring(&_OutputBuffer, ';', Fptr_);
      _CalcPtr->clearleadingspaces(&_OutputBuffer);
    }
    while (!eof && _OutputBuffer && _CalcPtr->IsThisChar(_OutputBuffer, 0, '#', 1));

    fclose(Fptr_);
    return _OutputBuffer;
  }

  return NULL;
}

/****************************************************************************/
void CalcProgram::CheckDefaultInputIfQuit()
{
  int x;
  int inc;

  for (x = 0; _OutputBuffer[x]; x++)
    if (!isspace(_OutputBuffer[x]))
      break;

  for (inc = 0; _OutputBuffer[x+inc] && _OutputBuffer[x+inc] == '\"'; inc++);

  if (inc)
  {
    --inc;
    x += inc;
  }

  if (CalculatorBase::icasestrcompn(&_OutputBuffer[x], "break", 5))
  {
    _DoQuit = _QuitAll = TRUE;
    _CalcPtr->ClearAnswerStr();
  }
  else if (CalculatorBase::icasestrcompn(&_OutputBuffer[x], "quit", 4))
  {
    _DoQuit = _QuitAll = TRUE;
    _QuitCalculator = TRUE;
    _CalcPtr->ClearAnswerStr();
  }
  else
  {
    _InputReceived = TRUE;

    if (CalculatorBase::icasestrcompn(&_OutputBuffer[x], "\"break\"", 7))
    {
      memmove(&_OutputBuffer[x + 6], &_OutputBuffer[x + 7], strlen(&_OutputBuffer[x + 7]) + 1);
      memmove(&_OutputBuffer[x], &_OutputBuffer[x + 1], strlen(&_OutputBuffer[x + 1]) + 1);
    }
    else if (CalculatorBase::icasestrcompn(&_OutputBuffer[x], "\"quit\"", 6))
    {
      memmove(&_OutputBuffer[x + 5], &_OutputBuffer[x + 6], strlen(&_OutputBuffer[x + 6]) + 1);
      memmove(&_OutputBuffer[x], &_OutputBuffer[x + 1], strlen(&_OutputBuffer[x + 1]) + 1);
    }
  }
}

/****************************************************************************/
void CalcProgram::ReceiveResponse(int Response_)
{
  if (Response_ == Mcalc_IOState::CALC_HALT)
  {
    _DoQuit = _QuitAll = TRUE;
    _QuitCalculator = TRUE;
    _CalcPtr->ClearAnswerStr();
    _CalcPtr->SetIoState(Mcalc_IOState::CALC_HALT);
  }
  else if (Response_ == Mcalc_IOState::BREAK_PROGRAM)
  {
    _DoQuit = _QuitAll = TRUE;
    _CalcPtr->ClearAnswerStr();

    if (_CalcPtr->HasErrors())
      _CalcPtr->SetIoState(Mcalc_IOState::CALC_ERROR);
    else
      _CalcPtr->SetIoState(Mcalc_IOState::CALC_RESPONSE);
  }
  else if (Response_ == Mcalc_IOState::INPUT_RECEIVED)
    _InputReceived = (ReadFromUserInputFile() != NULL);
}

/****************************************************************************/
int CalcProgram::WaitForGuiResponse()
{
  _CalcPtr->ResetRetryRead();
  int IoState_ = _CalcPtr->GetIoState(0);
  while (IoState_ != Mcalc_IOState::PROCESS &&
         IoState_ != Mcalc_IOState::CALC_HALT &&
         IoState_ != Mcalc_IOState::OUTPUT_FETCHED &&
         IoState_ != Mcalc_IOState::INPUT_RECEIVED &&
         IoState_ != Mcalc_IOState::BREAK_PROGRAM)
    IoState_ = _CalcPtr->GetIoState(1);

  return IoState_;
}

/****************************************************************************/
void CalcProgram::Execute_Command(const char* Str_)
{
  _CalcPtr->SetWaitForResponse(0);
  ProgramLine Line_(Str_, this);

  Line_.SetType(CommandType::COMMAND);
  Line_.WriteCommand();

  if (_CalcPtr->HasErrors())
    _DoQuit = _QuitAll = TRUE;
}

/****************************************************************************/
void CalcProgram::Execute_Command(size_t x)
{
  _CalcPtr->SetWaitForResponse(0);
  _Lines[x]->WriteCommand();

  if (_CalcPtr->HasErrors())
    _DoQuit = _QuitAll = TRUE;
  else if (_CalcPtr->HasCommandOutput() && ReadFromOutput())
  {
    #if CALCLIB_DEBUG6
      fputs(_CalcPtr->AnswerStr(), _CalcPtr->GetOutputFile());
    #else
      _CalcPtr->DisplayAnswerString();

      if (_CalcPtr->InAsyncMode())
      {
        _CalcPtr->SetIoState(_CalcPtr->WaitForResponse());
        ReceiveResponse(WaitForGuiResponse());
        _CalcPtr->SetWaitForResponse(0);
      }
    #endif
  }
  else if (_CalcPtr->InAsyncMode())
  {
    _CalcPtr->SetIoState(Mcalc_IOState::OUTPUT_READY);
    ReceiveResponse(WaitForGuiResponse());
    _CalcPtr->SetWaitForResponse(0);
  }
}

/****************************************************************************/
size_t CalcProgram::FindNextBlock(size_t x)
{
  while (x < _NumLines)
  {
    if (_Lines[x])
    {
      int Type_ = _Lines[x]->GetType();

      switch (Type_)
      {
        case CommandType::IF:
        case CommandType::INCSKIP:
        case CommandType::DECSKIP:
          break;

        case CommandType::IF_THEN:
        case CommandType::FOR:
        case CommandType::WHILE:
        case CommandType::REPEAT:
          return (x + _Lines[x]->GetBlockSize());

        case CommandType::IF_THEN_ELSE:
          return (x + _Lines[x]->GetBlockSize() + _Lines[x]->GetAltBlockSize());

        default:
          return x;
      }
    }
    ++x;
  }

  return (_NumLines - 1);
}

/****************************************************************************/
size_t CalcProgram::Execute_If(size_t x)
{
  _Lines[x]->WriteCondition();

  if (CheckCondition())
    return x;

  return FindNextBlock(x + 1);
}

/****************************************************************************/
CtrlExecData CalcProgram::Execute_If_Then(size_t x)
{
  CtrlExecData ExecData_;
  _Lines[x]->WriteCondition();
  size_t BlockSize_ = _Lines[x]->GetBlockSize();
  size_t Endpoint_ = x + BlockSize_;
  EndToken* End_ = _Lines[Endpoint_]->GetEndToken();

  if (CheckCondition())
  {
    ExecData_._Executed = TRUE;
    Push(End_);
    x = Execute_Block(x + 1, Endpoint_);
  }
  else
  {
    x = Endpoint_;
    ExecData_._Executed = FALSE;
  }

  ExecData_._LineNo = x;
  return ExecData_;
}

/****************************************************************************/
CtrlExecData CalcProgram::Execute_If_Then_Else(size_t x)
{
  CtrlExecData ExecData_;
  _Lines[x]->WriteCondition();

  size_t AltBlockSize_ = _Lines[x]->GetAltBlockSize();
  size_t BlockSize_ = _Lines[x]->GetBlockSize();
  size_t Elsepoint_ = x + BlockSize_;
  size_t Endpoint_ = Elsepoint_ + AltBlockSize_;
  int ConditionValid_ = CheckCondition();
  EndToken* End_ = _Lines[Endpoint_]->GetEndToken();
  Push(End_);

  if (ConditionValid_)
    x = Execute_Block(x + 1, Elsepoint_);
  else
  {
    x += BlockSize_;
    x = Execute_Block(x + 1, Endpoint_);
  }

  ExecData_._Executed = TRUE;
  ExecData_._LineNo = x;
  return ExecData_;
}

/****************************************************************************/
size_t CalcProgram::Execute_Else(size_t x)
{
  return (x + _Lines[x]->GetBlockSize());
}

/****************************************************************************/
CtrlExecData CalcProgram::Execute_For(size_t x)
{
  size_t y;
  CtrlExecData ExecData_;
  size_t BlockSize_ = _Lines[x]->GetBlockSize();
  size_t Endpoint_ = x + BlockSize_;
  EndToken* End_ = _Lines[Endpoint_]->GetEndToken();
  int PushedEnd_ = FALSE;
  int PastEnd_ = 0;

  _Lines[x]->SetVarToBegin();

  for (;;)
  {
    _Lines[x]->CheckVarEqualEnd();

    if (!CheckCondition())
    {
      if (!PushedEnd_)
      {
        Push(End_);
        PushedEnd_ = TRUE;
      }

      y = Execute_Block(x + 1, Endpoint_);

      if (y == Endpoint_)
        _Lines[x]->IncrVarWithStep();
      else
      {
        PastEnd_ = 1;
        break;
      }
    }
    else
      break;
  }

  if (PastEnd_)
    x = y;
  else
    x = Endpoint_;

  ExecData_._Executed = PushedEnd_;
  ExecData_._LineNo = x;
  return ExecData_;
}

/****************************************************************************/
CtrlExecData CalcProgram::Execute_While(size_t x)
{
  size_t y;
  CtrlExecData ExecData_;
  size_t BlockSize_ = _Lines[x]->GetBlockSize();
  size_t Endpoint_ = x + BlockSize_;
  EndToken* End_ = _Lines[Endpoint_]->GetEndToken();
  int PushedEnd_ = FALSE;
  int PastEnd_ = 0;

  for (;;)
  {
    _Lines[x]->WriteCondition();

    if (CheckCondition())
    {
      if (!PushedEnd_)
      {
        Push(End_);
        PushedEnd_ = TRUE;
      }

      y = Execute_Block(x + 1, Endpoint_);

      if (y != Endpoint_)
      {
        PastEnd_ = 1;
        break;
      }
    }
    else
      break;
  }

  if (PastEnd_)
    x = y;
  else
    x = Endpoint_;

  ExecData_._Executed = PushedEnd_;
  ExecData_._LineNo = x;
  return ExecData_;
}

/****************************************************************************/
CtrlExecData CalcProgram::Execute_Repeat(size_t x)
{
  size_t y;
  CtrlExecData ExecData_;
  size_t BlockSize_ = _Lines[x]->GetBlockSize();
  size_t Endpoint_ = x + BlockSize_;
  EndToken* End_ = _Lines[Endpoint_]->GetEndToken();
  int PushedEnd_ = FALSE;
  int PastEnd_ = 0;

  for (;;)
  {
    if (!PushedEnd_)
    {
      Push(End_);
      PushedEnd_ = TRUE;
    }

    y = Execute_Block(x + 1, Endpoint_);

    if (y == Endpoint_)
    {
      _Lines[x]->WriteCondition();

      if (CheckCondition())
        break;
    }
    else
    {
      PastEnd_ = 1;
      break;
    }
  }

  if (PastEnd_)
    x = y;
  else
    x = Endpoint_;

  ExecData_._Executed = PushedEnd_;
  ExecData_._LineNo = x;
  return ExecData_;
}

/****************************************************************************/
void CalcProgram::Execute_Prompt(size_t x)
{
  IoArguments* Args_ = _Lines[x]->GetIoArguments();
  ChrString TempStr_;
  ChrString VarStr_;
  size_t Max_ = Args_->_NumArgs;
  size_t i;

  for (i = 0; i < Max_ && !_DoQuit && !_QuitAll; i++)
  {
    TempStr_ = Args_->_Arguments[i];
    TempStr_.RemovePadding(" \t\n");
    VarStr_ = TempStr_;

    if (VarStr_[0] != '$' ||
        !(isalpha(VarStr_[1]) && InRange('A', VarStr_[1], 'z')))
      break;

    TempStr_ += "=? ";

    #if CALCLIB_DEBUG6
      fputs(TempStr_.c_str(), _CalcPtr->GetOutputFile());
    #else
      if (_CalcPtr->InAsyncMode())
        _CalcPtr->DisplayToUserPrompt(TempStr_.c_str());
      else
        _CalcPtr->UseDefaultOutputFnc(TempStr_.c_str());
    #endif

    _InputReceived = FALSE;
    if (_CalcPtr->InAsyncMode())
    {
      _CalcPtr->SetIoState(Mcalc_IOState::INPUT_REQUIRED);
      ReceiveResponse(WaitForGuiResponse());
    }
    else
    {
      _CalcPtr->UseDefaultInputFnc(_OutputBuffer, 256);
      CheckDefaultInputIfQuit();
    }

    if (_InputReceived && !_CalcPtr->HasErrors())
    {
      VarStr_ += "=";
      VarStr_ += _OutputBuffer;
      Execute_Command(VarStr_.c_str());
    }

    if (_CalcPtr->HasErrors() || _DoQuit)
      break;
  }
}

/****************************************************************************/
void CalcProgram::Execute_Input(size_t x)
{
  IoArguments* Args_ = _Lines[x]->GetIoArguments();
  ChrString TempStr_;
  ChrString VarStr_;

  if (!Args_ || Args_->_NumArgs > 2 || Args_->_NumArgs < 1)
    return;

  if (Args_->_NumArgs == 2 && _Lines[x]->GetType() == CommandType::INPUT_PROMPT)
  {
    TempStr_ = Args_->_Arguments[0];
    TempStr_.RemovePadding(" \t\n");
    TempStr_.RemovePadding("\"");
    TempStr_ += "? ";
    VarStr_ = Args_->_Arguments[1];
    VarStr_.RemovePadding(" \t\n");
  }
  else
  {
    TempStr_ = "? ";
    VarStr_ = Args_->_Arguments[0];
    VarStr_.RemovePadding(" \t\n");
  }

  #if CALCLIB_DEBUG6
    fputs(TempStr_.c_str(), _CalcPtr->GetOutputFile());
  #else
    if (_CalcPtr->InAsyncMode())
      _CalcPtr->DisplayToUserPrompt(TempStr_.c_str());
    else
      _CalcPtr->UseDefaultOutputFnc(TempStr_.c_str());
  #endif

  _InputReceived = FALSE;
  if (_CalcPtr->InAsyncMode())
  {
    _CalcPtr->SetIoState(Mcalc_IOState::INPUT_REQUIRED);
    ReceiveResponse(WaitForGuiResponse());
  }
  else
  {
    _CalcPtr->UseDefaultInputFnc(_OutputBuffer, 256);
    CheckDefaultInputIfQuit();
  }

  if (_InputReceived && !_CalcPtr->HasErrors())
  {
    VarStr_ += "=";
    VarStr_ += _OutputBuffer;
    Execute_Command(VarStr_.c_str());
  }
}

/****************************************************************************/
void CalcProgram::Execute_CalcInput(size_t x)
{
  IoArguments* Args_ = _Lines[x]->GetIoArguments();
  ChrString TempStr_;

  if (!Args_ || Args_->_NumArgs > 1)
    return;

  if (Args_->_NumArgs == 1)
  {
    TempStr_ = Args_->_Arguments[0];
    TempStr_.RemovePadding(" \t\n");
    TempStr_.RemovePadding("\"");
  }

  #if CALCLIB_DEBUG6
    fputs(TempStr_.c_str(), _CalcPtr->GetOutputFile());
  #else
    if (_CalcPtr->InAsyncMode())
      _CalcPtr->DisplayToUserPrompt(TempStr_.c_str());
    else
      _CalcPtr->UseDefaultOutputFnc(TempStr_.c_str());
  #endif

  _InputReceived = FALSE;
  if (_CalcPtr->InAsyncMode())
  {
    _CalcPtr->SetIoState(Mcalc_IOState::INPUT_REQUIRED);
    ReceiveResponse(WaitForGuiResponse());
  }
  else
  {
    _CalcPtr->UseDefaultInputFnc(_OutputBuffer, 256);
    _InputReceived = TRUE;
  }

  if (_InputReceived && !_CalcPtr->HasErrors())
    _CalcPtr->SetProgramStr(_OutputBuffer);
}

/****************************************************************************/
void CalcProgram::Execute_Pause(size_t x)
{
  char Dummy_[256];
  IoArguments* Args_;
  ChrString TempStr_;
  ChrString VarStr_;
  int Valid_;

  if (_Lines[x]->GetType() == CommandType::PAUSE_VALUE)
  {
    Args_ = _Lines[x]->GetIoArguments();
    if (Args_ && Args_->_NumArgs != 1)
      return;

    TempStr_ = Args_->_Arguments[0];
    TempStr_.RemovePadding(" \t\n");
    Fallible<Subscript> Fall1_ = TempStr_.find('\"');

    if (!Fall1_.valid())
    {
      Execute_Command(TempStr_.c_str());

      if (_CalcPtr->HasErrors())
        return;

      if (ReadFromOutput())
      {
        TempStr_ = _OutputBuffer;
        Valid_ = TRUE;
      }
      else
        Valid_ = FALSE;
    }
    else
    {
      TempStr_.RemovePadding("\"");
      Valid_ = TRUE;
    }

    if (Valid_)
    {
      #if CALCLIB_DEBUG6
        fputs(TempStr_.c_str(), _CalcPtr->GetOutputFile());
      #else
        if (_CalcPtr->InAsyncMode())
          _CalcPtr->DisplayToUserPrompt(TempStr_.c_str());
        else
          _CalcPtr->UseDefaultOutputFnc(TempStr_.c_str());
      #endif
    }
  }
  else
  {
    #if CALCLIB_DEBUG6
      fputs("<paused>", _CalcPtr->GetOutputFile());
    #else
      if (_CalcPtr->InAsyncMode())
        _CalcPtr->DisplayToUserPrompt("<paused>");
      else
        _CalcPtr->UseDefaultOutputFnc("<paused>");
    #endif
  }

  if (_CalcPtr->InAsyncMode())
  {
    _CalcPtr->SetIoState(Mcalc_IOState::INPUT_REQUIRED);
    WaitForGuiResponse();
  }
  else
    _CalcPtr->UseDefaultInputFnc(Dummy_, 256);
}

/****************************************************************************/
size_t CalcProgram::Execute_Menu(size_t x)
{
  char Buffer_[32];
  Menu* Args_ = _Lines[x]->GetMenu();
  MenuItem* Item_;
  ChrString TempStr_;
  ChrString VarStr_;
  ChrString OutputStr_;
  long i;

  Item_ = Args_ ? Args_->_Item:NULL;

  if (Item_)
  {
    TempStr_ = Args_->_Title;
    TempStr_.RemovePadding(" \t\n");
    TempStr_.RemovePadding("\"");
    TempStr_ += "\n";

    #if CALCLIB_DEBUG6
      fputs(TempStr_.c_str(), _CalcPtr->GetOutputFile());
    #else
      if (_CalcPtr->InAsyncMode())
        OutputStr_ += TempStr_;
      else
        _CalcPtr->UseDefaultOutputFnc(TempStr_.c_str());
    #endif

    for (i = 1; Item_; i++)
    {
      VarStr_ = Item_->_Text;
      VarStr_.RemovePadding(" \t\n");
      VarStr_.RemovePadding("\"");
      sprintf(Buffer_, "%ld", i);
      TempStr_ = Buffer_;
      TempStr_ += ":";
      TempStr_ += VarStr_;
      TempStr_ += "\n";

      #if CALCLIB_DEBUG6
        fputs(TempStr_.c_str(), _CalcPtr->GetOutputFile());
      #else
        if (_CalcPtr->InAsyncMode())
          OutputStr_ += TempStr_;
        else
          _CalcPtr->UseDefaultOutputFnc(TempStr_.c_str());
      #endif

      Item_ = Item_->_Next;
    }

    #if CALCLIB_DEBUG6
      fputs("? ", _CalcPtr->GetOutputFile());
    #else
      if (_CalcPtr->InAsyncMode())
      {
        OutputStr_ += "? ";
        _CalcPtr->DisplayToUserPrompt(OutputStr_.c_str());
      }
      else
        _CalcPtr->UseDefaultOutputFnc("? ");
    #endif

    _InputReceived = FALSE;
    if (_CalcPtr->InAsyncMode())
    {
      _CalcPtr->SetIoState(Mcalc_IOState::INPUT_REQUIRED);
      ReceiveResponse(WaitForGuiResponse());
    }
    else
    {
      _CalcPtr->UseDefaultInputFnc(_OutputBuffer, 256);
      _InputReceived = TRUE;
    }

    TempStr_ = _OutputBuffer;
    TempStr_.RemovePadding(" \t\n");

    if (_InputReceived && !_CalcPtr->HasErrors() && ::IsUlong(TempStr_.c_str(), 10))
    {
      Item_ = Args_->_Item;
      long Choice_ = atol(TempStr_.c_str());

      for (i = 1; i < Choice_ && Item_; i++)
        Item_ = Item_->_Next;

      if (!Item_)
        return Execute_Menu(x);
      else
      {
        for (i = x + 1; i < _NumLines; i++)
          if (_Lines[i]->GetType() == CommandType::LABEL &&
              StriComp(_Lines[i]->GetLabel(), Item_->_Label) == 0)
            return i;

        for (i = 0; i < x; i++)
          if (_Lines[i]->GetType() == CommandType::LABEL &&
              StriComp(_Lines[i]->GetLabel(), Item_->_Label) == 0)
            return i;

        return x;
      }
    }
  }

  return x;
}

/****************************************************************************/
size_t CalcProgram::Execute_Goto(size_t x)
{
  int i;
  for (i = x + 1; i < _NumLines; i++)
    if (_Lines[i]->GetType() == CommandType::LABEL &&
        StriComp(_Lines[i]->GetLabel(), _Lines[x]->GetLabel()) == 0)
      return i;

  for (i = 0; i < x; i++)
    if (_Lines[i]->GetType() == CommandType::LABEL &&
        StriComp(_Lines[i]->GetLabel(), _Lines[x]->GetLabel()) == 0)
      return i;

  return x;
}

/****************************************************************************/
size_t CalcProgram::Execute_IncSkip(size_t x)
{
  _Lines[x]->IncrVarWithStep();
  _Lines[x]->CheckVarGreaterThanEnd();

  if (CheckCondition())
    return FindNextBlock(x + 1);

  return x;
}

/****************************************************************************/
size_t CalcProgram::Execute_DecSkip(size_t x)
{
  _Lines[x]->IncrVarWithStep();
  _Lines[x]->CheckVarLessThanEnd();

  if (CheckCondition())
    return FindNextBlock(x + 1);

  return x;
}

/****************************************************************************/
void CalcProgram::Execute_Prgm(size_t x)
{
  _NextProg = new CalcProgram(_CalcPtr, _Lines[x]->GetName());

  if (_NextProg->ProgramLoaded())
  {
    _NextProg->Compile();

    if (!_NextProg->HasErrors())
    {
      _NextProg->Execute();
      _QuitAll = _NextProg->QuitAll();
      _QuitCalculator = _NextProg->QuitCalc();

      if (_QuitAll)
        _DoQuit = TRUE;
    }

    delete _NextProg;
    _NextProg = NULL;
  }
}

/****************************************************************************/
void CalcProgram::Execute_Display(size_t x)
{
  IoArguments* Args_ = _Lines[x]->GetIoArguments();
  ChrString OutputStr_;
  ChrString TempStr_;
  size_t Max_ = Args_->_NumArgs;
  size_t i;
  int OutDone_ = 0;

  for (i = 0; i < Max_ && !_DoQuit && !_QuitAll; i++)
  {
    TempStr_ = Args_->_Arguments[i];
    TempStr_.RemovePadding(" \t\n");

    if (TempStr_[0] == '\"')
    {
      OutDone_ = TRUE;
      TempStr_.RemovePadding("\"");
      #if CALCLIB_DEBUG6
        fputs(TempStr_.c_str(), _CalcPtr->GetOutputFile());
      #else
        OutputStr_ += TempStr_;
      #endif
    }
    else
    {
      _CalcPtr->SetWaitForResponse(0);
      _CalcPtr->EvalExpression(TempStr_.c_str(), _CalcPtr);

      if (_CalcPtr->HasErrors())
      {
        _DoQuit = _QuitAll = TRUE;
        break;
      }
      else if (ReadFromOutput())
      {
        OutDone_ = TRUE;
        #if CALCLIB_DEBUG6
          fputs(_OutputBuffer, _CalcPtr->GetOutputFile());
        #else
          OutputStr_ += _OutputBuffer;
        #endif
      }
    }
  }

  if (!_CalcPtr->HasErrors())
  {
    #if CALCLIB_DEBUG6
      if (Max_ && OutDone_)
        fputs("\n", _CalcPtr->GetOutputFile());
    #else
      if (Max_ && OutDone_)
        if (OutputStr_.strlen())
        {
          if (!_CalcPtr->InAsyncMode())
            OutputStr_ += "\n";
          _CalcPtr->DisplayMessage(_CalcPtr->GetOutputFile(), OutputStr_.c_str());
          _CalcPtr->CloseOutputFile();
        }

      if (_CalcPtr->InAsyncMode())
      {
        _CalcPtr->SetIoState(Mcalc_IOState::OUTPUT_READY);
        ReceiveResponse(WaitForGuiResponse());
        _CalcPtr->SetWaitForResponse(0);
      }
    #endif
  }
}

/****************************************************************************/
int CalcProgram::CheckTerminate(size_t x)
{
  if (_Stack)
  {
    EndToken* End_ = _Lines[x]->GetEndToken();

    if (End_)
      return (End_->Encloses(_Stack) || End_->Matches(_Stack));
  }

  return FALSE;
}

/****************************************************************************/
size_t CalcProgram::CheckTerminateNested(size_t x, CtrlExecData ExecData_, int& DoBreak_)
{
  size_t newx_ = ExecData_._LineNo;
  int Execed_ = ExecData_._Executed;

  int Type_ = _Lines[x]->GetType();
  size_t Endpoint_ = x + _Lines[x]->GetBlockSize();

  if (Type_ == CommandType::IF_THEN_ELSE)
    Endpoint_ += _Lines[x]->GetAltBlockSize();

  EndToken* End_ = _Lines[Endpoint_]->GetEndToken();
  int Match_ = End_->Matches(_Stack);

  if (!Execed_ || !Match_ || (x <= newx_ && newx_ < Endpoint_))
    return newx_;

  if (newx_ < _NumLines)
  {
    Type_ = _Lines[newx_]->GetType();

    if (Type_ == CommandType::ELSE)
    {
      newx_ = Execute_Else(newx_);

      if (newx_ < _NumLines)
      {
        if (_Lines[newx_]->GetType() == CommandType::END)
        {
          End_ = _Lines[newx_]->GetEndToken();
          Match_ = (newx_ == Endpoint_) && End_->Matches(_Stack);
          DoBreak_ = End_->Encloses(_Stack);

          if (DoBreak_ || Match_)
            delete Pop();
        }
      }
      else
        DoBreak_ = TRUE;
    }
    else if (Type_ == CommandType::END)
    {
      End_ = _Lines[newx_]->GetEndToken();
      Match_ = (newx_ == Endpoint_) && End_->Matches(_Stack);
      DoBreak_ = End_->Encloses(_Stack);

      if (DoBreak_ || Match_)
        delete Pop();
    }
  }
  else
    DoBreak_ = TRUE;

  return newx_;
}

/****************************************************************************/
size_t CalcProgram::Execute_Block(size_t Start_, size_t End_)
{
  int Type_;
  int DoBreak_ = _DoQuit;
  size_t x = Start_;

  while (x <= End_ && !DoBreak_ && !_DoQuit)
  {
    if (_Lines[x])
    {
      Type_ = _Lines[x]->GetType();

      switch (Type_)
      {
        case CommandType::IF:
          x = Execute_If(x);
          break;

        case CommandType::IF_THEN:
          x = CheckTerminateNested(x, Execute_If_Then(x), DoBreak_);
          break;

        case CommandType::IF_THEN_ELSE:
          x = CheckTerminateNested(x, Execute_If_Then_Else(x), DoBreak_);
          break;

        case CommandType::FOR:
          x = CheckTerminateNested(x, Execute_For(x), DoBreak_);
          break;

        case CommandType::WHILE:
          x = CheckTerminateNested(x, Execute_While(x), DoBreak_);
          break;

        case CommandType::REPEAT:
          x = CheckTerminateNested(x, Execute_Repeat(x), DoBreak_);
          break;

        case CommandType::GOTO:
          x = Execute_Goto(x);
          if (x > End_)
            End_ = _NumLines - 1;
          break;

        case CommandType::INCSKIP:
          x = Execute_IncSkip(x);
          break;

        case CommandType::DECSKIP:
          x = Execute_DecSkip(x);
          break;

        case CommandType::PROGRAM:
          Execute_Prgm(x);
          DoBreak_ = _DoQuit;
          break;

        case CommandType::DISPLAY:
          Execute_Display(x);
          break;

        case CommandType::COMMAND:
          Execute_Command(x);
          break;

        case CommandType::INPUT:
        case CommandType::INPUT_PROMPT:
          Execute_Input(x);
          break;

        case CommandType::CALCINPUT:
          Execute_CalcInput(x);
          break;

        case CommandType::PAUSE:
        case CommandType::PAUSE_VALUE:
          Execute_Pause(x);
          break;

        case CommandType::PROMPT:
          Execute_Prompt(x);
          break;

        case CommandType::MENU:
          x = Execute_Menu(x);
          if (x > End_)
            End_ = _NumLines - 1;
          break;

        case CommandType::STOP:
          DoBreak_ = TRUE;
          _DoQuit = _QuitAll = TRUE;
          break;

        case CommandType::RETURN:
          DoBreak_ = TRUE;
          _DoQuit = TRUE;
          break;

        case CommandType::END:
          DoBreak_ = CheckTerminate(x);
          break;

        case CommandType::ELSE:
          x = Execute_Else(x);
          if (x < _NumLines)
          {
            if (_Lines[x]->GetType() == CommandType::END)
              DoBreak_ = TRUE;
          }
          else
            DoBreak_ = TRUE;
          break;

        default:
          break;
      }
    }

    if (DoBreak_)
      break;
    else
      x++;
  }

  return x;
}

/****************************************************************************/
void CalcProgram::Execute()
{
  #if CALCLIB_DEBUG5
    printf("calling mcalc with session # = %s\n", Num_.c_str());
  #endif

  if (!HasErrors())
  {
    _Executed = TRUE;
    Execute_Block(0, _NumLines);
  }
}

/****************************************************************************/
void CalcProgram::RunProgram()
{
  if (ProgramLoaded())
  {
    Compile();
    Execute();
  }
}

/****************************************************************************/
const char* CalcProgram::GetLogFilename() const
{
  return LOG_FILE;
}

/****************************************************************************/
const char* CalcProgram::GetUserInputFilename() const
{
  return USER_INPUT_FILE;
}

/****************************************************************************/
const char* CalcProgram::GetUserPromptFilename() const
{
  return USER_PROMPT_FILE;
}

/****************************************************************************/
const char* CalcProgram::GetProgramOutputFilename() const
{
  return PROG_OUTPUT_FILE;
}

/****************************************************************************/
const char* CalcProgram::GetProgramIostateFilename() const
{
  return PROG_IOSTATE_FILE;
}

/****************************************************************************/
const char* CalcProgram::GetCurrentGraphFilename() const
{
  return CURRENTGRAPH_FILE;
}

/****************************************************************************/
MEMORYOPS_DEFN(LoopParams)
MEMORYOPS_DEFN(MenuItem)
MEMORYOPS_DEFN(Menu)
MEMORYOPS_DEFN(IoArguments)
MEMORYOPS_DEFN(EndToken)
MEMORYOPS_DEFN(ProgramLine)
MEMORYOPS_DEFN(CalcProgram)
MEMORYOPS_TEMPLATE_DEFN(IntCoords)

/****************************************************************************/
#endif




