// Window line buffer ADT classes
// The window line buffer class define methods for maintaining a window line
// object as parts used in a window screen object in a text console. The
// window line buffer class also has methods for positioning procedures for
// setting the display position within the window screen, setting the default
// message and default input into the window line, showing, hiding, erasing
// and retrieval of the input data.
//
// The window line buffer class define methods for maintaining an array of
// window line buffers. Methods are defined to show, hide, erase and
// retrieve from the entire array of window line buffers assigned to the
// window screen object.
//
#ifndef WINLINE_CPP
#define WINLINE_CPP
#ifndef WINLINE_H
  #include "winline.h"
#endif

/****************************************************************************/
ChainPosInfo::ChainPosInfo():
_enterb(false),
_vbuf(false),
_inwbox(false),
_chainable(false),
_error(false),
_vmax(0),
_vendpt(0),

_WrapMax(0),
_WrapIndex(0),

_reallen(0),
_copylen(0),
_index(0),
_startpt(0),
_endpt(0),
_boxpt(0),

_ChainPos(-1),
_BufInfo(NULL),
_StartInfo(NULL),
_EndInfo(NULL)
{}

/****************************************************************************/
ChainPosInfo::~ChainPosInfo()
{
  delete _StartInfo;
  delete _EndInfo;
  
  _StartInfo =
  _EndInfo = NULL;
}

/****************************************************************************/
void ChainPosInfo::RestorePrevInfo()
{
  // note: _ChainPos in reverse order from ReturnToThis method
  ChainPosInfo* ptr_ = (_ChainPos == 0) ? _EndInfo:
                       (_ChainPos == 1) ? _StartInfo:
                       (_ChainPos == 2) ? this:NULL;

  if (_ChainPos == 0 || _ChainPos == 1)
  {
    _enterb = ptr_->_enterb;
    _inwbox = ptr_->_inwbox;
    _chainable = ptr_->_chainable;
    _error = ptr_->_error;
  
    _vbuf = ptr_->_vbuf;
    _vmax = ptr_->_vmax;
    _vendpt = ptr_->_vendpt;

    _WrapMax = ptr_->_WrapMax;
    _WrapIndex = ptr_->_WrapIndex;

    _reallen = ptr_->_reallen;
    _copylen = ptr_->_copylen;
    _index = ptr_->_index;
    _startpt = ptr_->_startpt;
    _endpt = ptr_->_endpt;
    _boxpt = ptr_->_boxpt;
    _ChainPos = ptr_->_ChainPos;
  }
}

/****************************************************************************/
void ChainPosInfo::ReturnToThis()
{
  ChainPosInfo* ptr_ = (_ChainPos == 0) ? _StartInfo:
                       (_ChainPos == 1) ? _EndInfo:
                       (_ChainPos == 2) ? this:NULL;

  if (_ChainPos == 0 || _ChainPos == 1)
  {
    _enterb = ptr_->_enterb;
    _inwbox = ptr_->_inwbox;
    _chainable = ptr_->_chainable;
    _error = ptr_->_error;
  
    _vbuf = ptr_->_vbuf;
    _vmax = ptr_->_vmax;
    _vendpt = ptr_->_vendpt;

    _WrapMax = ptr_->_WrapMax;
    _WrapIndex = ptr_->_WrapIndex;

    _reallen = ptr_->_reallen;
    _copylen = ptr_->_copylen;
    _index = ptr_->_index;
    _startpt = ptr_->_startpt;
    _endpt = ptr_->_endpt;
    _boxpt = ptr_->_boxpt;
    _ChainPos = ptr_->_ChainPos;
  }
}

/****************************************************************************/
void ChainPosInfo::CopyToPosInfo(WindowLineBuffer* Ptr_)
{
  ChainPosInfo* ptr_ = (_ChainPos == 0) ? _StartInfo:
                       (_ChainPos == 1) ? _EndInfo:
                       (_ChainPos == 2) ? this:NULL;

  if (_ChainPos == 0 || _ChainPos == 1)
  {
    ptr_->_enterb = Ptr_->ExtendedBoxMode(ptr_->_vbuf, ptr_->_inwbox,
                                          &ptr_->_vendpt, NULL,
                                          &ptr_->_WrapIndex, &ptr_->_WrapMax);                  
    ptr_->_vmax = ptr_->_enterb ? Ptr_->VirtualBufferMax():
                                  Ptr_->_maxbuf;

    ptr_->_reallen =
    ptr_->_copylen = ::SafeStrLen(Ptr_->RawStrBuffer(0));
    ptr_->_index = Ptr_->_index;
    ptr_->_startpt = 0;
    ptr_->_endpt = Ptr_->_len;
    ptr_->_boxpt = Ptr_->_len;

    if (ptr_->_copylen > ptr_->_boxpt)
      ptr_->_copylen = ptr_->_boxpt;
  }
}

/****************************************************************************/
void ChainPosInfo::CopyInfoToThis(WindowLineBuffer* Ptr_)
{
  _enterb = Ptr_->ExtendedBoxMode(_vbuf, _inwbox,
                                  &_vendpt, NULL,
                                  &_WrapIndex, &_WrapMax);
  _vmax = _enterb ? Ptr_->VirtualBufferMax():
                    Ptr_->_maxbuf;

  _reallen =
  _copylen = ::SafeStrLen(Ptr_->RawStrBuffer(0));
  _index = Ptr_->_index;
  _startpt = 0;
  _endpt = Ptr_->_len;
  _boxpt = Ptr_->_len;

  if (_copylen > _boxpt)
    _copylen = _boxpt;
}

/****************************************************************************/
void ChainPosInfo::SetupInfoPtrs(WindowLineBuffer* Ptr_, int SeqPos_)
{
  _BufInfo = Ptr_;

  if (!_StartInfo)
    _StartInfo = new ChainPosInfo;

  if (!_EndInfo)
    _EndInfo = new ChainPosInfo;
  
  if (_BufInfo)
  {
    _ChainPos = (SeqPos_ == 0 && _BufInfo->VirtualBufferOffset() &&
                 _BufInfo->_bufferoffset == 0) ? 0:
                (SeqPos_ == 1 && !_BufInfo->VirtualBufferOffset() &&
                 _BufInfo->_bufferoffset != 0) ? 1:
                (SeqPos_ == 2) ? 2:-1;

    if (_ChainPos == 2)
      CopyInfoToThis(_BufInfo);
    else
      CopyToPosInfo(_BufInfo);
  }
  else
    _ChainPos = -1;
}

/****************************************************************************/
void ChainPosInfo::SetPullInfo(int start)
{
  if (!_BufInfo)
    return;

  ChainPosInfo* ptr_ = (_ChainPos == 0) ? _StartInfo:
                       (_ChainPos == 1) ? _EndInfo:
                       (_ChainPos == 2) ? this:NULL;

  if (!ptr_ || _ChainPos < 0)
    return;

  int rm_, cm_;
  int offset_ = 0;
  ptr_->_error = false;
  ptr_->_ChainPos = _ChainPos;
  ptr_->_BufInfo = _BufInfo;
  ptr_->_chainable = _BufInfo->IsChainable(false);
  ptr_->_index = start + ptr_->_startpt;

  if (_ChainPos == 0 && ptr_->_chainable && ptr_->_vbuf)
  {
    offset_ = (_BufInfo->_inboxoffset < ptr_->_vmax) ?
                    _BufInfo->_inboxoffset:
                    ptr_->_vmax;

    ptr_->_startpt += offset_;
    ptr_->_index = start + ptr_->_startpt;
    ptr_->_endpt += offset_;
    ptr_->_boxpt = ptr_->_endpt - ptr_->_startpt;
  }
  else if (_BufInfo->_bufferoffset > 0 && !ptr_->_enterb)
  {
    ptr_->_startpt = 0;
    ptr_->_index = start;
    ptr_->_endpt = _BufInfo->_len;
    ptr_->_boxpt = _BufInfo->_len;
  }

  rm_ = cm_ = ::SafeStrLen(_BufInfo->RawStrBuffer(ptr_->_startpt));
  if (cm_ > ptr_->_endpt)
    cm_ = ptr_->_endpt;

  offset_ = (ptr_->_reallen - offset_) - ptr_->_copylen;
  ptr_->_copylen = (ptr_->_index < ptr_->_endpt) ?
                         ::SafeStrLen(_BufInfo->RawStrBuffer(ptr_->_index)):0;
  ptr_->_reallen = rm_;
  ptr_->_copylen -= (offset_ > 0) ? offset_:0;
  if (ptr_->_copylen < 0)
    ptr_->_copylen = 0;
  else if (ptr_->_copylen > ptr_->_boxpt)
    ptr_->_copylen = ptr_->_boxpt;

  if (!(ptr_->_startpt <= ptr_->_index && ptr_->_index < ptr_->_endpt) ||
      !((rm_ > cm_ && ptr_->_endpt < rm_) ||
        (rm_ == cm_ && ptr_->_endpt >= rm_)) ||
      !(ptr_->_copylen <= ptr_->_reallen && cm_ <= rm_))
    ptr_->_error = true;
}

/****************************************************************************/
void ChainPosInfo::SetPushInfo(int start)
{
  if (!_BufInfo)
    return;

  ChainPosInfo* ptr_ = (_ChainPos == 0) ? _StartInfo:
                       (_ChainPos == 1) ? _EndInfo:
                       (_ChainPos == 2) ? this:NULL;

  if (!ptr_ || _ChainPos < 0)
    return;

  int rm_, cm_;
  int offset_ = 0;
  ptr_->_error = false;
  ptr_->_ChainPos = _ChainPos;
  ptr_->_BufInfo = _BufInfo;
  ptr_->_chainable = _BufInfo->IsChainable(false);
  ptr_->_index = start + ptr_->_startpt;

  if (_ChainPos == 0 && ptr_->_chainable && ptr_->_vbuf)
  {
    offset_ = (_BufInfo->_inboxoffset < ptr_->_vmax) ?
                    _BufInfo->_inboxoffset:
                    ptr_->_vmax;

    ptr_->_startpt += offset_;
    ptr_->_index = start + ptr_->_startpt;
    ptr_->_endpt += offset_;
    ptr_->_boxpt = ptr_->_endpt - ptr_->_startpt;
  }
  else if (_BufInfo->_bufferoffset > 0 && !ptr_->_enterb)
  {
    ptr_->_startpt = 0;
    ptr_->_index = start;
    ptr_->_endpt = _BufInfo->_len;
    ptr_->_boxpt = _BufInfo->_len;
  }

  rm_ = cm_ = ::SafeStrLen(_BufInfo->RawStrBuffer(ptr_->_startpt));
  if (cm_ > ptr_->_endpt)
    cm_ = ptr_->_endpt;

  offset_ = (ptr_->_reallen - offset_) - ptr_->_copylen;
  ptr_->_copylen = (ptr_->_index < ptr_->_endpt) ?
                         ::SafeStrLen(_BufInfo->RawStrBuffer(ptr_->_index)):0;
  ptr_->_reallen = rm_;
  ptr_->_copylen -= (offset_ > 0) ? offset_:0;
  if (ptr_->_copylen < 0)
    ptr_->_copylen = 0;
  else if (ptr_->_copylen > ptr_->_boxpt)
    ptr_->_copylen = ptr_->_boxpt;

  if (!(ptr_->_startpt <= ptr_->_index && ptr_->_index < ptr_->_endpt) ||
      !((rm_ > cm_ && ptr_->_endpt < rm_) ||
        (rm_ == cm_ && ptr_->_endpt >= rm_)) ||
      !(ptr_->_copylen <= ptr_->_reallen && cm_ <= rm_))
    ptr_->_error = true;
}

/****************************************************************************/
int ChainPosInfo::GetRefLineShift(bool MatchEndPts_)
{
  bool ReEntryValid_ = true;
  _baseposbuf = _BufInfo->RawStrBuffer(0, false);

  int endpt_ = 0;
  int shift_ = 0;
  int diff_ = 0;
  int len_ = _BufInfo->RawStrBufLen(0);
  int BackShift_ = _BufInfo->CalcBackShift(_baseposbuf);
  int reflinelen_ = _BufInfo->_refline ? _BufInfo->_refline->ExtendedIndex():0;

  if (BackShift_ > 0)
  {
    if (BackShift_ > reflinelen_)
      reflinelen_ = 0;
    else
      reflinelen_ -= BackShift_;
  }
  else
    BackShift_ = 0;

  if (reflinelen_ > len_)
    reflinelen_ = len_;

  _refshbuf = _BufInfo->RawStrBuffer(reflinelen_, false, &shift_);
  _refshjbuf = _BufInfo->_joinbuf + shift_;
  
  _reflineshift = shift_;
  _backshift = BackShift_;
  diff_ = _refshbuf - _baseposbuf;
  endpt_ = _BufInfo->_len - 1;

  if (_reflineshift > diff_)
    _reflineshift = diff_;

  _boxendterm = len_;
  _refshendterm = ::SafeStrLen(_refshbuf);

  _curbufpos = _BufInfo->_index + _reflineshift;
  if (_curbufpos >= _BufInfo->_maxbuf)
    _curbufpos = endpt_;

  _endptdex = endpt_ + _reflineshift;
  if (_endptdex >= _BufInfo->_maxbuf)
    _endptdex = endpt_;

  // refshindex should have same value as parent index if not shifted
  // below zero value index [0<--....x]
  _refshindex = _BufInfo->_index - _reflineshift;
  if (_refshindex < 0)
    _refshindex = 0;
  else
    _refshindex = _BufInfo->_index;

  _refshendpt = endpt_ - _reflineshift;
  if (_refshendpt < 0)
    _refshendpt = 0;

  _endstrdex = _boxendterm - 1;
  if (_endstrdex < 0)
    _endstrdex = 0;

  if (MatchEndPts_ && _endstrdex < _endptdex)
    _endstrdex = _endptdex;

  _refshendstr = _refshendterm - 1;
  if (_refshendstr < 0)
    _refshendstr = 0;

  if (MatchEndPts_ && _refshendstr < _refshendpt)
    _refshendstr = _refshendpt;

  return _reflineshift;
}

/****************************************************************************/
char* ChainPosInfo::GetBasePosBuffer()
{
  _baseposbuf = _BufInfo->RawStrBuffer(0, false);
  return _baseposbuf;
}

/****************************************************************************/
char* ChainPosInfo::GetRefshBuffer()
{
  GetRefLineShift();
  return _refshbuf;
}

/****************************************************************************/
char* ChainPosInfo::GetRefshJoinBuf()
{
  GetRefLineShift();
  return _refshjbuf;
}

/****************************************************************************/
int ChainPosInfo::GetBufBaseIndexPos(bool ReCalc_)
{
  int curbufpos_ = _curbufpos;

  if (ReCalc_)
  {
    int endpt_ = _BufInfo->_len - 1;
    curbufpos_ = _BufInfo->_index + GetRefLineShift();
    
    if (curbufpos_ >= _BufInfo->_maxbuf)
      curbufpos_ = endpt_;

    _curbufpos = curbufpos_;
  }

  return curbufpos_;
}

/****************************************************************************/
int ChainPosInfo::GetBufBaseEndPt(bool ReCalc_)
{
  int endptdex_ = _endptdex;

  if (ReCalc_)
  {
    int endpt_ = _BufInfo->_len - 1;
    endptdex_ = endpt_ + GetRefLineShift();
    
    if (endptdex_ >= _BufInfo->_maxbuf)
      endptdex_ = endpt_;

    _endptdex = endptdex_;
  }

  return endptdex_;
}

/****************************************************************************/
int ChainPosInfo::GetRefshCursorPos(int Base_, int CurPos_, bool ReCalc_)
{
  if (ReCalc_)
    GetRefLineShift();

  int shift_ = _BufInfo->EndBrksActive() ? _BufInfo->EndBrksLength(true):0;
  int refshcpos_ = CurPos_ - _reflineshift;
    
  if (refshcpos_ < (Base_ + shift_))
    refshcpos_ = Base_ + shift_;
  else
    refshcpos_ = CurPos_;

  return refshcpos_;
}

/****************************************************************************/
int ChainPosInfo::GetRefshIndexPos(bool ReCalc_)
{
  int refshindex_ = _refshindex;

  if (ReCalc_)
  {
    // refshindex should have same value as parent index if not shifted
    // below zero value index [0<--....x]
    refshindex_ = _BufInfo->_index - GetRefLineShift();
    
    if (refshindex_ < 0)
      refshindex_ = 0;
    else
      refshindex_ = _BufInfo->_index;

    _refshindex = refshindex_;
  }

  return refshindex_;
}

/****************************************************************************/
int ChainPosInfo::GetRefshEndPt(bool ReCalc_)
{
  int refshendpt_ = _refshendpt;

  if (ReCalc_)
  {
    int endpt_ = _BufInfo->_len - 1;
    refshendpt_ = endpt_ - GetRefLineShift();

    if (refshendpt_ < 0)
      refshendpt_ = 0;

    _refshendpt = refshendpt_;
  }

  return refshendpt_;
}

/****************************************************************************/
int ChainPosInfo::GetBufBaseEndStr(bool ReCalc_, bool MatchEndPts_)
{
  int endstrdex_ = _endstrdex;
  int shift_ = _reflineshift;

  if (ReCalc_)
  {
    shift_ = GetRefLineShift();
    endstrdex_ = _boxendterm - 1;
    
    if (endstrdex_ < 0)
      endstrdex_ = 0;

    _endstrdex = endstrdex_;
  }

  if (MatchEndPts_ && _endstrdex < _endptdex)
    endstrdex_ = _endstrdex = _endptdex;

  return endstrdex_;
}

/****************************************************************************/
int ChainPosInfo::GetRefshEndStr(bool ReCalc_, bool MatchEndPts_)
{
  int refshendstr_ = _refshendstr;
  int shift_ = _reflineshift;

  if (ReCalc_)
  {
    shift_ = GetRefLineShift();
    refshendstr_ = _refshendterm - 1;
    
    if (refshendstr_ < 0)
      refshendstr_ = 0;

    _refshendstr = refshendstr_;
  }

  if (MatchEndPts_ && _refshendstr < _refshendpt)
    refshendstr_ = _refshendstr = _refshendpt;

  return refshendstr_;
}

/****************************************************************************/
// Monitoring functions
/****************************************************************************/
const int* InsertDetect(void* Owner_, WindowLineBuffer*, const int* Ptr_)
{
  if (Owner_)
    ((WindowLineBuffer*)Owner_)->NotifyInsertMode(Ptr_);

  return Ptr_;
}

/****************************************************************************/
// WindowLineBuffer::SharedChainData class definition
/****************************************************************************/
WindowLineBuffer::SharedChainData::SharedChainData(WindowLineBuffer* wlbptr):
_owner(NULL),
_parent(wlbptr),
_reentryptr(NULL),

_nullcharstr(new char[2]),
_padcnt(0),
_cpadpos(-1),
_cpadchain(-1),
_cpadset(false),

_InWinMode(false),
_PrevLineReady(false),
_NextLineReady(false),
_countjoinedch(false),
_movedone(false),
_BoxPosSaved(false),
_BoxPosTrans(false),

_SavedOffset(0),
_SavedIndex(0),
_SavedExtIndex(0),
_SavedBoxDiff(0),

_NextOffset(0),
_NextIndex(0),
_NextExtIndex(0),
_NextBoxDiff(0),

_numnextlines(UNINIT_NL),
_WinLineNum(0)
{
  _nullcharstr[0] = _nullcharstr[1] = 0;
}

/****************************************************************************/
WindowLineBuffer::SharedChainData::~SharedChainData()
{
  delete[] _nullcharstr;
  _nullcharstr = NULL;
}

/****************************************************************************/
void WindowLineBuffer::SharedChainData::SetOwner(WindowLine* wlptr)
{
  _owner = wlptr;

  if (_owner && _owner->SharedData())
  {
    _owner->AssignSharedData(_InWinMode, _NextLineReady, _PrevLineReady);
    _WinLineNum = _owner->WindowLineNum();
  }
}

/****************************************************************************/
void WindowLineBuffer::SharedChainData::SetParent(WindowLineBuffer* wlbptr)
{
  _parent = wlbptr;
}
      
/****************************************************************************/
// WindowLineBuffer class definition
/****************************************************************************/
// PURPOSE:
//   Prompt box default constructor
//
// PRE:
//   an uninitialize Prompt box
//
// POST:
//   a Prompt box is constructed and assigned the default values specified.
//   If the msg argument is not NULL then the Prompt message of the box
//   is assigned to the argument string.
//   The string buffer is Cleared to all 0.
//
WindowLineBuffer::WindowLineBuffer(char* msg, WindowLineBuffer* next_,
                                   WindowLine* owner_, int Size_, int Length_):
_Owner(owner_),
_nextline(next_),
_refline(NULL),
_ChainPosPtr(NULL),
_Enterable(TRUE),
_Selectable(FALSE),
_msg(msg ? strcpy(new char[::SafeStrLen(msg)+1], msg):NULL),

_buffer(owner_ ? (new char[owner_->BoxInputLength() + 1]):
        Size_ ? (new char[Size_ + 1]):(new char[MAXBUFFER + 1])),
_retbuf((Length_ && Length_ <= Size_) ? (new char[Length_ + 1]):
        owner_ ? (new char[owner_->BoxInputLength() + 1]):
        Size_ ? (new char[Size_ + 1]):(new char[21])),
_joinbuf(owner_ ? (new char[owner_->BoxInputLength() + 1]):
         Size_ ? (new char[Size_ + 1]):(new char[MAXBUFFER + 1])),
_tranbuf(NULL),

_bufferalloc(true),
_retbufalloc(true),
_joinbufalloc(true),
_tranbufalloc(false),
_bufferscleared(0),

_bufferoffset(0),
_inboxoffset(0),
_inwboxcnt(0),
_MovedToEnd(false),
_EndCharInserted(0),
_EndCharReverted(false),
_DelKeyDone(false),
_DelWrapOffset(0),
_linestatus(UNINIT_NL),
_maxbuf(owner_ ? owner_->BoxInputLength():
        Size_ ? Size_:MAXBUFFER),
_len((Length_ && Length_ <= Size_) ? Length_:
     owner_ ? owner_->BoxInputLength():
     Size_ ? Size_:1),   // _len should never be zero
_retbufsize((Length_ && Length_ <= Size_) ? Length_:
            owner_ ? owner_->BoxInputLength():
            Size_ ? Size_:20),

_SharedDataErased(false),
_SharedData(new WindowLineBuffer::SharedChainData),

_minlen(1),              // minimum length should be at least 1
_chainminlen(0),         // no chains at initialization so zero
_index(0),
_xpos(WHEREX()),
_ypos(WHEREY()),
_noempty(0),
_InsertMode(TRUE),
_Escaped(FALSE),
_Hidden(TRUE),
_TextWidgetActive(TRUE),
_ChainPos(0),
_InFocus(FALSE),

_SavedCursX(WHEREX()),
_SavedCursY(WHEREY()),

_BufChainNum(0),
_WrapChars(false),

_RestFullBox(true),
_EndBrksActive(false),
_TrimWs(true),

// Internal data monitors
_insert_monitor(NULL),
_buffer_monitor(NULL),
_key_monitor(NULL),
_ext_monitor(NULL),
_index_monitor(NULL),
_escaped_monitor(NULL),
_hidden_monitor(NULL),
_active_monitor(NULL),
_infocus_monitor(NULL),
_inboxoffset_monitor(NULL),
_winlinenum_monitor(NULL),

// Handles to external objects, used by data monitors
_insert_handle(NULL),
_buffer_handle(NULL),
_key_handle(NULL),
_ext_handle(NULL),
_index_handle(NULL),
_escaped_handle(NULL),
_hidden_handle(NULL),
_active_handle(NULL),
_infocus_handle(NULL),
_inboxoffset_handle(NULL),
_winlinenum_handle(NULL)
{
  if (_SharedData)
    _SharedData->SetParent(this);

  WindowLineBuffer::SharedChainData* sdp = SharedData();

  if (sdp && _linestatus == UNINIT_NL)
  {
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;
  
    if (::SafeStrLen(_buffer))
      _linestatus = FILLED_NL;
  }

  _bufferalloc = _bufferalloc && _buffer != NULL;
  _retbufalloc = _retbufalloc && _retbuf != NULL;
  _joinbufalloc = _joinbufalloc && _joinbuf != NULL;

  _ChainPosPtr = new ChainPosInfo;
  _brackets[0] = CONTMORE_LEFT;  // Continue more to the left (left triangle)
  _brackets[1] = CONTMORE_RIGHT; // Continue more to the right (right triangle)
  int index;

  if (_bufferalloc)
    for (index = 0; index <= _maxbuf; index++)
      _buffer[index] = '\0';

  if (_joinbufalloc)
    for (index = 0; index < _maxbuf; index++)
      _joinbuf[index] = NOJOIN_CHAR;

  if (_joinbufalloc && index == _maxbuf)
    _joinbuf[index] = '\0';

  SetInputLength(_len, false);
}

/****************************************************************************/
// PURPOSE:
//   Prompt box copy construcor
//
// PRE:
//   An uninitialized Prompt box.
//
// POST:
//   a Prompt box is constructed and assigned the corresponding values
//   in the Prompt box argument passed to the constructor.
//
WindowLineBuffer::WindowLineBuffer(WindowLineBuffer& pb, int Size_, int Length_):
_Owner(pb._Owner),
_nextline(NULL),
_refline(NULL),
_ChainPosPtr(NULL),
_Enterable(pb._Enterable),
_Selectable(pb._Selectable),
_msg(pb._msg ? strcpy(new char[::SafeStrLen(pb._msg)+1], pb._msg):NULL),

_buffer(pb._maxbuf ? (new char[pb._maxbuf + 1]):
        Size_ ? (new char[Size_ + 1]):
                (new char[MAXBUFFER + 1])),
_retbuf(pb._len ? (new char[pb._len+1]):
        (Length_ && Length_ <= Size_) ? (new char[Length_+1]):
        Size_ ? (new char[Size_+1]):(new char[21])),
_joinbuf(pb._maxbuf ? (new char[pb._maxbuf + 1]):
         Size_ ? (new char[Size_ + 1]):
                 (new char[MAXBUFFER + 1])),
_tranbuf(NULL),

_bufferalloc(true),
_retbufalloc(true),
_joinbufalloc(true),
_tranbufalloc(false),
_bufferscleared(pb._bufferscleared),

_bufferoffset(0),
_inboxoffset(0),
_inwboxcnt(0),
_MovedToEnd(false),
_EndCharInserted(0),
_EndCharReverted(false),
_DelKeyDone(false),
_DelWrapOffset(0),
_linestatus(UNINIT_NL),
_maxbuf(pb._maxbuf ? pb._maxbuf:
        Size_ ? Size_:MAXBUFFER),
_len(pb._len ? pb._len:
     (Length_ && Length_ <= Size_) ? Length_:
     Size_ ? Size_:1),
_retbufsize(pb._len ? pb._len:
            (Length_ && Length_ <= Size_) ? Length_:
            Size_ ? Size_:20),

_SharedDataErased(false),
_SharedData(pb._SharedData),
            
_minlen(pb._minlen ? pb._minlen:1),
_chainminlen(0),
_index(pb._index),
_xpos(pb._xpos),
_ypos(pb._ypos),
_noempty(pb._noempty),
_InsertMode(TRUE),
_Escaped(FALSE),
_Hidden(TRUE),
_TextWidgetActive(TRUE),
_ChainPos(0),
_InFocus(FALSE),

_SavedCursX(WHEREX()),
_SavedCursY(WHEREY()),

_BufChainNum(pb._BufChainNum),
_WrapChars(false),

_RestFullBox(pb._RestFullBox),
_EndBrksActive(pb._EndBrksActive),
_TrimWs(pb._TrimWs),

// Internal data monitors
_insert_monitor(NULL),
_buffer_monitor(NULL),
_key_monitor(NULL),
_ext_monitor(NULL),
_index_monitor(NULL),
_escaped_monitor(NULL),
_hidden_monitor(NULL),
_active_monitor(NULL),
_infocus_monitor(NULL),
_inboxoffset_monitor(NULL),
_winlinenum_monitor(NULL),

// Handles to external objects, used by data monitors
_insert_handle(NULL),
_buffer_handle(NULL),
_key_handle(NULL),
_ext_handle(NULL),
_index_handle(NULL),
_escaped_handle(NULL),
_hidden_handle(NULL),
_active_handle(NULL),
_infocus_handle(NULL),
_inboxoffset_handle(NULL),
_winlinenum_handle(NULL)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  if (sdp && _linestatus == UNINIT_NL)
  {
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;
  
    if (::SafeStrLen(pb._buffer))
      _linestatus = FILLED_NL;
  }

  _bufferalloc = _bufferalloc && _buffer != NULL;
  _retbufalloc = _retbufalloc && _retbuf != NULL;
  _joinbufalloc = _joinbufalloc && _joinbuf != NULL;

  _ChainPosPtr = new ChainPosInfo;
  _brackets[0] = CONTMORE_LEFT;    // Continue more to the left (left triangle)
  _brackets[1] = CONTMORE_RIGHT;   // Continue more to the right (right triangle)
  int index;

  if (_bufferalloc)
  {
    strcpy(_buffer, pb._buffer);
  
    for (index = ::SafeStrLen(_buffer); index <= _maxbuf; index++)
      _buffer[index] = '\0';
  }

  if (_joinbufalloc)
  {
    strcpy(_joinbuf, pb._joinbuf);  
    _joinbuf[_maxbuf] = '\0';
    
    for (index = ::SafeStrLen(_buffer); index < _maxbuf; index++)
      _joinbuf[index] = NOJOIN_CHAR;
  }

  if (pb._minlen)
    SetMinimumLength(pb._minlen);
  else
    SetMinimumLength(1);

  SetInputLength(_len, false);
}

/****************************************************************************/
// PURPOSE:
//   Prompt box copy construcor for offsetted virtual buffer
//
// PRE:
//   An uninitialized Prompt box.
//
// POST:
//   a Prompt box is constructed and assigned the corresponding values
//   in the Prompt box argument passed to the constructor.
//
WindowLineBuffer::WindowLineBuffer(WindowLineBuffer& pb, int Size_, int Length_, int Offset_):
_Owner(NULL),
_nextline(NULL),
_refline(NULL),
_ChainPosPtr(NULL),
_Enterable(FALSE),
_Selectable(FALSE),
_msg(NULL),

_buffer((pb._maxbuf > Offset_) ? (pb._buffer + Offset_):NULL),
_retbuf(pb._retbuf),
_joinbuf(NULL),
_tranbuf((pb._maxbuf > Offset_ && pb._tranbuf) ? (pb._tranbuf + Offset_):NULL),

_bufferalloc(false),
_retbufalloc(false),
_joinbufalloc(false),
_tranbufalloc(false),
_bufferscleared(0),

_bufferoffset(Offset_),
_inboxoffset(0),
_inwboxcnt(0),
_MovedToEnd(false),
_EndCharInserted(0),
_EndCharReverted(false),
_DelKeyDone(false),
_DelWrapOffset(0),
_linestatus(UNINIT_NL),
_maxbuf((pb._maxbuf > Offset_) ? (pb._maxbuf - Offset_):
        (Size_ > Offset_) ? Size_:MAXBUFFER),
_len((pb._maxbuf > Offset_) ? (pb._maxbuf - Offset_):
     (Length_ && Length_ <= Size_ &&
      Length_ > Offset_) ? Length_:
     (Size_ > Offset_) ? Size_:1),
_retbufsize(pb._retbufsize),

_SharedDataErased(false),
_SharedData(pb._SharedData),

_minlen(pb._minlen ? pb._minlen:1),
_chainminlen(0),
_index(0),
_xpos(pb._xpos),
_ypos(pb._ypos),
_noempty(pb._noempty),
_InsertMode(TRUE),
_Escaped(FALSE),
_Hidden(TRUE),
_TextWidgetActive(TRUE),
_ChainPos(0),
_InFocus(FALSE),

_SavedCursX(WHEREX()),
_SavedCursY(WHEREY()),

_BufChainNum(pb._BufChainNum + 1),
_WrapChars(false),

_RestFullBox(pb._RestFullBox),
_EndBrksActive(pb._EndBrksActive),
_TrimWs(pb._TrimWs),

// Internal data monitors
_insert_monitor(NULL),
_buffer_monitor(NULL),
_key_monitor(NULL),
_ext_monitor(NULL),
_index_monitor(NULL),
_escaped_monitor(NULL),
_hidden_monitor(NULL),
_active_monitor(NULL),
_infocus_monitor(NULL),
_inboxoffset_monitor(NULL),
_winlinenum_monitor(NULL),

// Handles to external objects, used by data monitors
_insert_handle(NULL),
_buffer_handle(NULL),
_key_handle(NULL),
_ext_handle(NULL),
_index_handle(NULL),
_escaped_handle(NULL),
_hidden_handle(NULL),
_active_handle(NULL),
_infocus_handle(NULL),
_inboxoffset_handle(NULL),
_winlinenum_handle(NULL)
{
  int index;
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  if (sdp && _linestatus == UNINIT_NL)
  {
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;
  
    if (::SafeStrLen(pb.RawStrBuffer(0)))
      _linestatus = FILLED_NL;
  }
  
  _ChainPosPtr = new ChainPosInfo;
  _brackets[0] = CONTMORE_LEFT;  // Continue more to the left (left triangle)
  _brackets[1] = CONTMORE_RIGHT; // Continue more to the right (right triangle)

  if (IsEnterable() && _buffer)
  {
    strcpy(_buffer, pb.RawStrBuffer(0));
    for (index = ::SafeStrLen(_buffer); index <= _maxbuf; index++)
      _buffer[index] = '\0';
  }
  else if (_buffer)
  {
    for (index = 0; index <= _maxbuf; index++)
      _buffer[index] = '\0';
  }

  if (_buffer == pb._buffer+Offset_ &&
      _maxbuf == pb._maxbuf-Offset_ &&
      _len == pb._maxbuf-Offset_)
    _bufferoffset = Offset_;
  else
    _bufferoffset = -1;

  if (pb._minlen)
    SetMinimumLength(pb._minlen);
  else
    SetMinimumLength(1);

  if (_len <= 0)
    SetInputLength(1, false);   // _len should never be zero
}

/****************************************************************************/
const char* WindowLineBuffer::cMonitor(const char*(*monptr_)(void*, WindowLineBuffer*, const char*), void* This_, const char* ptr_)
{
  if (monptr_ != NULL)
  {
    SaveCursorPosition();
    const char* RetVal_ = (*monptr_)(This_, this, ptr_);
    RestoreCursorPosition();

    return RetVal_;
  }

  return ptr_;
}

/****************************************************************************/
const int* WindowLineBuffer::iMonitor(const int*(*monptr_)(void*, WindowLineBuffer*, const int*), void* This_, const int* ptr_)
{
  if (monptr_ != NULL)
  {
    SaveCursorPosition();
    const int* RetVal_ = (*monptr_)(This_, this, ptr_);
    RestoreCursorPosition();

    return RetVal_;
  }

  return ptr_;
}

/****************************************************************************/
void WindowLineBuffer::SaveCursorPosition()
{
  _SavedCursX = WHEREX();
  _SavedCursY = WHEREY();
}

/****************************************************************************/
void WindowLineBuffer::RestoreCursorPosition()
{
  GOTOXY(_SavedCursX, _SavedCursY);
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorInsertMode(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _insert_handle = This_;
  _insert_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorInsertMode(_insert_handle, _insert_monitor);

  return &_InsertMode;
}

/****************************************************************************/
const char* WindowLineBuffer::MonitorBuffer(void* This_, const char*(*FnPtr_)(void*, WindowLineBuffer*, const char*))
{
  _buffer_handle = This_;
  _buffer_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorBuffer(_buffer_handle, _buffer_monitor);

  return _buffer;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorKey(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _key_handle = This_;
  _key_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorKey(_key_handle, _key_monitor);

  return &_key;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorExtendedKey(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _ext_handle = This_;
  _ext_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorExtendedKey(_ext_handle, _ext_monitor);

  return &_extended;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorIndex(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _index_handle = This_;
  _index_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorIndex(_index_handle, _index_monitor);

  return &_index;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorEscapedStatus(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _escaped_handle = This_;
  _escaped_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorEscapedStatus(_escaped_handle, _escaped_monitor);

  return &_Escaped;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorInFocusStatus(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _infocus_handle = This_;
  _infocus_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorInFocusStatus(_infocus_handle, _infocus_monitor);

  return &_InFocus;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorHiddenStatus(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _hidden_handle = This_;
  _hidden_monitor = FnPtr_;
  return &_Hidden;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorActiveStatus(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _active_handle = This_;
  _active_monitor = FnPtr_;
  return &_TextWidgetActive;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorInboxOffset(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  _inboxoffset_handle = This_;
  _inboxoffset_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorInboxOffset(_inboxoffset_handle, _inboxoffset_monitor);

  return &_inboxoffset;
}

/****************************************************************************/
const int* WindowLineBuffer::MonitorWinLineNum(void* This_, const int*(*FnPtr_)(void*, WindowLineBuffer*, const int*))
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return NULL;
  
  _winlinenum_handle = This_;
  _winlinenum_monitor = FnPtr_;

  if (_nextline)
    _nextline->MonitorWinLineNum(_winlinenum_handle, _winlinenum_monitor);

  return &sdp->_WinLineNum;
}

/****************************************************************************/
// PURPOSE:
//   Prompt box data member destruction method
//
// PRE:
//   The data members of this Prompt box object with original values
//
// POST:
//   If the data members of the box object is not NULL then the data members
//   are deleted and assigned NULL
//
void WindowLineBuffer::Clear()
{
  delete[] _msg;
  _msg = NULL;

  delete _refline;
  _refline = NULL;

  delete _ChainPosPtr;
  _ChainPosPtr = NULL;

  if (IsEnterable() && !_bufferoffset && _bufferalloc)
  {
    delete[] _buffer;
    _buffer = NULL;
    _bufferalloc = false;
  }

  if (_joinbufalloc)
  {
    delete[] _joinbuf;
    _joinbuf = NULL;
    _joinbufalloc = false;
  }

  if (_tranbufalloc)
  {
    delete[] _tranbuf;
    _tranbuf = NULL;
    _tranbufalloc = false;
  }

  if (_retbufalloc)
  {
    ChainRemoveRetBuf(_retbuf);
  
    delete[] _retbuf;
    _retbuf = NULL;
    _retbufalloc = false;
  }

  EraseSharedChainData(_SharedData);
  cMonitor(_buffer_monitor, _buffer_handle, _buffer);
}

/****************************************************************************/
/****************************************************************************/
// PURPOSE:
//   Prompt box assignment operator
//
// PRE:
//   The data members of this Prompt box object with original values
//
// POST:
//   The data members of this object is assigned to the corresponding
//   data members of the Prompt box argument passed to the method.
//   This object is returned.
//
WindowLineBuffer& WindowLineBuffer::operator = (WindowLineBuffer& pb)
{
  if (this != &pb)
  {
    int index;
    int len_;
    bool RetBufSet_ = false;
    
    WindowLineBuffer::SharedChainData* sdp = SharedData();
    Clear();

    if (pb.BuffersCleared())
      pb.RestoreUnusedBuffers();

    _nextline = NULL;
    _Enterable = _bufferoffset ? FALSE:pb._Enterable;
    _Selectable = pb._Selectable;
    _msg = pb._msg ? strcpy(new char[::SafeStrLen(pb._msg)+1], pb._msg):NULL;
    _maxbuf = (pb._maxbuf ? pb._maxbuf:
               _len ? _len:MAXBUFFER);
    _len = (pb.InputLength() ? pb.InputLength():
           (_maxbuf <= SCREEN_STDWIDTH) ? _maxbuf:1);
    _index = pb._index;
    _xpos = pb._xpos;
    _ypos = pb._ypos;
    _noempty = pb._noempty;
    _RestFullBox = pb._RestFullBox;
    _EndBrksActive = pb._EndBrksActive;
    _brackets[0] = pb._brackets[0];
    _brackets[1] = pb._brackets[1];

    if (IsEnterable() && _bufferalloc &&
        ::SafeStrLen(pb.RawStrBuffer(0)) > ::SafeStrLen(RawStrBuffer(0, false)))
    {
      if (!BufferTypeCleared(INPUT_BUFTYPE))
      {
        len_ = _maxbuf ? _maxbuf:
               _len ? _len:MAXBUFFER;
        delete[] _buffer;
        _buffer = NewCString(len_, 0, true);
        _bufferalloc = _buffer != NULL;
      }

      if (!BufferTypeCleared(RETURN_BUFTYPE))
      {
        if (pb._retbuf && pb._retbufalloc)
        {
          if (_bufferoffset > 0 || _BufChainNum > 0)
            pb.SetReturnBuffer(this, pb._retbuf, false, pb._retbufsize);
          else
            pb.SetReturnBuffer(this, pb._retbuf, true, (_len > 20) ? _len:20);
        }
        else
          CreateReturnBuffer(pb._retbufsize);
      }
    }
    else if (!IsEnterable() && !RetBufSet_)
    {
      if (pb._retbuf && pb._retbufalloc)
        pb.SetReturnBuffer(this, pb._retbuf, false, pb._retbufsize);
      else
        CreateReturnBuffer(pb._retbufsize);

      RetBufSet_ = true;
    }

    if (IsEnterable() && BuffersCleared() && (!_bufferalloc || !_buffer))
    {
      if (BufferTypeCleared(INPUT_BUFTYPE))
      {
        len_ = _maxbuf ? _maxbuf:
               _len ? _len:MAXBUFFER;
        delete[] _buffer;
        _buffer = NewCString(len_, 0, true);
        _bufferalloc = _buffer != NULL;

        if (_bufferalloc)
          SetBufferTypeCleared(INPUT_BUFTYPE, false);
      }

      if (BufferTypeCleared(RETURN_BUFTYPE))
      {
        if (pb._retbuf && pb._retbufalloc)
        {
          if (_bufferoffset > 0 || _BufChainNum > 0)
            pb.SetReturnBuffer(this, pb._retbuf, false, pb._retbufsize);
          else
            pb.SetReturnBuffer(this, pb._retbuf, true, (_len > 20) ? _len:20);
        }
        else
          CreateReturnBuffer(pb._retbufsize);
      }
    }
    else if (!IsEnterable() && !RetBufSet_)
    {
      if (pb._retbuf && pb._retbufalloc)
        pb.SetReturnBuffer(this, pb._retbuf, false, pb._retbufsize);
      else
        CreateReturnBuffer(pb._retbufsize);

      RetBufSet_ = true;
    }

    cMonitor(_buffer_monitor, _buffer_handle, _buffer);

    if (IsEnterable())
    {
      ResetBoxData();
      _joinbufalloc = _joinbufalloc && _joinbuf != NULL;

      if (_nextline == NULL && sdp && _linestatus == UNINIT_NL)
      {
        if (sdp->_numnextlines < 0)
          sdp->_numnextlines = 0;
      
        if (::SafeStrLen(pb.RawStrBuffer(0)))
          _linestatus = FILLED_NL;
      }

      if ((sdp && sdp->_parent != this) ||
          (!_SharedData && pb._SharedData && pb._SharedData->_parent == &pb))
        _SharedData = pb._SharedData;
      else if (!_SharedData)
      {
        _SharedData = new WindowLineBuffer::SharedChainData;
        if (_SharedData)
          _SharedData->_parent = this;
      }

      if (!BufferTypeCleared(JOIN_BUFTYPE))
      {
        if (_joinbufalloc)
        {
          delete[] _joinbuf;
          _joinbuf = NewCString(_maxbuf, NOJOIN_CHAR, true);
          _joinbufalloc = _joinbuf != NULL;
        }
        else
        {
          delete[] _joinbuf;
          _joinbuf = NULL;
          _joinbufalloc = false;
        }
      }

      if (BufferTypeCleared(JOIN_BUFTYPE) && (!_joinbufalloc || !_joinbuf))
      {
        delete[] _joinbuf;
        _joinbuf = NewCString(_maxbuf, NOJOIN_CHAR, true);
        _joinbufalloc = _joinbuf != NULL;

        if (_joinbufalloc)
          SetBufferTypeCleared(JOIN_BUFTYPE, false);
      }

      if (!_bufferoffset && _buffer)
        strcpy(_buffer, pb.RawStrBuffer(0));

      if (_buffer)
        for (index = ::SafeStrLen(_buffer); index <= _maxbuf; index++)
          _buffer[index] = '\0';

      if (_joinbufalloc)
      {
        strcpy(_joinbuf, pb._joinbuf);
        _joinbuf[_maxbuf] = '\0';

        for (index = ::SafeStrLen(_joinbuf); index < _maxbuf; index++)
          _joinbuf[index] = NOJOIN_CHAR;
      }
    }

    if (pb._tranbufalloc && pb._tranbuf)
      CreateExtendedAsciiBuffer();

    if (pb._minlen)
      SetMinimumLength(pb._minlen);
    else
      SetMinimumLength(1);
  }

  return *this;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::Make(WindowLineBuffer& pbox, int Size_, int Length_)
{
  WindowLineBuffer* pb;
  pb = new WindowLineBuffer(pbox, Size_, Length_);
  return pb;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::Make(char* msg, WindowLineBuffer* next_,
                                         WindowLine* owner_, int Size_, int Length_)
{
  WindowLineBuffer* pb;
  pb = new WindowLineBuffer(msg, next_, owner_, Size_, Length_);
  return pb;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::MakeNextLine(WindowLineBuffer* wpb)
{
  WindowLineBuffer* nwlb;
  nwlb = wpb ? (new WindowLineBuffer(*wpb, wpb->_maxbuf, wpb->_len)):NULL;

  if (wpb && nwlb)
  {
    int msglen_ = ::SafeStrLen(wpb->_msg);
    nwlb->_BufChainNum = wpb->_BufChainNum + 1;
    nwlb->Erase();
    nwlb->_BufChainNum = wpb->_BufChainNum;
    nwlb->SetInputLength(wpb->_len, false);
    nwlb->SetXpos(wpb->_xpos + msglen_ + wpb->_len +
                  wpb->EndBrksLength(false));
    nwlb->SetYpos(wpb->_ypos);
  }
  else if (!nwlb->RawStrBuffer(0))
  {
    delete nwlb;
    nwlb = NULL;
  }

  return nwlb;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::MakeVirtualLine(WindowLineBuffer* wpb)
{
  WindowLineBuffer* nwlb;
  nwlb = wpb ? (new WindowLineBuffer(*wpb, wpb->_maxbuf, wpb->_len, _len)):NULL;

  if (wpb && nwlb && nwlb->_bufferoffset == _len)
  {
    nwlb->Hide();
    nwlb->SetXpos(wpb->_xpos);
    nwlb->SetYpos(wpb->_ypos);
    
    if (wpb->_joinbufalloc)
    {
      delete[] wpb->_joinbuf;
      wpb->_joinbuf = NULL;
      wpb->_joinbufalloc = false;
    }

    wpb->Hide();    
  }
  else if (nwlb->_bufferoffset <= 0 || !nwlb->RawStrBuffer(0))
  {
    delete nwlb;
    nwlb = NULL;
  }
  
  return nwlb;
}

/****************************************************************************/
/****************************************************************************/
void WindowLineBuffer::PrintBlanks(int Max_)
{
  for (int Index_ = 0; Index_ < Max_; Index_++)
    PUTCH(BLANK);
}

/****************************************************************************/
char* WindowLineBuffer::NewCString(int len_, char pad_, bool AddZeroTerm_)
{
  int ext_ = AddZeroTerm_ ? 1:0;
  char* buffer_ = len_ ? (new char[len_ + ext_]):NULL;
  bool AllocDone_ = buffer_ != NULL;

  if (AllocDone_)
  {
    ::memset(buffer_, pad_, len_);

    if (AddZeroTerm_)
      buffer_[len_] = 0;
  }

  return buffer_;
}

/****************************************************************************/
// PURPOSE:
//   Set Prompt message method
//
// PRE:
//   The original value of the message data member
//
// POST:
//   The message data member is assigned new memory copied from the argument
//   string.
//
char* WindowLineBuffer::SetMessage(const char* str)
{
  if (str != _msg)
  {
    delete[] _msg;
    _msg = str ? strcpy(new char[::SafeStrLen(str)+1], str):NULL;
  }

  return _msg;
}

/****************************************************************************/
void WindowLineBuffer::RemoveMessage()
{
  SetMessage(NULL);
}

/****************************************************************************/
void WindowLineBuffer::BlankMessage()
{
  int i, Len_;
  Len_ = ::SafeStrLen(_msg);

  for (i = 0; i < Len_; ++i)
    _msg[i] = ' ';
}

/****************************************************************************/
void WindowLineBuffer::PushInputString(char* str)
{
  if (!str)
    return;

  bool enterb_ = IsEnterable();
  bool vbuf_ = VirtualBufferOffset() && enterb_;
  short Index_;
  char* temps_ = strcpy(new char[::SafeStrLen(str)+1], str);

  if (vbuf_)
    _ChainPosPtr->SetupInfoPtrs(this, 0);

  if (IsValidString(str))
  {
    temps_ = Convert(temps_);

    if (vbuf_)
    {
      for (Index_ = ::SafeStrLen(temps_) - 1; Index_ >= 0; Index_--)
      {
        _ChainPosPtr->SetPushInfo(0);
        Push(0, temps_[Index_], NULL, _ChainPosPtr);
      }
    }
    else
    {
      for (Index_ = ::SafeStrLen(temps_) - 1; Index_ >= 0; Index_--)
        Push(0, temps_[Index_], NULL, NULL);
    }
  }

  if (temps_)  
  {
    delete[] temps_;
    temps_ = NULL;
  }
}

/****************************************************************************/
void WindowLineBuffer::FillCharAtPos(char ch, int pos, bool endterm_)
{
  if (IsEnterable())
  {
    int index = 0;
    if (BuffersCleared())
      RestoreUnusedBuffers();
    
    ResetBoxData();

    if (_buffer && pos <= strlen(_buffer) && pos + 1 < _maxbuf)
    {
      memset(&_buffer[pos], ch, _len-pos);
      if (endterm_ && _len < _maxbuf)
        _buffer[_len] = 0;

      for (index = _len; index <= _maxbuf; index++)
        _buffer[index] = '\0';

      if (_joinbufalloc)
      {
        memset(&_joinbuf[pos], NOJOIN_CHAR, _len-pos);
        _joinbuf[_maxbuf] = '\0';

        for (index = _len; index < _maxbuf; index++)
          _joinbuf[index] = NOJOIN_CHAR;
      }
      
      _buffer = Convert(_buffer);
    }
    else if (_buffer)
    {
      memset(&_buffer[pos], 0, _maxbuf-pos);

      if (_joinbufalloc)
      {
        memset(&_joinbuf[pos], NOJOIN_CHAR, _maxbuf-pos);
        _joinbuf[_maxbuf] = '\0';
      }
    }
  }
}

/****************************************************************************/
void WindowLineBuffer::SetInputStringAtPos(char* str, int len, int pos, bool endterm_)
{
  if (IsEnterable() && (!str || (strlen(str) <= _maxbuf && IsValidString(str))))
  {
    int index = 0;
    if (BuffersCleared())
      RestoreUnusedBuffers();
    
    ResetBoxData();

    if (_buffer && pos <= strlen(_buffer) &&
        str && pos + strlen(str) < _maxbuf)
    {
      memmove(&_buffer[pos], str, len);
      if (endterm_ && pos+len < _maxbuf)
        _buffer[pos+len] = 0;

      for (index = pos+len; index <= _maxbuf; index++)
        _buffer[index] = '\0';

      if (_joinbufalloc)
      {
        memset(&_joinbuf[pos], NOJOIN_CHAR, len);
        _joinbuf[_maxbuf] = '\0';

        for (index = pos+len; index < _maxbuf; index++)
          _joinbuf[index] = NOJOIN_CHAR;
      }
      
      _buffer = Convert(_buffer);
    }
    else if (_buffer)
    {
      memset(&_buffer[pos], 0, _maxbuf-pos);

      if (_joinbufalloc)
      {
        memset(&_joinbuf[pos], NOJOIN_CHAR, _maxbuf-pos);
        _joinbuf[_maxbuf] = '\0';
      }
    }
  }
}

/****************************************************************************/
// PURPOSE:
//   Set default input method
//
// PRE:
//   The original value of the input buffer data member
//
// POST:
//   The input buffer data member is assigned new memory copied from the
//   argument string. The remaining space in the buffer is NULL filled.
//
void WindowLineBuffer::SetInputString(char* str)
{
  if (IsEnterable() && (!str || (strlen(str) <= _maxbuf && IsValidString(str))))
  {
    int index = 0;
    if (BuffersCleared())
      RestoreUnusedBuffers();
    
    ResetBoxData();

    if (str && _buffer)
    {
      strcpy(_buffer, str);
      for (index = ::SafeStrLen(_buffer); index <= _maxbuf; index++)
        _buffer[index] = '\0';

      if (_joinbufalloc)
      {
        memset(_joinbuf, NOJOIN_CHAR, strlen(str));
        _joinbuf[_maxbuf] = '\0';

        for (index = ::SafeStrLen(_buffer); index < _maxbuf; index++)
          _joinbuf[index] = NOJOIN_CHAR;
      }
      
      _buffer = Convert(_buffer);
    }
    else if (_buffer)
      memset(_buffer, 0, _maxbuf);
  }
}

/****************************************************************************/
void WindowLineBuffer::ResetBoxData()
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  _index = 0;
  _inboxoffset = 0;
  _inwboxcnt = 0;
  _MovedToEnd = false;
  _EndCharInserted = 0;
  _EndCharReverted = false;
  _DelKeyDone = false;
  _DelWrapOffset = 0;

  // resetting join buffer
  if (_joinbufalloc)
  {
    memset(_joinbuf, NOJOIN_CHAR, _maxbuf);
    _joinbuf[_maxbuf] = '\0';
  }

  if (sdp)
  {
    sdp->_InWinMode = false;
    sdp->_NextLineReady = false;
    sdp->_PrevLineReady = false;

    sdp->_movedone = false;
    sdp->_BoxPosSaved = false;
    sdp->_BoxPosTrans = false;

    sdp->_SavedOffset = 0;
    sdp->_SavedIndex = 0;
    sdp->_SavedExtIndex = 0;
    sdp->_SavedBoxDiff = 0;
    
    sdp->_NextOffset = 0;
    sdp->_NextIndex = 0;
    sdp->_NextExtIndex = 0;
    sdp->_NextBoxDiff = 0;
  }
}

/****************************************************************************/
void WindowLineBuffer::CharPadToEnd(char ch, WindowLineBuffer::SharedChainData* ptr)
{
  int x;
  bool eol = false;
  bool cpadset_ = false;
  int cpadpos_ = ptr ? ptr->_cpadpos:-1;
  int cpadchain_ = ptr ? ptr->_cpadchain:-1;
  
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  if (!sdp)
    return;
  else
    sdp->_padmax = 0;

  // ppp1111ibbbrrr...
  //         <-- b or r occupies same cursor pos as index i so shift back 1
  // ppp1111bbbrrr...
  
  // sdp->_SavedOffset == _inwboxcnt : p
  // sdp->_SavedIndex == _index : i
  // sdp->_SavedBoxDiff == _refline->PutExt(key) : b
  // sdp->_SavedExtIndex == _refline->PutExt(key) : r
  //
  if (sdp->_BoxPosSaved)
  {
    sdp->_padmax = sdp->_SavedIndex +
                   sdp->_SavedBoxDiff +
                   sdp->_SavedExtIndex +
                   sdp->_SavedOffset;

    if (sdp->_SavedBoxDiff || sdp->_SavedExtIndex)
      sdp->_padmax -= CalcBackShift(_buffer);
  }
  else
    sdp->_padmax = -1;

  if (BuffersCleared())
    RestoreUnusedBuffers();
    
  _buffer[_len] = 0;
  
  if (_joinbufalloc)
    _joinbuf[_maxbuf] = '\0';

  if (ch != 0 && (!sdp->_cpadset || cpadpos_ < 0 || cpadchain_ < 0))
  {
    sdp->_cpadpos = cpadpos_;
    sdp->_cpadchain = cpadchain_;
    sdp->_cpadset = cpadset_ = false;

    if (_BufChainNum == 0)
      sdp->_padcnt = 0;
  }
  
  for (x = 0; x < _len; x++, sdp->_padcnt++)
  {
    if (!eol)
      eol = !_buffer[x] ||
            (ch == 0 && sdp->_cpadset &&
             cpadpos_ == x && cpadchain_ == _BufChainNum);
    
    if ((!_buffer[x] || eol) &&
        (sdp->_padmax < 0 || sdp->_padcnt < sdp->_padmax))
    {
      _buffer[x] = ch;
      
      if (_joinbufalloc)
        _joinbuf[x] = NOJOIN_CHAR;

      if (eol && ch && !sdp->_cpadset)
      {
        sdp->_cpadpos = x;
        sdp->_cpadchain = _BufChainNum;
        sdp->_cpadset = cpadset_ = true;
      }
    }
  }

  if (_joinbufalloc && eol)
    while (x < _maxbuf)
      _joinbuf[x++] = NOJOIN_CHAR;

  if (_joinbufalloc && x == _maxbuf)
    _joinbuf[x] = '\0';

  if (_nextline)
    _nextline->CharPadToEnd(ch, sdp->_cpadset ? sdp:ptr);
}

/****************************************************************************/
// PURPOSE:
//   Set input length method
//
// PRE:
//   The original value of the input length data member
//
// POST:
//   The input length data member is copied from the length argument.
//
void WindowLineBuffer::SetInputLength(int len, bool MakeVbuf_)
{
  if (len <= _maxbuf && len >= _minlen && len >= 1 && IsEnterable())
  {
    _len = len;

    if (MakeVbuf_ && _len < _maxbuf && _BufChainNum == 0 && !_nextline)
    {
      WindowLineBuffer* npb = MakeVirtualLine(this);
      if (npb)
        AddBoxChain(npb, false);
    }
  }
}

/****************************************************************************/
void WindowLineBuffer::DivBufSizeAsInputLength(int div_, bool MakeVbuf_)
{
  if (_maxbuf && IsEnterable())
  {
    int res_ = _maxbuf / div_;
    
    if (res_ >= _minlen && res_ >= 1)
    {
      _len = res_;

      if (MakeVbuf_ && _len < _maxbuf && _BufChainNum == 0 && !_nextline)
      {      
        WindowLineBuffer* npb = MakeVirtualLine(this);
        if (npb)
          AddBoxChain(npb, false);
      }
    }
  }
}

/****************************************************************************/
// PURPOSE:
//   Set minimum input length method
//
// PRE:
//   The original value of the minimum input length data member
//
// POST:
//   The mimimum input length data member is copied from the length argument.
//
void WindowLineBuffer::SetMinimumLength(int len)
{
  if (len <= _len)
  {
    _minlen = (len > 0) ? len:1;
    _chainminlen = 0;
  }
  else
  {
    _minlen = _len;
    _chainminlen = len - _len;

    if (_nextline && !VirtualBufferOffset())
      _nextline->SetMinimumLength(_chainminlen);
  }
}

/****************************************************************************/
int* WindowLineBuffer::WordChr(int* src_, int val_, size_t n)
{
  while (n)
    if (*src_ == val_)
      return src_;
    else
    {
      src_++;
      --n;
    }

  return NULL;
}

/****************************************************************************/
int WindowLineBuffer::EndBrksLength(bool OnGoto_) const
{
  return (_EndBrksActive ? (OnGoto_ ? 1:2):0);
}

/****************************************************************************/
void WindowLineBuffer::SetEndBrksActive(bool Flag_)
{
  _EndBrksActive = Flag_;

  if (_nextline && !VirtualBufferOffset())
    _nextline->SetEndBrksActive(Flag_);
}

/****************************************************************************/
void WindowLineBuffer::SetRestrictFullBox(bool Flag_)
{
  _RestFullBox = Flag_;

  if (_nextline && IsEnterable())
    _nextline->SetRestrictFullBox(Flag_);
}

/****************************************************************************/
void WindowLineBuffer::SetTrimWs(bool Flag_)
{
  _TrimWs = Flag_;

  if (_nextline && IsEnterable())
    _nextline->SetTrimWs(Flag_);
}

/****************************************************************************/
void WindowLineBuffer::SetNoEmpty(int flag)
{
  _noempty = flag;

  if (_nextline && IsEnterable())
    _nextline->SetNoEmpty(_noempty);
}

/****************************************************************************/
void WindowLineBuffer::NotifyInsertMode(const int* ModePtr_)
{
  if (ModePtr_)
    _InsertMode = *ModePtr_;

  iMonitor(_insert_monitor, _insert_handle, ModePtr_);
}

/****************************************************************************/
void WindowLineBuffer::SetInsertMode(int Mode_, WindowLineBuffer* Parent_)
{
  _InsertMode = Mode_;
  iMonitor(_insert_monitor, _insert_handle, &_InsertMode);

  if (Parent_)
    MonitorInsertMode(Parent_, InsertDetect);

  if (_nextline && IsEnterable())
    _nextline->SetInsertMode(_InsertMode);
}

/****************************************************************************/
bool WindowLineBuffer::SetXYpos(int x, int y)
{
  bool WasHid_ = _Hidden;
  int maxx_ = TextGraphics::TerminalScreenCols();
  int maxy_ = TextGraphics::TerminalScreenRows();

  if ((1 <= x && x <= maxx_) &&
      (1 <= y && y <= maxy_))
  {
    if (!WasHid_)
      Hide();

    _xpos = x;
    _ypos = y;

    if (_nextline)
    {
      int msglen_ = ::SafeStrLen(_msg);
      _nextline->SetXYpos(_xpos + msglen_ + _len + EndBrksLength(false), _ypos);
    }

    if (_Hidden && !WasHid_)
      Show();

    return true;
  }

  return false;
}

/****************************************************************************/
bool WindowLineBuffer::SetXpos(int x)
{
  bool WasHid_ = _Hidden;
  int maxx_ = TextGraphics::TerminalScreenCols();

  if (1 <= x && x <= maxx_)
  {
    if (!WasHid_)
      Hide();

    _xpos = x;

    if (_nextline)
    {
      int msglen_ = ::SafeStrLen(_msg);
      _nextline->SetXpos(_xpos + msglen_ + _len + EndBrksLength(false));
    }

    if (_Hidden && !WasHid_)
      Show();

    return true;
  }

  return false;
}

/****************************************************************************/
bool WindowLineBuffer::SetYpos(int y)
{
  bool WasHid_ = _Hidden;
  int maxy_ = TextGraphics::TerminalScreenRows();

  if (1 <= y && y <= maxy_)
  {
    if (!WasHid_)
      Hide();

    _ypos = y;

    if (_nextline)
      _nextline->SetYpos(_ypos);

    if (_Hidden && !WasHid_)
      Show();

    return true;
  }

  return false;
}

/****************************************************************************/
int WindowLineBuffer::GetXpos() const
{
  return _xpos;
}

/****************************************************************************/
int WindowLineBuffer::GetYpos() const
{
  return _ypos;
}

/****************************************************************************/
// PURPOSE:
//   input string validity check method
//
// PRE:
//   char* str : The original value of the input string
//
// POST:
//   Returns 1 if all the characters within the input string is defined
//   as valid, otherwise returns 0.
//
int WindowLineBuffer::IsValidString(char* str)
{
  return TRUE;
}

/****************************************************************************/
int WindowLineBuffer::IsValidLength() const
{
  return
  (
    (::SafeStrLen(RawStrBuffer(0)) >= _minlen || !_noempty) &&
    ((_nextline && !VirtualBufferOffset()) ?
          _nextline->IsValidLength():1)
  );
}

/****************************************************************************/
#if (!(defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__)) & (defined(__linux__) | defined(__unix__)))
int WindowLineBuffer::TranslateKeys(int key)
{
  return
  (
    GetKeyMap().KeyMatch(key, INSERT)   ? KEY_IC:
    GetKeyMap().KeyMatch(key, LEFT)     ? KEY_LEFT:
    GetKeyMap().KeyMatch(key, RIGHT)    ? KEY_RIGHT:
    GetKeyMap().KeyMatch(key, HOME)     ? KEY_HOME:
    GetKeyMap().KeyMatch(key, END)      ? KEY_END:
    GetKeyMap().KeyMatch(key, DELETE)   ? KEY_DC:
    GetKeyMap().KeyMatch(key, ENTER)    ? 10:
    GetKeyMap().KeyMatch(key, BACK)     ? KEY_BACKSPACE:
    GetKeyMap().KeyMatch(key, ESCAPE)   ? 27:
    GetKeyMap().KeyMatch(key, NEXT)     ? 9:
    GetKeyMap().KeyMatch(key, PAGEUP)   ? KEY_PPAGE:
    GetKeyMap().KeyMatch(key, PAGEDOWN) ? KEY_NPAGE:
    GetKeyMap().KeyMatch(key, UP)       ? KEY_UP:
    GetKeyMap().KeyMatch(key, DOWN)     ? KEY_DOWN:
                                          key
  );
}
#endif

/****************************************************************************/
// PURPOSE:
//   Keyboard retrieval procedure
//
// PRE:
//   The original value of _key and _extended data members
//
// POST:
//   The keyboard is read for a key and is assigned to _key data member.
//   If the key is extended non-ASCII then the _extended data member is set.
//   Returns the key read from the keyboard.
//
int WindowLineBuffer::GetKey()
{
  _extkey = _key = (int)GETCH();
  iMonitor(_key_monitor, _key_handle, &_key);
  _extended = (_key == 0 || _key == 0xE0) ? 256:0;
  iMonitor(_ext_monitor, _ext_handle, &_extended);

  if (_extended)
  {
    #if (!(defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__)) & (defined(__linux__) | defined(__unix__)))
      noecho();
    #endif
    _key = (int)GETCH();
    _extkey = _key + _extended;
    iMonitor(_key_monitor, _key_handle, &_key);

    #if (!(defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__)) & (defined(__linux__) | defined(__unix__)))
      _extkey = TranslateKeys(_extkey);
    #endif
  }  
  #if (!(defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__)) & (defined(__linux__) | defined(__unix__)))
  else
    _extkey = TranslateKeys(_extkey);
  #endif

  return _key;
}

/****************************************************************************/
int WindowLineBuffer::VbufStrLenNoTerm(bool NoTerm_) const
{
  if (!IsEnterable() || _bufferoffset || _BufChainNum)
    return 0;

  char lastchar_ = 0;

  int startpt_ = 0;
  int endpt_ = _len;
  
  int len_ = 0;
  int vmax_ = 0;

  if (_buffer)
  {
    vmax_ = VirtualBufferMax();
    if (_inboxoffset < vmax_)
      vmax_ = _inboxoffset;

    startpt_ += vmax_;
    endpt_ += vmax_;
    lastchar_ = _buffer[endpt_];

    if (!NoTerm_)
      _buffer[endpt_] = 0;
    
    len_ = ::SafeStrLen(&_buffer[startpt_]);
    _buffer[endpt_] = lastchar_;
  }

  return len_;
}

/****************************************************************************/
int WindowLineBuffer::VbufStrLen(int* lastnwspos_, int* StartPos_,
                                 int* EndPos_, char* LastCharp_) const
{
  if (!IsEnterable() || _bufferoffset || _BufChainNum)
    return 0;

  char lastchar_ = 0;
  int startpt_ = 0;
  int endpt_ = _len;
  int len_ = 0;
  int vmax_ = 0;
  int WrapIndex_ = 0;
  int WrapMax_ = 0;
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;

  if (_buffer)
  {
    WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
    WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
  
    vmax_ = VirtualBufferMax();
    if (_inboxoffset < vmax_)
      vmax_ = _inboxoffset;

    startpt_ += vmax_;
    endpt_ += vmax_;

    #if (WINLINE_ONLYCWSCHR & WINLINE_WSENDMOVE)
      if (lastnwspos_)
        *lastnwspos_ = HasNonWsChars(&_buffer[startpt_]);
    #else
      if (lastnwspos_)
        *lastnwspos_ =
          (_inboxoffset > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false)
           _inboxoffset >= WrapMax_ && WrapIndex_ <= _inboxoffset) ?
               HasNonNullChars(&_buffer[startpt_]):
               HasNonWsChars(&_buffer[startpt_]);
    #endif
    
    lastchar_ = _buffer[endpt_];
    _buffer[endpt_] = 0;
    len_ = ::SafeStrLen(&_buffer[startpt_]);

    if (StartPos_)
      *StartPos_ = startpt_;

    if (EndPos_)
      *EndPos_ = endpt_;

    if (LastCharp_)
      *LastCharp_ = lastchar_;

    _buffer[endpt_] = lastchar_;
  }

  return len_;
}

/****************************************************************************/
void WindowLineBuffer::ClearDelWrapOffset(int boxshift_, int WrapMax_)
{
  if (_DelKeyDone && _DelWrapOffset > 0)
  {
    int bufoffset_ = boxshift_ - WrapMax_;
    
    if (bufoffset_ <= 0)
    {
      _DelKeyDone = false;
      _DelWrapOffset = 0;
    }
    else
      _DelWrapOffset = bufoffset_;
  }
}

/****************************************************************************/
bool WindowLineBuffer::CompareVal(int Val1_, int Val2_, int Relation_)
{
  return
  (
    (Relation_ == RelationalEnum::EQUAL)     ?             (Val1_ == Val2_):
    (Relation_ == RelationalEnum::NOT_EQUAL) ?             (Val1_ != Val2_):
    (Relation_ == RelationalEnum::LESS_THAN) ?             (Val1_ < Val2_):
    (Relation_ == RelationalEnum::LESS_THAN_OR_EQUAL) ?    (Val1_ <= Val2_):
    (Relation_ == RelationalEnum::GREATER_THAN) ?          (Val1_ > Val2_):
    (Relation_ == RelationalEnum::GREATER_THAN_OR_EQUAL) ? (Val1_ >= Val2_):
                                                           false
  );
}

/****************************************************************************/
bool WindowLineBuffer::AtMaxWrap(int WrapIndex_, int WrapMax_, int endpt_, int Relation_,
                                 bool AtPosIndex_, bool InclDelWrapOff_,
                                 bool* AtEnd_, bool* PastEnd_)
{
  bool enterb_ = IsEnterable();
  bool NextBufOff_ = _nextline && _nextline->_bufferoffset;
  bool LwAllowed_ = _Owner && ((WindowLine*)_Owner)->LineWrapAllowed();

  bool vbuf_ = BufferChainPosition() == 0 && enterb_ && NextBufOff_ &&
               VirtualBufferOffset();
  bool inwbox_ = BufferChainPosition() == 0 && enterb_ && !vbuf_ && LwAllowed_;

  bool bAtEnd_;
  bool bPastEnd_;
  bool CheckPastEnd_ = Relation_ == RelationalEnum::GREATER_THAN ||
                       Relation_ == RelationalEnum::GREATER_THAN_OR_EQUAL;
  
  bool CompareWithDelWrapOffset_ = InclDelWrapOff_ && _DelWrapOffset;

  if (!AtPosIndex_ && endpt_ == 0)
    endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);

  if (CompareWithDelWrapOffset_)
  {
    endpt_ -= _DelWrapOffset;
    WrapMax_ = GetMaxWrap(WrapMax_);
  }

  bAtEnd_ = _index == endpt_;
  bPastEnd_ = _index > endpt_;

  if (AtEnd_)
    *AtEnd_ = bAtEnd_;

  if (PastEnd_)
    *PastEnd_ = bPastEnd_;

  return (((AtPosIndex_ && (bAtEnd_ || (CheckPastEnd_ && bPastEnd_))) ||
           (!AtPosIndex_ && (0 <= _index && _index <= endpt_))) &&
          CompareVal(WrapIndex_, WrapMax_, Relation_));
}

/****************************************************************************/
bool WindowLineBuffer::CursorMoveCondMet(int WrapIndex_, int WrapMax_, int endpt_, int NonWsLen_)
{
  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);
  bool CondMet_ = false;
  bool AtMaxWrap_ = false;

  int Lesser_ = RelationalEnum::LESS_THAN;
  int LesserOrEqual_ = RelationalEnum::LESS_THAN_OR_EQUAL;

  if (inwbox_)
  {
    AtMaxWrap_ = AtMaxWrap(WrapIndex_, WrapMax_, 0, Lesser_, false, false);
  
    if (_index < ::SafeStrLen(_buffer) && _index < endpt_ &&
        (!_inwboxcnt || (_inwboxcnt > 0 &&
         (AtMaxWrap_ || (_inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt)))))
    {
      if (_inwboxcnt > 0)
      {
        if (AtMaxWrap_)
        {
          if (_inwboxcnt <= WrapMax_)
            CondMet_ = (NonWsLen_ < _len) ? (_index < NonWsLen_):
                                            (_index < endpt_);
        }
        else if (_inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt)
        {
          if (NonWsLen_ < _len)
            CondMet_ = (_inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt && _index < NonWsLen_);
          else
            CondMet_ = (AtMaxWrap(WrapIndex_, WrapMax_, 0, LesserOrEqual_, false, false) &&
                        _inwboxcnt == WrapMax_ && _index < endpt_);
        }
        else if (_inwboxcnt < WrapMax_)
          CondMet_ = _index < endpt_;
      }
      else
        CondMet_ = !_inwboxcnt && _index < ::SafeStrLen(_buffer) && _index < endpt_;
    }
  }
  else if (vbuf_)
  {
    AtMaxWrap_ = AtMaxWrap(WrapIndex_, WrapMax_, 0, Lesser_, false, false);
  
    if (_index < VbufStrLenNoTerm(false) && _index < endpt_ &&
        (!_inboxoffset || (_inboxoffset > 0 &&
         (AtMaxWrap_ || (_inboxoffset >= WrapMax_ && WrapIndex_ <= _inboxoffset)))))
    {
      if (_inboxoffset > 0)
      {
        if (AtMaxWrap_)
        {
          if (_inboxoffset <= WrapMax_)
            CondMet_ = (NonWsLen_ < _len) ? (_index < NonWsLen_):
                                            (_index < endpt_);
        }
        else if (_inboxoffset >= WrapMax_ && WrapIndex_ <= _inboxoffset)
        {
          if (NonWsLen_ < _len)
            CondMet_ = (_inboxoffset >= WrapMax_ && WrapIndex_ <= _inboxoffset && _index < NonWsLen_);
          else
            CondMet_ = (AtMaxWrap(WrapIndex_, WrapMax_, 0, LesserOrEqual_, false, false) &&
                        _inboxoffset == WrapMax_ && _index < endpt_);
        }
        else if (_inboxoffset < WrapMax_)
          CondMet_ = _index < endpt_;
      }
      else
        CondMet_ = !_inboxoffset && _index < VbufStrLenNoTerm(false) && _index < endpt_;
    }
  }

  return CondMet_;
}

/****************************************************************************/
// PURPOSE:
//   Method for putting extended non-ASCII keyboard keys to the console
//   output.
//
// PRE:
//   int key : the key to be outputted.
//
// POST:
//   The key is outputted to the screen. Appropriate changes to the
//   internal input buffer is made.
//
bool WindowLineBuffer::PutExtKey(int key)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return false;

  char nextchar_;
  char nextjoin_;
  char nextchain_;

  int pos;
  int WrapIndex_ = 0;
  int WrapMax_ = 0;
  int EndReached_ = 0;
  int NextNonWsLen_ = 0;
  int NonWsLen_ = 0;
  int vmax_ = 0;
  int endpt_ = 0;
  int endptdex_ = 0;
  int index_ = 0;
  int delta_ = 0;
  int bufoffset_ = 0;
  int Equal_ = RelationalEnum::EQUAL;
  int Greater_ = RelationalEnum::GREATER_THAN;
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;
  int Lesser_ = RelationalEnum::LESS_THAN;
  int LesserOrEqual_ = RelationalEnum::LESS_THAN_OR_EQUAL;

  bool AtZero_ = false;
  bool DoChain_ = false;
  bool GotoDone_ = false;
  bool EndMet_ = false;
  bool NextLen_ = false;
  bool NullChar_ = false;
  bool PrevNull_ = false;
  bool CondMet_ = false;
  bool PosMoveDone_ = false;
  bool KeyDone_ = false;
  bool PastEndPt_ = false;
  bool AtEndPt_ = false;
  bool vbuf_ = false;
  bool inwbox_ = false;
  bool RefLineMoved_ = false;
  bool MoveReversed_ = false;
  bool MoveDone_ = false;
  bool AtDexEnd_ = false;
  bool AtMaxWrap_ = false;
  bool AtEnd_;
  bool PastEnd_;  
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_,
                                 &EndReached_, &NextNonWsLen_,
                                 &WrapIndex_, &WrapMax_);

  #if WINLINE_DEBUG3
    bool Debug_ = true;
  #else
    bool Debug_ = false;
  #endif

  sdp->_InWinMode = enterb_ ?
      ((WindowLine*)_Owner)->InWindowMode(&sdp->_NextLineReady,
                                          &sdp->_PrevLineReady):false;
      
  if (inwbox_)
  {
    SetRefLineMovePending(true, false);
    NonWsLen_ = NextNonWsLen_;
  }
  else
    VbufStrLen(&NonWsLen_);

  // Action: TOGGLE_INSERT
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  if (GetKeyMap().KeyMatch(key, INSERT))
#elif (defined(__linux__) | defined(__unix__))
  if (key == KEY_IC)
#endif
  {
    SetInsertMode(!GetInsertMode());
    PosMoveDone_ = true;
  }

  // Action: GO_LEFT (chain push if WindowLine::_wraparoundindex > 0)
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, LEFT))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_LEFT)
#endif
  {
    if (_Owner)
    {
      DoChain_ = IsChainable(false);
      endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);
    }

    // Modified to wrap chars around first buffer in window buffer object
    if (!vbuf_ && !inwbox_ && _index > 0)
    {
      GOTOXY(wherex()-1, _ypos);
      PosMoveDone_ = true;
      --_index;
      iMonitor(_index_monitor, _index_handle, &_index);

      if (_refline)
        _refline->DecrBoxIndex();
    }
    else if (vbuf_)
    {
      if (BuffersCleared())
        RestoreUnusedBuffers();
    
      if (_index == 0 &&
          DoChain_ && WrapIndex_ > 0 && _inboxoffset > 0)
      {
        PrevNull_ = _buffer[_inboxoffset + endpt_] == 0;
        _inboxoffset--;
        nextchar_ = _buffer[_inboxoffset];
        ((WindowLine*)_Owner)->DecrWrapAround(1);
        WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
        WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();

        pos = wherex();
        ShowInput();
        GOTOXY(pos, _ypos);
        PosMoveDone_ = true;
        iMonitor(_inboxoffset_monitor, _inboxoffset_handle, &_inboxoffset);

        if (WrapIndex_ > _inboxoffset)
        {
          ((WindowLine*)_Owner)->DecrWrapAround(WrapIndex_ - _inboxoffset);
          if (_refline)
            _refline->SetBoxIndex(_refline->BufferIndex() - _inboxoffset);

          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
        }
        else if (_refline)
          _refline->DecrBoxIndex();

        NextLen_ = EndReached_ ? 0:NextNonWsLen_;
        NullChar_ = _buffer[_inboxoffset + endpt_] == 0;

        if ((AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, true, false) &&
             nextchar_ && PrevNull_ && !NextLen_ && !NullChar_) ||
            (_inboxoffset == 0 && VbufStrLenNoTerm() <= _len))
        {
          _MovedToEnd = false;
          _EndCharInserted = _buffer[_inboxoffset + endpt_];
          _EndCharReverted = true;
        }

        ClearDelWrapOffset(_inboxoffset, WrapMax_);
      }
      else if (_index > 0)
      {
        GOTOXY(wherex()-1, _ypos);
        PosMoveDone_ = true;
        --_index;
        iMonitor(_index_monitor, _index_handle, &_index);

        endpt_ += _inboxoffset;
        NextLen_ = EndReached_ ? 0:NextNonWsLen_;
        NullChar_ = _buffer[endpt_] == 0;

        if ((AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, true, false) &&
             !NextLen_ && !NullChar_) ||
            (_inboxoffset == 0 && VbufStrLenNoTerm() <= _len))
        {
          _MovedToEnd = false;
          _EndCharInserted = _buffer[endpt_];
          _EndCharReverted = true;
        }
        
        if (_refline)
          _refline->DecrBoxIndex();
      }
    }
    else if (inwbox_)
    {
      if (BuffersCleared())
        RestoreUnusedBuffers();

      AtEndPt_ = IsAtEndPt(&PastEndPt_);
      
      if (_index == 0 && !(PastEndPt_ || CursorAtEOL(true)) &&
          DoChain_ && WrapIndex_ > 0 && _inwboxcnt > 0)
      {
        _WrapChars = true;
        PrevNull_ = _buffer[endpt_] == 0;
        _inwboxcnt--;
        nextchar_ = TakeNextLastChar();
        nextjoin_ = NOJOIN_CHAR;
        MoveDone_ = Push(0, nextchar_, &nextjoin_, NULL);
        WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
        WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();

        pos = wherex();
        ShowInput();
        GOTOXY(pos, _ypos);
        PosMoveDone_ = MoveDone_;

        if (WrapIndex_ > _inwboxcnt)
        {
          ((WindowLine*)_Owner)->DecrWrapAround(WrapIndex_ - _inwboxcnt);
          if (_refline)
            _refline->SetBoxIndex(_refline->BufferIndex() - _inwboxcnt);

          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
        }
        else if (_refline)
        {
          if (_refline->BoxDiff() || _refline->ExtendedIndex())
            _refline->PutExtKey(key);
          else
            _refline->DecrBoxIndex();
        }

        if (MoveDone_)
        {
          NullChar_ = _buffer[endpt_] == 0;

          if ((AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, true, false) &&
               nextchar_ && PrevNull_ && !NullChar_) ||
              (_inwboxcnt == 0 && NextLength() == 0))
          {
            _MovedToEnd = false;
            _EndCharInserted = _buffer[endpt_];
            _EndCharReverted = true;
          }
        }

        ClearDelWrapOffset(_inwboxcnt, WrapMax_);
      }
      else
      {
        if (PastEndPt_ || CursorAtEOL(true))
        {
          // new
          _refline->PutExtKey(key);
          pos = wherex();
          ShowInput();
          GOTOXY(pos, _ypos);
          PosMoveDone_ = false;
          AtZero_ = _index == 0;
        }

        if (!PosMoveDone_ && _index > 0)
        {
          GOTOXY(wherex()-1, _ypos);
          PosMoveDone_ = true;
          --_index;
          iMonitor(_index_monitor, _index_handle, &_index);

          NextLen_ = 0;
          NullChar_ = _buffer[endpt_] == 0;

          if ((AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, true, false) &&
               !NextLen_ && !NullChar_) ||
              (_inwboxcnt == 0 && NextLength() == 0))
          {
            _MovedToEnd = false;
            _EndCharInserted = _buffer[endpt_];
            _EndCharReverted = true;
          }
          
          if (_refline && RefLineMovePending())
            _refline->DecrBoxIndex();
        }
        else if (!PosMoveDone_ && AtZero_ &&
                 _refline->ExtendedIndexToggled())
        {
          _refline->ResetExtendedIndexToggle();
          PutExtKey(key);
        }
      }
    }
  }

  // Action: GO_RIGHT (chain pull if _index >= WindowLine::_inputlen)
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, RIGHT))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_RIGHT)
#endif  
  {
    if (_Owner)
    {
      DoChain_ = IsChainable(true);
      endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);
    }

    // Modified to wrap chars around first buffer in window buffer object
    if (inwbox_)
    {
      if (BuffersCleared())
        RestoreUnusedBuffers();

      if ((sdp->_InWinMode || Debug_) && AllBoxFilled())
      {
        SaveBoxPos();
        IncrBoxChains(false);
        RestoreBoxPos();
      }

      AtMaxWrap_ = AtMaxWrap(WrapIndex_, WrapMax_, endpt_, LesserOrEqual_, true, true);

      if (AtMaxWrap_ && (DoChain_ || _EndCharInserted) &&
          (!_MovedToEnd || (!EndReached_ && AtMaxWrap(WrapIndex_, WrapMax_, endpt_, Lesser_, true, false))))
      {
        _WrapChars = true;
        _inwboxcnt++;
        _MovedToEnd =
        AtDexEnd_ = AtMaxWrap(WrapIndex_, WrapMax_, endpt_, GreaterOrEqual_, true, true);

        nextchar_ = _buffer[0];
        nextjoin_ = NOJOIN_CHAR;

        index_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
        MoveDone_ = Pull(0, &nextjoin_, &nextchain_, NULL);
        NullChar_ = nextchain_ == 0;
        pos = ((WindowLine*)_Owner)->GetWrapAroundIndex();

        if (index_ < pos)
          delta_ = pos - index_;
        else
          delta_ = 0;

        if (!_MovedToEnd || !EndReached_ || nextchain_)
        {
          if (MoveDone_ && nextchar_)
            Join(nextchar_);
        }
        else
        {
          index_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          Push(0, nextchar_, &nextjoin_, NULL);
          pos = ((WindowLine*)_Owner)->GetWrapAroundIndex();

          if (index_ > pos)
            delta_ -= (index_ - pos);

          MoveDone_ = false;
          MoveReversed_ = true;
        }

        if (AtDexEnd_ || MoveReversed_ ||
            (ChainedBufferEndReached() &&
             AtMaxWrap(WrapIndex_+1, WrapMax_, 0, GreaterOrEqual_, false, true)))
        {
          if (MoveDone_ && !MoveReversed_ &&
              ((WindowLine*)_Owner)->GetWrapAroundIndex() < _inwboxcnt)
            ((WindowLine*)_Owner)->IncrWrapAround(1);
          else if (MoveReversed_)
          {
            --_inwboxcnt;

            if (delta_ > 0)
              ((WindowLine*)_Owner)->DecrWrapAround(delta_);
            else if (delta_ < 0)
              ((WindowLine*)_Owner)->IncrWrapAround(-delta_);
          }

          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
          _WrapChars = false;
        }

        pos = wherex();
        ShowInput();
        GOTOXY(pos, _ypos);
        PosMoveDone_ = MoveDone_;

        if (_EndCharInserted &&
            NullChar_ && AtDexEnd_ && MoveDone_ && !MoveReversed_ &&
            AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, true) &&
            WrapIndex_ == _inwboxcnt)
        {
          _EndCharInserted = 0;
          _EndCharReverted = false;
        }

        AtEndPt_ = IsAtEndPt(&PastEndPt_, _refline);

        if (_refline)
          if (PastEndPt_ && AtDexEnd_ && _MovedToEnd)
            _refline->PutExtKey(key);
          else
            _refline->IncrBoxIndex();
      }
      else
      {
        if (CursorMoveCondMet(WrapIndex_, WrapMax_, endpt_, NonWsLen_))
        {
          GOTOXY(wherex()+1, _ypos);
          PosMoveDone_ = true;
          ++_index;
          iMonitor(_index_monitor, _index_handle, &_index);

          if (_refline)
            _refline->IncrBoxIndex();
        }
        
        AtEndPt_ = IsAtEndPt(&PastEndPt_, _refline);

        if (!PosMoveDone_ && (PastEndPt_ || CursorAtEOL(false)))
        {
          AtEndPt_ = _index == endpt_;
          _refline->PutExtKey(key);
          
          if (CursorAtEOL(true) && _index < endpt_)
          {
            GOTOXY(wherex()+1, _ypos);
            PosMoveDone_ = true;
            ++_index;
          }
          else
            PosMoveDone_ = false;
          
           pos = wherex();
          ShowInput();
          GOTOXY(pos, _ypos);

          if (!PosMoveDone_ && AtEndPt_ &&
              _refline->ExtendedIndexToggled())
          {
            _refline->ResetExtendedIndexToggle();
            PutExtKey(key);
          }
        }
      }
    }
    else if (vbuf_)
    {
      if (BuffersCleared())
        RestoreUnusedBuffers();

      AtMaxWrap_ = AtMaxWrap(WrapIndex_, WrapMax_, endpt_, LesserOrEqual_, true, true);
    
      if (AtMaxWrap_ && (DoChain_ || _EndCharInserted) &&
          (!_MovedToEnd || (!EndReached_ && AtMaxWrap(WrapIndex_, WrapMax_, endpt_, Lesser_, true, false))))
      {
        _inboxoffset++;
        _MovedToEnd =
        AtDexEnd_ = AtMaxWrap(WrapIndex_, WrapMax_, endpt_, GreaterOrEqual_, true, true);

        ((WindowLine*)_Owner)->IncrWrapAround(1);
        VirtualBufferOffset(&EndReached_);
        MoveDone_ = true;

        vmax_ = VirtualBufferMax();
        MoveReversed_ = _inboxoffset > vmax_;
        NullChar_ = MoveReversed_ ? true:(_buffer[_inboxoffset + _index] == 0);

        if (AtDexEnd_ || MoveReversed_ ||
            (EndReached_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, true)))
        {
          if (MoveReversed_)
          {
            --_inboxoffset;
            ((WindowLine*)_Owner)->DecrWrapAround(1);

            MoveDone_ = false;
            _MovedToEnd = true;
          }
        
          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
        }

        pos = wherex();
        ShowInput();
        GOTOXY(pos, _ypos);
        PosMoveDone_ = MoveDone_;
        iMonitor(_inboxoffset_monitor, _inboxoffset_handle, &_inboxoffset);

        if (_EndCharInserted &&
            NullChar_ && AtDexEnd_ && MoveDone_ && !MoveReversed_ &&
            AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, true) &&
            WrapIndex_ == _inboxoffset)
        {
          _EndCharInserted = 0;
          _EndCharReverted = false;
        }

        if (_refline)
          _refline->IncrBoxIndex();
      }
      else if (CursorMoveCondMet(WrapIndex_, WrapMax_, endpt_, NonWsLen_))
      {
        GOTOXY(wherex()+1, _ypos);
        PosMoveDone_ = true;
        ++_index;
        iMonitor(_index_monitor, _index_handle, &_index);

        if (_refline)
          _refline->IncrBoxIndex();
      }
    }
    else if (!vbuf_ && !inwbox_ &&
             _index < ::SafeStrLen(_buffer) && _index < endpt_)
    {
      GOTOXY(wherex()+1, _ypos);
      PosMoveDone_ = true;
      ++_index;
      iMonitor(_index_monitor, _index_handle, &_index);

      if (_refline)
        _refline->IncrBoxIndex();
    }
  }

  // Action: GO_HOME
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, HOME))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_HOME)
#endif  
  {
    if (BuffersCleared())
      RestoreUnusedBuffers();

    CondMet_ = (vbuf_ && ((_index == 0 && _inboxoffset > 0 && WrapIndex_ > 0) || _index > 0)) ||
               (inwbox_ && ((_index == 0 && _inwboxcnt > 0 && WrapIndex_ > 0) || _index > 0));

    if (vbuf_ && CondMet_)
    {
      KeyDone_ = PosMoveDone_ = CondMet_;
      
      for (index_ = 0; KeyDone_ && CondMet_; index_++)
      {
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::LEFTARROW).SCAN + 256);
        #elif (defined(__linux__) | defined(__unix__))
          KeyDone_ = PutExtKey(KEY_LEFT);
        #endif

        if (index_ == 0)
        {
          PosMoveDone_ = KeyDone_;
          
          if (!KeyDone_)
            CondMet_ = false;
        }

        if (KeyDone_)
        {
          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          CondMet_ = (_index == 0 && _inboxoffset > 0 && WrapIndex_ > 0) || _index > 0;
        }
      }
    }
    else if (inwbox_ && CondMet_)
    {
      KeyDone_ = PosMoveDone_ = CondMet_;

      for (index_ = 0; KeyDone_ && CondMet_; index_++)
      {
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::LEFTARROW).SCAN + 256);
        #elif (defined(__linux__) | defined(__unix__))
          KeyDone_ = PutExtKey(KEY_LEFT);
        #endif

        if (index_ == 0)
        {
          PosMoveDone_ = KeyDone_;

          if (!KeyDone_)
            CondMet_ = false;
        }

        if (KeyDone_)
        {
          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          CondMet_ = (_index == 0 && _inwboxcnt > 0 && WrapIndex_ > 0) || _index > 0;
        }
      }
    }

    if (!PosMoveDone_)
    {
      endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);

      if ((!vbuf_ && !inwbox_) || !CondMet_)
      {
        GOTOXY(_xpos + ::SafeStrLen(_msg) + EndBrksLength(true), _ypos);
        PosMoveDone_ = true;
        _index = 0;
        iMonitor(_index_monitor, _index_handle, &_index);

        if (vbuf_)
        {
          endpt_ += _inboxoffset;
          NextLen_ = EndReached_ ? 0:(VbufStrLenNoTerm() > _len);
          NullChar_ = _buffer[endpt_] == 0;
        }
        else if (inwbox_)
        {
          NextLen_ = EndReached_ ? 0:NextLength();
          NullChar_ = _buffer[endpt_] == 0;
        }

        if ((vbuf_ || inwbox_) && !NextLen_ && !NullChar_)
        {
          _MovedToEnd = false;
          _EndCharInserted = _buffer[endpt_];
          _EndCharReverted = true;
        }

        if (_refline)
        {
          _refline->ResetBoxIndex();
          _refline->PutExtKey(key);
          ResetBoxIndex();
        }
      }
    }
    else if (_refline)
    {
      _refline->ResetBoxIndex();
      _refline->PutExtKey(key);
      ResetBoxIndex();
    }
  }

  // Action: GO_END
  #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, END))
  #elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_END)
  #endif
  {
    if (BuffersCleared())
      RestoreUnusedBuffers();

    endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);

    if ((sdp->_InWinMode || Debug_) && inwbox_ && AllBoxFilled())
    {
      SaveBoxPos();
      IncrBoxChains(false);
      RestoreBoxPos();
    }    

    #if (WINLINE_ONLYCWSCHR & WINLINE_WSENDMOVE)
      if (inwbox_)
        NonWsLen_ = HasNonWsChars(_buffer);
      else if (vbuf_)
        VbufStrLen(&NonWsLen_);
    #else    
      if (inwbox_)
        NonWsLen_ =
          (_inwboxcnt > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false) &&
           _inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt) ?
               HasNonNullChars(_buffer):
               HasNonWsChars(_buffer);
      else if (vbuf_)
        VbufStrLen(&NonWsLen_);
    #endif

    CondMet_ = (vbuf_ || inwbox_) &&
                 ((AtMaxWrap(WrapIndex_, WrapMax_, endpt_, Lesser_, true, false) && !EndReached_) ||
                  CursorMoveCondMet(WrapIndex_, WrapMax_, endpt_, NonWsLen_));

    if (vbuf_ && CondMet_)
    {
      KeyDone_ = PosMoveDone_ = CondMet_;

      for (index_ = 0; KeyDone_ && CondMet_; index_++)
      {
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
        #elif (defined(__linux__) | defined(__unix__))
          KeyDone_ = PutExtKey(KEY_RIGHT);
        #endif

        if (index_ == 0)
        {
          PosMoveDone_ = KeyDone_;

          if (!KeyDone_)
            CondMet_ = false;
        }

        if (KeyDone_)
        {
          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
          VirtualBufferOffset(&EndReached_);
          VbufStrLen(&NonWsLen_);
          CondMet_ = ((AtMaxWrap(WrapIndex_, WrapMax_, endpt_, Lesser_, true, false) && !EndReached_) ||
                      CursorMoveCondMet(WrapIndex_, WrapMax_, endpt_, NonWsLen_));
        }
      }
    }
    else if (inwbox_ && CondMet_)
    {
      KeyDone_ = PosMoveDone_ = CondMet_;

      for (index_ = 0; KeyDone_ && CondMet_; index_++)
      {
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
        #elif (defined(__linux__) | defined(__unix__))
          KeyDone_ = PutExtKey(KEY_RIGHT);
        #endif

        if (index_ == 0)
        {
          PosMoveDone_ = KeyDone_;

          if (!KeyDone_)
            CondMet_ = false;
        }

        if (KeyDone_)
        {
          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
          EndReached_ = ChainedBufferEndReached();

          #if (WINLINE_ONLYCWSCHR & WINLINE_WSENDMOVE)
            NonWsLen_ = HasNonWsChars(_buffer);
          #else
            NonWsLen_ =
              (_inwboxcnt > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false) &&
               _inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt) ?
                   HasNonNullChars(_buffer):
                   HasNonWsChars(_buffer);
          #endif

          CondMet_ = ((AtMaxWrap(WrapIndex_, WrapMax_, endpt_, Lesser_, true, false) && !EndReached_) ||
                      CursorMoveCondMet(WrapIndex_, WrapMax_, endpt_, NonWsLen_));
        }
      }
    }

    if (!PosMoveDone_)
    {
      GotoDone_ = false;
      bufoffset_ = vbuf_   ? _inboxoffset:
                   inwbox_ ? _inwboxcnt:0;
      endpt_ = index_ = vbuf_ ? VbufStrLenNoTerm():SafeStrLen(_buffer);

      if (endpt_ >= _len && _len > 0)
      {
        GotoDone_ = true;
        if ((!vbuf_ && !inwbox_) || endpt_ > _len)
          endpt_ = index_ = _len;

        if ((!vbuf_ && !inwbox_) || !CondMet_)
        {
          GOTOXY(_xpos + ::SafeStrLen(_msg) + endpt_ + EndBrksLength(true) - 1, _ypos);
          PosMoveDone_ = true;
          _index = index_ - 1;
          iMonitor(_index_monitor, _index_handle, &_index);

          if (_refline)
          {
            _refline->SetBoxIndex(bufoffset_ + _index);
            _refline->PutExtKey(key);
          }
        }
      }

      if (!GotoDone_ && _len > 0)
        if ((!vbuf_ && !inwbox_) || !CondMet_)
        {
          GOTOXY(_xpos + ::SafeStrLen(_msg) + endpt_ + EndBrksLength(true), _ypos);
          PosMoveDone_ = true;
          _index = index_;
          iMonitor(_index_monitor, _index_handle, &_index);

          if (_refline)
          {
            _refline->SetBoxIndex(bufoffset_ + _index);
            _refline->PutExtKey(key);
          }
        }
    }
    else if (_refline)
    {
      bufoffset_ = vbuf_   ? _inboxoffset:
                   inwbox_ ? _inwboxcnt:0;

      _refline->SetBoxIndex(bufoffset_ + _index);
      _refline->PutExtKey(key);
    }
  }

  // Action: DELETE
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, DELETE))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_DC)
#endif
  {
    if (_index < _len && IsEnterable())
    {
      NullChar_ = false;
      EndMet_ = false;
    
      nextjoin_ = NOJOIN_CHAR;
      vmax_ = VirtualBufferMax() + _len;
      endptdex_ = 0;
      bufoffset_ = vbuf_   ? _inboxoffset:
                   inwbox_ ? _inwboxcnt:0;

      index_ = vbuf_ ? (bufoffset_ + _index):_index;
      endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);
      endptdex_ = vbuf_ ? (bufoffset_ + endpt_):endpt_;

      if (vbuf_)
      {
        _ChainPosPtr->SetupInfoPtrs(this, 0);
        _ChainPosPtr->SetPullInfo(_index);
      }

      AtEndPt_ = IsAtEndPt(&PastEndPt_);

      if (AtMaxWrap(WrapIndex_, WrapMax_, endpt_, GreaterOrEqual_, true, true) &&
          (PastEndPt_ || CursorAtEOL(false)) && sdp->_NextLineReady && inwbox_)
      {
        bufoffset_ += _index;
        SaveBoxPos();
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::HOME).SCAN + 256);
        #else
          PutExtKey(KEY_HOME);
        #endif
        
        ((WindowLine*)_Owner)->ShiftUp(NULL, bufoffset_, NULL, 0, 0);
        PosMoveDone_ = KeyDone_ = true;
        RestoreBoxPos();

        if (_nextline == NULL && sdp && _linestatus == UNINIT_NL)
        {
          if (sdp->_numnextlines < 0)
            sdp->_numnextlines = 0;
      
          if (::SafeStrLen(_buffer))
            _linestatus = FILLED_NL;
        }
      }
      else if (vbuf_ || inwbox_)
      {
        if (BuffersCleared())
          RestoreUnusedBuffers();

        if (vbuf_ && endptdex_ == vmax_)
        {
          EndMet_ = true;
          --index_;
          --endptdex_;
        }
        else if ((sdp->_InWinMode || Debug_) && inwbox_ && AllBoxFilled())
        {
          SaveBoxPos();
          IncrBoxChains(false);
          RestoreBoxPos();
        }
      
        PrevNull_ = _buffer[endptdex_] == 0;
        AtDexEnd_ = AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, true);
        KeyDone_ = Pull(_index, &nextjoin_, NULL, vbuf_ ? _ChainPosPtr:NULL) != 0;

        if (vbuf_ && AtDexEnd_ && EndMet_)
          bufoffset_ = --_inboxoffset;
        else if (inwbox_ && AtDexEnd_ && EndMet_)
          bufoffset_ = --_inwboxcnt;

        WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
        WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();

        pos = wherex();
        ShowInput();
        GOTOXY(pos, _ypos);
        PosMoveDone_ = KeyDone_;

        if (WrapIndex_ > bufoffset_)
        {
          ((WindowLine*)_Owner)->DecrWrapAround(WrapIndex_ - bufoffset_);
          if (_refline)
            _refline->SetBoxIndex(_refline->BufferIndex() - bufoffset_);

          WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
          WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
        }
        else if (_refline)
          _refline->DecrBoxIndex();

        NullChar_ = _buffer[endptdex_] == 0;
        NextLen_ = vbuf_ ? (EndReached_ ? 0:NextNonWsLen_):0;

        _DelKeyDone = true;
        _DelWrapOffset = bufoffset_ - WrapMax_;
        
        if (_DelWrapOffset < 0)
          _DelWrapOffset = 0;

        if (_EndCharInserted && AtDexEnd_ &&
            AtMaxWrap(WrapIndex_, WrapMax_, endpt_, GreaterOrEqual_, true, true) &&
            WrapIndex_ == bufoffset_ && !PrevNull_ && NullChar_)
        {
          _MovedToEnd = true;
          _EndCharInserted = 0;
          _EndCharReverted = false;
        }

        if ((!_EndCharInserted && !NextLen_ &&
             AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, false, false) &&
             PrevNull_ && !NullChar_) ||
            (vbuf_ && _inboxoffset == 0 && VbufStrLenNoTerm() <= _len) ||
            (inwbox_ && _inwboxcnt == 0 && NextLength() == 0))
        {
          _MovedToEnd = false;
          _EndCharInserted = _buffer[endptdex_];
          _EndCharReverted = true;
        }

        if (_nextline == NULL && sdp && _linestatus == UNINIT_NL)
        {
          if (sdp->_numnextlines < 0)
            sdp->_numnextlines = 0;

          if (::SafeStrLen(_buffer))
            _linestatus = FILLED_NL;
        }

        ClearDelWrapOffset(bufoffset_, WrapMax_);

        if (_refline)
          _refline->PutExtKey(key);
      }
      else if (!vbuf_ && !inwbox_)
      {
        KeyDone_ = Pull(_index, &nextjoin_, NULL, NULL) != 0;

        pos = wherex();
        ShowInput();
        GOTOXY(pos, _ypos);
        PosMoveDone_ = KeyDone_;

        if (_refline)
          _refline->PutExtKey(key);
      }
    }
  }

  if (PosMoveDone_ && RefLineMovePending())
    CommitRefLinePendingMove();

  SetMoveDone(PosMoveDone_);
  return PosMoveDone_;
}

/****************************************************************************/
// PURPOSE:
//   Method for putting standard ASCII keyboard keys to the console output.
//
// PRE:
//   int key : the key to be outputted.
//
// POST:
//   The key is outputted to the screen. Appropriate changes to the
//   internal input buffer is made.
//
bool WindowLineBuffer::PutStdKey(int key)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp || !_Owner)
    return false;

  char joinkey_ = 0;
  char* refshbuf_ = NULL;
  
  int pos;
  int len_;
  int vmax_ = 0;
  int index_ = 0;
  int endpt_ = 0;
  int oldkey_ = 0;
  int refshindex_ = 0;
  int refshendpt_ = 0;
  int EndReached_ = 0;
  int NextNonWsLen_ = 0;
  int NonWsLen_ = 0;
  int offset_ = 0;
  int WrapIndex_ = 0;
  int WrapMax_ = 0;
  int ExtIndex_ = 0;
  int BoxDiff_ = 0;
  int RefLineShift_ = 0;
  int MaxLines_ = sdp->_numnextlines;
  int LesserOrEqual_ = RelationalEnum::LESS_THAN_OR_EQUAL;
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;
  int Greater_ = RelationalEnum::GREATER_THAN;

  WindowLineBuffer* StartPtr_;
  bool NonNullFound_ = false;
  bool NextBufEmpty_ = false;
  bool KeyDone_ = false;
  bool AtEndPt_ = false;
  bool PastEndPt_ = false;
  bool PosMoveDone_ = false;
  bool vbuf_ = false;
  bool inwbox_ = false;
  bool curposatend_ = false;
  bool ShiftDownDone_ = false;
  bool AllLinesFilled_ = false;
  bool AddNewLinesWhenFull_ = false;
  bool AtLineChainLimit_ = false;
  bool RefLineMoved_ = false;
  bool HasPrevNulls_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_,
                                 &EndReached_, &NextNonWsLen_,
                                 &WrapIndex_, &WrapMax_);

  int bufoffset_ = vbuf_   ? _inboxoffset:
                   inwbox_ ? _inwboxcnt:0;
  bool DoChain_ = IsChainable(true);
  bool SavedTrimWs_;

  #if WINLINE_DEBUG3
    bool Debug_ = true;
  #else
    bool Debug_ = false;
  #endif

  sdp->_InWinMode = enterb_ ?
      ((WindowLine*)_Owner)->InWindowMode(&sdp->_NextLineReady,
                                          &sdp->_PrevLineReady):false;
  
  if (inwbox_)
  {
    SetRefLineMovePending(true, false);

    NonWsLen_ = NextNonWsLen_;
    AllLinesFilled_ = ((WindowLine*)_Owner)->AllLineFilled();
    AddNewLinesWhenFull_ = ((WindowLine*)_Owner)->AddNewLinesWhenFull();
    AtLineChainLimit_ = ((WindowLine*)_Owner)->AtLineChainLimit();
  }
  else
    VbufStrLen(&NonWsLen_);

  // Action: NEWLINE
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  if (_InsertMode && GetKeyMap().KeyMatch(key, ENTER))
#elif (defined(__linux__) | defined(__unix__))
  if (_InsertMode && key == 10)
#endif
  {
    if (IsEnterable())
    {
      // owner shift down to next window line instead of going to next
      // line when enter key is pressed.

      if (_Owner)
      {
        offset_ = _inwboxcnt + _index;

        if (inwbox_ && sdp->_InWinMode &&
            sdp->_NextLineReady && !AllLinesFilled_)
        {
          AtEndPt_ = IsAtEndPt(&PastEndPt_);

          #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
            PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::HOME).SCAN + 256);
          #elif (defined(__linux__) | defined(__unix__))
            PutExtKey(KEY_HOME);
          #endif

          if (_refline)
            _refline->PutStdKey(key);

          ShowInput();
          pos = WHEREX();
          GOTOXY(pos, _ypos);

          if (PastEndPt_ || CursorAtEOL(false))
            ShiftDownDone_ = ((WindowLine*)_Owner)->ShiftDown(_buffer, 0, NULL, MaxLines_, MaxLines_);
          else
            ShiftDownDone_ = ((WindowLine*)_Owner)->ShiftDown(_buffer, offset_, NULL, 0, 0);

          PosMoveDone_ = KeyDone_ = ShiftDownDone_;
          pos = _xpos;
          SavedTrimWs_ = _TrimWs;
          _TrimWs = true;

          ShowInput();
          GOTOXY(pos, _ypos);
          _TrimWs = SavedTrimWs_;
        }
        else if (inwbox_ && sdp->_InWinMode &&
                 AllLinesFilled_ && AddNewLinesWhenFull_ && !AtLineChainLimit_)
        {
          ((WindowLine*)_Owner)->AddNewWindowLines();
          return PutStdKey(key);
        }
        else if (enterb_)
        {
          ((WindowLine*)_Owner)->SendUnprocessedKey(key, _extended, this);
          PosMoveDone_ = KeyDone_ = true;
        }
      }
    }
  }

  // Action: GO_BACK
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, BACK) && IsEnterable())
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_BACKSPACE && IsEnterable())
#endif
  {
    if ((_index > 0) ||
        (vbuf_ && WrapIndex_ > 0 && _inboxoffset > 0) ||
        (inwbox_ && WrapIndex_ > 0 && _inwboxcnt > 0))
    {
      #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
        KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::LEFTARROW).SCAN + 256);
        AtEndPt_ = IsAtEndPt(&PastEndPt_);
        NonNullFound_ = false;

        vmax_ = VirtualBufferMax();
        if (_inboxoffset < vmax_)
          vmax_ = _inboxoffset;

        if (_ChainPosPtr)
          if (vbuf_)
            _ChainPosPtr->SetupInfoPtrs(this, 0);
          else if (inwbox_)
            _ChainPosPtr->SetupInfoPtrs(this, 2);

        RefLineShift_ = GetRefLineShift();
        refshbuf_ = vbuf_ ? &_buffer[vmax_]:_ChainPosPtr->GetRefshBuffer();

        refshindex_ = _ChainPosPtr->GetRefshIndexPos();
        refshendpt_ = (!vbuf_ && !inwbox_ && _nextline) ?
                          _len:_ChainPosPtr->GetRefshEndPt();

        for (pos = refshindex_;
             !NonNullFound_ && pos <= refshendpt_ && pos < _len; pos++)
          if (refshbuf_[pos])
            NonNullFound_ = true;
        
        if (KeyDone_ &&
            (!(PastEndPt_ && !NonNullFound_ && PeekNextChar() == 0)))
          PosMoveDone_ =
          KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::DELETE).SCAN + 256);
      #elif (defined(__linux__) | defined(__unix__))
        KeyDone_ = PutExtKey(KEY_LEFT);
        AtEndPt_ = IsAtEndPt(&PastEndPt_);
        NonNullFound_ = false;

        vmax_ = VirtualBufferMax();
        if (_inboxoffset < vmax_)
          vmax_ = _inboxoffset;

        if (_ChainPosPtr)
          if (vbuf_)
            _ChainPosPtr->SetupInfoPtrs(this, 0);
          else if (inwbox_)
            _ChainPosPtr->SetupInfoPtrs(this, 2);

        RefLineShift_ = _ChainPosPtr->GetRefLineShift(true);
        refshbuf_ = vbuf_ ? &_buffer[vmax_]:_ChainPosPtr->GetRefshBuffer();

        refshindex_ = _ChainPosPtr->GetRefshIndexPos();
        refshendpt_ = (!vbuf_ && !inwbox_ && _nextline) ?
                          _len:_ChainPosPtr->GetRefshEndPt();

        for (pos = refshindex_;
             !NonNullFound_ && pos <= refshendpt_ && pos < _len; pos++)
          if (refshbuf_[pos])
            NonNullFound_ = true;

        if (KeyDone_ &&
            (!(PastEndPt_ && !NonNullFound_ && PeekNextChar() == 0)))
          PosMoveDone_ =
          KeyDone_ = PutExtKey(KEY_DC);
      #endif
      
      iMonitor(_index_monitor, _index_handle, &_index);
    }
  }

  // Action: PUTKEY
  else if (IsUserKey(key))
  {
    if (BuffersCleared())
      RestoreUnusedBuffers();

    AtEndPt_ = IsAtEndPt(&PastEndPt_, _refline,
                         &RefLineMoved_, &ExtIndex_, &BoxDiff_);
    HasPrevNulls_ = _index && !_buffer[_index];
    
    if (HasPrevNulls_)
    {
      HasPrevNulls_ = false;
      for (pos = _index - 1; pos >= 0 && !HasPrevNulls_; pos--)
        if (!_buffer[pos])
          HasPrevNulls_ = true;
    }
    
    if (PastEndPt_ && (ExtIndex_ || BoxDiff_) &&
        (sdp->_InWinMode || Debug_) && inwbox_ &&
        (HasPrevNulls_ || AllBoxFilled()))
    {
      SaveBoxPos();
      StartPtr_ = IncrBoxChains(true);
      RestoreBoxPos();
      
      PosMoveDone_ = PutStdKey(key);
    }
    else if (_index < _len && IsEnterable())
    {
      bool NullEnd_ = false;
      bool NullChar_ = false;
      bool EntryOk_ = false;
      bool EndMet_ = false;

      int vmax_ = VirtualBufferMax() + _len;
      int savedindex_ = 0;
      int endptdex_ = 0;
      int Offset_ = vbuf_ ? _inboxoffset:
                    inwbox_ ? _inwboxcnt:0;

      #if (WINLINE_ONLYCWSCHR & WINLINE_WSENDMOVE)
        if (inwbox_)
          NonWsLen_ = HasNonWsChars(_buffer);
        else if (vbuf_)
          VbufStrLen(&NonWsLen_);
      #else
        if (inwbox_)
          NonWsLen_ =
            (Offset_ > 0 &&
             AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false) &&
             Offset_ >= WrapMax_ && WrapIndex_ <= Offset_) ?
                 HasNonNullChars(_buffer):
                 HasNonWsChars(_buffer);
        else if (vbuf_)
          VbufStrLen(&NonWsLen_);
      #endif

      index_ = vbuf_ ? (Offset_ + _index):_index;
      endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);
      endptdex_ = vbuf_ ? (Offset_ + endpt_):endpt_;
      NullEnd_ = !_buffer[endptdex_];

      if (!_buffer[index_])
      {
        NullChar_ = true;
        _buffer[index_+1] = '\0';

        if (_joinbufalloc)
        {
          if (index_+1 >= _maxbuf)
            _joinbuf[_maxbuf] = '\0';
          else
            _joinbuf[index_+1] = NOJOIN_CHAR;
        }
      }

      // replace ' ' with 0xAE, '\t' with 0xAF, '\r' with 0xB0
      // and '\n' with 0xB1: for vbuf and inwbox mode input boxes
      RefLineShift_ = GetRefLineShift();  // new
      oldkey_ = key;
      
      if (vbuf_ || inwbox_)
        key = Convert(key);

      if (AtMaxWrap(WrapIndex_, WrapMax_, endpt_, GreaterOrEqual_, true, false) &&
          (!vbuf_ || index_ < vmax_) && _index <= NonWsLen_ && NullChar_ &&
          Offset_ >= WrapMax_ && WrapIndex_ == Offset_)
      {
        _buffer[index_] = key;
        EntryOk_ = KeyDone_ = true;
        
        if (_joinbufalloc)
        {
          if (index_ >= _maxbuf)
            _joinbuf[_maxbuf] = '\0';
          else
            _joinbuf[index_] = NOJOIN_CHAR;
        }

        _MovedToEnd = false;
        _EndCharInserted = key;
        _EndCharReverted = true;

        // BackFill null spaces in buffer with 0xAE instead
        savedindex_ = index_;
        while (index_ > 0 && !_buffer[index_-1])
          _buffer[--index_] = SPACE_CHAR;

        if (_joinbufalloc && index_ != savedindex_)
        {
          endptdex_ = index_;
          for (index_ = savedindex_; index_ > 0 && index_ > endptdex_; index_--)
            _joinbuf[index_-1] = NOJOIN_CHAR;
        }

        if (_refline)
          _refline->PutStdKey(key);
      }
      else if (!vbuf_ || index_ < vmax_)
      {
        if (_InsertMode)
        {
          if (vbuf_)
          {
            _ChainPosPtr->SetupInfoPtrs(this, 0);
            _ChainPosPtr->SetPushInfo(_index);
          }

          if (!_EndCharReverted)
            _EndCharInserted = 0;

          joinkey_ = NOJOIN_CHAR;
          EntryOk_ = KeyDone_ = Push(_index, key, &joinkey_, vbuf_ ? _ChainPosPtr:NULL);

          if (AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false) &&
              _buffer[endptdex_] && NullEnd_ && Offset_ >= WrapMax_ &&
              WrapIndex_ == Offset_)
          {
            _MovedToEnd = false;
            _EndCharInserted = _buffer[endptdex_];
            _EndCharReverted = true;
          }

          if (_refline)
            _refline->PutStdKey(key);
        }
        else
        {
          _buffer[index_] = key;
          EntryOk_ = KeyDone_ = true;

          if (_joinbufalloc)
          {
            if (index_ >= _maxbuf)
              _joinbuf[_maxbuf] = '\0';
            else
              _joinbuf[index_] = NOJOIN_CHAR;
          }

          if (AtMaxWrap(WrapIndex_, WrapMax_, endpt_, GreaterOrEqual_, true, false) &&
              _index <= NonWsLen_ && Offset_ >= WrapMax_ &&
              WrapIndex_ == Offset_)
          {
            _MovedToEnd = false;
            _EndCharInserted = key;

            if (key && NullChar_)
              _EndCharReverted = true;
          }

          // BackFill null spaces in buffer with 0xAE instead
          savedindex_ = index_;
          while (index_ > 0 && !_buffer[index_-1])
            _buffer[--index_] = SPACE_CHAR;

          if (_joinbufalloc && index_ != savedindex_)
          {
            endptdex_ = index_;
            for (index_ = savedindex_; index_ > 0 && index_ > endptdex_; index_--)
              _joinbuf[index_-1] = NOJOIN_CHAR;
          }

          if (_refline)
            _refline->PutStdKey(key);
        }
      }

      if (EntryOk_)
      {
        PosMoveDone_ = true;

        if (_nextline == NULL && sdp && _linestatus == UNINIT_NL)
        {
          if (sdp->_numnextlines < 0)
            sdp->_numnextlines = 0;
      
          if (::SafeStrLen(_buffer))
            _linestatus = FILLED_NL;
        }

        PUTCH(oldkey_);
        pos = wherex();
        ShowInput();

        if (_index + 1 <= endpt_)
        {
          GOTOXY(pos, _ypos);
          ++_index;
        }
        else
        {
          if (pos > 1)
            --pos;

          GOTOXY(pos, _ypos);
          EndMet_ = true;
        }

        if ((sdp->_InWinMode || Debug_) && inwbox_ && AllBoxFilled())
        {
          SaveBoxPos();
          IncrBoxChains(false);
          RestoreBoxPos();
        }
      }

      iMonitor(_index_monitor, _index_handle, &_index);
      if (_refline)
        _refline->SetBoxIndex(bufoffset_ + _index);

      if (vbuf_ || inwbox_)
        if (EndMet_)        
        {
          NextBufEmpty_ = !_nextline || !_nextline->_buffer ||
                          _nextline->IsEmpty();

          if (_DelKeyDone && _DelWrapOffset > 0)
          {
            if (!NextBufEmpty_)
              SaveBoxPos();
          
            #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
              KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::HOME).SCAN + 256);
              
              if (KeyDone_)
                if (NextBufEmpty_)
                  KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::END).SCAN + 256);
                else
                  RestoreBoxPos();
            #elif (defined(__linux__) | defined(__unix__))
              KeyDone_ = PutExtKey(KEY_HOME);
              
              if (KeyDone_)
                if (NextBufEmpty_)
                  KeyDone_ = PutExtKey(KEY_END);
                else
                  RestoreBoxPos();
            #endif
          }

          #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
            if (KeyDone_)
              KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
          #elif (defined(__linux__) | defined(__unix__))
            if (KeyDone_)
              KeyDone_ = PutExtKey(KEY_RIGHT);
          #endif
          iMonitor(_index_monitor, _index_handle, &_index);
        }
        else
          ClearDelWrapOffset(Offset_, WrapMax_);
    }
  }

  if (PosMoveDone_ && RefLineMovePending())
    CommitRefLinePendingMove();

  SetMoveDone(PosMoveDone_);
  return PosMoveDone_;
}

/****************************************************************************/
void WindowLineBuffer::CommitRefLinePendingMove()
{
  if (_refline)
    _refline->CommitPendingMove();
}

/****************************************************************************/
void WindowLineBuffer::SetRefLineMovePending(bool pending_, bool done_)
{
  if (_refline)
    _refline->SetMovePending(pending_, done_);
}

/****************************************************************************/
bool WindowLineBuffer::RefLineMovePending() const
{
  return (_refline ? _refline->MovePending():false);
}

/****************************************************************************/
void WindowLineBuffer::SetRefLineMoveDone(bool move_)
{
  if (_refline)
    _refline->SetMoveDone(move_);
}

/****************************************************************************/
bool WindowLineBuffer::RefLineMoveDone() const
{
  return (_refline ? _refline->MoveDone():false);
}

/****************************************************************************/
void WindowLineBuffer::SetMoveDone(bool move_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  
  if (sdp)
    sdp->_movedone = move_;
}

/****************************************************************************/
bool WindowLineBuffer::MoveDone() const
{
  const WindowLineBuffer::SharedChainData* sdp = ConstSharedData();
  return (sdp ? sdp->_movedone:false);
}

/****************************************************************************/
int WindowLineBuffer::IsUserKey(int key)
{
  return
  (
    !_extended &&
    #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
      !GetKeyMap().KeyMatch(key, BACK) &&
      !GetKeyMap().KeyMatch(key, ESCAPE) &&
      !GetKeyMap().KeyMatch(key, ENTER) &&
      !GetKeyMap().KeyMatch(key, NEXT) &&
    #elif (defined(__linux__) | defined(__unix__))
      key != KEY_BACKSPACE &&
      key != 27 &&
      key != 10 &&
      key != 9 &&
    #endif
    isprint(key)
  );
}

/****************************************************************************/
void WindowLineBuffer::SetChainPos(int Pos_)
{
  _ChainPos = Pos_;

  if (_nextline)
    _nextline->SetChainPos(ChainPosition() + 1);
}

/****************************************************************************/
void WindowLineBuffer::EraseSharedChainData(WindowLineBuffer::SharedChainData* ptr)
{
  if (ptr && ptr->_parent == this)
    delete _SharedData;

  _SharedDataErased = true;
  _SharedData = NULL;
  
  if (_nextline)
    _nextline->EraseSharedChainData(NULL);
}

/****************************************************************************/
void WindowLineBuffer::ChainRemoveRetBuf(char* RetBufPtr_)
{
  if (_nextline)
  {
    if (_nextline->_retbuf == RetBufPtr_)
    {
      _nextline->_retbuf = NULL;
      _nextline->_retbufsize = 0;
      _nextline->_retbufalloc = false;
    }

    _nextline->ChainRemoveRetBuf(RetBufPtr_);
  }
}

/****************************************************************************/
bool WindowLineBuffer::RemoveBoxChains(WindowLineBuffer* wlptr)
{
  // new
  bool Removed_ = false;
  WindowLineBuffer* retptr = NULL;
  WindowLineBuffer* thisptr = NULL;

  if (wlptr)
  {
    #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
      PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::HOME).SCAN + 256);
    #else
      PutExtKey(KEY_HOME);
    #endif
  
    retptr = DetachNext(wlptr, false);
    Removed_ = retptr != NULL;

    if (Removed_)
      while (retptr)
      {
        thisptr = retptr;
        retptr = retptr->_nextline;
        thisptr->_nextline = NULL;
        delete thisptr;
      }
  }

  return Removed_;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::LastChainBox()
{
  // new
  WindowLineBuffer* retptr = this;
  while (retptr->_nextline)
    retptr = retptr->_nextline;

  return retptr;
}

/****************************************************************************/
char WindowLineBuffer::ConfirmBufferChains(WindowLineBuffer* root, int ChainNum_)
{
  char lastchar_ = 0;

  if (!root && BufferChainPosition() == 0)
    root = this;

  if (_nextline)
  {
    lastchar_ = _nextline->ConfirmBufferChains(root, ChainNum_+1);
    
    if (!lastchar_)
      lastchar_ = SafeStrLen(_buffer) ? _buffer[strlen(_buffer) - 1]:0;
  }
  else
    lastchar_ = SafeStrLen(_buffer) ? _buffer[strlen(_buffer) - 1]:0;

  int size_ = 0;
  int WrapIndex_;
  int WrapMax_;
  
  WindowLineBuffer::SharedChainData* sdp = NULL;

  if (ChainNum_ == 0 && (!root || BufferChainPosition() == 0))
  {
    SetBufferChainNum(this, 0);
    
    WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
    WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();

    if (WrapIndex_ > _inwboxcnt && _Owner)
    {
      ((WindowLine*)_Owner)->DecrWrapAround(WrapIndex_ - _inwboxcnt);
      
      if (_refline)
        _refline->SetBoxIndex(_refline->BufferIndex() - _inwboxcnt);

      WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
      WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
    }

    _DelWrapOffset = _inwboxcnt - WrapMax_;
    _DelKeyDone = _DelWrapOffset > 0;
        
    if (_DelWrapOffset < 0)
      _DelWrapOffset = 0;

    ClearDelWrapOffset(_inwboxcnt, WrapMax_);
    _EndCharInserted = lastchar_;
    _EndCharReverted = true;
    
    return lastchar_;
  }

  _maxbuf = root->_maxbuf ? root->_maxbuf:MAXBUFFER;
  _len = root->_len ? root->_len:1;
  _retbufsize = root->_retbufsize ? root->_retbufsize:
                root->_len ? root->_len:20;

  _Owner = root->_Owner;
  _Enterable = root->_Enterable;
  _Selectable = root->_Selectable;
  _bufferscleared = root->_bufferscleared;

  delete _refline;
  _refline = NULL;
  delete _msg;
  _msg = NULL;

  _SharedData = root->_SharedData;
  _SharedDataErased = _SharedData == NULL;

  _minlen = root->_minlen ? root->_minlen:1;
  _RestFullBox = root->_RestFullBox;
  _EndBrksActive = root->_EndBrksActive;
  _TrimWs = root->_TrimWs;

  if (!(BuffersCleared() && BufferTypeCleared(INPUT_BUFTYPE)) && !_buffer)
  {
    size_ = _maxbuf;
    
    if (size_)
    {
      _buffer = NewCString(size_, 0, true);
      _bufferalloc = _buffer != NULL;
    }

    SetBufferTypeCleared(INPUT_BUFTYPE, false);
  }

  if (!(BuffersCleared() && BufferTypeCleared(RETURN_BUFTYPE)) && !_retbuf)
  {
    size_ = _retbufsize;

    if (size_)
    {
      if (root->_retbuf && root->_retbufalloc)
        root->SetReturnBuffer(this, root->_retbuf, false, root->_retbufsize);
      else
        CreateReturnBuffer(root->_retbufsize);
        
      _retbufalloc = _retbuf != NULL;
    }

    SetBufferTypeCleared(RETURN_BUFTYPE, false);
  }

  if (!(BuffersCleared() && BufferTypeCleared(JOIN_BUFTYPE)) && !_joinbuf)
  {
    size_ = _maxbuf;
    
    if (size_)
    {
      _joinbuf = NewCString(size_, NOJOIN_CHAR, true);
      _joinbufalloc = _joinbuf != NULL;
    }
    
    SetBufferTypeCleared(JOIN_BUFTYPE, false);
  }

  sdp = SharedData();

  if (sdp && _linestatus == UNINIT_NL)
  {
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;
  
    if (::SafeStrLen(root->_buffer))
      _linestatus = FILLED_NL;
  }

  _bufferalloc = _bufferalloc && _buffer != NULL;
  _retbufalloc = _retbufalloc && _retbuf != NULL;
  _joinbufalloc = _joinbufalloc && _joinbuf != NULL;

  _brackets[0] = CONTMORE_LEFT;    // Continue more to the left (left triangle)
  _brackets[1] = CONTMORE_RIGHT;   // Continue more to the right (right triangle)

  SetInputLength(_len, false);
  SetBufferChainNum(this, ChainNum_);
  SetNoEmpty(root->_noempty);
  SetInsertMode(GetInsertMode(), sdp->_parent);

  return lastchar_;
}

/****************************************************************************/
void WindowLineBuffer::ReCalcChainWbuffer(bool ReCalcSdp_, WindowLine* wlptr)
{
  WindowLine::SharedChainData* wlsdp = _Owner ? ((WindowLine*)_Owner)->SharedData():NULL;
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  
  if (!sdp || !wlsdp)
    return;

  // new
  WindowLineBuffer* cur = this;
  WindowLineBuffer* wlbptr = sdp->_parent;
  int size_ = 0;
  int len_ = 0;
  int index;

  if (!_buffer)
  {
    if (BuffersCleared())
      RestoreUnusedBuffers();
  
    size_ = _maxbuf ? _maxbuf:MAXBUFFER;
    
    if (size_)
    {
      _buffer = NewCString(size_, 0, true);
      _bufferalloc = _buffer != NULL;
    }

    if (BufferTypeCleared(INPUT_BUFTYPE) && _bufferalloc)
      SetBufferTypeCleared(INPUT_BUFTYPE, false);
  }
                             
  if (!_retbuf)
  {
    if (wlbptr->_retbuf && wlbptr->_retbufalloc)
    {
      if (_bufferoffset > 0 || _BufChainNum > 0)
        wlbptr->SetReturnBuffer(this, wlbptr->_retbuf, false, wlbptr->_retbufsize);
      else
        wlbptr->SetReturnBuffer(this, wlbptr->_retbuf, true, (_len > 20) ? _len:20);
    }
    else
      CreateReturnBuffer(wlbptr->_retbufsize);

    if (BufferTypeCleared(RETURN_BUFTYPE) && _retbufalloc)
      SetBufferTypeCleared(RETURN_BUFTYPE, false);
  }

  if (!_joinbuf)
  {
    size_ = _maxbuf ? _maxbuf:MAXBUFFER;
    
    if (size_)
    {
      _joinbuf = NewCString(size_, NOJOIN_CHAR, true);
      _joinbufalloc = _joinbuf != NULL;
    }
    
    if (BufferTypeCleared(JOIN_BUFTYPE) && _joinbufalloc)
      SetBufferTypeCleared(JOIN_BUFTYPE, false);
  }

  if (!_ChainPosPtr)
    _ChainPosPtr = new ChainPosInfo;

  if (!_bufferoffset)
  {
    _bufferoffset = 0;
    
    if (!_maxbuf)
      _maxbuf = wlbptr->_maxbuf ? wlbptr->_maxbuf:MAXBUFFER;

    if (!_len)
      _len = wlbptr->_len ? wlbptr->_len:1;

    if (!_retbufsize)
      _retbufsize = wlbptr->_len ? wlbptr->_len:20;

    if (!_buffer)
    {
      _buffer = NewCString(_maxbuf, 0, true);
      _bufferalloc = _buffer != NULL;

      if (BufferTypeCleared(INPUT_BUFTYPE) && _bufferalloc)
        SetBufferTypeCleared(INPUT_BUFTYPE, false);
    }

    if (!_retbuf)
    {
      _retbufsize = wlbptr->_len ? wlbptr->_len:20;
      _retbuf = NewCString(_retbufsize, 0, true);
      _retbufalloc = _retbuf != NULL;

      if (BufferTypeCleared(RETURN_BUFTYPE) && _retbufalloc)
        SetBufferTypeCleared(RETURN_BUFTYPE, false);
    }

    if (!_joinbuf)
    {
      _joinbuf = NewCString(_maxbuf, NOJOIN_CHAR, true);
      _joinbufalloc = _joinbuf != NULL;

      if (BufferTypeCleared(JOIN_BUFTYPE) && _joinbufalloc)
        SetBufferTypeCleared(JOIN_BUFTYPE, false);
    }

    if (wlbptr->_tranbufalloc && wlbptr->_tranbuf)
      CreateExtendedAsciiBuffer();
  }
  else
  {
    if (!_maxbuf)
      _maxbuf = (wlbptr->_maxbuf > _bufferoffset) ?
                         (wlbptr->_maxbuf - _bufferoffset):MAXBUFFER;

    if (!_len)
      _len = (wlbptr->_maxbuf > _bufferoffset) ?
                         (wlbptr->_maxbuf - _bufferoffset):1;

    if (!_retbufsize)
      _retbufsize = wlbptr->_retbufsize;

    if (!_buffer)
      _buffer = (wlbptr->_maxbuf > _bufferoffset) ?
                         (wlbptr->_buffer + _bufferoffset):NULL;

    if (!_retbuf)
      _retbuf = wlbptr->_retbuf;
      
    _joinbuf = NULL;

    if (!_tranbuf)
      _tranbuf = (wlbptr->_maxbuf > _bufferoffset && wlbptr->_tranbuf) ?
                         (wlbptr->_tranbuf + _bufferoffset):NULL;
  }

  _inboxoffset = 0;
  _inwboxcnt = 0;
  
  _index = 0;
  _WrapChars = false;

  if (wlptr)
    _Owner = wlptr;

  if (sdp && ReCalcSdp_)
  {
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;
  
    if (::SafeStrLen(_buffer))
    {
      if (_linestatus != INIT_NL)
        _linestatus = FILLED_NL;
    }
    else
      _linestatus = UNINIT_NL;

    for (index = 0; cur && cur->_nextline; index++)
      cur = cur->_nextline;

    sdp->_numnextlines = index;

    if (wlptr)
      sdp->SetOwner(wlptr);
  }

  _bufferalloc = _bufferalloc && _buffer != NULL;
  _retbufalloc = _retbufalloc && _retbuf != NULL;
  _joinbufalloc = _joinbufalloc && _joinbuf != NULL;

  if (_Owner != wlsdp->_currentrow || _BufChainNum)
  {
    _Escaped = sdp->_parent->_Escaped;
    _Hidden = TRUE;
    _InFocus = FALSE;
    _TextWidgetActive = sdp->_parent->_TextWidgetActive;
  }

  if (_bufferalloc)
    for (index = ::SafeStrLen(_buffer); index <= _maxbuf; index++)
      _buffer[index] = '\0';

  if (_joinbufalloc)
    for (index = ::SafeStrLen(_buffer); index < _maxbuf; index++)
      _joinbuf[index] = NOJOIN_CHAR;

  if (_joinbufalloc && index == _maxbuf)
    _joinbuf[index] = '\0';  
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::SetupChainObject(WindowLineBuffer* ptr, ControlWindow* Owner_)
{
  // new
  WindowLine::SharedChainData* wlsdp = Owner_ ? ((WindowLine*)Owner_)->SharedData():NULL;
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  
  if (!sdp || !wlsdp)
    return false;

  if (!ptr)
    return ptr;

  WindowLineBuffer* cur = ptr;
  WindowLineBuffer* wlbptr = sdp->_parent;
  int len_ = 0;
  int size_ = 0;
  int index;

  ptr->_Owner = _Owner;
  delete ptr->_refline;
  ptr->_refline = NULL;
  ptr->_Enterable = _Enterable;
  ptr->_Selectable = _Selectable;
  delete ptr->_msg;
  ptr->_msg = NULL;

  if (ptr && !ptr->_buffer)
  {
    if (ptr->BuffersCleared())
      ptr->RestoreUnusedBuffers();
  
    size_ = _maxbuf ? _maxbuf:MAXBUFFER;

    if (size_)
    {
      ptr->_buffer = NewCString(size_, 0, true);
      ptr->_bufferalloc = ptr->_buffer != NULL;
    }

    if (ptr->BufferTypeCleared(INPUT_BUFTYPE) && ptr->_bufferalloc)
      ptr->SetBufferTypeCleared(INPUT_BUFTYPE, false);
  }
                             
  if (!ptr->_retbuf)
  {
    if (wlbptr->_retbuf && wlbptr->_retbufalloc)
    {
      if (ptr->_bufferoffset > 0 || ptr->_BufChainNum > 0)
        wlbptr->SetReturnBuffer(ptr, wlbptr->_retbuf, false, wlbptr->_retbufsize);
      else
        wlbptr->SetReturnBuffer(ptr, wlbptr->_retbuf, true, (_len > 20) ? _len:20);
    }
    else
      ptr->CreateReturnBuffer(wlbptr->_retbufsize);

    if (ptr->BufferTypeCleared(RETURN_BUFTYPE) && ptr->_retbufalloc)
      ptr->SetBufferTypeCleared(RETURN_BUFTYPE, false);
  }

  if (!ptr->_joinbuf)
  {
    size_ = _maxbuf ? _maxbuf:MAXBUFFER;

    if (size_)
    {
      ptr->_joinbuf = NewCString(size_, NOJOIN_CHAR, true);
      ptr->_joinbufalloc = ptr->_joinbuf != NULL;
    }

    if (ptr->BufferTypeCleared(JOIN_BUFTYPE) && ptr->_joinbufalloc)
      ptr->SetBufferTypeCleared(JOIN_BUFTYPE, false);
  }

  if (!ptr->_ChainPosPtr)
    ptr->_ChainPosPtr = new ChainPosInfo;

  if (!ptr->_bufferoffset)
  {
    ptr->_bufferoffset = 0;

    if (!ptr->_maxbuf)
      ptr->_maxbuf = _maxbuf ? _maxbuf:MAXBUFFER;

    if (!ptr->_len)
      ptr->_len = _len ? _len:1;

    if (!ptr->_retbufsize)
      ptr->_retbufsize = _len ? _len:20;

    if (!ptr->_buffer)
    {
      ptr->_buffer = NewCString(ptr->_maxbuf, 0, true);
      ptr->_bufferalloc = ptr->_buffer != NULL;

      if (ptr->BufferTypeCleared(INPUT_BUFTYPE) && ptr->_bufferalloc)
        ptr->SetBufferTypeCleared(INPUT_BUFTYPE, false);
    }
    
    if (!ptr->_retbuf)    
    {
      ptr->_retbufsize = _len ? _len:20;
      ptr->_retbuf = NewCString(ptr->_retbufsize, 0, true);
      ptr->_retbufalloc = ptr->_retbuf != NULL;

      if (ptr->BufferTypeCleared(RETURN_BUFTYPE) && ptr->_retbufalloc)
        ptr->SetBufferTypeCleared(RETURN_BUFTYPE, false);
    }

    if (!ptr->_joinbuf)
    {
      ptr->_joinbuf = NewCString(ptr->_maxbuf, NOJOIN_CHAR, true);
      ptr->_joinbufalloc = ptr->_joinbuf != NULL;

      if (ptr->BufferTypeCleared(JOIN_BUFTYPE) && ptr->_joinbufalloc)
        ptr->SetBufferTypeCleared(JOIN_BUFTYPE, false);
    }

    if (_tranbufalloc && _tranbuf)
      CreateExtendedAsciiBuffer();
  }
  else
  {
    if (!ptr->_maxbuf)
      ptr->_maxbuf = (_maxbuf > ptr->_bufferoffset) ?
                         (_maxbuf - ptr->_bufferoffset):MAXBUFFER;

    if (!ptr->_len)
      ptr->_len = (_maxbuf > ptr->_bufferoffset) ?
                         (_maxbuf - ptr->_bufferoffset):1;

    if (!ptr->_retbufsize)
      ptr->_retbufsize = _retbufsize;

    if (!ptr->_buffer)
      ptr->_buffer = (_maxbuf > ptr->_bufferoffset) ?
                         (_buffer + ptr->_bufferoffset):NULL;

    if (!ptr->_retbuf)
      ptr->_retbuf = _retbuf;
      
    ptr->_joinbuf = NULL;

    if (!ptr->_tranbuf)
      ptr->_tranbuf = (_maxbuf > ptr->_bufferoffset && _tranbuf) ?
                         (_tranbuf + ptr->_bufferoffset):NULL;
  }

  ptr->_inboxoffset = 0;
  ptr->_inwboxcnt = 0;

  ptr->_SharedDataErased = false;
  ptr->_SharedData = _SharedData;
  
  ptr->_index = 0;  
  ptr->_WrapChars = false;

  ptr->_RestFullBox = _RestFullBox;
  ptr->_EndBrksActive = _EndBrksActive;
  ptr->_TrimWs = _TrimWs;

  if (sdp)
  {
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;
  
    if (::SafeStrLen(ptr->_buffer))
    {
      if (ptr->_linestatus != INIT_NL)
        ptr->_linestatus = FILLED_NL;
    }
    else
      ptr->_linestatus = UNINIT_NL;

    for (index = 0; cur && cur->_nextline; index++)
      cur = cur->_nextline;

    sdp->_numnextlines = index;
  }

  ptr->_bufferalloc = ptr->_bufferalloc && ptr->_buffer != NULL;
  ptr->_retbufalloc = ptr->_retbufalloc && ptr->_retbuf != NULL;
  ptr->_joinbufalloc = ptr->_joinbufalloc && ptr->_joinbuf != NULL;

  ptr->_brackets[0] = CONTMORE_LEFT;    // Continue more to the left (left triangle)
  ptr->_brackets[1] = CONTMORE_RIGHT;   // Continue more to the right (right triangle)

  if (Owner_ != wlsdp->_currentrow || ptr->_BufChainNum)
  {
    ptr->_Escaped = sdp->_parent->_Escaped;
    ptr->_Hidden = TRUE;
    ptr->_InFocus = FALSE;
    ptr->_TextWidgetActive = sdp->_parent->_TextWidgetActive;
  }

  if (ptr->_bufferalloc)
    for (index = ::SafeStrLen(ptr->_buffer); index <= ptr->_maxbuf; index++)
      ptr->_buffer[index] = '\0';

  if (ptr->_joinbufalloc)
    for (index = ::SafeStrLen(ptr->_buffer); index < ptr->_maxbuf; index++)
      ptr->_joinbuf[index] = NOJOIN_CHAR;

  if (ptr->_joinbufalloc && index == ptr->_maxbuf)
    _joinbuf[index] = '\0';  
}

/****************************************************************************/
/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::Chain(WindowLineBuffer* next_)
{
  WindowLineBuffer* retptr = AddBoxChain(next_, true);
  return retptr;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::AddBoxChain(WindowLineBuffer* next_, bool ChainPWinBox_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  WindowLineBuffer* retptr = NULL;
  WindowLineBuffer* nextptr = NULL;

  if (_nextline)
  {
    retptr = _nextline->AddBoxChain(next_, ChainPWinBox_);

    if (_BufChainNum == 0 && retptr && retptr->_nextline)
    {
      if (_linestatus == UNINIT_NL || _linestatus == FILLED_NL)
        _linestatus = INIT_NL;

      if (sdp)
        if (sdp->_numnextlines <= 0)
          sdp->_numnextlines = 1;
        else
          sdp->_numnextlines++;

      retptr->_nextline->_SharedData = _SharedData;
      retptr->_nextline->_SharedDataErased = _SharedData == NULL;

      if (_retbuf && _retbufalloc)
        SetReturnBuffer(retptr->_nextline, _retbuf, false, _retbufsize);
      else
        retptr->_nextline->CreateReturnBuffer(_retbufsize);

      if (_Owner && retptr->_nextline)
      {
        nextptr = retptr->_nextline;
        nextptr->SetOwner(_Owner, nextptr);

        if (ChainPWinBox_)
          ((WindowLine*)_Owner)->ChainPbox(nextptr, nextptr->_BufChainNum);
      }
    }
  }
  else
  {    
    if (!_refline && _BufChainNum == 0)
    {
      _refline = WindowRefLineInfo::Make(0, this, _Owner, _len);
      retptr = _nextline ? NULL:this;
    }

    _nextline = next_;

    if (_nextline)
    {
      SetupChainObject(_nextline, _Owner);
      SetBufferChainNum(_nextline, _BufChainNum+1);

      if (_retbuf && _retbufalloc)
        SetReturnBuffer(_nextline, _retbuf, false, _retbufsize);

      _nextline->SetChainPos(ChainPosition() + 1);
      _nextline->SetNoEmpty(_noempty);
      _nextline->SetInsertMode(GetInsertMode(), sdp->_parent);

      _nextline->MonitorBuffer(_buffer_handle, _buffer_monitor);
      _nextline->MonitorKey(_key_handle, _key_monitor);
      _nextline->MonitorExtendedKey(_ext_handle, _ext_monitor);
      _nextline->MonitorIndex(_index_handle, _index_monitor);
      _nextline->MonitorEscapedStatus(_escaped_handle, _escaped_monitor);
      _nextline->MonitorInboxOffset(_inboxoffset_handle, _inboxoffset_monitor);
      _nextline->MonitorWinLineNum(_winlinenum_handle, _winlinenum_monitor);
      // _nextline->MonitorInFocusStatus(_infocus_handle, _infocus_monitor);

      if (_chainminlen)
        _nextline->SetMinimumLength(_chainminlen);

      if (_TextWidgetActive)
      {
        _nextline->Activate();

        if (_refline && _BufChainNum == 0)
          _refline->Activate();
      }
      else
        _nextline->Deactivate();
    }

    if (retptr && retptr->_nextline)
    {
      if (_linestatus == UNINIT_NL || _linestatus == FILLED_NL)
        _linestatus = INIT_NL;

      if (sdp)
        if (sdp->_numnextlines <= 0)
          sdp->_numnextlines = 1;
        else
          sdp->_numnextlines++;

      retptr->_nextline->_SharedData = _SharedData;

      if (_Owner && retptr->_nextline)
      {
        nextptr = retptr->_nextline;
        nextptr->SetOwner(_Owner, nextptr);

        if (ChainPWinBox_)
          ((WindowLine*)_Owner)->ChainPbox(nextptr, nextptr->_BufChainNum);
      }
    }
  }

  return (retptr ? retptr:this);
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::CopyEndLineCharData(WindowLine* owner_,
                                                        WindowLineBuffer* ptr, bool Init_)
{
  bool Done_ = false;

  int BoxInLen_ = owner_ ? owner_->BoxInputLength():0;
  int BoxCnt_ = 0;
  int bufoffset_;
  int WrapIndex_;
  int WrapMax_;

  WindowLineBuffer::SharedChainData* sdp = NULL;

  if (!ptr && Init_)
  {
    _MovedToEnd = false;
    _EndCharInserted = 0;
    _EndCharReverted = false;
    _DelKeyDone = false;
    _DelWrapOffset = 0;

    Done_ = true;
  }  
  else if (ptr && this != ptr)
  {
    _MovedToEnd = ptr->_MovedToEnd;
    _EndCharInserted = ptr->_EndCharInserted;
    _EndCharReverted = ptr->_EndCharReverted;
    _DelKeyDone = ptr->_DelKeyDone;
    _DelWrapOffset = ptr->_DelWrapOffset;

    Done_ = true;
  }

  if (Done_)
  {
    if (_DelKeyDone)
    {
      WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
      WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
      BoxCnt_ = (ptr && !Init_) ? ptr->_inwboxcnt:_inwboxcnt;
    
      if (WrapIndex_ > BoxCnt_)
      {
        ((WindowLine*)_Owner)->DecrWrapAround(WrapIndex_ - BoxCnt_);
        if (_refline)
          _refline->SetBoxIndex(_refline->BufferIndex() - BoxCnt_);

        WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();        
        WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
        BoxCnt_ = (ptr && !Init_) ? ptr->_inwboxcnt:_inwboxcnt;
      }

      _DelWrapOffset = BoxCnt_ - WrapMax_;
        
      if (_DelWrapOffset < 0)
        _DelWrapOffset = 0;

      ClearDelWrapOffset(BoxCnt_, WrapMax_);
    }

    if (!_maxbuf)
      _maxbuf = owner_ ? BoxInLen_:MAXBUFFER;

    if (!_maxbuf)
      _maxbuf = owner_ ? BoxInLen_:MAXBUFFER;

    if (!_len)
      _len = owner_ ? BoxInLen_:1;

    if (!_retbufsize)
      _retbufsize = owner_ ? BoxInLen_:20;

    sdp = SharedData();

    if (sdp && _linestatus == UNINIT_NL)
    {
      if (sdp->_numnextlines < 0)
        sdp->_numnextlines = 0;
  
      if (::SafeStrLen(_buffer))
        _linestatus = FILLED_NL;
    }

    _bufferalloc = _bufferalloc && _buffer != NULL;
    _retbufalloc = _retbufalloc && _retbuf != NULL;
    _joinbufalloc = _joinbufalloc && _joinbuf != NULL;
    _tranbufalloc = _tranbufalloc && _tranbuf != NULL;

    if (_brackets)
    {
      _brackets[0] = CONTMORE_LEFT;  // Continue more to the left (left triangle)
      _brackets[1] = CONTMORE_RIGHT; // Continue more to the right (right triangle)
      SetEndBrksActive(false);
    }

    if (_bufferalloc && _buffer)
      _buffer[_maxbuf] = '\0';

    if (_joinbufalloc && _joinbuf)
      _joinbuf[_maxbuf] = '\0';

    if (_refline)
      _refline->ResetBoxIndex();

    if (ptr && !Init_ &&
        sdp && sdp->_BoxPosSaved)
    {
      if (ptr->_refline)
        ptr->_refline->ResetBoxIndex();

      sdp->_SavedIndex = ptr->_index;
      sdp->_SavedExtIndex = ptr->_refline ? ptr->_refline->ExtendedIndex():0;
      sdp->_SavedBoxDiff = ptr->_refline ? ptr->_refline->BoxDiff():0;
      sdp->_SavedOffset = ptr->_inwboxcnt;
    }
  }

  return this;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::InitFirstBox(WindowLine* owner_)
{
  _Owner = owner_;
  _Enterable = TRUE;
  _Selectable = FALSE;

  delete _ChainPosPtr;
  _ChainPosPtr = new ChainPosInfo;
  int BoxInLen_ = owner_ ? owner_->BoxInputLength():0;
  
  if (!_maxbuf)
    _maxbuf = owner_ ? BoxInLen_:MAXBUFFER;

  if (!_buffer)  
  {  
    _buffer = NewCString(_maxbuf, 0, true);
    _bufferalloc = _buffer != NULL;

    if (BufferTypeCleared(INPUT_BUFTYPE) && _bufferalloc)
      SetBufferTypeCleared(INPUT_BUFTYPE, false);
  }

  if (_retbufalloc && _retbuf)
  {
    ChainRemoveRetBuf(_retbuf);
    delete[] _retbuf;
  }

  _retbuf = NULL;
  _retbufsize = owner_ ? BoxInLen_:20;

  _retbuf = NewCString(_retbufsize, 0, true);
  _retbufalloc = _retbuf != NULL;

  if (BufferTypeCleared(RETURN_BUFTYPE) && _retbufalloc)
    SetBufferTypeCleared(RETURN_BUFTYPE, false);

  if (!_joinbuf)
  {
    _joinbuf = NewCString(_maxbuf, NOJOIN_CHAR, true);
    _joinbufalloc = _joinbuf != NULL;

    if (BufferTypeCleared(JOIN_BUFTYPE) && _joinbufalloc)
      SetBufferTypeCleared(JOIN_BUFTYPE, false);
  }

  _bufferalloc = _buffer != NULL;
  _retbufalloc = _retbuf != NULL;
  _joinbufalloc = _joinbuf != NULL;
  _tranbufalloc = _tranbuf != NULL;

  _bufferscleared = 0;
  _bufferoffset = 0;
  _inboxoffset = 0;
  _inwboxcnt = 0;

  _linestatus = UNINIT_NL;
  _BufChainNum = 0;
  _WrapChars = false;

  if (!_maxbuf)
    _maxbuf = owner_ ? BoxInLen_:MAXBUFFER;

  if (!_len)
    _len = owner_ ? BoxInLen_:1;

  if (!_retbufsize)
    _retbufsize = owner_ ? BoxInLen_:20;

  if (_BufChainNum == 0 && _SharedDataErased)
  {
    _SharedDataErased = false;
    _SharedData = new WindowLineBuffer::SharedChainData;
  }

  if (!_refline && _BufChainNum == 0)
    _refline = WindowRefLineInfo::Make(0, this, _Owner, _len);
  
  _minlen = 1;
  _chainminlen = 0;
  _index = 0;
  // _xpos = WHEREX();
  // _ypos = WHEREY();
  _noempty = 0;
  _InsertMode = TRUE;
  _Escaped = FALSE;
  _Hidden = TRUE;
  _TextWidgetActive = FALSE;
  _ChainPos = 0;
  _InFocus = FALSE;

  if (_SharedData && !_SharedDataErased && _BufChainNum == 0)
    _SharedData->SetParent(this);

  WindowLineBuffer::SharedChainData* sdp = SharedData();

  if (sdp && _linestatus == UNINIT_NL)
  {
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;
  
    if (::SafeStrLen(_buffer))
      _linestatus = FILLED_NL;
  }

  _bufferalloc = _bufferalloc && _buffer != NULL;
  _retbufalloc = _retbufalloc && _retbuf != NULL;
  _joinbufalloc = _joinbufalloc && _joinbuf != NULL;
  _tranbufalloc = _tranbufalloc && _tranbuf != NULL;

  if (_brackets)
  {
    _brackets[0] = CONTMORE_LEFT;  // Continue more to the left (left triangle)
    _brackets[1] = CONTMORE_RIGHT; // Continue more to the right (right triangle)
    SetEndBrksActive(false);
  }
  
  if (_bufferalloc && _buffer)
    _buffer[_maxbuf] = '\0';

  if (_joinbufalloc && _joinbuf)
    _joinbuf[_maxbuf] = '\0';

  if (sdp && sdp->_BoxPosSaved)
  {
    if (_refline)
      _refline->ResetBoxIndex();

    sdp->_SavedIndex = _index;
    sdp->_SavedExtIndex = _refline ? _refline->ExtendedIndex():0;
    sdp->_SavedBoxDiff = _refline ? _refline->BoxDiff():0;
    sdp->_SavedOffset = _inwboxcnt;
  }

  ReCalcChainWbuffer(true, owner_);
  return this;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::IncrBoxChains(bool blankpad_)
{
  // new
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return NULL;
  
  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);
  int reflinelen_ = _refline ? _refline->ExtendedIndex():0;

  if (!inwbox_)
    return NULL;
  else
    sdp->_InWinMode = enterb_ ?
        ((WindowLine*)_Owner)->InWindowMode(&sdp->_NextLineReady,
                                            &sdp->_PrevLineReady):false;

  #if WINLINE_DEBUG3
    bool Debug_ = true;
  #else
    bool Debug_ = false;
  #endif
  
  #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
    PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::HOME).SCAN + 256);
  #else
    PutExtKey(KEY_HOME);
  #endif

  WindowLineBuffer* newbufp_ = NULL;
  char* Blanks_ = blankpad_ ? (new char[_len+1]):NULL;

  int x;
  int index_ = reflinelen_ % _len;
  int numfilled_ = reflinelen_ / _len;
  int newchains_ = numfilled_ + ((index_ > 0) ? 1:0);
  int numlines_ = sdp->_numnextlines;

  bool ChainAdded_ = false;
  bool boxpadded_ = false;
  bool AllFilled_ = AllBoxFilled();

  if ((sdp->_InWinMode || Debug_) && inwbox_ && AllFilled_)
  {
    ++newchains_;
    boxpadded_ = true;
  }

  Blanks_ = blankpad_ ? NewCString(_len, SPACE_CHAR, true):NULL;

  // cancel previous refline move done indicator since we're adding
  // new box chains up to the point of the cursor position as indicated
  // by the refline object
  if (newchains_)
  {
    SetRefLineMoveDone(false);

    // if blankpad indicated as argument then blank pad previous box chains
    // from last non-blank char until last box of chain
    if (blankpad_)
      CharPadToEnd(SPACE_CHAR);
    else
      CharPadToEnd(0);
  }

  if (AllFilled_)
  {
    for (x = 0; x < newchains_; x++)    
    {
      newbufp_ = WindowLineBuffer::MakeNextLine(this);
    
      if (newbufp_ && (Blanks_ || !blankpad_))
      {
        if (x < numfilled_ || x > numfilled_)
        {
          if (blankpad_ && sdp->_padcnt < sdp->_padmax)
            newbufp_->SetInputString(Blanks_);
          else
            newbufp_->CharPadToEnd(0);
        }
        else if ((index_ > 0 || boxpadded_) && x == numfilled_)
        {
          if (!boxpadded_)
          {
            ::memset(Blanks_, SPACE_CHAR, index_);
            Blanks_[index_] = 0;
          }

          if (blankpad_)
          {
            if (sdp->_padcnt < sdp->_padmax)
              newbufp_->SetInputString(Blanks_);
            else
              newbufp_->CharPadToEnd(0);

            if (!boxpadded_)
              blankpad_ = false;
          }
          else
            newbufp_->CharPadToEnd(0);
        }

        Chain(newbufp_);
      }
    }
  }

  if (Blanks_)
  {
    delete[] Blanks_;
    Blanks_ = NULL;
  }

  ChainAdded_ = newchains_ && numlines_ != sdp->_numnextlines;
  return (ChainAdded_ ? LastChainBox():NULL);
}

/****************************************************************************/
void WindowLineBuffer::SaveBoxPos()
{
  // new
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;

  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);  
  
  sdp->_BoxPosSaved = true;
  sdp->_SavedIndex = _index;
  sdp->_SavedExtIndex = _refline ? _refline->ExtendedIndex():0;
  sdp->_SavedBoxDiff = _refline ? _refline->BoxDiff():0;
  sdp->_SavedOffset = vbuf_   ? _inboxoffset:
                      inwbox_ ? _inwboxcnt:0;
}

/****************************************************************************/
void WindowLineBuffer::ReAlignBoxIndex()
{
  int curpos_ = wherex();
  int homepos_ = _xpos + ::SafeStrLen(_msg);

  if (EndBrksActive())
    homepos_ += EndBrksLength(true);

  if (curpos_ > homepos_)
    _index = curpos_ - homepos_;
  else
    _index = 0;
}

/****************************************************************************/
void WindowLineBuffer::RestoreBoxPos(WindowLineBuffer* wlptr)
{
  int EndReached_;
  int NextNonWsLen_;
  int WrapIndex_;
  int WrapMax_;
  int RefLineIndex_ = _refline ? _refline->BufferIndex():0;

  bool vbuf_;  
  bool inwbox_;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_,
                                 &EndReached_, &NextNonWsLen_,
                                 &WrapIndex_, &WrapMax_);

  int endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  if (!sdp)
    return;

  if (sdp->_BoxPosSaved && wlptr)
  {
    sdp->_BoxPosTrans = true;
    
    if (WrapMax_ < sdp->_SavedOffset)
    {
      sdp->_NextIndex = RefLineIndex_ + sdp->_SavedBoxDiff;
      sdp->_NextExtIndex = sdp->_SavedOffset - WrapMax_;
      sdp->_NextBoxDiff = sdp->_SavedBoxDiff;
      sdp->_NextOffset = WrapMax_;
    }
    else
    {
      sdp->_NextIndex = sdp->_SavedIndex;
      sdp->_NextExtIndex = sdp->_SavedExtIndex;
      sdp->_NextBoxDiff = sdp->_SavedBoxDiff;
      sdp->_NextOffset = sdp->_SavedOffset;
    }
  }

  RestoreBoxPos();
}

/****************************************************************************/
void WindowLineBuffer::RestoreBoxPos()
{
  // new
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;

  if (sdp->_BoxPosSaved)
  {
    bool KeyDone_;
    bool vbuf_ = false;
    bool inwbox_ = false;
    bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);

    int Offset_ = 0;
    int SavedIndex_;
    int SavedExtIndex_;
    int SavedBoxDiff_;
    int SavedOffset_;
    int endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);    

    if (sdp->_BoxPosTrans)
    {
      SavedIndex_ = sdp->_NextIndex;
      SavedExtIndex_ = _refline ? sdp->_NextExtIndex:0;
      SavedBoxDiff_ = _refline ? sdp->_NextBoxDiff:0;
      SavedOffset_ = sdp->_NextOffset;
      sdp->_BoxPosTrans = false;
    }
    else
    {
      SavedIndex_ = sdp->_SavedIndex;
      SavedExtIndex_ = _refline ? sdp->_SavedExtIndex:0;
      SavedBoxDiff_ = _refline ? sdp->_SavedBoxDiff:0;
      SavedOffset_ = sdp->_SavedOffset;
    }

    ReAlignBoxIndex();
    #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
      PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::HOME).SCAN + 256);
    #else
      PutExtKey(KEY_HOME);
    #endif

    if (sdp->_SavedOffset > 0)
    {
      for (KeyDone_ = true; _index < endpt_ && KeyDone_;)
      {
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
        #elif (defined(__linux__) | defined(__unix__))
          KeyDone_ = PutExtKey(KEY_RIGHT);
        #endif
      }

      if (KeyDone_ && _index == endpt_)
      {
        Offset_ = vbuf_   ? _inboxoffset:
                  inwbox_ ? _inwboxcnt:0;

        for (KeyDone_ = true; Offset_ < SavedOffset_ && KeyDone_;)
        {
          #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
            KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
          #elif (defined(__linux__) | defined(__unix__))
            KeyDone_ = PutExtKey(KEY_RIGHT);
          #endif

          Offset_ = vbuf_   ? _inboxoffset:
                    inwbox_ ? _inwboxcnt:0;
        }

        for (KeyDone_ = true; _refline && _refline->ExtendedIndex() < SavedExtIndex_ && KeyDone_;)
        {
          #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
            KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
          #elif (defined(__linux__) | defined(__unix__))
            KeyDone_ = PutExtKey(KEY_RIGHT);
          #endif
        }

        for (KeyDone_ = true; _index > SavedIndex_ && KeyDone_;)
        {
          #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
            KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::LEFTARROW).SCAN + 256);
          #elif (defined(__linux__) | defined(__unix__))
            KeyDone_ = PutExtKey(KEY_LEFT);
          #endif
        }
      }
    }
    else
    {    
      for (KeyDone_ = true; _index < SavedIndex_ && KeyDone_;)
      {
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
        #elif (defined(__linux__) | defined(__unix__))
          KeyDone_ = PutExtKey(KEY_RIGHT);
        #endif
      }
    }
  }
}

/****************************************************************************/
void WindowLineBuffer::SetBufferTypeCleared(int BufType_, bool Cleared_)
{
  if (Cleared_)
    _bufferscleared |= BufType_;
  else
    _bufferscleared &= ~BufType_;
}

/****************************************************************************/
bool WindowLineBuffer::BuffersCleared() const
{
  bool enterb_ = IsEnterable();
  bool vbuf_ = enterb_ && VirtualBufferOffset();
  bool inwbox_ = enterb_ && !vbuf_ && ((WindowLine*)_Owner)->LineWrapAllowed();
  return (inwbox_ && !vbuf_ && _BufChainNum && _bufferscleared);
}

/****************************************************************************/
int WindowLineBuffer::RestoreUnusedBuffers()
{
  int index;
  int len_;
  WindowLineBuffer* wlbptr = NULL;
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  
  if (!sdp)  
    return 0;

  wlbptr = sdp->_parent;

  if (IsEnterable() && BuffersCleared() && (!_buffer || !_bufferalloc))
  {
    if (BufferTypeCleared(INPUT_BUFTYPE))
    {
      delete[] _buffer;
      len_ = wlbptr->_maxbuf ? wlbptr->_maxbuf:MAXBUFFER;
      _buffer = NewCString(len_, 0, true);
      _bufferalloc = _buffer != NULL;

      if (_bufferalloc)
        SetBufferTypeCleared(INPUT_BUFTYPE, false);
    }

    if (BufferTypeCleared(RETURN_BUFTYPE))
    {
      if (wlbptr->_retbuf && wlbptr->_retbufalloc)
      {
        if (_bufferoffset > 0 || _BufChainNum > 0)
          wlbptr->SetReturnBuffer(this, wlbptr->_retbuf, false, wlbptr->_retbufsize);
        else
          wlbptr->SetReturnBuffer(this, wlbptr->_retbuf, true, (_len > 20) ? _len:20);
      }
      else
        CreateReturnBuffer(wlbptr->_retbufsize);
    }

    if (BufferTypeCleared(JOIN_BUFTYPE) && (!_joinbufalloc || !_joinbuf))
    {
      delete[] _joinbuf;
      len_ = wlbptr->_maxbuf ? wlbptr->_maxbuf:MAXBUFFER;
      _joinbuf = NewCString(len_, NOJOIN_CHAR, true);
      _joinbufalloc = _joinbuf != NULL;

      if (_joinbufalloc)
        SetBufferTypeCleared(JOIN_BUFTYPE, false);
    }

    if (wlbptr->_tranbufalloc && wlbptr->_tranbuf &&
        BufferTypeCleared(TRANS_BUFTYPE) && (!_tranbufalloc || !_tranbuf))
      CreateExtendedAsciiBuffer();
  }
  else if (!IsEnterable())
  {
    if (wlbptr->_retbuf && wlbptr->_retbufalloc)
      wlbptr->SetReturnBuffer(this, wlbptr->_retbuf, false, wlbptr->_retbufsize);
    else
      CreateReturnBuffer(wlbptr->_retbufsize);
  }

  return _bufferscleared;
}

/****************************************************************************/
int WindowLineBuffer::ClearUnusedBuffers()
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  bool enterb_ = IsEnterable();
  bool vbuf_ = enterb_ && sdp->_InWinMode && VirtualBufferOffset();
  bool inwbox_ = enterb_ && sdp->_InWinMode && !vbuf_ && ((WindowLine*)_Owner)->LineWrapAllowed();
  bool atnextbuf_ = inwbox_ && !vbuf_ && _BufChainNum;
  bool bufcleared_ = BuffersCleared();
 
  if (!sdp || bufcleared_)
    return 0;

  if (atnextbuf_ && !bufcleared_ && IsEmpty())
  {
    if (!BufferTypeCleared(INPUT_BUFTYPE) && _bufferalloc && _buffer)
    {
      delete[] _buffer;
      _buffer = NULL;
      _bufferalloc = false;
      SetBufferTypeCleared(INPUT_BUFTYPE, true);
    }

    if (!BufferTypeCleared(JOIN_BUFTYPE) && _joinbufalloc && _joinbuf)
    {
      delete[] _joinbuf;
      _joinbuf = NULL;
      _joinbufalloc = false;
      SetBufferTypeCleared(JOIN_BUFTYPE, true);
    }
    
    if (!BufferTypeCleared(TRANS_BUFTYPE) && _tranbufalloc && _tranbuf)
    {
      delete[] _tranbuf;
      _tranbuf = NULL;
      _tranbufalloc = false;
      SetBufferTypeCleared(TRANS_BUFTYPE, true);
    }
  }

  return _bufferscleared;
}

/****************************************************************************/
bool WindowLineBuffer::EndBoxEmpty()
{
  // new
  int x;
  int max = ((WindowLine*)_Owner)->GetBoxCount(true);
  WindowLineBuffer* ptr = this;
  
  for (x = 0; x < max && ptr->_nextline; x++)
    ptr = ptr->_nextline;

  return (ptr ? ptr->IsEmpty():false);
}

/****************************************************************************/
bool WindowLineBuffer::AllBoxFilled()
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);
  
  int x;
  int max = ((WindowLine*)_Owner)->GetBoxCount(true);
  WindowLineBuffer* ptr = this;
  bool NotEmpty_ = !IsEmpty();
  bool Filled_ = NotEmpty_;
  
  for (x = 0; x < max && Filled_ && ptr->_nextline; x++)
  {
    ptr = ptr->_nextline;
    Filled_ = Filled_ && !ptr->IsEmpty();
  }

  bool ret = (ptr ? Filled_:false);
  ret = ret || (inwbox_ && sdp && sdp->_numnextlines == 0 && NotEmpty_) ||
               (BufferChainPosition() == 0 && !_nextline && NotEmpty_);

  return ret;
}

/****************************************************************************/
/****************************************************************************/
int WindowLineBuffer::HasNonWsChars(const char* str_) const
{
  int x = 0;
  int max = str_ ? ::SafeStrLen(str_):0;

  if (max)
    for (x = max-1; x >= 0; x--)
      if (!isspace(str_[x]))
        return (x+1);

  return 0;
}

/****************************************************************************/
int WindowLineBuffer::HasNonNullChars(const char* str_) const
{
  int x = 0;
  int max = str_ ? ::SafeStrLen(str_):0;
  bool Reverted_ = true;

  if (max)
    for (x = max-1; x >= 0; x--)
    {
      Revert(str_[x], &Reverted_);
      if (!isspace(str_[x]) && !Reverted_)
        return (x+1);
    }

  return 0;
}

/****************************************************************************/
int WindowLineBuffer::CursorAtEOL(bool Reverse_)
{
  int WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
  int WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;
  char* RawBuf_ = RawStrBuffer(0);

  bool reflinemoved_ = _refline ?
         ((_refline->ExtendedIndex() > 0 || _refline->BoxDiff() > 0) && _refline->MoveDone()):
         false;
  bool PosMoveValid_ = Reverse_ ?
         reflinemoved_:
         ((_index && _index == strlen(RawBuf_)) ||
          (BufferChainPosition() == 0 && RawBuf_ && IsEmpty()) ||
          reflinemoved_);

  return
  (
    PosMoveValid_ &&
    AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false)
  );
}

/****************************************************************************/
bool WindowLineBuffer::IsAtEndPt(bool* PastEndPt_, WindowRefLineInfo* RefLinePtr_,
                                 bool* RefLineMoved_, int* ExtIndex_, int* BoxDiff_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return false;

  WindowRefLineInfo* RefPtr_ = _refline ? _refline:
                             RefLinePtr_ ? RefLinePtr_:NULL;

  bool enterb_ = IsEnterable();
  bool vbuf_ = enterb_ && VirtualBufferOffset();
  bool inwbox_ = BufferChainPosition() == 0 && enterb_ && !vbuf_ &&
                 ((WindowLine*)_Owner)->LineWrapAllowed();
  bool reflinemoved_ = RefPtr_ ?
       ((RefPtr_->ExtendedIndex() > 0 || RefPtr_->BoxDiff() > 0) && RefPtr_->MoveDone()):
       false;

  int Greater_ = RelationalEnum::GREATER_THAN;
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;
  int WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
  int WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();

  #if WINLINE_DEBUG3
    bool Debug_ = true;
  #else
    bool Debug_ = false;
  #endif

  bool AtEndPt_ = inwbox_ && _MovedToEnd &&
                  AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false) &&
                  (sdp->_InWinMode || Debug_);

  // RefLinePtr_ method parameter is not NULL then refline movement:
  //   (_extindex > 0 or _boxpos > 0) will not be checked. default == NULL.
  //
  if (PastEndPt_)
    *PastEndPt_ = AtEndPt_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, Greater_, false, false) &&
                  (RefLinePtr_ || !_refline || reflinemoved_);

  if (RefLineMoved_)
    *RefLineMoved_ = reflinemoved_;

  if (ExtIndex_)
    *ExtIndex_ = RefPtr_->ExtendedIndex();

  if (BoxDiff_)
    *BoxDiff_ = RefPtr_->BoxDiff();

  return AtEndPt_;
}

/****************************************************************************/
bool WindowLineBuffer::ExtendedBoxMode(bool& vbuf_, bool& inwbox_,
                                       int* EndReached_, int* NextNonWsLen_,
                                       int* WrapIndex_, int* WrapMax_) const
{
  // new
  int wrapdex_ = 0;
  int wrapmax_ = 0;
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;
  
  bool enterb_ = IsEnterable();
  bool NextBufOff_ = _nextline && _nextline->_bufferoffset;
  bool LwAllowed_ = _Owner && ((WindowLine*)_Owner)->LineWrapAllowed();

  vbuf_ = BufferChainPosition() == 0 && enterb_ && NextBufOff_ &&
          VirtualBufferOffset(EndReached_, NextNonWsLen_);
  inwbox_ = BufferChainPosition() == 0 && enterb_ && !vbuf_ && LwAllowed_;

  if (_Owner)
  {
    wrapdex_ = enterb_ ? ((WindowLine*)_Owner)->GetWrapAroundIndex():0;
    wrapmax_ = enterb_ ? ((WindowLine*)_Owner)->GetMaxWrapAround():0;
  }

  if (inwbox_ && EndReached_)
    *EndReached_ = ChainedBufferEndReached();

  #if (WINLINE_ONLYCWSCHR & WINLINE_WSENDMOVE)
    if (NextNonWsLen_ && inwbox_)
      *NextNonWsLen_ = HasNonWsChars(_buffer);
  #else
    if (NextNonWsLen_ && inwbox_)
      *NextNonWsLen_ =
        (_inwboxcnt > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false) &&
         _inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt) ?
             HasNonNullChars(_buffer):
             HasNonWsChars(_buffer);
  #endif

  if (WrapIndex_)
    *WrapIndex_ = wrapdex_;

  if (WrapMax_)
    *WrapMax_ = wrapmax_;

  return enterb_;
}

/****************************************************************************/
// Has extended _nextline implemented as WindowLineBuffer with buffer offset
//
bool WindowLineBuffer::ChainedBufferEndReached(int* UnJoinedLen_) const
{
  WindowLineBuffer* ptr;
  bool ret = false;
  int index_;
  int max;

  if (_nextline && IsEnterable() && BufferChainPosition() == 0)
  {
    if (_nextline->_joinbuf && _nextline->_buffer)
    {
      ptr = _nextline;
      for (index_ = 0; ptr; index_++)
        ptr = ptr->_nextline;

      max = index_ * _len;
      ret = (_inwboxcnt >= max) &&
            (!(*_nextline->RawStrBuffer(0)) ||
             (_nextline->_joinbuf && _nextline->_buffer &&
              _nextline->_joinbuf[0] == _nextline->_buffer[0]));
    }

    if (UnJoinedLen_)
    {
      if (ret)
        *UnJoinedLen_ = 0;
      else if (_nextline->_joinbuf && _nextline->_buffer)
      {
        for (index_ = 0;
             *_nextline->RawStrBuffer(index_) &&
             _nextline->_joinbuf[index_] != _nextline->_buffer[index_];
             index_++);
        *UnJoinedLen_ = index_;
      }
    }
  }
  else if (UnJoinedLen_ && _nextline &&
           _nextline->_joinbuf && _nextline->_buffer)
  {
    for (index_ = 0;
         *_nextline->RawStrBuffer(index_) &&
         _nextline->_joinbuf[index_] != _nextline->_buffer[index_];
         index_++);
    *UnJoinedLen_ = index_;
  }

  return ret;
}

/****************************************************************************/
// Has extended _nextline implemented as WindowLineBuffer with buffer offset
//
int WindowLineBuffer::VirtualBufferOffset(int* EndReached_,
                                          int* NextNonWsLen_, int* NextNonNullLen_) const
{
  int ret = 0;

  if (_nextline && IsEnterable())
  {  
    ret = _nextline->_bufferoffset;
    
    if (EndReached_)
    {
      *EndReached_ = _nextline->_maxbuf > 0 &&
                     (_nextline->_maxbuf - _inboxoffset) <= 0;

      if (NextNonWsLen_)
        *NextNonWsLen_ = *EndReached_ ? 0:HasNonWsChars(_nextline->_buffer + _inboxoffset);

      if (NextNonNullLen_)
        *NextNonNullLen_ = *EndReached_ ? 0:HasNonNullChars(_nextline->_buffer + _inboxoffset);
    }
  }

  return ret;
}

/****************************************************************************/
// Return the max remaining length of the extended _nextline buffer offset
//
int WindowLineBuffer::VirtualBufferMax() const
{
  int ret = 0;
  
  if (_nextline && IsEnterable())
  {
    ret = _nextline->_bufferoffset;
    if (ret)
      ret = _nextline->_maxbuf;
  }

  return ret;
}

/****************************************************************************/
int WindowLineBuffer::IsChainable(bool HasChars_) const
{
  int EndReached_ = 0;
  int NextNonWsLen_ = 0;
  
  bool enterb_ = IsEnterable();
  bool vbuf_ = BufferChainPosition() == 0 && enterb_ &&
               VirtualBufferOffset(&EndReached_, &NextNonWsLen_);

  return
  (
    vbuf_ ?
      (HasChars_ ? (_nextline && IsEnterable() && NextNonWsLen_ > 0):
                   (_nextline && IsEnterable())):

      (HasChars_ ? (_nextline && IsEnterable() && NextLength() > 0):
                   (_nextline && IsEnterable()))
  );
}

/****************************************************************************/
int WindowLineBuffer::NextLengthHelper() const
{
  int lastnwschar_ = HasNonWsChars(_buffer);
  
  if (_nextline && IsEnterable() && !lastnwschar_)
    return _nextline->NextLengthHelper();

  return lastnwschar_;
}

/****************************************************************************/
int WindowLineBuffer::NextLength() const
{
  const WindowLineBuffer::SharedChainData* sdp = ConstSharedData();
  if (!sdp || !_nextline)
    return 0;

  int UnJoinedLen_ = 0;
  int EndReached_ = 0;

  bool enterb_ = IsEnterable();
  bool vbuf_ = BufferChainPosition() == 0 && enterb_ &&
               VirtualBufferOffset(&EndReached_);
  bool inwbox_ = BufferChainPosition() == 0 &&  enterb_ && !vbuf_ &&
                 ((WindowLine*)_Owner)->LineWrapAllowed();
  int ret = _nextline->NextLengthHelper();

  if (!sdp->_countjoinedch && inwbox_)
  {
    EndReached_ = ChainedBufferEndReached(&UnJoinedLen_);

    if (!EndReached_ && ret > UnJoinedLen_)
      ret = UnJoinedLen_;
    else if (EndReached_)
      ret = 0;
  }

  return ret;
}

/****************************************************************************/
int WindowLineBuffer::ThisLength() const
{
  const WindowLineBuffer::SharedChainData* sdp = ConstSharedData();
  if (!sdp || !sdp->_parent)
    return 0;

  int UnJoinedLen_ = 0;
  int EndReached_ = 0;

  bool enterb_ = IsEnterable();
  bool vnextbuf_ = _nextline && _nextline->_bufferoffset;
  bool vbuf_ = BufferChainPosition() == 0 && enterb_ &&
               (_bufferoffset || vnextbuf_);
  bool inwbox_ = BufferChainPosition() == 0 &&  enterb_ && !vbuf_ &&
                 ((WindowLine*)_Owner)->LineWrapAllowed();                 
  int ret = enterb_ ? HasNonWsChars(_buffer):0;
  
  if (!sdp->_countjoinedch && inwbox_)
  {
    EndReached_ = ChainedBufferEndReached(&UnJoinedLen_);

    if (!EndReached_ && ret > UnJoinedLen_)
      ret = UnJoinedLen_;
    else if (EndReached_)
      ret = 0;
  }

  return ret;
}

/****************************************************************************/
void WindowLineBuffer::SetCountJoinedChars(bool Flag_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  if (sdp)
    sdp->_countjoinedch = Flag_;

  if (_nextline)
    _nextline->SetCountJoinedChars(Flag_);
}

/****************************************************************************/
int WindowLineBuffer::TakeNextLastCharHelper(bool Peek_)
{
  int lastnwschar_ = 0;

  if (NextLength() > 0)
    return _nextline->TakeNextLastCharHelper();
  else
    lastnwschar_ = HasNonWsChars(_buffer);

  char takech_ = lastnwschar_ ? _buffer[lastnwschar_-1]:0;
  bool DoExtract_ = takech_ && IsEnterable() && lastnwschar_;

  if (!Peek_ && DoExtract_)
  {
    _buffer[lastnwschar_-1] = 0;

    if (_joinbufalloc)
    {
      if (lastnwschar_-1 >= _maxbuf)
        _joinbuf[_maxbuf] = 0;
      else
        _joinbuf[lastnwschar_-1] = NOJOIN_CHAR;
    }

    if (_index >= lastnwschar_-1 && _index > 0)
    {
      _index = lastnwschar_ - 1;
      _index--;
      
      if (_index < 0)
        _index = 0;
    }
  }

  return takech_;
}

/****************************************************************************/
int WindowLineBuffer::TakeNextLastChar(bool Peek_)
{
  if (!_nextline)
    return 0;

  return _nextline->TakeNextLastCharHelper(Peek_);
}

/****************************************************************************/
int WindowLineBuffer::IsDummyLine() const
{
  return false;
}

/****************************************************************************/
int WindowLineBuffer::IsWindowLineBuffer() const
{
  return true;
}

/****************************************************************************/
int WindowLineBuffer::ChainPosition() const
{
  return _ChainPos;
}

/****************************************************************************/
int WindowLineBuffer::BufferChainPosition() const
{
  return _BufChainNum;
}

/****************************************************************************/
int WindowLineBuffer::IsFullBox() const
{
  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);

  int copylen_ = ::SafeStrLen(RawStrBuffer(0));
  char lastchar_ = (copylen_ == _len) ? *RawStrBuffer(_len - 1):0;
  return (_nextline ? (lastchar_ && _nextline->IsFullBox()):
                      (inwbox_ ? (lastchar_ &&
                                  !isspace(lastchar_) &&
                                  !IsEquivSpace(lastchar_)):lastchar_));
}

/****************************************************************************/
int WindowLineBuffer::RestrictFullBox() const
{
  return _RestFullBox;
}

/****************************************************************************/
bool WindowLineBuffer::EndBrksActive() const
{
  return _EndBrksActive;
}

/****************************************************************************/
bool WindowLineBuffer::ShouldTrimWs() const
{
  return _TrimWs;
}

/****************************************************************************/
int WindowLineBuffer::BufferIndex() const
{
  return _index;
}

/****************************************************************************/
int WindowLineBuffer::InBoxOffset() const
{
  return _inboxoffset;
}

/****************************************************************************/
int WindowLineBuffer::InWBoxCount() const
{
  return _inwboxcnt;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::DetachNext(WindowLineBuffer* Chainp_, bool Setup_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return NULL;

  WindowLineBuffer* ptr = this;
  WindowLineBuffer* RetChain_ = NULL;

  bool vbuf_ = false;
  bool inwbox_ = false;  
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);
  int SavedBufChainNum_;

  if (inwbox_ && Chainp_)
  {
    while (ptr && ptr != Chainp_)
      ptr = ptr->_nextline;

    if (ptr && ptr == Chainp_)
    {
      ptr = ptr->_nextline;
      Chainp_->ChainRemoveRetBuf(Chainp_->_retbuf);
      Chainp_->ChainRemoveRetBuf(sdp->_parent->_retbuf);
      Chainp_->_nextline = NULL;

      sdp->_numnextlines = ptr->BufferChainPosition()-1;
      ptr->SetOwner(Setup_ ? _Owner:NULL, ptr);
      ptr->EraseSharedChainData(NULL);
      ptr->_bufferalloc = ptr->_buffer != NULL;
      ptr->_retbufalloc = ptr->_retbuf != NULL;
      ptr->_joinbufalloc = ptr->_joinbuf != NULL;

      ptr->_ChainPosPtr = !ptr->_ChainPosPtr ? (new ChainPosInfo):NULL;
      ptr->_brackets[0] = CONTMORE_LEFT;  // Continue more to the left (left triangle)
      ptr->_brackets[1] = CONTMORE_RIGHT; // Continue more to the right (right triangle)
      ptr->_joinbufalloc = ptr->_joinbufalloc && ptr->_joinbuf != NULL;

      ptr->_Enterable = Chainp_->_Enterable;
      ptr->_Selectable = Chainp_->_Selectable;
      ptr->_InsertMode = Chainp_->_InsertMode;
      ptr->_Escaped = FALSE;
      ptr->_Hidden = TRUE;
      ptr->_InFocus = FALSE;
      ptr->_TextWidgetActive = FALSE;
      
      SavedBufChainNum_ = ptr->_BufChainNum;
      SetBufferChainNum(ptr, 0);
      // SetReturnBuffer(ptr, _retbuf, false, _retbufsize);

      ptr->_refline = Setup_ ? WindowRefLineInfo::Make(0, ptr, _Owner, ptr->_len):NULL;
      ((WindowLine*)_Owner)->FindControlCount();
      
      RetChain_ = ptr;

      if (RetChain_ && !RetChain_->_buffer)
      {
        SetBufferChainNum(RetChain_, SavedBufChainNum_);
        if (RetChain_->BuffersCleared())
          RetChain_->RestoreUnusedBuffers();
        SetBufferChainNum(RetChain_, 0);
      }
    }
  }

  return RetChain_;
}

/****************************************************************************/
WindowLineBuffer* WindowLineBuffer::DetachThis(WindowLineBuffer* Chainp_, bool Setup_)
{
  WindowLineBuffer* ptr = NULL;
  WindowLineBuffer* RetChain_ = NULL;

  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);

  if (inwbox_ && Chainp_)
  {
    ptr = DetachNext(Chainp_, false);
    RetChain_ = WindowLineBuffer::Make(*Chainp_);

    if (RetChain_)
    {
      Erase();
      RetChain_->SetOwner(Setup_ ? _Owner:NULL, RetChain_);
      RetChain_->EraseSharedChainData(NULL);
      RetChain_->_bufferalloc = RetChain_->_buffer != NULL;
      RetChain_->_retbufalloc = RetChain_->_retbuf != NULL;
      RetChain_->_joinbufalloc = RetChain_->_joinbuf != NULL;

      RetChain_->_ChainPosPtr = !RetChain_->_ChainPosPtr ? (new ChainPosInfo):NULL;
      RetChain_->_brackets[0] = CONTMORE_LEFT;  // Continue more to the left (left triangle)
      RetChain_->_brackets[1] = CONTMORE_RIGHT; // Continue more to the right (right triangle)
      RetChain_->_joinbufalloc = RetChain_->_joinbufalloc && RetChain_->_joinbuf != NULL;

      RetChain_->_Enterable = Chainp_->_Enterable;
      RetChain_->_Selectable = Chainp_->_Selectable;
      RetChain_->_InsertMode = Chainp_->_InsertMode;
      RetChain_->_Escaped = FALSE;
      RetChain_->_Hidden = TRUE;
      RetChain_->_InFocus = FALSE;
      RetChain_->_TextWidgetActive = FALSE;
      
      SetBufferChainNum(RetChain_, 0);
      // SetReturnBuffer(RetChain_, _retbuf, false, _retbufsize);

      RetChain_->_refline = Setup_ ? WindowRefLineInfo::Make(0, RetChain_, _Owner, RetChain_->_len):NULL;

      if (ptr && ptr != RetChain_)
        RetChain_->AddBoxChain(ptr, false);

      ((WindowLine*)_Owner)->FindControlCount();
    }
  }

  return RetChain_;
}

/****************************************************************************/
void WindowLineBuffer::ChainPush(char ch, char* join_, ChainPosInfo* Info_)
{
  if (_nextline)  
  {
    int index_ = 0;
    bool enterb_ = IsEnterable();
    bool vbuf_ = VirtualBufferOffset() && enterb_;
    
    if (vbuf_)
    {
      index_ = _inboxoffset;
      _nextline->SetOwner(_Owner, _nextline);

      if (Info_)
      {
        Info_->SetupInfoPtrs(_nextline, 1);
        Info_->SetPushInfo(_inboxoffset);
      }
    }

    _nextline->Push(index_, ch, join_, Info_);

    if (vbuf_)
    {
      _nextline->SetOwner(NULL, _nextline);

      if (Info_)
        Info_->RestorePrevInfo();
    }
  }
}

/****************************************************************************/
void WindowLineBuffer::ChainJoin(char ch)
{
  if (_nextline)
    _nextline->Join(ch);
}

/****************************************************************************/
char WindowLineBuffer::ChainPull(char* join_, char* chain_, ChainPosInfo* Info_)
{
  if (_nextline)
  {
    int index_ = 0;
    bool enterb_ = IsEnterable();
    bool vbuf_ = VirtualBufferOffset() && enterb_;
    
    if (vbuf_)
    {
      index_ = _inboxoffset;
      _nextline->SetOwner(_Owner, _nextline);

      if (Info_)
      {
        Info_->SetupInfoPtrs(_nextline, 1);
        Info_->SetPullInfo(_inboxoffset);
      }
    }
      
    char ret_ = _nextline->Pull(index_, join_, chain_, Info_);

    if (vbuf_)
    {
      _nextline->SetOwner(NULL, _nextline);

      if (Info_)
        Info_->RestorePrevInfo();
    }

    return ret_;
  }

  return '\0';
}

/****************************************************************************/
char WindowLineBuffer::PeekNextChar(int Index_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return NULL;

  int endpt_ = _len - 1;
  int Lesser_ = RelationalEnum::LESS_THAN;

  bool enterb_ = IsEnterable();
  bool vbuf_ = enterb_ && VirtualBufferOffset();
  bool inwbox_ = BufferChainPosition() == 0 && enterb_ && !vbuf_ &&
                 ((WindowLine*)_Owner)->LineWrapAllowed();

  int WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
  int WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
  bool NotAtDexEnd_ = AtMaxWrap(WrapIndex_, WrapMax_, endpt_, Lesser_, true, false);

  if (_nextline && (inwbox_ || vbuf_) && NotAtDexEnd_ && sdp->_InWinMode)
    return _nextline->ChrRetrieve(Index_);

  return '\0';
}

/****************************************************************************/
bool WindowLineBuffer::Push(int start, char ch, char* join_, ChainPosInfo* Info_)
{
  if (BuffersCleared())
    RestoreUnusedBuffers();

  if ((!Info_ || Info_->_ChainPos == 2) && _ChainPosPtr)
  {
    _ChainPosPtr->SetupInfoPtrs(this, 2);
    _ChainPosPtr->SetPushInfo(start);

    Info_ = _ChainPosPtr;
  }
  else if (Info_)
    Info_->ReturnToThis();

  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return false;

  bool enterb_ = _Owner && IsEnterable();
  int index_;
  int PrevJoin_ = 0;
  int WrapMax_ = enterb_ ? ((WindowLine*)_Owner)->GetMaxWrapAround():0;
  int WrapIndex_;
  int totallen_ = ::SafeStrLen(&_buffer[Info_->_startpt]);
  int atend_ = totallen_ >= (Info_->_endpt - Info_->_startpt);
  bool DoJoin_ = join_ && _joinbufalloc && atend_;
  char nextchar_ = 0;
  char nextjoin_ = 0;
  char lastchar_ = atend_ ? _buffer[Info_->_endpt - 1]:0;
  char lastjoin_ = DoJoin_ ? _joinbuf[Info_->_endpt - 1]:0;
  
  atend_ = lastchar_ != 0;
  bool DoChain_ = atend_ && Info_->_chainable;
  bool MoveDone_ = false;
  bool UserKey_ = false;
  bool ChainDone_ = false;
  bool WrapDone_ = false;
  bool FullBox_ = (_nextline && IsEnterable()) ? _nextline->IsFullBox():
                                                 IsFullBox();

  if (atend_)
  {
    --Info_->_copylen;
    atend_ = 1;
  }

  if (Info_->_copylen < 0)
    Info_->_copylen = 0;

  if (!Info_->_error && (!_RestFullBox || !FullBox_ || !atend_))
  {
    if (Info_->_copylen)
    {
      index_ = Info_->_index + Info_->_copylen + atend_;
      nextchar_ = Info_->_vbuf ? _buffer[index_]:0;
      nextjoin_ = (index_ < _maxbuf) ? NOJOIN_CHAR:0;

      index_ = (Info_->_index+1) + Info_->_copylen;
      MoveDone_ = Info_->_copylen > 0;

      if (MoveDone_)
      {
        memmove(&_buffer[Info_->_index + 1], &_buffer[Info_->_index], Info_->_copylen);
        _buffer[index_] = nextchar_;

        if (join_ && _joinbufalloc)
        {
          memmove(&_joinbuf[Info_->_index + 1], &_joinbuf[Info_->_index], Info_->_copylen);
          _joinbuf[index_] = nextjoin_;
          _joinbuf[_maxbuf] = 0;
        }
      }
    }

    if (0 <= Info_->_index && Info_->_index < Info_->_endpt)
    {
      _buffer[Info_->_index] = ch;
      MoveDone_ = true;

      if (join_ && _joinbufalloc)
      {
        _joinbuf[Info_->_index] = *join_;
        if (DoChain_ && DoJoin_)
          *join_ = lastjoin_;
      }

      if (DoChain_ && lastchar_)
      {
        ChainPush(lastchar_, join_, Info_);
        ChainDone_ = true;
      }
    }

    if (_nextline == NULL && sdp && _linestatus == UNINIT_NL)
    {
      if (sdp->_numnextlines < 0)
        sdp->_numnextlines = 0;
      
      if (::SafeStrLen(_buffer))
        _linestatus = FILLED_NL;
    }

    if ((ChainDone_ || (!atend_ && Info_->_chainable && _inwboxcnt > 0)) &&
        Info_->_inwbox && _Owner && IsEnterable())
      if ((UserKey_=IsUserKey(_extkey)) || _WrapChars)
      {
        WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
        DoChain_ = Info_->_chainable;

        if (_WrapChars && !UserKey_ && WrapIndex_ > 0)
        {
          ((WindowLine*)_Owner)->DecrWrapAround(1);
          --WrapIndex_;
          WrapDone_ = true;
        }
        else if (UserKey_)
        {
          WrapMax_ = enterb_ ? ((WindowLine*)_Owner)->GetMaxWrapAround():0;
          _WrapChars = false;
        }

        if (!UserKey_)
          _WrapChars = _nextline && DoChain_ && !UserKey_ &&
                       Info_->_inwbox && WrapIndex_ > 0;
      }
  }

  return MoveDone_;
}

/****************************************************************************/
void WindowLineBuffer::Join(char ch)
{
  if (BuffersCleared())
    RestoreUnusedBuffers();

  int slen_ = ::SafeStrLen(_buffer);
  bool HasRoom_ = slen_ < _len;
  
  char added_[2];
  added_[0] = ch;
  added_[1] = 0;

  if (_nextline && (NextLength() > 0 || !HasRoom_ || _EndCharInserted))
    ChainJoin(ch);
  else if (HasRoom_ && _buffer && BufferChainPosition() > 0)
  {
    slen_ = strlen(_buffer);
    strcat(_buffer, added_);

    if (_joinbufalloc)
    {
      _joinbuf[slen_] = ch;
      _joinbuf[_maxbuf] = 0;
    }
  }
}

/****************************************************************************/
char WindowLineBuffer::Pull(int start, char* join_, char* chain_, ChainPosInfo* Info_)
{
  if (BuffersCleared())
    RestoreUnusedBuffers();

  if ((!Info_ || Info_->_ChainPos == 2) && _ChainPosPtr)
  {
    _ChainPosPtr->SetupInfoPtrs(this, 2);
    _ChainPosPtr->SetPullInfo(start);

    Info_ = _ChainPosPtr;
  }
  else if (Info_)
    Info_->ReturnToThis();

  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return false;

  bool FindNws_ = false;  
  bool enterb_ = _Owner && IsEnterable();
  int Lesser_ = RelationalEnum::LESS_THAN;
  int LesserOrEqual_ = RelationalEnum::LESS_THAN_OR_EQUAL;
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;
  int BoxCount_ = enterb_ ? ((WindowLine*)_Owner)->GetBoxCount(true):0;
  int WrapIndex_ = enterb_ ? ((WindowLine*)_Owner)->GetWrapAroundIndex():0;
  int WrapMax_ = enterb_ ? ((WindowLine*)_Owner)->GetMaxWrapAround():0;
  int NullChar_ = 0;
  bool inwbox_ = Info_->_inwbox;
  bool vbuf_ = Info_->_vbuf;
  bool DoChain_ = Info_->_chainable && !Info_->_error;
  bool ChainDone_ = false;
  bool HasEndChar_ = false;
  bool DelKey_ = false;
  bool MoveDone_ = false;
  bool WrapDone_ = false;
  char firstchar_ = _buffer[Info_->_index];
  char firstjoin_ = (join_ && _joinbufalloc) ? _joinbuf[Info_->_index]:0;
  char nextchar_ = 0;
  char lastchar_;
  int lastnwspos_;
  int blanklen_ = 0;
  int index_;

  if (Info_->_copylen < 0)
    Info_->_copylen = 0;

  if (!Info_->_copylen && !Info_->_error)
  {
    if (_nextline && IsEnterable())
    {
      int x;
      start = Info_->_index;

      for (x = start;
           start < Info_->_endpt && !ChainedBufferEndReached(); start++)
      {
        _buffer[start] = ChainPull(join_, chain_, Info_);
        ChainDone_ = true;
        
        if (join_ && _joinbufalloc)
          _joinbuf[start] = *join_;

        if (chain_)
          *chain_ = _buffer[start];

        if (!_buffer[start])
          break;
      }

      if (_nextline == NULL && sdp && _linestatus == UNINIT_NL)
      {
        if (sdp->_numnextlines < 0)
          sdp->_numnextlines = 0;
      
        if (::SafeStrLen(_buffer))
          _linestatus = FILLED_NL;
      }

      if (x != start && _WrapChars && ChainDone_ &&
          _Owner && Info_->_inwbox && IsEnterable())
      {
        WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
        WrapMax_ = ((WindowLine*)_Owner)->GetMaxWrapAround();
        DoChain_ = Info_->_chainable;

        #if WINLINE_ONLYCWSCHR
          lastnwspos_ = HasNonWsChars(_buffer);
        #else
          lastnwspos_ =
            (BufferChainPosition() < BoxCount_-1 &&
             _inwboxcnt > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false)
             _inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt) ?
                 HasNonNullChars(_buffer):
                 HasNonWsChars(_buffer);
        #endif

        if (AtMaxWrap(WrapIndex_, WrapMax_, 0, Lesser_, false, false))
        {
          ((WindowLine*)_Owner)->IncrWrapAround(start - x);
          WrapIndex_ += (start - x);
          WrapDone_ = true;
        }

        _WrapChars = (_nextline && DoChain_ && Info_->_inwbox) &&
                     (AtMaxWrap(WrapIndex_, WrapMax_, 0, Lesser_, false, false) ||
                       ((lastnwspos_ < _len) ? (_inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt):
                                               (_inwboxcnt == WrapMax_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, LesserOrEqual_, false, false))));
      }
    }
  }
  else if (_buffer[Info_->_index] && Info_->_copylen && !Info_->_error)
  {
    blanklen_ = 0;
    NullChar_ = (_buffer[(Info_->_index+1) + Info_->_copylen - 1] == 0) ? 1:0;

    if (DoChain_ || _bufferoffset > 0)
    {
      if (_bufferoffset == 0 && DoChain_)
      {
        FindNws_ = IsChainable(true);
      
        if (FindNws_)
        {
          inwbox_ = enterb_ && !VirtualBufferOffset() &&
                    ((WindowLine*)_Owner)->LineWrapAllowed();
          WrapIndex_ = enterb_ ? ((WindowLine*)_Owner)->GetWrapAroundIndex():0;
          WrapMax_ = enterb_ ? ((WindowLine*)_Owner)->GetMaxWrapAround():0;
          DoChain_ = inwbox_ ?
                     (BufferChainPosition() > 0 ||
                      (BufferChainPosition() == 0 &&
                       0 <= WrapIndex_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, Lesser_, false, false))):true;

          if (DoChain_)
          {
            nextchar_ = ChainPull(join_, chain_, Info_);
            ChainDone_ = true;

            if (chain_)
              *chain_ = nextchar_;
          }
        }
      }
      else
        FindNws_ = IsChainable(true);

      HasEndChar_ = false;
      MoveDone_ = Info_->_copylen > 0;

      if (MoveDone_)
      {
        memmove(&_buffer[Info_->_index], &_buffer[Info_->_index + 1], Info_->_copylen);
        if (join_ && _joinbufalloc)
          memmove(&_joinbuf[Info_->_index], &_joinbuf[Info_->_index + 1], Info_->_copylen);

        HasEndChar_ = ChainDone_ || (_MovedToEnd && _EndCharInserted);
      }

      if (MoveDone_ && FindNws_)
      {
        lastchar_ = _buffer[Info_->_endpt];
        if (Info_->_vbuf)
          _buffer[Info_->_endpt] = 0;

        #if (WINLINE_ONLYCWSCHR & WINLINE_WSENDMOVE)
          if (inwbox_)
            lastnwspos_ = HasNonWsChars(&_buffer[Info_->_startpt]);
          else if (vbuf_)
            lastnwspos_ = HasNonWsChars(&_buffer[Info_->_startpt]);
        #else
          if (inwbox_)
            lastnwspos_ =
              (BufferChainPosition() < BoxCount_-1 &&
               _inwboxcnt > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false) &&
               _inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt) ?
                   HasNonNullChars(&_buffer[Info_->_startpt]):
                   HasNonWsChars(&_buffer[Info_->_startpt]);
          else if (vbuf_)
            lastnwspos_ =
              (_bufferoffset == 0 &&
               _inboxoffset > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false)
               _inboxoffset >= WrapMax_ && WrapIndex_ <= _inboxoffset) ?
                   HasNonNullChars(&_buffer[Info_->_startpt]):
                   HasNonWsChars(&_buffer[Info_->_startpt]);
        #endif

        blanklen_ = (Info_->_endpt - Info_->_startpt) - lastnwspos_ - 1;

        if (Info_->_vbuf)
          _buffer[Info_->_endpt] = lastchar_;

        if (!ChainDone_)
          nextchar_ = 0;
      }
      
      if (MoveDone_ && blanklen_ > 0 && FindNws_ && nextchar_ && HasEndChar_)
        ::memset(&_buffer[Info_->_startpt+lastnwspos_+1], SPACE_CHAR, blanklen_);

      if (MoveDone_ && !nextchar_ && (!FindNws_ || (HasEndChar_ && !Info_->_vbuf)))
      {
        index_ = Info_->_index + Info_->_copylen - NullChar_;
        while (index_ <= Info_->_endpt)
          _buffer[index_++] = 0;

        if (join_ && _joinbufalloc)
        {
          index_ = Info_->_index + Info_->_copylen - NullChar_;
          while (index_ <= Info_->_endpt)
            _joinbuf[index_++] = NOJOIN_CHAR;

          _joinbuf[_maxbuf] = 0;
        }
      }

      if (HasEndChar_)
      {
        _buffer[Info_->_endpt - 1] = nextchar_;
        MoveDone_ = true;

        if (join_ && _joinbufalloc)
        {
          _joinbuf[Info_->_endpt - 1] = *join_;
          *join_ = firstjoin_;
        }

        if (!Info_->_vbuf)
        {
          _buffer[Info_->_endpt] = 0;
          
          if (join_ && _joinbufalloc)
          {
            if (Info_->_endpt >= _maxbuf)
              _joinbuf[_maxbuf] = 0;
            else
              _joinbuf[Info_->_endpt] = NOJOIN_CHAR;
          }
        }
      }
    }
    else
    {
      MoveDone_ = Info_->_copylen > 0;

      if (MoveDone_)
      {
        memmove(&_buffer[Info_->_index], &_buffer[Info_->_index + 1], Info_->_copylen);
        if (join_)
          memmove(&_joinbuf[Info_->_index], &_joinbuf[Info_->_index + 1], Info_->_copylen);
      }

      if (!Info_->_vbuf)
      {
        _buffer[Info_->_index + Info_->_copylen - NullChar_] = 0;
        _buffer[Info_->_endpt] = 0;

        if (join_)
        {
          _joinbuf[Info_->_index + Info_->_copylen - NullChar_] = NOJOIN_CHAR;

          if (Info_->_endpt >= _maxbuf)
            _joinbuf[_maxbuf] = 0;
          else
            _joinbuf[Info_->_endpt] = NOJOIN_CHAR;
        }
      }
    }

    if (_nextline == NULL && sdp && _linestatus == UNINIT_NL)
    {
      if (sdp->_numnextlines < 0)
        sdp->_numnextlines = 0;
      
      if (::SafeStrLen(_buffer))
        _linestatus = FILLED_NL;
    }

    if (ChainDone_ && Info_->_inwbox && _Owner && IsEnterable())
    #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
      if ((DelKey_=GetKeyMap().KeyMatch(_extkey, DELETE)) || _WrapChars)
    #elif (defined(__linux__) | defined(__unix__))
      if ((DelKey_=(_extkey == KEY_DC)) || _WrapChars)
    #endif
      {
        WrapIndex_ = ((WindowLine*)_Owner)->GetWrapAroundIndex();
        DoChain_ = Info_->_chainable;
        
        #if WINLINE_ONLYCWSCHR
          lastnwspos_ = HasNonWsChars(_buffer);
        #else
          lastnwspos_ =
            (BufferChainPosition() < BoxCount_-1 &&
             _inwboxcnt > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false)
             _inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt) ?
                 HasNonNullChars(_buffer):
                 HasNonWsChars(_buffer);
        #endif

        if (_WrapChars && !DelKey_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, Lesser_, false, false))
        {
          ((WindowLine*)_Owner)->IncrWrapAround(1);
          ++WrapIndex_;
          WrapDone_ = true;
        }
        else if (DelKey_)
        {
          WrapMax_ = enterb_ ? ((WindowLine*)_Owner)->GetMaxWrapAround():0;
          _WrapChars = false;
        }

        if (!DelKey_)
          _WrapChars = (_nextline && DoChain_ && !DelKey_ && Info_->_inwbox) &&
                        (AtMaxWrap(WrapIndex_, WrapMax_, 0, Lesser_, false, false) ||
                           ((lastnwspos_ < _len) ? (_inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt):
                                                   (_inwboxcnt == WrapMax_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, LesserOrEqual_, false, false))));
      }
  }

  if (chain_ && !ChainDone_)
    *chain_ = 0;

  if (join_)
    *join_ = MoveDone_ ? firstjoin_:0;

  return (MoveDone_ ? firstchar_:0);
}

/****************************************************************************/
void WindowLineBuffer::DetermineWrapChars()
{
  int WrapIndex_ = 0;
  int WrapMax_ = 0;
  bool DoChain_ = false;
  int lastnwspos_ = 0;
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;
  int LesserOrEqual_ = RelationalEnum::LESS_THAN_OR_EQUAL;
  int Lesser_ = RelationalEnum::LESS_THAN;

  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_,
                                 NULL, &lastnwspos_,
                                 &WrapIndex_, &WrapMax_);

  #if WINLINE_ONLYCWSCHR
    lastnwspos_ = inwbox_ ? HasNonWsChars(_buffer):0;
  #else
    lastnwspos_ = inwbox_ ?
      ((_inwboxcnt > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false)
        _inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt) ?
            HasNonNullChars(_buffer):
            HasNonWsChars(_buffer)):0;
  #endif
    
  DoChain_ = IsChainable(false);

  if (_WrapChars && WrapIndex_ > 0 && _Owner && enterb_)
    _WrapChars = (_nextline && DoChain_ && inwbox_) &&
                 (0 < WrapIndex_ &&
                    (AtMaxWrap(WrapIndex_, WrapMax_, 0, Lesser_, false, false) ||
                       ((lastnwspos_ < _len) ? (_inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt):
                                               (_inwboxcnt == WrapMax_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, LesserOrEqual_, false, false)))));
  else
    _WrapChars = false;
}

/****************************************************************************/
bool WindowLineBuffer::CreateExtendedAsciiBuffer()
{
  bool bufcleared_ = false;
  bool bufdeleted_ = false;
  int len_;

  if (!BuffersCleared())
  {
    if (_tranbufalloc)
    {
      delete[] _tranbuf;
      _tranbufalloc = false;
      bufdeleted_ = true;
    }

    _tranbuf = NULL;
  }

  if (BuffersCleared() && (!_tranbufalloc || !_tranbuf))
  {
    if (!bufdeleted_ && _tranbuf)
    {
      delete[] _tranbuf;
      _tranbufalloc = false;
      bufdeleted_ = true;
    }
    
    _tranbuf = NULL;
    bufcleared_ = true;
  }

  if (_bufferoffset)
    return false;

  len_ = _maxbuf ? _maxbuf:MAXBUFFER;
  _tranbuf = NewCString(len_, 0, true);
  _tranbufalloc = _tranbuf != NULL;

  if (bufcleared_ && _tranbufalloc)
    SetBufferTypeCleared(TRANS_BUFTYPE, false);

  return _tranbufalloc;
}

/****************************************************************************/
void WindowLineBuffer::ResetRefPos(int MoveToPos_)
{
  int EndReached_ = 0;
  int NextNonWsLen_ = 0;
  int NonWsLen_ = 0;
  int WrapIndex_ = 0;
  int WrapMax_ = 0;

  bool KeyDone_ = true;
  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_,
                                 &EndReached_, &NextNonWsLen_,
                                 &WrapIndex_, &WrapMax_);

  int Lesser_ = RelationalEnum::LESS_THAN;
  int endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);
  int bufoffset_ = vbuf_ ? _inboxoffset:
                   inwbox_ ? _inwboxcnt:0;

  // new
  if (vbuf_)
    VbufStrLen(&NonWsLen_);
  
  if (MoveToPos_ == 0 &&
      ((_index == 0 && bufoffset_ > 0 && WrapIndex_ > 0) || _index > 0))
  {
    if (_refline)
      _refline->ResetBoxIndex();

    while (KeyDone_ &&
           ((_index == 0 && bufoffset_ > 0 && WrapIndex_ > 0) || _index > 0))
    {
      #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
        KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::LEFTARROW).SCAN + 256);
      #elif (defined(__linux__) | defined(__unix__))
        KeyDone_ = PutExtKey(KEY_LEFT);
      #endif

      bufoffset_ = vbuf_ ? _inboxoffset:
                   inwbox_ ? _inwboxcnt:0;
    }
  }
  else if (MoveToPos_ == 1 &&
           ((AtMaxWrap(WrapIndex_, WrapMax_, endpt_, Lesser_, true, false) &&
            !EndReached_) || CursorMoveCondMet(WrapIndex_, WrapMax_, endpt_, NonWsLen_)))
  {
    if (_refline)
      _refline->SetBoxIndex(bufoffset_ + endpt_);

    while (KeyDone_ &&
           ((AtMaxWrap(WrapIndex_, WrapMax_, endpt_, Lesser_, true, false) &&
            !EndReached_) || CursorMoveCondMet(WrapIndex_, WrapMax_, endpt_, NonWsLen_)))
    {
      #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
        KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
      #elif (defined(__linux__) | defined(__unix__))
        KeyDone_ = PutExtKey(KEY_RIGHT);
      #endif
    }
  }
}

/****************************************************************************/
void WindowLineBuffer::ReAdjustLine(int Offset_)
{
  // new
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);
  
  if (!sdp || !inwbox_)
    return;

  int x = _len - Offset_;
  int start = ::SafeStrLen(_buffer);
  int max = _len - start;
  
  bool KeyDone_ = true;
  char* NewStr_ = NULL;
  WindowLineBuffer* ptr = this;

  if (start && start > x)
  {
    start = x;
    max = _len - start;
  }

  if (_nextline && !_nextline->IsEmpty())
  {
    if (start < _len)
    {
      NewStr_ = strcpy(new char[_len+1], _buffer);
      for (x = start; (NewStr_[x] = _buffer[x]) && x < _len; x++);
      while (x < _len)
        NewStr_[x++] = SPACE_CHAR;
      NewStr_[_len] = 0;
      SetInputString(NewStr_);
      delete[] NewStr_;
    }
    else
      start = _len - 1;

    KeyDone_ = true;
    while (_index < start && KeyDone_)
      #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
        KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
      #elif (defined(__linux__) | defined(__unix__))
        KeyDone_ = PutExtKey(KEY_RIGHT);
      #endif

    KeyDone_ = true;
    for (x = 0; x < max; x++)
      #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
        KeyDone_ = PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::DELETE).SCAN + 256);
      #elif (defined(__linux__) | defined(__unix__))
        KeyDone_ = PutExtKey(KEY_DC);
      #endif
  }

  for (x = 0; ptr->_nextline; x++)
  {
    ptr->SetBufferChainNum(ptr->_nextline, ptr->_BufChainNum + 1);
    ptr->_nextline->SetChainPos(ptr->ChainPosition() + 1);
    ptr = ptr->_nextline;
  }

  sdp->_numnextlines = x;
}

/****************************************************************************/
void WindowLineBuffer::ShiftDownAndSetStr(char* NewStr_)
{
  if (_nextline)
    _nextline->ShiftDownAndSetStr(_buffer);
  SetInputString(NewStr_);
}

/****************************************************************************/
bool WindowLineBuffer::InChSet(char ch, const char* chset, size_t* Index_)
{
  size_t dummy;
  
  if (Index_)
    *Index_ = 0;
  else
    Index_ = &dummy;

  if (chset == NULL || *chset == 0)
    return FALSE;

  if (*chset == '0' && ch == 0)
    return TRUE;

  if (*chset == '-' || *chset == '0')
  {
    ++chset;
    *Index_ += 1;
  }

  for (;*chset; chset++)
  {
    if (ch == *chset)
      return TRUE;
      
    *Index_ += 1;
  }

  *Index_ = 0;
  return FALSE;
}

/****************************************************************************/
char* WindowLineBuffer::RemoveTrailSpaceChars(char* Str_, bool UnReverted_)
{
  char chset_[5];

  if (UnReverted_)
  {
    chset_[0] = SPACE_CHAR;
    chset_[1] = TAB_CHAR;
    chset_[2] = RETURN_CHAR;
    chset_[3] = NEWLINE_CHAR;
    chset_[4] = 0;
  }
  else
    strcpy(chset_, " \t\r\n");
  
  for (size_t l = ::SafeStrLen(Str_); l;)
    if (InChSet(Str_[--l], chset_))
      Str_[l] = '\0';
    else
      break;

  return Str_;
}

/****************************************************************************/
bool WindowLineBuffer::IsEquivSpace(char ch) const
{
  return
  (
    Uchar(ch) == SPACE_CHAR ||
    Uchar(ch) == TAB_CHAR ||
    Uchar(ch) == RETURN_CHAR ||
    Uchar(ch) == NEWLINE_CHAR
  );
}

/****************************************************************************/
Uchar WindowLineBuffer::Convert(char ch, bool* Converted_) const
{
  Uchar oldch = Uchar(ch);
  Uchar newch;

  newch = (ch == ' ')  ? Uchar(SPACE_CHAR):
          (ch == '\t') ? Uchar(TAB_CHAR):
          (ch == '\r') ? Uchar(RETURN_CHAR):
          (ch == '\n') ? Uchar(NEWLINE_CHAR):oldch;

  if (Converted_)
    *Converted_ = newch != oldch;

  return newch;
}

/****************************************************************************/
char WindowLineBuffer::Revert(Uchar ch, bool* Reverted_) const
{
  char oldch = char(ch);
  char newch;
  
  newch = (ch == Uchar(SPACE_CHAR))   ? ' ':
          (ch == Uchar(TAB_CHAR))     ? '\t':
          (ch == Uchar(RETURN_CHAR))  ? '\r':
          (ch == Uchar(NEWLINE_CHAR)) ? '\n':oldch;

  if (Reverted_)
    *Reverted_ = newch != oldch;

  return newch;
}

/****************************************************************************/
char* WindowLineBuffer::Convert(char* Str_)
{
  if (!Str_)
    return Str_;

  // replace ' ' with 0xAE, '\t' with 0xAF, '\r' with 0xB0
  // and '\n' with 0xB1: for vbuf and inwbox mode input boxes
  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);

  size_t x;
  size_t len;
  
  if (vbuf_ || inwbox_ || BufferChainPosition() > 0)
  {
    len = ::SafeStrLen(Str_);

    for (x = 0; x < len; x++)
      Str_[x] = Convert(Str_[x]);
  }

  return Str_;
}

/****************************************************************************/
char* WindowLineBuffer::Revert(char* Str_)
{
  if (!Str_)
    return Str_;

  // replace 0xAE with ' ', 0xAF with '\t', 0xB0 with '\r'
  // and 0xB1 with '\n': for vbuf and inwbox mode input boxes
  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);
  
  size_t x;
  size_t len;

  if (vbuf_ || inwbox_ || BufferChainPosition() > 0)
  {
    len = ::SafeStrLen(Str_);

    for (x = 0; x < len; x++)
      Str_[x] = Revert(Str_[x]);
  }

  return Str_;
}

/****************************************************************************/
void WindowLineBuffer::FillString(char* Str_, char FillCh_, int Start_, int End_)
{
  for (;Start_ < End_; Start_++)
    Str_[Start_] = FillCh_;

  Str_[End_] = '\0';
}

/****************************************************************************/
/****************************************************************************/
// PURPOSE:
//   End input test procedure
//
// PRE:
//   int key : the key to be tested as the termination key for input.
//
// POST:
//   Returns 1 if the key is defined as a termination key,
//   otherwise returns 0.
//
int WindowLineBuffer::ShouldEnd(int key, int zerotozero)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  WindowLine::SharedChainData* wlsdp = NULL;

  int status_;
  int coldir_ = 0;
  int linedir_ = 0;
  bool TransferDone_ = false;
  bool VmoveDone_ = false;
  bool DnKeyDone_ = false;

  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);

  #if WINLINE_DEBUG3
    bool Debug_ = true;
  #else
    bool Debug_ = false;
  #endif
  
  sdp->_InWinMode = (enterb_ && _Owner) ?
      ((WindowLine*)_Owner)->InWindowMode(&sdp->_NextLineReady,
                                          &sdp->_PrevLineReady):false;
  wlsdp = _Owner ? ((WindowLine*)_Owner)->SharedData():NULL;

  // Action: GO_DOWN, ESCAPE
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  if (GetKeyMap().KeyMatch(key, ESCAPE))
#elif (defined(__linux__) | defined(__unix__))
  if (key == 27)
#endif
  {
    // If in window mode and encoded null move (stay in place)
    _Escaped = TRUE;
    iMonitor(_escaped_monitor, _escaped_handle, &_Escaped);
    linedir_ = status_ = 1;
    VmoveDone_ = true;
  }

  // Action: GO_DOWN, IF_NOTCHAINED_ESCAPE
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, ENTER))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == 10)
#endif  
  {
    _Escaped = (wlsdp && sdp && sdp->_InWinMode) ?
                   (!sdp->_NextLineReady &&
                    wlsdp->_ReturnOnLastLine && wlsdp->_LastLineTerminated):
                   (_nextline == NULL || vbuf_ ||
                    (wlsdp->_ReturnOnLastLine && wlsdp->_LastLineTerminated));

    // If in window mode and encoded as vertical cursor move down
    iMonitor(_escaped_monitor, _escaped_handle, &_Escaped);
    linedir_ = 1;

    if (sdp && _Escaped &&
        wlsdp->_ReturnOnLastLine && wlsdp->_LastLineTerminated)
      status_ = 1;
    else
      status_ = (sdp && sdp->_InWinMode) ? (sdp->_NextLineReady ? linedir_:0):linedir_;

    VmoveDone_ = true;
  }

  // Action: GO_DOWN, GO_NEXT
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, NEXT) || (DnKeyDone_=GetKeyMap().KeyMatch(key, DOWN)))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == 9 || (DnKeyDone_=(key == KEY_DOWN)))
#endif
  {
    // If in window mode and encoded as vertical cursor move down
    linedir_ = 1;
    status_ = (DnKeyDone_ && sdp && sdp->_InWinMode) ? (sdp->_NextLineReady ? linedir_:0):linedir_;
    if (DnKeyDone_)
      VmoveDone_ = true;
  }

  // Action: GO_UP
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, UP))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_UP)  
#endif
  {
    // If in window mode and encoded as vertical cursor move up
    linedir_ = -1;
    status_ = (sdp && sdp->_InWinMode) ? (sdp->_PrevLineReady ? linedir_:0):linedir_;
    VmoveDone_ = true;
  }

  // Action: GO_BACK, GO_LEFT (Note Plural Actions For Single Key)
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, BACK) || GetKeyMap().KeyMatch(key, LEFT))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_BACKSPACE || key == KEY_LEFT)
#endif
  {
    coldir_ = -1;
    status_ = (_index <= 0 && zerotozero) ? ((!vbuf_ && !inwbox_) ? coldir_:0):0;
  }

  // Action: GO_RIGHT (Note Plural Actions For Single Key)
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, RIGHT) || IsUserKey(key))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_RIGHT || IsUserKey(key))
#endif
  {
    coldir_ = 1;
    status_ = (_index >= _len) ? ((!vbuf_ && !inwbox_) ? coldir_:0):0;
  }

  // Otherwise, no action
  else
    coldir_ = linedir_ = status_ = 0;

  if (_Escaped && sdp->_parent)
    sdp->_parent->SetEscaped(true);

  if (sdp && inwbox_ && VmoveDone_ && status_ != 0 &&
         (sdp->_InWinMode || Debug_) &&
         (_Escaped || (sdp->_NextLineReady && status_ == 1) ||
                      (sdp->_PrevLineReady && status_ == -1)))
  {
    ((WindowLine*)_Owner)->TransferBoxPosToNextLine(_Escaped ? 0:status_);
    TransferDone_ = true;
  }
  else if (!sdp->_NextLineReady && status_ == 1)
    status_ = vbuf_ ? 1:0;
  else if (!sdp->_PrevLineReady && status_ == -1)
    status_ = vbuf_ ? -1:0;

  if (linedir_ != 0 && !TransferDone_ &&
      (_Escaped || (linedir_ == 1 && !sdp->_NextLineReady) ||
                   (linedir_ == -1 && !sdp->_PrevLineReady)))
  {
    SaveBoxPos();
    ((WindowLine*)_Owner)->ShowAll();
    RestoreBoxPos();
  }

  return status_;
}

/****************************************************************************/
// PURPOSE:
//   Get line input procedure
//
// PRE:
//   The original input buffer
//
// POST:
//   Gets input from the keyboard and place it in the input buffer until
//   a termination is press.
//
int WindowLineBuffer::GetLine(int status_)
{
  int lastindex_;
  int zerotozero_;

  ResetInputState(status_);

  cMonitor(_buffer_monitor, _buffer_handle, _buffer);
  iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
  iMonitor(_active_monitor, _active_handle, &_TextWidgetActive);

  for (;;)
  {
    GetKey();
    lastindex_ = _index;

    if (_extended)
      PutExtKey(_extkey);
    else
      PutStdKey(_extkey);

    zerotozero_ = !lastindex_ && !_index && !_WrapChars;
    DetermineWrapChars();

    if (status_ = ShouldEnd(_extkey, zerotozero_))
      break;
  }

  return status_;
}

/****************************************************************************/
void WindowLineBuffer::ResetInputState(int status_)
{
  _Escaped = FALSE;
  _index = (status_ == -1) ? ::SafeStrLen(_buffer):0;

  if (status_ == -1 && ::SafeStrLen(_buffer) >= _len && _len > 0)
    _index = _len - 1;

  iMonitor(_escaped_monitor, _escaped_handle, &_Escaped);
  iMonitor(_index_monitor, _index_handle, &_index);
}

/****************************************************************************/
// PURPOSE:
//   Read input procedure
//
// POST:
//   Calls procedure to Show the contents of the input buffer as default
//   input to the Prompt box then calls GetLine to get additional input.
//   from the keyboard.
//
int WindowLineBuffer::ReadInput(int status_)
{
  ShowInput(status_);
  return GetLine(status_);
}

/****************************************************************************/
// PURPOSE:
//   Method to Erase the input
//
// POST:
//   Erases the internal buffer by NULL padding entire buffer.
//
void WindowLineBuffer::Erase()
{
  bool enterb_ = IsEnterable();
  bool LwAllowed_ = enterb_ ? ((WindowLine*)_Owner)->LineWrapAllowed():0;
  bool ShowChained_ = enterb_ ? ((WindowLine*)_Owner)->ShowChainedBoxes():1;

  if (!IsEnterable() || _bufferoffset)
    return;

  if (!IsActive())
    return;

  if (_nextline)
  {
    if (!_nextline->_buffer)
      if (_nextline->BuffersCleared())
        _nextline->RestoreUnusedBuffers();
  
    _nextline->Erase();
  }

  if (::SafeStrLen(_buffer))
  {
    if (!_Hidden && ((!_BufChainNum || !LwAllowed_) || ShowChained_))
    {
      _InFocus = TRUE;
      iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

      if (UsesColor())
        SetupColors(ColorInfo::BACKGROUND_COLOR);
      else
        UseColor(FALSE);

      ResetPrompt();
      _InFocus = FALSE;
      iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
    }

    int max = VirtualBufferOffset() ? _maxbuf:_len;
    int index;

    if (_bufferalloc && _buffer)
      for (index = 0; index < max; index++)
        _buffer[index] = '\0';

    if (_joinbufalloc)
      for (index = 0; index < max; index++)
        _joinbuf[index] = NOJOIN_CHAR;

    if (_joinbufalloc && index == _maxbuf)
      _joinbuf[index] = '\0';
  }
}

/****************************************************************************/
// PURPOSE:
//   reset user Prompt method
//
// POST:
//   The user Prompt is setup at the specified position and the original
//   contents of the line is Cleared
//
void WindowLineBuffer::ResetPrompt()
{
  int WrapIndex_ = 0;
  int WrapMax_ = 0;
  int Equal_ = RelationalEnum::EQUAL;

  bool StartMet_ = false;
  bool EndMet_ = false;

  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_, NULL, NULL,
                                 &WrapIndex_, &WrapMax_);
  bool LwAllowed_ = enterb_ ? ((WindowLine*)_Owner)->LineWrapAllowed():0;
  bool ShowChained_ = enterb_ ? ((WindowLine*)_Owner)->ShowChainedBoxes():0;

  if (!IsEnterable() || _bufferoffset ||
      (_BufChainNum && LwAllowed_ && !ShowChained_))
    return;

  if (EndBrksActive())
  {
    GOTOXY(_xpos + ::SafeStrLen(_msg), _ypos);
    PrintBlanks(_len + EndBrksLength(false));
    GOTOXY(_xpos + ::SafeStrLen(_msg), _ypos);

    StartMet_ = (vbuf_ && _inboxoffset == 0) || (inwbox_ && _inwboxcnt == 0);
    EndMet_ = vbuf_ ?
                ((WrapIndex_ == WrapMax_ && WrapMax_ == 0 && _inboxoffset == 0) ||
                 (_inboxoffset > WrapMax_ && WrapIndex_ == _inboxoffset) ||
                 (_inboxoffset == WrapMax_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, false, false))):
              inwbox_ ?
                ((WrapIndex_ == WrapMax_ && WrapMax_ == 0 && _inwboxcnt == 0) ||
                 (_inwboxcnt > WrapMax_ && WrapIndex_ == _inwboxcnt) ||
                 (_inwboxcnt == WrapMax_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, false, false))):0;

    if (vbuf_)
    {
      if (EndMet_ &&
          (AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, false, false) &&
           WrapMax_ == 0 && _inboxoffset == 0) ||
          (!(_inboxoffset > WrapMax_ && WrapIndex_ == _inboxoffset) &&
           !(_inboxoffset == WrapMax_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, false, false))))
        EndMet_ = !IsChainable(true);
    }
    else if (inwbox_)
    {
      if (EndMet_ &&
          (AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, false, false) &&
           WrapMax_ == 0 && _inwboxcnt == 0) ||
          (!(_inwboxcnt > WrapMax_ && WrapIndex_ == _inwboxcnt) &&
           !(_inwboxcnt == WrapMax_ && AtMaxWrap(WrapIndex_, WrapMax_, 0, Equal_, false, false))))
        EndMet_ = !IsChainable(true);
    }

    if (StartMet_ && (vbuf_ || inwbox_))
    {
      SetupColors(ColorInfo::HIGHLIGHT_COLORS);
      PUTCH(' ');
    }    
    else
    {
      SetupColors(CONTMORE_HFG, CONTMORE_HBG);
      PUTCH(_brackets[0]);
      SetupColors(ColorInfo::HIGHLIGHT_COLORS);
    }
    
    GOTOXY(wherex() + _len, _ypos);

    if (EndMet_ && (vbuf_ || inwbox_))
    {
      SetupColors(ColorInfo::HIGHLIGHT_COLORS);
      PUTCH(' ');
    }
    else
    {
      SetupColors(CONTMORE_HFG, CONTMORE_HBG);
      PUTCH(_brackets[1]);
      SetupColors(ColorInfo::HIGHLIGHT_COLORS);
    }

    GOTOXY(_xpos + ::SafeStrLen(_msg) + EndBrksLength(true), _ypos);
  }
  else
  {
    GOTOXY(_xpos + ::SafeStrLen(_msg), _ypos);
    PrintBlanks(_len + EndBrksLength(false));
    GOTOXY(_xpos + ::SafeStrLen(_msg), _ypos);
  }
}

/****************************************************************************/
// PURPOSE:
//   Show Prompt message method
//
// POST:
//   Goes to the specified coordinates and Shows the Prompt message
//   to the screen
//
void WindowLineBuffer::ShowMessage()
{
  bool enterb_ = IsEnterable();
  bool LwAllowed_ = enterb_ ? ((WindowLine*)_Owner)->LineWrapAllowed():0;
  bool ShowChained_ = enterb_ ? ((WindowLine*)_Owner)->ShowChainedBoxes():0;

  if (!IsEnterable() || _bufferoffset ||
      (_BufChainNum && LwAllowed_ && !ShowChained_))
    return;

  GOTOXY(_xpos, _ypos);

  if (_msg)
    CPUTS(_msg);
}

/****************************************************************************/
// PURPOSE:
//   Show input method
//
// POST:
//   Shows the Prompt box and Shows the original contents of the input
//   buffer within the box
//
void WindowLineBuffer::ShowInput(int status_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  bool vbuf_ = false;
  bool inwbox_ = false;
  bool enterb_ = ExtendedBoxMode(vbuf_, inwbox_);

  bool LwAllowed_ = enterb_ ? ((WindowLine*)_Owner)->LineWrapAllowed():0;
  bool ShowChained_ = inwbox_ ? ((WindowLine*)_Owner)->ShowChainedBoxes():0;

  if (sdp)
    sdp->_InWinMode = enterb_ ?
        ((WindowLine*)_Owner)->InWindowMode(&sdp->_NextLineReady,
                                            &sdp->_PrevLineReady):false;

  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;
  int BoxCount_ = inwbox_ ? ((WindowLine*)_Owner)->GetBoxCount(true):0;
  int WrapIndex_ = enterb_ ? ((WindowLine*)_Owner)->GetWrapAroundIndex():0;
  int endpt_ = (!vbuf_ && !inwbox_ && _nextline) ? _len:(_len-1);  
  int index_ = 0;
  int boxdiff_ = 0;
  int bufdex_ = 0;

  if (!IsEnterable() || _bufferoffset ||
      (_BufChainNum && LwAllowed_ && !ShowChained_))
  {
    if (_buffer && ShouldTrimWs() && inwbox_ &&
        (WrapIndex_ == 0 && BufferChainPosition() == BoxCount_-1))
    {
      if (_buffer && _bufferalloc)
        _buffer = RemoveTrailSpaceChars(_buffer, true);

      if (_joinbufalloc)
      {
        index_ = strlen(_buffer);        
        while (index_ <= _len)
          _joinbuf[index_++] = NOJOIN_CHAR;

        _joinbuf[_maxbuf] = 0;
      }
    }
  
    return;
  }
    
  if (_len)  
  {
    int startpos_ = 0;
    int endpos_ = 0;
    char lastchar_ = 0;
    char* strptr_ = NULL;
    int lastnwspos_ = 0;

    ResetPrompt();
    if (_buffer && ShouldTrimWs() &&
        (vbuf_ || !inwbox_ ||
         (WrapIndex_ == 0 && BufferChainPosition() == BoxCount_-1)))
    {
      if (_buffer && _bufferalloc)
        _buffer = RemoveTrailSpaceChars(_buffer, true);

      if (_joinbufalloc)
      {
        index_ = strlen(_buffer);        
        while (index_ <= _len)
          _joinbuf[index_++] = NOJOIN_CHAR;

        _joinbuf[_maxbuf] = 0;
      }
    }

    if (vbuf_)
    {
      VbufStrLen(&lastnwspos_, &startpos_, &endpos_, &lastchar_);
      _buffer[endpos_] = 0;

      strptr_ = StrRetrieve(true, true, CalcBackShift(_buffer));
      if (strptr_ && strptr_[0])
        CPUTS(&strptr_[startpos_]);

      _buffer[endpos_] = lastchar_;
    }
    else
    {
      if (!_buffer)
        if (BuffersCleared())
          RestoreUnusedBuffers();

      strptr_ = StrRetrieve(true, true, CalcBackShift(_buffer));
      if (strptr_ && strptr_[0])
        CPUTS(strptr_);
    }

    if (_nextline && !VirtualBufferOffset())
      _nextline->ShowInput();

    GOTOXY(_xpos + ::SafeStrLen(_msg) + EndBrksLength(true), _ypos);
    boxdiff_ = _refline ? _refline->BoxDiff():0;
    bufdex_ = _refline ? _refline->BufferIndex():0;

    if (status_ == -1)
    {
      if (inwbox_ && _BufChainNum == 0)
      {
        #if (WINLINE_ONLYCWSCHR & WINLINE_WSENDMOVE)
          lastnwspos_ = HasNonWsChars(_buffer);
        #else
          lastnwspos_ =
            (_inwboxcnt > 0 && AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, false) &&
             _inwboxcnt >= WrapMax_ && WrapIndex_ <= _inwboxcnt) ?
                 HasNonNullChars(_buffer):
                 HasNonWsChars(_buffer);
        #endif
      }
      else if (!vbuf_)
        lastnwspos_ = ::SafeStrLen(_buffer);

      if (lastnwspos_ > endpt_)
        GOTOXY(wherex() + endpt_, _ypos);
      else
      {
        if (boxdiff_ + lastnwspos_ > endpt_)
          GOTOXY(wherex() + endpt_, _ypos);
        else
          GOTOXY(wherex() + boxdiff_ + lastnwspos_, _ypos);
      }

      if (lastnwspos_ >= _len && _len > 0 &&
          !sdp->_BoxPosTrans && _BufChainNum == 0)
      {
        _index = (lastnwspos_ % _len) + boxdiff_;
        
        if (vbuf_)
        {
          _inboxoffset = lastnwspos_ - _len;
          
          if (_inboxoffset < 0)
            _inboxoffset = 0;
          else if (boxdiff_)
          {
            _index = bufdex_ + boxdiff_;
            
            if (_index > endpt_)
              _index = endpt_;
          }
          else
            _index = endpt_;
        }
        else if (inwbox_)
        {
          _inwboxcnt = lastnwspos_ - _len;

          if (_inwboxcnt < 0)
            _inwboxcnt = 0;
          else if (boxdiff_)
          {
            _index = bufdex_ + boxdiff_;

            if (_index > endpt_)
              _index = endpt_;
          }
          else
            _index = endpt_;
        }
        else if (lastnwspos_ > _len)
          _index = endpt_ - 1;
        
        iMonitor(_index_monitor, _index_handle, &_index);
        sdp->_BoxPosSaved = false;
      }
      else
        sdp->_BoxPosSaved =
        sdp->_BoxPosTrans = false;

      if (vbuf_ && _inboxoffset)
      {
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::HOME).SCAN + 256);
        #elif (defined(__linux__) | defined(__unix__))
          PutExtKey(KEY_HOME);
        #endif
      }
      else if (inwbox_ && _inwboxcnt && sdp && sdp->_InWinMode)
      {
        #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
          PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::END).SCAN + 256);
        #elif (defined(__linux__) | defined(__unix__))
          PutExtKey(KEY_END);
        #endif
      }
    }
  }
}

/****************************************************************************/
// PURPOSE:
//   Prompt user method
//
// POST:
//   Shows the Prompt message and the Prompt box then Prompts user for
//   additional input
//
int WindowLineBuffer::Prompt(int status_)
{
  bool enterb_ = IsEnterable();
  bool LwAllowed_ = enterb_ ? ((WindowLine*)_Owner)->LineWrapAllowed():0;
  bool ShowChained_ = enterb_ ? ((WindowLine*)_Owner)->ShowChainedBoxes():0;

  if (!IsEnterable() || _bufferoffset ||
      (_BufChainNum && LwAllowed_ && !ShowChained_))
    return status_;

  if (!IsActive())
    return status_;

  if (_len)
  {
    _InFocus = TRUE;
    iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

    SetupColors(ColorInfo::HIGHLIGHT_COLORS);
    ShowMessage();
    status_ = ReadInput(status_);
    Show();

    _InFocus = FALSE;
    iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
    _Hidden = FALSE;
    iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
  }

  return status_;
}

/****************************************************************************/
// PURPOSE:
//   Shows Prompt box method
//
// POST:
//   Shows Prompt message and Prompt box with contents of input buffer
//
void WindowLineBuffer::Show()
{
  bool enterb_ = IsEnterable();
  bool LwAllowed_ = enterb_ ? ((WindowLine*)_Owner)->LineWrapAllowed():0;
  bool ShowChained_ = enterb_ ? ((WindowLine*)_Owner)->ShowChainedBoxes():0;

  if (!IsEnterable() || _bufferoffset ||
      (_BufChainNum && LwAllowed_ && !ShowChained_))
    return;

  if (!IsActive())
    return;

  if (_nextline && !VirtualBufferOffset())
    _nextline->Show();

  _InFocus = TRUE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

  SetupColors(ColorInfo::NORMAL_COLORS);
  ShowMessage();
  ShowInput(1);

  _InFocus = FALSE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
  _Hidden = FALSE;
  iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
}

/****************************************************************************/
void WindowLineBuffer::ShowInput()
{
  bool enterb_ = IsEnterable();
  bool LwAllowed_ = enterb_ ? ((WindowLine*)_Owner)->LineWrapAllowed():0;
  bool ShowChained_ = enterb_ ? ((WindowLine*)_Owner)->ShowChainedBoxes():0;

  if (!IsEnterable() || _bufferoffset ||
      (_BufChainNum && LwAllowed_ && !ShowChained_))
    return;

  _InFocus = TRUE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

  ShowInput(1);

  _InFocus = FALSE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
  _Hidden = FALSE;
  iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
}

/****************************************************************************/
// PURPOSE:
//   Hide Prompt box method
//
// POST:
//   goes to the specified coordinates of the Prompt box then Clears the
//   line containing the Prompt box.
//
void WindowLineBuffer::Hide()
{
  if (!_Owner)
    return;
  
  bool enterb_ = IsEnterable();  
  bool LwAllowed_ = enterb_ ? ((WindowLine*)_Owner)->LineWrapAllowed():0;
  bool ShowChained_ = enterb_ ? ((WindowLine*)_Owner)->ShowChainedBoxes():0;

  if (!IsEnterable() || _bufferoffset ||
      (_BufChainNum && LwAllowed_ && !ShowChained_))
    return;

  if (!IsActive())
    return;

  if (_nextline && !VirtualBufferOffset())
    _nextline->Hide();

  _InFocus = TRUE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

  if (UsesColor())
    SetupColors(ColorInfo::BACKGROUND_COLOR);
  else
    UseColor(FALSE);

  GOTOXY(_xpos, _ypos);
  PrintBlanks(::SafeStrLen(_msg) + _len + EndBrksLength(false));
  GOTOXY(_xpos, _ypos);

  _InFocus = FALSE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
  _Hidden = TRUE;
  iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
}

/****************************************************************************/
void WindowLineBuffer::SetEnterable(Boolean Flag_)
{
  _Enterable = _bufferoffset ? FALSE:Flag_;
}

/****************************************************************************/
void WindowLineBuffer::SetSelectable(Boolean Flag_)
{
  _Selectable = Flag_;
}

/****************************************************************************/
int WindowLineBuffer::GetHotKey() const
{
  return 0;
}

/****************************************************************************/
int WindowLineBuffer::GetShortCut() const
{
  return 0;
}

/****************************************************************************/
int WindowLineBuffer::GetHotKeyType() const
{
  return 0;
}

/****************************************************************************/
void WindowLineBuffer::Select()
{
  if (!IsSelected())
    if (_maxbuf)
      *_buffer = BLANK;
}

/****************************************************************************/
void WindowLineBuffer::Deselect()
{
  if (_maxbuf)
    *_buffer = 0;
}

/****************************************************************************/
void WindowLineBuffer::SetEscaped(bool Flag_)
{
  _Escaped = Flag_;

  if (_refline && _BufChainNum == 0)
    _refline->SetEscaped(Flag_);

  if (_nextline)
    _nextline->SetEscaped(Flag_);
}

/****************************************************************************/
void WindowLineBuffer::Activate()
{
  _TextWidgetActive = TRUE;
  iMonitor(_active_monitor, _active_handle, &_TextWidgetActive);

  if (_refline && _BufChainNum == 0)
    _refline->Activate();

  if (_nextline)
  {
    if (IsEmpty() && _nextline->BuffersCleared())
      _nextline->RestoreUnusedBuffers();
  
    _nextline->Activate();
  }
}

/****************************************************************************/
void WindowLineBuffer::Deactivate()
{
  _TextWidgetActive = FALSE;
  iMonitor(_active_monitor, _active_handle, &_TextWidgetActive);

  if (_refline && _BufChainNum == 0)
    _refline->Deactivate();

  if (_nextline)
  {
    if (!IsEmpty())
      _nextline->ClearUnusedBuffers();
    else if (_nextline->BuffersCleared())
      _nextline->RestoreUnusedBuffers();

    _nextline->Deactivate();
  }
}

/****************************************************************************/
void WindowLineBuffer::DecrBoxIndex()
{
  bool enterb_ = IsEnterable();
  bool vbuf_ = enterb_ && VirtualBufferOffset();

  if (vbuf_)
  {
    if (_inboxoffset - 1 >= 0)
      _inboxoffset--;
  }
  else
  {
    if (_inwboxcnt - 1 >= 0)
      _inwboxcnt--;
  }
}

/****************************************************************************/
void WindowLineBuffer::IncrBoxIndex()
{
  int boxcnt_ = ((WindowLine*)_Owner)->GetBoxCount(true);
  int maxcnt_ = _len * boxcnt_;
  bool enterb_ = IsEnterable();
  bool vbuf_ = enterb_ && VirtualBufferOffset();

  if (vbuf_)
  {
    if (_inboxoffset + 1 <= _maxbuf || _maxbuf == -1)
      _inboxoffset++;
  }
  else
  {
    if (_inwboxcnt + 1 <= maxcnt_ || maxcnt_ <= 0)
      _inwboxcnt++;
  }
}

/****************************************************************************/
void WindowLineBuffer::ResetBoxIndex()
{
  bool enterb_ = IsEnterable();
  bool vbuf_ = enterb_ && VirtualBufferOffset();

  if (vbuf_)
    _inboxoffset = 0;
  else
    _inwboxcnt = 0;
}

/****************************************************************************/
void WindowLineBuffer::SetBoxIndex(int Val_)
{
  bool enterb_ = IsEnterable();
  bool vbuf_ = enterb_ && VirtualBufferOffset();

  if (vbuf_)
  {  
    if (0 <= Val_ && Val_ <= _maxbuf || (Val_ >= 0 && _maxbuf == -1))
      _inboxoffset = Val_ - _index;
  }
  else
  {
    if (0 <= Val_ || (Val_ >= 0 && _maxbuf == -1))
      _inwboxcnt = Val_ - _index;
  }
}

/****************************************************************************/
Boolean WindowLineBuffer::IsEmpty() const
{
  return (!_buffer || !::SafeStrLen(_buffer));
}

/****************************************************************************/
Boolean WindowLineBuffer::IsEscaped() const
{
  return _Escaped;
}

/****************************************************************************/
Boolean WindowLineBuffer::IsActive() const
{
  return _TextWidgetActive;
}

/****************************************************************************/
Boolean WindowLineBuffer::IsEnterable() const
{
  return (_Enterable && _bufferoffset <= 0);
}

/****************************************************************************/
Boolean WindowLineBuffer::IsSelectable() const
{
  return _Selectable;
}

/****************************************************************************/
Boolean WindowLineBuffer::IsSelected() const
{
  return (_buffer && ::SafeStrLen(_buffer));
}

/****************************************************************************/
Boolean WindowLineBuffer::IsValid() const
{
  return (!(IsEmpty() && _noempty) && IsValidLength());
}

/****************************************************************************/
Boolean WindowLineBuffer::IsInFocus() const
{
  return _InFocus;
}

/****************************************************************************/
char WindowLineBuffer::LastKeyRead(int* Ext_) const
{
  if (Ext_)
    *Ext_ = _extended;

  return _key;
}

/****************************************************************************/
int WindowLineBuffer::LastExtKeyRead() const
{
  return _extkey;
}

/****************************************************************************/
const char* WindowLineBuffer::RawStrBuffer(int index_) const
{
  const WindowLineBuffer::SharedChainData* sdp = ConstSharedData();
  if (!sdp)
    return NULL;
    
  if (!_buffer)
    return sdp->_nullcharstr;

  const char* ret_ = _buffer;
  int len_ = strlen(_buffer);

  if (index_ > len_)
    index_ = len_;
  else if (index_ < 0)
    index_ = 0;

  return &ret_[index_];
}

/****************************************************************************/
int WindowLineBuffer::RawStrBufLen(int index_) const
{
  return ::SafeStrLen(RawStrBuffer(index_));
}

/****************************************************************************/
char* WindowLineBuffer::RawStrBuffer(int index_, bool Revert_, int* shift_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return NULL;

  char* ret_ = Revert_ ? StrRetrieve(false, Revert_):_buffer;
  if (!ret_)
    return sdp->_nullcharstr;

  int len_ = strlen(ret_);

  if (index_ > len_)
    index_ = len_;
  else if (index_ < 0)
    index_ = 0;

  if (shift_)
    *shift_ = index_;

  return &ret_[index_];
}

/****************************************************************************/
int WindowLineBuffer::RawStrBufLen(int index_, int* shift_)
{
  return ::SafeStrLen(RawStrBuffer(index_, false, shift_));
}

/****************************************************************************/
int WindowLineBuffer::GetRefLineShift()
{
  int len_ = SafeStrLen(_buffer);
  int BackShift_ = CalcBackShift(_buffer);
  int reflinelen_ = _refline ? _refline->ExtendedIndex():0;

  if (BackShift_ > 0)
    if (BackShift_ > reflinelen_)
      reflinelen_ = 0;
    else
      reflinelen_ -= BackShift_;

  if (reflinelen_ > len_)
    reflinelen_ = len_;

  return reflinelen_;
}

/****************************************************************************/
int WindowLineBuffer::CalcBackShift(const char* retstr_)
{
  // calculate number of valid back spaces to subtract from refline shift
  // that is equal to the value of refline extended index since characters
  // in the starting buffer is pulled to adjoining chained buffers
  //
  // example: [buf0: b,b,b,_,_] [buf1: c,c]
  //
  // in the above situation the method would return 2 since c,c are the two
  // characters pulled from buf0 to the adjoining buffer buf1
  //
  WindowLineBuffer* nextptr_ = NULL;

  int WrapIndex_ = 0;
  int WrapMax_ = 0;
  int BackShift_ = 0;
  int RetStrLen_ = 0;
  int NextLen_ = 0;
  int ShiftedSpace_ = 0;
  int GreaterOrEqual_ = RelationalEnum::GREATER_THAN_OR_EQUAL;

  bool AtDexEnd_ = false;
  bool AtEndPt_ = false;
  bool PastEndPt_ = false;
  bool enterb_ = IsEnterable();
  bool ReEntryValid_ = true;

  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return NULL;

  if (!sdp->_reentryptr)
    sdp->_reentryptr = this;
  else
    ReEntryValid_ = sdp->_reentryptr != this;

  if (!ReEntryValid_)
  {
    sdp->_reentryptr = NULL;
    return 0;
  }

  if (_Owner)
  {
    WrapIndex_ = enterb_ ? ((WindowLine*)_Owner)->GetWrapAroundIndex():0;
    WrapMax_ = enterb_ ? ((WindowLine*)_Owner)->GetMaxWrapAround():0;
    AtDexEnd_ = AtMaxWrap(WrapIndex_, WrapMax_, 0, GreaterOrEqual_, false, true);
  }

  AtEndPt_ = IsAtEndPt(&PastEndPt_, _refline);
  RetStrLen_ = ::SafeStrLen(retstr_);
  NextLen_ = _nextline ? _nextline->RawStrBufLen(0):0;

  if (PastEndPt_ && AtDexEnd_ && _MovedToEnd &&
      RetStrLen_ < _len && NextLen_)
  {
    ShiftedSpace_ = NextLen_ - (_len - RetStrLen_);
    nextptr_ = _nextline->_nextline;
    
    while (ShiftedSpace_ < 0 && nextptr_)
    {
      ShiftedSpace_ += nextptr_->RawStrBufLen(0);
      nextptr_ = nextptr_->_nextline;
    }

    if (ShiftedSpace_ < 0)
      ShiftedSpace_ += _len - RetStrLen_;
    else
      ShiftedSpace_ = _len - RetStrLen_;

    BackShift_ = ShiftedSpace_;
  }

  if (sdp->_reentryptr)
    sdp->_reentryptr = NULL;

  return BackShift_;
}

/****************************************************************************/
char* WindowLineBuffer::StrRetrieve(bool ExtRefLine_, bool Revert_, int BackShift_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (!sdp)
    return NULL;

  if (!_buffer)
    return sdp->_nullcharstr;

  size_t len_ = strlen(_buffer);

  if (!_retbuf || len_ >= _retbufsize)
  {
    if (_retbufalloc && _retbuf)
    {
      ChainRemoveRetBuf(_retbuf);
      delete[] _retbuf;
    }

    _retbuf = NULL;
    if (len_ >= _retbufsize)
      _retbufsize = len_ + 10;

    _retbuf = NewCString(_retbufsize, 0, true);
    _retbufalloc = true;

    if (BufferTypeCleared(RETURN_BUFTYPE) && _retbufalloc)
      SetBufferTypeCleared(RETURN_BUFTYPE, false);
  }

  strcpy(_retbuf, _buffer);
  char* ret_ = Revert_ ? Revert(_retbuf):_retbuf;
  len_ = strlen(ret_);
  size_t reflinelen_ = (ExtRefLine_ && _refline) ?
                           _refline->ExtendedIndex():0;

  if (ExtRefLine_ && BackShift_ > 0)
    if (size_t(BackShift_) > reflinelen_)
      reflinelen_ = 0;
    else
      reflinelen_ -= BackShift_;

  if (reflinelen_ > len_)
    reflinelen_ = len_;

  return (ret_ + reflinelen_);
}

/****************************************************************************/
char WindowLineBuffer::ChrRetrieve(int Index_)
{
  char retch_ = 0;
  if (_buffer && 0 <= Index_ && Index_ < ::SafeStrLen(_buffer))
    retch_ = _buffer[Index_];

  retch_ = Revert(retch_);
  return retch_;
}

/****************************************************************************/
Boolean WindowLineBuffer::IsOwner(ControlWindow* Pwin_) const
{
  return (_Owner == Pwin_);
}

/****************************************************************************/
void WindowLineBuffer::ChainSetOwner(ControlWindow* Pwin_, TextControl* TxtCtrl_)
{
  WindowLineBuffer* ptr = (WindowLineBuffer*)TxtCtrl_;
  WindowLine* wlptr = (WindowLine*)Pwin_;
  WindowLine* tcwlptr = (WindowLine*)(ptr->_Owner);

  if (TxtCtrl_ && tcwlptr && tcwlptr != wlptr &&
      !TxtCtrl_->IsDummyLine() && !wlptr->HasControlArrays(tcwlptr))
    if (TxtCtrl_->BufferChainPosition() == 0)
      ptr->_Owner->ReplaceControl(TxtCtrl_);
    else
      ptr->_Owner->RemoveControl(TxtCtrl_);

  ptr->_Owner = Pwin_;

  if (ptr && ptr->_nextline && !TxtCtrl_->IsDummyLine())
    ChainSetOwner(Pwin_, ptr->_nextline);

  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (sdp)
    sdp->SetOwner((WindowLine*)Pwin_);
}

/****************************************************************************/
void WindowLineBuffer::SetOwner(ControlWindow* Pwin_, TextControl* TxtCtrl_)
{
  WindowLineBuffer* ptr = (WindowLineBuffer*)TxtCtrl_;
  WindowLine* wlptr = (WindowLine*)Pwin_;
  WindowLine* tcwlptr = (WindowLine*)(ptr->_Owner);

  if (TxtCtrl_ && tcwlptr && tcwlptr != wlptr &&
      !TxtCtrl_->IsDummyLine() && !wlptr->HasControlArrays(tcwlptr))
    if (TxtCtrl_->BufferChainPosition() == 0)
      ptr->_Owner->ReplaceControl(TxtCtrl_);
    else
      ptr->_Owner->RemoveControl(TxtCtrl_);

  ptr->_Owner = Pwin_;

  WindowLineBuffer::SharedChainData* sdp = SharedData();
  if (sdp)
    sdp->SetOwner((WindowLine*)Pwin_);
}

/****************************************************************************/
void WindowLineBuffer::CreateReturnBuffer(int Size_)
{
  int BoxInLen_ = _Owner ? ((WindowLine*)_Owner)->BoxInputLength():0;
  int AllocSize_ = (Size_ > 20)     ? Size_:
                   (BoxInLen_ > 20) ? BoxInLen_:
                   (_len > 20)      ? _len:20;

  _retbuf = NewCString(AllocSize_, 0, true);
  _retbufsize = AllocSize_;
  _retbufalloc = _retbuf != NULL;

  if (_retbufalloc)
  {
    SetBufferTypeCleared(RETURN_BUFTYPE, false);

    if (_nextline)
      _nextline->SetReturnBuffer(_nextline, _retbuf, false, _retbufsize);
  }
}

/****************************************************************************/
void WindowLineBuffer::SetReturnBuffer(WindowLineBufferType* Lbuf_, char* RetBufPtr_, bool Alloc_, int Size_)
{
  bool bufdeleted_ = false;
  bool bufcleared_ = false;

  if (Lbuf_)
    if (this != Lbuf_)
      Lbuf_->SetReturnBuffer(Lbuf_, RetBufPtr_, Alloc_, Size_);
    else if (RetBufPtr_ && _retbuf != RetBufPtr_)
    {
      if (!BuffersCleared())
      {
        if (_retbufalloc)
        {
          ChainRemoveRetBuf(_retbuf);
          delete[] _retbuf;
          bufdeleted_ = true;
        }

        _retbuf = NULL;
      }

      if (BuffersCleared() && BufferTypeCleared(RETURN_BUFTYPE) &&
          (!_retbufalloc || !_retbuf))
      {
        if (!bufdeleted_ && _retbuf)
        {
          ChainRemoveRetBuf(_retbuf);
          delete[] _retbuf;
          bufdeleted_ = true;
        }
        
        _retbuf = NULL;
        bufcleared_ = true;
      }

      if (Alloc_)
      {
        if (Size_ <= 0)
          Size_ = strlen(RetBufPtr_);

        _retbuf = NewCString(Size_, 0, true);
        _retbufsize = Size_;
        _retbufalloc = _retbuf != NULL;

        if (_retbufalloc)
        {
          strcpy(_retbuf, RetBufPtr_);

          if (bufcleared_)
            SetBufferTypeCleared(RETURN_BUFTYPE, false);
        }

        if (_nextline)
          _nextline->SetReturnBuffer(_nextline, _retbuf, Alloc_, Size_);
      }
      else
      {
        _retbufalloc = false;
        if (Size_ <= 0)
          Size_ = strlen(RetBufPtr_);

        _retbuf = RetBufPtr_;
        _retbufsize = Size_;

        if (bufcleared_)
          SetBufferTypeCleared(RETURN_BUFTYPE, false);

        if (_nextline)
          _nextline->SetReturnBuffer(_nextline, RetBufPtr_, Alloc_, Size_);
      }
    }
}

/****************************************************************************/
void WindowLineBuffer::SetBufferChainNum(WindowLineBufferType* Lbuf_, int Index_)
{
  if (Lbuf_)
    if (this != Lbuf_)
      Lbuf_->SetBufferChainNum(Lbuf_, Index_);
    else
    {
      _BufChainNum = Index_;
      
      if (_nextline)
        _nextline->SetBufferChainNum(_nextline, Index_+1);
    }
}

/****************************************************************************/
void WindowLineBuffer::SetWindowLineNum(int Index_)
{
  WindowLineBuffer::SharedChainData* sdp = SharedData();

  if (sdp)
    sdp->_WinLineNum = Index_;
}

/****************************************************************************/
int WindowLineBuffer::InputLength() const
{
  return _len;
}

/****************************************************************************/
void WindowLineBuffer::SetupColors(int Fg_, int Bg_)
{
  _ColorInfo.SetupColors(Fg_, Bg_);
  // GetDefaultColorInfo().SetupColors(Fg_, Bg_);
}

/****************************************************************************/
void WindowLineBuffer::SetupColors(int Mode_)
{
  if (!_ColorInfo.SetupColors(Mode_))
    GetDefaultColorInfo().SetupColors(Mode_);
}

/****************************************************************************/
void WindowLineBuffer::UseColor(Boolean Flag_)
{
  if (_ColorInfo._UseColor)
    *_ColorInfo._UseColor = Flag_;
  else
    _ColorInfo._UseColor = new Boolean(Flag_);

  GetDefaultColorInfo().UseColor(Flag_);
}

/****************************************************************************/
void WindowLineBuffer::SetNormalColors(int FgCol_, int BgCol_)
{
  if (_ColorInfo._NormalColors)
  {
    _ColorInfo._NormalColors[0] = FgCol_;
    _ColorInfo._NormalColors[1] = BgCol_;
  }
  else
  {
    _ColorInfo._NormalColors = new int[2];
    SetNormalColors(FgCol_, BgCol_);
  }
}

/****************************************************************************/
void WindowLineBuffer::SetHighLightColors(int FgCol_, int BgCol_)
{
  if (_ColorInfo._HighLightColors)
  {
    _ColorInfo._HighLightColors[0] = FgCol_;
    _ColorInfo._HighLightColors[1] = BgCol_;
  }
  else
  {
    _ColorInfo._HighLightColors = new int[2];
    SetHighLightColors(FgCol_, BgCol_);
  }
}

/****************************************************************************/
void WindowLineBuffer::SetHotKeyColor(int Col_)
{
  if (_ColorInfo._HkColor)
    *_ColorInfo._HkColor = Col_;
  else
    _ColorInfo._HkColor = new int(Col_);
}

/****************************************************************************/
void WindowLineBuffer::SetBackGroundColor(int Col_)
{
  if (_ColorInfo._BackGroundColor)
    *_ColorInfo._BackGroundColor = Col_;
  else
    _ColorInfo._BackGroundColor = new int(Col_);
}

/****************************************************************************/
void WindowLineBuffer::UseDefaultColors()
{
  _ColorInfo.ClearColors();
  UseColor();
}

/****************************************************************************/
Boolean WindowLineBuffer::UsesColor() const
{
  return
  (
    _ColorInfo._UseColor ?
      *(_ColorInfo._UseColor):
      *(GetDefaultColorInfo()._UseColor)
  );
}

/****************************************************************************/
int* WindowLineBuffer::NormalColors() const
{
  return
  (
    _ColorInfo._NormalColors ?
      _ColorInfo._NormalColors:
      GetDefaultColorInfo()._NormalColors
  );
}

/****************************************************************************/
int* WindowLineBuffer::HighLightColors() const
{
  return
  (
    _ColorInfo._HighLightColors ?
      _ColorInfo._HighLightColors:
      GetDefaultColorInfo()._HighLightColors
  );
}

/****************************************************************************/
int WindowLineBuffer::HotKeyColor() const
{
  return
  (
    _ColorInfo._HkColor ?
      *(_ColorInfo._HkColor):
      *(GetDefaultColorInfo()._HkColor)
  );
}

/****************************************************************************/
int WindowLineBuffer::BackGroundColor() const
{
  return
  (
    _ColorInfo._BackGroundColor ?
      *(_ColorInfo._BackGroundColor):
      *(GetDefaultColorInfo()._BackGroundColor)
  );
}

/****************************************************************************/
// WindowRefLineInfo class definition
/****************************************************************************/
const int* InsertDetect(void* Owner_, WindowRefLineInfo*, const int* Ptr_)
{
  if (Owner_)
    ((WindowRefLineInfo*)Owner_)->NotifyInsertMode(Ptr_);

  return Ptr_;
}

/****************************************************************************/
// PURPOSE:
//   Prompt box default constructor
//
// PRE:
//   an uninitialize Prompt box
//
// POST:
//   a Prompt box is constructed and assigned the default values specified.
//   If the msg argument is not NULL then the Prompt message of the box
//   is assigned to the argument string.
//   The string buffer is Cleared to all 0.
//
WindowRefLineInfo::WindowRefLineInfo(int BufChainNum_, WindowLineBuffer* Parent_,
                                 ControlWindow* Pwin_, int Length_):
_Owner(Pwin_),
_Enterable(FALSE),
_Selectable(FALSE),

_parent((BufChainNum_ == 0) ? Parent_:NULL),
_movedone(false),
_haspendingmove(false),
_pendingmove(false),
_extindextoggled(false),

_parentindex(0),
_inboxoffset(0),
_inwboxcnt(0),
_inwbox(false),
_vbuf(false),
_maxbuf(-1),
_len(Length_ ? Length_:1),   // _len should never be zero
_minlen(1),                  // minimum length should be at least 1
_distance(0),
_index(0),
_extindex(0),
_boxpos(0),
_boxdiff(0),
_xpos(WHEREX()),
_ypos(WHEREY()),
_noempty(0),
_InsertMode(TRUE),
_Escaped(FALSE),
_Hidden(TRUE),
_TextWidgetActive(FALSE),
_InFocus(FALSE),
_SavedCursX(WHEREX()),
_SavedCursY(WHEREY()),
_BufChainNum(-99),
_WinLineNum(0),

// Internal data monitors
_insert_monitor(NULL),
_key_monitor(NULL),
_ext_monitor(NULL),
_index_monitor(NULL),
_escaped_monitor(NULL),
_hidden_monitor(NULL),
_active_monitor(NULL),
_infocus_monitor(NULL),
_inboxoffset_monitor(NULL),
_winlinenum_monitor(NULL),

// Handles to external objects, used by data monitors
_insert_handle(NULL),
_key_handle(NULL),
_ext_handle(NULL),
_index_handle(NULL),
_escaped_handle(NULL),
_hidden_handle(NULL),
_active_handle(NULL),
_infocus_handle(NULL),
_inboxoffset_handle(NULL),
_winlinenum_handle(NULL)
{
  if (_parent && BufChainNum_ == 0)
  {
    _parentindex = _parent->_index;
    _inboxoffset = _parent->_inboxoffset;
    _inwboxcnt = _parent->_inwboxcnt;
    _parent->ExtendedBoxMode(_vbuf, _inwbox);
  }
}

/****************************************************************************/
// PURPOSE:
//   Prompt box copy construcor
//
// PRE:
//   An uninitialized Prompt box.
//
// POST:
//   a Prompt box is constructed and assigned the corresponding values
//   in the Prompt box argument passed to the constructor.
//
WindowRefLineInfo::WindowRefLineInfo(WindowRefLineInfo& pb, int Length_):
_Owner(pb._Owner),
_Enterable(FALSE),
_Selectable(pb._Selectable),

_parent(pb._parent),
_movedone(false),
_haspendingmove(false),
_pendingmove(false),
_extindextoggled(false),

_parentindex(pb._parentindex),
_inboxoffset(pb._inboxoffset),
_inwboxcnt(pb._inwboxcnt),
_inwbox(pb._inwbox),
_vbuf(pb._vbuf),
_maxbuf(-1),
_len(pb._len ? pb._len:
     Length_ ? Length_:1),   // _len should never be zero

_minlen(pb._minlen ? pb._minlen:1),
_distance(0),
_index(pb._index),
_extindex(pb._extindex),
_boxpos(pb._index),
_boxdiff(pb._boxdiff),
_xpos(pb._xpos),
_ypos(pb._ypos),
_noempty(pb._noempty),
_InsertMode(TRUE),
_Escaped(FALSE),
_Hidden(TRUE),
_TextWidgetActive(FALSE),
_InFocus(FALSE),
_SavedCursX(WHEREX()),
_SavedCursY(WHEREY()),
_BufChainNum(-99),
_WinLineNum(0),

// Internal data monitors
_insert_monitor(NULL),
_key_monitor(NULL),
_ext_monitor(NULL),
_index_monitor(NULL),
_escaped_monitor(NULL),
_hidden_monitor(NULL),
_active_monitor(NULL),
_infocus_monitor(NULL),
_inboxoffset_monitor(NULL),
_winlinenum_monitor(NULL),

// Handles to external objects, used by data monitors
_insert_handle(NULL),
_key_handle(NULL),
_ext_handle(NULL),
_index_handle(NULL),
_escaped_handle(NULL),
_hidden_handle(NULL),
_active_handle(NULL),
_infocus_handle(NULL),
_inboxoffset_handle(NULL),
_winlinenum_handle(NULL)
{
  if (pb._minlen)
    SetMinimumLength(pb._minlen);
  else
    SetMinimumLength(1);

  if (_inwbox)
    _distance = _inwboxcnt + _index;
  else if (_vbuf)
    _distance = _inboxoffset + _index;

  if (_parent && _parent->BufferChainPosition() == 0)
  {
    _parentindex = _parent->_index;
    _inboxoffset = _parent->_inboxoffset;
    _inwboxcnt = _parent->_inwboxcnt;
    _parent->ExtendedBoxMode(_vbuf, _inwbox);
  }
}

/****************************************************************************/
const char* WindowRefLineInfo::cMonitor(const char*(*monptr_)(void*, WindowRefLineInfo*, const char*), void* This_, const char* ptr_)
{
  if (monptr_ != NULL)
  {
    SaveCursorPosition();
    const char* RetVal_ = (*monptr_)(This_, this, ptr_);
    RestoreCursorPosition();

    return RetVal_;
  }

  return ptr_;
}

/****************************************************************************/
const int* WindowRefLineInfo::iMonitor(const int*(*monptr_)(void*, WindowRefLineInfo*, const int*), void* This_, const int* ptr_)
{
  if (monptr_ != NULL)
  {
    SaveCursorPosition();
    const int* RetVal_ = (*monptr_)(This_, this, ptr_);
    RestoreCursorPosition();

    return RetVal_;
  }

  return ptr_;
}

/****************************************************************************/
void WindowRefLineInfo::SaveCursorPosition()
{
  _SavedCursX = WHEREX();
  _SavedCursY = WHEREY();
}

/****************************************************************************/
void WindowRefLineInfo::RestoreCursorPosition()
{
  GOTOXY(_SavedCursX, _SavedCursY);
}

/****************************************************************************/
const int* WindowRefLineInfo::MonitorInsertMode(void* This_, const int*(*FnPtr_)(void*, WindowRefLineInfo*, const int*))
{
  _insert_handle = This_;
  _insert_monitor = FnPtr_;

  return &_InsertMode;
}

/****************************************************************************/
const int* WindowRefLineInfo::MonitorKey(void* This_, const int*(*FnPtr_)(void*, WindowRefLineInfo*, const int*))
{
  _key_handle = This_;
  _key_monitor = FnPtr_;

  return &_key;
}

/****************************************************************************/
const int* WindowRefLineInfo::MonitorExtendedKey(void* This_, const int*(*FnPtr_)(void*, WindowRefLineInfo*, const int*))
{
  _ext_handle = This_;
  _ext_monitor = FnPtr_;

  return &_extended;
}

/****************************************************************************/
const int* WindowRefLineInfo::MonitorIndex(void* This_, const int*(*FnPtr_)(void*, WindowRefLineInfo*, const int*))
{
  _index_handle = This_;
  _index_monitor = FnPtr_;

  return &_index;
}

/****************************************************************************/
const int* WindowRefLineInfo::MonitorEscapedStatus(void* This_, const int*(*FnPtr_)(void*, WindowRefLineInfo*, const int*))
{
  _escaped_handle = This_;
  _escaped_monitor = FnPtr_;

  return &_Escaped;
}

/****************************************************************************/
const int* WindowRefLineInfo::MonitorInFocusStatus(void* This_, const int*(*FnPtr_)(void*, WindowRefLineInfo*, const int*))
{
  _infocus_handle = This_;
  _infocus_monitor = FnPtr_;

  return &_InFocus;
}

/****************************************************************************/
const int* WindowRefLineInfo::MonitorHiddenStatus(void* This_, const int*(*FnPtr_)(void*, WindowRefLineInfo*, const int*))
{
  _hidden_handle = This_;
  _hidden_monitor = FnPtr_;
  return &_Hidden;
}

/****************************************************************************/
const int* WindowRefLineInfo::MonitorActiveStatus(void* This_, const int*(*FnPtr_)(void*, WindowRefLineInfo*, const int*))
{
  _active_handle = This_;
  _active_monitor = FnPtr_;
  return &_TextWidgetActive;
}

/****************************************************************************/
// PURPOSE:
//   Prompt box data member destruction method
//
// PRE:
//   The data members of this Prompt box object with original values
//
// POST:
//   If the data members of the box object is not NULL then the data members
//   are deleted and assigned NULL
//
void WindowRefLineInfo::Clear()
{}

/****************************************************************************/
/****************************************************************************/
// PURPOSE:
//   Prompt box assignment operator
//
// PRE:
//   The data members of this Prompt box object with original values
//
// POST:
//   The data members of this object is assigned to the corresponding
//   data members of the Prompt box argument passed to the method.
//   This object is returned.
//
WindowRefLineInfo& WindowRefLineInfo::operator = (WindowRefLineInfo& pb)
{
  if (this != &pb)
  {
    Clear();

    _Enterable = FALSE;
    _Selectable = pb._Selectable;

    _parent = pb._parent;    
    _parentindex = pb._parentindex;
    _inboxoffset = pb._inboxoffset;
    _inwboxcnt = pb._inwboxcnt;
    _inwbox = pb._inwbox;
    _vbuf = pb._vbuf;

    if (_parent && _parent->BufferChainPosition() == 0)
    {
      _parentindex = _parent->_index;
      _inboxoffset = _parent->_inboxoffset;
      _inwboxcnt = _parent->_inwboxcnt;
      _parent->ExtendedBoxMode(_vbuf, _inwbox);
    }

    _maxbuf = -1;
    _len = pb.InputLength() ? pb.InputLength():1;
    
    _minlen = pb._minlen ? pb._minlen:1;
    _index = pb._index;
    _extindex = pb._extindex;
    _xpos = pb._xpos;
    _ypos = pb._ypos;
    _noempty = pb._noempty;    

    if (pb._minlen)
      SetMinimumLength(pb._minlen);
    else
      SetMinimumLength(1);
  }

  return *this;
}

/****************************************************************************/
WindowRefLineInfo* WindowRefLineInfo::Make(int BufChainNum_, WindowLineBuffer* parent_,
                                       ControlWindow* Pwin_, int Length_)
{
  WindowRefLineInfo* pb;
  pb = new WindowRefLineInfo(BufChainNum_, parent_, Pwin_, Length_);
  return pb;
}

/****************************************************************************/
WindowRefLineInfo* WindowRefLineInfo::Make(WindowRefLineInfo& pbox, int Length_)
{
  WindowRefLineInfo* pb;
  pb = new WindowRefLineInfo(pbox, Length_);
  return pb;
}

/****************************************************************************/
WindowLineBuffer* WindowRefLineInfo::MakeNextLine(WindowLineBuffer* wpb)
{
  WindowLineBuffer* nwlb;
  nwlb = wpb ? (new WindowLineBuffer(*wpb, wpb->_maxbuf, wpb->_len)):NULL;

  if (wpb && nwlb)
  {
    int msglen_ = ::SafeStrLen(wpb->_msg);
    nwlb->SetInputLength(wpb->_len, false);
    nwlb->SetXpos(wpb->_xpos + msglen_ + wpb->_len +
                  wpb->EndBrksLength(false));
    nwlb->SetYpos(wpb->_ypos);
  }
  
  return nwlb;
}

/****************************************************************************/
/****************************************************************************/
void WindowRefLineInfo::PrintBlanks(int Max_)
{
  for (int Index_ = 0; Index_ < Max_; Index_++)
    PUTCH(BLANK);
}

/****************************************************************************/
// PURPOSE:
//   Set Prompt message method
//
// PRE:
//   The original value of the message data member
//
// POST:
//   The message data member is assigned new memory copied from the argument
//   string.
//
char* WindowRefLineInfo::SetMessage(char* str)
{
  return NULL;
}

/****************************************************************************/
void WindowRefLineInfo::RemoveMessage()
{
  SetMessage(NULL);
}

/****************************************************************************/
void WindowRefLineInfo::BlankMessage()
{}

/****************************************************************************/
void WindowRefLineInfo::PushInputString(char* str)
{}

/****************************************************************************/
void WindowRefLineInfo::FillCharAtPos(char ch, int pos, bool endterm_)
{}

/****************************************************************************/
void WindowRefLineInfo::SetInputStringAtPos(char* str, int len, int pos, bool endterm_)
{}

/****************************************************************************/
// PURPOSE:
//   Set default input method
//
// PRE:
//   The original value of the input buffer data member
//
// POST:
//   The input buffer data member is assigned new memory copied from the
//   argument string. The remaining space in the buffer is NULL filled.
//
void WindowRefLineInfo::SetInputString(char* str)
{}

/****************************************************************************/
// PURPOSE:
//   Set input length method
//
// PRE:
//   The original value of the input length data member
//
// POST:
//   The input length data member is copied from the length argument.
//
void WindowRefLineInfo::SetInputLength(int len, bool MakeVbuf_)
{
  _len = len;
}

/****************************************************************************/
void WindowRefLineInfo::DivBufSizeAsInputLength(int div_, bool MakeVbuf_)
{
  int res_ = _len / div_;
  _len = res_;
}

/****************************************************************************/
// PURPOSE:
//   Set minimum input length method
//
// PRE:
//   The original value of the minimum input length data member
//
// POST:
//   The mimimum input length data member is copied from the length argument.
//
void WindowRefLineInfo::SetMinimumLength(int len)
{
  if (len <= _len)
    _minlen = (len > 0) ? len:1;
  else
    _minlen = _len;
}

/****************************************************************************/
int* WindowRefLineInfo::WordChr(int* src_, int val_, size_t n)
{
  while (n)
    if (*src_ == val_)
      return src_;
    else
    {
      src_++;
      --n;
    }

  return NULL;
}

/****************************************************************************/
void WindowRefLineInfo::SetNoEmpty(int flag)
{
  _noempty = flag;
}

/****************************************************************************/
void WindowRefLineInfo::NotifyInsertMode(const int* ModePtr_)
{
  if (ModePtr_)
    _InsertMode = *ModePtr_;

  iMonitor(_insert_monitor, _insert_handle, ModePtr_);
}

/****************************************************************************/
void WindowRefLineInfo::SetInsertMode(int Mode_, WindowRefLineInfo* Parent_)
{
  _InsertMode = Mode_;
  iMonitor(_insert_monitor, _insert_handle, &_InsertMode);

  if (Parent_)
    MonitorInsertMode(Parent_, InsertDetect);
}

/****************************************************************************/
bool WindowRefLineInfo::SetXYpos(int x, int y)
{
  int maxx_ = TextGraphics::TerminalScreenCols();
  int maxy_ = TextGraphics::TerminalScreenRows();

  if ((1 <= x && x <= maxx_) &&
      (1 <= y && y <= maxy_))
  {
    Hide();
    _xpos = x;
    _ypos = y;
    return true;
  }

  return false;
}

/****************************************************************************/
bool WindowRefLineInfo::SetXpos(int x)
{
  int maxx_ = TextGraphics::TerminalScreenCols();

  if (1 <= x && x <= maxx_)
  {
    Hide();
    _xpos = x;
    return true;
  }

  return false;
}

/****************************************************************************/
bool WindowRefLineInfo::SetYpos(int y)
{
  int maxy_ = TextGraphics::TerminalScreenRows();

  if (1 <= y && y <= maxy_)
  {
    Hide();
    _ypos = y;
    return true;
  }

  return false;
}

/****************************************************************************/
int WindowRefLineInfo::GetXpos() const
{
  return _xpos;
}

/****************************************************************************/
int WindowRefLineInfo::GetYpos() const
{
  return _ypos;
}

/****************************************************************************/
#if (!(defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__)) & (defined(__linux__) | defined(__unix__)))
int WindowRefLineInfo::TranslateKeys(int key)
{
  return
  (
    GetKeyMap().KeyMatch(key, INSERT)   ? KEY_IC:
    GetKeyMap().KeyMatch(key, LEFT)     ? KEY_LEFT:
    GetKeyMap().KeyMatch(key, RIGHT)    ? KEY_RIGHT:
    GetKeyMap().KeyMatch(key, HOME)     ? KEY_HOME:
    GetKeyMap().KeyMatch(key, END)      ? KEY_END:
    GetKeyMap().KeyMatch(key, DELETE)   ? KEY_DC:
    GetKeyMap().KeyMatch(key, ENTER)    ? 10:
    GetKeyMap().KeyMatch(key, BACK)     ? KEY_BACKSPACE:
    GetKeyMap().KeyMatch(key, ESCAPE)   ? 27:
    GetKeyMap().KeyMatch(key, NEXT)     ? 9:
    GetKeyMap().KeyMatch(key, PAGEUP)   ? KEY_PPAGE:
    GetKeyMap().KeyMatch(key, PAGEDOWN) ? KEY_NPAGE:
    GetKeyMap().KeyMatch(key, UP)       ? KEY_UP:
    GetKeyMap().KeyMatch(key, DOWN)     ? KEY_DOWN:
                                          key
  );
}
#endif

/****************************************************************************/
// PURPOSE:
//   Keyboard retrieval procedure
//
// PRE:
//   The original value of _key and _extended data members
//
// POST:
//   The keyboard is read for a key and is assigned to _key data member.
//   If the key is extended non-ASCII then the _extended data member is set.
//   Returns the key read from the keyboard.
//
int WindowRefLineInfo::GetKey()
{
  _extkey = _key = (int)GETCH();
  iMonitor(_key_monitor, _key_handle, &_key);
  _extended = (_key == 0 || _key == 0xE0) ? 256:0;
  iMonitor(_ext_monitor, _ext_handle, &_extended);

  if (_extended)
  {
    #if (!(defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__)) & (defined(__linux__) | defined(__unix__)))
      noecho();
    #endif
    _key = (int)GETCH();
    _extkey = _key + _extended;
    iMonitor(_key_monitor, _key_handle, &_key);

    #if (!(defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__)) & (defined(__linux__) | defined(__unix__)))
      _extkey = TranslateKeys(_extkey);
    #endif
  }
#if (!(defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__)) & (defined(__linux__) | defined(__unix__)))
  else
    _extkey = TranslateKeys(_extkey);
#endif

  return _key;
}

/****************************************************************************/
void WindowRefLineInfo::CommitPendingMove()
{
  if (_haspendingmove)
  {
    SetMoveDone(_pendingmove);
    _haspendingmove = false;
    _pendingmove = false;    
  }
}

/****************************************************************************/
void WindowRefLineInfo::SetMovePending(bool pending_, bool done_)
{
  _haspendingmove = pending_;
  _pendingmove = pending_ ? done_:false;
}

/****************************************************************************/
bool WindowRefLineInfo::MovePending() const
{
  return _pendingmove;
}

/****************************************************************************/
void WindowRefLineInfo::SetMoveDone(bool move_)
{
  _movedone = move_;
  
  if (_movedone)
  {
    _haspendingmove = false;
    _pendingmove = false;
  }
}

/****************************************************************************/
bool WindowRefLineInfo::MoveDone() const
{
  return _movedone;
}

/****************************************************************************/
// PURPOSE:
//   Method for putting extended non-ASCII keyboard keys to the console
//   output.
//
// PRE:
//   int key : the key to be outputted.
//
// POST:
//   The key is outputted to the screen. Appropriate changes to the
//   internal input buffer is made.
//
bool WindowRefLineInfo::PutExtKey(int key)
{
  int index_ = 0;
  int endpt_ = _len-1;
  
  bool KeyDone_ = false;
  bool BoxPosMoved_ = false;

  // Action: TOGGLE_INSERT
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  if (GetKeyMap().KeyMatch(key, INSERT))
#elif (defined(__linux__) | defined(__unix__))
  if (key == KEY_IC)
#endif
    SetInsertMode(!GetInsertMode());

  // Action: GO_LEFT (chain push if WindowLine::_wraparoundindex > 0)
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, LEFT))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_LEFT)
#endif  
  {
    if (_parent)
    {
      _parentindex = _parent->_index;
      _inboxoffset = _parent->_inboxoffset;
      _inwboxcnt = _parent->_inwboxcnt;

      if (_inwbox)
        _distance = _inwboxcnt + _index;
      else if (_vbuf)
        _distance = _inboxoffset + _index;
    }

    // Modified to wrap chars around first buffer in window buffer object
    if (_extindex > 0 || _inwboxcnt > 0 || _boxdiff > 0)
    {
      if (_boxpos > 0 && _boxpos >= _index)
      {
        --_boxpos;
        BoxPosMoved_ = _index > 0 && (_boxdiff == 0 || _boxpos < _index);
      }

      if (_extindex > 0)
      {
        // Only decrement _extindex value if parentindex value is at zero
        //
        if (_parentindex == 0)
        {
          _boxpos = _index = _parentindex;
          BoxPosMoved_ = false;

          _extindextoggled = _extindex == 1; // transitioned from 1-->0
          --_extindex;
        }

        // decrement _index value if _boxpos value have moved below it
        //
        if (BoxPosMoved_)
        {
          --_index;

          if (_boxpos < _index)
            _boxpos = _index;
        }
      }
      else
      {
        if (_parentindex == 0)
        {
          _boxpos = _index = _parentindex;
          BoxPosMoved_ = false;
        }
      
        if (BoxPosMoved_)
        {
          --_index;

          if (_boxpos < _index)
            _boxpos = _index;
        }
      }

      _boxdiff = _boxpos - _index;

      if (_boxdiff < 0)
      {
        _boxpos = _index;
        _boxdiff = 0;
      }
      else if (!_boxdiff && _index != _parentindex)
      {
        _index = _parentindex;

        if (_index > endpt_)
          _index = endpt_;

        if (_boxpos < _index)
          _boxpos = _index;
        else if (_boxpos > endpt_)
          _boxpos = endpt_;
      }

      index_ = _index + _boxdiff + _extindex;
      KeyDone_ = true;
      iMonitor(_index_monitor, _index_handle, &index_);
    }
  }

  // Action: GO_RIGHT (chain pull if _extindex >= WindowLine::_inputlen)
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, RIGHT))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_RIGHT)
#endif  
  {
    if (_parent)
    {
      _parentindex = _parent->_index;
      _inboxoffset = _parent->_inboxoffset;
      _inwboxcnt = _parent->_inwboxcnt;

      if (_inwbox)
        _distance = _inwboxcnt + _index;
      else if (_vbuf)
        _distance = _inboxoffset + _index;
    }

    if (_extindex + 1 <= _maxbuf || _maxbuf == -1)    
    {
      if (_index != _parentindex)
      {
        // if _index < _parentindex then assign _index to _parentindex
        // to syncrhonize. Synchronize _boxpos to match with _index.
        //
        _index = _parentindex;

        if (_index > endpt_)
          _index = endpt_;

        if (_boxpos < _index)
          _boxpos = _index;
        else if (_boxpos > endpt_)
          _boxpos = endpt_;

        if (_boxpos < endpt_)
          ++_boxpos;
      }
      else
      {
        // only increase boxpos if _index == _parentindex
        // synchronize _boxpos with _index
        //
        if (_boxpos < _index)
          _boxpos = _index;
        else if (_boxpos > endpt_)
          _boxpos = endpt_;

        if (_boxpos < endpt_)
          ++_boxpos;
      }

      // if inwboxcnt offset value is non-zero then increment _extindex
      //
      if (_inwboxcnt > 0)
      {
        _extindextoggled = _extindex == 0; // transitioned from 0-->1
        ++_extindex;
      }

      _boxdiff = _boxpos - _index;

      // ensure that boxdiff value is non-negative
      //
      if (_boxdiff < 0)
      {
        _boxpos = _index;
        _boxdiff = 0;
      }
      else if (!_boxdiff && _index != _parentindex)
      {
        _index = _parentindex;

        if (_index > endpt_)
          _index = endpt_;

        if (_boxpos < _index)
          _boxpos = _index;
        else if (_boxpos > endpt_)
          _boxpos = endpt_;
      }
      
      index_ = _index + _boxdiff + _extindex;
      KeyDone_ = true;
      iMonitor(_index_monitor, _index_handle, &index_);
    }
  }

  // Action: GO_HOME
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, HOME))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_HOME)
#endif  
  {    
    _boxpos = 0;
    _index = 0;
    _extindex = 0;
    KeyDone_ = true;

    if (_parent)
    {
      _parent->ResetRefPos(0);

      _boxpos = _index =
      _parentindex = _parent->_index;
      _inboxoffset = _parent->_inboxoffset;
      _inwboxcnt = _parent->_inwboxcnt;

      if (_inwbox)
        _distance = _inwboxcnt + _index;
      else if (_vbuf)
        _distance = _inboxoffset + _index;
    }

    iMonitor(_index_monitor, _index_handle, &_index);
  }

  // Action: GO_END
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, END))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_END)
#endif
  {
    _boxpos = endpt_;
    _index = endpt_;
    _extindex = 0;
    KeyDone_ = true;

    if (_parent)
    {
      _parent->ResetRefPos(1);

      _boxpos = _index =
      _parentindex = _parent->_index;
      _inboxoffset = _parent->_inboxoffset;
      _inwboxcnt = _parent->_inwboxcnt;

      if (_inwbox)
        _distance = _inwboxcnt + _index;
      else if (_vbuf)
        _distance = _inboxoffset + _index;
    }
  
    iMonitor(_index_monitor, _index_handle, &_index);
  }

  // Action: DELETE
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, DELETE))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_DC)
#endif
  {
    if (ExtendedIndex() == 0)
    {
      _extindex = 0;
      KeyDone_ = true;

      if (_parent)
      {
        _boxpos = _index =
        _parentindex = _parent->_index;
        _inboxoffset = _parent->_inboxoffset;
        _inwboxcnt = _parent->_inwboxcnt;

        if (_inwbox)
          _distance = _inwboxcnt + _index;
        else if (_vbuf)
          _distance = _inboxoffset + _index;
      }
    }
  }

  SetMoveDone(KeyDone_);
  return _movedone;
}

/****************************************************************************/
// PURPOSE:
//   Method for putting standard ASCII keyboard keys to the console output.
//
// PRE:
//   int key : the key to be outputted.
//
// POST:
//   The key is outputted to the screen. Appropriate changes to the
//   internal input buffer is made.
//
bool WindowRefLineInfo::PutStdKey(int key)
{
  bool KeyDone_ = false;
  bool AtEndPt_ = false;
  bool PrevLineReady_ = false;
  bool NextLineReady_ = false;
  bool InWinMode_ = false;
  
  int EndReached_ = 0;  
  int NextNonWsLen_ = 0;

  #if WINLINE_DEBUG3
    bool Debug_ = true;
  #else
    bool Debug_ = false;
  #endif

  InWinMode_ = ((WindowLine*)_Owner)->InWindowMode(&NextLineReady_,
                                                   &PrevLineReady_);
  _parent->ExtendedBoxMode(_vbuf, _inwbox,
                           &EndReached_, &NextNonWsLen_);

  // Action: NEWLINE
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  if (_InsertMode && GetKeyMap().KeyMatch(key, ENTER))
#elif (defined(__linux__) | defined(__unix__))
  if (_InsertMode && key == 10)
#endif
  {
    if (ExtendedIndex() == 0)
    {
      _extindex = 0;
      KeyDone_ = true;

      if (_parent)
      {
        _boxpos = _index =
        _parentindex = _parent->_index;
        _inboxoffset = _parent->_inboxoffset;
        _inwboxcnt = _parent->_inwboxcnt;

        if (_inwbox)
          _distance = _inwboxcnt + _index;
        else if (_vbuf)
          _distance = _inboxoffset + _index;
      }
    }
  }

  // Action: PUTKEY
  else if (IsUserKey(key))
  {
    if (ExtendedIndex() == 0)
    {
      _extindex = 0;
      KeyDone_ = true;

      if (_parent)
      {
        _boxpos = _index =
        _parentindex = _parent->_index;
        _inboxoffset = _parent->_inboxoffset;
        _inwboxcnt = _parent->_inwboxcnt;

        if (_inwbox)
          _distance = _inwboxcnt + _index;
        else if (_vbuf)
          _distance = _inboxoffset + _index;
      }
    }
  }

  SetMoveDone(KeyDone_);
  return _movedone;
}

/****************************************************************************/
int WindowRefLineInfo::IsUserKey(int key)
{
  return
  (
    !_extended &&
    #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
      !GetKeyMap().KeyMatch(key, BACK) &&
      !GetKeyMap().KeyMatch(key, ESCAPE) &&
      !GetKeyMap().KeyMatch(key, ENTER) &&
      !GetKeyMap().KeyMatch(key, NEXT) &&
    #elif (defined(__linux__) | defined(__unix__))
      key != KEY_BACKSPACE &&
      key != 27 &&
      key != 10 &&
      key != 9 &&
    #endif
    isprint(key)
  );
}

/****************************************************************************/
void WindowRefLineInfo::ShiftDownAndSetStr(char* NewStr_)
{
  SetInputString(NewStr_);
}

/****************************************************************************/
void WindowRefLineInfo::FillString(char* Str_, char FillCh_, int Start_, int End_)
{
  for (;Start_ < End_; Start_++)
    Str_[Start_] = FillCh_;

  Str_[End_] = '\0';
}

/****************************************************************************/
/****************************************************************************/
// PURPOSE:
//   End input test procedure
//
// PRE:
//   int key : the key to be tested as the termination key for input.
//
// POST:
//   Returns 1 if the key is defined as a termination key,
//   otherwise returns 0.
//
int WindowRefLineInfo::ShouldEnd(int key, int zerotozero)
{
  int status_;

  // Action: GO_DOWN, ESCAPE
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  if (GetKeyMap().KeyMatch(key, ESCAPE))	
#elif (defined(__linux__) | defined(__unix__))
  if (key == 27)
#endif
  {
    _Escaped = TRUE;
    iMonitor(_escaped_monitor, _escaped_handle, &_Escaped);
    status_ = 1;
  }

  // Action: GO_DOWN, IF_NOTCHAINED_ESCAPE
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, ENTER))	
#elif (defined(__linux__) | defined(__unix__))
  else if (key == 10)
#endif  
  {
    _Escaped = TRUE;
    iMonitor(_escaped_monitor, _escaped_handle, &_Escaped);    
    status_ = 1;
  }

  // Action: GO_DOWN, GO_NEXT
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, NEXT) || GetKeyMap().KeyMatch(key, DOWN))	
#elif (defined(__linux__) | defined(__unix__))
  else if (key == 9 || key == KEY_DOWN)
#endif
    status_ = 1;

  // Action: GO_UP
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, UP))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_UP)
#endif
    status_ = -1;

  // Action: GO_BACK, GO_LEFT (Note Plural Actions For Single Key)
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, BACK) || GetKeyMap().KeyMatch(key, LEFT))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_BACKSPACE || key == KEY_LEFT)
#endif
    status_ = (_index <= 0 && zerotozero) ? -1:0;

  // Action: GO_RIGHT (Note Plural Actions For Single Key)
#if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
  else if (GetKeyMap().KeyMatch(key, RIGHT) || IsUserKey(key))
#elif (defined(__linux__) | defined(__unix__))
  else if (key == KEY_RIGHT || IsUserKey(key))
#endif
    status_ = (_index >= _len) ? 1:0;

  // Otherwise, no action
  else
    status_ = 0;

  return status_;
}

/****************************************************************************/
// PURPOSE:
//   Get line input procedure
//
// PRE:
//   The original input buffer
//
// POST:
//   Gets input from the keyboard and place it in the input buffer until
//   a termination is press.
//
int WindowRefLineInfo::GetLine(int status_)
{
  int lastindex_;
  int zerotozero_;

  ResetInputState(status_);

  iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
  iMonitor(_active_monitor, _active_handle, &_TextWidgetActive);

  if (_parent)
    return _parent->GetLine(status_);
  else
  {
    for (;;)
    {
      if (_parent)
        _parent->GetKey();
      else
        GetKey();

      lastindex_ = _index;

      if (_extended)
        PutExtKey(_extkey);
      else
        PutStdKey(_extkey);

      zerotozero_ = !lastindex_ && !_index;

      if (_parent)
      {
        _parent->DetermineWrapChars();
        if (status_ = _parent->ShouldEnd(_extkey, zerotozero_))
          break;
      }
    }
  }

  return status_;
}

/****************************************************************************/
void WindowRefLineInfo::ResetInputState(int status_)
{
  _Escaped = FALSE;
  // _index = (status_ == -1) ? 1:0;
  _index = 0;

  iMonitor(_escaped_monitor, _escaped_handle, &_Escaped);
  iMonitor(_index_monitor, _index_handle, &_index);
}

/****************************************************************************/
// PURPOSE:
//   Read input procedure
//
// POST:
//   Calls procedure to Show the contents of the input buffer as default
//   input to the Prompt box then calls GetLine to get additional input.
//   from the keyboard.
//
int WindowRefLineInfo::ReadInput(int status_)
{
  if (_parent)
    _parent->ShowInput(status_);
  return GetLine(status_);
}

/****************************************************************************/
// PURPOSE:
//   Method to Erase the input
//
// POST:
//   Erases the internal buffer by NULL padding entire buffer.
//
void WindowRefLineInfo::Erase()
{
  if (!IsActive())
    return;

  if (!_Hidden)
  {
    _InFocus = TRUE;
    iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

    if (UsesColor())
      SetupColors(ColorInfo::BACKGROUND_COLOR);
    else
      UseColor(FALSE);
    
    ResetPrompt();
    _InFocus = FALSE;
    iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
  }
}

/****************************************************************************/
// PURPOSE:
//   reset user Prompt method
//
// POST:
//   The user Prompt is setup at the specified position and the original
//   contents of the line is Cleared
//
void WindowRefLineInfo::ResetPrompt()
{
  if (!IsActive())
    return;

  GOTOXY(_xpos, _ypos);
  PrintBlanks(_len);
  GOTOXY(_xpos, _ypos);
}

/****************************************************************************/
// PURPOSE:
//   Show Prompt message method
//
// POST:
//   Goes to the specified coordinates and Shows the Prompt message
//   to the screen
//
void WindowRefLineInfo::ShowMessage()
{
  GOTOXY(_xpos, _ypos);
}

/****************************************************************************/
// PURPOSE:
//   Show input method
//
// POST:
//   Shows the Prompt box and Shows the original contents of the input
//   buffer within the box
//
void WindowRefLineInfo::ShowInput(int status_)
{
  if (!IsActive())
    return;

  if (_len)
  {
    ResetPrompt();

    GOTOXY(_xpos, _ypos);

    if (status_ == -1)
      GOTOXY(wherex(), _ypos);
  }
}

/****************************************************************************/
// PURPOSE:
//   Prompt user method
//
// POST:
//   Shows the Prompt message and the Prompt box then Prompts user for
//   additional input
//
int WindowRefLineInfo::Prompt(int status_)
{
  if (!IsActive())
    return status_;

  if (_len)
  {
    _InFocus = TRUE;
    iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

    SetupColors(ColorInfo::HIGHLIGHT_COLORS);
    ShowMessage();
    SetupColors(ColorInfo::NORMAL_COLORS);
    status_ = ReadInput(status_);
    Show();

    _InFocus = FALSE;
    iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
    _Hidden = FALSE;
    iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
  }

  return status_;
}

/****************************************************************************/
// PURPOSE:
//   Shows Prompt box method
//
// POST:
//   Shows Prompt message and Prompt box with contents of input buffer
//
void WindowRefLineInfo::Show()
{
  if (!IsActive())
    return;

  _InFocus = TRUE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

  SetupColors(ColorInfo::NORMAL_COLORS);
  ShowMessage();
  ShowInput(1);

  _InFocus = FALSE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
  _Hidden = FALSE;
  iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
}

/****************************************************************************/
void WindowRefLineInfo::ShowInput()
{
  if (!IsActive())
    return;

  _InFocus = TRUE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

  ShowInput(1);

  _InFocus = FALSE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
  _Hidden = FALSE;
  iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
}

/****************************************************************************/
// PURPOSE:
//   Hide Prompt box method
//
// POST:
//   goes to the specified coordinates of the Prompt box then Clears the
//   line containing the Prompt box.
//
void WindowRefLineInfo::Hide()
{
  if (!IsActive())
    return;

  _InFocus = TRUE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);

  GOTOXY(_xpos, _ypos);

  if (UsesColor())
    SetupColors(ColorInfo::BACKGROUND_COLOR);
  else
    UseColor(FALSE);

  PrintBlanks(_len);
  GOTOXY(_xpos, _ypos);

  _InFocus = FALSE;
  iMonitor(_infocus_monitor, _infocus_handle, &_InFocus);
  _Hidden = TRUE;
  iMonitor(_hidden_monitor, _hidden_handle, &_Hidden);
}

/****************************************************************************/
int WindowRefLineInfo::NextLengthHelper() const
{
  return (_index + _boxdiff + _extindex);
}

/****************************************************************************/
int WindowRefLineInfo::NextLength() const
{
  return (_index + _boxdiff + _extindex);
}

/****************************************************************************/
int WindowRefLineInfo::TakeNextLastCharHelper()
{
  return 0;
}

/****************************************************************************/
int WindowRefLineInfo::TakeNextLastChar()
{
  return 0;
}

/****************************************************************************/
int WindowRefLineInfo::VirtualBufferOffset(int* EndReached_) const
{
  return 0;
}

/****************************************************************************/
int WindowRefLineInfo::VirtualBufferMax() const
{
  return 0;
}

/****************************************************************************/
int WindowRefLineInfo::IsChainable(bool HasChars_) const
{
  return true;
}

/****************************************************************************/
int WindowRefLineInfo::IsDummyLine() const
{
  return true;
}

/****************************************************************************/
int WindowRefLineInfo::IsWindowLineBuffer() const
{
  return false;
}

/****************************************************************************/
int WindowRefLineInfo::ChainPosition() const
{
  return -99;
}

/****************************************************************************/
int WindowRefLineInfo::BufferChainPosition() const
{
  return -99;
}

/****************************************************************************/
int WindowRefLineInfo::IsFullBox() const
{
  return false;
}

/****************************************************************************/
int WindowRefLineInfo::RestrictFullBox() const
{
  return false;
}

/****************************************************************************/
bool WindowRefLineInfo::EndBrksActive() const
{
  return false;
}

/****************************************************************************/
bool WindowRefLineInfo::ShouldTrimWs() const
{
  return false;
}

/****************************************************************************/
int WindowRefLineInfo::BufferIndex() const
{
  return _index;
}

/****************************************************************************/
int WindowRefLineInfo::InBoxOffset() const
{
  return _inboxoffset;
}

/****************************************************************************/
int WindowRefLineInfo::InWBoxCount() const
{
  return _inwboxcnt;
}

/****************************************************************************/
int WindowRefLineInfo::InputLength() const
{
  return _len;
}

/****************************************************************************/
int WindowRefLineInfo::ParentIndex() const
{
  return _parentindex;  // same as _parent->_index
}

/****************************************************************************/
void WindowRefLineInfo::ResetExtendedIndexToggle()
{
  _extindextoggled = false;
}

/****************************************************************************/
bool WindowRefLineInfo::ExtendedIndexToggled() const
{
  return _extindextoggled;
}

/****************************************************************************/
int WindowRefLineInfo::ExtendedIndex() const
{
  return _extindex;  // extra char. lengths tacked onto end of line
}

/****************************************************************************/
int WindowRefLineInfo::BoxPos() const
{
  return _boxpos;   // actual cursor position relative to _len in box
}

/****************************************************************************/
int WindowRefLineInfo::BoxDiff() const
{
  return _boxdiff;  // difference from cursor box pos. to index pos.
}

/****************************************************************************/
int WindowRefLineInfo::Distance() const
{
  return _distance; // absolute distance from zero position start pt.
}

/****************************************************************************/
void WindowRefLineInfo::SetEnterable(Boolean Flag_)
{}

/****************************************************************************/
void WindowRefLineInfo::SetSelectable(Boolean Flag_)
{
  _Selectable = Flag_;
}

/****************************************************************************/
int WindowRefLineInfo::GetHotKey() const
{
  return 0;
}

/****************************************************************************/
int WindowRefLineInfo::GetShortCut() const
{
  return 0;
}

/****************************************************************************/
int WindowRefLineInfo::GetHotKeyType() const
{
  return 0;
}

/****************************************************************************/
void WindowRefLineInfo::Select()
{}

/****************************************************************************/
void WindowRefLineInfo::Deselect()
{}

/****************************************************************************/
void WindowRefLineInfo::Activate()
{
  _TextWidgetActive = TRUE;
  iMonitor(_active_monitor, _active_handle, &_TextWidgetActive);
}

/****************************************************************************/
void WindowRefLineInfo::Deactivate()
{
  _TextWidgetActive = FALSE;
  iMonitor(_active_monitor, _active_handle, &_TextWidgetActive);
}

/****************************************************************************/
void WindowRefLineInfo::DecrBoxIndex()
{
  if (_index - 1 >= 0 && _extindex == 0 && _boxdiff == 0)
  {
    _index--;

    if (_boxpos != _index)
    {
      _boxpos = _index;
      _boxdiff = 0;
    }
  }

  if (_parent)
  {
    _parentindex = _parent->_index;
    _inboxoffset = _parent->_inboxoffset;
    _inwboxcnt = _parent->_inwboxcnt;

    if (_inwbox)
      _distance = _inwboxcnt + _parentindex;
    else if (_vbuf)
      _distance = _inboxoffset + _parentindex;

    if (_index >= _len)
      _boxpos = _index = _len-1;
  }
}

/****************************************************************************/
void WindowRefLineInfo::IncrBoxIndex()
{
  if ((_index + 1 <= _maxbuf || _maxbuf == -1) && _extindex == 0 && _boxdiff == 0)
  {
    _index++;

    if (_boxpos != _index)
    {
      _boxpos = _index;
      _boxdiff = 0;
    }
  }

  if (_parent)
  {
    _parentindex = _parent->_index;
    _inboxoffset = _parent->_inboxoffset;
    _inwboxcnt = _parent->_inwboxcnt;

    if (_inwbox)
      _distance = _inwboxcnt + _parentindex;
    else if (_vbuf)
      _distance = _inboxoffset + _parentindex;

    if (_index >= _len)
      _boxpos = _index = _len-1;
  }
}

/****************************************************************************/
void WindowRefLineInfo::ResetBoxIndex()
{
  _boxpos =
  _index = 0;
  _extindex = 0;

  if (_parent)
  {
    _boxpos = _index =
    _parentindex = _parent->_index;
    _inboxoffset = _parent->_inboxoffset;
    _inwboxcnt = _parent->_inwboxcnt;

    if (_inwbox)
      _distance = _inwboxcnt + _parentindex;
    else if (_vbuf)
      _distance = _inboxoffset + _parentindex;
  }  
}

/****************************************************************************/
void WindowRefLineInfo::SetBoxIndex(int Val_)
{
  if (0 <= Val_ && Val_ < _maxbuf || (Val_ >= 0 && _maxbuf == -1))
  {
    _boxpos =
    _index = Val_;
    _extindex = 0;
  }

  if (_parent)
  {
    _parentindex = _parent->_index;
    _inboxoffset = _parent->_inboxoffset;
    _inwboxcnt = _parent->_inwboxcnt;

    if (_inwbox)
      _distance = _inwboxcnt + _parentindex;
    else if (_vbuf)
      _distance = _inboxoffset + _parentindex;

    if (_index >= _len)
      _boxpos = _index = _len-1;
  }
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsEmpty() const
{
  return TRUE;
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsEscaped() const
{
  return _Escaped;
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsActive() const
{
  return _TextWidgetActive;
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsEnterable() const
{
  return _Enterable;
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsSelectable() const
{
  return _Selectable;
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsSelected() const
{
  return TRUE;
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsValid() const
{
  return (!(IsEmpty() && _noempty) && IsValidLength());
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsInFocus() const
{
  return _InFocus;
}

/****************************************************************************/
char WindowRefLineInfo::LastKeyRead(int* Ext_) const
{
  if (Ext_)
    *Ext_ = _extended;

  return _key;
}

/****************************************************************************/
int WindowRefLineInfo::LastExtKeyRead() const
{
  return _extkey;
}

/****************************************************************************/
char WindowRefLineInfo::ChrRetrieve(int Index_)
{
  return 0;
}

/****************************************************************************/
Boolean WindowRefLineInfo::IsOwner(ControlWindow* Pwin_) const
{
  return (_Owner == Pwin_);
}

/****************************************************************************/
void WindowRefLineInfo::ChainSetOwner(ControlWindow* Pwin_, TextControl* TxtCtrl_)
{
  SetOwner(Pwin_, TxtCtrl_);
}

/****************************************************************************/
void WindowRefLineInfo::SetOwner(ControlWindow* Pwin_, TextControl* TxtCtrl_)
{
  if (!TxtCtrl_->IsDummyLine())
    return;

  WindowRefLineInfo* ptr = (WindowRefLineInfo*)TxtCtrl_;
  WindowLine* wlptr = (WindowLine*)Pwin_;
  WindowLine* tcwlptr = (WindowLine*)_Owner;

  if (TxtCtrl_ && tcwlptr && tcwlptr != wlptr &&
      !wlptr->HasControlArrays(tcwlptr))
    ptr->_Owner->RemoveControl(TxtCtrl_);

  ptr->_Owner = Pwin_;
}

/****************************************************************************/
void WindowRefLineInfo::SetBufferChainNum(WindowLineBufferType* Lbuf_, int Index_)
{
  if (Lbuf_)
    if (this != Lbuf_)
      Lbuf_->SetBufferChainNum(Lbuf_, Index_);
    else
      _BufChainNum = Index_;
}

/****************************************************************************/
void WindowRefLineInfo::SetReturnBuffer(WindowLineBufferType* Lbuf_,
                                      char* RetBufPtr_, bool Alloc_, int Size_)
{}

/****************************************************************************/
void WindowRefLineInfo::CreateReturnBuffer(int Size_)
{}

/****************************************************************************/
void WindowRefLineInfo::SetWindowLineNum(int Index_)
{
  _WinLineNum = Index_;
}

/****************************************************************************/
void WindowRefLineInfo::SetupColors(int Mode_)
{
  if (!_ColorInfo.SetupColors(Mode_))
    GetDefaultColorInfo().SetupColors(Mode_);
}

/****************************************************************************/
void WindowRefLineInfo::UseColor(Boolean Flag_)
{
  if (_ColorInfo._UseColor)
    *_ColorInfo._UseColor = Flag_;
  else
    _ColorInfo._UseColor = new Boolean(Flag_);

  GetDefaultColorInfo().UseColor(Flag_);
}

/****************************************************************************/
void WindowRefLineInfo::SetNormalColors(int FgCol_, int BgCol_)
{
  if (_ColorInfo._NormalColors)
  {
    _ColorInfo._NormalColors[0] = FgCol_;
    _ColorInfo._NormalColors[1] = BgCol_;
  }
  else
  {
    _ColorInfo._NormalColors = new int[2];
    SetNormalColors(FgCol_, BgCol_);
  }
}

/****************************************************************************/
void WindowRefLineInfo::SetHighLightColors(int FgCol_, int BgCol_)
{
  if (_ColorInfo._HighLightColors)
  {
    _ColorInfo._HighLightColors[0] = FgCol_;
    _ColorInfo._HighLightColors[1] = BgCol_;
  }
  else
  {
    _ColorInfo._HighLightColors = new int[2];
    SetHighLightColors(FgCol_, BgCol_);
  }
}

/****************************************************************************/
void WindowRefLineInfo::SetHotKeyColor(int Col_)
{
  if (_ColorInfo._HkColor)
    *_ColorInfo._HkColor = Col_;
  else
    _ColorInfo._HkColor = new int(Col_);
}

/****************************************************************************/
void WindowRefLineInfo::SetBackGroundColor(int Col_)
{
  if (_ColorInfo._BackGroundColor)
    *_ColorInfo._BackGroundColor = Col_;
  else
    _ColorInfo._BackGroundColor = new int(Col_);
}

/****************************************************************************/
void WindowRefLineInfo::UseDefaultColors()
{
  _ColorInfo.ClearColors();
  UseColor();
}

/****************************************************************************/
Boolean WindowRefLineInfo::UsesColor() const
{
  return
  (
    _ColorInfo._UseColor ?
      *(_ColorInfo._UseColor):
      *(GetDefaultColorInfo()._UseColor)
  );
}

/****************************************************************************/
int* WindowRefLineInfo::NormalColors() const
{
  return
  (
    _ColorInfo._NormalColors ?
      _ColorInfo._NormalColors:
      GetDefaultColorInfo()._NormalColors
  );
}

/****************************************************************************/
int* WindowRefLineInfo::HighLightColors() const
{
  return
  (
    _ColorInfo._HighLightColors ?
      _ColorInfo._HighLightColors:
      GetDefaultColorInfo()._HighLightColors
  );
}

/****************************************************************************/
int WindowRefLineInfo::HotKeyColor() const
{
  return
  (
    _ColorInfo._HkColor ?
      *(_ColorInfo._HkColor):
      *(GetDefaultColorInfo()._HkColor)
  );
}

/****************************************************************************/
int WindowRefLineInfo::BackGroundColor() const
{
  return
  (
    _ColorInfo._BackGroundColor ?
      *(_ColorInfo._BackGroundColor):
      *(GetDefaultColorInfo()._BackGroundColor)
  );
}

/****************************************************************************/
// WinCoordsData class definition
/****************************************************************************/
WinCoordsData::WinCoordsData(int WinStartx_, int WinStarty_,
                             int WinMaxx_, int WinMaxy_):
_Allocated(false),
_screenxpos(1),
_screenypos(1),
_screenmaxx(0),
_screenmaxy(0),

_winboxxpos(1),
_winboxypos(1),
_winboxstartx(WinStartx_),
_winboxstarty(WinStarty_),
_winboxmaxx(WinMaxx_),
_winboxmaxy(WinMaxy_),

_winviewxoffset(0),
_winviewyoffset(0),
_winviewxpos(1),
_winviewypos(1),
_winviewmaxx(WinMaxx_),
_winviewmaxy(WinMaxx_)
{
  TextGraphics::RetrieveScreenDimensions();
  _screenmaxx = TextGraphics::SCREENLEN;
  _screenmaxy = TextGraphics::SCREENHT;
}

/****************************************************************************/
WinCoordsData::WinCoordsData(const WinCoordsData& Obj_):
_Allocated(false),
_screenxpos(1),
_screenypos(1),
_screenmaxx(0),
_screenmaxy(0),

_winboxxpos(1),
_winboxypos(1),
_winboxstartx(Obj_._winboxstartx),
_winboxstarty(Obj_._winboxstarty),
_winboxmaxx(Obj_._winboxmaxx),
_winboxmaxy(Obj_._winboxmaxy),

_winviewxoffset(0),
_winviewyoffset(0),
_winviewxpos(1),
_winviewypos(1),
_winviewmaxx(Obj_._winviewmaxx),
_winviewmaxy(Obj_._winviewmaxx)
{
  TextGraphics::RetrieveScreenDimensions();
  _screenmaxx = TextGraphics::SCREENLEN;
  _screenmaxy = TextGraphics::SCREENHT;
}

/****************************************************************************/
WinCoordsData* WinCoordsData::Make(int WinStartx_, int WinStarty_,
                                   int WinMaxx_, int WinMaxy_)
{
  WinCoordsData* ret = new WinCoordsData(WinStartx_, WinStarty_, WinMaxx_, WinMaxy_);
  
  if (ret)
    ret->_Allocated = true;

  return ret;
}

/****************************************************************************/
WinCoordsData* WinCoordsData::Make(const WinCoordsData& Obj_)
{
  WinCoordsData* ret = new WinCoordsData(Obj_);
  
  if (ret)
    ret->_Allocated = true;

  return ret;
}

/****************************************************************************/
WinCoordsData& WinCoordsData::operator = (const WinCoordsData& Obj_)
{
  if (this != &Obj_)
  {
    _winboxstartx = Obj_._winboxstartx;
    _winboxstarty = Obj_._winboxstarty;
    _winboxmaxx = Obj_._winboxmaxx;
    _winboxmaxy = Obj_._winboxmaxy;

    _winviewmaxx = Obj_._winviewmaxx;
    _winviewmaxy = Obj_._winviewmaxx;

    TextGraphics::RetrieveScreenDimensions();
    _screenmaxx = TextGraphics::SCREENLEN;
    _screenmaxy = TextGraphics::SCREENHT;
  }

  return *this;
}

/****************************************************************************/
void WinCoordsData::CopyCursorPosition(const WinCoordsData& Obj_)
{
  _screenxpos = Obj_._screenxpos;
  _screenypos = Obj_._screenypos;

  _winboxxpos = Obj_._winboxxpos;
  _winboxypos = Obj_._winboxypos;

  _winviewxoffset = Obj_._winviewxoffset;
  _winviewyoffset = Obj_._winviewyoffset;
  _winviewxpos = Obj_._winviewxpos;
  _winviewypos = Obj_._winviewypos;
}

/****************************************************************************/
void WinCoordsData::SetWinBoxDimensions(int WinStartx_, int WinStarty_,
                                        int WinMaxx_, int WinMaxy_)
{
  _winboxstartx = WinStartx_;
  _winboxstarty = WinStarty_;
  _winboxmaxx = WinMaxx_;
  _winboxmaxy = WinMaxy_;
}

/****************************************************************************/
void WinCoordsData::SetWinViewMaxXY(int maxx_, int maxy_)
{
  _winviewmaxx = maxx_;
  _winviewmaxy = maxy_;
}

/****************************************************************************/
int WinCoordsData::WinViewLength() const
{
  return (_winviewmaxx - _winboxstartx + 1);
}

/****************************************************************************/
int WinCoordsData::WinViewHeight() const
{
  return (_winviewmaxy - _winboxstarty + 1);
}

/****************************************************************************/
int WinCoordsData::WinBoxLength() const
{
  return (_winboxmaxx - _winboxstartx + 1);
}

/****************************************************************************/
int WinCoordsData::WinBoxHeight() const
{
  return (_winboxmaxy - _winboxstarty + 1);
}

/****************************************************************************/
// WindowLine::SharedChainData class definition
/****************************************************************************/
WindowLine::SharedChainData::SharedChainData(WindowLine* wlptr):
_parent(wlptr),
_currentrow(NULL),
_rowcursor(NULL),

_InWinMode(false),
_PrevLineReady(false),
_NextLineReady(false),
_ReturnOnLastLine(false),
_LastLineTerminated(false),
_AddNewLinesWhenFull(false),
_WinLinePosTransfer(false),

_rowcursorfound(false),
_movedone(false),
_rowpossaved(false),
_hasavailemptyline(false),
_numnextlines(UNINIT_NL),
_rowpos(0),
_savedrowpos(0),
_availlineno(0),
_newlineslimit(-1)
{}

/****************************************************************************/
void WindowLine::SharedChainData::SetParent(WindowLine* wlptr)
{
  _parent = wlptr;
}

/****************************************************************************/
// WindowLine class definition
/****************************************************************************/
// PURPOSE:
//   Prompt window default constructor
//
// PRE:
//   an uninitialize Prompt window
//
// POST:
//   a Prompt window is constructed and assigned the default values
//   specified.
//
WindowLine::WindowLine(int Max_, int InputLen_, int VbufLen_,
                       WinCoordsData* CoordData_):
_pbox(NULL),
_Info(new ControlInfo((Max_ > 0) ? Max_:1)),
_nextline(NULL),
_prevline(NULL),
_RefPtr(NULL),
_DummyRef(NULL),

_SharedDataErased(false),
_SharedData(new WindowLine::SharedChainData),

_LineTerminated(false),
_LineWrapAllowed(true),
_ShowChainedBoxes(false),
_WindowLineActive(true),
_EmptyLine(true),

_linestatus(UNINIT_NL),
_maxbox((Max_ > 0) ? Max_:1),
_boxcount(0),

_WinCoords(CoordData_),
_inputlen(0),
_vbuflen(VbufLen_),
_maxwraparound(0),
_wraparoundindex(0),
_WindowLineNum(0)
{
  int index;
  WindowLineBuffer* pb;
  WindowLine::SharedChainData* sdp = SharedData();

  if (sdp && _linestatus == UNINIT_NL)
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;

  if (_SharedData)
    InitRowCursor();

  _DummyRef = new WindowLineRef(this, -1);
  SetInputLength(InputLen_);
  MaxControl((Max_ > 0) ? Max_:1);

  if (_WinCoords && _pbox)
  {
    if (VbufLen_)
    {
      if (VbufLen_ < InputLen_)
        VbufLen_ = InputLen_;

      pb = WindowLineBuffer::Make(NULL, NULL, NULL, VbufLen_);
    }
    else
      pb = WindowLineBuffer::Make(NULL, NULL, this);

    if (pb)
    {
      if (VbufLen_)
      {
        pb->SetEndBrksActive(true);
        pb->SetInputLength(InputLen_, true);
      }
      else
        pb->SetEndBrksActive(false);

      pb->SetXpos(_WinCoords->_winboxstartx);
      pb->SetYpos(_WinCoords->_winboxstarty);
      (*this)[0] = pb;

      if (!VbufLen_)
      {
        if (Max_ <= 0)
          Max_ = 1;
      
        for (index = 1; index < Max_; index++)
          ((WindowLineBuffer*)((*this)[0]))->Chain(WindowLineBuffer::MakeNextLine(pb));
      }
    }
  }
}

/****************************************************************************/
// PURPOSE:
//   Prompt window copy construcor
//
// PRE:
//   An uninitialized Prompt window.
//
// POST:
//   a Prompt window is constructed and assigned the corresponding values
//   in the Prompt window argument passed to the constructor.
//
WindowLine::WindowLine(WindowLine& pw):
_pbox(NULL),
_Info(new ControlInfo(pw._maxbox)),
_nextline(NULL),
_prevline(NULL),
_RefPtr(NULL),
_DummyRef(NULL),

_SharedDataErased(false),
_SharedData(pw._SharedData),

_LineTerminated(pw._LineTerminated),
_LineWrapAllowed(pw._LineWrapAllowed),
_ShowChainedBoxes(pw._ShowChainedBoxes),
_WindowLineActive(pw._WindowLineActive),
_EmptyLine(pw._EmptyLine),

_linestatus(UNINIT_NL),
_maxbox(pw._maxbox),
_boxcount(pw._boxcount),

_WinCoords(pw._WinCoords ? WinCoordsData::Make(*pw._WinCoords):NULL),
_inputlen(pw._inputlen),
_vbuflen(pw._vbuflen),
_maxwraparound(pw._maxwraparound),
_wraparoundindex(0),
_WindowLineNum(pw._WindowLineNum)
{
  int index = 0;
  int VbufLen_ = 0;

  WindowLine::SharedChainData* sdp = SharedData();
  WindowLineBuffer* wlbptr = NULL;
  WindowLineBuffer* rootwlbptr = NULL;

  if (sdp && _linestatus == UNINIT_NL)
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;

  if (_maxbox)
  {
    NewPboxes(_maxbox);
    NewRefPtrs(_maxbox);
  }

  if (!sdp->_parent && _SharedData)
    InitRowCursor();
  
  _DummyRef = new WindowLineRef(this, -1);
  SetInputLength(pw._inputlen);

  if (_RefPtr)
    _RefPtr[0] = NULL;

  rootwlbptr = SetupFirstBox(pw, VbufLen_);

  if (!VbufLen_ && rootwlbptr)
    for (index = 1; index < _maxbox; index++)
    {
      if (_RefPtr)
        _RefPtr[index] = NULL;

      if (pw[index] && _pbox)
      {
        _pbox[index] = WindowLineBuffer::Make(*pw[index]);
        SetControlData(_pbox, _Info, index, false);

        if (_pbox[index] && rootwlbptr && index > 0)
          LinkNextControl(_pbox, index-1);
      }
      else if (!HasControlArrays(&pw))
        RemoveControlData(_pbox, _Info, index, false);
    }

  // Omit copy construction of next window line
  // if (pw._nextline)
  //   _nextline = new WindowLine(*pw._nextline);
}

/****************************************************************************/
WindowLine::~WindowLine()
{
  Clear();
  
  delete _DummyRef;
  _DummyRef = NULL;

  if (_WinCoords && _WinCoords->_Allocated)
  {
    delete _WinCoords;
    _WinCoords = NULL;
  }

  EraseSharedChainData(_SharedData);
}

/****************************************************************************/
/****************************************************************************/
void WindowLine::AssignSharedData(bool& InWinMode_,
                                  bool& NextLineReady_, bool& PrevLineReady_)
{
  WindowLine::SharedChainData* sdp = SharedData();

  if (sdp)
  {
    InWinMode_ = sdp->_InWinMode;
    NextLineReady_ = sdp->_NextLineReady;
    PrevLineReady_ = sdp->_PrevLineReady;
  }
}

/****************************************************************************/
TextControl** WindowLine::NewPboxes(size_t max, bool classdata_)
{
  size_t x;
  TextControl** arrptr_ = NULL;

  if (max)
  {
    arrptr_ = new TextControl*[max];
    for (x = 0; x < max; x++)
      arrptr_[x] = NULL;
  }

  if (classdata_)
    _pbox = arrptr_;

  return arrptr_;
}

/****************************************************************************/
WindowLineRef** WindowLine::NewRefPtrs(size_t max, bool classdata_)
{
  size_t x;
  WindowLineRef** arrptr_ = NULL;

  if (max)
  {
    arrptr_ = new WindowLineRef*[max];
    for (x = 0; x < max; x++)
      arrptr_[x] = NULL;
  }

  if (classdata_)
    _RefPtr = arrptr_;

  return arrptr_;
}

/****************************************************************************/
void WindowLine::EraseSharedChainData(WindowLine::SharedChainData* ptr)
{
  if (ptr && ptr->_parent == this)
  {
    delete _SharedData;
    _SharedData = NULL;
  }

  _SharedDataErased = true;
  if (_nextline)
    _nextline->EraseSharedChainData(NULL);
}

/****************************************************************************/
void WindowLine::FindLineStatus()
{
  if (_nextline == NULL && _prevline == NULL)
    _linestatus = UNINIT_NL;
  else if (_nextline)
    _linestatus = INIT_NL;
  else if (_prevline)
    _linestatus = JOINED_NL;

  _EmptyLine = IsEmpty();
}

/****************************************************************************/
WindowLineBuffer* WindowLine::SetupFirstBox(WindowLine& pw, int& VbufLen_)
{
  int InputLen_ = 0;
  WindowLineBuffer* rootwlbptr = NULL;

  if (_pbox && _WinCoords)
  {
    if (pw[0])
    {
      VbufLen_ = pw[0]->VirtualBufferMax();
      InputLen_ = pw[0]->InputLength();

      if (VbufLen_)
      {
        if (VbufLen_ < InputLen_)
          VbufLen_ = InputLen_;

        _pbox[0] = WindowLineBuffer::Make(NULL, NULL, NULL, VbufLen_);
        ((WindowLineBuffer*)_pbox[0])->SetEndBrksActive(true);
        ((WindowLineBuffer*)_pbox[0])->SetInputLength(InputLen_, true);
      }
      else
      {
        _pbox[0] = WindowLineBuffer::Make(*pw[0]);
        ((WindowLineBuffer*)_pbox[0])->SetEndBrksActive(false);
      }
    }
    else
    {
      _pbox[0] = WindowLineBuffer::Make(NULL, NULL, this);
      ((WindowLineBuffer*)_pbox[0])->SetEndBrksActive(false);
    }
    
    if (_pbox[0])
    {
      ((WindowLineBuffer*)_pbox[0])->SetXpos(_WinCoords->_winboxstartx);
      ((WindowLineBuffer*)_pbox[0])->SetYpos(_WinCoords->_winboxstarty);
    }

    SetControlData(_pbox, _Info, 0, false);
    rootwlbptr = (WindowLineBuffer*)(_pbox[0]);
  }
  else if (!HasControlArrays(&pw))
    ReplaceControlData(_pbox, _Info, 0, false);

  return rootwlbptr;
}

/****************************************************************************/
// PURPOSE:
//   Prompt box assignment operator
//
// PRE:
//   The data members of this Prompt box object with original values
//
// POST:
//   The data members of this object is assigned to the corresponding
//   data members of the Prompt box argument passed to the method.
//   This object is returned.
//
WindowLine& WindowLine::operator = (WindowLine& wline_)
{
  if (this != &wline_)
  {
    Clear();
    
    int index = 0;
    int VbufLen_ = 0;
    
    WindowLine::SharedChainData* sdp = SharedData();
    WindowLineBuffer* wlbptr = NULL;
    WindowLineBuffer* rootwlbptr = NULL;

    if (_WinCoords && _WinCoords->_Allocated)
    {
      delete _WinCoords;
      _WinCoords = NULL;
    }

    // new
    _pbox = NULL;
    _Info = new ControlInfo(wline_._maxbox);
    _nextline = NULL;
    _RefPtr = NULL;
    _DummyRef = NULL;

    _LineTerminated = wline_._LineTerminated;
    _LineWrapAllowed = wline_._LineWrapAllowed;
    _ShowChainedBoxes = wline_._ShowChainedBoxes;
    FindLineStatus();

    if (sdp && _linestatus == UNINIT_NL)
      if (sdp->_numnextlines < 0)
        sdp->_numnextlines = 0;

    if ((sdp && sdp->_parent != this) ||
        (!_SharedData && wline_._SharedData && wline_._SharedData->_parent == &wline_))
      _SharedData = wline_._SharedData;
    else if (!_SharedData)
    {
      _SharedData = new WindowLine::SharedChainData;
      if (_SharedData)
        _SharedData->_parent = this;
    }

    if (!sdp->_parent && _SharedData)
      InitRowCursor();    

    _maxbox = wline_._maxbox;
    _boxcount = wline_._boxcount;

    _WinCoords = wline_._WinCoords ? WinCoordsData::Make(*wline_._WinCoords):NULL;
    SetInputLength(wline_._inputlen);
    _vbuflen = wline_._vbuflen;
    _maxwraparound = wline_._maxwraparound;
    _wraparoundindex = 0;
    _WindowLineNum = wline_._WindowLineNum;
    ResetLineData(true);

    // allocating memory and data
    if (_maxbox)
    {
      NewPboxes(_maxbox);
      NewRefPtrs(_maxbox);
    }

    if (_RefPtr)
      _RefPtr[0] = NULL;

    rootwlbptr = SetupFirstBox(wline_, VbufLen_);

    if (!VbufLen_ && rootwlbptr)
      for (index = 1; index < _maxbox; index++)
      {
        if (_RefPtr)
          _RefPtr[index] = NULL;

        if (wline_[index] && _pbox)
        {
          _pbox[index] = WindowLineBuffer::Make(*wline_[index]);
          SetControlData(_pbox, _Info, index, false);

          if (_pbox[index] && rootwlbptr && index > 0)
            LinkNextControl(_pbox, index-1);
        }
        else if (!HasControlArrays(&wline_))
          RemoveControlData(_pbox, _Info, index, false);
      }

    FindControlCount();
  }

  return *this;
}

/****************************************************************************/
WindowLine* WindowLine::Make(WindowLine& wline_)
{
  WindowLine* wl;
  wl = new WindowLine(wline_);
  return wl;
}

/****************************************************************************/
WindowLine* WindowLine::Make(int Max_, int InputLen_, int VbufLen_, WinCoordsData* CoordData_)
{
  WindowLine* wl;
  wl = new WindowLine(Max_, InputLen_, VbufLen_, CoordData_);
  return wl;
}

/****************************************************************************/
WindowLine* WindowLine::MakeNextLine(WindowLine* wl)
{
  WindowLine* nwl;
  nwl = wl ? (new WindowLine(*wl)):NULL;
  WindowLine::SharedChainData* sdp = nwl ? nwl->SharedData():NULL;

  if (wl && nwl && sdp)
  {  
    nwl->_WindowLineNum = wl->_WindowLineNum + 1;
    wl->SetRowCursor(nwl, false);
    wl->SetupChainWlineObject(nwl, true);

    nwl->Erase();
    nwl->SetInputLength(wl->BoxInputLength());
    nwl->SetXpos(wl->GetXpos());
    nwl->SetYpos(wl->GetYpos()+1);
    wl->RestoreRowCursor(TO_CURRENT);
  }
  else if (!nwl->_pbox)
  {
    delete nwl;
    nwl = NULL;
  }

  return nwl;
}

/****************************************************************************/
/****************************************************************************/
void WindowLine::SetRowCursor(WindowLine* ptr, bool AsCurrent_)
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;

  sdp->_rowcursor = ptr;

  if (AsCurrent_)
  {
    sdp->_currentrow = ptr;

    if (ptr != this)
    {
      sdp->_currentrow->Activate();
      Deactivate();
    }
  }

  sdp->_rowpos = sdp->_rowcursor->_WindowLineNum;
}

/****************************************************************************/
void WindowLine::SetEscaped(bool Flag_)
{
  Deactivate();

  if (_pbox && _pbox[0])
    ((WindowLineBuffer*)_pbox[0])->SetEscaped(Flag_);
}

/****************************************************************************/
void WindowLine::Activate()
{
  // new
  _WindowLineActive = true;
  
  if (_pbox && _pbox[0])
    _pbox[0]->Activate();
}

/****************************************************************************/
void WindowLine::Deactivate()
{
  // new
  _WindowLineActive = false;
  
  if (_pbox && _pbox[0])
    _pbox[0]->Deactivate();
}

/****************************************************************************/
void WindowLine::InitRowCursor()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;

  sdp->_parent =
  sdp->_rowcursor =
  sdp->_currentrow = this;
  sdp->_rowpos = 0;
}

/****************************************************************************/
void WindowLine::RestoreRowCursor(int DestPos_)
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;

  if (DestPos_ == TO_PARENT)
    sdp->_rowcursor = sdp->_parent;
  else if (DestPos_ == TO_CURRENT)
    sdp->_rowcursor = sdp->_currentrow;

  sdp->_rowpos = sdp->_rowcursor->_WindowLineNum;
}

/****************************************************************************/
int WindowLine::FindMaxWrapAround()
{
  int ret = 0;
  int box_ = 0;
  int endbox_ = 0;
  int startbox_ = 0;
  int endpt_ = 0;
  int NextNonWsLen_ = 0;
  int NonWsLen_ = 0;
  int EndReached_ = 0;
  int BufChainPos_ = 0;
  int startlen_ = 0;
  int index_ = 0;

  bool DoChain_ = false;
  bool WrapCond_ = false;
  bool OverLimit_ = false;
  bool enterb_ = false;
  bool vbuf_ = false;
  bool inwbox_ = false;

  if (_pbox && _pbox[0])
  {
    BufChainPos_ = _pbox[0]->BufferChainPosition();
    
    enterb_ = _pbox[0]->IsEnterable();
    vbuf_ = BufChainPos_ == 0 && enterb_ &&
            _pbox[0]->VirtualBufferOffset(&EndReached_, &NextNonWsLen_);
    inwbox_ = BufChainPos_ == 0 && enterb_ && !vbuf_ &&
              LineWrapAllowed();
  }
  
  WindowLineBuffer* wlbptr_ = NULL;
  char* SavedRetBuf_ = NULL;
  char* Str_ = NULL;

  if (_maxbox && vbuf_)
  {
    if (_pbox[0]->VirtualBufferMax())
      ret = _pbox[0]->NextLength();
    else
      ret = 0;

    _maxwraparound = ret;
  }
  else if (_pbox && _pbox[0])
  {
    ret = GetBoxCount(true);

    if (_boxcount > 1)
    {
      wlbptr_ = (WindowLineBuffer*)(_pbox[0]);
      if (wlbptr_ && inwbox_)
        wlbptr_->SetCountJoinedChars(true);

      for (endpt_ = 0; _pbox[endpt_]; endpt_++);
      for (ret = (endpt_ < _boxcount) ? (endpt_-1):(_boxcount-1);
           ret && _pbox[ret-1]->NextLength() <= 0; ret--);

      if (ret > 0 && wlbptr_)
      {
        index_ = ret;
        --ret;
        
        endbox_ = _pbox[ret]->NextLength();
        startbox_ = _pbox[0]->ThisLength();
        startlen_ = wlbptr_->InputLength();

        _maxwraparound = startbox_ + endbox_;
        _maxwraparound += ret * _inputlen;

        if (_maxwraparound / _inputlen > 1)
        {
          box_ = _maxwraparound % _inputlen;
          
          if (box_ == 0)
            box_ = _inputlen;
        }

        _maxwraparound -= startlen_;
        ret = _maxwraparound;
      }
      else
        ret = box_ = _maxwraparound = 0;

      if (wlbptr_ && inwbox_)
      {
        if (wlbptr_->_retbuf)
        {
          Str_ = wlbptr_->_retbuf;
          SavedRetBuf_ = strcpy(new char[strlen(Str_)+1], Str_);
        }
        
        Str_ = wlbptr_->StrRetrieve(false, false);
        endpt_ = wlbptr_->InputLength() - 1;
        DoChain_ = wlbptr_->IsChainable(true);
        
        WrapCond_ = (wlbptr_->BufferIndex() == ::SafeStrLen(Str_) - 1 ||
                     wlbptr_->BufferIndex() == endpt_) &&
                    DoChain_ && _wraparoundindex < _maxwraparound;

        OverLimit_ = wlbptr_->BufferIndex() <= ::SafeStrLen(Str_) &&
                     wlbptr_->BufferIndex() <= endpt_ &&
                     (!wlbptr_->InWBoxCount() ||
                       (wlbptr_->InWBoxCount() > 0 &&
                         (_wraparoundindex < _maxwraparound ||
                           (wlbptr_->InWBoxCount() >= _maxwraparound &&
                              _wraparoundindex <= wlbptr_->InWBoxCount()))));

        if (!WrapCond_ && OverLimit_ && box_)
        {
          #if WINLINE_ONLYCWSCHR
            NonWsLen_ = wlbptr_->HasNonWsChars(Str_);
          #else
            NonWsLen_ = wlbptr_->HasNonNullChars(Str_);
          #endif
        
          if (index_ > 0 && NonWsLen_ > 0 && NonWsLen_ < _inputlen)
          {
            --index_;
            _maxwraparound = index_ * _inputlen + endbox_;
            _maxwraparound -= (_inputlen - NonWsLen_);
            ret = _maxwraparound;
          }
        }

        if (SavedRetBuf_)
        {
          strcpy(wlbptr_->_retbuf, SavedRetBuf_);
          delete[] SavedRetBuf_;
          SavedRetBuf_ = NULL;
        }
        
        wlbptr_->SetCountJoinedChars(false);
      }
    }
    else
      ret = 0;
  }

  return ret;
}

/****************************************************************************/
int WindowLine::FindControlCount()
{
  int index;
  int cnt = 0;

  if (_pbox && _maxbox)
  {
    for (index = 0; index < _maxbox; index++)
      if (_pbox[index])
        ++cnt;

    _boxcount = cnt;
  }

  return cnt;
}

/****************************************************************************/
int WindowLine::GetBoxCount(bool Contiguous_)
{
  int index;
  int cnt = 0;

  if (Contiguous_)
  {
    if (_pbox && _maxbox)
    {
      for (index = 0; index < _maxbox; index++)
        if (_pbox[index])
          ++cnt;
        else
          break;
    }
  }
  else
    cnt = FindControlCount();

  return cnt;
}

/****************************************************************************/
void WindowLine::ResetLineData(bool ResetRow_)
{
  // new
  WindowLine::SharedChainData* sdp = SharedData();

  _LineTerminated = false;
  _maxwraparound = FindMaxWrapAround();
  _wraparoundindex = 0;

  if (sdp)
  {
    sdp->_InWinMode = false;
    sdp->_PrevLineReady = false;
    sdp->_NextLineReady = false;
    sdp->_LastLineTerminated = false;

    sdp->_movedone = false;
    sdp->_rowpossaved = false;
    sdp->_hasavailemptyline = false;
    sdp->_savedrowpos = 0;
    sdp->_availlineno = 0;

    if (ResetRow_)
      RestoreRowCursor(TO_PARENT);
  }
}

/****************************************************************************/
bool WindowLine::IsAtEndPt()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return false;

  return (sdp->_rowpos == sdp->_numnextlines);
}

/****************************************************************************/
bool WindowLine::EndLineEmpty()
{
  // new
  WindowLine* ptr = this;
  
  while (ptr->_nextline)
    ptr = ptr->_nextline;

  return (ptr ? ptr->IsEmpty():false);
}

/****************************************************************************/
bool WindowLine::AtLineChainLimit()
{
  // new
  WindowLine::SharedChainData* sdp = SharedData();

  if (sdp)
    return ((sdp->_newlineslimit < 0) ?
               false:(sdp->_numnextlines >= sdp->_newlineslimit));

  return false;
}

/****************************************************************************/
bool WindowLine::AllLineFilled()
{
  // new
  WindowLine::SharedChainData* sdp = SharedData();
  WindowLine* ptr = this;
  bool Filled_ = !IsEmpty();
  
  while (Filled_ && ptr->_nextline)
  {
    ptr = ptr->_nextline;
    Filled_ = Filled_ && !ptr->IsEmpty();
  }

  if (sdp)
    sdp->_hasavailemptyline = !Filled_;

  return Filled_;
}

/****************************************************************************/
WindowLine* WindowLine::LastChainLine()
{
  // new
  WindowLine* lastline_ = this;
  while (lastline_->_nextline)
    lastline_ = lastline_->_nextline;

  return lastline_;
}

/****************************************************************************/
WindowLine* WindowLine::IncrLineChains(int NumWinLines_)
{
  // new
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || AtLineChainLimit())
    return NULL;
  
  sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  #if WINLINE_DEBUG3
    bool Debug_ = true;
  #else
    bool Debug_ = false;
  #endif
  
  WindowLine* lastline_ = NULL;
  WindowLine* newlinep_ = this;
  WindowLine* lineptr_ = this;

  int x;
  int newchains_ = 0;
  int maxlines_ = (sdp->_newlineslimit > 0) ?
                      sdp->_newlineslimit:
                      (sdp->_numnextlines + GROW_INCR);
  int growincr_ = 0;
  ++maxlines_;

  if (NumWinLines_)
  {
    newchains_ = NumWinLines_ - sdp->_numnextlines - 1;

    if (newchains_ < 0)
      newchains_ = 0;

    if (newchains_ > 0 && sdp->_numnextlines + newchains_ + 1 >= maxlines_)
      newchains_ = maxlines_ - sdp->_numnextlines - 1;
  }
  else
  {
    if (maxlines_ % GROW_INCR)
      growincr_ = GROW_INCR - (maxlines_ % GROW_INCR);
    else
      growincr_ = GROW_INCR;

    if ((sdp->_InWinMode || Debug_) && AllLineFilled())
      newchains_ += growincr_;

    if (newchains_ > 0 && sdp->_numnextlines + newchains_ + 1 >= maxlines_)
      newchains_ = maxlines_ - sdp->_numnextlines - 1;
  }

  for (x = 0; x < newchains_ && newlinep_; x++)
  {
    newlinep_ = WindowLine::MakeNextLine(lineptr_);
    
    if (newlinep_)
    {
      sdp->_parent->ChainWline(newlinep_, true);
      lineptr_ = newlinep_;
    }
  }

  if (newchains_ && newlinep_)
    lastline_ = LastChainLine();

  return lastline_;
}

/****************************************************************************/
WindowLine* WindowLine::DetachNext(WindowLine* Chainp_)
{
  // new
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return NULL;

  WindowLine* ptr = this;
  WindowLine* RetChain_ = NULL;

  if (Chainp_)
  {
    while (ptr && ptr != Chainp_)
      ptr = ptr->_nextline;

    if (ptr && ptr == Chainp_)
    {
      ptr = ptr->_nextline;
      Chainp_->_nextline = NULL;

      sdp->_numnextlines = ptr->WindowLineNum();
      SetWindowLineNum(ptr, 0);

      RetChain_ = ptr;
    }
  }

  return RetChain_;
}

/****************************************************************************/
WindowLine* WindowLine::DetachThis(WindowLine* Chainp_)
{
  WindowLine* ptr = NULL;
  WindowLine* RetChain_ = NULL;

  if (Chainp_)
  {
    ptr = DetachNext(Chainp_);
    RetChain_ = WindowLine::Make(*Chainp_);

    if (RetChain_)
    {
      Erase();
      SetWindowLineNum(RetChain_, 0);

      if (ptr && ptr != RetChain_)
        RetChain_->ChainWline(ptr, false);
    }
  }

  return RetChain_;
}

/****************************************************************************/
bool WindowLine::RemoveLineChains(WindowLine* wlptr)
{
  // new
  bool Removed_ = false;
  WindowLine* retptr = NULL;
  WindowLine* thisptr = NULL;

  if (wlptr)
  {
    retptr = DetachNext(wlptr);
    Removed_ = retptr != NULL;

    if (Removed_)
      while (retptr)
      {
        thisptr = retptr;
        retptr = retptr->_nextline;
        thisptr->_nextline = NULL;
        retptr->_prevline = NULL;
        delete thisptr;
      }
  }

  return Removed_;
}

/****************************************************************************/
void WindowLine::SaveLinePos()
{
  // new
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;

  sdp->_rowpossaved = true;
  sdp->_savedrowpos = _WindowLineNum;
}

/****************************************************************************/
WindowLine* WindowLine::RestoreLinePos()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || !sdp->_rowpossaved)
    return NULL;

  bool Found_ = false;
  RestoreRowCursor(TO_PARENT);
  
  while (sdp->_rowcursor)
  {
    Found_ = sdp->_rowcursor->_WindowLineNum == sdp->_savedrowpos;
    
    if (Found_)
      break;
    else
      sdp->_rowcursor = sdp->_rowcursor->_nextline;
  }

  if (Found_)
  {
    sdp->_rowpos = sdp->_rowcursor->_WindowLineNum;
    return sdp->_rowcursor;
  }
  else
    RestoreRowCursor(TO_PARENT);
  
  return NULL;
}

/****************************************************************************/
int WindowLine::GetWrapAroundIndex()
{
  return _wraparoundindex;
}

/****************************************************************************/
char* WindowLine::RetrieveInOrder()
{
  char* RetBuf_ = NULL;
  char* Str_ = NULL;
  
  int BufChainPos_ = 0;
  int BufIndex_ = 0;
  
  bool DoChain_ = false;
  bool enterb_ = false;
  bool vbuf_ = false;
  bool inwbox_ = false;

  if (_pbox && _pbox[0] &&
      !_pbox[0]->IsDummyLine() && _pbox[0]->IsWindowLineBuffer())
  {
    BufIndex_ = ((WindowLineBuffer*)_pbox[0])->BufferIndex();
    BufChainPos_ = _pbox[0]->BufferChainPosition();
    
    DoChain_ = _pbox[0]->IsChainable(false);
    enterb_ = _pbox[0]->IsEnterable();
    vbuf_ = BufChainPos_ == 0 && enterb_ &&
            _pbox[0]->VirtualBufferOffset();
    inwbox_ = BufChainPos_ == 0 && enterb_ && !vbuf_ &&
              LineWrapAllowed();

    while (BufIndex_ > 0 ||
           (GetWrapAroundIndex() > 0 && DoChain_ && (vbuf_ || inwbox_)))
    {
      #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
        ((WindowLineBuffer*)_pbox[0])->PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::LEFTARROW).SCAN + 256);
      #elif (defined(__linux__) | defined(__unix__))
        ((WindowLineBuffer*)_pbox[0])->PutExtKey(KEY_LEFT);
      #endif

      BufIndex_ = ((WindowLineBuffer*)_pbox[0])->BufferIndex();
      BufChainPos_ = _pbox[0]->BufferChainPosition();
      DoChain_ = _pbox[0]->IsChainable(false);
    }

    int x;
    int sz = ((WindowLineBuffer*)_pbox[0])->InputLength() * 2;
    sz += FindMaxWrapAround();

    int max = GetBoxCount(true);
    RetBuf_ = sz ? (new char[sz + 1]):(new char[WindowLineBuffer::MAXBUFFER + 1]);
    RetBuf_[0] = 0;

    for (x = 0; x < max; x++)
    {
      if (_pbox[x])
        Str_ = ((WindowLineBuffer*)_pbox[x])->StrRetrieve(false, true);
      else
        break;
      
      if (Str_)
        strcat(RetBuf_, Str_);
    }

    if (((WindowLineBuffer*)_pbox[0])->ShouldTrimWs() && RetBuf_)
      RetBuf_ = WindowLineBuffer::RemoveTrailSpaceChars(RetBuf_, false);
  }

  return RetBuf_;
}

/****************************************************************************/
WindowLine& WindowLine::SetWinBlockProperties(int InWin_,
                                              int RetOnLast_,
                                              int AddNewLines_,
                                              int NewLinesLimit_)
{
  WindowLine::SharedChainData* sdp = SharedData();

  if (sdp)
  {
    if (InWin_ != -99)
      sdp->_InWinMode = InWin_;
    else
      sdp->_InWinMode = true;

    if (RetOnLast_ != -99)
      sdp->_ReturnOnLastLine = RetOnLast_;
    else
      sdp->_ReturnOnLastLine = !sdp->_InWinMode;

    if (AddNewLines_ != -99)
      sdp->_AddNewLinesWhenFull = AddNewLines_;
    else
      sdp->_AddNewLinesWhenFull = false;

    if (NewLinesLimit_ != -99)
      sdp->_newlineslimit = NewLinesLimit_;
    else
      sdp->_newlineslimit = -1;
  }

  return *this;
}

/****************************************************************************/
WindowLine& WindowLine::SetWinLineProperties(int LwAllowed_,
                                             int ShowChained_,
                                             int RestrictFull_,
                                             int TrimWs_,
                                             int EndBrksActive_)
{
  if (LwAllowed_ != -99)
    _LineWrapAllowed = LwAllowed_;
  else
    _LineWrapAllowed = true;
  
  if (ShowChained_ != -99)
    _ShowChainedBoxes = ShowChained_;
  else
    _ShowChainedBoxes = !_LineWrapAllowed;

  if (_pbox && _pbox[0] &&
      !_pbox[0]->IsDummyLine() && _pbox[0]->IsWindowLineBuffer())
  {
    if (RestrictFull_ != -99)
      ((WindowLineBuffer*)_pbox[0])->SetRestrictFullBox(RestrictFull_);
    else
      ((WindowLineBuffer*)_pbox[0])->SetRestrictFullBox(_LineWrapAllowed);

    if (TrimWs_ != -99)
      ((WindowLineBuffer*)_pbox[0])->SetTrimWs(TrimWs_);
    else
      ((WindowLineBuffer*)_pbox[0])->SetTrimWs(((WindowLineBuffer*)_pbox[0])->RestrictFullBox());

    if (EndBrksActive_ != -99)
      ((WindowLineBuffer*)_pbox[0])->SetEndBrksActive(EndBrksActive_);
    else
      ((WindowLineBuffer*)_pbox[0])->SetEndBrksActive(((WindowLineBuffer*)_pbox[0])->VirtualBufferOffset());
  }
    
  return *this;
}

/****************************************************************************/
void WindowLine::SendUnprocessedKey(int Key_, int Extended_,
                                    WindowLineBuffer* WinBufPtr_)
{
  WindowLine::SharedChainData* sdp = SharedData();
  bool LastLineEnded_ = !_nextline && sdp->_ReturnOnLastLine;

  if (!Extended_ && WinBufPtr_->GetKeyMap().KeyMatch(Key_, TextControl::ENTER))
    if (!sdp->_InWinMode || LastLineEnded_)
    {    
      _LineTerminated = true;

      if (LastLineEnded_)
        sdp->_LastLineTerminated = true;
    }
}

/****************************************************************************/
WindowLineBuffer* WindowLine::ChainPbox(WindowLineBuffer* WinBufPtr_, int PboxNum_)
{
  int x;
  bool AtIndex_ = false;
  
  for (x = 0; x < _maxbox; x++)
  {
    AtIndex_ = x == PboxNum_;
    if (AtIndex_ && _pbox[x] == NULL)
      break;
  }

  if (AtIndex_ || x == _maxbox)
  {
    if (x == _maxbox)
    {
      while (_maxbox <= PboxNum_)
      {
        if (_maxbox % GROW_INCR)
          Grow(GROW_INCR - (_maxbox % GROW_INCR));
        else
          Grow(GROW_INCR);
      }

      x = PboxNum_;
      AtIndex_ = true;
    }

    if (AtIndex_)
    {
      (*this)[x] = WinBufPtr_;
      FindControlCount();

      return WinBufPtr_;
    }
  }

  return NULL;
}

/****************************************************************************/
void WindowLine::ReCalcChainWline(bool ReCalcSdp_, bool ReCalcStatus_)
{
  // new
  WindowLine* cur;
  int index;
  int x;

  if (!_Info && _maxbox)
    _Info = new ControlInfo(_maxbox);

  if (!_WinCoords)
    _WinCoords = WinCoordsData::Make(*_WinCoords);

  _LineTerminated = false;
  _WindowLineActive = false;
  _wraparoundindex = 0;

  WindowLine::SharedChainData* sdp = SharedData();

  if (sdp && ReCalcSdp_)
  {
    cur = sdp->_parent;
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;

    for (index = 0; cur->_nextline; index++)
      cur = cur->_nextline;

    sdp->_numnextlines = index;
    sdp->_LastLineTerminated = false;
  }

  if (!_pbox && _maxbox)
    NewPboxes(_maxbox);

  if (!_RefPtr && _maxbox)
    NewRefPtrs(_maxbox);

  if (!_DummyRef)
    _DummyRef = new WindowLineRef(this, -1);

  if (ReCalcStatus_)
  {
    FindLineStatus();
    FindControlCount();
    FindMaxWrapAround();
  }

  if (_pbox && _pbox[0])
  {
    if (_prevline)
    {
      cur = _prevline;
      SetXpos(cur->GetXpos());
      SetYpos(cur->GetYpos()+1);
    }
    else
    {
      SetXpos(_WinCoords->_winboxstartx);
      SetYpos(_WinCoords->_winboxstarty);
    }
  
    ((WindowLineBuffer*)_pbox[0])->ReCalcChainWbuffer(true, this);
    ((WindowLineBuffer*)_pbox[0])->ConfirmBufferChains(NULL, 0);
  }
}

/****************************************************************************/
WindowLine* WindowLine::SetupChainWlineObject(WindowLine* ptr, bool ChainPb_)
{
  if (!ptr)
    return ptr;

  WindowLine* cur;
  WindowLineBuffer* wlbptr = NULL;
  WindowLineBuffer* rootwlbptr = NULL;

  int VbufLen_ = 0;
  int index;

  if (!ptr->_Info)
    ptr->_Info = new ControlInfo(_maxbox);

  if (!ptr->_WinCoords)
    ptr->_WinCoords = _WinCoords ? WinCoordsData::Make(*_WinCoords):NULL;

  ptr->_nextline = NULL;
  ptr->_RefPtr = NULL;
  ptr->_DummyRef = NULL;

  ptr->_SharedDataErased = false;
  ptr->_SharedData = _SharedData;

  ptr->_LineTerminated = false;
  ptr->_LineWrapAllowed = _LineWrapAllowed;
  ptr->_ShowChainedBoxes = _ShowChainedBoxes;
  ptr->_WindowLineActive = false;

  ptr->_maxbox = _maxbox;
  ptr->_boxcount = _boxcount;

  ptr->_inputlen = _inputlen;
  ptr->_maxwraparound = _maxwraparound;
  ptr->_wraparoundindex = 0;
  ptr->_WindowLineNum = _WindowLineNum;

  WindowLine::SharedChainData* sdp = SharedData();

  if (sdp)
  {
    cur = sdp->_parent;
    if (sdp->_numnextlines < 0)
      sdp->_numnextlines = 0;

    for (index = 0; cur->_nextline; index++)
      cur = cur->_nextline;

    sdp->_numnextlines = index;
    sdp->_LastLineTerminated = false;
  }

  if (!ptr->_pbox && ptr->_maxbox)
    ptr->NewPboxes(ptr->_maxbox);

  if (!ptr->_RefPtr && ptr->_maxbox)
    ptr->NewRefPtrs(ptr->_maxbox);

  if (!ptr->_DummyRef)
    ptr->_DummyRef = new WindowLineRef(ptr, -1);

  rootwlbptr = ptr->SetupFirstBox(*this, VbufLen_);

  if (!VbufLen_ && rootwlbptr)
    for (index = 1; index < ptr->_maxbox; index++)
    {
      if (ptr->_RefPtr)
        ptr->_RefPtr[index] = NULL;

      if ((*this)[index] && ptr->_pbox)
      {
        ptr->_pbox[index] = WindowLineBuffer::Make(*((*this)[index]));
        ptr->SetControlData(ptr->_pbox, ptr->_Info, index, false);

        if (ptr->_pbox[index] && rootwlbptr && index > 0 && ChainPb_)
          ptr->LinkNextControl(ptr->_pbox, index-1);
      }
      else if (!ptr->HasControlArrays(this))
        ptr->RemoveControlData(ptr->_pbox, ptr->_Info, index, false);
    }

  ptr->FindControlCount();
}

/****************************************************************************/
WindowLine* WindowLine::ChainWline(WindowLine* next_, bool IncrNumChains_)
{
  WindowLine* retptr = NULL;
  WindowLine::SharedChainData* sdp = SharedData();

  if (_nextline)
  {
    retptr = _nextline->ChainWline(next_, IncrNumChains_);

    if (_WindowLineNum == 0 && retptr && retptr->_nextline)
    {
      if (retptr->_linestatus == UNINIT_NL ||
          retptr->_linestatus == JOINED_NL)
      {
        retptr->_linestatus = INIT_NL;
        retptr->_EmptyLine = retptr->IsEmpty();
      }

      if (sdp && IncrNumChains_)
        if (sdp->_numnextlines <= 0)
          sdp->_numnextlines = 1;
        else
          sdp->_numnextlines++;

      retptr->_nextline->_prevline = retptr;
      retptr->_nextline->FindLineStatus();
      retptr->_nextline->_SharedData = _SharedData;
    }
  }
  else
  {
    if (_WindowLineNum == 0)
      retptr = _nextline ? NULL:this;

    _nextline = next_;

    if (_nextline)
    {
      SetWindowLineNum(_nextline, _WindowLineNum+1);
      _nextline->FindLineStatus();
    }

    if (retptr && retptr->_nextline)
    {
      if (retptr->_linestatus == UNINIT_NL ||
          retptr->_linestatus == JOINED_NL)
      {
        retptr->_linestatus = INIT_NL;
        retptr->_EmptyLine = retptr->IsEmpty();
      }

      if (sdp && IncrNumChains_)
        if (sdp->_numnextlines <= 0)
          sdp->_numnextlines = 1;
        else
          sdp->_numnextlines++;

      retptr->_nextline->_prevline = retptr;
      retptr->_nextline->FindLineStatus();
      retptr->_nextline->_SharedData = _SharedData;
    }
  }

  return (retptr ? retptr:this);
}

/****************************************************************************/
bool WindowLine::InWindowMode(bool* HaveNext_, bool* HavePrev_)
{
  int x;
  int BaseNum_ = _WindowLineNum;
  bool NextAvail_;
  bool PrevAvail_;
  bool EmptyAvail_;

  WindowLine::SharedChainData* sdp = SharedData();
  WindowLine* ptr = this;
  WindowLine* lastlink_ = NULL;

  if (!sdp)
    return false;
  else
  {
    sdp->_availlineno = 0;
    sdp->_hasavailemptyline = false;

    if (HaveNext_)
      sdp->_NextLineReady = false;

    if (HavePrev_)
      sdp->_PrevLineReady = false;
  }

  lastlink_ = this;
  ptr = _nextline;

  for (x = BaseNum_+1; x <= sdp->_numnextlines && ptr; x++)
  {
    ptr->FindLineStatus();
    EmptyAvail_ = ptr->_EmptyLine && ptr->_linestatus == JOINED_NL;

    if (EmptyAvail_)
    {
      sdp->_availlineno = x;
      sdp->_hasavailemptyline = true;

      break;
    }
    else
    {
      lastlink_ = ptr;
      ptr = ptr->_nextline;
    }
  }

  if (sdp->_hasavailemptyline && ptr && lastlink_)
  {
    ptr = lastlink_;
    lastlink_ = lastlink_->_nextline;
  
    for (--x; EmptyAvail_ && x >= 0 && ptr; x--)
    {
      ptr->FindLineStatus();
      EmptyAvail_ = ptr->_EmptyLine &&
                    (ptr->_linestatus == JOINED_NL || ptr->_linestatus == INIT_NL);

      if (EmptyAvail_)
      {
        lastlink_ = ptr;
        ptr = ptr->_prevline;
      }
      else if (lastlink_)
      {
        sdp->_availlineno = lastlink_->_WindowLineNum;
        sdp->_hasavailemptyline = true;
        
        break;
      }
    }

    if (EmptyAvail_ && (x < 0 || !ptr))
    {
      ptr = lastlink_;
      EmptyAvail_ = ptr && ptr->_EmptyLine &&
                    (ptr->_linestatus == JOINED_NL || ptr->_linestatus == INIT_NL);

      if (EmptyAvail_ && ptr)
      {
        sdp->_availlineno = ptr->_WindowLineNum;
        sdp->_hasavailemptyline = true;
      }
    }
  }

  if (HaveNext_)
  {
    ptr = _nextline;

    for (x = BaseNum_+1; x <= sdp->_numnextlines && ptr; x++)
    {
      ptr->FindLineStatus();
      NextAvail_ = (ptr->_linestatus == JOINED_NL || ptr->_linestatus == INIT_NL);

      if (NextAvail_)
      {
        sdp->_NextLineReady = true;
        break;
      }
      else
        ptr = ptr->_nextline;
    }

    *HaveNext_ = sdp->_NextLineReady;
  }

  if (HavePrev_)
  {
    ptr = _prevline;

    for (x = BaseNum_-1; x >= 0 && ptr; x--)
    {
      ptr->FindLineStatus();
      PrevAvail_ = ptr->_linestatus == INIT_NL;

      if (PrevAvail_)
      {
        sdp->_PrevLineReady = true;
        break;
      }
      else
        ptr = ptr->_prevline;
    }

    *HavePrev_ = sdp->_PrevLineReady;
  }

  return sdp->_InWinMode;
}

/****************************************************************************/
bool WindowLine::AddNewWindowLines(int NewLines_)
{
  WindowLine::SharedChainData* sdp = SharedData();
  WindowLine* lastline_ = NULL;

  if (!sdp)
    return false;

  if (NewLines_)
    NewLines_ += sdp->_numnextlines + 1;

  SetRowCursor(this, false);
  SaveLinePos();
  lastline_ = IncrLineChains(NewLines_);
  RestoreLinePos();

  return (lastline_ != NULL);
}

/****************************************************************************/
bool WindowLine::AddNewLinesWhenFull() const
{
  const WindowLine::SharedChainData* sdp = ConstSharedData();
  return (sdp ? sdp->_AddNewLinesWhenFull:false);
}

/****************************************************************************/
bool WindowLine::ReturnOnLastLine() const
{
  const WindowLine::SharedChainData* sdp = ConstSharedData();
  return (sdp ? sdp->_ReturnOnLastLine:false);
}

/****************************************************************************/
void WindowLine::SetWindowLineNum(WindowLine* Wline_, int Index_)
{
  if (Wline_)
    if (this != Wline_)
      Wline_->SetWindowLineNum(Wline_, Index_);
    else
      _WindowLineNum = Index_;
}

/****************************************************************************/
int WindowLine::SetInputLength(int Length_)
{
  if (0 < Length_)
    if (_WinCoords)
      _inputlen = (Length_ > _WinCoords->_winboxmaxx) ?
                      _WinCoords->_winboxmaxx:
                      Length_;
    else
      _inputlen = Length_;

  return _inputlen;
}

/****************************************************************************/
void WindowLine::IncrWrapAround(int Num_)
{
  _maxwraparound = FindMaxWrapAround();

  if (_wraparoundindex <= _maxwraparound &&
      (0 < Num_ && Num_ <= (_maxwraparound-_wraparoundindex)+1))
  {
    _wraparoundindex += Num_;

    if (_wraparoundindex > _maxwraparound+1)
      _wraparoundindex = _maxwraparound+1;
  }

  if (_wraparoundindex < 0)
    _wraparoundindex = 0;
}

/****************************************************************************/
void WindowLine::DecrWrapAround(int Num_)
{
  if (_wraparoundindex > 0 &&
      (0 < Num_ && Num_ <= _wraparoundindex))
  {
    _wraparoundindex -= Num_;

    if (_wraparoundindex < 0)
      _wraparoundindex = 0;
  }

  if (_wraparoundindex < 0)
    _wraparoundindex = 0;
}

/****************************************************************************/
int WindowLine::WinBoxToScreenX(int xpos_)
{
  if (_WinCoords)
  {
    if (xpos_ < 0)
      xpos_ = _WinCoords->_winboxxpos;

    xpos_ += _WinCoords->_winboxstartx;

    if (xpos_ >= _WinCoords->_winboxmaxx || xpos_ < 1)
      xpos_ = -1;
  }
  else
  {
    if (xpos_ < 0)
      xpos_ = 0;
  }

  return xpos_;
}

/****************************************************************************/
int WindowLine::WinBoxToScreenY(int ypos_)
{
  if (_WinCoords)
  {
    if (ypos_ < 0)
      ypos_ = _WinCoords->_winboxypos;

    ypos_ += _WinCoords->_winboxstarty;

    if (ypos_ >= _WinCoords->_winboxmaxy || ypos_ < 1)
      ypos_ = -1;
  }
  else
  {
    if (ypos_ < 0)
      ypos_ = 0;
  }

  return ypos_;
}

/****************************************************************************/
int WindowLine::WinViewToWinBoxX(int xpos_)
{
  if (_WinCoords)
  {
    if (xpos_ < 0)
      xpos_ = _WinCoords->_winviewxpos;

    xpos_ -= _WinCoords->_winviewxoffset;

    if (xpos_ >= _WinCoords->_winboxmaxx || xpos_ < 1)
      xpos_ = -1;
  }
  else
  {
    if (xpos_ < 0)
      xpos_ = 0;
  }

  return xpos_;
}

/****************************************************************************/
int WindowLine::WinViewToWinBoxY(int ypos_)
{
  if (_WinCoords)
  {
    if (ypos_ < 0)
      ypos_ = _WinCoords->_winviewypos;

    ypos_ -= _WinCoords->_winviewyoffset;

    if (ypos_ >= _WinCoords->_winboxmaxy || ypos_ < 1)
      ypos_ = -1;
  }
  else
  {
    if (ypos_ < 0)
      ypos_ = 0;
  }

  return ypos_;
}

/****************************************************************************/
bool WindowLine::NoMatchesToPrevBoxes(WindowLineBuffer* Nbox_, WindowLineBuffer* PrevBox_,
                                      TextControl** Pbox_, int Maxbox_)
{
  int x;
  bool NoMatch_ = Nbox_ && Nbox_ != PrevBox_;

  for (x = 0; NoMatch_ && x < Maxbox_; x++)
    NoMatch_ = Nbox_ != Pbox_[x];

  return NoMatch_;
}

/****************************************************************************/
void WindowLine::EraseLineInfo(bool MemDelete_)
{
  if (MemDelete_)
  {
    Clear();

    _maxbox = 0;
    _boxcount = 0;
    _maxwraparound = 0;
    _wraparoundindex = 0;
    _inputlen = 0;
  }
  else
  {
    _pbox = NULL;
    _Info = NULL;
    _RefPtr = NULL;

    _maxbox = 0;
    _boxcount = 0;
    _maxwraparound = 0;
    _wraparoundindex = 0;
    _inputlen = 0;
  }  
}

/****************************************************************************/
WindowLine* WindowLine::CopyLineInfo(WindowLine* ptr_, bool DeepCopy_, int Length_)
{
  int index;
  int VbufLen_ = 0;
  
  WindowLine::SharedChainData* sdp = SharedData();
  WindowLineBuffer* rootwlbptr = NULL;
  WindowLineBuffer* wlbptr = NULL;

  if (DeepCopy_ && ptr_)
  {
    NewPboxes(ptr_->_maxbox);
    NewRefPtrs(ptr_->_maxbox);
    _Info = ptr_->_maxbox ? (new ControlInfo(ptr_->_maxbox)):NULL;
    _EmptyLine = ptr_->_EmptyLine;

    _maxbox = ptr_->_maxbox;
    _boxcount = ptr_->_boxcount;
    _maxwraparound = ptr_->_maxwraparound;
    _wraparoundindex = ptr_->_wraparoundindex;
    _inputlen = ptr_->_inputlen;

    if (sdp && _linestatus == UNINIT_NL)
      if (sdp->_numnextlines < 0)
        sdp->_numnextlines = 0;

    if ((sdp && sdp->_parent != this) ||
        (!_SharedData && ptr_->_SharedData && ptr_->_SharedData->_parent == ptr_))
      _SharedData = ptr_->_SharedData;
    else if (!_SharedData)
    {
      _SharedData = new WindowLine::SharedChainData;
      if (_SharedData)
        _SharedData->_parent = this;
    }

    rootwlbptr = SetupFirstBox(*ptr_, VbufLen_);

    if (!VbufLen_ && rootwlbptr)
      for (index = 1; index < _maxbox; index++)
      {
        if (_RefPtr)
          _RefPtr[index] = NULL;

        if ((*ptr_)[index] && _pbox && (!Length_ || index < Length_))
        {
          _pbox[index] = WindowLineBuffer::Make(*((*ptr_)[index]));
          SetControlData(_pbox, _Info, index, false);

          if (_pbox[index] && rootwlbptr && index > 0)
            LinkNextControl(_pbox, index-1);
        }
        else if (!HasControlArrays(ptr_))
          RemoveControlData(_pbox, _Info, index, false);
      }

    if (ptr_ == _prevline)
    {
      SetInputLength(ptr_->BoxInputLength());
      SetXpos(ptr_->GetXpos());
      SetYpos(ptr_->GetYpos()+1);
    }

    ReCalcChainWline(false, true);
  }
  else if (ptr_)
  {
    _pbox = ptr_->_pbox;
    _Info = ptr_->_Info;
    _RefPtr = ptr_->_RefPtr;
    _EmptyLine = ptr_->_EmptyLine;

    _maxbox = ptr_->_maxbox;
    _boxcount = ptr_->_boxcount;
    _maxwraparound = ptr_->_maxwraparound;
    _wraparoundindex = ptr_->_wraparoundindex;
    _inputlen = ptr_->_inputlen;

    if (sdp && _linestatus == UNINIT_NL)
      if (sdp->_numnextlines < 0)
        sdp->_numnextlines = 0;

    if ((sdp && sdp->_parent != this) ||
        (!_SharedData && ptr_->_SharedData && ptr_->_SharedData->_parent == ptr_))
      _SharedData = ptr_->_SharedData;
    else if (!_SharedData)
    {
      _SharedData = new WindowLine::SharedChainData;
      if (_SharedData)
        _SharedData->_parent = this;
    }

    rootwlbptr = _maxbox ? ((WindowLineBuffer*)_pbox[0]):NULL;

    if (rootwlbptr)
      for (index = 0; index < _maxbox; index++)
      {
        if (_RefPtr && _RefPtr[index])
          _RefPtr[index]->ResetLineRef(this, index);

        if (_pbox && _pbox[index] && (!Length_ || index < Length_))
        {
          _pbox[index] = (*ptr_)[index];
          SetControlData(_pbox, _Info, index, false);

          if (_pbox[index] && rootwlbptr && index > 0)
            LinkNextControl(_pbox, index-1);
        }
        else if (!HasControlArrays(ptr_))
          RemoveControlData(_pbox, _Info, index, false);
      }

    if (ptr_ == _prevline)
    {
      SetInputLength(ptr_->BoxInputLength());
      SetXpos(ptr_->GetXpos());
      SetYpos(ptr_->GetYpos()+1);
    }

    ReCalcChainWline(false, true);
  }

  return this;
}

/****************************************************************************/
void WindowLine::EraseBox(int BoxIndex_)
{
  // new
  if (!_pbox)
    return;
  
  int x = BoxIndex_;
  WindowLineBuffer* wbptr_ = (WindowLineBuffer*)(_pbox[x]);
  WindowLineBuffer* tptr_ = NULL;

  if (x < _maxbox && wbptr_)
    wbptr_->Erase();

  if (x >= _maxbox)
    if (_maxbox % GROW_INCR)
      Grow(GROW_INCR - (_maxbox % GROW_INCR));
    else
      Grow(GROW_INCR);

  FindControlCount();
}

/****************************************************************************/
void WindowLine::ReplaceBox(int BoxIndex_, TextControl* ptr_)
{
  // new
  if (!_pbox)
    return;
  
  int x = BoxIndex_;
  WindowLineBuffer* wbptr_ = (WindowLineBuffer*)ptr_;
  WindowLineBuffer* tptr_ = NULL;

  if (x >= _maxbox && wbptr_)
    if (_maxbox % GROW_INCR)
      Grow(GROW_INCR - (_maxbox % GROW_INCR));
    else
      Grow(GROW_INCR);

  while (wbptr_)
  {
    for (x = BoxIndex_; x < _maxbox && wbptr_; x++)
    {
      if (_pbox[x] != wbptr_)
      {
        InitializeBox(wbptr_, x, true, true);
        (*this)[x] = wbptr_;

        if (x > 0)
        {
          tptr_ = ((WindowLineBuffer*)_pbox[x-1]);
          if (tptr_->_nextline != wbptr_)
            tptr_->_nextline = wbptr_;
        }
      }
      
      wbptr_ = wbptr_->_nextline;
    }

    if (x == _maxbox && wbptr_)
      if (_maxbox % GROW_INCR)
        Grow(GROW_INCR - (_maxbox % GROW_INCR));
      else
        Grow(GROW_INCR);

    FindControlCount();
  }
}

/****************************************************************************/
void WindowLine::InitializeBox(WindowLineBuffer* wbptr_, int x,
                               bool CopyBox_, bool JoinBoxToEnd_)
{
  WindowLineBuffer* tptr_ = NULL;

  if (_pbox[x] && !_pbox[x]->VirtualBufferMax() && CopyBox_)
  {
    tptr_ = ((WindowLineBuffer*)_pbox[x]);
    wbptr_->_BufChainNum = tptr_->_BufChainNum;
    wbptr_->SetInputLength(tptr_->_len, false);
          
    wbptr_->SetXpos(tptr_->_xpos);
    wbptr_->SetYpos(tptr_->_ypos);

    if (_pbox[x] != wbptr_)
      DeleteControl(_pbox[x]);
  }
  else if (x && _pbox[x-1] && !_pbox[x-1]->VirtualBufferMax() && JoinBoxToEnd_)
  {
    tptr_ = ((WindowLineBuffer*)_pbox[x-1]);
    int msglen_ = ::SafeStrLen(tptr_->_msg);
    wbptr_->_BufChainNum = tptr_->_BufChainNum + 1;
    wbptr_->SetInputLength(tptr_->_len, false);
          
    wbptr_->SetXpos(tptr_->_xpos + msglen_ + tptr_->_len +
                    tptr_->EndBrksLength(false));
    wbptr_->SetYpos(tptr_->_ypos);
  }
  else if (_WinCoords && !_vbuflen && x == 0)
  {
    wbptr_->_BufChainNum = 0;
    wbptr_->SetEndBrksActive(false);
    wbptr_->SetInputLength(_inputlen, false);

    wbptr_->SetXpos(_WinCoords->_winboxstartx);
    wbptr_->SetYpos(_WinCoords->_winboxstarty);
  }
}

/****************************************************************************/
// Peform shift down of entered line to next WindowLine object
bool WindowLine::ShiftDown(char* NewStr_, int Offset_,
                           TextControl** Pbox_, int Boxdex_, int Maxbox_)
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return false;

  char* cStr_ = NULL;
  bool cStrShifted_ = false;
  bool ShiftDownDone_ = false;
  int mult_;
  int diff_;
  int len_;

  // new
  if (_nextline && sdp->_hasavailemptyline)
  {
    if (Offset_ < _inputlen)
    {
      // if passed Pbox_ and Maxbox_ is NULL and 0 then assign method
      // parameters with object's data members
      if (!Pbox_)
        Pbox_ = _pbox;

      if (!Maxbox_)
        Maxbox_ = _maxbox;

      // String offset is less than box input length and Pbox_ is specified
      if (Maxbox_ && Pbox_)
      {
        SetRowCursor(this, false);

        if (!Offset_ && Boxdex_ && Boxdex_ >= Maxbox_)
        {
          // shift down an empty line
          sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);
          ShiftDownDone_ = _nextline->ShiftDownNext(NULL, 0, Pbox_, Maxbox_, Maxbox_);
        }
        else if (Offset_ > 0 || Boxdex_)
        {
          len_ = NewStr_ ? strlen(&NewStr_[Offset_]):0;
          cStr_ = (NewStr_ && 0 <= len_ && Offset_ <= _inputlen) ?
                      strcpy(new char[_inputlen + 1], &NewStr_[Offset_]):NULL;
          sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);
          cStrShifted_ = cStr_ && Offset_ > 0;

          if (cStr_)
          {
            if (cStrShifted_)
              ShiftDownDone_ = _nextline->ShiftDownNext(cStr_, Offset_, Pbox_, Boxdex_+1, Maxbox_);
            else
              ShiftDownDone_ = _nextline->ShiftDownNext(NULL, 0, Pbox_, Boxdex_, Maxbox_);

            if (cStr_)
            {
              memset(cStr_, 0, _inputlen+1);
              ChainFillString(cStr_, len_, Offset_, Pbox_, Boxdex_, Maxbox_);
            }
          }
        }
        else
        {
          sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);
          ShiftDownDone_ = _nextline->ShiftDownNext(NULL, 0, Pbox_, Boxdex_, Maxbox_);
        }
      }
      else
      {
        // should be mult_ == 0, diff_ == 0 since Offset_ < _inputlen
        mult_ = Offset_ / _inputlen;
        diff_ = _inputlen * mult_;

        if (!mult_ && !diff_)
        {
          NewStr_ = ((WindowLineBuffer*)_pbox[0])->StrRetrieve();
          ShiftDownDone_ = ShiftDown(NewStr_, Offset_, _pbox, mult_, _maxbox);
        }
        else if (diff_ >= ::SafeStrLen(NewStr_) && diff_ >= _inputlen)
        {
          NewStr_ = ((WindowLineBuffer*)_pbox[mult_])->StrRetrieve();
          ShiftDownDone_ = ShiftDown(NewStr_, Offset_ - diff_, _pbox, mult_, _maxbox);
        }
      }
    }
    else
    {
      // should be mult_ >= 1, diff_ >= _inputlen since Offset_ >= _inputlen
      mult_ = Offset_ / _inputlen;
      diff_ = _inputlen * mult_;

      // should be diff_ > strlen(NewStr_) since input line is continuous
      // across chained boxes which includes white space characters.
      if (diff_ >= ::SafeStrLen(NewStr_) && diff_ >= _inputlen)
      {
        NewStr_ = ((WindowLineBuffer*)_pbox[mult_])->StrRetrieve();
        ShiftDownDone_ = ShiftDown(NewStr_, Offset_ - diff_, _pbox, mult_, _maxbox);
      }
    }
  }
  else if (AllLineFilled() && sdp->_AddNewLinesWhenFull && !AtLineChainLimit())
  {
    // increment new line chains if all lines in window block filled
    AddNewWindowLines();
    ShiftDownDone_ = ShiftDown(NewStr_, Offset_, Pbox_, Boxdex_, Maxbox_);
  }

  if (cStr_)
  {
    delete[] cStr_;
    cStr_ = NULL;
  }

  return ShiftDownDone_;
}

/****************************************************************************/
// Peform shift down of entered line to next WindowLine object
bool WindowLine::ShiftDownNext(char* NewStr_, int Offset_,
                               TextControl** Pbox_, int Boxdex_, int Maxbox_)
{
  WindowLineBuffer::SharedChainData* wlbsdp;
  WindowLine::SharedChainData* sdp = SharedData();
  
  if (!sdp)
    return false;

  // new
  int x;
  int wlinenum_;
  bool NoNewStr_ = false;
  bool NboxEmpty_ = false;
  bool NullBox_ = false;
  bool ShiftDownDone_ = false;
  bool ShiftDownEmptyLine_ = false;
  bool PboxesValid_ = Pbox_ && _pbox &&
                      Pbox_[0] && _pbox[0] && Pbox_ != _pbox;

  WindowLineBuffer* Nbox_ = NULL;
  WindowLineBuffer* Tptr_;
  WindowLineBuffer* PrevBox_ = NULL;
  WindowLineBuffer* FirstBox_ = NULL;
  WindowLine* ptr = this;
  WindowLine* OwnerWindow_;

  if (PboxesValid_)
  {
    if (Boxdex_ || !NewStr_)
    {
      if (Boxdex_ && NewStr_ && Boxdex_ < Maxbox_)
      {
        Tptr_ = ((WindowLineBuffer*)Pbox_[Boxdex_-1]);
        PrevBox_ = ((WindowLineBuffer*)Pbox_[Boxdex_]);
        FirstBox_ = ((WindowLineBuffer*)Pbox_[0]);
        
        if (PrevBox_)
        {
          if (PrevBox_->IsEmpty())
            NboxEmpty_ = true;
          else
            Nbox_ = FirstBox_->DetachNext(Tptr_, true);
        }
        else
        {
          NboxEmpty_ = true;
          NullBox_ = true;
        }

        if (Nbox_)
        {
          OwnerWindow_ = (WindowLine*)(Nbox_->_Owner);
          if (!NboxEmpty_)
            Nbox_->ChainSetOwner(this, Nbox_);

          if ((*OwnerWindow_)[0] == NULL)
            (*OwnerWindow_)[0] = WindowLineBuffer::Make(NULL, NULL, OwnerWindow_);
        }
        else if (NullBox_ && Tptr_)
        {
          OwnerWindow_ = (WindowLine*)(Tptr_->_Owner);

          if ((*OwnerWindow_)[Boxdex_] == NULL)
            ((WindowLineBuffer*)((*this)[0]))->Chain(WindowLineBuffer::MakeNextLine(Tptr_));
        }
      }
      else if (!NewStr_ && Boxdex_ < Maxbox_)
      {
        Nbox_ = ((WindowLineBuffer*)Pbox_[Boxdex_]);
        
        if (Boxdex_-1 >= 0 && Nbox_)
        {
          NoNewStr_ = true;
          
          Tptr_ = ((WindowLineBuffer*)Pbox_[Boxdex_-1]);
          PrevBox_ = ((WindowLineBuffer*)Pbox_[Boxdex_]);
          FirstBox_ = ((WindowLineBuffer*)Pbox_[0]);
          
          if (PrevBox_)
          {
            if (PrevBox_->IsEmpty())
              NboxEmpty_ = true;
            else
              Nbox_ = FirstBox_->DetachNext(Tptr_, true);
          }
          else
          {
            NboxEmpty_ = true;
            NullBox_ = true;
          }

          if (Nbox_)
          {
            OwnerWindow_ = (WindowLine*)(Nbox_->_Owner);
            if (!NboxEmpty_)
              Nbox_->ChainSetOwner(this, Nbox_);

            if ((*OwnerWindow_)[0] == NULL)
              (*OwnerWindow_)[0] = WindowLineBuffer::Make(NULL, NULL, OwnerWindow_);
          }
          else if (NullBox_ && Tptr_)
          {
            OwnerWindow_ = (WindowLine*)(Tptr_->_Owner);

            if ((*OwnerWindow_)[Boxdex_] == NULL)
              ((WindowLineBuffer*)((*this)[0]))->Chain(WindowLineBuffer::MakeNextLine(Tptr_));
          }
        }
        else if (Nbox_)
        {
          NoNewStr_ = true;
          
          Tptr_ = Nbox_;
          PrevBox_ = Nbox_;
          FirstBox_ = ((WindowLineBuffer*)Pbox_[0]);
          
          if (PrevBox_)
          {
            if (PrevBox_->IsEmpty())
              NboxEmpty_ = true;
            else
              Nbox_ = FirstBox_->DetachThis(Tptr_, true);
          }
          else
          {
            NboxEmpty_ = true;
            NullBox_ = true;
          }

          if (Nbox_)
          {
            OwnerWindow_ = (WindowLine*)(Nbox_->_Owner);
            if (!NboxEmpty_)
              Nbox_->ChainSetOwner(this, Nbox_);

            if ((*OwnerWindow_)[0] == NULL)
              (*OwnerWindow_)[0] = WindowLineBuffer::Make(NULL, NULL, OwnerWindow_);
          }
          else if (NullBox_ && Tptr_)
          {
            OwnerWindow_ = (WindowLine*)(Tptr_->_Owner);

            if ((*OwnerWindow_)[0] == NULL)
              (*OwnerWindow_)[0] = WindowLineBuffer::Make(NULL, NULL, OwnerWindow_);
          }
        }
      }
      else if (!NewStr_ && Boxdex_ >= Maxbox_)
      {
        // shift down an empty line
        ShiftDownEmptyLine_ = true;
        NoNewStr_ = true;
        NboxEmpty_ = true;
        Nbox_ = NULL;
      }

      // follow next link down to available line WindowLine object
      wlinenum_ = ptr->_WindowLineNum;
      if (sdp->_NextLineReady && sdp->_hasavailemptyline)
        for (x = wlinenum_;
             x < sdp->_availlineno &&
             ptr->_WindowLineNum != sdp->_availlineno; x++)
          ptr = ptr->_nextline;

      // shift down all lines until line where shifted down contents are
      // to be placed
      if (ptr->_WindowLineNum == sdp->_availlineno)
        for (;x > wlinenum_ && ptr->_WindowLineNum != wlinenum_; x--)
          if (ptr->_prevline)
          {
            ptr->CopyLineInfo(ptr->_prevline, false);
            ptr->_prevline->EraseLineInfo(false);
            ptr = ptr->_prevline;
          }

      if (!Nbox_ && !NboxEmpty_)
      {
        SetRowCursor(this, false);
        ShiftDownDone_ = false;
      }
      else if (ptr && ptr->_WindowLineNum == wlinenum_ && ptr->_prevline &&
               (!ptr->_pbox || ptr->IsEmpty()))
      {
        ShiftDownDone_ = true;
        SetRowCursor(ptr, false);
        if (ptr->_pbox)
          ptr->EraseLineInfo(true);
      
        ptr->CopyLineInfo(ptr->_prevline, true, (NewStr_ ? 1:0));

        if (NewStr_ && !ShiftDownEmptyLine_)
        {
          ptr->SetBoxToString(NewStr_, 0);
          Tptr_ = (*ptr)[0];
          OwnerWindow_ = (WindowLine*)(Tptr_->_Owner);
          
          if (Tptr_ && Nbox_ && Tptr_ != Nbox_ && !NoNewStr_ && !NboxEmpty_)
          {
            if (FirstBox_)
              Tptr_->CopyEndLineCharData(OwnerWindow_, FirstBox_);

            Tptr_->AddBoxChain(Nbox_, false);
          }
          else if (Nbox_)
          {
            Nbox_->InitFirstBox(OwnerWindow_);
          
            if (NboxEmpty_)
              Nbox_->CopyEndLineCharData(OwnerWindow_, NULL, true);
            else if (!NullBox_ && FirstBox_ && Nbox_ != FirstBox_)
              Tptr_->CopyEndLineCharData(OwnerWindow_, FirstBox_);
          }

          if (Tptr_)
          {
            if (NboxEmpty_)
            {
              ptr->EraseBox(1);

              if (Nbox_ && NoMatchesToPrevBoxes(Nbox_, PrevBox_, Pbox_, Maxbox_))
              {
                delete Nbox_;
                Nbox_ = NULL;
              }
            }
            else if (Nbox_)
              ptr->ReplaceBox(1, Nbox_);

            NullBox_ = false;
            Nbox_ = (*ptr)[1];
            
            if (!Nbox_)
            {
              Tptr_ = (*ptr)[0];
              NullBox_ = true;
            }
          }
          else
          {
            NullBox_ = false;
            Tptr_ = WindowLineBuffer::Make(NULL, NULL, ptr);

            if (Tptr_)
            {
              InitializeBox(Tptr_, 0, false, false);
              (*ptr)[0] = Tptr_;
              NullBox_ = true;
            }
          }

          if (NullBox_)
          {
            OwnerWindow_ = (WindowLine*)(Tptr_->_Owner);
            
            if ((*OwnerWindow_)[1] == NULL && Tptr_)
              ((WindowLineBuffer*)((*OwnerWindow_)[0]))->Chain(WindowLineBuffer::MakeNextLine(Tptr_));
          }

          ptr->ReCalcChainWline(true, true);

          if (Tptr_)
            Tptr_->ReAdjustLine(Offset_);
        }
        else
        {
          Tptr_ = (*ptr)[0];
          OwnerWindow_ = (WindowLine*)(Tptr_->_Owner);
          
          if (Tptr_ && Nbox_ && Tptr_ != Nbox_ && !NoNewStr_ && !NboxEmpty_)
          {
            if (FirstBox_)
              Tptr_->CopyEndLineCharData(OwnerWindow_, FirstBox_);

            Tptr_->AddBoxChain(Nbox_, false);
          }
          else if (Nbox_)
          {
            Nbox_->InitFirstBox(OwnerWindow_);
            
            if (NboxEmpty_)
              Nbox_->CopyEndLineCharData(OwnerWindow_, NULL, true);
            else if (!NullBox_ && FirstBox_ && Nbox_ != FirstBox_)
              Tptr_->CopyEndLineCharData(OwnerWindow_, FirstBox_);
          }

          if (Tptr_)
          {
            if (NboxEmpty_)
            {
              ptr->EraseBox(0);

              if (Nbox_ && NoMatchesToPrevBoxes(Nbox_, PrevBox_, Pbox_, Maxbox_))
              {
                delete Nbox_;
                Nbox_ = NULL;
              }
            }
            else if (Nbox_)
              ptr->ReplaceBox(0, Nbox_);

            NullBox_ = false;
            Nbox_ = (*ptr)[1];

            if (!Nbox_)
            {
              Tptr_ = (*ptr)[0];
              NullBox_ = true;
            }
          }
          else
          {
            NullBox_ = false;
            Tptr_ = WindowLineBuffer::Make(NULL, NULL, ptr);

            if (Tptr_)
            {
              InitializeBox(Tptr_, 0, false, false);
              (*ptr)[0] = Tptr_;
              NullBox_ = true;
            }
          }

          if (NullBox_)
          {
            OwnerWindow_ = (WindowLine*)(Tptr_->_Owner);
            
            if ((*OwnerWindow_)[1] == NULL && Tptr_)
              ((WindowLineBuffer*)((*OwnerWindow_)[0]))->Chain(WindowLineBuffer::MakeNextLine(Tptr_));
          }

          ptr->ReCalcChainWline(true, true);
        }
      }
    }
  }

  return ShiftDownDone_;
}

/****************************************************************************/
bool WindowLine::ShiftUp(char* NewStr_, int Offset_,
                         TextControl** Pbox_, int Boxdex_, int Maxbox_)
{
  // new
  if (Boxdex_ >= Maxbox_ || !_nextline)
    return false;

  if (!Pbox_)
    Pbox_ = _pbox;

  if (!Maxbox_)
    Maxbox_ = _maxbox;

  if (!NewStr_ && _nextline)
    NewStr_ = _nextline->RetrieveInOrder();

  if (!NewStr_ || !strlen(NewStr_))
    return false;

  Boxdex_ = 0;
  if (!Pbox_ || !Pbox_[Boxdex_])
    return false;

  int x;
  int max;
  
  bool KeyDone_;
  bool vbuf_ = false;
  bool inwbox_ = false;

  WindowLineBuffer* Nbox_ = NULL;
  WindowLineBuffer* Tptr_ = ((WindowLineBuffer*)Pbox_[Boxdex_]);
  
  Tptr_->SetRefLineMoveDone(false);
  Tptr_->IncrBoxChains(true);

  #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
    KeyDone_ = Tptr_->PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::END).SCAN + 256);
  #else
    KeyDone_ = Tptr_->PutExtKey(KEY_END);
  #endif

  if (KeyDone_)
  {
    #if (defined(__TURBOC__) | defined(__BORLANDC__) | defined(__DJGPP__))
      KeyDone_ = Tptr_->PutExtKey((int)KeyboardCodes::CODETABLE(KeyboardCodes::RIGHTARROW).SCAN + 256);
    #elif (defined(__linux__) | defined(__unix__))
      KeyDone_ = Tptr_->PutExtKey(KEY_RIGHT);
    #endif

    if (KeyDone_)
    {
      max = strlen(NewStr_);
      for (x = 0; x < max && KeyDone_; x++)
        KeyDone_ = Tptr_->PutStdKey(NewStr_[x]);

      return _nextline->ShiftUpNext(NULL, 0, Pbox_, Boxdex_, Maxbox_);
    }
  }

  return false;
}

/****************************************************************************/
bool WindowLine::ShiftUpNext(char* NewStr_, int Offset_,
                             TextControl** Pbox_, int Boxdex_, int Maxbox_)
{
  // new
  int x;
  bool PboxesValid_ = Pbox_ && _pbox &&
                      Pbox_[0] && _pbox[0] && Pbox_ != _pbox;
  
  WindowLineBuffer* Tptr_;
  WindowLine* ptr = this;
  WindowLine* nextptr = _nextline;
  Boxdex_ = 0;
  
  if (PboxesValid_)
    while (nextptr)
    {
      Maxbox_ = ptr->_maxbox;
      Tptr_ = (*nextptr)[0];
      
      for (x = Boxdex_; x < Maxbox_; x++)
      {
        if (Tptr_)
        {
          ptr->ReplaceBox(x, Tptr_);
          Tptr_ = Tptr_->_nextline;
        }
        else if (ptr->_pbox[x])
          ptr->DeleteControl(ptr->_pbox[x]);
      }

      ptr = ptr->_nextline;
      nextptr = nextptr->_nextline;
    }

  if (ptr && !nextptr)
  {
    Maxbox_ = ptr->_maxbox;
    for (x = 0; x < Maxbox_ && ptr->_pbox[x]; x++)
      ptr->_pbox[x]->Erase();

    return true;
  }

  return false;
}

/****************************************************************************/
void WindowLine::TransferBoxPosToNextLine(int linedir_)
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;

  // new window line box
  WindowLineBuffer* Nbox_ = NULL;

  // this current window line box
  WindowLineBuffer* Tptr_ = ((WindowLineBuffer*)_pbox[0]);

  if (linedir_ == 1)
  {
    if (_nextline)
    {
      sdp->_WinLinePosTransfer = true;
      Nbox_ = (*_nextline)[0];
      SetRowCursor(_nextline, true);
      sdp->_WinLinePosTransfer = false;
    }
    else
      Nbox_ = NULL;
  }
  else if (linedir_ == -1)
  {
    if (_prevline)
    {
      sdp->_WinLinePosTransfer = true;
      Nbox_ = (*_prevline)[0];
      SetRowCursor(_prevline, true);
      sdp->_WinLinePosTransfer = false;
    }
    else
      Nbox_ = NULL;
  }
  else
    Nbox_ = NULL;

  if (Nbox_)
  {
    Tptr_->SaveBoxPos();
    Nbox_->RestoreBoxPos(Tptr_);
    ShowAll();
  }
}

/****************************************************************************/
void WindowLine::ChainFillString(char NewCh_, int Offset_,
                                 TextControl** Pbox_, int Boxdex_, int Maxbox_)
{
  // new
  // if Length_ == -1 then fill NewStr_ to end
  if (Boxdex_ < Maxbox_ && Pbox_[Boxdex_])
    ((WindowLineBuffer*)Pbox_[Boxdex_])->FillCharAtPos(NewCh_, Offset_, true);

  if (Boxdex_+1 < Maxbox_ && Pbox_[Boxdex_+1])
    ChainFillString(NewCh_, 0, Pbox_, Boxdex_+1, Maxbox_);
}

/****************************************************************************/
void WindowLine::ChainFillString(char* NewStr_, int Length_, int Offset_,
                                 TextControl** Pbox_, int Boxdex_, int Maxbox_)
{
  // new
  if (Boxdex_ < Maxbox_ && Pbox_[Boxdex_])
    ((WindowLineBuffer*)Pbox_[Boxdex_])->SetInputStringAtPos(NewStr_, Length_, Offset_, true);

  if (Boxdex_+1 < Maxbox_ && Pbox_[Boxdex_+1])
    ChainFillString(NewStr_, _inputlen, 0, Pbox_, Boxdex_+1, Maxbox_);
}

/****************************************************************************/
void WindowLine::SetBoxToString(char* NewStr_, int index)
{
  if (_pbox && _pbox[index])
  {
    _pbox[index]->Erase();
    ((WindowLineBuffer*)_pbox[index])->SetInputString(NewStr_);
  }
}

/****************************************************************************/
void WindowLine::RemoveRefPtrData(WindowLineRef** RefPtrArray_, int index)
{
  if (RefPtrArray_)
    RefPtrArray_[index] = NULL;
}

/****************************************************************************/
void WindowLine::ReplaceControlData(TextControl** CtrlArray_, ControlInfo* CtrlInfo_,
                                    int index, bool CheckNextLinks_)
{
  if (CtrlArray_ && index < _maxbox)
  {
    WindowLineBuffer* wlbptr = NULL;
    WindowLineBuffer* rootwlbptr = NULL;
    WindowLineBuffer* pb = (WindowLineBuffer*)(CtrlArray_[index]);

    if (pb)
    {
      rootwlbptr = pb;
      pb = new WindowLineBuffer(*rootwlbptr);
      pb->Activate();
      pb->Erase();
      pb->Deactivate();
    }

    CtrlArray_[index] = pb;

    if (CheckNextLinks_ && index && CtrlArray_[index-1] && pb)
      LinkNextControl(CtrlArray_, index-1);
  }
}

/****************************************************************************/
void WindowLine::LinkNextControl(TextControl** CtrlArray_, int index)
{
  if (CtrlArray_ && CtrlArray_[index])
  {
    WindowLineBuffer* rootwlbptr = (WindowLineBuffer*)(CtrlArray_[0]);
    WindowLineBuffer* wlbptr = (WindowLineBuffer*)(CtrlArray_[index]);
    WindowLineBuffer* newpb = (WindowLineBuffer*)CtrlArray_[index+1];

    if (rootwlbptr && wlbptr && newpb)
    {      
      if (wlbptr->_nextline && wlbptr->_nextline != newpb)
        UnlinkNextControl(CtrlArray_, index);

      if (wlbptr->_nextline != newpb)
        rootwlbptr->AddBoxChain(newpb, false);
    }
  }
}

/****************************************************************************/
void WindowLine::UnlinkNextControl(TextControl** CtrlArray_, int index)
{
  if (CtrlArray_ && CtrlArray_[index])
  {
    WindowLineBuffer* rootwlbptr = (WindowLineBuffer*)(CtrlArray_[0]);
    WindowLineBuffer* wlbptr = (WindowLineBuffer*)(CtrlArray_[index]);

    if (rootwlbptr && wlbptr && wlbptr->_nextline)
      rootwlbptr->DetachNext(wlbptr, false);
  }
}

/****************************************************************************/
void WindowLine::RemoveControlData(TextControl** CtrlArray_, ControlInfo* CtrlInfo_,
                                   int index, bool CheckNextLinks_)
{
  if (CtrlArray_ && index < _maxbox)
    CtrlArray_[index] = NULL;

  if (CtrlInfo_ && index < _maxbox)
    CtrlInfo_->SetHotKeys(index, NULL);

  if (CheckNextLinks_ && index && CtrlArray_[index-1])
    UnlinkNextControl(CtrlArray_, index-1);
}

/****************************************************************************/
void WindowLine::SetControlData(TextControl** CtrlArray_, ControlInfo* CtrlInfo_,
                                int index, bool CheckNextLinks_)
{
  if (CtrlArray_ && CtrlArray_[index])
  {
    CtrlArray_[index]->SetOwner(this, CtrlArray_[index]);
    ((WindowLineBuffer*)CtrlArray_[index])->SetWindowLineNum(_WindowLineNum);
    
    int InputLen_ = ((WindowLineBuffer*)CtrlArray_[index])->InputLength();
    if (!_inputlen || (0 < InputLen_ && _inputlen < InputLen_))
      _inputlen = InputLen_;

    if (CtrlInfo_)
      CtrlInfo_->SetHotKeys(index, CtrlArray_[index]);

    if (CheckNextLinks_ && index && CtrlArray_[index-1] && CtrlArray_[index])
      LinkNextControl(CtrlArray_, index-1);
  }
}

/****************************************************************************/
// PURPOSE:
//   Prompt window data member destruction method
//
// PRE:
//   The data members of this Prompt window object with original values
//
// POST:
//   If the data members of the window object is not NULL then the data
//   members are deleted and assigned NULL. The array of Prompt boxes
//   are destroyed.
//
void WindowLine::Clear()
{
  if (_pbox)
  {
    Delete();

    delete[] _pbox;
    _pbox = NULL;
  }

  if (_RefPtr)
  {
    delete[] _RefPtr;
    _RefPtr = NULL;
  }

  if (_Info)
  {
    delete _Info;
    _Info = NULL;
  }
}

/****************************************************************************/
TextControl* WindowLine::Control(int Index_)
{
  return _pbox[Index_];
}

/****************************************************************************/
void WindowLine::Control(int Index_, TextControl* TxtCtrl_)
{
  _pbox[Index_] = TxtCtrl_;

  if (_pbox[Index_])
  {
    SetControlData(_pbox, _Info, Index_, true);
    FindControlCount();
  }
}

/****************************************************************************/
// PURPOSE:
//   Sets the maximum Prompt box limit on the box array
//
// PRE:
//   The previous size of the Prompt box array
//
// POST:
//   The size of the Prompt box array is set to the specified maximum
//   All contents of the array are Cleared first before resizing the array
//
void WindowLine::MaxControl(int max)
{
  if (_pbox)
  {
    Delete();

    delete[] _pbox;
    _pbox = NULL;
  }

  if (_RefPtr)
  {
    delete[] _RefPtr;
    _RefPtr = NULL;  
  }

  _maxbox = max;

  if (max)
  {  
    NewPboxes(_maxbox);
    NewRefPtrs(_maxbox);
  
    if (_pbox && _RefPtr)
      for (int index = 0; index < _maxbox; index++)
      {
        _RefPtr[index] = NULL;
        RemoveControlData(_pbox, _Info, index, false);
      }
  }
}

/****************************************************************************/
int WindowLine::MaxControl()
{
  int x;
  for (x = 0; x < _maxbox; x++)
    if (_pbox[x] == NULL)
      break;
      
  return x;
}

/****************************************************************************/
// PURPOSE:
//   Method to Grow the size of the array by some number passed as an
//   argument to the method. The array contents are preserved.
//
// PRE:
//   The old size of the Prompt box array and its contents
//
// POST:
//   The array size is Grown by the extra size given and the original
//   contents of the array are preserved in the new array.
//
void WindowLine::Grow(int extra)
{
  int index;
  int newlen = _maxbox + extra;
  TextControl** newp = NULL;
  WindowLineRef** newr = NULL;
  ControlInfo* newi = NULL;
  bool AllocDone_ = false;

  if (newlen)
  {
    newp = NewPboxes(newlen, false);
    newr = NewRefPtrs(newlen, false);
    newi = new ControlInfo(newlen);
    AllocDone_ = newp && newr && newi;

    if (!AllocDone_)
      return;
  }

  for (index = 0; index < _maxbox; index++)
  {
    if (_pbox && _pbox[index])
    {
      newp[index] = _pbox[index];
      _pbox[index] = NULL;
    }
    else
      newp[index] = NULL;
    
    SetControlData(newp, newi, index, false);

    if (_RefPtr && _RefPtr[index])
    {
      newr[index] = _RefPtr[index];
      _RefPtr[index] = NULL;
    }
    else
      newr[index] = NULL;
  }

  for (;index < newlen; index++)
  {
    newp[index] = NULL;
    newr[index] = NULL;

    RemoveControlData(newp, newi, index, false);
  }

  Clear();

  _maxbox = newlen;
  _pbox = newp;
  _Info = newi;
  _RefPtr = newr;
}

/****************************************************************************/
Boolean WindowLine::AnyActive()
{
  for (int Index_ = 0; Index_ < _maxbox; Index_++)
    if (_pbox[Index_] && _pbox[Index_]->IsActive())
      return TRUE;

  return FALSE;
}

/****************************************************************************/
Boolean WindowLine::ValidityCheck()
{
  for (int Index_ = 0; Index_ < _maxbox; Index_++)
    if (_pbox[Index_] && _pbox[Index_]->IsActive() &&
        !_pbox[Index_]->IsValid())
      return FALSE;

  return TRUE;
}

/****************************************************************************/
void WindowLine::ShowWindowLineAt(int LineNum_, int Ypos_, int Xpos_)
{
  WindowLineBuffer* wlbptr;
  WindowLine::SharedChainData* sdp = SharedData();
  
  bool Found_ = false;
  bool done = false;
  
  char* strptr_ = NULL;

  int index;
  int x = WHEREX();
  int y = WHEREY();

  GOTOXY(Xpos_, Ypos_);
  
  if (!sdp)
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  if (sdp->_InWinMode)
  {
    sdp->_currentrow->Deactivate();
    RestoreRowCursor(TO_PARENT);
    Found_ = sdp->_rowcursor->WindowLineNum() == LineNum_;

    while (sdp->_rowcursor && !Found_)
    {
      sdp->_rowcursor = sdp->_rowcursor->_nextline;
      Found_ = sdp->_rowcursor->WindowLineNum() == LineNum_;
    }

    if (Found_)
    {
      sdp->_rowcursor->Activate();

      for (index = 0; index < sdp->_rowcursor->_maxbox; index++)
        if (sdp->_rowcursor->_pbox[index] &&
            sdp->_rowcursor->_pbox[index]->IsActive() &&
            !sdp->_rowcursor->_pbox[index]->IsEscaped())
        {
          wlbptr = ((WindowLineBuffer*)sdp->_rowcursor->_pbox[index]);
          strptr_ = wlbptr->StrRetrieve(true, true, wlbptr->CalcBackShift(wlbptr->RawStrBuffer(0)));
          
          if (strptr_)
          {
            CPUTS(strptr_);
            done = true;
          }
        }

      if (!done)
        CPUTS("(NULL)");
      
      sdp->_rowcursor->Deactivate();

      GETCH();
      CPUTS("                    ");
    }

    RestoreRowCursor(TO_CURRENT);
    sdp->_currentrow->Activate();
  }

  GOTOXY(x, y);
}

/****************************************************************************/
// PURPOSE:
//   Cycle through the entire Prompt box array and Prompt the user with
//   each one.
//
// POST:
//   The Prompt box array is cycled through and each Prompt box element
//   within the array is used to Prompt the user.
//
int WindowLine::Prompt(int StartIndex_)
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || (sdp->_InWinMode && !sdp->_rowcursor))
    return 0;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  int cnt = 0;
  int index = 0;
  int status_ = 1;
  bool done_ = false;
  WindowLine* ptr = this;

  if (!sdp->_InWinMode)
    index = (StartIndex_ >= _maxbox) ? 0:
            (StartIndex_ < 0) ? (_maxbox - 1):
             StartIndex_;

  if (sdp->_InWinMode && (ptr != sdp->_rowcursor || !ptr->IsActive()))
  {
    index = sdp->_rowcursor->WindowLineNum();
    sdp->_rowcursorfound = false;

    if (index > _WindowLineNum)
    {
      cnt = ptr->_WindowLineNum;
      
      while (ptr && (ptr->_WindowLineNum < index || !ptr->IsActive()))
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }
    else if (index < _WindowLineNum)
    {
      cnt = index;
      
      while (ptr && (ptr->_WindowLineNum > index || !ptr->IsActive()))
      {
        ptr = ptr->_prevline;
        ++cnt;
      }
    }
    else if (sdp && sdp->_parent && !sdp->_InWinMode)
    {
      ptr = sdp->_parent;
      cnt = ptr->_WindowLineNum;
      
      while (ptr && !ptr->IsActive())
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }

    if (ptr && sdp && cnt <= sdp->_numnextlines)
    {
      sdp->_rowcursorfound =
        (ptr->_WindowLineNum == index && ptr->IsActive()) ||
        (!sdp->_InWinMode && ptr->IsActive());

      if (sdp->_rowcursorfound && !sdp->_rowcursor->IsEscaped())
        sdp->_rowcursor->Prompt(StartIndex_);
      else
        status_ = 0;
    }
    else
      status_ = 0;
  }
  else if (AnyActive())
  {
    // while (!LineTerminated() && !WinBlockTerminated())
    while (!LineTerminated() && cnt < _maxbox)
    {
      if (_pbox[index] && _pbox[index]->IsActive())
      {
        sdp->_rowcursorfound =
          (_WindowLineNum == sdp->_rowcursor->WindowLineNum() && IsActive()) ||
          (!sdp->_InWinMode && ptr->IsActive());

        if (!_pbox[index]->IsEscaped() || !sdp->_InWinMode)
          status_ = _pbox[index]->Prompt(status_);

        Boolean hkeypressed_ = _Info->HotKeyPressed();
        if (sdp->_InWinMode && _pbox[index] && _pbox[index]->IsEscaped())
          status_ = 0;

        if (hkeypressed_)
        {
          if (status_ != 0)
            index = _Info->GetIndex();
            
          _Info->ResetHotKeys();
        }

        if (!hkeypressed_ || !status_)
        {
          if (_pbox[index] && _pbox[index]->IsEscaped())
            if (ValidityCheck())
              break;
            else
              status_ = 0;

          if (AnyActive())
          {
            ++cnt;
            index += status_;
            done_ = true;
          }
          else if (!IsActive() && !done_ && sdp && sdp->_InWinMode)
          {
            index = sdp->_rowcursor->WindowLineNum();
            sdp->_rowcursorfound = _WindowLineNum == index &&
                                   sdp->_rowcursor->IsActive();
            
            if (!sdp->_rowcursorfound && !sdp->_rowcursor->IsEscaped())
              status_ = Prompt(0);
            else if (sdp->_rowcursorfound)
              break;

            if (sdp->_rowcursor->IsEscaped())
              break;
            else if (!status_)
              ++cnt;
          }
        }
      }
      else
      {
        if (AnyActive())
        {
          ++cnt;
          ++index;
          done_ = true;
        }
        else if (!IsActive() && !done_ && sdp && sdp->_InWinMode)
        {
          index = sdp->_rowcursor->WindowLineNum();
          sdp->_rowcursorfound = _WindowLineNum == index &&
                                 sdp->_rowcursor->IsActive();
          
          if (!sdp->_rowcursorfound && !sdp->_rowcursor->IsEscaped())
            status_ = Prompt(0);
          else if (sdp->_rowcursorfound)
            break;

          if (sdp->_rowcursor->IsEscaped())
            break;
          else if (!status_)
            ++cnt;
        }
      }

      index = (index >= _maxbox) ? 0:
         (index < 0) ? (_maxbox - 1):
         index;
    }
  }

  return status_;
}

/****************************************************************************/
// PURPOSE:
//   Cycle through the entire Prompt box array and Show to the screen
//   each one.
//
// POST:
//   The Prompt box array is cycled through and each Prompt box element
//   within the array is Shown to the screen.
//
void WindowLine::Show()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || (sdp->_InWinMode && !sdp->_rowcursor))
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  int cnt = 0;
  int index = 0;
  bool done_ = false;
  WindowLine* ptr = this;

  if (sdp->_InWinMode && (ptr != sdp->_rowcursor || !ptr->IsActive()))
  {
    index = sdp->_rowcursor->WindowLineNum();
    sdp->_rowcursorfound = false;

    if (index > _WindowLineNum)
    {
      cnt = ptr->_WindowLineNum;
      
      while (ptr && (ptr->_WindowLineNum < index || !ptr->IsActive()))
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }
    else if (index < _WindowLineNum)
    {
      cnt = index;
      
      while (ptr && (ptr->_WindowLineNum > index || !ptr->IsActive()))
      {
        ptr = ptr->_prevline;
        ++cnt;
      }
    }
    else if (sdp && sdp->_parent && !sdp->_InWinMode)
    {
      ptr = sdp->_parent;
      cnt = ptr->_WindowLineNum;
      
      while (ptr && !ptr->IsActive())
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }

    if (ptr && sdp && cnt <= sdp->_numnextlines)
    {
      sdp->_rowcursorfound =
        (ptr->_WindowLineNum == index && ptr->IsActive()) ||
        (!sdp->_InWinMode && ptr->IsActive());

      if (sdp->_rowcursorfound && !sdp->_rowcursor->IsEscaped())
        sdp->_rowcursor->Show();
    }
  }
  else if (AnyActive())
  {
    sdp->_rowcursorfound =
      (_WindowLineNum == sdp->_rowcursor->WindowLineNum() && IsActive()) ||
      (!sdp->_InWinMode && ptr->IsActive());
  
    for (index = 0; index < _maxbox; index++)
      if (_pbox[index] && _pbox[index]->IsActive() &&
          (!_pbox[index]->IsEscaped() || !sdp->_InWinMode))
      {
        _pbox[index]->Show();
        done_ = true;
      }

    if ((ptr != sdp->_rowcursor || !IsActive()) &&
        !done_ && sdp && sdp->_InWinMode)
    {
      index = sdp->_rowcursor->WindowLineNum();
      sdp->_rowcursorfound = _WindowLineNum == index &&
                             sdp->_rowcursor->IsActive();
      
      if (!sdp->_rowcursorfound && !sdp->_rowcursor->IsEscaped())
        Show();
    }
  }
}

/****************************************************************************/
// PURPOSE:
//   Cycle through the entire Prompt box array and Clear each one from the
//   screen.
//
// POST:
//   The Prompt box array is cycled through and each Prompt box element
//   within the array is Cleared from the screen.
//
void WindowLine::Hide()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || (sdp->_InWinMode && !sdp->_rowcursor))
    return;
  else
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  int cnt = 0;
  int index = 0;
  bool done_ = false;
  WindowLine* ptr = this;

  if (sdp->_InWinMode && (ptr != sdp->_rowcursor || !ptr->IsActive()))
  {
    index = sdp->_rowcursor->WindowLineNum();
    sdp->_rowcursorfound = false;
    
    if (index > _WindowLineNum)
    {
      cnt = ptr->_WindowLineNum;
      
      while (ptr && (ptr->_WindowLineNum < index || !ptr->IsActive()))
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }
    else if (index < _WindowLineNum)
    {
      cnt = index;
      
      while (ptr && (ptr->_WindowLineNum > index || !ptr->IsActive()))
      {
        ptr = ptr->_prevline;
        ++cnt;
      }
    }
    else if (sdp && sdp->_parent && !sdp->_InWinMode)
    {
      ptr = sdp->_parent;
      cnt = ptr->_WindowLineNum;
      
      while (ptr && !ptr->IsActive())
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }

    if (ptr && sdp && cnt <= sdp->_numnextlines)
    {
      sdp->_rowcursorfound =
        (ptr->_WindowLineNum == index && ptr->IsActive()) ||
        (!sdp->_InWinMode && ptr->IsActive());

      if (sdp->_rowcursorfound && !sdp->_rowcursor->IsEscaped())
        sdp->_rowcursor->Hide();
    }
  }
  else if (AnyActive())
  {
    sdp->_rowcursorfound =
      (_WindowLineNum == sdp->_rowcursor->WindowLineNum() && IsActive()) ||
      (!sdp->_InWinMode && ptr->IsActive());

    for (index = 0; index < _maxbox; index++)
      if (_pbox[index] && _pbox[index]->IsActive() &&
          (!_pbox[index]->IsEscaped() || !sdp->_InWinMode))
      {
        _pbox[index]->Hide();
        done_ = true;
      }

    if ((ptr != sdp->_rowcursor || !IsActive()) &&
        !done_ && sdp && sdp->_InWinMode)
    {
      index = sdp->_rowcursor->WindowLineNum();
      sdp->_rowcursorfound = _WindowLineNum == index &&
                             sdp->_rowcursor->IsActive();

      if (!sdp->_rowcursorfound && !sdp->_rowcursor->IsEscaped())
        Hide();
    }
  }
}

/****************************************************************************/
// PURPOSE:
//   Cycle through the entire Prompt box array and Erase each Prompt box.
//
// POST:
//   The Prompt box array is cycled through and each Prompt box element
//   within the array is Erased.
//
void WindowLine::Erase()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || (sdp->_InWinMode && !sdp->_rowcursor))
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  int cnt = 0;
  int index = 0;
  bool done_ = false;
  WindowLine* ptr = this;

  if (sdp->_InWinMode && ptr != sdp->_rowcursor)
  {
    index = sdp->_rowcursor->WindowLineNum();
    sdp->_rowcursorfound = false;

    if (index > _WindowLineNum)
    {
      cnt = ptr->_WindowLineNum;

      while (ptr && ptr->_WindowLineNum < index)
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }
    else if (index < _WindowLineNum)
    {
      cnt = index;
      
      while (ptr && ptr->_WindowLineNum > index)
      {
        ptr = ptr->_prevline;
        ++cnt;
      }
    }
    else if (sdp && sdp->_parent && !sdp->_InWinMode)
    {
      ptr = sdp->_parent;
      cnt = ptr->_WindowLineNum;
      
      while (ptr && !ptr->IsActive())
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }

    if (ptr && sdp && cnt <= sdp->_numnextlines)
    {
      sdp->_rowcursorfound =
        ptr->_WindowLineNum == index ||
        (!sdp->_InWinMode && ptr->IsActive());

      if (sdp->_rowcursorfound)
        sdp->_rowcursor->Erase();
    }
  }
  else
  {
    sdp->_rowcursorfound =
      _WindowLineNum == sdp->_rowcursor->WindowLineNum() ||
      (!sdp->_InWinMode && ptr->IsActive());

    for (index = 0; index < _maxbox; index++)
      if (_pbox[index])
      {
        _pbox[index]->Erase();
        done_ = true;
      }

    if (ptr != sdp->_rowcursor && !done_ && sdp && sdp->_InWinMode)
    {
      index = sdp->_rowcursor->WindowLineNum();
      sdp->_rowcursorfound = _WindowLineNum == index;

      if (!sdp->_rowcursorfound)
        Erase();
    }
  }
}

/****************************************************************************/
// PURPOSE:
//   Cycle through the entire Prompt box array and delete each Prompt box.
//   This method does not depend on whether the text control is in an
//   active state or not. All controls regardless of their active state will
//   be deleted.
//
// POST:
//   The Prompt box array is cycled through and each Prompt box deleted
//
void WindowLine::Delete()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || (sdp->_InWinMode && !sdp->_rowcursor))
    return;
  else
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  int cnt = 0;
  int index = 0;
  bool done_ = false;
  WindowLine* ptr = this;

  if (sdp->_InWinMode && ptr != sdp->_rowcursor)
  {
    index = sdp->_rowcursor->WindowLineNum();
    sdp->_rowcursorfound = false;

    if (index > _WindowLineNum)
    {
      cnt = ptr->_WindowLineNum;
      
      while (ptr && ptr->_WindowLineNum < index)
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }
    else if (index < _WindowLineNum)
    {
      cnt = index;
      
      while (ptr && ptr->_WindowLineNum > index)
      {
        ptr = ptr->_prevline;
        ++cnt;
      }
    }
    else if (sdp && sdp->_parent && !sdp->_InWinMode)
    {
      ptr = sdp->_parent;
      cnt = ptr->_WindowLineNum;
      
      while (ptr && !ptr->IsActive())
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }

    if (ptr && sdp && cnt <= sdp->_numnextlines)
    {
      sdp->_rowcursorfound =
        ptr->_WindowLineNum == index ||
        (!sdp->_InWinMode && ptr->IsActive());

      if (sdp->_rowcursorfound)
        sdp->_rowcursor->Delete();
    }
  }
  else
  {
    sdp->_rowcursorfound =
      _WindowLineNum == sdp->_rowcursor->WindowLineNum() ||
      (!sdp->_InWinMode && ptr->IsActive());

    for (index = 0; index < _maxbox; index++)
    {
      if (_pbox && _pbox[index] && _pbox[index]->IsOwner(this))
      {
        delete _pbox[index];
        RemoveControlData(_pbox, _Info, index);
        _pbox[index] = NULL;
        done_ = true;
      }

      if (_RefPtr && _RefPtr[index])
      {
        delete _RefPtr[index];
        _RefPtr[index] = NULL;
        done_ = true;
      }
    }

    if (ptr != sdp->_rowcursor && !done_ && sdp && sdp->_InWinMode)
    {
      index = sdp->_rowcursor->WindowLineNum();
      sdp->_rowcursorfound = _WindowLineNum == index;
    
      if (!sdp->_rowcursorfound)
        Delete();
    }
    else
      FindControlCount();
  }
}

/****************************************************************************/
// PURPOSE:
//   Cycle through the entire Prompt box array and remove each Prompt box.
//   This method does not depend on whether the text control is in an
//   active state or not. All controls regardless of their active state will
//   be removed.
//
// POST:
//   The Prompt box array is cycled through and each Prompt box removed,
//   but not deleted or modified
//
void WindowLine::Remove()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || (sdp->_InWinMode && !sdp->_rowcursor))
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  int cnt = 0;
  int index = 0;
  bool done_ = false;
  WindowLine* ptr = this;
  TextControl* TxtCtrl_;

  if (sdp->_InWinMode && ptr != sdp->_rowcursor)
  {
    index = sdp->_rowcursor->WindowLineNum();
    sdp->_rowcursorfound = false;
    
    if (index > _WindowLineNum)
    {
      cnt = ptr->_WindowLineNum;
      
      while (ptr && ptr->_WindowLineNum < index)
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }
    else if (index < _WindowLineNum)
    {
      cnt = index;
      
      while (ptr && ptr->_WindowLineNum > index)
      {
        ptr = ptr->_prevline;
        ++cnt;
      }
    }
    else if (sdp && sdp->_parent && !sdp->_InWinMode)
    {
      ptr = sdp->_parent;
      cnt = ptr->_WindowLineNum;
      
      while (ptr && !ptr->IsActive())
      {
        ptr = ptr->_nextline;
        ++cnt;
      }
    }

    if (ptr && sdp && cnt <= sdp->_numnextlines)
    {
      sdp->_rowcursorfound =
        ptr->_WindowLineNum == index ||
        (!sdp->_InWinMode && ptr->IsActive());

      if (sdp->_rowcursorfound)
        sdp->_rowcursor->Remove();
    }
  }
  else
  {
    sdp->_rowcursorfound =
      _WindowLineNum == sdp->_rowcursor->WindowLineNum() ||
      (!sdp->_InWinMode && ptr->IsActive());

    for (index = 0; index < _maxbox; index++)
      if (_pbox[index] && _pbox[index]->IsOwner(this))
      {
        TxtCtrl_ = _pbox[index];
        TxtCtrl_->SetOwner(NULL, TxtCtrl_);
        _Info->SetHotKeys(index, NULL);
        done_ = true;
      }

    if (ptr != sdp->_rowcursor && !done_ && sdp && sdp->_InWinMode)
    {
      index = sdp->_rowcursor->WindowLineNum();
      sdp->_rowcursorfound = _WindowLineNum == index;
    
      if (!sdp->_rowcursorfound)
        Remove();
    }
    else
      FindControlCount();
  }
}

/****************************************************************************/
void WindowLine::ShowAll()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  if (sdp->_InWinMode)
  {
    sdp->_currentrow->Deactivate();
    RestoreRowCursor(TO_PARENT);

    while (sdp->_rowcursor)
    {
      sdp->_rowcursor->Activate();
      Show();
      sdp->_rowcursor->Deactivate();
      sdp->_rowcursor = sdp->_rowcursor->_nextline;
    }

    RestoreRowCursor(TO_CURRENT);
    sdp->_currentrow->Activate();
    Show();
  }
}

/****************************************************************************/
void WindowLine::HideAll()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  if (sdp->_InWinMode)
  {
    sdp->_currentrow->Deactivate();
    RestoreRowCursor(TO_PARENT);

    while (sdp->_rowcursor)
    {
      sdp->_rowcursor->Activate();
      Hide();
      sdp->_rowcursor->Deactivate();
      sdp->_rowcursor = sdp->_rowcursor->_nextline;
    }

    RestoreRowCursor(TO_CURRENT);
    sdp->_currentrow->Activate();
    Hide();
  }
}

/****************************************************************************/
void WindowLine::EraseAll()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  if (sdp->_InWinMode)
  {
    sdp->_currentrow->Deactivate();
    RestoreRowCursor(TO_PARENT);

    while (sdp->_rowcursor)
    {
      sdp->_rowcursor->Activate();
      Erase();
      sdp->_rowcursor->Deactivate();
      sdp->_rowcursor = sdp->_rowcursor->_nextline;
    }

    RestoreRowCursor(TO_CURRENT);
    sdp->_currentrow->Activate();
    Erase();
  }
}

/****************************************************************************/
void WindowLine::DeleteAll()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  if (sdp->_InWinMode)
  {
    sdp->_currentrow->Deactivate();
    RestoreRowCursor(TO_PARENT);

    while (sdp->_rowcursor)
    {
      sdp->_rowcursor->Activate();
      Delete();
      sdp->_rowcursor->Deactivate();
      sdp->_rowcursor = sdp->_rowcursor->_nextline;
    }

    RestoreRowCursor(TO_CURRENT);
    sdp->_currentrow->Activate();
    Delete();
  }
}

/****************************************************************************/
void WindowLine::RemoveAll()
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp)
    return;
  else  
    sdp->_InWinMode = InWindowMode(&sdp->_NextLineReady, &sdp->_PrevLineReady);

  if (sdp->_InWinMode)
  {
    sdp->_currentrow->Deactivate();
    RestoreRowCursor(TO_PARENT);

    while (sdp->_rowcursor)
    {
      sdp->_rowcursor->Activate();
      Remove();
      sdp->_rowcursor->Deactivate();
      sdp->_rowcursor = sdp->_rowcursor->_nextline;
    }

    RestoreRowCursor(TO_CURRENT);
    sdp->_currentrow->Activate();
    Remove();
  }
}

/****************************************************************************/
// PURPOSE:
//   Prompt box Index operator. To index individual Prompt boxes from the
//   Prompt window given the position of the Prompt box within the array.
//
// PRE:
//   int index : the index position of the Prompt box in the array
//
// POST:
//   Go into the specified index position in the Prompt box array and
//   Retrieve its element.
//
WindowLineRef& WindowLine::operator [] (int index)
{
  ASSERT((index >= 0 && index < _maxbox), USER_Message_t::ERRMSG_INDEXRANGE);

  if (!_RefPtr)
    return *_DummyRef;
  
  if (!_RefPtr[index])
    _RefPtr[index] = new WindowLineRef(this, index);
    
  return *_RefPtr[index];
}

/****************************************************************************/
WindowLineBuffer* WindowLine::NextSelected(int& Start_)
{
  Boolean Found_ = FALSE;

  for (;Start_ < _maxbox; Start_++)
    if (Found_ = _pbox[Start_] != NULL && _pbox[Start_]->IsSelected())
      break;

  if (Found_)
  {
    ++Start_;
    return (*this)[Start_ - 1];
  }

  return NULL;
}

/****************************************************************************/
void WindowLine::DeleteControl(TextControl* TxtCtrl_)
{
  int index;
  for (index = 0; index < _maxbox; index++)
    if (_pbox[index] && _pbox[index] == TxtCtrl_ && _pbox[index]->IsOwner(this))
      break;

  if (index < _maxbox)
  {
    if (_pbox)
    {
      delete _pbox[index];
      RemoveControlData(_pbox, _Info, index);
      _pbox[index] = NULL;
    }

    if (_RefPtr)
    {
      delete _RefPtr[index];
      _RefPtr[index] = NULL;
    }

    FindControlCount();
  }
}

/****************************************************************************/
bool WindowLine::HasControlArrays(WindowLine* Pwin_)
{
  return
  (
    Pwin_ ? (_pbox == Pwin_->_pbox ||
             _Info == Pwin_->_Info ||
             _RefPtr == Pwin_->_RefPtr):false
  );
}

/****************************************************************************/
void WindowLine::RemoveControl(TextControl* TxtCtrl_)
{
  int index;
  for (index = 0; index < _maxbox; index++)
    if (_pbox[index] && _pbox[index] == TxtCtrl_ && _pbox[index]->IsOwner(this))
      break;

  if (index < _maxbox)
  {
    RemoveControlData(_pbox, _Info, index);
    RemoveRefPtrData(_RefPtr, index);
    FindControlCount();
  }
}

/****************************************************************************/
void WindowLine::ReplaceControl(TextControl* TxtCtrl_)
{
  int index;
  WindowLineBuffer* ptr = (WindowLineBuffer*)TxtCtrl_;
  
  for (index = 0; index < _maxbox; index++)
    if (_pbox[index] && _pbox[index] == TxtCtrl_ && _pbox[index]->IsOwner(this))
      break;

  if (index < _maxbox)
  {
    if (index == 0 && ptr && ptr->BufferChainPosition() == 0)
    {
      ReplaceControlData(_pbox, _Info, index);
      FindControlCount();
    }
    else if (index)
    {
      RemoveControlData(_pbox, _Info, index);
      RemoveRefPtrData(_RefPtr, index);
      FindControlCount();
    }
  }
}

/****************************************************************************/
TextControl* WindowLine::ConfirmControlPtr(TextControl* ctrlptr_,
                                           int& index_, int* changed_)
{
  int x;
  if (changed_)
    *changed_ = 0;
  
  for (x = 0; x < _maxbox; x++)
    if (_pbox[x] && _pbox[x] == ctrlptr_)
      break;

  if (x != _maxbox && x != index_)
  {
    index_ = x;
    if (changed_)
      *changed_ = 2;
  }
  else if (_pbox[index_] && _pbox[index_] != ctrlptr_)
  {
    if (changed_)
      *changed_ = 1;

    return _pbox[index_];
  }

  return ctrlptr_;
}

/****************************************************************************/
ControlInfo* WindowLine::GetControlInfo()
{
  return _Info;
}

/****************************************************************************/
void WindowLine::UseColor(Boolean Flag_)
{
  int index;
  for (index = 0; index < _maxbox; index++)
    if (_pbox[index])
      _pbox[index]->UseColor(Flag_);
}

/****************************************************************************/
void WindowLine::SetNormalColors(int FgCol_, int BgCol_)
{
  int index;
  for (index = 0; index < _maxbox; index++)
    if (_pbox[index])
      _pbox[index]->SetNormalColors(FgCol_, BgCol_);
}

/****************************************************************************/
void WindowLine::SetHighLightColors(int FgCol_, int BgCol_)
{
  int index;
  for (index = 0; index < _maxbox; index++)
    if (_pbox[index])
      _pbox[index]->SetHighLightColors(FgCol_, BgCol_);
}

/****************************************************************************/
void WindowLine::SetHotKeyColor(int Col_)
{
  int index;
  for (index = 0; index < _maxbox; index++)
    if (_pbox[index])
      _pbox[index]->SetHotKeyColor(Col_);
}

/****************************************************************************/
void WindowLine::UseDefaultColors()
{
  int index;
  for (index = 0; index < _maxbox; index++)
    if (_pbox[index])
      _pbox[index]->UseDefaultColors();
}

/****************************************************************************/
Boolean WindowLine::IsSelected() const
{
  return ((_pbox && _pbox[0]) ? _pbox[0]->IsSelected():false);
}

/****************************************************************************/
Boolean WindowLine::IsEmpty() const
{
  return ((_pbox && _pbox[0]) ? _pbox[0]->IsEmpty():false);
}

/****************************************************************************/
Boolean WindowLine::IsEscaped() const
{
  return ((_pbox && _pbox[0]) ? _pbox[0]->IsEscaped():false);
}

/****************************************************************************/
Boolean WindowLine::IsValid() const
{
  return ((_pbox && _pbox[0]) ? _pbox[0]->IsValid():false);
}

/****************************************************************************/
Boolean WindowLine::IsActive() const
{
  return
  ((_pbox && _pbox[0]) ?(_WindowLineActive && _pbox[0]->IsActive()):false);
}

/****************************************************************************/
Boolean WindowLine::IsInFocus() const
{
  return ((_pbox && _pbox[0]) ? _pbox[0]->IsInFocus():false);
}

/****************************************************************************/
WindowLine& WindowLine::GotoWindowLine(int linenum_)
{
  WindowLine::SharedChainData* sdp = SharedData();
  if (!sdp || (0 <= linenum_ && linenum_ < sdp->_numnextlines))
    return *this;
  
  WindowLine* ptr = this;

  if (_WindowLineNum < linenum_)
  {
    while (ptr->WindowLineNum() < linenum_)
      ptr = ptr->_nextline;
  }
  else if (_WindowLineNum > linenum_)  
  {
    while (ptr->WindowLineNum() > linenum_)
      ptr = ptr->_prevline;
  }

  return *ptr;
}

/****************************************************************************/
const WindowLine& WindowLine::GotoWindowLine(int linenum_) const
{
  const WindowLine::SharedChainData* sdp = ConstSharedData();
  if (!sdp || (0 <= linenum_ && linenum_ < sdp->_numnextlines))
    return *this;

  const WindowLine* ptr = this;

  if (_WindowLineNum < linenum_)
  {
    while (ptr->WindowLineNum() < linenum_)
      ptr = ptr->_nextline;
  }
  else if (_WindowLineNum > linenum_)  
  {
    while (ptr->WindowLineNum() > linenum_)
      ptr = ptr->_prevline;
  }

  return *ptr;
}

/****************************************************************************/
WindowLine& WindowLine::GotoNextWindowLine(bool* done_)
{
  int linenum_ = _WindowLineNum+1;
  WindowLine::SharedChainData* sdp = SharedData();

  if (!sdp || !(0 <= linenum_ && linenum_ < sdp->_numnextlines))
  {
    if (done_)
      *done_ = false;
    return *this;
  }

  if (done_)
    *done_ = true;
  return GotoWindowLine(linenum_);
}

/****************************************************************************/
const WindowLine& WindowLine::GotoNextWindowLine(bool* done_) const
{
  int linenum_ = _WindowLineNum+1;
  const WindowLine::SharedChainData* sdp = ConstSharedData();

  if (!sdp || !(0 <= linenum_ && linenum_ < sdp->_numnextlines))
  {
    if (done_)
      *done_ = false;
    return *this;
  }

  if (done_)
    *done_ = true;
  return GotoWindowLine(linenum_);
}

/****************************************************************************/
WindowLine& WindowLine::GotoPrevWindowLine(bool* done_)
{
  int linenum_ = _WindowLineNum-1;
  WindowLine::SharedChainData* sdp = SharedData();

  if (!sdp || !(0 <= linenum_ && linenum_ < sdp->_numnextlines))
  {
    if (done_)
      *done_ = false;
    return *this;
  }

  if (done_)
    *done_ = true;
  return GotoWindowLine(linenum_);
}

/****************************************************************************/
const WindowLine& WindowLine::GotoPrevWindowLine(bool* done_) const
{
  int linenum_ = _WindowLineNum-1;
  const WindowLine::SharedChainData* sdp = ConstSharedData();

  if (!sdp || !(0 <= linenum_ && linenum_ < sdp->_numnextlines))
  {
    if (done_)
      *done_ = false;
    return *this;
  }

  if (done_)
    *done_ = true;
  return GotoWindowLine(linenum_);
}

/****************************************************************************/
bool WindowLine::SetXYpos(int x, int y)
{
  // this current window line box
  WindowLineBuffer* Tptr_ = NULL;

  if (_pbox && _pbox[0] &&
      !_pbox[0]->IsDummyLine() && _pbox[0]->IsWindowLineBuffer())
  {
    Tptr_ = ((WindowLineBuffer*)_pbox[0]);
    Tptr_->SetXYpos(x, y);

    if (_nextline)
      _nextline->SetXYpos(x, y+1);
    
    return true;
  }

  return false;
}

/****************************************************************************/
bool WindowLine::SetXpos(int x)
{
  // this current window line box
  WindowLineBuffer* Tptr_ = NULL;

  if (_pbox && _pbox[0] &&
      !_pbox[0]->IsDummyLine() && _pbox[0]->IsWindowLineBuffer())
  {
    Tptr_ = ((WindowLineBuffer*)_pbox[0]);
    Tptr_->SetXpos(x);

    if (_nextline)
      _nextline->SetXpos(x);
    
    return true;
  }

  return false;
}

/****************************************************************************/
bool WindowLine::SetYpos(int y)
{
  // this current window line box
  WindowLineBuffer* Tptr_ = NULL;

  if (_pbox && _pbox[0] &&
      !_pbox[0]->IsDummyLine() && _pbox[0]->IsWindowLineBuffer())
  {
    Tptr_ = ((WindowLineBuffer*)_pbox[0]);
    Tptr_->SetYpos(y);

    if (_nextline)
      _nextline->SetYpos(y+1);
    
    return true;
  }

  return false;
}

/****************************************************************************/
int WindowLine::GetXpos() const
{
  // this current window line box
  WindowLineBuffer* Tptr_ = NULL;

  if (_pbox && _pbox[0] &&
      !_pbox[0]->IsDummyLine() && _pbox[0]->IsWindowLineBuffer())
  {
    Tptr_ = ((WindowLineBuffer*)_pbox[0]);
    return Tptr_->GetXpos();
  }

  return 0;
}

/****************************************************************************/
int WindowLine::GetYpos() const
{
  // this current window line box
  WindowLineBuffer* Tptr_ = NULL;

  if (_pbox && _pbox[0] &&
      !_pbox[0]->IsDummyLine() && _pbox[0]->IsWindowLineBuffer())
  {
    Tptr_ = ((WindowLineBuffer*)_pbox[0]);
    return Tptr_->GetYpos();
  }

  return 0;
}

/****************************************************************************/
/****************************************************************************/
WindowLineRef::WindowLineRef(WindowLine* Boss_, int Index_):
_BossPtr(Boss_),
_Index(Index_),
_Cleared(false),
_InProcess(false)
{}

/****************************************************************************/
void WindowLineRef::ResetLineRef(WindowLine* Boss_, int Index_)
{
  _BossPtr = Boss_;
  _Index = Index_;
  _Cleared = false;
  _InProcess = false;
}

/****************************************************************************/
WindowLineRef& WindowLineRef::operator = (WindowLineBuffer* Pbox_)
{
  if (_Index >= 0)
  {
    _Cleared = false;
    _InProcess = true;
    _BossPtr->Control(_Index, Pbox_);

    _InProcess = false;
    _Cleared = true;
  }
  
  return *this;
}

/****************************************************************************/
WindowLineRef::operator WindowLineBuffer* ()
{
  WindowLineBuffer* ret = NULL;

  if (_Index >= 0)
  {
    _Cleared = false;
    _InProcess = true;
    ret = WindowLineRef::CastToWindowLineBuffer(_BossPtr->Control(_Index));

    _InProcess = false;
    _Cleared = true;
  }

  ASSERT(_Index >= 0, USER_Message_t::ERRMSG_NULLPTR);
  return ret;
}

/****************************************************************************/
WindowLineBuffer* WindowLineRef::operator -> ()
{
  WindowLineBuffer* ret = NULL;

  if (_Index >= 0)
  {
    _Cleared = false;
    _InProcess = true;
    ret = WindowLineRef::CastToWindowLineBuffer(_BossPtr->Control(_Index));

    _InProcess = false;
    _Cleared = true;
  }

  ASSERT(ret != NULL, USER_Message_t::ERRMSG_NULLPTR);
  return ret;
}

/****************************************************************************/
/****************************************************************************/
#if WINLINE_DEBUG
int main()
{
  CLRSCR();

  int index_ = 0;
  char* retstr_ = NULL;
  WindowLineBuffer* pb;
  WindowLineBuffer* npb;

  #if WINLINE_DEBUG1
    #if WINLINE_DEBUGEXL
      WindowLine pw(4, 10, 0, WinCoordsData::Make(2, 2, 75, 6));
    #else
      WindowLine pw(4, 5, 0, WinCoordsData::Make(2, 2, 75, 6));
    #endif

    #if WINLINE_DEBUG1A
      pw.SetWinLineProperties(true);
      pw.SetWinBlockProperties(false, true);
    #elif WINLINE_DEBUG1B
      pw.SetWinLineProperties(true);
      pw.SetWinBlockProperties(true, true);
      pw.AddNewWindowLines(4);
    #elif WINLINE_DEBUG1C
      pw.SetWinLineProperties(true);
      pw.SetWinBlockProperties(true, true, true);
    #else
      pw.SetWinLineProperties(false);
      pw.SetWinBlockProperties(false, true);
    #endif
    
    int index;
    pw.UseColor();
    pw.ShowAll();
    pw.Prompt();
    pw.UseColor(FALSE);
    // pw.Hide();

    GOTOXY(1, 12);
    for (index = 0; index < pw.MaxControl(); index++)
    {
      pb = pw[index];
      cout <<pb->StrRetrieve() <<endl;
      GETCH();
    }

    retstr_ = pw.RetrieveInOrder();
    GOTOXY(1, 15);
    
    if (retstr_)
    {
      cout <<endl <<"Line #1 In Order: " <<retstr_ <<endl;
      delete retstr_;
    }

    #if WINLINE_DEBUG3
      WindowLine* ptr = &pw;
      
      for (index = 2; ptr->NextLinePtr(); index++)
      {
        ptr = ptr->NextLinePtr();
        retstr_ = ptr->RetrieveInOrder();
        GOTOXY(1, 14+index);

        if (retstr_)
        {
          cout <<endl <<"Line #" <<index <<" In Order: " <<retstr_ <<endl;
          delete retstr_;
        }
      }
    #endif
  #endif

  #if WINLINE_DEBUG2
    #if WINLINE_DEBUGEXL
      WindowLine pw(0, 10, 40, WinCoordsData::Make(2, 2, 75, 6));
    #else
      WindowLine pw(0, 5, 20, WinCoordsData::Make(2, 2, 75, 6));
    #endif

    pb = pw[0];
    pb->SetTrimWs(false);

    int index;
    pw.UseColor();
    pw.Show();
    pw.Prompt();
    pw.UseColor(FALSE);
    pw.Hide();

    GOTOXY(1, 5);
    for (index = 0; index < pw.MaxControl(); index++)
    {
      pb = pw[index];
      cout <<pb->StrRetrieve() <<endl;
      GETCH();
    }

    GETCH();
  #endif
}

#endif
/****************************************************************************/
#endif
